////////////////////////////////////////////////////////////////////////////////////////////////////
// cmodel.cpp (client and modeler)
// This class will implement the model object 
// 
// by Eric J Krejci
// Copyright (C) 1999 Eric J Krejci 
// Revised : 31 Jan 2004 
//  
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
// 
// Contact:
//              email   worldii@viagloria.com
//              web             www.viagloria.com
////////////////////////////////////////////////////////////////////////////////////////////////////
#include <stdlib.h>
#include <stdio.h> 
#include <string.h>

#include "cworld.hpp"
#include "cstring.hpp"

////////////////////////////////////////////////////////////////////////////////////////////////////
// Defines
////////////////////////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////////////////////////
// Externals
////////////////////////////////////////////////////////////////////////////////////////////////////

extern char logfilename[];

////////////////////////////////////////////////////////////////////////////////////////////////////
// CModel Constructor
////////////////////////////////////////////////////////////////////////////////////////////////////
CModel::CModel()
{

	// This is for debugging and for inititalization 
	xpos	= 0.0;
	ypos	= 0.0;
	zpos	= 0.0;
	roll	= 0.0;
	pitch	= 0.0;
	yaw	= 0.0;
//	sublist	= 0;
	partlist= 0;
} ///////////////////////////////////////////////////////////////////////////////////end Constructor

////////////////////////////////////////////////////////////////////////////////////////////////////
// CModel Destructor
////////////////////////////////////////////////////////////////////////////////////////////////////
CModel::~CModel()
{
     // BUILD DESTRUCTOR !!!!!!!!
	// 977K memory leak !!!!!!!!
} ////////////////////////////////////////////////////////////////////////////////////end Destructor

////////////////////////////////////////////////////////////////////////////////////////////////////
// CModel Destructor
////////////////////////////////////////////////////////////////////////////////////////////////////
BOOL CModel::Init()
{
	return TRUE;
} //////////////////////////////////////////////////////////////////////////////////////////end Init

///////////////////////////////////////////////////////////////////////////////////////////////////
// CModel Destructor
///////////////////////////////////////////////////////////////////////////////////////////////////
//BOOL CModel::Init(BOOL (*pfunc)(char *,HWND tv,int n,LPARAM p),HWND tv)
//{
//
//	// set the function pointer
//	treeadd=pfunc;
//	hwndtv=tv;
//	return TRUE;
//} /////////////////////////////////////////////////////////////////////////////////////////end Init

////////////////////////////////////////////////////////////////////////////////////////////////////
// CModel SetFileName
////////////////////////////////////////////////////////////////////////////////////////////////////
void CModel::SetFileName(const char  *f)
{
	myfilename.SetString(f);
} ///////////////////////////////////////////////////////////////////////////////////end SetFileName
	
////////////////////////////////////////////////////////////////////////////////////////////////////
// CModel InsertPart
////////////////////////////////////////////////////////////////////////////////////////////////////
BOOL CModel::InsertPart(CPart *p)
{

	// This should look A LOT like InsertRegion 
        // use existing allocated object and insert it.
        p->nextpart=partlist;
        partlist=p;
        return TRUE;


} ///////////////////////////////////////////////////////////////////////////////////end SetFileName

///////////////////////////////////////////////////////////////////////////////////////////////////
// CModel Load
///////////////////////////////////////////////////////////////////////////////////////////////////
BOOL CModel::Load(char *filename, float x, float y, float z, float ro, float pi, float ya,int level)
{
FILE		*f;
float		subxpos,subypos,subzpos;
float		subroll,subpitch,subyaw;
int		i,subcount,partcount;
char		token[80],modelname[80];	// this is where tokens are returned to
CModel		*submodel;
CPart		*part;


	xpos=x; ypos=y;zpos=z;
	roll=ro; pitch=pi; yaw=ya;

	// is it there
	if (!(f=fopen(filename,"r"))) return (FALSE);

	// get submod count
	myfilename.SetString(filename);

	fscanf(f,"%s %d", token,&subcount);
//	if (treeadd) (*treeadd)(filename,hwndtv,level,(LPARAM)this);

	// get submod info so it can load
	for (i=0;i<subcount;i++)
	{
		fscanf(f,"%s %s %f %f %f %f %f %f",token,modelname,
						&subxpos,&subypos,&subzpos,
						&subroll,&subpitch,&subyaw);

		submodel=new CModel;
		if (!submodel) return FALSE;
		else
		{
		CInode *iptr;

//			submodel->Init(treeadd,hwndtv);

			// load submodel
			if (!(submodel->Load(modelname,subxpos,subypos,subzpos,subroll,subpitch,subyaw,level+1))) 
				return FALSE;

			// insert submodel
			iptr = new CInode;
			iptr->model=submodel;
			iptr->nextnode=sublist;
			sublist=iptr;
			
		}
	}

	// get part count
	fscanf(f,"%s %d", token,&partcount);
	for (i=0;i<partcount;i++)
	{
		part=new CPart;
//		if (!(part->Load(f,treeadd,modelname,hwndtv,level+1))) return FALSE;
		if (!(part->Load(f,modelname,level+1))) return FALSE;
		
		// insert part at head of list
		part->nextpart=partlist;	
		partlist=part;
	}
	
	// compute matricies
	D3DXMatrixTranslation(&matworld, xpos, ypos, zpos);
	
	fclose(f);	
	return TRUE;
} /////////////////////////////////////////////////////////////////////////////////////////end Load

///////////////////////////////////////////////////////////////////////////////////////////////////
// Render
///////////////////////////////////////////////////////////////////////////////////////////////////
HRESULT CModel::Render(LPDIRECT3DDEVICE8 pd3d,	D3DXMATRIX *pmatview, 
						D3DXMATRIX *pmatproj, 
						D3DXMATRIX *pmatworld)
{
CInode		*iptr;
CPart		*partptr;
D3DXMATRIX	mattemp,submatworld;
D3DXMATRIX	matrot;   

	// Accumulate translations from higher up models
	D3DXMatrixTranslation(&mattemp, xpos, ypos, zpos);
	D3DXMatrixMultiply(&submatworld, &mattemp, pmatworld);
	D3DXMatrixIdentity(&matrot);

        D3DXMatrixRotationX(&mattemp, pitch);         // Pitch
        D3DXMatrixMultiply(&matrot, &matrot, &mattemp);
        D3DXMatrixRotationY(&mattemp, yaw);           // Yaw
        D3DXMatrixMultiply(&matrot, &matrot, &mattemp);
        D3DXMatrixRotationZ(&mattemp, roll);          // Roll
        D3DXMatrixMultiply(&matrot, &matrot, &mattemp);
 
        // Apply the rotation matrices to complete the world matrix.
        D3DXMatrixMultiply(&submatworld, &matrot, &submatworld);
    
	// Render each of _this_ models submodels
	iptr=sublist;
	while (iptr)
	{
		if (iptr->model)
		{
			if (FAILED((iptr->model)->Render(pd3d,pmatview,pmatproj,&submatworld))) return E_FAIL;
		}
		iptr=iptr->nextnode;
	}

	// Render _this_ models own parts and lists
	partptr=partlist;
	while (partptr)
	{
		if (FAILED(partptr->Render(pd3d,pmatview,pmatproj,&submatworld))) return E_FAIL;
		partptr=partptr->nextpart;
	}

	return S_OK;
} ///////////////////////////////////////////////////////////////////////////////////////end Render


///////////////////////////////////////////////////////////////////////////////////////////////////
// CModel InitDeviceObjects
///////////////////////////////////////////////////////////////////////////////////////////////////
HRESULT CModel::InitDeviceObjects(LPDIRECT3DDEVICE8 pd3d)
{
CInode	*iptr;
CPart	*partptr;

	// Init each of _this_ models submodels
	iptr=sublist;
	while (iptr)
	{
		if (iptr->model)
		{
			if (FAILED((iptr->model)->InitDeviceObjects(pd3d))) return E_FAIL;
		}
		iptr=iptr->nextnode;
	}

	// Init _this_ models own parts and lists
	partptr=partlist;
	while (partptr)
	{
		if (FAILED(partptr->InitDeviceObjects(pd3d))) return E_FAIL;
		partptr=partptr->nextpart;
	}

	return S_OK;
} ////////////////////////////////////////////////////////////////////////////end InitDeviceObjects

///////////////////////////////////////////////////////////////////////////////////////////////////
// CModel InvalidateDeviceObjects
///////////////////////////////////////////////////////////////////////////////////////////////////
HRESULT CModel::InvalidateDeviceObjects()
{  
CInode	*iptr;
CPart	*partptr;

	// Invalidate each of _this_ models submodels
	iptr=sublist;
	while (iptr)
	{
		if (iptr->model)
		{
			if (FAILED((iptr->model)->InvalidateDeviceObjects())) return E_FAIL;
		}
		iptr=iptr->nextnode;
	}

	// Invalidate _this_ models own parts and lists
	partptr=partlist;
	while (partptr)
	{
		if (FAILED(partptr->InvalidateDeviceObjects())) return E_FAIL;
		partptr=partptr->nextpart;
	}

	return S_OK;

} //////////////////////////////////////////////////////////////////////end InvalidateDeviceObjects

///////////////////////////////////////////////////////////////////////////////////////////////////
// CModel RestoreDeviceObjects
///////////////////////////////////////////////////////////////////////////////////////////////////
HRESULT CModel::RestoreDeviceObjects(LPDIRECT3DDEVICE8 pd3d)
{

CInode	*iptr;
CPart	*partptr;

	// Init each of _this_ models submodels
	iptr=sublist;
	while (iptr)
	{
		if (iptr->model)
		{
			if (FAILED((iptr->model)->RestoreDeviceObjects(pd3d))) return E_FAIL;
		}
		iptr=iptr->nextnode;
	}

	// Init _this_ models own parts and lists
	partptr=partlist;
	while (partptr)
	{
		if (FAILED(partptr->RestoreDeviceObjects(pd3d))) return E_FAIL;
		partptr=partptr->nextpart;
	}

	return S_OK;

} /////////////////////////////////////////////////////////////////////////end RestoreDeviceObjects

///////////////////////////////////////////////////////////////////////////////////////////////////
// CModel Destroy
///////////////////////////////////////////////////////////////////////////////////////////////////
HRESULT CModel::Destroy()
{
CInode	*iptr;
CPart	*partptr;

	// Destroy each of _this_ models submodels
	iptr=sublist;
	while (iptr)
	{
		if (iptr->model)
		{
			if (FAILED((iptr->model)->Destroy())) return E_FAIL;
		}
		iptr=iptr->nextnode;
	}

	// Destroy _this_ models own parts and lists
	partptr=partlist;
	while (partptr)
	{
		if (FAILED(partptr->Destroy())) return E_FAIL;
		partptr=partptr->nextpart;
	}
	return S_OK;
} //////////////////////////////////////////////////////////////////////////////////////end Destroy

///////////////////////////////////////////////////////////////////////////////////////////////////
// CModel Save
///////////////////////////////////////////////////////////////////////////////////////////////////
BOOL CModel::Save()
{
FILE		*f;
int		count;//,partcount;
CInode		*p;
CPart		*part;


	// is it there
	if (!(f=fopen(myfilename.GetString(),"w"))) return (FALSE);

	// count submodels
	count=0;
	p=sublist;
	while (p)
	{
		if (p->model) count++;
		p=p->nextnode;
	}

	fprintf(f,"%s %d\n", "[subcount]",count);

	// save submodels
	p=sublist;
	while (p)
	{
		// Now save each model name and offset and such
		fprintf(f,"%s %s %.2f %.2f %.2f %.2f %.2f %.2f\n",
			"[mod]",p->model->myfilename.GetString(),
			p->model->xpos,p->model->ypos,p->model->zpos,
			p->model->roll,p->model->pitch,p->model->yaw);
		
		if (!((p->model)->Save())) 
		{
			return FALSE;
		}
		p=p->nextnode;
	}

	// count parts
	count=0;
	part=partlist;
	while (part)
	{
		if (part) count++;
		part=part->nextpart;
	}

	fprintf(f,"%s %d\n", "[partcount]",count);

	// save parts
	part=partlist;
	while (part)
	{
		// Now save each model name and offset and such
		if (!(part->Save(f))) 
		{
			return FALSE;
		}
		part=part->nextpart;
	}

	fclose(f);	
	return TRUE;
} /////////////////////////////////////////////////////////////////////////////////////////end Save



