////////////////////////////////////////////////////////////////////////////////////////////////////
// font.cpp -- This is it.
// This is the font module for the first real attempt to put together my 3d world game.
//
// World III
//
// by Eric J Krejci
// Copyright (C) 1999 Eric J Krejci
// Revised : 29 Jan 2004
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////
//  This draws from example code by Jeff Molofee '99
//  (ported to Linux/SDL by Ti Leggett '01)
//
//  Jeff Molofee: Yes i did find it useful :) Thanks !
//
//  Visit Jeff at http://nehe.gamedev.net/
//
//  or for port-specific comments, questions, bugreports etc.
//  email to leggett@eecs.tulane.edu
////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////
// Includes
////////////////////////////////////////////////////////////////////////////////////////////////////
#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <math.h>
#include <string.h>
#include <GL/gl.h>
#include <GL/glu.h>
#include <GL/glx.h>

#include "SDL.h"
#include "client.h"
#include "cstring.hpp"
#include "wiii_types.h"

GLuint  base; /* Base Display List For The Font Set */
////////////////////////////////////////////////////////////////////////////////////////////////////
// Externals
////////////////////////////////////////////////////////////////////////////////////////////////////
extern void	Quit(int);
extern GLfloat	xpos, zpos, yrot;
extern int	width, height;
extern GLfloat	ratio;
extern CString	commandstring;

////////////////////////////////////////////////////////////////////////////////////////////////////
// KillFont
// function to recover memory form our list of characters
////////////////////////////////////////////////////////////////////////////////////////////////////
GLvoid KillFont( GLvoid )
{
	glDeleteLists( base, 96 );
//	return;
	} //////////////////////////////////////////////////////////////////////////////end KillFont

////////////////////////////////////////////////////////////////////////////////////////////////////
// BuildFont to build our font list 
// function to build our font list 
////////////////////////////////////////////////////////////////////////////////////////////////////
GLvoid BuildFont(GLvoid)
{
Display *dpy;		// Our current X display 
XFontStruct *fontInfo;	// Our font info 

	// Sotrage for 96 characters 
	base = glGenLists( 96 );

	// Get our current display long enough to get the fonts */
	dpy = XOpenDisplay( NULL );

	// Get the font information 
	fontInfo = XLoadQueryFont( dpy, "-adobe-helvetica-medium-r-normal--18-*-*-*-p-*-iso8859-1" );

	// If the above font didn't exist try one that should 
	if ( fontInfo == NULL )
	{
		fontInfo = XLoadQueryFont( dpy, "fixed" );
		// If that font doesn't exist, something is wrong 
		if ( fontInfo == NULL )
		{
			fprintf( stderr, "no X font available?\n" );
			Quit( 1 );
			}
		}

	// generate the list 
	glXUseXFont( fontInfo->fid, 32, 96, base );

	// Recover some memory 
	XFreeFont( dpy, fontInfo );

	// close the display now that we're done with it 
	XCloseDisplay( dpy );

	} /////////////////////////////////////////////////////////////////////////////end BuildFont

////////////////////////////////////////////////////////////////////////////////////////////////////
// GlPrint
// Print our GL text to the screen 
////////////////////////////////////////////////////////////////////////////////////////////////////
GLvoid GlPrint( const char *fmt, ... )
{
char text[256]; // Holds our string 
va_list ap;     // Pointer to our list of elements 

	// If there's no text, do nothing 
	if ( fmt == NULL ) return;

	// Parses The String For Variables 
	va_start( ap, fmt );
	// Converts Symbols To Actual Numbers 
	vsprintf( text, fmt, ap );
	va_end( ap );

	// Pushes the Display List Bits 
	glPushAttrib( GL_LIST_BIT );

	// Sets base character to 32 
	glListBase( base - 32 );

	// Draws the text 
	glCallLists( strlen( text ), GL_UNSIGNED_BYTE, text );

	// Pops the Display List Bits 
	glPopAttrib( );
	} ///////////////////////////////////////////////////////////////////////////////end GlPrint

////////////////////////////////////////////////////////////////////////////////////////////////////
// Here goes our drawing code 
//
////////////////////////////////////////////////////////////////////////////////////////////////////
BOOL DrawText( GLvoid )
{

	// Preserve matrix
	glPushMatrix();

	// setup for 2D rendering for the text
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluOrtho2D(0.0, (GLdouble)width, 0.0, (GLdouble)height);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
        glShadeModel( GL_FLAT );
	
	// Set the colors
	glColor3f( 1.0f, 1.0f, 1.0f);

	// Position The Text On The Screen 
	glRasterPos2i(100 ,100);

	// Print text to the screen 
	GlPrint( "Present Position Xpos=%7.2f Zpos=%7.2f Yrot=%7.2f COMMANDSTRING='%s'", xpos,zpos,yrot,commandstring.GetString() );

	// Restore saved matrix  
	glPopMatrix();

        // reset for 3D rendering 
        glMatrixMode( GL_PROJECTION );
        glLoadIdentity( );
        gluPerspective( 45.0f, ratio, 0.1f, 100.0f );
        glMatrixMode( GL_MODELVIEW );
        glShadeModel( GL_SMOOTH );
        glLoadIdentity( );

	return( TRUE );
	} //////////////////////////////////////////////////////////////////////////////end DrawText

