////////////////////////////////////////////////////////////////////////////////////////////////////
// charclient.cpp -- This is it.
// This is the main module for the client to the worldiii game.
//  
// World III
//
// by Eric J Krejci
// Copyright (C) 1999 Eric J Krejci
// Revised : 05 Dec 2005
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////
// Includes
////////////////////////////////////////////////////////////////////////////////////////////////////
#include <unistd.h>
#include <pthread.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/timeb.h>
#ifdef SOLARIS
#include <strings.h>
#endif

#include "rudp.h"
#include "cusock.hpp"
#include "wiii_util.h"

////////////////////////////////////////////////////////////////////////////////////////////////////
// charclient.cpp
// this test program is to test the server in charmode
////////////////////////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////////////////////////
// Globals
////////////////////////////////////////////////////////////////////////////////////////////////////
FILE			*logfile;
//char			logfilename[50]="logs/charclient.log";
int			loglevel;
long			logcount;
pthread_mutex_t		logmutex;
pthread_mutex_t		screenmutex;
pthread_mutex_t		printmutex;
bool			consoleconnected;       // tells when we are connected to console.
//CUSock			consock;                // This is the console socket
pthread_attr_t		attr;                   // thread attr structure for setting attributes.
bool			donelistening;
bool			connected;
struct timeb		starttime;
//pthread_mutexattr_t	m_attr;
extern	void		ThreadLog(char *s);
char			serverhostname[80]="worldiii";


////////////////////////////////////////////////////////////////////////////////////////////////////
// LockPrint
////////////////////////////////////////////////////////////////////////////////////////////////////
void LockPrint(char *s)
{
	pthread_mutex_lock(&printmutex);
	printf(s);
	pthread_mutex_unlock(&printmutex);

	} /////////////////////////////////////////////////////////////////////////////end LockPrint

////////////////////////////////////////////////////////////////////////////////////////////////////
// PortMon
////////////////////////////////////////////////////////////////////////////////////////////////////
void *PortMon(void *ptr)
{
int             count;
char            *mesg;                  // the protocol will give me a buffer with a packet in it
char 		buf[80];
CUSock		*s;

	// unpack the integer parameter that points to our structure in serverlist
	LockPrint("Portmon: Entering Portmon\n");
	s=(CUSock*)ptr;
	while (1)
	{
		count=s->Recv(&mesg, 0);

		// Got something, print it
		if (count>0)
		{
			snprintf(buf, 80, "%s\n",mesg);
			LockPrint(buf);
			delete [] mesg;
			}
		} // end while loop

	s->Close();

	LockPrint("EXITING PORTMON THREAD===>>>>>\n");
	return (0);
	} ///////////////////////////////////////////////////////////////////////////////end PortMon

////////////////////////////////////////////////////////////////////////////////////////////////////
// dg_cli
// session is unused in client, as it is stored in the protocol
////////////////////////////////////////////////////////////////////////////////////////////////////
void dg_cli(FILE *fp, CUSock *s, sockaddr_in *pservaddr, socklen_t servlen,int delay)
{
int 			i;
char			sendline[MAXLINE];
socklen_t		len;
struct sockaddr_in 	*preply_addr;

	ThreadLog("dg_cli: Entered.");
	preply_addr=(struct sockaddr_in *)malloc(servlen);
	while (connected)
	{
		//LockPrint("Waiting for input (Before fgets)\n");
		fgets(sendline, MAXLINE, fp);
		if (!connected)  LockPrint("Lost Connection (After fgets)\n"); 
	
		// chomp the newline :)
		for (i=MAXLINE-1;i>=0;i--) if (sendline[i]=='\n') sendline[i]=' ';

		// send the typed int line to the server
		if (strncmp(sendline,"close||",8)==0)
		{
			s->Close();
			connected=false;
			}
		else
		{
			s->Send(sendline, strlen(sendline), 0);
			len=servlen;
			}
		}
	LockPrint("Lost connection (while).\n");
	} ////////////////////////////////////////////////////////////////////////////////end dg_cli

////////////////////////////////////////////////////////////////////////////////////////////////////
// ParseCommandLine
////////////////////////////////////////////////////////////////////////////////////////////////////
void ParseCommandLine(int argc, char** argv)
{
int     i;

        // set defaults for parms
        strncpy(serverhostname,"worldiii",80);
        loglevel=0;

        //getting parms
        for (i=0;i<argc;i++)
        {
                // -h which host to manage
                if ((strncmp(argv[i],"-h",2))==0)
                {
                        if (argc>=i+2)
                        {
                                printf("Found -h = '%s'\n",argv[i+1]);
                                sleep(1);
                                strncpy(serverhostname,argv[i+1],80);
                                }
                        i++;
                        }

                // --loglevel how much logging to do
                if ((strncmp(argv[i],"--loglevel",10))==0)
                {
                        printf("DEBUGMODE ON\n");
                        loglevel=atoi(argv[i+1]);
                        i++;
                        }
                } //end for loop

        } //////////////////////////////////////////////////////////////////////end ParseCommandLine

////////////////////////////////////////////////////////////////////////////////////////////////////
// Init
////////////////////////////////////////////////////////////////////////////////////////////////////
void Init(int argc, char** argv)
{
	ftime(&starttime);
	ParseCommandLine(argc,argv);
	
	logcount=0;
//	logfile=fopen(logfilename,"w");
//	fclose(logfile);

	ThreadLog("Init: Entered.");

        if (loglevel>0)
        {
        int i;

                for (i=0;i<argc;i++) printf("Argument %d, s='%s' \n",i,argv[i]);
                }

	// init the threading model
//	pthread_mutexattr_init(&m_attr);
//#ifdef LINUX
//	pthread_mutexattr_settype(&m_attr,PTHREAD_MUTEX_ERRORCHECK_NP);
//#endif

	pthread_attr_init(&attr);    // initialize attr with default attributes
	pthread_attr_setscope(&attr, PTHREAD_SCOPE_SYSTEM);  // system-wide contention
//	pthread_mutex_init(&screenmutex, &m_attr);
//	pthread_mutex_init(&printmutex, &m_attr);
//	pthread_mutex_init(&logmutex, &m_attr);
	InitMutex(&screenmutex);
	InitMutex(&printmutex);
	InitMutex(&logmutex);
	consoleconnected=false;
	donelistening=false;
	ThreadLog("Init: Exited.");
	} //////////////////////////////////////////////////////////////////////////////////end Init

////////////////////////////////////////////////////////////////////////////////////////////////////
// ConnectFunc
// These functions are not required
// They are provided for notification purposes
// They are called by a different thread
////////////////////////////////////////////////////////////////////////////////////////////////////
int ConnectFunc(connecttype *cparm)
{
	// OK OK, we should have a mutex here,
	// but this is test code, and i am being lazy 
	connected=true;
	printf("Received connect signal.\n");
	return (0);
	} ///////////////////////////////////////////////////////////////////////////end ConnectFunc

////////////////////////////////////////////////////////////////////////////////////////////////////
// DisconnectFunc
// These functions are not required
// They are provided for notification purposes
// They are called by a different thread
////////////////////////////////////////////////////////////////////////////////////////////////////
int DisconnectFunc(connecttype *cparm)
{
	// OK OK, we should have a mutex here,
	// but this is test code, and i am being lazy 
	connected=false;
	printf("Received disconnect signal.\n");
	return (0);
	} ////////////////////////////////////////////////////////////////////////end DisconnectFunc

// main
////////////////////////////////////////////////////////////////////////////////////////////////////
int main (int argc , char *argv[])
{
int delay;
CUSock sock;
struct sockaddr_in servaddr;
pthread_t	tid;
hostent         *hent;
char            buf[100];
char            dst[50];

	Init(argc, argv);
	ThreadLog("Main: Entered.");

	// setup connection dest and family type
	bzero(&servaddr, sizeof(servaddr));
	servaddr.sin_family = AF_INET;

	// NOTE that IP and PORT are in NET form
	servaddr.sin_port = htons(BASEWORLDPORT);

        hent=gethostbyname(serverhostname);
        if (hent==NULL)
        {
                ThreadLog("Connect: Gethostbyname failed.");
                connected=false;
                printf("Connect: Gethostname failed. \n");
                return(1);
                }
        else
        {
                servaddr.sin_addr=*((struct in_addr*)hent->h_addr);
                if (loglevel>=2)
                {
                        snprintf(buf,100,"Connecting to IP = '%s'\n",
                                inet_ntop(AF_INET,(void*)&(servaddr.sin_addr),dst,50)
                                );
                        printf(buf);
                        }
                }

	ThreadLog("Main: Before SetLogFunc call.");
	sock.SetLogFunc(ThreadLog);
	ThreadLog("Main: Before sock.Init call.");
//	sock.Init(&m_attr);
	sock.Init();
	// the client isnt going to wander off and do other things 
	// while waiting for connect 
	sock.SetDisConnFunc(DisconnectFunc);
	sock.Create(&servaddr,sizeof(servaddr));
	if (!sock.Connect()) 
	{
		ThreadLog("Main: Failed to connect.");
		exit (1);
		}	
	else connected=true;
	
	ThreadLog("Main: Before dg_cli call.");
	// create the listener thread 
	pthread_create(&tid, &attr, PortMon, (void*)(&sock));

	// goto the sending thread
	dg_cli(stdin, &sock,  &servaddr, sizeof(servaddr),delay);
	exit (0);

} //////////////////////////////////////////////////////////////////////////////////////////end main



