////////////////////////////////////////////////////////////////////////////////////////////////////
// client.cpp -- This is it.
// This is the main module for the first real attempt to put together my 3d world game.
// This is the main module for the client.
//
// World III
//
// by Eric J Krejci
// Copyright (C) 1999-2005 Eric J Krejci
// Revised : 31 Jan 2006
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation version 2.
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
// Contact:
//              email           worldii@viagloria.com
//              web             www.viagloria.com
////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////
//  This draws from example code by Jeff Molofee '99 
//  (ported to Linux/SDL by Ti Leggett '01)
// 
//  Jeff Molofee: Yes i did find it useful :) Thanks !
// 
//  Visit Jeff at http://nehe.gamedev.net/
//  
//  or for port-specific comments, questions, bugreports etc. 
//  email to leggett@eecs.tulane.edu
////////////////////////////////////////////////////////////////////////////////////////////////////

 
////////////////////////////////////////////////////////////////////////////////////////////////////
// Include
////////////////////////////////////////////////////////////////////////////////////////////////////
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <GL/gl.h>
#include <GL/glu.h>
#include <pthread.h>
#include <sys/timeb.h>
#include <unistd.h>



#include "SDL.h"
#include "client.h"
#include "cfont.hpp"
#include "ctextwin.hpp"
#include "cworld.hpp"
#include "cusock.hpp"
#include "wiii_util.h"

////////////////////////////////////////////////////////////////////////////////////////////////////
// Define
////////////////////////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////////////////////////
// Globals
////////////////////////////////////////////////////////////////////////////////////////////////////
// This is our SDL surface 
SDL_Surface *surface;

GLfloat		angle;

//int width, height;
GLfloat ratio;
GLfloat yrot;				// Camera rotation variable 
GLfloat xpos, ypos, zpos;			// Camera pos variable 

GLfloat walkbias, walkbiasangle;	// Head-bobbing variables 
GLfloat lookupdown;
GLint T0     = 0;
GLint Frames = 0;
char  framestring[100];

// Light Values 
GLfloat LightAmbient[]  = { 0.5f, 0.5f, 0.5f, 0.0f };
GLfloat LightDiffuse[]  = { 1.0f, 1.0f, 1.0f, 0.0f };
GLfloat LightPosition[] = { 0.0f, 0.0f, 2.0f, 0.0f };

// constant used for converting to radians 
const float piover180 = 0.0174532925f;

//GLuint 			filter;				// Which Filter To Use 
GLuint 			texture[3];			// Storage for 3 textures 

//CFont			font;
CUSock			*sock;
//CTextWin		textwindow;
CWorld			*world;
int 			TurnRight;			
int 			TurnLeft;		
int 			Forward;
int 			Backward;		
int 			PgUp;
int 			PgDn;		
int 			Home;		
FILE			*logfile;
//char			logfilename[50]="logs/client.log";
char			winsize[50];
long			logcount;

pthread_t		portmon_tid;
pthread_mutex_t		logmutex;
pthread_mutex_t		screenmutex;
pthread_attr_t		attr;                   // thread attr structure for setting attributes.
//pthread_mutexattr_t     m_attr;
struct timeb		starttime;
bool			connected;
int			loglevel;

CString			*playerid;
CString			*characterid;

////////////////////////////////////////////////////////////////////////////////////////////////////
// Externals
////////////////////////////////////////////////////////////////////////////////////////////////////
extern GLvoid 	BuildFont(GLvoid);
extern int 	DrawText(GLvoid);
extern GLvoid 	KillFont(GLvoid);
extern void	HandleKey(SDL_Event *event, CWorld *w);
extern bool	commandstarted;         // ie did we see a ']' at the first char position ?
extern bool	firstchar;         // are we getting stuff after the first char position ?

////////////////////////////////////////////////////////////////////////////////////////////////////
// Code
////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////
// function to release/destroy our resources and restoring the old desktop 
////////////////////////////////////////////////////////////////////////////////////////////////////
void Quit( int returnCode )
{
//	KillFont();

	// clean up the window 
	SDL_Quit();

	// and exit appropriately 
	exit( returnCode );
	} //////////////////////////////////////////////////////////////////////////////////end Quit

////////////////////////////////////////////////////////////////////////////////////////////////////
// function to reset our viewport after a window resize 
////////////////////////////////////////////////////////////////////////////////////////////////////
int ResizeWindow(CWorld *wo, int w, int h)
{

	// Protect against a divide by zero 
	if ( h == 0 ) h = 1;

	ratio = ( GLfloat )w / ( GLfloat )h;

	// Setup our viewport. 
	glViewport( 0, 0, ( GLint )w, ( GLint )h );

	// change to the projection matrix and set our viewing volume. 
	glMatrixMode( GL_PROJECTION );
	glLoadIdentity( );

	// Set our perspective 
	gluPerspective( 45.0f, ratio, 0.1f, 100.0f );

	// Make sure we're chaning the model view and not the projection 
	glMatrixMode( GL_MODELVIEW );

	// Reset The View 
	glLoadIdentity( );
	
	// tell the CWorld engine what the window parms are
	wo->SetWindow(w,h);

	return( true );
	} //////////////////////////////////////////////////////////////////////////end ResizeWindow

////////////////////////////////////////////////////////////////////////////////////////////////////
// general SDL initialization function 
////////////////////////////////////////////////////////////////////////////////////////////////////
int InitSDL(int videoFlags, int w, int h)
{
const SDL_VideoInfo	*videoInfo;			// this holds some info about our display
char			buf[BUFFSIZE];

	// initialize SDL 
	if ( SDL_Init( SDL_INIT_VIDEO|SDL_INIT_NOPARACHUTE ) < 0 )
	{
		fprintf( stderr, "Video initialization failed: %s\n", SDL_GetError( ) );
		Quit( 1 );
		}

	// Fetch the video info 
	videoInfo = SDL_GetVideoInfo( );

	if ( !videoInfo )
	{
		fprintf( stderr, "Video query failed: %s\n", SDL_GetError( ) );
		Quit( 1 );
		}

	// the flags to pass to SDL_SetVideoMode 
	videoFlags  = SDL_OPENGL;          // Enable OpenGL in SDL 
	videoFlags |= SDL_GL_DOUBLEBUFFER; // Enable double buffering 
	videoFlags |= SDL_HWPALETTE;       // Store the palette in hardware 
	videoFlags |= SDL_RESIZABLE;       // Enable window resizing 

	// This checks to see if surfaces can be stored in memory 
	if ( videoInfo->hw_available ) videoFlags |= SDL_HWSURFACE;
	else videoFlags |= SDL_SWSURFACE;

	// This checks if hardware blits can be done 
	if ( videoInfo->blit_hw ) videoFlags |= SDL_HWACCEL;

	// Sets up OpenGL double buffering 
	SDL_GL_SetAttribute( SDL_GL_DOUBLEBUFFER, 1 );

	// get a SDL surface 
	snprintf(buf,BUFFSIZE,"InitSDL: w = %d h = %d",w,h);
	ThreadLog(buf);
	surface = SDL_SetVideoMode( w, h, SCREEN_BPP, videoFlags );

	// Verify there is a surface 
	if ( !surface )
	{
		fprintf( stderr,  "Video mode set failed: %s\n", SDL_GetError( ) );
		Quit( 1 );
		}
	} ///////////////////////////////////////////////////////////////////////////////end InitSDL

////////////////////////////////////////////////////////////////////////////////////////////////////
// general OpenGL initialization function 
////////////////////////////////////////////////////////////////////////////////////////////////////
int InitGL( GLvoid )
{
CString fontname;

	glEnable( GL_TEXTURE_2D );				// Enable Texture Mapping 
	glShadeModel( GL_SMOOTH );				// Enable smooth shading 
	glClearColor( 0.0f, 0.4f, 0.7f, 0.0f );			// Set the background sky blue 
	glClearDepth( 1.0f );					// Depth buffer setup 
	glEnable( GL_DEPTH_TEST );				// Enables Depth Testing 
	glDepthFunc( GL_LEQUAL );				// The Type Of Depth Test To Do 
	glHint( GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST );	// Really Nice Perspective Calculations 
	glLightfv( GL_LIGHT1, GL_AMBIENT, LightAmbient );	// Setup The Ambient Light 
	glLightfv( GL_LIGHT1, GL_DIFFUSE, LightDiffuse );	// Setup The Diffuse Light 
	glLightfv( GL_LIGHT1, GL_POSITION, LightPosition );	// Position The Light 
	glEnable( GL_LIGHT1 );					// Enable Light One 

	lookupdown    = 0.0f;
	walkbias      = 0.0f;
	walkbiasangle = 0.0f;
    
	glColor4f( 1.0f, 1.0f, 1.0f, 0.0f);
	// Full Brightness, 50% Alpha 

	glBlendFunc( GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA );
	// Blending Function For Translucency Based On Source Alpha Value 

//	// set frontface mode
	glPolygonMode(GL_FRONT,GL_FILL);
	glPolygonMode(GL_BACK,GL_LINE);
	glFrontFace(GL_CCW);
	glCullFace(GL_BACK);
//	glEnable(GL_CULL_FACE);
//	glEnable(GL_BLEND);

	return( true );
	} ////////////////////////////////////////////////////////////////////////////////end InitGL

////////////////////////////////////////////////////////////////////////////////////////////////////
// ParseCommandLine
////////////////////////////////////////////////////////////////////////////////////////////////////
void ParseCommandLine(int argc, char** argv, int *w, int *h)
{
char	buf[BUFFSIZE];
int	i;

        //getting parms
        for (i=0;i<argc;i++)
        {
		if (strncmp(argv[i],"-s",2)==0)
		{
			snprintf(buf,BUFFSIZE,"ParseCommandLine: Found -s = '%s' \n",argv[i]);
			printf(buf);
			ThreadLog(buf);
			if (strncmp(argv[i+1],"1024x768",8)==0) { *w=1024; *h=768; i++; i++; continue; }
			if (strncmp(argv[i+1], "800x600",7)==0) {  *w=800; *h=600; i++; i++; continue; }
			if (strncmp(argv[i+1], "640x480",7)==0) {  *w=640; *h=480; i++; i++; continue; }
			if (strncmp(argv[i+1], "320x240",7)==0) {  *w=320; *h=240; i++; i++; continue; }
			if (strncmp(argv[i+1], "160x120",7)==0) {  *w=160; *h=120; i++; i++; continue; }
			snprintf(buf,BUFFSIZE,"ParseCommandLine: w = %d h = %d",*w,*h);
			ThreadLog(buf);
			sleep(3);
			}

                // --loglevel how much logging to do
                if ((strncmp(argv[i],"--loglevel",10))==0)
                {
                        printf("DEBUGMODE ON\n");
                        if (argc>=i+2) loglevel=atoi(argv[i+1]);

			// point to next args
                        i++;
                        i++;
			sleep(3);
			continue;
                        }

                // --h print command line options
                if ((strncmp(argv[i],"-h",2))==0)
                {
                        printf("usage: server [-h] | [-s <resolution>] [--loglevel {0-9}]\n");
                        exit (0);
                        }
                } //end for loop

	} //////////////////////////////////////////////////////////////////////end ParseCommandLine

////////////////////////////////////////////////////////////////////////////////////////////////////
// general initialization function 
////////////////////////////////////////////////////////////////////////////////////////////////////
int Init(int argc, char**argv, int *w, int *h)
{

	// check for command line parms
	// set default screen size
	*w=800;
	*h=600;
	ParseCommandLine(argc, argv, w, h);

	// set up mutexing
//	pthread_mutexattr_init(&m_attr);
//	pthread_mutexattr_settype(&m_attr,PTHREAD_MUTEX_ERRORCHECK_NP);
//	pthread_mutex_init(&logmutex, &m_attr);
	InitMutex(&logmutex);

	// set up logging
	ftime(&starttime);
	loglevel=0;
	logcount=0;
	ThreadLog("Init: Logging started.");

	// init command entry stuff
	commandstarted=false;         // ie did we see a ']' at the first char position ?
	firstchar=true;         // are we getting stuff after the first char position ?

	// setup threading
	pthread_attr_init(&attr);    // initialize attr with default attributes
	pthread_attr_setscope(&attr, PTHREAD_SCOPE_SYSTEM);  // system-wide contention

	// create the socket
	sock = new CUSock;
	sock->SetLogFunc(ThreadLog);

	// 
	playerid = new CString;
	characterid = new CString;

	} //////////////////////////////////////////////////////////////////////////////////end Init

////////////////////////////////////////////////////////////////////////////////////////////////////
// PrintError 
////////////////////////////////////////////////////////////////////////////////////////////////////
void PrintError( int e )
{
	switch (e)
	{
	case CW_OK:
		printf ("No Error.\n");
		break;

	case CW_ERR_FILE_OPEN:
		printf ("Failed to open file.\n");
		break;

	case CW_ERR_CONVERSION:
		printf ("Failed to scanconvert input.\n");
		break;

	case CW_ERR_TEXLOAD:
		printf ("Failed to load textures.\n");
		break;

	case CW_ERR_RENDER:
		printf ("Failed to render.\n");
		break;

	case CW_ERR_SUBMODLOAD:
		printf ("Failed to load sub model.\n");
		break;

	case CW_ERR_SUBPARTLOAD:
		printf ("Failed to load model part.\n");
		break;

	case CW_ERR_MEMALLOCATE:
		printf ("Failed to allocate memory.\n");
		break;

	default:
		printf ("Unhandled Error.\n");
		break;
		}
		
	} ////////////////////////////////////////////////////////////////////////////end PrintError

////////////////////////////////////////////////////////////////////////////////////////////////////
// main
////////////////////////////////////////////////////////////////////////////////////////////////////
int main( int argc, char **argv )
{
int		result;				// results of call
int		videoFlags;			// Flags to pass to SDL_SetVideoMode
int		done = false;			// main loop variable
SDL_Event	event;				// used to collect events
int 		isActive = true;		// whether or not the window is active
CString		text;
GLfloat		matrix[16];
char		buf[BUFFSIZE];
int 		screen_width, screen_height;

	printf("Init\n");
	Init(argc, argv, &screen_width, &screen_height);
	snprintf(buf,BUFFSIZE,"Main: After Init: w = %d h = %d",screen_width,screen_height);
	ThreadLog(buf);

	// initialize SDL 
	InitSDL(videoFlags, screen_width, screen_height);
	ThreadLog("Main: SDL Init complete.");

	// initialize OpenGL 
	InitGL();
	ThreadLog("Main: OpenGL Init complete.");

	// Read in the data 
	world = new CWorld;
	ThreadLog("Main: Created world engine.");
	world->SetLogFunc(ThreadLog);

	// setup the mutexing
//	pthread_mutexattr_init(&m_attr);
//	pthread_mutexattr_settype(&m_attr,PTHREAD_MUTEX_ERRORCHECK_NP);

	// Initialize the world engine
//	if (!(world->Init(&m_attr))) 
	if (!(world->Init())) 
	{
		ThreadLog("Main: Error world engine Init()  failed.");
		exit (1);
		}
	world->SetTextWinSize(100,100);
	ThreadLog("Main: Initialized world engine.");

	if ((result=world->Load("world1.dat"))!=CW_OK) PrintError(result);
	ThreadLog("Main: Loaded World.");

	// resize the initial window 
	ResizeWindow(world, screen_width, screen_height);

	// start the simulator running
	world->Run();

	// wait for events 
	while ( !done )
	{
		// handle the events in the queue 

		while ( SDL_PollEvent( &event ) )
		{
		SDL_keysym	keysym;

			switch( event.type )
			{
			case SDL_ACTIVEEVENT:
				// Something's happend with our focus
				// If we lost focus or we are iconified, we
				// shouldn't draw the screen
				if ( event.active.gain == 0 ) isActive = false;
				else isActive = true;
				break;

			case SDL_VIDEORESIZE:
				// handle resize event 
				surface = SDL_SetVideoMode( event.resize.w, event.resize.h, SCREEN_BPP, videoFlags );
				if ( !surface )
				{
					snprintf(buf,BUFFSIZE,"Main: Could not get a surface after resize: %s\n", SDL_GetError( ) );
					ThreadLog(buf);
					Quit( 1 );
					}
				screen_width=event.resize.w;
				screen_height=event.resize.h;
				snprintf(buf,BUFFSIZE,"Main: Resizing with w=%d h=%d %s\n", screen_width, screen_height );
				ThreadLog(buf);
				printf(buf);
				ResizeWindow(world, screen_width, screen_height);
				break;

			case SDL_KEYDOWN:
			case SDL_KEYUP:
				// handle key presses 
				HandleKey(&event, world);
				break;

			case SDL_QUIT:
				// handle quit requests 
				done = true;
				break;

			default:
				break;
				} // end switch
			} // end while poll event
		// simulate the scene
		world->Sim();

		// draw the scene 
		if ( isActive ) 
		{
//			CalcFrameRate();
			world->Render();
			// Text
//			DrawText();
//			text.SetString(framestring);
//			font.DrawText(0, 700, 1.0f, 1.0f, 1.0f, width, height, &text);
//			text.SetString("THIS IS A REALLY BIG TEST STRING");
//			font.DrawText(300, 300, 1.0f, 1.0f, 1.0f, width, height, &text);
//			textwindow.InsertLine(&text);
//			textwindow.DrawTextWindow(width, height);
			}
		} // end while done

	// clean ourselves up and exit 
//	dmalloc_shutdown();
	Quit( 0 );

	// Should never get here 
	return( 0 );
	} //////////////////////////////////////////////////////////////////////////////////end main

