////////////////////////////////////////////////////////////////////////////////////////////////////
// Client
// input.cpp -- This is it.
// contains all the keyboard input handling stuff
//
// World III
//
// by Eric J Krejci
// Copyright (C) 1999 Eric J Krejci
// Revised : 31 Jan 2006
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation version 2.
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
// Contact:
//              email           worldii@viagloria.com
//              web             www.viagloria.com
////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////
// Includes
////////////////////////////////////////////////////////////////////////////////////////////////////
#include <sys/types.h>
#include <sys/socket.h>
#include <arpa/inet.h>

#include "SDL.h"
#include "client.h"
#include "cstring.hpp"
#include "cworld.hpp"
#include "cusock.hpp"
#include "wiii_util.h"

////////////////////////////////////////////////////////////////////////////////////////////////////
// Local Globals
////////////////////////////////////////////////////////////////////////////////////////////////////
CString		commandstring;
char		keybuf[256];
bool		commandstarted;		// ie did we see a ']' at the first char position ?
bool		firstchar;		// are we getting stuff after the first char position ?

////////////////////////////////////////////////////////////////////////////////////////////////////
// Externals
////////////////////////////////////////////////////////////////////////////////////////////////////
extern void			Quit(int);
extern int			TurnRight;
extern int			TurnLeft;
extern int			Forward;
extern int			Backward;
extern int			PgUp;
extern int			PgDn;
extern int			Home;
extern SDL_Surface		*surface;
extern CUSock			*sock;
extern bool			connected;
extern pthread_mutexattr_t	m_attr;
extern void 			*PortMon(void *ptr);
extern pthread_t		portmon_tid;
extern pthread_attr_t		attr; 
extern CWorld			*world;
extern CString			*playerid;
extern CString			*characterid;

////////////////////////////////////////////////////////////////////////////////////////////////////
// ConnectFunc
// These functions are not required
// They are provided for notification purposes
// They are called by a different thread
////////////////////////////////////////////////////////////////////////////////////////////////////
int ConnectFunc(connecttype *cparm)
{
	// OK OK, we should have a mutex here,
	// but this is test code, and i am being lazy
	connected=true;
	ThreadLog("ConnectFunc: Connect callback.");
	return (0);
	} ///////////////////////////////////////////////////////////////////////////end ConnectFunc
                                                                                                                                        
////////////////////////////////////////////////////////////////////////////////////////////////////
// DisconnectFunc
// These functions are not required
// They are provided for notification purposes
// They are called by a different thread
////////////////////////////////////////////////////////////////////////////////////////////////////
int DisconnectFunc(connecttype *cparm)
{
	// OK OK, we should have a mutex here,
	// but this is test code, and i am being lazy
	connected=false;
	ThreadLog("DisconnectFunc: Disconnect callback.");
	return (0);
	} ////////////////////////////////////////////////////////////////////////end DisconnectFunc

////////////////////////////////////////////////////////////////////////////////////////////////////
// Connect
////////////////////////////////////////////////////////////////////////////////////////////////////
bool Connect(CString *name)
{
struct sockaddr_in	servaddr;
struct hostent 		ret;
struct hostent 		*res;
char			ipbuf[BUFFSIZE];
char			buf[BUFFSIZE],*dum;
char			*hostname;
int			myerr,result,portint;
CString 		logitem;
void 			*p;

	// Init the sock and connect it
	bzero(&servaddr, sizeof(servaddr));
	servaddr.sin_family = AF_INET;
                                                                                                                                        
	// NOTE that IP and PORT are in NET form
	servaddr.sin_port = htons(BASEWORLDPORT);
//	servaddr.sin_port = htons(atoi(port->GetString()));
	name->GetString(&hostname);
	result=gethostbyname_r(hostname, &ret, ipbuf, BUFFSIZE, &res,&myerr);
	if (hostname) delete [] hostname;
	if (result==0)
	{ 
		logitem.SetString("Connect: IP addr='");
//		snprintf(buf,BUFFSIZE,"Connect: IP addr='%x' port='%s'",ret.h_addr_list[0],port->GetString());
		snprintf(buf,BUFFSIZE,"%x",ret.h_addr_list[0]);
//		logitem=logitem+buf+"' port='"+(*port)+"'\n";
		logitem=logitem+buf+"' port='"+"'\n";
//		printf(buf);
		logitem.Display();

		servaddr.sin_addr=*((struct in_addr *)(ret.h_addr_list[0]));
                                                                                                                                        
		printf ("The uintl = '%x' \n",servaddr.sin_addr.s_addr);
		ThreadLog("Connect: Before SetLogFunc call.");
		sock->SetLogFunc(ThreadLog); 
		ThreadLog("Connect: Before sock.Init call.");
//		sock->Init(&m_attr);
		sock->Init();

		// the client isn't going to wander off and do other things
		// while waiting for connect
		sock->SetDisConnFunc(DisconnectFunc);
		sock->Create(&servaddr,sizeof(servaddr));
		if (!sock->Connect())
		{
			ThreadLog("Connect: Failed to connect.");
			return (false);
			}
		else connected=true;
		ThreadLog("Connect: CONNECTED!");

		// now store sock ptr in world
		world->SetSock(sock);

		// now start the portmon. 
		// this listens for returning messages and displays them
		p=(void *)world;
//		pthread_create( &portmon_tid, &attr, PortMon, p);
		CreateThread( &portmon_tid, PortMon, p);

		return (true);
		}
	else return (false);
	
	} ///////////////////////////////////////////////////////////////////////////////end Connect

////////////////////////////////////////////////////////////////////////////////////////////////////
// DisConnect
////////////////////////////////////////////////////////////////////////////////////////////////////
bool DisConnect(CString *name)
{
	if (connected==true)
	{
		// close port and reset values
		sock->Close();
		connected=false;
		world->SetSock(0);
		}
	} ////////////////////////////////////////////////////////////////////////////end DisConnect

////////////////////////////////////////////////////////////////////////////////////////////////////
// HandleInputCommand
////////////////////////////////////////////////////////////////////////////////////////////////////
int HandleInputCommand(CString *cs, CWorld *w)
{
CString *temp;
char	buf[BUFFSIZE];

	// OK Use the CString Tokenizer to get the parts of the commands
	cs->SetChar(0,' ');		// we know its a command so chop off the leading ']'
	cs->TokenInit(" ");		// tell the tokenizer to break on {' '}
	temp=cs->GetToken();		// get another CString with the leading token
	if (temp==NULL) return (1);     // there was no token to be found, ie no command, so return error
	printf("HandleInputCommand: Passed temp==NULL\n");

	// so now handle some immediate commands, first look for connect
//	if (((*temp)==TOKEN_CONNECT)==0)
	if ((*temp)==TOKEN_CONNECT)
	{
	CString *hostname;

		if (hostname=cs->GetToken())
		{
		CString	mesg;

			printf("HandleInputCommand: Before Connect call\n");
			if (Connect(hostname))
			{
			char cc='+';
					
				// show that we connected
//				w->Append(cc);
				mesg.SetString("Connected.");
				w->InsertLine(&mesg);
				}
			else
			{
			char cc='-';
				
				// found a command!
//				w->Append(cc);
				mesg.SetString("Not Connected.");
				w->InsertLine(&mesg);
				}
			}
		return(0);
		}

	// look for tell
//	if (((*temp)==TOKEN_TELL)==0)
	if ((*temp)==TOKEN_TELL)
	{
	CString	*destcharacter;
	CString	*mesg;

		if (destcharacter=(cs->GetToken()))
		if (mesg=(cs->GetRemainder()))
		{
			w->InsertLine(playerid);
			snprintf(buf,BUFFSIZE,"%s|%s|%s tells you %s||",
				TOKEN_TELL,
				destcharacter->GetStringPtr(),
				characterid->GetStringPtr(),
				mesg->GetStringPtr());
			w->InsertLine(buf); 
			sock->Send(buf,strlen(buf),0);
			}
		else  w->InsertLine("Error found after tell. Please retry."); 
		return(0);
		}

	// look for playerid
//	if (((*temp)==TOKEN_PLAYERID)==0)
	if ((*temp)==TOKEN_PLAYERID)
	{
	CString	*password;

		if (playerid=(cs->GetToken()))
		if (password=(cs->GetToken()))
		{
			w->InsertLine(playerid);
			snprintf(buf,BUFFSIZE,"%s|%s|%s||",
				TOKEN_PLAYERID,
				playerid->GetStringPtr(),
				password->GetStringPtr());
			sock->Send(buf,strlen(buf),0);
			}
		else  w->InsertLine("Error found after playerid. Please retry."); 
		return(0);
		}

	// look for listplayers
//	if (((*temp)==TOKEN_LISTACTIVEPLAYERS)==0)
	if ((*temp)==TOKEN_LISTACTIVEPLAYERS)
	{
		snprintf(buf,BUFFSIZE,"%s||",TOKEN_LISTACTIVEPLAYERS);
		sock->Send(buf,strlen(buf),0);
		w->InsertLine("Found listact.");
		return(0);
		}


	// look for play
//	if (((*temp)==TOKEN_PLAY)==0)
	if ((*temp)==TOKEN_PLAY)
	{
	CString	*charselected;

		if (charselected=(cs->GetToken()))
		{
			*characterid=*charselected;
			snprintf(buf,BUFFSIZE,"%s|%s||",TOKEN_PLAY,charselected->GetStringPtr());
			sock->Send(buf,strlen(buf),0);
			}
		else  w->InsertLine("Error found after play. Please retry."); 
		return(0);
		}

	// look for ping
//	if (((*temp)=="ping")==)
	if ((*temp)=="ping")
	{
		snprintf(buf,BUFFSIZE,"ping||");
		sock->Send(buf, strlen(buf), 0);
		return(0);
		}
	
	} ////////////////////////////////////////////////////////////////////end HandleInputCommand

////////////////////////////////////////////////////////////////////////////////////////////////////
// HandleKey
// function to handle key press events
// TurnRight
// TurnLeft
// Forward
// Backward
//  KMOD_NONE  = 0x0000,
//  KMOD_LSHIFT= 0x0001,
//  KMOD_RSHIFT= 0x0002,
//  KMOD_LCTRL = 0x0040,
//  KMOD_RCTRL = 0x0080,
//  KMOD_LALT  = 0x0100,
//  KMOD_RALT  = 0x0200,
//  KMOD_LMETA = 0x0400,
//  KMOD_RMETA = 0x0800,
//  KMOD_NUM   = 0x1000,
//  KMOD_CAPS  = 0x2000,
//  KMOD_MODE  = 0x4000,
////////////////////////////////////////////////////////////////////////////////////////////////////
void HandleKey(SDL_Event *event, CWorld *w)
{
SDLMod		modkeys;
SDLKey		key;
static char 	c;

        modkeys=SDL_GetModState();
        switch (event->type)
        {
        case    SDL_KEYDOWN:
//		printf ("KEYDOWN \n");
		key=event->key.keysym.sym;
                switch ( (key) )
                {
                case SDLK_ESCAPE:
                        // ESC key was pressed
                        Quit( 0 );
                        break;
                                                                                                                                                                   
                case SDLK_F1:
                        // F1 key was pressed this toggles fullscreen mode
                        SDL_WM_ToggleFullScreen( surface );
                        break;

                case SDLK_RIGHT:
                        // Right arrow key was pressed
			w->RightArrow(true);
                        break;

                case SDLK_LEFT:
                        // Left arrow key was pressed
			w->LeftArrow(true);
                        break;

                case SDLK_UP:
                        // Up arrow key was pressed this moves the player forward
			w->UpArrow(true);
                        break;

                case SDLK_DOWN:
                        // Down arrow key was pressed this causes the player to move backwards
			w->DnArrow(true);
                        break;

                case SDLK_PAGEUP:
                        // Lookup or down
			w->PgUp(true);
                        break;

                case SDLK_INSERT:
                        // Lookup or down
			w->Ins(true);
                        break;

                case SDLK_DELETE:
                        // Lookup or down
			w->Del(true);
                        break;

                case SDLK_PAGEDOWN:
                        // Lookup or down
			w->PgDn(true);
                        break;

                case SDLK_HOME:
                        // Lookup or down
			w->LevelLook();
                        break;

		case SDLK_BACKSPACE:
			commandstring.Chop();
			w->Chop();
			break;

		case SDLK_RETURN:
			if (firstchar) break;
			if ((!firstchar)&&(commandstarted))
			{
			char cc='!';
				
				// found a command!
				w->Append(cc);		// append exclamation point to say we recognize a command
				w->Return();
				HandleInputCommand(&commandstring,w);
				commandstring.Reset();
				}
			if ((!firstchar)&&(!commandstarted))
			{
				// this could possibly be the place to put talking commands
				commandstring.Reset();
				w->Return();
				}
			firstchar=true;
			commandstarted=false;
			break;

		case SDLK_RIGHTBRACKET:
			c=*((char*)(&key));
			w->Append(c);
			commandstring.Append(c);
			if (firstchar)
			{
				commandstarted=true;
				firstchar=false;
				}
			else firstchar=false;
			break;

		case SDLK_EXCLAIM:
		case SDLK_QUOTEDBL:
		case SDLK_HASH:
		case SDLK_DOLLAR:
		case SDLK_AMPERSAND:
		case SDLK_QUOTE:
		case SDLK_LEFTPAREN:
		case SDLK_RIGHTPAREN:
		case SDLK_ASTERISK:
		case SDLK_PLUS:
		case SDLK_COMMA:
		case SDLK_MINUS:
		case SDLK_PERIOD:
		case SDLK_SLASH:
		case SDLK_COLON:
		case SDLK_SEMICOLON:
		case SDLK_LESS:
		case SDLK_EQUALS:
		case SDLK_GREATER:
		case SDLK_QUESTION:
		case SDLK_AT:
		case SDLK_LEFTBRACKET:
		case SDLK_BACKSLASH:
		case SDLK_CARET:
		case SDLK_UNDERSCORE:
		case SDLK_BACKQUOTE:
			c=*((char*)(&key));
			w->Append(c);
			commandstring.Append(c);
			break;

		case SDLK_F2:
		case SDLK_F3:
		case SDLK_F4:
		case SDLK_F5:
		case SDLK_F6:
		case SDLK_F7:
		case SDLK_F8:
		case SDLK_F9:
		case SDLK_F10:
		case SDLK_F11:
		case SDLK_F12:
		case SDLK_F13:
		case SDLK_F14:
		case SDLK_F15:
			break;

		case SDLK_LSHIFT:
		case SDLK_RSHIFT:
			w->Shift(true);
			break;

		case SDLK_LALT:
		case SDLK_RALT:
			break;

		case SDLK_LCTRL:
		case SDLK_RCTRL:
			break;

		case SDLK_LMETA:
		case SDLK_RMETA:
			break;

		default:	// get all the "visible" characters
			c=*((char*)(&key));
//			w->Append(&c);
//			commandstring.Append(&c);
			w->Append(c);
//			printf ("After w->Append() \n");
			commandstring.Append(c);
//			printf ("After commandstring.Append() \n");
			if (firstchar) commandstarted=false;
			firstchar=false;
			break;
			} //end switch keysym
		break;

        case    SDL_KEYUP:

                switch ( event->key.keysym.sym )
                {
                case SDLK_RIGHT:
                        // Right arrow key was pressed
			w->RightArrow(false);
                        break;

                case SDLK_LEFT:
                        // Left arrow key was pressed
			w->LeftArrow(false);
                        break;

                case SDLK_UP:
                        // Up arrow key was pressed this moves the player forward
			w->UpArrow(false);
                        break;

                case SDLK_DOWN:
                        // Up arrow key was pressed this moves the player forward
			w->DnArrow(false);
                        break;

                case SDLK_PAGEUP:
                        // Lookup or down
			w->PgUp(false);
                        break;

                case SDLK_PAGEDOWN:
                        // Lookup or down
			w->PgDn(false);
                        break;

                case SDLK_INSERT:
                        // Lookup or down
			w->Ins(false);
                        break;

                case SDLK_DELETE:
                        // Lookup or down
			w->Del(false);
                        break;

                case SDLK_HOME:
                        // Lookup or down
                        break;

		case SDLK_LSHIFT:
		case SDLK_RSHIFT:
			w->Shift(false);
			break;


                        } // end switch keysym
                break;
                } // end switch event->type

        return;

	} /////////////////////////////////////////////////////////////////////////////end HandleKey



