////////////////////////////////////////////////////////////////////////////////////////////////////
// console.cpp 
// This is the console aplication.
// This will allow the server to be spawned as standalone. 
//
// World III
//
// by Eric J Krejci
// Copyright (C) 1999 Eric J Krejci 
// Revised : 28 Jun 2006
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation version 2.
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
// Contact:
//              email           worldii@viagloria.com
//              web             www.viagloria.com
////////////////////////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////////////////////////
// Defines
////////////////////////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////////////////////////
// Includes
////////////////////////////////////////////////////////////////////////////////////////////////////
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>
#include <curses.h>
#include <strings.h>
#include <sys/types.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/socket.h>
#include <sys/wait.h>
#include <pthread.h>
#include <sys/timeb.h>
#include <netdb.h>

#include "wiii_structures.h"
#include "wiii_curses.h"
#include "console.h"
#include "worldiii.h"
#include "wiii_util.h"
#include "cusock.hpp"
//#include "copts.hpp"

////////////////////////////////////////////////////////////////////////////////////////////////////
// Globals
////////////////////////////////////////////////////////////////////////////////////////////////////
//CThreadList	tlist;				// All threads will get tracked via this structure
pthread_attr_t		attr; 				// thread attr structure for setting attributes.
//pthread_mutexattr_t	m_attr;


bool		keepmon;
bool		connected;
int		menucontext;
int		listenstate;
//CIterator	serverlist;		// A list of all servers and their info
pthread_mutex_t screenmutex;
pthread_mutex_t logmutex;
FILE 		*logfile;
//char		logfilename[50]="logs/worldiii.log";
//char		portmonlogname[50]="logs/portmon.log";
//char		qloaderlogname[50]="logs/qloader.log";
//char		handlerlogname[50]="logs/handler.log";
char		serverhostname[80]="worldiii";
char		serverhostaddr[15];
CUSock		*consock;                // This is the console socket
long		logcount;
int		loglevel;
                                        // all control traffic from console and status output
                                        // use this socket
struct timeb    starttime;



////////////////////////////////////////////////////////////////////////////////////////////////////
// External Functions
////////////////////////////////////////////////////////////////////////////////////////////////////
extern	void *	PortMon(void *p);
extern	void	ThreadLog(char *s);

////////////////////////////////////////////////////////////////////////////////////////////////////
// ParseCommandLine
////////////////////////////////////////////////////////////////////////////////////////////////////
void ParseCommandLine(int argc, char** argv)
{
int	i;

	// set defaults for parms
	strncpy(serverhostname,"worldiii",80);
	loglevel=0;

	//getting parms
	for (i=0;i<argc;i++)
	{
		// -h which host to manage
		if ((strncmp(argv[i],"-h",2))==0)
		{
			if (argc>=i+2)
			{
				printf("Found -h = '%s'\n",argv[i+1]);
				sleep(1);
				strncpy(serverhostname,argv[i+1],80);
				}
			i++;
			}

		// --loglevel how much logging to do
		if ((strncmp(argv[i],"--loglevel",10))==0)
		{
                	printf("DEBUGMODE ON\n");
	                loglevel=atoi(argv[i+1]);
			i++;
			}
		} //end for loop

	} //////////////////////////////////////////////////////////////////////end ParseCommandLine

////////////////////////////////////////////////////////////////////////////////////////////////////
// Init
////////////////////////////////////////////////////////////////////////////////////////////////////
void Init(int argc, char** argv)
{

	// Parse command line
	ParseCommandLine(argc,argv);

	// print out parms if requested
	if (loglevel>0)
	{
	int i;

		for (i=0;i<argc;i++) printf("Argument %d, s='%s' \n",i,argv[i]);
		sleep (2);
		}
	
	
	// init the threading model 
//	pthread_mutexattr_init(&m_attr);
	pthread_attr_init(&attr);    // initialize attr with default attributes 
	pthread_attr_setscope(&attr, PTHREAD_SCOPE_SYSTEM);  // system-wide contention 

//#ifdef LINUX
//	pthread_mutexattr_settype(&m_attr,PTHREAD_MUTEX_ERRORCHECK_NP);
//#endif
//#ifdef SOLARIS
//	pthread_mutexattr_settype(&m_attr,PTHREAD_MUTEX_ERRORCHECK);
//	pthread_setconcurrency(10);
//#endif

//	pthread_mutex_init(&screenmutex, &m_attr);
//	pthread_mutex_init(&logmutex, &m_attr);
	InitMutex(&screenmutex);
	InitMutex(&logmutex);
	connected=false;
	} //////////////////////////////////////////////////////////////////////////////////end Init

////////////////////////////////////////////////////////////////////////////////////////////////////
// InitSock
////////////////////////////////////////////////////////////////////////////////////////////////////
void InitSock()
{

     } /////////////////////////////////////////////////////////////////////////////////end InitSock

////////////////////////////////////////////////////////////////////////////////////////////////////
// DeInit
////////////////////////////////////////////////////////////////////////////////////////////////////
void DeInit()
{
     // 

     } ///////////////////////////////////////////////////////////////////////////////////end DeInit

////////////////////////////////////////////////////////////////////////////////////////////////////
// Ping
////////////////////////////////////////////////////////////////////////////////////////////////////
int PingClient(char *c, curswin *p)
{
     MyPrint(p->mainwin,"Enter the number of the server's client to ping",true);
     return (0);
     } /////////////////////////////////////////////////////////////////////////////////////end Ping

////////////////////////////////////////////////////////////////////////////////////////////////////
// Show
////////////////////////////////////////////////////////////////////////////////////////////////////
int Show(curswin *p)
{
     MyPrint(p->mainwin,"Show",true);
     return (0);
     } /////////////////////////////////////////////////////////////////////////////////////end Ping

////////////////////////////////////////////////////////////////////////////////////////////////////
// InitScreen
////////////////////////////////////////////////////////////////////////////////////////////////////
void InitScreen(curswin **p)
{
	// init the ncurses library
	initscr();
	cbreak();
	noecho();
	nonl();
	intrflush(stdscr, false);
	keypad(stdscr, true);

	// get mem for curswin
	(*p) = new curswin;

	// create the windows
	(*p)->menuwin =newwin(int(LINES/2),int(COLS/2), 0,0);
	(*p)->debugwin=newwin(int(LINES/2),int(COLS/2),0,int(COLS/2));
	(*p)->mainwin =newwin(int(LINES/2),COLS,int(LINES/2),0);
	scrollok((*p)->menuwin, true);
	scrollok((*p)->debugwin,true);
	scrollok((*p)->mainwin, true);
	menucontext=MAIN_CONTEXT;
	} ///////////////////////////////////////////////////////////////////////////////end InitScreen

////////////////////////////////////////////////////////////////////////////////////////////////////
// DeInitScreen
////////////////////////////////////////////////////////////////////////////////////////////////////
void DeInitScreen(curswin *p)
{
     // DeInit the ncurses library
     wrefresh(p->menuwin);
     wrefresh(p->debugwin);
     wrefresh(p->mainwin);
     endwin();
     } /////////////////////////////////////////////////////////////////////////////end DeInitScreen

////////////////////////////////////////////////////////////////////////////////////////////////////
// Menu
////////////////////////////////////////////////////////////////////////////////////////////////////
void PrintMenu(curswin *p)
{
     switch (menucontext)
     {
     case MAIN_CONTEXT:
          MyLocPrint(p->menuwin,0,0,"Main                             ");
          MyLocPrint(p->menuwin,1,0,"'s' = Start/Stop/Manage Server   ");
          MyLocPrint(p->menuwin,2,0,"'c' = Communicate with clients   ");
          MyLocPrint(p->menuwin,3,0,"'m' = Manage connections         ");
          MyLocPrint(p->menuwin,4,0,"'d' = Show server statistics     ");
          MyLocPrint(p->menuwin,5,0,"'t' = Test                       ");
          MyLocPrint(p->menuwin,6,0,"                                 ");
          MyLocPrint(p->menuwin,7,0,"                                 ");
          MyLocPrint(p->menuwin,8,0,"'z' = Exit console.              ");
          break;

     case START_CONTEXT:
          MyLocPrint(p->menuwin,0,0,"Manage Servers                   ");
          MyLocPrint(p->menuwin,1,0,"'c' = Connect to server          ");
          MyLocPrint(p->menuwin,2,0,"'d' = Disconnect from server     ");
          MyLocPrint(p->menuwin,3,0,"'a' = Start server               ");
          MyLocPrint(p->menuwin,4,0,"'b' = Stop server                ");
          MyLocPrint(p->menuwin,5,0,"'l' = Start listening            ");
          MyLocPrint(p->menuwin,6,0,"'u' = Stop listening             ");
          MyLocPrint(p->menuwin,7,0,"                                 ");
          MyLocPrint(p->menuwin,8,0,"'z' = Main Menu                  ");
          break;

     case MANAGE_CONTEXT:
          MyLocPrint(p->menuwin,0,0,"Manage Connections               ");
          MyLocPrint(p->menuwin,1,0,"'p' = Pause Connection           ");
          MyLocPrint(p->menuwin,2,0,"'r' = Resume Connection          ");
          MyLocPrint(p->menuwin,3,0,"'s' = Disconnect single client   ");
          MyLocPrint(p->menuwin,4,0,"'g' = Disconnect group of clients");
          MyLocPrint(p->menuwin,5,0,"'a' = Disconnect all clients     ");
          MyLocPrint(p->menuwin,6,0,"                                 ");
          MyLocPrint(p->menuwin,7,0,"                                 ");
          MyLocPrint(p->menuwin,8,0,"'z' = Main Menu                  ");
          break;

     case DATA_CONTEXT:
          MyLocPrint(p->menuwin,0,0,"Show Statistics                  ");
          MyLocPrint(p->menuwin,1,0,"'s' = Summary                    ");
          MyLocPrint(p->menuwin,2,0,"'c' = Server statistics          ");
          MyLocPrint(p->menuwin,3,0,"'d' = Database statistics        ");
          MyLocPrint(p->menuwin,4,0,"                                 ");
          MyLocPrint(p->menuwin,5,0,"                                 ");
          MyLocPrint(p->menuwin,6,0,"                                 ");
          MyLocPrint(p->menuwin,7,0,"                                 ");
          MyLocPrint(p->menuwin,8,0,"'z' = Main Menu                  ");
          break;

     case COMM_CONTEXT:
          MyLocPrint(p->menuwin,0,0,"Communicate with Clients         ");
          MyLocPrint(p->menuwin,1,0,"'l' = List clients               ");
          MyLocPrint(p->menuwin,2,0,"'s' = Send msg, single client    ");
          MyLocPrint(p->menuwin,3,0,"'g' = Send msg, group of clients ");
          MyLocPrint(p->menuwin,4,0,"'a' = Send msg, all clients      ");
          MyLocPrint(p->menuwin,5,0,"                                 ");
          MyLocPrint(p->menuwin,6,0,"                                 ");
          MyLocPrint(p->menuwin,7,0,"                                 ");
          MyLocPrint(p->menuwin,8,0,"'z' = Main Menu                  ");
          break;

     case TEST_CONTEXT:
          MyLocPrint(p->menuwin,0,0,"Test Communicate with Server     ");
          MyLocPrint(p->menuwin,1,0,"'s' = Show servers               ");
          MyLocPrint(p->menuwin,2,0,"'p' = Ping servers               ");
          MyLocPrint(p->menuwin,3,0,"'d' = Debug                      ");
          MyLocPrint(p->menuwin,4,0,"                                 ");
          MyLocPrint(p->menuwin,5,0,"                                 ");
          MyLocPrint(p->menuwin,6,0,"                                 ");
          MyLocPrint(p->menuwin,7,0,"                                 ");
          MyLocPrint(p->menuwin,8,0,"'z' = Main Menu                  ");
     default:
          break;
          }
     } ////////////////////////////////////////////////////////////////////////////////end PrintMenu

////////////////////////////////////////////////////////////////////////////////////////////////////
// Connect
////////////////////////////////////////////////////////////////////////////////////////////////////
void Connect(curswin *cw)
{
char		buf[100];
char		dst[50];
pthread_t       ttemp;
//void 		*p=NULL;
struct sockaddr_in servaddr;
hostent		*hent;

	bzero(&servaddr, sizeof(servaddr));
	servaddr.sin_family = AF_INET;

	// NOTE that IP and PORT are in NET form
	servaddr.sin_port = htons(CONWORLDPORT);
	hent=gethostbyname(serverhostname);
	if (hent==NULL)
	{
		ThreadLog("Connect: Gethostbyname failed.");
		connected=false;
		MyPrint(cw->debugwin,"Connect: Gethostname failed. \n", true);
		return;
		}
	else
	{
		servaddr.sin_addr=*((struct in_addr*)hent->h_addr);
		if (loglevel>=2)
		{
			snprintf(buf,100,"Connecting to IP = '%s'\n",
				inet_ntop(AF_INET,(void*)&(servaddr.sin_addr),dst,50)
				);
			MyPrint(cw->debugwin,buf, true);
			}
		}

	
	// ThreadLog("Main: Before SetLogFunc call.");
	consock->SetLogFunc(ThreadLog);
	if (loglevel>=1) ThreadLog("Connect: Before sock.Init call.\n");
	if (!consock->Init())
	{
		ThreadLog("Connect: ERROR consock->Init() Failed.\n");
		exit(1);
		}
	consock->Create(&servaddr,sizeof(servaddr));
	if (!consock->Connect())
	{
		ThreadLog("Connect: Failed to connect.\n");
		connected=false;
		}
	else
	{
		// start a new thread to handle the portmon 
		if (loglevel>=1) ThreadLog("Connect: Connected, exiting.\n");
		CreateThread( &ttemp, PortMon, (void *)cw);
		connected=true;
		}

	} ///////////////////////////////////////////////////////////////////////////////end Connect 

////////////////////////////////////////////////////////////////////////////////////////////////////
// ExecComm
////////////////////////////////////////////////////////////////////////////////////////////////////
void ExecComm(char *c, curswin *p)
{
//char testmsg[]="This is a message from console. Hello World.\n";
char msg[sizeof(int)];
//int  l,m;
unsigned char  len,m;

	switch (menucontext)
	{
	case MAIN_CONTEXT:
		switch (c[0])
		{
		case 's':
			menucontext=START_CONTEXT;
			break;

		case 'm':
			menucontext=MANAGE_CONTEXT;
			break;

		case 'c':
			menucontext=COMM_CONTEXT;
			break;

		case 'd':
			menucontext=DATA_CONTEXT;
			break;

		case 't':
			menucontext=TEST_CONTEXT;
			break;

		case 'z':
			menucontext=END_CONTEXT;
			break;

		default:
			beep();                              //("Please try again\n");
			break;
			} // end MAIN_CONTEXT
		break;

	case START_CONTEXT: 
		switch (c[0])
		{
		case 'a':
			if (connected) 
			{
				MyPrint(p->debugwin,"Start server \n", true);
				}
			else beep();
			break;

		case 'b':
			if (connected)
			{
				MyPrint(p->debugwin,"Terminate server \n", true);
				m=CON_COMM_TERM;
				len=sizeof(unsigned char);
				memcpy(msg,(void*)(&m),len);
				consock->Send((char*)msg, len, 0);
				// consock->Close()  is done at end of portmon
				keepmon=false;
				connected=false;
				}
			else beep();
			break;

		case 'c':
			if (connected)
			{
				beep();
				}
			else
			{
				MyPrint(p->debugwin,"Connect to server.\n", true);
				Connect(p); 
				ThreadLog("ExecComm: Connected, exiting.");
				}
			break;

		case 'd':
			if (connected)
			{
				MyPrint(p->debugwin,"Disconnect from server.\n", true);
				keepmon=false; 
				connected=false; 
				}
			else beep();
			break;

		case 'l':
			if (connected)
			{
				ThreadLog("ExecCom: Accept entered.\n");
				MyPrint(p->debugwin,"Accept client connections to server.\n", true);
				m=CON_COMM_LISTEN;
				len=sizeof(unsigned char);
				memcpy(msg,(void*)(&m),len);
				if (loglevel>=1) ThreadLog("ExecCom: Accept Before Send.\n");
				consock->Send((char*)msg, len, 0);
				if (loglevel>=1) ThreadLog("ExecCom: Accept After Send.\n");
				}
			else beep();
			break;

		case 'u':
			if (connected)
			{
				MyPrint(p->debugwin,"Pause client connections to server.\n", true);
				m=CON_COMM_PAUSE;
				len=sizeof(unsigned char);
				memcpy(msg,(void*)(&m),len);
				consock->Send((char*)msg, len, 0);
				}
			else beep();
			break;

		case 'z':
			menucontext=MAIN_CONTEXT;
			break;

		default:
			beep();                              //("Please try again\n");
			break;
			} // end START_CONTEXT
		break;

	case MANAGE_CONTEXT: 
		if (connected)
		switch (c[0])
		{
		case 'r':
			MyPrint(p->debugwin,"Resume simulation of client gameplay.\n", true);
			break;

		case 'p':
			MyPrint(p->debugwin,"Pause simulation of client gameplay.\n", true);
			break;

		case 's':
			// here we close a connection
			if (connected)
			{
				MyPrint(p->debugwin,"Disconnect single client \n", true);
				m=CON_COMM_CLOSE;
				len=sizeof(unsigned char);
				memcpy(msg,(void*)(&m),len);
				consock->Send((char*)msg, len, 0);
				}
			else beep();
			break;

		case 'g':
			MyPrint(p->debugwin,"Disconnect a group of clients \n", true);
			break;

		case 'a':
			MyPrint(p->debugwin,"Disconnect all clients \n", true);
			break;

		case 'z':
			menucontext=MAIN_CONTEXT;
			break;

		default:
			beep();                              //("Please try again\n");
			break;
			} // end MANAGE_CONTEXT
		else 
		{
			beep();
			menucontext=MAIN_CONTEXT;
			}
		break;

	case DATA_CONTEXT: 
		if (connected)
		switch (c[0])
		{
		case 's':
			MyPrint(p->debugwin,"Show summary statistics \n", true);
			break;

		case 'c':
			MyPrint(p->debugwin,"Show server statistics \n", true);
			break;

		case 'd':
			MyPrint(p->debugwin,"Show database statistics \n", true);
			break;

		case 'z':
			menucontext=MAIN_CONTEXT;
			break;
 
		default:
			beep();                              //("Please try again\n");
			break;
			} // DATA_CONTEXT
		else 
		{
			beep();
			menucontext=MAIN_CONTEXT;
			}
		break;

	case TEST_CONTEXT:
		switch (c[0])
		{
		case 's':
			if (connected)
			{
				MyPrint(p->debugwin,"Show servers \n", true);
				m=CON_COMM_SHOW_SERVERS;
				len=sizeof(unsigned char);
				memcpy(msg,(void*)(&m),len);
				consock->Send((char*)msg, len, 0);
				}
			else beep();
			break;

		case 'p':
			if (connected)
			{     
				MyPrint(p->debugwin,"Ping server \n", true);
				m=CON_COMM_SHOW_SERVERS;
				len=sizeof(unsigned char);
				memcpy(msg,(void*)(&m),len);
				consock->Send((char*)msg, len, 0);
				}
			else beep();
			break;

		case 'd':
			if (connected)
			{     
				MyPrint(p->debugwin,"Debug Mode \n", true);
				m=CON_COMM_DEBUG;
				len=sizeof(unsigned char);
				memcpy(msg,(void*)(&m),len);
				consock->Send((char*)msg, len, 0);
				}
			else beep();
			break;

		case 'z':
			menucontext=MAIN_CONTEXT;
			break;

		default:
			beep();                              //("Please try again\n");
			break;
			} // end TEST_CONTEXT
		break;

	case COMM_CONTEXT: 
		if (connected)
		switch (c[0])
		{
		case 'l':
			if (connected)
			{
				MyPrint(p->debugwin,"List clients  \n", true);
				m=CON_COMM_LIST_CLIENTS;
				len=sizeof(unsigned char);
				memcpy(msg,(void*)(&m),len);
				consock->Send((char*)msg, len, 0);
				}
			else beep();
			break;

		case 's':
			MyPrint(p->debugwin,"Send Messgage Single Client \n", true);
			break;

		case 'g':
			MyPrint(p->debugwin,"Send Messgage To A Group Of Clients \n", true);
			break;

		case 'a':
			MyPrint(p->debugwin,"Send Messgage To All Clients \n", true);
			break;

		case 'z':
			menucontext=MAIN_CONTEXT;
			break;

		default:
			beep();                              //("Please try again\n");
			break;
			} // end COMM_CONTEXT
		else 
		{
			beep();
			menucontext=MAIN_CONTEXT;
			}
		break;
		}
	} //////////////////////////////////////////////////////////////////////////////end ExecComm

////////////////////////////////////////////////////////////////////////////////////////////////////
// Main
////////////////////////////////////////////////////////////////////////////////////////////////////
int main(int argc , char *argv[])
{
char		c[4];
char		buf[BUFFSIZE];
//WINDOW		*debugwin;
//WINDOW  	*mainwin;
//WINDOW  	*menuwin;
curswin		*winp;


	// init the screen & then threading and such
	// erase the screen
	Init(argc, argv);		// Init threading model

	consock = new CUSock;

	InitScreen(&winp);	// Init the curses library & screen mutex

	// Init the screen
	clear();
	refresh();

	// Start the listener
	// pack any params for the listener thread
	// tlist is a global thread tracking structure
	//c[0]=' ';
	do  
	{
		if (loglevel>=1) ThreadLog("Main: Print Menu.\n");
		PrintMenu(winp);
		if (loglevel>=1) ThreadLog("Main: Getting char.\n");
		c[0]=(char)getch();
		c[1]=0;
		if (loglevel>=2)
		{
			snprintf(buf,BUFFSIZE,"Main: Execute Command with char='%s'.\n",c);
			ThreadLog(buf);
			}
		ExecComm(c,winp);
		ThreadLog("Main: After execute.\n");
		} while ((menucontext!=END_CONTEXT)||(c[0]!='z'));

	if (connected)
	{
		// close socket
		keepmon=false;
		connected=false;
		}
	DeInit();
	MyPrint(winp->debugwin,"Before DeInitScreen \n", true);
	sleep(1);
	DeInitScreen(winp);
	if (loglevel>=1) printf("Returning from main.\n");
	} //////////////////////////////////////////////////////////////////////////////////end main

