////////////////////////////////////////////////////////////////////////////////////////////////////
// charclient.cpp -- This is it.
// This is the main module for the client to the worldiii game.
//  
// World III
//
// by Eric J Krejci
// Copyright (C) 1999 Eric J Krejci
// Revised : 31 Jan 2006
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation version 2.
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
// Contact:
//              email           worldii@viagloria.com
//              web             www.viagloria.com
////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////
// Includes
////////////////////////////////////////////////////////////////////////////////////////////////////
#include <pthread.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/timeb.h>

#include "rudp.h"
#include "cusock.hpp"
#include "wiii_util.h"

////////////////////////////////////////////////////////////////////////////////////////////////////
// client.c
// this test program is to start understanding udp
////////////////////////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////////////////////////
// Globals
////////////////////////////////////////////////////////////////////////////////////////////////////
pthread_mutex_t logmutex;
FILE *          logfile;
char            logfilename[50]="logs/worldiii-charclient.log";
long            logcount;
pthread_mutex_t screenmutex;
bool            consoleconnected;       // tells when we are connected to console.
CUSock          *newconsock;            // We use TCP for console
CUSock          consock;                // This is the console socket
bool            debug;                  // This puts server into debug mode 
pthread_attr_t  attr;                   // thread attr structure for setting attributes.
bool            donelistening;
struct timeb	starttime;
	

////////////////////////////////////////////////////////////////////////////////////////////////////
// dg_cli
// session is unused in client, as it is stored in the protocol
////////////////////////////////////////////////////////////////////////////////////////////////////
void dg_cli(FILE *fp, CUSock *s, sockaddr_in *pservaddr, socklen_t servlen)
{
int 			n;
char			sendline[MAXLINE], recvline[MAXLINE+1],dst[50];
socklen_t		len;
struct sockaddr_in 	*preply_addr;
char            	*mesg;                  // the protocol will give me a buffer with a packet in it
char			buf[100];
unsigned long		session=0;

	MyLog("dg_cli: Entered.",logfilename);
	preply_addr=(struct sockaddr_in *)malloc(servlen);
	while (fgets(sendline, MAXLINE, fp)!= NULL)
	{
		s->Send(sendline, strlen(sendline), 0);
		len=servlen;
//		n=s->Recv(&mesg, MAXLINE, 0, preply_addr, &servlen);
		n=s->Recv(&mesg, MAXLINE, 0);
		if (n>0)
		{
//			if (pservaddr->sin_addr.s_addr!=preply_addr->sin_addr.s_addr)
//			{
//				printf("Reply from %s (ignored)\n",
//					inet_ntop(AF_INET,(void*)&(pservaddr->sin_addr.s_addr),dst,50));
//				continue;
//				}
			mesg[n]=0;
			printf("Response = '%s'\n",mesg);
			delete [] mesg;
			}
		}
	} ////////////////////////////////////////////////////////////////////////////////end dg_cli

////////////////////////////////////////////////////////////////////////////////////////////////////
// Init
////////////////////////////////////////////////////////////////////////////////////////////////////
void Init(int argc, char** argv)
{
	ftime(&starttime);
			
	logcount=0;
	logfile=fopen(logfilename,"w");
	fclose(logfile);

	MyLog("Init: Entered.",logfilename);

	// find out if we are debugging
	if ((argc==2) && (strcmp(argv[1],"debug")==0))
	{
		printf("DEBUGMODE ON\n");
		debug=TRUE;
		}
	if (debug)
	{
	int i;
		for (i=0;i<argc;i++) printf("Argument %d, s='%s' \n",i,argv[i]);
		}

	// init the threading model
	pthread_attr_init(&attr);    // initialize attr with default attributes
	pthread_attr_setscope(&attr, PTHREAD_SCOPE_SYSTEM);  // system-wide contention
	pthread_mutex_init(&screenmutex, NULL);
	pthread_mutex_init(&logmutex, NULL);
	consoleconnected=FALSE;
	donelistening=FALSE;
	MyLog("Init: Exited.",logfilename);
	} //////////////////////////////////////////////////////////////////////////////////end Init

////////////////////////////////////////////////////////////////////////////////////////////////////
// main
////////////////////////////////////////////////////////////////////////////////////////////////////
int main (int argc , char *argv[])
{
//int sockfd;
CUSock sock;
struct sockaddr_in servaddr;

	Init(argc, argv);
	MyLog("Main: Entered.",logfilename);

	if (argc!=2) 
	{
		printf("usage: client <IPAddress>\n");
		exit(1);
	}
	
	bzero(&servaddr, sizeof(servaddr));
	servaddr.sin_family = AF_INET;

	// NOTE that IP and PORT are in NET form
	servaddr.sin_port = htons(BASEWORLDPORT);
	inet_pton(AF_INET, argv[1], &servaddr.sin_addr);

	printf ("The uintl = '%x' \n",servaddr.sin_addr.s_addr);
	MyLog("Main: Before SetLogFunc call.",logfilename);
	sock.SetLogFunc(MyLog,logfilename);
	MyLog("Main: Before sock.Init call.",logfilename);
	sock.Init();
	sock.Create(&servaddr,sizeof(servaddr));
	if (!sock.Connect()) 
	{
		MyLog("Main: Failed to connect.",logfilename);
		exit (1);
		}	
	
	MyLog("Main: Before dg_cli call.",logfilename);
	dg_cli(stdin, &sock,  &servaddr, sizeof(servaddr));
	exit (0);

} //////////////////////////////////////////////////////////////////////////////////////////end main
