////////////////////////////////////////////////////////////////////////////////////////////////////
// server.cpp -- This is it.
// This is the main module for the first real attempt to put together my 3d world game.
// This will spawn the listener then become the console listener thread.
//   
// World III
//
// by Eric J Krejci
// Copyright (C) 1999 Eric J Krejci 
// Revised : 31 Jan 2006
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation version 2.
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
// Contact:
//              email           worldii@viagloria.com
//              web             www.viagloria.com
////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////
// Includes
////////////////////////////////////////////////////////////////////////////////////////////////////
#include <sys/timeb.h>
#include <pthread.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>


#include "rudp.h"
#include "cusock.hpp"
#include "wiii_util.h"

////////////////////////////////////////////////////////////////////////////////////////////////////
// Globals
////////////////////////////////////////////////////////////////////////////////////////////////////
pthread_mutex_t logmutex;
FILE *          logfile;
char            logfilename[50]="logs/worldiii-server.log";
long            logcount;
pthread_mutex_t screenmutex;
bool            consoleconnected;       // tells when we are connected to console.
CUSock		*newconsock;		// We use TCP for console
unsigned long	consession;		// Console session
sockaddr_in	consoleaddr;		// Console IP address
bool		debug;			// This puts server into debug mode 
pthread_attr_t	attr;			// thread attr structure for setting attributes.
bool		donelistening;
struct timeb    starttime;

////////////////////////////////////////////////////////////////////////////////////////////////////
// Code
////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////
// ConPrint
////////////////////////////////////////////////////////////////////////////////////////////////////
//void ConPrint(CUSock *sock, char *s, bool consoleconnected, unsigned long session)
//{
////int count, code;
////bool timeout;
//struct sockaddr_in dest;
//
//	// send stuff out the socket only if connected.
//	if (consoleconnected)
//	{
//		sock->Send(s, strlen(s),0,&dest,sizeof(sockaddr),session);
//		}
//	} //////////////////////////////////////////////////////////////////////////////end ConPrint

////////////////////////////////////////////////////////////////////////////////////////////////////
// dg_echo
////////////////////////////////////////////////////////////////////////////////////////////////////
void dg_echo(CUSock *s)	
{
int 	n;
//socklen_t		len;
char			dst[50];
char			*mesg;			// the protocol will give me a buffer with a packet in it
char			*dest;
unsigned long int	addrtemp;
struct sockaddr_in	addr;
socklen_t		addrlen;
unsigned long		session;		// The value returned by Recv is in HOST form

	MyLog("dg_echo: Entered.",logfilename);
	dest = new char [INET_ADDRSTRLEN];
	for (;;)
	{
		printf("dg_echo: Before Recv \n");
		addrlen=sizeof(addr);
		n=s->Recv(&mesg,MAXLINE,0,&addr,&addrlen,&session);	// protocol will return a buffer
		if (n>0)
		{
			mesg[n-1]=0;
			printf("dg_echo: After zero term \n");
			if (inet_ntop(AF_INET, &addr, dest, addrlen)==NULL) 
				printf ("Error inet_ntop returned error.\n");
			printf("got mesg='%s' from %s, of length %d \n", mesg, inet_ntoa(addr.sin_addr), n);
			s->Send(mesg, n, 1, &addr, addrlen,session);
			}
		}
	} ///////////////////////////////////////////////////////////////////////////////end dg_echo

////////////////////////////////////////////////////////////////////////////////////////////////////
// Init
////////////////////////////////////////////////////////////////////////////////////////////////////
void Init(int argc, char** argv)
{
	ftime(&starttime);
		
	logcount=0;
	logfile=fopen(logfilename,"w");
	fclose(logfile);
	MyLog("Init: Entered (Logs initialized).",logfilename);

	// find out if we are debugging
	if ((argc==2) && (strcmp(argv[1],"debug")==0))
	{
		printf("DEBUGMODE ON\n");
		debug=TRUE;
		}

	if (debug)
	{
	int i;

		for (i=0;i<argc;i++) printf("Argument %d, s='%s' \n",i,argv[i]);
		}

	// init the threading model
	pthread_attr_init(&attr);    // initialize attr with default attributes
	pthread_attr_setscope(&attr, PTHREAD_SCOPE_SYSTEM);  // system-wide contention
	pthread_mutex_init(&screenmutex, NULL);
	pthread_mutex_init(&logmutex, NULL);
	consoleconnected=FALSE;
	donelistening=FALSE;
	MyLog("Init: Exited.",logfilename);
	} //////////////////////////////////////////////////////////////////////////////////end Init

////////////////////////////////////////////////////////////////////////////////////////////////////
// main
////////////////////////////////////////////////////////////////////////////////////////////////////
int main (int argc , char *argv[])
{
//int sockfd;
struct sockaddr_in servaddr;
//struct sockaddr_in cliaddr;
CUSock	sock;

	Init(argc, argv);	
	MyLog("Main: After Init.",logfilename);
	sock.SetLogFunc(MyLog,logfilename);
	sock.Init();
	bzero(&servaddr, sizeof(servaddr));

	// NOTE IP and PORT are in NET form !!	
	servaddr.sin_addr.s_addr = htonl(INADDR_ANY);
	servaddr.sin_port        = htons(BASEWORLDPORT);
	servaddr.sin_family      = AF_INET; 
	sock.Create(&servaddr,sizeof(servaddr));
	if (!sock.Bind())
	{
		MyLog("Main: Bind Failed. Exiting.",logfilename);	
		return (1);
		}
	sock.Listen();
	MyLog("Main: Before dg_echo call.",logfilename);
	dg_echo(&sock);				//, &cliaddr, sizeof(cliaddr));

	} //////////////////////////////////////////////////////////////////////////////////end main
