///////////////////////////////////////////////////////////////////////////////////////////////////
// GCons
// cgnomewin.cpp : this is used by the gcons graphical console application.
//
// by Eric J Krejci
// Copyright (C) 1999 Eric J Krejci
// Revised : 13 Sep 2007
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation version 2.
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
// Contact:
//              email           worldii@viagloria.com
//              web             www.viagloria.com
///////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////
// Includes
///////////////////////////////////////////////////////////////////////////////////////////////////
#include <stdio.h>
#include <errno.h>
#include <sys/timeb.h>
#include "cgnomewin.hpp"
#include <gtkmm/stock.h>
#include <iostream>
#include "worldiii.h"

///////////////////////////////////////////////////////////////////////////////////////////////////
// Defines
///////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////
// Externals
///////////////////////////////////////////////////////////////////////////////////////////////////
extern int loglevel;

////////////////////////////////////////////////////////////////////////////////////////////////////
// Constructor
////////////////////////////////////////////////////////////////////////////////////////////////////
//CGnomeWin::CGnomeWin(): m_Label1("Contents of tab 1"),  m_Label2("Contents of tab 2"),  m_Button_Quit("Quit")
CGnomeWin::CGnomeWin()

{
	set_title("WorldIII Console");
	set_default_size(400,400);

	// put menu bar at the top of the box and the other stuff below it.
	add(m_Box);

	MakeMenu();
	MakeNoteBook();

	// Make the main page level stuff
	m_Box.pack_start(m_Notebook);
	m_Box.pack_start(m_ButtonBox, Gtk::PACK_SHRINK);
	m_Button_Quit.set_label("QUIT!!");
	m_ButtonBox.pack_start(m_Button_Quit, Gtk::PACK_SHRINK);
	m_Button_Quit.signal_clicked().connect(sigc::mem_fun(*this, &CGnomeWin::OnButtonQuit) );
	
	show_all_children();

	} ///////////////////////////////////////////////////////////////////////////end constructor

////////////////////////////////////////////////////////////////////////////////////////////////////
// Name: ~CEngine()
// Desc: Font class destructor
////////////////////////////////////////////////////////////////////////////////////////////////////
CGnomeWin::~CGnomeWin()
{
	} ////////////////////////////////////////////////////////////////////////////end destructor

////////////////////////////////////////////////////////////////////////////////////////////////////
// MakeMenu
// This does all the things needed to make the Menu
////////////////////////////////////////////////////////////////////////////////////////////////////
void CGnomeWin::MakeMenu()
{
	
	// Create actions for menus and tool bars
	actiongroup = Gtk::ActionGroup::create();

	// File|New submenu
	actiongroup->add(Gtk::Action::create("FileNewStandard", Gtk::Stock::NEW, "_New", "Create a new file"),
		sigc::mem_fun(*this, &CGnomeWin::OnMenuFileNewGeneric));

	actiongroup->add(Gtk::Action::create("FileNewFoo",      Gtk::Stock::NEW, "New Foo", "Create a new foo"),
		sigc::mem_fun(*this, &CGnomeWin::OnMenuFileNewGeneric));

	actiongroup->add(Gtk::Action::create("FileNewGoo",      Gtk::Stock::NEW, "_New Goo", "Create a new goo"),
		sigc::mem_fun(*this, &CGnomeWin::OnMenuFileNewGeneric));

	// File menu
	actiongroup->add(Gtk::Action::create("FileMenu", "File"));

	// Sub menu
	actiongroup->add(Gtk::Action::create("FileNew", Gtk::Stock::NEW));
	actiongroup->add(Gtk::Action::create("FileQuit", Gtk::Stock::QUIT),
		sigc::mem_fun(*this, &CGnomeWin::OnMenuFileQuit));

	// Edit menu
	actiongroup->add(Gtk::Action::create("EditMenu", "Edit"));
	actiongroup->add(Gtk::Action::create("EditCopy", Gtk::Stock::COPY),
		sigc::mem_fun(*this, &CGnomeWin::OnMenuOthers));
	actiongroup->add(Gtk::Action::create("EditPaste", Gtk::Stock::PASTE),
		sigc::mem_fun(*this, &CGnomeWin::OnMenuOthers));
	actiongroup->add(Gtk::Action::create("EditSomething", "Something"),
		Gtk::AccelKey("<control><alt>S"),
		sigc::mem_fun(*this, &CGnomeWin::OnMenuOthers));

	// Choices menu, to demonstrate Radio items
	actiongroup->add( Gtk::Action::create("ChoicesMenu", "Choices") );
		Gtk::RadioAction::Group group_userlevel;
	choiceone = Gtk::RadioAction::create(group_userlevel, "ChoiceOne", "One");
	actiongroup->add(choiceone,
		sigc::mem_fun(*this, &CGnomeWin::OnMenuChoicesOne) );
	choicetwo = Gtk::RadioAction::create(group_userlevel, "ChoiceTwo", "Two");
		actiongroup->add(choicetwo,
		sigc::mem_fun(*this, &CGnomeWin::OnMenuChoicesTwo) );

	// Comm Menu
	actiongroup->add(Gtk::Action::create("CommMenu", "Communicate"));
	actiongroup->add(Gtk::Action::create("ListClients", "List All Clients"),
		Gtk::AccelKey("<f1>"),
		sigc::mem_fun(*this, &CGnomeWin::OnMenuListClients));

	actiongroup->add(Gtk::Action::create("SearchClient", "Search For Client"),
		Gtk::AccelKey("<f1>"),
		sigc::mem_fun(*this, &CGnomeWin::OnMenuSearchClient));

	actiongroup->add(Gtk::Action::create("SendSingleClient", "Send To Single Client"),
		Gtk::AccelKey("<f2>"),
		sigc::mem_fun(*this, &CGnomeWin::OnMenuSendSingleClient));

	actiongroup->add(Gtk::Action::create("SendGroupClients", "Send To A Group Of Clients"),
		Gtk::AccelKey("<f3>"),
		sigc::mem_fun(*this, &CGnomeWin::OnMenuSendGroupClients));

	actiongroup->add(Gtk::Action::create("SendAllClients", "Send To All Clients"),
		Gtk::AccelKey("<f4>"),
		sigc::mem_fun(*this, &CGnomeWin::OnMenuSendAllClients));

	actiongroup->add(Gtk::Action::create("SessionWithClient", "Converse With Client"),
		Gtk::AccelKey("<f4>"),
		sigc::mem_fun(*this, &CGnomeWin::OnMenuSessionWithClient));

	actiongroup->add(Gtk::Action::create("PauseConnect", "Pause Client"),
		Gtk::AccelKey("<f4>"),
		sigc::mem_fun(*this, &CGnomeWin::OnMenuSessionWithClient));

	actiongroup->add(Gtk::Action::create("ResumeConnect", "Resume Client"),
		Gtk::AccelKey("<f4>"),
		sigc::mem_fun(*this, &CGnomeWin::OnMenuSessionWithClient));

	actiongroup->add(Gtk::Action::create("DisconSingleClient", "Disconnect Client"),
		Gtk::AccelKey("<f4>"),
		sigc::mem_fun(*this, &CGnomeWin::OnMenuSessionWithClient));

	actiongroup->add(Gtk::Action::create("DisconGroupClients", "Disconnect Group Of Clients"),
		Gtk::AccelKey("<f4>"),
		sigc::mem_fun(*this, &CGnomeWin::OnMenuSessionWithClient));

	actiongroup->add(Gtk::Action::create("DisconAllClients", "Disconnect All Clients"),
		Gtk::AccelKey("<f4>"),
		sigc::mem_fun(*this, &CGnomeWin::OnMenuSessionWithClient));

	// Server Menu
	actiongroup->add(Gtk::Action::create("ServerMenu", "Server"));
	actiongroup->add(Gtk::Action::create("ConnectServer", "Connect To Server"),
		Gtk::AccelKey("<f1>"),
		sigc::mem_fun(*this, &CGnomeWin::OnMenuConnectServer));

	actiongroup->add(Gtk::Action::create("DisconnectServer", "Disconnect From Server"),
		Gtk::AccelKey("<f2>"),
		sigc::mem_fun(*this, &CGnomeWin::OnMenuDisconnectServer));

	actiongroup->add(Gtk::Action::create("StartServer", "Start Server"),
		Gtk::AccelKey("<f3>"),
		sigc::mem_fun(*this, &CGnomeWin::OnMenuStartServer));

	actiongroup->add(Gtk::Action::create("StopServer", "Stop Server"),
		Gtk::AccelKey("<f4>"),
		sigc::mem_fun(*this, &CGnomeWin::OnMenuStopServer));

	actiongroup->add(Gtk::Action::create("StartListening", "Start Listening"),
		Gtk::AccelKey("<f5>"),
		sigc::mem_fun(*this, &CGnomeWin::OnMenuStartListening));

	actiongroup->add(Gtk::Action::create("StopListening", "Stop Listening"),
		Gtk::AccelKey("<f6>"),
		sigc::mem_fun(*this, &CGnomeWin::OnMenuStopListening));

	// Test Menu
	actiongroup->add(Gtk::Action::create("TestMenu", "Test"));
	actiongroup->add(Gtk::Action::create("ShowServers", "Show All Servers"),
		Gtk::AccelKey("<control><alt>s"),
		sigc::mem_fun(*this, &CGnomeWin::OnMenuShowServers));

	actiongroup->add(Gtk::Action::create("PingServer", "Ping Server"),
		Gtk::AccelKey("<control><alt>p"),
		sigc::mem_fun(*this, &CGnomeWin::OnMenuPingServer));

	actiongroup->add(Gtk::Action::create("DebugLevel", "Debug Level"),
		Gtk::AccelKey("<control><alt>d"),
		sigc::mem_fun(*this, &CGnomeWin::OnMenuDebugLevel));

	actiongroup->add(Gtk::Action::create("DebugLevel1", "Debug Level 1"),
		Gtk::AccelKey("<control><alt>d"),
		sigc::mem_fun(*this, &CGnomeWin::OnMenuDebugLevel1));

	// Help menu
	actiongroup->add( Gtk::Action::create("HelpMenu", "Help") );
	actiongroup->add( Gtk::Action::create("HelpAbout", Gtk::Stock::HELP),
		sigc::mem_fun(*this, &CGnomeWin::OnMenuOthers) );

	uimgr = Gtk::UIManager::create();
	uimgr->insert_action_group(actiongroup);

	add_accel_group(uimgr->get_accel_group());

	// Layout the actions in a menubar and toolbar
	  Glib::ustring ui_info =
        "<ui>"
        "  <menubar name='MenuBar'>"
        "    <menu action='FileMenu'>"
        "      <menu action='FileNew'>"
        "        <menuitem action='FileNewStandard'/>"
        "        <menuitem action='FileNewFoo'/>"
        "        <menuitem action='FileNewGoo'/>"
        "      </menu>"
        "      <separator/>"
        "      <menuitem action='FileQuit'/>"
        "    </menu>"
        "    <menu action='EditMenu'>"
        "      <menuitem action='EditCopy'/>"
        "      <menuitem action='EditPaste'/>"
        "      <menuitem action='EditSomething'/>"
        "    </menu>"
        "    <menu action='ChoicesMenu'>"
        "      <menuitem action='ChoiceOne'/>"
        "      <menuitem action='ChoiceTwo'/>"
        "    </menu>"
        "    <menu action='CommMenu'>"
        "      <menuitem action='ListClients'/>"
        "      <menuitem action='SearchClient'/>"
        "      <menuitem action='SendSingleClient'/>"
        "      <menuitem action='SendGroupClients'/>"
        "      <menuitem action='SendAllClients'/>"
        "      <menuitem action='SessionWithClient'/>"
        "      <menuitem action='PauseConnect'/>"
        "      <menuitem action='ResumeConnect'/>"
        "      <menuitem action='DisconSingleClient'/>"
        "      <menuitem action='DisconGroupClients'/>"
        "      <menuitem action='DisconAllClients'/>"
        "    </menu>"
        "    <menu action='ServerMenu'>"
        "      <menuitem action='ConnectServer'/>"
        "      <menuitem action='DisconnectServer'/>"
        "      <menuitem action='StartServer'/>"
        "      <menuitem action='StopServer'/>"
        "      <menuitem action='StartListening'/>"
        "      <menuitem action='StopListening'/>"
        "    </menu>"
        "    <menu action='TestMenu'>"
        "      <menuitem action='ShowServers'/>"
        "      <menuitem action='PingServer'/>"
        "      <menu action='DebugLevel'>"
        "        <menuitem action='DebugLevel1'/>"
        "        <menuitem action='DebugLevel1'/>"
        "        <menuitem action='DebugLevel1'/>"
        "        <menuitem action='DebugLevel1'/>"
        "      </menu>"
        "    </menu>"
        "    <menu action='HelpMenu'>"
        "      <menuitem action='HelpAbout'/>"
        "    </menu>"
        "  </menubar>"
        "  <toolbar  name='ToolBar'>"
        "    <toolitem action='FileNewStandard'/>"
        "    <toolitem action='FileQuit'/>"
        "  </toolbar>"
        "</ui>";

#ifdef GLIBMM_EXCEPTIONS_ENABLED
	try
	{
		uimgr->add_ui_from_string(ui_info);
		}
	catch(const Glib::Error& ex)
	{
		std::cerr << "building menus failed: " <<  ex.what();
		}
#else
	std::auto_ptr<Glib::Error> ex;
	uimgr->add_ui_from_string(ui_info, ex);
	if(ex.get())
	{
		std::cerr << "building menus failed: " <<  ex->what();
		}
#endif //GLIBMM_EXCEPTIONS_ENABLED

	// Get the menubar and toolbar widgets, and add them to a container widget:
	Gtk::Widget* pMenubar = uimgr->get_widget("/MenuBar");
	if(pMenubar)  m_Box.pack_start(*pMenubar, Gtk::PACK_SHRINK);

	Gtk::Widget* pToolbar = uimgr->get_widget("/ToolBar") ;
	if(pToolbar)  m_Box.pack_start(*pToolbar, Gtk::PACK_SHRINK);
	} //////////////////////////////////////////////////////////////////////////////end MakeMenu

////////////////////////////////////////////////////////////////////////////////////////////////////
// MakeNoteBook
// This does all the things needed to make the Menu
////////////////////////////////////////////////////////////////////////////////////////////////////
void CGnomeWin::MakeNoteBook()
{
// m_Label1("Contents of tab 1"),  m_Label2("Contents of tab 2"),  m_Button_Quit("Quit")
	//Add the Notebook, with the button underneath:
	m_Notebook.set_border_width(10);

	// Make each not book page
	MakeNoteBookP1();
	MakeNoteBookP2();
	MakeNoteBookP3();

	// connect the signal for the notebook
	m_Notebook.signal_switch_page().connect(sigc::mem_fun(*this, &CGnomeWin::OnNotebookSwitchPage) );
	} //////////////////////////////////////////////////////////////////////////end MakeNoteBook

////////////////////////////////////////////////////////////////////////////////////////////////////
// MakeNoteBookP1
// This does all the things needed to make the Menu
////////////////////////////////////////////////////////////////////////////////////////////////////
void CGnomeWin::MakeNoteBookP1()
{
	// Draw the controls
	startbutton.set_label("Start Server!!");
	stopbutton.set_label("Stop Server!!");
	connectbutton.set_label("Connect");
	disconbutton.set_label("Disconnect");
	listenbutton.set_label("Listen");
	stoplistenbutton.set_label("Stop Listen");

	connectlabel.set_label("Disconnected");

	// add container for buttons
	m_Notebook.append_page(vbox1, "Server Control");

	vbox1.pack_start(hbox1);
	vbox1.pack_start(hbox2);
	vbox1.pack_start(hbox3);
	
	// pack buttons and add them
	startbuttonbox.pack_start(startbutton,Gtk::PACK_SHRINK);
	stopbuttonbox.pack_start(stopbutton,Gtk::PACK_SHRINK);
	connectbuttonbox.pack_start(connectbutton,Gtk::PACK_SHRINK);
	disconbuttonbox.pack_start(disconbutton,Gtk::PACK_SHRINK);
	listenbuttonbox.pack_start(listenbutton,Gtk::PACK_SHRINK);
	stoplistenbuttonbox.pack_start(stoplistenbutton,Gtk::PACK_SHRINK);


	hbox1.pack_start(servertextbox);
	hbox1.pack_start(startbuttonbox);
	hbox1.pack_start(stopbuttonbox);

	hbox2.pack_start(connectbuttonbox);
	hbox2.pack_start(disconbuttonbox);
	hbox2.pack_start(connectlabel);

	hbox3.pack_start(listenbuttonbox);
	hbox3.pack_start(stoplistenbuttonbox);



	// Connect up signals
	startbutton.signal_clicked().connect(sigc::mem_fun(*this, &CGnomeWin::OnStartButton) );
	stopbutton.signal_clicked().connect(sigc::mem_fun(*this, &CGnomeWin::OnStopButton) );

	// Set up text  box
	servertextbox.set_max_length(50);
	servertextbox.set_text("server");
	servertextbox.set_text(servertextbox.get_text() + " name");
	servertextbox.select_region(0, servertextbox.get_text_length());

	
	} ////////////////////////////////////////////////////////////////////////end MakeNoteBookP1

////////////////////////////////////////////////////////////////////////////////////////////////////
// MakeNoteBookP2
// This does all the things needed to make the Menu
////////////////////////////////////////////////////////////////////////////////////////////////////
void CGnomeWin::MakeNoteBookP2()
{
	// Draw the controls
	m_Label2.set_label("Contents of tab 1");
	m_Notebook.append_page(m_Label2, "Server Messages");
	
	} ////////////////////////////////////////////////////////////////////////end MakeNoteBookP2

////////////////////////////////////////////////////////////////////////////////////////////////////
// MakeNoteBookP3
// This does all the things needed to make the Menu
////////////////////////////////////////////////////////////////////////////////////////////////////
void CGnomeWin::MakeNoteBookP3()
{
	// Draw the controls
	m_Label3.set_label("Contents of tab 2");
	m_Notebook.append_page(m_Label3, "Client Management");
	
	} ////////////////////////////////////////////////////////////////////////end MakeNoteBookP3

////////////////////////////////////////////////////////////////////////////////////////////////////
// SetLogFunc
////////////////////////////////////////////////////////////////////////////////////////////////////
//void CGnomeWin::SetLogFunc(void (*pfunc)(char*))
//{
//	LogFunc=pfunc;
//	} ////////////////////////////////////////////////////////////////////////////end SetLogFunc

////////////////////////////////////////////////////////////////////////////////////////////////////
// SetCStringLogFunc
////////////////////////////////////////////////////////////////////////////////////////////////////
//void CGnomeWin::SetCStringLogFunc(void (*pfunc)(CString*))
//{
//	CStringLogFunc=pfunc;
//	} /////////////////////////////////////////////////////////////////////end SetCStringLogFunc

////////////////////////////////////////////////////////////////////////////////////////////////////
// ProtLog
////////////////////////////////////////////////////////////////////////////////////////////////////
//void CEngine::ProtLog(CString *s)
//{
//	if (CStringLogFunc) (*CStringLogFunc)(s);
//	} ///////////////////////////////////////////////////////////////////////////////end ProtLog

////////////////////////////////////////////////////////////////////////////////////////////////////
// ProtLog
////////////////////////////////////////////////////////////////////////////////////////////////////
//void CEngine::ProtLog(char *s)
//{
//	if (LogFunc) (*LogFunc)(s);
//	} ///////////////////////////////////////////////////////////////////////////////end ProtLog

////////////////////////////////////////////////////////////////////////////////////////////////////
// Init
// To add an instruction, add to following places
// - here in the Init()
// - add instruction procedure
// - add to Convert() procedure
// - add to AddToSymTab procedure
// - add to Check() procedure
// - add to cengine.hpp defines section
// - add to cengine.hpp private procedure section
////////////////////////////////////////////////////////////////////////////////////////////////////
//bool CGnomeWin::Init()
//{
//int	i;

//	if (loglevel>=1) ProtLog("CEngine:: Init: Entered. \n ");

//	if (loglevel>=1) ProtLog("CEngine:: Init: Exited. \n ");
 //       return( true );
//	} //////////////////////////////////////////////////////////////////////////////////end Init

////////////////////////////////////////////////////////////////////////////////////////////////////
// OnMenuFileQuit
////////////////////////////////////////////////////////////////////////////////////////////////////
void CGnomeWin::OnMenuFileQuit()
{
	// if (LogFunc) (*LogFunc)(s);
	// Close the main window to stop the Gtk::Main::run().
	hide(); 
	} ////////////////////////////////////////////////////////////////////////end OnMenuFileQuit

////////////////////////////////////////////////////////////////////////////////////////////////////
// OnMenuFileNewGeneric
////////////////////////////////////////////////////////////////////////////////////////////////////
void CGnomeWin::OnMenuFileNewGeneric()
{
	// if (LogFunc) (*LogFunc)(s);
	std::cout << "A File|New menu item was selected." << std::endl;
	} //////////////////////////////////////////////////////////////////end OnMenuFileNewGeneric

////////////////////////////////////////////////////////////////////////////////////////////////////
// OnMenuOthers
////////////////////////////////////////////////////////////////////////////////////////////////////
void CGnomeWin::OnMenuOthers()
{
	// if (LogFunc) (*LogFunc)(s);
	std::cout << "A menu item was selected." << std::endl;
	} //////////////////////////////////////////////////////////////////////////end OnMenuOthers

////////////////////////////////////////////////////////////////////////////////////////////////////
// OnMenuChoicesOne
////////////////////////////////////////////////////////////////////////////////////////////////////
void CGnomeWin::OnMenuChoicesOne()
{
	// if (LogFunc) (*LogFunc)(s);
	Glib::ustring message;
	if(choiceone->get_active()) message = "Choice 1 was selected.";
	else message = "Choice 1 was deselected";

	std::cout << message << std::endl;

	} //////////////////////////////////////////////////////////////////////end OnMenuChoicesOne

////////////////////////////////////////////////////////////////////////////////////////////////////
// OnMenuChoicesTwo
////////////////////////////////////////////////////////////////////////////////////////////////////
void CGnomeWin::OnMenuChoicesTwo()
{
	// if (LogFunc) (*LogFunc)(s);
	Glib::ustring message;
	if(choicetwo->get_active()) message = "Choice 2 was selected.";
	else message = "Choice 2 was deselected";

	std::cout << message << std::endl;

	} ///////////////////////////////////////////////////////////////////////end OnMenuChoicesTwo

////////////////////////////////////////////////////////////////////////////////////////////////////
// Menu Routines
////////////////////////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////////////////////////
// OnMenuShowServers
////////////////////////////////////////////////////////////////////////////////////////////////////
void CGnomeWin::OnMenuShowServers()
{
	// if (LogFunc) (*LogFunc)(s);
	Glib::ustring message;
	message = "ShowServers was selected";

	std::cout << message << std::endl;

	} /////////////////////////////////////////////////////////////////////end OnMenuShowServers

////////////////////////////////////////////////////////////////////////////////////////////////////
// OnMenuPingServer
////////////////////////////////////////////////////////////////////////////////////////////////////
void CGnomeWin::OnMenuPingServer()
{
	// if (LogFunc) (*LogFunc)(s);
	Glib::ustring message;
	message = "PingServer was selected";

	std::cout << message << std::endl;

	} //////////////////////////////////////////////////////////////////////end OnMenuPingServer

////////////////////////////////////////////////////////////////////////////////////////////////////
// OnMenuDebugLevel
////////////////////////////////////////////////////////////////////////////////////////////////////
void CGnomeWin::OnMenuDebugLevel()
{
	// if (LogFunc) (*LogFunc)(s);
	Glib::ustring message;
	message = "DebugLevel was selected";

	std::cout << message << std::endl;

	} //////////////////////////////////////////////////////////////////////end OnMenuDebugLevel

////////////////////////////////////////////////////////////////////////////////////////////////////
// OnMenuDebugLevel1
////////////////////////////////////////////////////////////////////////////////////////////////////
void CGnomeWin::OnMenuDebugLevel1()
{
	// if (LogFunc) (*LogFunc)(s);
	Glib::ustring message;
	message = "DebugLevel One was selected";

	std::cout << message << std::endl;

	} /////////////////////////////////////////////////////////////////////end OnMenuDebugLevel1

////////////////////////////////////////////////////////////////////////////////////////////////////
// OnMenuConnectServer
////////////////////////////////////////////////////////////////////////////////////////////////////
void CGnomeWin::OnMenuConnectServer()
{
	// if (LogFunc) (*LogFunc)(s);
	Glib::ustring message;
	message = "Connect To Server was selected";

	std::cout << message << std::endl;

	} //////////////////////////////////////////////////////////////////////end OnMenuConnectServer

////////////////////////////////////////////////////////////////////////////////////////////////////
// OnMenuDisconnectServer
////////////////////////////////////////////////////////////////////////////////////////////////////
void CGnomeWin::OnMenuDisconnectServer()
{
	// if (LogFunc) (*LogFunc)(s);
	Glib::ustring message;
	message = "Disconnect from Server was selected";

	std::cout << message << std::endl;

	} //////////////////////////////////////////////////////////////////////end OnMenuDisconnectServer

////////////////////////////////////////////////////////////////////////////////////////////////////
// OnMenuStartServer
////////////////////////////////////////////////////////////////////////////////////////////////////
void CGnomeWin::OnMenuStartServer()
{
	// if (LogFunc) (*LogFunc)(s);
	Glib::ustring message;
	message = "Start Server was selected";

	std::cout << message << std::endl;

	} //////////////////////////////////////////////////////////////////////end OnMenuStartServer

////////////////////////////////////////////////////////////////////////////////////////////////////
// OnMenuStopServer
////////////////////////////////////////////////////////////////////////////////////////////////////
void CGnomeWin::OnMenuStopServer()
{
	// if (LogFunc) (*LogFunc)(s);
	Glib::ustring message;
	message = "Stop Server was selected";

	std::cout << message << std::endl;

	} //////////////////////////////////////////////////////////////////////end OnMenuStopServer

////////////////////////////////////////////////////////////////////////////////////////////////////
// OnMenuStartListening
////////////////////////////////////////////////////////////////////////////////////////////////////
void CGnomeWin::OnMenuStartListening()
{
	// if (LogFunc) (*LogFunc)(s);
	Glib::ustring message;
	message = "Start Listening was selected";

	std::cout << message << std::endl;

	} //////////////////////////////////////////////////////////////////end OnMenuStartListening

////////////////////////////////////////////////////////////////////////////////////////////////////
// OnMenuStopListening
////////////////////////////////////////////////////////////////////////////////////////////////////
void CGnomeWin::OnMenuStopListening()
{
	// if (LogFunc) (*LogFunc)(s);
	Glib::ustring message;
	message = "Stop Listening was selected";

	std::cout << message << std::endl;

	} ///////////////////////////////////////////////////////////////////end OnMenuStopListening

////////////////////////////////////////////////////////////////////////////////////////////////////
// OnMenuListClients
////////////////////////////////////////////////////////////////////////////////////////////////////
void CGnomeWin::OnMenuListClients()
{
	// if (LogFunc) (*LogFunc)(s);
	Glib::ustring message;
	message = "List Clients was selected";

	std::cout << message << std::endl;

	} /////////////////////////////////////////////////////////////////////end OnMenuListClients

////////////////////////////////////////////////////////////////////////////////////////////////////
// OnMenuSendSingleClient
////////////////////////////////////////////////////////////////////////////////////////////////////
void CGnomeWin::OnMenuSendSingleClient()
{
	// if (LogFunc) (*LogFunc)(s);
	Glib::ustring message;
	message = "Send to single client";

	std::cout << message << std::endl;

	} ////////////////////////////////////////////////////////////////end OnMenuSendSingleClient

////////////////////////////////////////////////////////////////////////////////////////////////////
// OnMenuSendGroupClients
////////////////////////////////////////////////////////////////////////////////////////////////////
void CGnomeWin::OnMenuSendGroupClients()
{
	// if (LogFunc) (*LogFunc)(s);
	Glib::ustring message;
	message = "Send to group of clients";

	std::cout << message << std::endl;

	} ////////////////////////////////////////////////////////////////end OnMenuSendGroupClients

////////////////////////////////////////////////////////////////////////////////////////////////////
// OnMenuSendAllClients
////////////////////////////////////////////////////////////////////////////////////////////////////
void CGnomeWin::OnMenuSendAllClients()
{
	// if (LogFunc) (*LogFunc)(s);
	Glib::ustring message;
	message = "Send to all of clients";

	std::cout << message << std::endl;

	} //////////////////////////////////////////////////////////////////end OnMenuSendAllClients

////////////////////////////////////////////////////////////////////////////////////////////////////
// OnMenuSessionWithClient
////////////////////////////////////////////////////////////////////////////////////////////////////
void CGnomeWin::OnMenuSessionWithClient()
{
	// if (LogFunc) (*LogFunc)(s);
	Glib::ustring message;
	message = "Converse with client";

	std::cout << message << std::endl;

	} ///////////////////////////////////////////////////////////////end OnMenuSessionWithClient

////////////////////////////////////////////////////////////////////////////////////////////////////
// OnMenuSearchClient
////////////////////////////////////////////////////////////////////////////////////////////////////
void CGnomeWin::OnMenuSearchClient()
{
	// if (LogFunc) (*LogFunc)(s);
	Glib::ustring message;
	message = "Search For client";

	std::cout << message << std::endl;

	} ////////////////////////////////////////////////////////////////////end OnMenuSearchClient

////////////////////////////////////////////////////////////////////////////////////////////////////
// OnStartButton
////////////////////////////////////////////////////////////////////////////////////////////////////
void CGnomeWin::OnStartButton()
{
Glib::ustring message;

	message = "Start Server called with '";
	std::cout << message << servertextbox.get_text() << "'" << std::endl;
	} /////////////////////////////////////////////////////////////////////////end OnStartButton

////////////////////////////////////////////////////////////////////////////////////////////////////
// OnStopButton
////////////////////////////////////////////////////////////////////////////////////////////////////
void CGnomeWin::OnStopButton()
{
Glib::ustring message;

	message = "Stop Server called with '";
	std::cout << message << servertextbox.get_text() << "'" << std::endl;
//	std::cout << message << std::endl;
	} //////////////////////////////////////////////////////////////////////////end OnStopButton

////////////////////////////////////////////////////////////////////////////////////////////////////
// OnButtonQuit
////////////////////////////////////////////////////////////////////////////////////////////////////
void CGnomeWin::OnButtonQuit()
{
	hide();
	} //////////////////////////////////////////////////////////////////////////end OnButtonQuit

////////////////////////////////////////////////////////////////////////////////////////////////////
// OnNotebookSwitchPage
////////////////////////////////////////////////////////////////////////////////////////////////////
void CGnomeWin::OnNotebookSwitchPage( GtkNotebookPage* page, guint page_num)
{
	std::cout << "Switched to tab with index " << page_num << std::endl;

	//You can also use m_Notebook.get_current_page() to get this index.

	} //////////////////////////////////////////////////////////////////end OnNotebookSwitchPage





