////////////////////////////////////////////////////////////////////////////////////////////////////
// cengine.hpp 
// This is the script engine module for the first real attempt to put together my 3d world game.
//
// World III
//
// by Eric J Krejci
// Copyright (C) 1999 Eric J Krejci
// Revised : 11 May 2007
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation version 2.
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
// Contact:
//              email           worldii@viagloria.com
//              web             www.viagloria.com
////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////
// Defines
////////////////////////////////////////////////////////////////////////////////////////////////////

#define		MAXSYMTAB		20
#define		MAXSCRIPT		1000
#define		MAXDIRECTIVE		5
#define		MAXEXTOPCODE		200	// represents opcodes 1000-1199
//#define		OUTFILENAMELEN		255


///////////////
// Howto to display error codes
///////////////
#define		CE_PRINT		0
#define		CE_LOG			1
#define		CE_BOTH			2

///////////////
// Error codes
///////////////
#define		CE_ERR_NOERR		0	// No error
#define		CE_ERR_SPUF		1	// Error stack pointer underflow
#define		CE_ERR_SPOF		2	// Error stack pointer overflow
#define		CE_ERR_APND		3	// Error during append
#define		CE_ERR_TYPE		4	// Error wrong types
#define		CE_ERR_BRK		5	// Error break encountered
#define		CE_ERR_OPPRC		6	// Error No procedure for that opcode 
#define		CE_ERR_OPCINVAL		7	// Error attempted to register a new function with invalid opcode
#define		CE_ERR_LOAD 		8	// Error during load of file to be assmbled
#define		CE_ERR_NOFILE	 	9	// Error no file specified
#define		CE_ERR_UNHANDLED 	10	// Error bad operation specified
#define		CE_ERR_BADFILE 		11	// Error bad file specified
#define		CE_ERR_NOLABEL 		12	// Error expected label
#define		CE_ERR_NOINST 		13	// Error expected inst
#define		CE_ERR_INVTYPE 		14	// Error invalid type specified
#define		CE_ERR_REDEF 		15	// Error attempted to redfine symbol 
#define		CE_ERR_BADREG 		16	// Error specified bad register 
#define		CE_ERR_LENGTH 		17	// Error specified bad immediate parm length 
#define		CE_ERR_SYMTABADD	18	// Error while adding to symtab 
#define		CE_ERR_CONV		19	// Error while converting  
#define		CE_ERR_NOSYM		20	// Error symbol not found while converting  
#define		CE_ERR_PARM		21	// Error too many or not enough parms
#define		CE_ERR_BADOPC		22	// Error bad opcode
#define		CE_ERR_INTERR		23	// Error internal error. Please file bugreport

///////////////////////
// Assembler commands
///////////////////////
#define		CE_ASM_OPEN		0	// Assembler should open a file 
#define		CE_ASM_FIRST		1	// Assembler should conduct first pass, make symtab 
#define		CE_ASM_SECOND		2	// Assembler should perform second pass, assemble 
#define		CE_ASM_WRITE		3	// Assembler should write output bin 
#define		CE_ASM_LIST		4	// Assembler should write list to logfile 
#define		CE_ASM_OUTPUT		5	// Assembler outputfile  
#define		CE_ASM_LSTFILE		6	// Assembler lstfile  

////////////////////////
// State machine states
////////////////////////
#define		ST_RESET		0	// Entry into this state implies complete engine reset and first pass reset
#define		ST_BEGIN		1	// Get the next line
#define		ST_END			2	// End the first pass
#define		ST_COMMENT		3	// Found a comment
#define		ST_ERR4			4	// Found an error. Found something that was not op label or dir
#define		ST_LABEL		5	// Found a comment
#define		ST_ERR6			6	// Found an error. Found something that was not op or dir
#define		ST_OPCODE		7	// Found an opcode
#define		ST_CHECK		10	// Check to ensure command is correct
#define		ST_SYMTAB		11	// Check was good so add to symtab
#define		ST_SYMTABERR		12	// There was an error adding to symtab
//#define	ST_EOL			13	// Found an EOL
#define		ST_DIRECTIVE		14	// Found a assembler directive
#define		ST_DIR_ONE_PARM		15	// Get parm for directive
#define		ST_CHECK_ERR		16	// If there was an error checking the line
#define		ST_ONE_PARM		19	// Get parm for opcode 
#define		ST_TWO_PARM		20	// Get parm for opcode
#define		ST_THREE_PARM		21	// Get parm for opcode
#define		ST_FOUR_PARM		22	// Get parm for opcode
#define		ST_FIVE_PARM		23	// Get parm for opcode
#define		ST_SIX_PARM		24	// Get parm for opcode
#define		ST_ERR25		25	// Found an error. Found something that was not an opcode
#define		ST_ERR26		26	// Found an error. Found something that was not an opcode
#define		ST_EXTOP		27	// Found an externally defined opecode. 
#define		ST_ASSEM		28	// During second pass only. 

///////////////////
// Opcodes
///////////////////
#define		CNT		0
#define		ADD		1
#define		SUB		2
#define		MUL		3
#define		DIVD		4
#define		DIVI		5
#define		MOD		6
#define		EXP		7
#define		NEG		8
#define		PUSH		9
#define		POP		10
#define		JMP		11
#define		JEQ		12
#define		JLT		13
#define		JGT		14
#define		JV		15
#define		JNZ		16
#define		CALL		17
#define		CEQ		18
#define		CLT		19
#define		CGT		20
#define		CV		21
#define		CNZ		22
#define		RET		23
#define		REQ		24
#define		RLT		25
#define		RGT		26
#define		RV		27
#define		RNZ		28
#define		CMP		29
#define		CPI		30
#define		LD		31
#define		LDI		32
#define		AND		33
#define		OR		34
#define		XOR		35
#define		NOT		36
#define		CNV		37
#define		INC		38
#define		DEC		39
#define		INCM		40
#define		DECM		41
#define		STM		42
#define		HLT		43
#define		LDSP		44
#define		LDTI		45
#define		LDM		46
#define		LDR		47
#define		STR		48
#define		MAXOPCODE	49

///////////////////
// Assem Directives
///////////////////
#define		EQU			"equ"
#define		ORG			"org"
#define		DEFB			"db"
#define		DEFS			"ds"
#define		DEFD			"dd"

////////////////////////////////////////////////////////////////////////////////////////////////////
// Includes
////////////////////////////////////////////////////////////////////////////////////////////////////
#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <math.h>
#include <string.h>

#ifndef _CENGINE_HPP_
#define _CENGINE_HPP_
#include "cstring.hpp"
#include "wiii_structures.h"

class CEngine;

typedef void (CEngine::*ptr2memberfunc)();
typedef void (*ptrtoextfunc)(parmstruct *);
typedef struct EXTOPCODE {
	ptrtoextfunc	ptr;		// storage the enternal function ptr
	int		bin;		// count of registers to pass to external function
	int		iin;
	int		uin;
	int		din;
	int		sin;
	int		bout;		// count of registers to receive from external function
	int		iout;
	int		uout;
	int		dout;
	int		sout;
	parmstruct	p;		// actual parameters to pass
	void		*extra;		// what is this ?
	} extopcode;
typedef struct SYMTABTYPE
{
	bool		used;
	CString		symbol;
	unsigned long	value;
	} symtabtype;
typedef struct MNEMTYPE
{
	CString		symbol;
	int		parmcount;
	} mnemtype;

////////////////////////////////////////////////////////////////////////////////////////////////////
// CEngine
////////////////////////////////////////////////////////////////////////////////////////////////////
class CEngine
{
public:
				// Constructor / destructor
				CEngine();
				~CEngine();
	bool			Init();		// To have aa nonfail constructor.
	void			SetLogFunc(void (*pfunc)(char*));
	void			SetCStringLogFunc(void (*pfunc)(CString*));

	unsigned long int	GetValue(unsigned long int);
	void 			SetValue(unsigned long int,unsigned long int);
	void 			SetPC(unsigned long int);
	int 			Run();
	void 			Reset();
	int 			SingleStep();
	void 			GetBools(bool b[]);
	void 			GetInts(long int is[]);
	void 			GetUInts(unsigned long int is[]);
	void 			GetDbls(double dbls[]);
	void 			GetStrings(CString s[]);
	void 			GetState(
					unsigned long int *, 
					unsigned long int *, 
					bool*, 
					bool*, 
					bool*, 
					bool*
					);
								
	void			SetInputFile(CString *);	// 
	void			SetOutputFile(CString *);
	int			Assemble(int, CString *,bool);			// int says what to do
	
	int			Register(ptrtoextfunc ptr,			// install an external function
					int bin,  int iin,  int uin,  int din,  int sin,	// to extend the abilities of the
					int bout, int iout, int uout, int dout, int sout,	// script engine and to pass data
					unsigned long opc,			// in and out. 
					void *);				// The void is for what ever other info is needed
	void			Error(int e,int which);				// This writes the error message for the number
										// given, to the log
private:
	// Functions for assmbly
	int 			LoadLine(FILE *f, CString*);
	int 			Pass(int pass, bool cmdline);					// 
	int 			WriteBin();					
	int 			ListBin();					// Lists to Log File
	void			ResetSymtab();
	void			ProtLog(char *s);
	void			ProtLog(CString *s);
	void			LoadFuncArray();
	void			DumpSymtab();
	bool			IsOpCode(CString *, int*);
	bool			IsExtOpCode(CString *);
	bool			IsDir(CString *);
	bool			IsLabel(CString *);
	int			GetParmCountOpCode(CString *);
	int			GetParmCountDir(CString *);
	bool 			Check_Reg(CString *r);
	int			Check_LDR(						// return 0 = OK
					CString *l, 				// not zero = error code
					CString *c, 
					CString *p1, CString *p2, 
					CString *p3, CString *p4 
					);
	int			Check_STR(						// return 0 = OK
					CString *l, 				// not zero = error code
					CString *c, 
					CString *p1, 
					CString *p2, 
					CString *p3  
					);
	int			Check_LDM(						// return 0 = OK
					CString *l, 				// not zero = error code
					CString *c, 
					CString *p1, CString *p2, 
					CString *p3, CString *p4 
					);
	int			Check_LDI(						// return 0 = OK
					CString *l, 				// not zero = error code
					CString *c, 
					CString *p1, CString *p2, 
					CString *p3, CString *p4 
					);
	int			Check_AND(						// return 0 = OK
					CString *l, 				// not zero = error code
					CString *c, 
					CString *p1, CString *p2, 
					CString *p3, CString *p4, 
					CString *p5, CString *p6 
					);
	int			Check_ADDSUB(						// return 0 = OK
					CString *l, 				// not zero = error code
					CString *c, 
					CString *p1, CString *p2, 
					CString *p3, CString *p4, 
					CString *p5, CString *p6 
					);
	int			Check_CMP(						// return 0 = OK
					CString *l, 				// not zero = error code
					CString *c, 
					CString *p1, CString *p2, 
					CString *p3, CString *p4 
					);
	int			Check(						// return 0 = OK
					CString *l, 				// not zero = error code
					CString *c, 
					CString *p1, CString *p2, 
					CString *p3, CString *p4, 
					CString *p5, CString *p6,
					int);
	int			AddToSymTab(					// return 0 = OK
					CString *l, 				// not zero = error code
					CString *c, 
					CString *p1, CString *p2, 
					CString *p3, CString *p4, 
					CString *p5, CString *p6,
					int);
	int			Convert(					// return 0 = OK
					CString *l, 				// not zero = error code
					CString *c, 
					CString *p1, CString *p2, 
					CString *p3, CString *p4, 
					CString *p5, CString *p6,
					int,
					FILE*
					);
	bool 			Lookup(CString *l, unsigned long *u);


	// Overloaded functions to get parms from memory array
	void			GetParms(
					char *vt1,
					int  *idx1
					);
	void			GetParms(
					char *vt1,
					int  *idx1,
					char *vt2,
					int  *idx2
					);
	void			GetParms(
					unsigned long int  *addr,
					char *vt1,
					int  *idx1
					);
	void			GetParms(
					char *vt1,
					int  *idx1,
					int  *idx2
					);
	void			GetParms(
					char *vt1,
					int  *idx1,
					int  *idx2,
					int  *idx3
					);
	void			GetParms(
					char *vt1,
					int  *idx1,
					unsigned long  *len,
					unsigned long  *addr
					);
	void			GetParms(
					char *vt1,
					int  *idx1,
					char *vt2,
					int  *idx3,
					char *rt,
					int  *ri
					);
	int			CallExt();
	bool 			InsertOpCode(CString *,CString *,int, FILE *);
	bool 			InsertType(CString *,FILE *);
	bool 			InsertInt(CString *,FILE *);
	bool 			InsertAddr(CString *,FILE *);
	bool 			InsertString(CString *,FILE *);
	bool 			InsertBool(CString *,FILE *);
	
	// functions to implement codes
	void			OpCode_cnt();
	void			OpCode_add();
	void			OpCode_sub();
	void			OpCode_mul();
	void			OpCode_divd();
	void			OpCode_divi();
	void			OpCode_mod();
	void			OpCode_exp();
	void			OpCode_neg();
	void			OpCode_push();
	void			OpCode_pop();
	void			OpCode_jmp();
	void			OpCode_jeq();
	void			OpCode_jlt();
	void			OpCode_jgt();
	void			OpCode_jv();
	void			OpCode_jnz();
	void			OpCode_call();
	void			OpCode_ceq();
	void			OpCode_clt();
	void			OpCode_cgt();
	void			OpCode_cv();
	void			OpCode_cnz();
	void			OpCode_ret();
	void			OpCode_req();
	void			OpCode_rlt();
	void			OpCode_rgt();
	void			OpCode_rv();
	void			OpCode_rnz();
	void			OpCode_cmp();
	void			OpCode_cpi();
	void			OpCode_ld();
	void			OpCode_ldr();
	void			OpCode_ldsp();
	void			OpCode_ldti();
	void			OpCode_ldi();
	void			OpCode_ldm();
	void			OpCode_and();
	void			OpCode_or();
	void			OpCode_xor();
	void			OpCode_not();
	void			OpCode_cnv();
	void			OpCode_inc();
	void			OpCode_dec();
	void			OpCode_incm();
	void			OpCode_decm();
	void			OpCode_stm();
	void			OpCode_str();
//	void			OpCode_hlt();		// implemented by Run procedure

	// registers
	bool			bools[10];		// Registers
	unsigned long int	uints[10];		// Registers 
	long int		ints[10];		// Registers 
	double			dbls[10];		// Registers 
	CString			strs[10];		// Registers 
	bool			cy;			// carry flag
	bool			eq;			// equal flag
	bool			gt;			// greater than flag
	bool			lt;			// less than flag
	bool			ss;			// single step flag
	int			engerr;			// error code 
 	unsigned long int	pc;
 	unsigned long int	sp;

	// setting break points
	bool			breakon;		// if breakon==TRUE then stop before executing breakaddr
	unsigned long int	breakaddr;		// the place to stop before 
	
	// memory array
	unsigned long int	scripts[MAXSCRIPT];		// the array that holds all the scripts

	// logging functions
	void			(*LogFunc)(char *s);
	void			(*CStringLogFunc)(CString *s);

	// functions 
	mnemtype		Mnem[MAXOPCODE];
	mnemtype		Direct[MAXDIRECTIVE];	// assembler directives
	ptr2memberfunc		OpCodes[MAXOPCODE];
	extopcode		ExtOpCodes[MAXEXTOPCODE];	// these opcodes are 1000 and up, but they are stored
							// in indecies i-1000. ie 1000 is stored in 0, 
							// 1001 is in 1, 1002 in 2


	long			linenum;		// this keeps track of the current line of source
	unsigned long		stmtcntr;		// this keeps track of the current address during assebmly
	unsigned long		org;			// this keeps track of the org
	CString			inputfile;		// these are the files for input and out put for assebmly
	CString			outputfile;		// where the binary gets written
	CString			lstfile;		// where the optional listing gets written
	symtabtype		*symtab;
	int			symcount;		// how many symbols are there in table?
							// this points to the next slot
	bool			makelistfile;		// if set the assembler writes to listfile as it goes.
	};


////////////////////////////////////////////////////////////////////////////////////////////////////
// Setup function pointer type
////////////////////////////////////////////////////////////////////////////////////////////////////


#endif //_CENGINE_HPP_


