//////////////////////////////////////////////////////////////////////////////////////////////
// citem.hpp
// 
// Item and friend classes
// these classes implement a doubly linked list
// and also support a multi child tree list
// the value is 
//
// by Eric J Krejci
// Copyright (C) 1999 Eric J Krejci 
// Revised : 26 Jan 2006
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation version 2.
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
// Contact:
//              email           worldii@viagloria.com
//              web             www.viagloria.com
//////////////////////////////////////////////////////////////////////////////////////////////
#if !defined(_CITEM_HPP_)  //skip this stuff if it has already been seen
#define _CITEM_HPP_        //by the compiler

//////////////////////////////////////////////////////////////////////////////////////////////
// Includes
//////////////////////////////////////////////////////////////////////////////////////////////
#include "worldiii.h"
//#include "wiii_types.h"
#include <pthread.h>
#include "cusock.hpp"


//////////////////////////////////////////////////////////////////////////////////////////////
//Node class
//////////////////////////////////////////////////////////////////////////////////////////////
class CItem;  //forward declare Item 
class CChar;  //forward declare Item 
typedef int WIIHANDLE;
class CNode
{
friend class CIterator;
public:
	CNode();
	~CNode();
private:
	CItem		*child;       //points the actual child 
	CNode		*nextnode;    //points to the next node in the child list
     };

////////////////////////////////////////////////////////////////////////////////////////////////
// Item class
// This class simply has the pointersand the data in derived classes
////////////////////////////////////////////////////////////////////////////////////////////////
class CItem 
{
friend class CIterator;       //allows the iterator class to get inside 

public:
     			CItem();
	virtual		~CItem();
//	int           	GetVal() {return (value);}
//	void          	Setval(int i) {value=i;}

virtual	int		GetServerId();
virtual	OBJIDTYPE	GetObjectId();
virtual	CUSock		*GetSockPtr();
virtual	bool            HarvestPortmon();	// When all three return true, the server is harvested
virtual	bool            HarvestServer();	// When all three return true, the server is harvested
virtual OBJIDTYPE	GetCharacterID();	// Returns the Character Object ID
virtual CChar		*GetCharacterPtr();	// Returns a ptr to the Char obj

private:
	pthread_mutex_t	mutex;             //the object that will help prevent simultaneous access to instance
//     	static int 	itemcount;         //the count of all items
//	static int	itemid;            //the id of the next item to be created 
//	int		id;                //the id of this item
//  	int		value;             //FOR DEBUGGING
	int		errorstate;
	CItem		*nextitem;         //pointer to next
	CItem		*previtem;         //pointer to previous
	CNode		*childlist;        //points to the child list
	CItem		*parent;           //points to parent 
	};


//////////////////////////////////////////////////////////////////////////////////////////////
// typedefs
//////////////////////////////////////////////////////////////////////////////////////////////
typedef struct cursortype {
     CItem		*curr;		// The item this cursor points to
     bool		inuse;		// Is this cursor allocated ? false = can be reallocated
     unsigned long	serverid;	// The server id, we will check that a server created this
					// cursor so we dont accidentally have one thread use
					// anothers cursor. Use Thread ID. 
     } curstype;

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Iterator class
// This class maintains the list.
/////////////////////////////////////////////////////////////////////////////////////////////////////////////
class CIterator
{
public:
			CIterator();
			~CIterator();

	bool 		BegList () {return (curr==root);}
	bool 		BegList (WIIHANDLE curs) {return (curr==root);}

	bool 		EndList () {return (curr==0);}
	bool 		EndList (WIIHANDLE curs);

	CItem 		*MoveFirst() {return (curr=root);}
	CItem 		*MoveFirst(WIIHANDLE curs);

	CItem 		*MoveLast() {return (curr=last);}
	CItem 		*MoveLast(WIIHANDLE curs) {return (curr=last);}

	CItem 		*MoveNext();
	CItem 		*MoveNext(WIIHANDLE curs);			// This is an overloaded function

	CItem 		*MovePrev();
	CItem 		*MovePrev(WIIHANDLE curs);

	CItem 		*DeleteItem(CItem *delitem);
	CItem		*DeleteItem(WIIHANDLE curs);

	CItem 		*GetCurrent();				// returns pointer to the current item
	CItem 		*GetCurrent(WIIHANDLE curs);		// returns pointer to the current item

	int 		Lock();
	int 		UnLock();
	CItem 		*SetCurrent(CItem *r);			// sets next to the same pointer after 
								// verifying that it is in the list
	CItem 		*FindItem(CItem *r);			// returns a pointer if the item is 
								// in the list, next unaffected
	CItem 		*InsertItem(CItem *additem);
	CItem 		*AppendItem(CItem *additem);
	void  		DisplayList();
	void  		BackDisplayList();

	int  		EndChildList () {return (cptr==0);}
	CNode 		*MoveFirstChild() {return (cptr=curr->childlist);} 	// moves to the first 
										// child of the curr item
	CNode 		*MoveNextChild();  			// similar to above
	CItem 		*GetCurrentChild(); 			// returns pointer to the current item
	CItem 		*AdoptChild(CItem *parent,CItem *child);
	void  		OrphanChild(CItem *child);
	WIIHANDLE	GetCursor(unsigned long serverid);	// allocate a cursor 
	bool		ReleaseCursor(WIIHANDLE curs,
				unsigned long serverid);	// Release a cursor

private:
	pthread_mutex_t	mutex;
	CItem  		*root;
	CItem  		*curr;
	CNode  		*cptr; 					// this points to the current Node 
								// which points to a child
	int		curscount;				// This is how many cursors we have made room for
								// 10 => [0]..[9]
	curstype  	cursarray[MAXCURSORS];			// This is an array of cursors, this allows many current
								// pointers into list. Since this is for multithreading
								// the routines that access these lists via these cursors
								// will need to use the Lock and Unlock functions.
								// Should we check to see if a thread has the right to use
								// a cursor ?

	CItem  		*last;
	int    		errorstate;
	};


#endif //_CITEM_HPP_
