////////////////////////////////////////////////////////////////////////////////////////////////////
// cqueue.hpp -- This is it.
// This is a first in first out queue. It will be implemented as a singly linked list with 
// void pointers to the data being listed. The caling functions will be required to know what type
// data in kept.
//
// World III
//
// by Eric J Krejci
// Copyright (C) 1999 Eric J Krejci
// Revised : 26 Jan 2006
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation version 2.
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
// Contact:
//              email           worldii@viagloria.com
//              web             www.viagloria.com
////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////
// Includes
////////////////////////////////////////////////////////////////////////////////////////////////////
#include <stdio.h>
#include <stdlib.h>

#ifndef _CQUEUE_HPP_
#define _CQUEUE_HPP_



////////////////////////////////////////////////////////////////////////////////////////////////////
// Node struct
////////////////////////////////////////////////////////////////////////////////////////////////////
typedef struct nodetype
{
	void		*data;
	nodetype	*next;
	} node;

////////////////////////////////////////////////////////////////////////////////////////////////////
// CQueue
//
// Assumptions and semantics
// for empty list
//	- count==0
//	- BegList==EndList==false
//	- 
//
// BegList=>curr==head
// EndList=>curr==0 (with non empty list). Curr is either valid and points to an item , or 0 for endlist
// Append occurs after tail, empty list inserts single
// Prepend occurs before head, empty list inserts single
// Insert occurs before curr, empty list inserts single, if curr==0 (end list) then Insert does append
//
// Since this class does not have absolute control over what the data-> points to
// it wont care about it at all. The management of that memory belongs to external code.
// 
// curr is NOT always valid !
////////////////////////////////////////////////////////////////////////////////////////////////////
class CQueue
{
public:
	// Constructor / destructor
	CQueue();
	~CQueue();
	bool		Append(void*);		// Append item at end (tail) of list
	bool		Prepend(void*);		// Insert item before first item in list
	bool		Insert(void*);		// Insert item before curr position in list
	bool		DelCurr();		// Delete current node, remember This CANT DELETE ITEM POINTED AT !!!
	void		SetMaxCount(		// Set the Max Q size item count
				unsigned long
				);		// Get void pointer to item at current location
	unsigned long	GetCount();		// Get void pointer to item at current location
	void		*GetItem();		// Get void pointer to item at current location
	void		*SetItem(void*);	// Set pointer to item given at current node location
	void		*MoveFirst();		// Point to same item as head pointer
	void		*MoveLast();		// Point to same item as tail pointer
	void		*MoveNext();		// Point to next item in list
	bool		IsEmpty();		// Is queue empty ?
	bool		IsFull();		// Is queue full ?
	bool		BegList();		// Are we at Begin of list ?
	bool		EndList();		// Are we at End of list ?
	void		Print();		// Print the data pointer and the next pointer
	
private:
	bool		AddFirst(void*);	// This adds the first item when the list is empty
	bool		FixTP();		// This adds the first item when the list is empty
	unsigned long	count;
	unsigned long	max;
	nodetype	*head;			// head of the list
	nodetype	*tail;			// tail of the list
	nodetype	*curr;			// curr location in list
	nodetype	*tp;			// tp->next=curr (always, except if BegList)

	};




#endif //_CQUEUE_HPP_


