/////////////////////////////////////////////////////////////////////////////////////
// cstring.hpp 
// Should have done this a LONG time ago 
// 
// by Eric J Krejci
// Copyright (C) 1999 Eric J Krejci 
// Revised : 13 Jan 2007
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation version 2.
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
// Contact:
//              email           worldii@viagloria.com
//              web             www.viagloria.com
/////////////////////////////////////////////////////////////////////////////////////

#if !defined(_CSTRING_HPP_)  //skip this stuff if it has already been seen
#define _CSTRING_HPP_        //by the compiler

/////////////////////////////////////////////////////////////////////////////////////
// Includes
/////////////////////////////////////////////////////////////////////////////////////
#include <stdio.h>

/////////////////////////////////////////////////////////////////////////////////////
// Defines
// These are class specific defines
/////////////////////////////////////////////////////////////////////////////////////
#define CHUNKLENGTH	10
#define MAX_DELIM_LEN	10
#define INT_LEN		10
#define ULINT_LEN	20	// this is not meant to be the actual size,
				// we are just setting aside some buf space.

typedef void(*LOGFUNCPTRTYPE)(char*);			// create a type 

/////////////////////////////////////////////////////////////////////////////////////
// CString class
/////////////////////////////////////////////////////////////////////////////////////
class CString
{

public:
			CString();				// Default constructor
			CString(const char *s);			// Init from a literal string
			CString(char c, int n);			// Pad a string with char c
			CString(const CString &other);		// Copy constructor
			~CString();
	void		SetLogFunc(void (*pfunc)(char*));	// Set the logging function to use
	void		SetChar(int index, char newchar); 	// Set the char at index to newchar
	void		Reset();						
	void		SetString(const char *s); 		// Set the string to value given 
	void		SetString(const char *s,int cnt); 	// Set the string to value given 
	char		GetChar(int index);			// Return the char at index
	unsigned long	GetCount();				// Return the count of instanciated objects
	char		*GetString(char**);			// Needs to create a string and pass it out
	char		*GetStringPtr();			// This is a conscious decision here folks.
								// The reason is that of performance
								// Dont mess with the internal string guys!
	int		GetLength() const {return length;}
	int		Length() const {return length;}
	void		Display() const {printf("%s",buf);}	// This is a diagnostic
	void		Append(const char *addition);		// Append string
	bool		Append(const char addition);		// Append char
	int		Append(int addition, char *fmt, int n);	// Append string conversion of the int
	int		Append(int addition); 			// Append string conversion of the int
	int		Append(unsigned long addition); 	// Append string conversion of ul int
	int		Append( long int addition, 		// Append string conversion of the long int
				char *fmt, 
				int n);	
	int		Append( unsigned long int addition, 	// Append string conversion of the ulongint
				char *fmt, 
				int n);
	int		Append( float addition, 		// Append string conversion of the float
				char *fmt, 
				int n);
	int		Append( double addition, 		// Append string conversion of the double
				char *fmt, 
				int n);
	CString		&operator=(const CString &other);	// Assignment
	bool		operator==(const CString &other);	// Boolean comparison, true or false, not equal, greater or less
	bool		operator==(const char *other);		// Boolean comparison, true or false, not equal, greater or less
	bool		operator>(const CString &other);	// Boolean comparison, true or false, not equal, greater or less
	bool		operator>(const char *other);		// Boolean comparison, true or false, not equal, greater or less
	bool		operator>=(const CString &other);	// Boolean comparison, true or false, not equal, greater or less
	bool		operator>=(const char *other);		// Boolean comparison, true or false, not equal, greater or less
	bool		operator<(const CString &other);	// Boolean comparison, true or false, not equal, greater or less
	bool		operator<(const char *other);		// Boolean comparison, true or false, not equal, greater or less
	bool		operator<=(const CString &other);	// Boolean comparison, true or false, not equal, greater or less
	bool		operator<=(const char *other);		// Boolean comparison, true or false, not equal, greater or less
	bool		operator!=(const CString &other);	// Boolean comparison, true or false, not equal, greater or less
	bool		operator!=(const char *other);		// Boolean comparison, true or false, not equal, greater or less
	CString		operator+(const CString &other);	// To "ADD" two CStrings together
	CString		operator+(const char *other);		// To "ADD" a string to a CString
friend	CString		operator+(const char*,			// To "ADD" a CString to a string
				const CString &second);
	int		CompareN(const char *,int);		// Boolean comparison
	int		StrNCmp(const char *c,int l);		// Wraps the strncmp function
	int		Chop(const char *c);			// chops at first occurence of char
	int		Chop();					// chops one char off the end, 
	void		Chomp();				// chops all nonprint chars off the end of string 
								// returns new length
	bool		FGetS(FILE *);				// reads in one line, returns success or failure
	bool		GetLine(FILE *);			// reads in one line, returns success or failure
	void		TokenInit(const char *c);		// Sets up to tokenize on char c
	CString		*GetToken();				// Returns a cstring containing just token
	bool		SetDelim(const char *c);		// Sets the delim string, does not reset the tokenizer
								// Token Init should have been called first.

	bool		GetFloat(float*);			// finds the next token, converts to float 
	bool		GetInt(int*);				// finds the next token, converts to int 
	bool		GetLongInt(long int*);			// finds the next token, converts to long int 
	bool		GetULongInt(unsigned long int*);	// finds the next token, converts to unsigned long int 
	bool		GetBool(bool*);				// finds the next token, converts to bool 

	CString		*GetRemainder();			// Returns a cstring containing the
	int		ConvertBackSlash();			// Converts all instances of \some char to proper code
								// \n becomes newline
								// \t becomes tab
								// returns count of howmany coversions took place

	int		ConvertBinary();			// Converts all instances of some binary char to proper backslashedchar
								// \10 becomes \n 
								// returns count of howmany coversions took place

								// remainder of the string from present pos
								// to end

	void		Log();					// Writes it self to log via the LogFunc Procedure
	void		Log(char *);				// As above but with char * prepended
	void		Log(char *,char *);			// As above but with prepend and append
	void		Print();				// Writes it self to stdout 
	void		Print(char *);				// As above but with char * prepended
	void		Print(char *,char *);			// As above but with prepend and append

private:
	void		IntAppend(const char *addition);	// Append string
	char		*GetStringInt();			// For Internal use ONLY
	bool		SelectToken(int *start);		// Select the token
	int		length;					// does NOT include the \0 at end
								// but buffer size DOES
	char		*buf;					//
	int 		nextchar;				// the nextchar under consideration
	char		dstring[MAX_DELIM_LEN];			// delimiter string, each char in dstring couldbe a delimiter
								// ie for white space, a space or a tab
	int		delim_len;				// length of delimiter string
	bool		tokinitready;				// tells whether a token has been specified yet.
static	LOGFUNCPTRTYPE	LogFunc;				// variable function pointer. so do this instead
static	unsigned long	count;
static	unsigned long	logfunc;				// maybe we can store it here.
	};

#endif //_CSTRING_HPP_

