///////////////////////////////////////////////////////////////////////////////////////////////////
// Client
// ctextwin.hpp : This is a class to support the text window for player communications
//
// by Eric J Krejci
// Copyright (C) 1999 Eric J Krejci
// Revised : 26 Jan 2006
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation version 2.
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
// Contact:
//              email           worldii@viagloria.com
//              web             www.viagloria.com
///////////////////////////////////////////////////////////////////////////////////////////////////
#if !defined(_CTEXTWIN_HPP_)  //skip this stuff if it has already been seen
#define _CTEXTWIN_HPP_        //by the compiler

///////////////////////////////////////////////////////////////////////////////////////////////////
// The way this will work is this.
// 1) The CTextWin will be made of an array of CTextLine s
// 2) CTextLine s will be derived from CString with a color feature
// 
///////////////////////////////////////////////////////////////////////////////////////////////////


///////////////////////////////////////////////////////////////////////////////////////////////////
// Includes
///////////////////////////////////////////////////////////////////////////////////////////////////
#include "wiii_util.h"
//#include "wiii_types.h"
#include "worldiii.h"
#include "cstring.hpp"
#include "cfont.hpp"

///////////////////////////////////////////////////////////////////////////////////////////////////
// Defines
// These are class specific defines
///////////////////////////////////////////////////////////////////////////////////////////////////


///////////////////////////////////////////////////////////////////////////////////////////////////
// CTextWin class
// 
///////////////////////////////////////////////////////////////////////////////////////////////////
class CTextWin 
{
public:
			CTextWin();
			~CTextWin();
	bool		Init();		// to have a non fail constructor, 
//	bool		Init(pthread_mutexattr_t *m_attr);		// to have a non fail constructor, 
	void		SetLogFunc(
				void (*pfunc)(char *)
				);
	void		BackSpace();					// Does the backspace function
	void		InsertLine(CString *);				// Inserts a line into the circular buffer
	void		InsertLine(char *);				// Inserts a line into the circular buffer
//	bool		InsertChar(char*);				// Inserts a char into the next position 
	bool		InsertChar(char);				// Inserts a char into the next position 
	void		SetBottomLine(int);				// Sets the bottom line to be displayed
	void		SetTextWinPos(int,int);				// Sets the window position in the client screen
	void		SetTextWinSize(int,int);			// Sets the window size in the client screen
	void		DrawTextWindow(int, int);			// Draws the internally contained text window
									// at its preset set size and position
	int		Chop(const char *c);				// 
	int		Chop();						// 
	void		Append(const char *addition);			// 
	void		Append(const char addition);			// 
	void		Return();					// 

private:
	int     LocalLock(
			pthread_mutex_t *,
			char*
			);                      // Lock the internal mutex to guard the socket calls
	
	int     LocalUnLock(
        		pthread_mutex_t *,
        		char*
			);                      // UnLock the internal mutex to guard the socket calls

	int		IncLine(int *);
	int		DecLine(int *);
	void		ProtLog(char *s);
	void		(*LogFunc)(char *s);
	CString		buffer[MESSAGE_BOX_SIZE];			// 
	CFont		*pFont;
	int		left,top;					// The window position
	int		width,height;					// The window position
	int		linesdisplayed;					// The number of lines displayed
	int		bottomline;					// The bottom line displayed
	int		nextline;					// The insertion point
	int		nextchar;					// The insertion point
	int		linecount;					// How many lines in buffer
	pthread_mutex_t *mutex;						// to protect against multiple access from
									// display thread and portmon thread. (abd others)
	};

#endif //_CTEXTWIN_HPP_

