////////////////////////////////////////////////////////////////////////////////////////////////////
// Merged
// ctimer.hpp
// This class will implement a timer that can read system time the provide elapsed 
// times to calling procedures 
// 
// by Eric J Krejci
// Copyright (C) 1999 Eric J Krejci 
// Revised : 26 Jan 2006
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation version 2.
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
// Contact:
//              email           worldii@viagloria.com
//              web             www.viagloria.com
////////////////////////////////////////////////////////////////////////////////////////////////////
#if !defined(_CTIMER_HPP_)  //skip this stuff if it has already been seen
#define _CTIMER_HPP_        //by the compiler

////////////////////////////////////////////////////////////////////////////////////////////////////
// Includes
////////////////////////////////////////////////////////////////////////////////////////////////////
#include <netinet/in.h>
#include <sys/socket.h>
#include <netdb.h>
#include <stdlib.h>

//#include "wiii_types.h"


////////////////////////////////////////////////////////////////////////////////////////////////////
// Defines
// These are class specific defines
////////////////////////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////////////////////////
// CTimer class
// The timer class should provide these kinds of time
// Time since timer was Init => InitSecs
// Time since Run was executed => ElapsedSecs
//	This resets when Stop is called, but Pause stops it from counting up.
// Time since last GetDelta was called => DeltaSecs
// 	This resets when Stop is called, and when it is first called this gives time since Run was called
//	This does not pause with a Pause call
////////////////////////////////////////////////////////////////////////////////////////////////////
class CTimer
{
public:
		CTimer();
		~CTimer();
	void	Init();				// to have a non fail constructor, 
//	void    Pause();			// to pause so that calls to elapsed do not increment the clock 
	void    Run();				// to run so that calls to elapsed DO increment the clock 
	void    Stop();				// to run so that calls to elapsed DO increment the clock 
	void	GetInit(long *secs, 		// Time since timer was Init
			long *usecs);
	void	GetElapsed(long *secs, 		// Time since Run was called, 
			long *usecs);
	void	GetDelta(long *secs, 		// 
			long *usecs);
private:

	long	initsecs;			// the time this timer was initted
	long	initusecs;			//
	bool	run;				//

	long	runsecs;			// the time when the most recent run was called
	long	runusecs;			

	long	prevsecs;			// used to help compute detla
	long    prevusecs;			// 

	};

#endif //_CTIMER_HPP_

