////////////////////////////////////////////////////////////////////////////////////////////////////
// Merged
// cusock.hpp
// This class will implement a message storage place 
// 
// by Eric J Krejci
// Copyright (C) 1999 Eric J Krejci 
// Revised : 26 Jan 2006
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation version 2.
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
// Contact:
//              email           worldii@viagloria.com
//              web             www.viagloria.com
////////////////////////////////////////////////////////////////////////////////////////////////////
#if !defined(_CUSOCK_HPP_)  //skip this stuff if it has already been seen
#define _CUSOCK_HPP_        //by the compiler

////////////////////////////////////////////////////////////////////////////////////////////////////
// Includes
////////////////////////////////////////////////////////////////////////////////////////////////////
#include <netinet/in.h>
#include <sys/socket.h>
#include <netdb.h>
#include <stdlib.h>

#ifdef SOLARIS
#include <thread.h>
#endif

#include "rudp.h"
#include "worldiii.h"
//#include "wiii_types.h"
#include "wiii_structures.h"


////////////////////////////////////////////////////////////////////////////////////////////////////
// Defines
// These are class specific defines
////////////////////////////////////////////////////////////////////////////////////////////////////
#define	CALL_BACK_MSG_NEW	0
#define	CALL_BACK_MSG_DISC	1
#define	CALL_BACK_MSG_CLOSE	2

// These flags are used in recvwait
#define FLAG_NOFLAGS		0
#define FLAG_WAIT_TIMED		1

////////////////////////////////////////////////////////////////////////////////////////////////////
// Structures
////////////////////////////////////////////////////////////////////////////////////////////////////
typedef struct conarray 
{
	unsigned long	session;	// session ID, so there can be session from same IP (supports firewalls)
	in_addr_t	IP;		// IP address of client  
	in_port_t	port;		// port of client  
	unsigned long	serial;		// the next serial number for sent packets
	unsigned long	rser;		// the last serial received from peer
	unsigned long	retryttw;	// time to wait (for ack for retry transmit for sent packet)
	unsigned long	retryto;	// time waited so far (for ack for retry transmit for sent packet)
	unsigned long	retranttw;	// time to wait (for prev packet for send retrans request)
	unsigned long	retranto;	// time waited so far (for prev packet for send retrans request)
	bool		connected;	// flag are we connected ?
	unsigned long	kattw;		// time to wait since any acked packets of any kind (Keep alive)
	unsigned long	ka_last;	// last time received so far
	int		next;		// points towards tail
	int		prev;		// points towards head
	int		state;		// shows wether the connection is fully formed or what not
					// =no connection
	} conarraytype;

int ConnectCallBack(connecttype *cparm);

////////////////////////////////////////////////////////////////////////////////////////////////////
// CUSock class
////////////////////////////////////////////////////////////////////////////////////////////////////
class CUSock
{
friend void	*PortMan(void *);		// This is the routine for managing a port
public:
		CUSock();
		~CUSock();
	bool	Init(
//			pthread_mutexattr_t*	// Just for now to debug a threading issue
						// we will eventually put it back to initting here.
			);			// to have a non fail constructor, 
	bool	Create(
			sockaddr_in*,
			socklen_t
			);			// This creates the socket and stores the connection info in object 
	void	SetLogFunc(
			void (*pfunc)(char*)	// This is the function name var
			); 			// sets the logging function to call,
	void	SetConnectFunc(
			int (*pfunc)(connecttype*)
			); 			// sets the NewConnect function to call,
	void	SetDisConnFunc(
			int (*pfunc)(connecttype*)
			); 			// sets the DisConnect function to call,
	void	SetCloseFunc(
			int (*pfunc)(connecttype*)
			);			// sets the CloseFunc function to call,
	void	DeInit();			// to have a non fail destructor, 


	bool	Connect();			// To connect to server at an already set address
	int     ConnectWait();			// Waits for second part of connecting: the server reponse
	int     ConnectResp();			// Does third part of connecting, sending final ack to server
        bool    Bind();				// To bind with a created socket and already set addres info
						// connection request 
        bool	Listen();			// to listen for a connection request 
        int     Accept(CUSock *);		// to accept connection request and then return
                                                // the new sockfd to manage it
        int     Send(				// Server
			char *, 		// The data to send
			int,	 		// length of data to send
			int,	 		// flags
			struct sockaddr_in *,	// dest address
			socklen_t,		// length of dest address
			unsigned long		// session
			);			

        int     Send(				// Client
			char *, 		// The data to send
			int,	 		// length of data to send
			int	 		// flags
			);			

	int     Recv(				// CLIENT FORM
			char **,		// A place to copy data to
			unsigned int 		// flags, FIXME, THIS IS FOR THINGS LIKE, STOP AND WAIT VS TIMEOUT
						// FUNCTIONALITY
			);			

	int     Recv(				// Server form
			char **,		// A place to copy data to
			unsigned int, 		// flags
			struct sockaddr_in *,	// a place for the from IP 
			socklen_t *,		// the length of size of struct containing peer addr info 
			unsigned long*		// session
			);			

        void     Close(				// SERVER FORM
			connecttype *cparm	// the connection to close
			);			//  
		
	int 	Close();			// CLIENT FORM

        void 	*GetMutexPTR();			// Get the mutex for this socket
	bool	StartPortMan();			// Start the port manager thread, that monitors for 
						// retries and serials and sending and receiving
	bool	PausePortMan();			// Pause the thread. 	

private:
	void	Set_Packet_Header(		// Over loaded
		char *c
		);

	void	Set_Packet_Header(		// Over loaded
		unsigned long p
		);

	void	*Get_Packet_DATA_Ptr_InHeap(	// Just get a ptr to the data in the heap block
		unsigned long p 		// This is ok because we are trying to avoid copying
		);				// as much as possible.

	void	Get_Packet_DATA_InHeap(		// Copy the received data from the heap block to a buffer
		unsigned long p ,
		char *,
		int
		);

	void	Set_Packet_InHeap(		// Copy the sending PACKET into the heap block
		unsigned long p ,		// THIS IS THE WHOLE PACKET
		char *,
		int
		);

	void	Set_Packet_DATA_InHeap(		// Copy the sending data into the heap block
		unsigned long p ,		// This is the packet data only
		char *,
		int
		);

	void	Set_Packet_NOACK_InHeap(	// Set flag in packet in heap
		unsigned long p 
		);

	void	Set_Packet_ACK_InHeap(		// Set flag in packet in heap
		unsigned long p 
		);

	void	Set_Packet_FLG_InHeap(		// Set flag in packet in heap
		unsigned long p, 
		unsigned char flag
		);

	unsigned long Get_Packet_SER_InBuff(	// Gets Serial from a buffer containing a packet
		char *c 
		);

	unsigned long Get_Packet_SES_InBuff(	// Gets Session from a buffer containing a packet
		char *c 
		);

	unsigned long Get_Packet_SES_InHeap(	// Gets Session from the heap block 
		unsigned long p 
		);

	void	Set_Packet_SES_InHeap(
		unsigned long p, 
		unsigned long ser
		);

	void	Set_Packet_RET_InHeap(
		unsigned long p, 
		unsigned long ser
		);

	void 	Set_Packet_SIZE_InHeap(
		unsigned long p, 
		unsigned short size
		);

	void 	Set_Block_PORT_InHeap(
		unsigned long p, 
		in_port_t port
		);

	unsigned long Get_Packet_SER_InHeap(	// Wraps setting the parms in and out of the packed heap
		unsigned long p 
		);

	void 	Set_Packet_SER_InHeap(		// Wraps setting the parms in and out of the packed heap
		unsigned long p, 
		unsigned long s
		);

	unsigned short 	Get_Packet_SIZE_InHeap(	// Wraps getting the parms in and out of the packed heap
		unsigned long 			// This way all changes occur in one area
		);

	unsigned long Get_Block_NEXT_InHeap(	// Wraps getting the next block in the heap
		unsigned long p 
		);

	in_addr_t Get_Block_IP_InHeap(		// Wraps getting the IP from the block in the heap
		unsigned long p 
		);

	void	Set_Block_IP_InHeap(		// Wraps setting the IP from the block in the heap
		unsigned long p, 
		in_addr_t addr
		);

	in_port_t Get_Block_PORT_InHeap(	// Wraps getting the PORT from the block in the heap
		unsigned long p 
		);

	void 	Zero_Block_InHeap(		// Wraps setting the block in the heap to zeros
		unsigned long p 
		);

	void 	Set_Block_NEXT_InHeap(		// Wraps setting the next block in the heap
		unsigned long p, 
		unsigned long v
		);

	void 	Set_Hash_NEXT_InHeap(		// Wraps setting the next hashnode in the heap
		unsigned long p, 
		unsigned long v
		);

	unsigned long Get_Hash_NEXT_InHeap(	// Wraps getting the next hashnode in the heap
		unsigned long p 
		);

	unsigned long Get_Hash_CONSINDEX_InHeap(	// Wraps getting the next connections index in the heap
		unsigned long p 
		);

	void Set_Hash_CONSINDEX_InHeap(		// Wraps getting the next hashnode in the heap
		unsigned long p,
		unsigned long v
		);

	void DeleteBlockFromQueue(		// Given a queue and possibly an endqueue pointer, this func
		unsigned long *qroot, 		// searches for and deallocates and deletes all blocks in the 
		unsigned long *endp, 		// queues identified by the qroot given that match IP and session
		unsigned long session, 
		in_addr_t IP
		);


	void	ProtLog(			// I suspect that i had a race in the way i was logging
		char * s 			// I had (unwittingly) hiearchical locks, in differing orders 
						// (classic race). Fix: give each thread its own logging, with
		);				// its own mutexes if required.
						// EVEN BETTER FIX: GIVE ALL THREADS THIER OWN LOGGING

	int	SendTo(	
		void *,
		int length,
		struct sockaddr_in *a, 
		socklen_t alen	
		);				// The common low level sendto with select for readiness

	void	RecvWait(unsigned int);		// Wait for condition variable to be set, block thread 
	void	RecvSignal();			// Set condition variable, wake up thread

	unsigned long	
		GetConnectionIndex(		// Function overloaded. returns the index to the connectionarray 
		in_addr_t,			// for the given parameters.
		char *,
		int
		);				// Go find and return the session index

	unsigned long	
		GetConnectionIndex(		// Function overloaded. Returns the index to the connectionarray 
		in_addr_t,			// for the given parameters.
		in_port_t,
		unsigned long
		);				// Go find and return the session index

	bool	ValidIPSes(			// Check to see if packet is from a connected client
		in_addr_t,			// ip address of peer sending info
		char *,				// packet
		int,				// packet len
		unsigned long*
		);				// returns session index if true

	bool	NeedAck(char *, int);		// Is the packet requesting an ack

	int	SendAck(
			char *, 
			int, 
			struct sockaddr_in*,
			socklen_t
			);			// Send an ack for the given packet

	bool	IsMissingPrev(			// Are there any previous packets missing
			char *, 		// packet
			int,			// len
			unsigned long
			);			// sessionindex

	int	PutInRecvQueue(
			char *, 
			int,
			struct sockaddr_in*,
			unsigned long
			);			// put packet into recvqueue

	int	CheckWaitQFreeUp(
			char *,
			int,
			struct sockaddr_in*
			);			// See if given packet frees someone else from waitq

	int	AddToWaitQueue(
			char *,
			int,
			struct sockaddr_in*
			);			// Add this packet to wait q

	int	RequestRetrans(
			char *,
			int,
			struct sockaddr_in*,
			socklen_t);		// Request retrans based on given packet

	bool	IsClose(char *,int);		// Is this ctrl packet a close ?
	bool	IsPing(char *,int);		// Is this ctrl packet a ping ?
	bool	IsAck(char *,int);		// Is this ctrl packet an ack ?
	bool	IsAck(unsigned long);		// Is this ctrl packet an ack ?
	bool 	IsRetrans(char *, int);		// Is this ctrl packet a retrans request ?
	bool	IsConnect(char *, int);		// Is this ctrl packet a connect request ?
	bool	IsComplete(char *, int);	// Is this ctrl packet a completion request ?
	bool	IsDisconnect(char *, int);	// Is this ctrl packet a disconnect request ?
	bool	IsDisconnectComplete(char *, int);	// Is this ctrl packet a disconnect request ?
	bool	IsRUDP(char *);			// Is this char buffer the beginning of an RUDP ?
	bool	IsDATA(char *);			// Does this char buff contain a data packet ?
	bool	IsCTRL(char *);			// Does this char buff contain a ctrl packet ?
	bool	IsRecvQEmpty();			// Is the recvqueue empty ?
	int	HandlePing(char *, int);	// Use the given ack to remove packet from retry q
//	int	HandleClose(
//			char *, 
//			int,
//			struct sockaddr_in*
//			);			// Use the given ack to remove packet from retry q

	int	HandleAck(
			char *, 
			int,
			struct sockaddr_in*
			);			// Use the given ack to remove packet from retry q

	int 	HandleRetrans(
			char *, 
			int,
			struct sockaddr_in*,
			socklen_t
			);			// Retransmit the requested packet

	int	HandleConnect(	
			char *, 
			int,
			struct sockaddr_in*
			);			// Setup a connection 

	int	HandleComplete(
			char *, 
			int,
			struct sockaddr_in*
			);			// Complete a connection 

	int	HandleDisconnect(
			char *, 
			int, 
			struct sockaddr_in*
			);			// take down a connection
	int	HandleDisconnectComplete(
			char *, 
			int, 
			struct sockaddr_in*
			);			// take down a connection
 	int	AllocPkt();			// Allocate a packet from the heap for either retry or sendq
	unsigned long	AllocHsh();		// Allocate a packet from the heap for either retry or sendq
	int	AllocConSlot();			// Allocate a slot in the connections array
	int	DeAllocPkt(unsigned long);	// DeAllocate a packet from the heap 
	int	DeAllocHsh(unsigned long);	// DeAllocate a packet from the heap 
	void	SetForCloseConSlot(int);	// Moves the given slot in the connections array to the close list
	void	DeAllocateConSlot(int);		// Moves the given slot in the connections array to the free list
	int	DeAllocConSlot(int);		// DeAllocate the given slot in the connections array
	int	AddToSendQueue(unsigned long );	// Allocate a packet from the heap for either retry or sendq
	int	SendData();
	int	RecvData();

	unsigned long
		GetSerial(			// returns status
			unsigned long*,		// return value 
			in_port_t*,		// return value 
			in_addr_t , 		// ip to use as hash 
			unsigned long , 	// session to lookup 
			int 
			);			// how much to increment serial by

	int	CheckCloseList();
	int	CheckRetries();
	int	CheckRetrans();
	int	CheckKeepAlive();
	void	Set_Packet_Values(		// Set the packet header data
		char *packet,
		unsigned long ser,
		unsigned long retry,
		unsigned long ses,
		unsigned char dc,
		unsigned char ack,
		unsigned short len
		);

        int     LocalLock(
			pthread_mutex_t *, 
			char* 
			);			// Lock the internal mutex to guard the socket calls 

        int     LocalUnLock(
			pthread_mutex_t *, 
			char* 
			);			// UnLock the internal mutex to guard the socket calls

	char 	rawbuf[600];			// this is a place to put the initially received data
	int    	sockfd;				// the socket file descriptor
        int    	port;				// the port to connect on
        int    	state;				// the state of the socket, 
						// ST_NOTINIT, ST_INIT, ST_CONNECTED, ST_CLOSED 
                               			// ST_NOTINIT->ST_INIT
						// ST_INIT->ST_CONNECTED
						// ST_INIT->ST_INIT
						// ST_CONNECTED->ST_CLOSED
						// ST_CLOSED->ST_INIT
						// data may be sent * received in the ST_CONNECTED state only 
        struct	hostent *he;
        struct	sockaddr_in
			serv_addr;		// connector's address information

        pthread_mutex_t	*skmutex;		// Mutex to protect the object from
                                                // concurrent access.
//	pthread_mutex_t	*recv_cndmutex;		// Mutex for cond variable
	pthread_cond_t	*recv_condvar;		// Cond variable for waiting

						// Actual code will be handled in os.cpp
	char		sockettype;		// 's' => server, 'c' => client
						// server tracks and assigns sessions and has connect array, 
	 					// client has single connection struct 
	char		*packetheap;		// Pointer to the heap that holds all the retry packets
	char		*hashheap;		// Pointer to the heap that holds all the nodes for the hash
	unsigned long 	pktheapmap[PACKETHEAPCOUNT];	// Allocation map for the above heap
	unsigned long 	hshheapmap[HASHHEAPCOUNT];	// Allocation map for the above heap
	unsigned long 	sendqueue;		// Where external threads dump there outgoing stuff 
	unsigned long 	retryqueue;		// Where stuff waits for acks
	unsigned long 	recvqueue;		// Where external threads get the incoming stuff
	unsigned long 	recvqueuetail;		// Where external threads get the incoming stuff
	unsigned long 	waitqueue;		// Where incoming packets wait if we need a retrans to 
	unsigned long 	waitqueuetail;		// Where incoming packets wait if we need a retrans to 
						// keep serialized in order
	unsigned long 	pktcurr;		// Start looking from here next time so that we'll find an
	unsigned long 	hshcurr;		// open block sooner.

	conarraytype	*connections;		// an array of current connections
	unsigned long	sessionhash[256];	// take the least significant bye of the IP and use it as a
						// has key to cut down the number of lookups 
	 					// This will be the last byte of four in the IP address
	 					// The IP is stored in NETWORK byte order (bigendian)
	 					// Most significant bye first.
	int		allochead;		// This is so we can search the list of sessions faster
	int		alloctail;		// we wont have to search a sparsely popped array to get all sessions
	int		freehead;		// so we can alloc a known free slot fast
	int		freetail;
	int		closehead;		// list of those connections to close at some point
	int		closetail;
	pthread_t	portmanthread;		// Thread handle for the portmanager thread
	pthread_t	initthread;		// Thread handle for the thread that called init

	struct timeval	tv;			// This is how long we wait for data at the port. This will be adjusted
						// up or down by the code based on tuning parameters.
	bool		timedout;		// Did we timeout last time ?
	int		errorcode;		// Last code received
	struct sockaddr_in
			addr;			// The address we are binding to or sending to
	socklen_t	addrlen;

	
	int		logcount;
//	int		(*LogFunc) (char * s, char * log);	// Pointer to the logging function
	void		(*LogFunc) (char * s);			// Pointer to the logging function
	int		(*ConnectFunc) (connecttype*);		// Pointer to the connect callback function
	int		(*DisConnectFunc) (connecttype*);	// Pointer to the disconnect callback function
	FILE		*logfile;
	char		extlogfile[LOGFILENAMELEN];	// This is set to the ext thread logfile
        char		locallogfile[LOGFILENAMELEN];	// This is loaded with the correct logfile for each thread
	
				
	};

#endif //_CUSOCK_HPP_

