///////////////////////////////////////////////////////////////////////////////////////////////////
// Client and modeler NEW VERSION
// cworld.hpp : This class can
// read in from file the description list of models that comprise the world
// save world state to file
// render world (collection of models) to screen
//	with animation
// 
//
// by Eric J Krejci
// Copyright (C) 1999 Eric J Krejci
// Revised : 26 Jan 2006
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation version 2.
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
// Contact:
//              email           worldii@viagloria.com
//              web             www.viagloria.com
///////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////
// The CWorld Class will start to be come more of a real rendering engine.
// It will be where 
//	- keyboard input is taken
//	- where the scene is rendered
//	- where simulation is done
//	- where preculling based on 
//		- visibility
//		- region 
//		- whether inside something or not is done.
///////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////
// Defines
///////////////////////////////////////////////////////////////////////////////////////////////////

#if !defined(_CWORLD_HPP_)  //skip this stuff if it has already been seen
#define _CWORLD_HPP_        //by the compiler

///////////////////////////////////////////////////////////////////////////////////////////////////
// Includes
///////////////////////////////////////////////////////////////////////////////////////////////////
#include <pthread.h>
#include <GL/gl.h>
#include <GL/glu.h>
#include "SDL.h"
#include "SDL_image.h"


//#include "wiii_types.h"
#include "wiii_structures.h"
#include "worldiii.h"
#include "ctextwin.hpp"
#include "ctimer.hpp"
#include "cusock.hpp"
#include "citem.hpp"

///////////////////////////////////////////////////////////////////////////////////////////////////
// Defines
// These are class specific defines
///////////////////////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////////////////////
// Defines
// These are class specific defines
///////////////////////////////////////////////////////////////////////////////////////////////////
#define 	CW_OK				0	// the operation completed successfully
#define		CW_ERR_FILE_OPEN		1	// File open failed 
#define		CW_ERR_CONVERSION		2	// File fscanf failed 
#define		CW_ERR_TEXLOAD			3	// Image load failed 
#define		CW_ERR_RENDER			4	// Render failed 
#define		CW_ERR_SUBMODLOAD		5	// Submodel load failed 
#define		CW_ERR_SUBPARTLOAD		6	// Subpart load failed 
#define		CW_ERR_LISTLOAD			7	// Subpart load failed 
#define		CW_ERR_MEMALLOCATE		8	// Mem alloc failure  
#define		CW_ERR_UKNOWNLIST		9	// Unknown type of list  

#define		TOKEN_TRISTRIP			"[tristrip]"
#define		TOKEN_TRILIST			"[trilist]"
#define		TOKEN_TRIFAN			"[trifan]"

class CIndex;
class CRegion;
class CPart;
class CInode;
class CModInode;
class CRegInode;
class CPortal;
class CRoom;
class CWorld;
class CList;
  
///////////////////////////////////////////////////////////////////////////////////////////////////
// CModel class
///////////////////////////////////////////////////////////////////////////////////////////////////
class CModel 
{


public:
				CModel();			// Init pointers
				~CModel();

	bool			Init();				// sets pointers into data struct;
//	void			SetFileName(const char*);	// DEBUG!!!!
	bool			InsertPart(CPart*);		// Given existing CPart insert in front 
	int			Load(char *filename, 
					int level
					);			// The level in the tree add
	int			Destroy();
	int			Render();
	bool			Save(char *filename);		// Save the file to the specififed 
//	bool			Save();				// Save the file to the specififed 

private:
	int			LoadHead( 
					FILE *f, int level, 
					CString **tok, int *val
					);
	int			LoadLine(
					FILE *, char**, char**,
					float *,float *,float *,
					float *,float *,float *
					);

	CModInode		*sublist;			// points to list of submodels
	CPart			*partlist;			// points to list of parts

	VECT			*pos;				// they should stay with 
	VECT			*rot;				// their owm model

	}; ///////////////////////////////////////////////////////////////////////////////end CModel

////////////////////////////////////////////////////////////////////////////////////////////////////
// CPortal class
////////////////////////////////////////////////////////////////////////////////////////////////////
class CPortal
{

friend class CWorld;
friend class CRoom;
public:
				CPortal();			// Init pointers
				~CPortal();

private:
	long			roomid;				// this is the objid of the room we are linking to
								// this is a one way link, to be able to come back
								// the other room must have a link to this room
	CModel			*model;				// This is the model that represents the 
								// portal for rendering purposes,
								// with submodels and animation et al
	float			dist;				// When with in this distance to portal
								// char is going into the portal
	CRoom			*room;				//
	CString			modelfilename;			// The filename of the model for the portal
	CPortal			*nextportal;			// Pointer to next  in linked list of portals
	CString			portalname;			// the name of the portal
	CString			portalfilename;			// the name of the portal
	bool			open;				// Can chars get through the portal ?
	bool 			translucent;			// Can chars see through the portal ?
	};

///////////////////////////////////////////////////////////////////////////////////////////////////
// CRoom class
///////////////////////////////////////////////////////////////////////////////////////////////////
class CRoom
{

friend class CWorld;

public:
				CRoom();			// Init pointers
				~CRoom();

	bool			Init();				// sets pointers into data struct;
								// sets size;
	bool			InsertRoom(CRoom *r);		// Insert a given room at beginning of the list 
	bool			InsertRoom(CRoom *r,CRoom *pr);	// Insert a given room after the given room 

private:
	long			roomid;				// OBJID, same as used for other objects,
								// this implies that rooms have representations (CModels)
								// Just like anything else
	float			xpos;
	float			ypos;
	float			zpos;
	float			pitch;				// Can you tell i was a pilot once ?
	float			roll;				// rotations about various axes
	float			yaw;
	CModel			*model;				// This is the model that represents the 
								// room for rendering purposes,
								// with submodels and animation et al
	CString			modelfilename;			// The filename of the model for the room
	CPortal			portals[MAXPORTALS];		// Array of doors or windows to other rooms
								// Through which things can be seen
								// or through which chars can try to pass.
	CString			roomname;			// The name of the room itself
	CString			roomfilename;			// The filename of the room itself
	};

///////////////////////////////////////////////////////////////////////////////////////////////////
// CRegion class
///////////////////////////////////////////////////////////////////////////////////////////////////
class CRegion 
{

friend class CWorld;       //allows the CWorld class to get inside

public:
		CRegion();			// Init pointers
		~CRegion();
	void	SetIndex(int,int,int);		// Sets the index location for the region

private:

	int      x;
	int      y;
	int      z;
//	CRegion	*xnextreg;			// next region in CWorld region xlist
//	CRegion	*xprevreg;			// prev region in CWorld region xlist
//	CRegion	*ynextreg;			// next region in CWorld region ylist
//	CRegion	*yprevreg;			// prev region in CWorld region ylist
//	CRegion	*znextreg;			// next region in CWorld region zlist
//	CRegion	*zprevreg;			// prev region in CWorld region zlist
	CInode	*xmodh;				// beginning of cmodel list for sorted x
	CInode	*xmodt;				// end of cmodel list 
	CInode	*ymodh;				// beginning of cmodel list for sorted y
	CInode	*ymodt;				// end of cmodel list 
	CInode	*zmodh;				// beginning of cmodel list for sorted z
	CInode	*zmodt;				// end of cmodel list 
	CPortal	*nextportalnode;		// beginning of cportal list

	};

///////////////////////////////////////////////////////////////////////////////////////////////////
// CWorld class
///////////////////////////////////////////////////////////////////////////////////////////////////
class CWorld
{

public:
			CWorld();			// Init pointers
			~CWorld();
	bool		Init();								// sets pointers into data struct;
//	bool		Init(pthread_mutexattr_t *m_attr);				// sets pointers into data struct;
//	bool		Init(bool (*pfunc)(char *,HWND hwndTV,int,LPARAM),HWND hwndTV);	// sets pointers into data struct;
	void		SetLogFunc(
				void (*pfunc)(char*)
				);
	bool		InsertRegion(CRegion *r);	// Given an existing REGION struct
	bool		InsertRegionIntoIndex(
				CRegion *r,
				int whichindex
				);	// Given an existing REGION struct
							// Insert it at beginning of list
	bool		InsertModel(
				CModel *,
				CRegion *,
				GLfloat,
				GLfloat,
				GLfloat
				);			// Given an existing CModel 
							// Insert it at beginning of list
	void		SetSock(CUSock *s);		// Set the socket ptr
	CUSock		*GetSock();			// Get socket ptr
	bool		InsertModel(CModel *,CModel *);	// Given an existing CModel 
							// Insert it as a submodel
//	bool		InsertModel(CModel *,CModInode *);	// Given an existing CModel 
							// Insert it in given list
	bool		AddToMovers(CModel *);		// Given an existing CModel 
							// Add it to movers list
	CRegion		*FindRegionUnsorted(int,int,int);	// Find the region matching the indecies given
	CModel		*FindModel(CString *);		// Find the model matching the filename given
							// starting from the given region


	void		Print();			// DEBUGG !!!!!! DONOT USE IN PRODUCTION!!!!
	void		Print(CRegion *);		// DEBUGG !!!!!! DONOT USE IN PRODUCTION!!!!
//	void		Print(CModInode *,CModel *);	// DEBUGG !!!!!! DONOT USE IN PRODUCTION!!!!
	void		Print(CModel *,CModel *);	// DEBUGG !!!!!! DONOT USE IN PRODUCTION!!!!
	void		Print(CPart *);			// DEBUGG !!!!!! DONOT USE IN PRODUCTION!!!!
	void		Traverse();			// 
	void		Traverse(CRegion *);		// 
//	void		Traverse(CModInode *,CModel *);	// 
	void		Traverse(CModel *,CModel *);	// 
	void		Traverse(CPart *);		// 
	void		NewMover(posup*);				// Add a new mover for simulation & display
	void		MoversUpdate(posup*);				// Allows for updating the movement state of a moving object
	void		SimMover(int);					// Sim will be moved to inside cworld object
	void		DoMe();						// Get my inputs and do my motion
	void		Sim();						// Sim will be moved to inside cworld object
	void		SendUpdatesToServer();				// This updates server with my state
	void		Run();
	void		SetModel(int id, CString *o);
	void		Ins(bool);
	void		Del(bool);
	void		PgUp(bool);
	void		PgDn(bool);
	void		LevelLook();
	void		UpArrow(bool);
	void		DnArrow(bool);
	void		LeftArrow(bool);
	void		RightArrow(bool);
	void		SetSpeed(float);
	void		GetPosup(posup*);				// Get all state at once
	void		GetPos(float*,float*,float*);			// Get xpos, ypos, zpos
	void		SetObjID(CString *o);
	GLint		Load(char *filename);				// load the world from the specified file
	bool		LoadMover(char *filename);			// load the model from the specified file
	int		Render();
	bool		Save(char *filename);				// Save the file to the specififed file. If it exists
									// it will be overwritten. If not, it will be created.
//	void		Append(char *addition);				// This gets appended to the command string
	void		Append(char addition);				// This gets appended to the command string
	void		InsertLine(CString *);				// Wrapper for ctextwin.InsertLine
	void		InsertLine(char *);				// Wrapper for ctextwin.InsertLine
	void		Chop();						// 
	void		Return();						// 
	void		Shift(bool);						// 
	void		SetWindow(int,int);				// Tells CWorld what the display windows dimensions are
	void		SetTextWinSize(int,int);			// Tells CWorld how big the text window is

private:
	int		LoadHead( 					//
				FILE *f,  
				CString **tok, int *val
				);
	int		LoadLine(
				FILE *f, char **token, char **modelname,
				float *x, float *y, float *z,
				float *roll,  float *pitch, float *yaw
				);
	int		CalcFrameRate();				// calc the frame rate
	void		ProtLog(char *s);				// per thread logging


	void		(*LogFunc)(char *s);				//pointers to the function that will log for us
	pthread_mutex_t	*mutex;						// gotta prevent display thread, (main thread) and 
									// handler thread from simultaneously accessing
	GLint		frames;						// this is used for calculating the frame rate
	GLint		t0;						// for frame rate
	char		framestring[100];				// for frame rate


	GLfloat         matrix[16];					// a place to store the matrices

	CTimer		*simtime;					// 
//	CWinTimer	updatetime;					// 
	CUSock		*sock;						// This is the sock for comms with the server
	CTextWin	*textwindow;					// The place you see what you typed 
//	CFont		*frame_rate;					// The place you see the frame rate 
	CString		*commandstring;					// The place you type
	CFont		*font;						// The font to use
	int		windowwidth;					// width and height of display window
	int		windowheight;

	bool		pgup;						// the look up key
	bool		pgdn;						// the look dn key
	bool		ins;						// the move up key
	bool		del;						// the move dn key
	bool		uparrow;					// the move forward key
	bool		dnarrow;					// the move backward key
	bool		leftarrow;					// the turn left key
	bool		rightarrow;					// the right key
	bool		shift;						// 
	bool		alt;						// 
	bool		ctrl;						// 
	bool		meta;						// 
	float		turnaccel;					// to accelerate turns, the longer turning the faster the turn

	CIndex		*xindex;					// an index to CRooms, sorted on xpos
									// this is the only access to room objects
	GLint		fastregionscount;				// how many fastregions do we have ?
	CRegion		*fastregions[NUMFASTREGIONS];			// The close by regions. This list gets updated on a move.
	CRegion		*regions[NUMREGIONS];				// All the close by regions. For now just an array 

	CIterator	xreg;						// Three lists that maintain the regions in order
	CIterator	yreg;						// on each axis
	CIterator	zreg;
	CIterator	fastreg;					// this list tracks which are the fast regions

									// till i can figure out how to do it better.
//	CRegion		*xregh;						// The region list
//	CRegion		*xregt;						// The region list
//	CRegion		*yregh;						// The region list
//	CRegion		*yregt;						// The region list
//	CRegion		*zregh;						// The region list
//	CRegion		*zregt;						// The region list
	CInode		*movers;					// a list of models to use for movers
	CRoom		*rooms;						// All rooms in the world. The nextroom pointer
									// is simply for list maint, the chars go through the 
									// portal array of pointers
//	bool		(*treeadd)(char *,HWND hwndTV, int n,LPARAM p);	// A function to add entries to the tree 
//	HWND		hwndtv;						// control display (if present)
	CHARSTATES	movingchars[MAXNUMOFVISIBLECHARS];		// This array holds the state of all VISIBLE moving things
	bool		inside;						// whether to use the outside or inside renderer
	}; ///////////////////////////////////////////////////////////////////////////////end CWorld


////////////////////////////////////////////////////////////////////////////////////////////////////
// CIndex class
////////////////////////////////////////////////////////////////////////////////////////////////////
class CIndex
{

friend class CWorld;

public:
				CIndex();			// Init pointers
				~CIndex();

	bool			Init();				// sets pointers into data struct;
								// sets size;
	bool			InsertRoom(CRoom *r);		// Insert a given room at beginning of the list 

private:
	CRoom			*rooms[MAXROOMS];				// array of pointers to CRoom objects
	long			roomcount;			// how many rooms are there
	long			allocated;			// how many index slots have been allocated
	};

///////////////////////////////////////////////////////////////////////////////////////////////////
// CRegInode class
///////////////////////////////////////////////////////////////////////////////////////////////////
class CRegInode
{
friend class CWorld;
friend class CRegion;
                                                                                                                                           
public:
				CRegInode();
				~CRegInode();
	void		SetPos(
				GLuint,         // xlocation
				GLuint,         // ylocation
				GLuint          // zlocation
				);
                                                                                                                                           
private:
	CRegion		region;         // the region we're poiting to
	CRegInode	*next;          //
	CRegInode	*prev;          //
	};

///////////////////////////////////////////////////////////////////////////////////////////////////
// CInode class
// used to get from the CRegion structures to the models
///////////////////////////////////////////////////////////////////////////////////////////////////
class CInode
{
friend class CWorld;       //allows the CWorld class to get inside
friend class CModel;       //allows the CWorld class to get inside
public:
			CInode();                    // Init pointers
			~CInode();
	void		SetTrans(GLfloat);
	void		SetRot(GLfloat);
private:
        CInode			*next;          // next model sorted in list
        CInode			*prev;          // prev model sorted in list
	CModel			*model;		// model in list
	GLfloat			xpos;		// x 
	GLfloat			ypos;		// y 
	GLfloat			zpos;		// z
	GLfloat			roll;		// roll
	GLfloat			pitch;		// pitch
	GLfloat			yaw;		// yaw
						// x and roll go together
						// y and pitch go together
						// z and yaw go together
	};

///////////////////////////////////////////////////////////////////////////////////////////////////
// CModInode class
// used inside models to get to submodels
///////////////////////////////////////////////////////////////////////////////////////////////////
class CModInode
{
friend class CWorld;       //allows the CWorld class to get inside
friend class CModel;       //allows the CWorld class to get inside
public:
				CModInode();                    // Init pointers
				~CModInode();
	void		SetTrans(
				GLfloat,
				GLfloat,
				GLfloat
				);
	void		SetRot(
				GLfloat,
				GLfloat,
				GLfloat
				);
private:
	CModel			*model;         // either model or portal is used, not both at the same
//      CPortal			*portal;        // time. Just using the same Inode for two lists
        CModInode		*next;          // of different classes of objects
        CModInode		*prev;          // of different classes of objects
                                                                                                                                           
	GLfloat			xpos;
	GLfloat			ypos;
	GLfloat			zpos;
	GLfloat			roll;
	GLfloat			pitch;
	GLfloat			yaw;
	};
                                                                                                                                           
///////////////////////////////////////////////////////////////////////////////////////////////////
// CPart class
///////////////////////////////////////////////////////////////////////////////////////////////////
class CPart
{

friend class CModel;

public:
			CPart();			// Init pointers
			~CPart();

	bool		Init();				// sets pointers into data struct;
							// sets size;
	bool		InsertList(CList *l);		// Insert a given list at next frame index 
	bool		InsertList(CList *l,int i);	// Insert a given list at frame index i
	int		Load(FILE *f, int);
	bool		Save(FILE *f);
	int		Render();			// each cmodel should have its own matworld matrix
	int		Destroy();			// maybe we do want a destroy

private:
	int		framecount;			// how many frames to render for animation
	CList		*list[MAXFRAMECOUNT];		// An array pointers to CList
	CPart		*nextpart;			// points to list of parts
	};

///////////////////////////////////////////////////////////////////////////////////////////////////
// CList class
///////////////////////////////////////////////////////////////////////////////////////////////////
class CList 
{


public:
				CList();			// Init pointers
				~CList();

	bool			Init();				// sets pointers into data struct;
	int			Load(FILE *f, int level);	// The level in the tree add
	int			Destroy();
	int			Render();
	bool			Save(FILE *f);				// Save the file to the specififed 

private:
	int			LoadTriStrip(FILE *f, int level);
	int			LoadTriList(FILE *f, int level);
	int			LoadTriFan(FILE *f, int level);
	int			LoadHead(FILE *f, int level, 	// this loads the header stuff
					CString **tok, 
					CString **tex);
	int			LoadLine(			// this loads a vertex	
					FILE *, int level,
					MODVECT *
					);
//					int);
	int			RenderStrip();
	int			RenderList();
	int			RenderFan();

	VECT			*pos;				// moved in from cworld, they should stay with 
	VECT			*rot;				// their owm model
	MODVECT 		*list;				// all the vertices and their data
	GLuint			texture;			// this will turn into an array of texture names
	SDL_Surface		*TextureImage;			// load image
	int			tricount;
	int			vertcount;
	int			dontknow;
	GLuint			listtype;			// this tells what kind of list we have

	};


///////////////////////////////////////////////////////////////////////////////////////////////////
// Helper Functions
///////////////////////////////////////////////////////////////////////////////////////////////////
extern void CheckForGLErrors(void (*logfunc)(char*));

#endif //_CWORLD_HPP_

