////////////////////////////////////////////////////////////////////////////////////////////////////
// cxitem.hpp
// 
// CXItem and friend classes
// these classes implement a doubly linked list
// and also support a multi child tree list
// this is an extension of the other class. It supports multiple link pairs. The reason for this
// is so we can have lists in different orders. EX sorting the list of objects by x or y or z
// coordinates.	Also supports having sub lists.
// I THINK WE SHOULD REMOVE THE CHILD STUFF. THE LISTS ARE COMPLICATED ENOUGH AS IS, AND WE HAVE YET
// TO USE THEM.
//
// by Eric J Krejci
// Copyright (C) 1999 Eric J Krejci 
// Revised : 01 Jun 2007
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation version 2.
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
// Contact:
//              email           worldii@viagloria.com
//              web             www.viagloria.com
////////////////////////////////////////////////////////////////////////////////////////////////////
#if !defined(_CXITEM_HPP_)  //skip this stuff if it has already been seen
#define _CXITEM_HPP_        //by the compiler

////////////////////////////////////////////////////////////////////////////////////////////////////
// Includes
////////////////////////////////////////////////////////////////////////////////////////////////////
#include <pthread.h>

#include "worldiii.h"
//#include "wiii_types.h"
#include "cusock.hpp"

////////////////////////////////////////////////////////////////////////////////////////////////////
// Defines
////////////////////////////////////////////////////////////////////////////////////////////////////
#define		CX_STATE_NOTINIT	0
#define		CX_STATE_LIST		1
#define		CX_STATE_CURS		2

////////////////////////////////////////////////////////////////////////////////////////////////////
// Type Defs
////////////////////////////////////////////////////////////////////////////////////////////////////
typedef int WIIIHANDLE;
typedef int WIIILIST;

////////////////////////////////////////////////////////////////////////////////////////////////////
//Node class
////////////////////////////////////////////////////////////////////////////////////////////////////
class CXItem;  //forward declare Item 
class CChar;  //forward declare Item 

class CXNode
{
friend class CXIterator;
public:
	CXNode();
	~CXNode();
private:
	CXItem		*child;       //points the actual child 
	CXNode		*nextnode;    //points to the next node in the child list
     };

////////////////////////////////////////////////////////////////////////////////////////////////////
// XItem class
// This class simply has the pointersand the data in derived classes
////////////////////////////////////////////////////////////////////////////////////////////////////
class CXItem 
{
friend class CXIterator;       //allows the iterator class to get inside 

public:
     			CXItem();
	virtual		~CXItem();
	int           	GetVal() {return (value);}
	void          	SetVal(int i) {value=i;}

virtual	int		GetServerId();
virtual	OBJIDTYPE	GetObjectId();
virtual	CUSock		*GetSockPtr();
virtual	bool            HarvestPortmon();	// When all three return true, the server is harvested
virtual	bool            HarvestServer();	// When all three return true, the server is harvested
virtual OBJIDTYPE	GetCharacterID();	// Returns the Character Object ID
virtual CChar		*GetCharacterPtr();	// Returns a ptr to the Char obj
	int 		ItemReadLock();
	int 		ItemTryReadLock();
	int 		ItemWriteLock();
	int 		ItemTryWriteLock();
	int 		ItemUnLock();

private:
	static int 	itemcount;		//the count of all items
	static int	itemid;			//the id of the next item to be created 
	int		id;			//the id of this item
	int		value;			//FOR DEBUGGING
	bool		delflag;		// if set means this item should be deleted by cursor motion
						// routines, and should also be skipped by cursor motion routines



//	pthread_mutex_t	mutex;			//the object that will help prevent simultaneous access to instance
	pthread_rwlock_t	rwlock;		//Implement read write lock functions
	int		errorstate;
	CXItem		*nextitem[MAXLISTS];	//pointer to next
	CXItem		*previtem[MAXLISTS];	//pointer to previous
						// the head[i] and tail[i] pair will use, and only use,
						// the nextitem[i] and previtem[i] pairs on each
						// object in the list.
//	CXNode		*childlist;		//points to the child list
	CXItem		*parent;		//points to parent 
	};


////////////////////////////////////////////////////////////////////////////////////////////////////
// typedefs
////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////
// cursortype
////////////////////////////////////////////////////////////////////////////////////////////////////
typedef struct cursorxtype {
	CXItem		*curr;		// The item this cursor points to
	bool		inuse;		// Is this cursor allocated ? false = can be reallocated
	unsigned long	serverid;	// The server id, we will check that a server created this
					// cursor so we dont accidentally have one thread use
					// anothers cursor. Use Thread ID. 
	WIIILIST	list;		// Which list does this cursor use, and work in ?
					// A cursor stays in its own list.
//	pthread_mutex_t	mutex;		// A way to lock the cursor, for deletions and such
	} cursxtype;

typedef struct listxtype {
	CXItem		*head;		// The head item of this list
	CXItem		*tail;		// The tail item of this list 
	bool		inuse;		// Is this cursor allocated ? false = can be reallocated
	pthread_mutex_t	mutex;		// A way to lock the list
	} listx_t;

////////////////////////////////////////////////////////////////////////////////////////////////////
// CXIterator class
// This class maintains the list.
//
// This class implements a doubly linked list with multiple link chains. Why ? So that there can be a 
// main list, (of all objects say) and the sublists (possibly all or a subset) of objects. These 
// lists can be ordered on X position or other things.
//
// Lock access order
// 1) lock the iterator
// 2) lock the cursor
// 3) lock the object
//
//
// Note that list=0 will be the complete list of all items added. It is thus a special list.
// This is so that we wont lose access to any items by accident. And that when we delete there 
// will be a list that will give access to all items.
////////////////////////////////////////////////////////////////////////////////////////////////////
class CXIterator
{
public:
			CXIterator();
			~CXIterator();

	bool 		BegList (WIIIHANDLE curs) ;		// Pointing at item head points at.

	bool 		EndList (WIIIHANDLE curs);		// Pointing at item tail points at.
	bool 		MoreItems(WIIIHANDLE curs);		// This means that the cursor still points to 
								// a valid item. When false the cursor does not
								// point to an item. This can be used in 
								// either direction

	CXItem 		*MoveFirst(WIIIHANDLE curs);		// move to item head points at
	CXItem 		*MoveLast(WIIIHANDLE curs) ;		// move to item tail points at
	CXItem 		*MoveNext(WIIIHANDLE curs);		// move towards tail
	CXItem 		*MovePrev(WIIIHANDLE curs);		// move towards head
	CXItem		*DeleteItem(WIIIHANDLE curs);		// delete curr-> of curs
								// 2005.12.11
								// DeleteItem will lock the object. It will also lock 
								// the iterator so that adjustments to all the cursors
								// can be made if required to delete the item. Then it will 
								// release the iterator once deleting is done.

	CXItem 		*GetCurrent(WIIIHANDLE curs);		// returns pointer to the current item
								// 2005.12.11
								// GetCurrent will lock the object before returning
								// a pointer. This way thread one with its cursor
								// can access the object and not need to worry that thread
								// two will come along and delete it in the middle of one's 
								// access. Have a wait or fail immediately option. GetCurrent
								// needs to check that curr still points to a real object 
								// AFTER it locks the iterator. Once the object is locked 
								// the iterator can be unlocked for others to use.

	bool		ReleaseCurrent(WIIIHANDLE curs);	// Releases lock on current object
								// 2005.12.11
								// This is called after the object is manipulated or data read.
								// until thread "i" calls this on an object, thread "j" will
								// not be able to delete it or access it, so on.

	CXItem 		*SetCurrent(				// sets next to the same pointer after
				WIIIHANDLE curs, 
				CXItem *r
				);
								// verifying that it is in the list
	CXItem 		*FindItem(WIIIHANDLE curs,CXItem *r);	// returns a pointer if the item is 
								// in the list, next unaffected
	CXItem 		*InsertItem(				// Gotta say which list to put it in
				WIIIHANDLE curs, 
				CXItem *additem
				);

	CXItem 		*AppendItem(				// Gotta say which list to put it in
				WIIIHANDLE curs,
				CXItem *additem
				);

	void  		DisplayList(WIIIHANDLE curs);		// DEBUG USE ONLY
	void  		BackDisplayList(WIIIHANDLE curs);	// DEBUG USE ONLY

	int  		EndChildList () {return (cptr==0);}
//	CXNode 		*MoveFirstChild() {return (cptr=curr->childlist);} 	// moves to the first 
										// child of the curr item
//	CXNode 		*MoveNextChild();  			// similar to above
//	CXItem 		*GetCurrentChild(); 			// returns pointer to the current item
//	CXItem 		*AdoptChild(CXItem *parent,CXItem *child);
//	void  		OrphanChild(CXItem *child);
	int		GetCursor(				// allocate a cursor
				WIIILIST list,			// EVERYTHING is DONE with allocated cursors now
				unsigned long serverid,		// the default cursor is now no longer available
				WIIIHANDLE *curs		// each cursor belongs to a list.
				);
	int		AddLinkedList(WIIILIST *);		// setup another linked list, and return which list it is
	bool		DelLinkedList(WIIILIST);		// remove linked list
	void		GetLinkedLists(WIIILIST **,int *count);	// show linked lists in an array with count being a count . . .
	int		GetCountOfLinkedLists();			// shows how many linked lists there are
								
	int		ReleaseCursor(
				WIIIHANDLE curs,
				unsigned long serverid
				);				// Release a cursor

private:
	int 		IterLock();
	int 		IterUnLock();
	int 		ListLock(WIIILIST);
	int 		ListUnLock(WIIILIST);
	bool		CheckCursor(WIIIHANDLE curs);		// Check to see if cursor is valid
	CXItem		*RemoveItemFromList(			// remove an item from a single list.
				WIIILIST l,
				CXItem *,
				WIIIHANDLE curs			//
				);				
	void 		P_AppendItem(				// This is used to implement the zero list
				WIIIHANDLE curs,		// Which is used only to ensure we can delete everything
				CXItem *additem			// when time to delete.
				);
	void 		P_InsertItem(				// This is used to implement the zero list
				WIIIHANDLE curs, 		// Which is used only to ensure we can delete everything
				CXItem *additem			// when time to delete.
				);
	CXItem 		*P_MovePrev(WIIIHANDLE curs,
				WIIILIST list
				);				// does the actual moving 
	CXItem 		*P_MoveNext(WIIIHANDLE curs,
				WIIILIST list
				);				// does the actual moving 
	CXItem 		*P_MoveLast(WIIIHANDLE curs,
				WIIILIST list
				);				// does the actual moving 
	CXItem 		*P_MoveFirst(WIIIHANDLE curs,
				WIIILIST list
				);				// does the actual moving 
	CXItem 		*P_DeleteItem(
				WIIIHANDLE curs,	
				CXItem *
				);				// does the actual deleting
	bool		P_StdChecks(
				WIIIHANDLE curs,	
				WIIILIST list
				);
	




	pthread_mutex_t	mutex;
	listx_t		pointers[MAXLISTS];			// array of linked lists
								// the pointers[i].head and pointers[i].tail pair will use, 
								// and only use,
								// the nextitem[i] and previtem[i] pairs on each
								// objectin the list.
	CXNode  	*cptr; 					// this points to the current Node 
								// which points to a child
	int		curscount;				// This is how many cursors we have in use now
								
	int		maxlistcount;				// This is how many cursors we have made room for
								// 10 => [0]..[9]
	cursxtype  	cursarray[MAXCURSORS];			// This is an array of cursors, this allows many current
								// pointers into list. Since this is for multithreading
								// the routines that access these lists via these cursors
								// will need to use the Lock and Unlock functions.
								// Should we check to see if a thread has the right to use
								// a cursor ?

	int    		state;					// ready to allocate cursors ? (is there a linked list ?)
	int    		errorstate;
	};


#endif //_CXITEM_HPP_



