////////////////////////////////////////////////////////////////////////////////////////////////////
// rudp.h
//
// by Eric J Krejci
// Copyright Eric J Krejci February 1998
// Revised : 26 Jan 2006
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation version 2.
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
// Contact:
//              email           worldii@viagloria.com
//              web             www.viagloria.com
////////////////////////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////////////////////////
// Includes
////////////////////////////////////////////////////////////////////////////////////////////////////



////////////////////////////////////////////////////////////////////////////////////////////////////
// To connect 
// - server starts up
// - binds socket
// - then calls the "listen" this calls returns
// - the client starts and binds and such
// - then sends connect request
// - then server hears the request
// 	- builds connection
// 		- allocates connections array slot
// 		- assigns session number
// 		- creates hash entry
// 		- et al
// 	- responds with how many overdue packets he will keep per session
// - client responds with how many overdue packets *HE* will keep. this is the completing the 
//   connection portion
////////////////////////////////////////////////////////////////////////////////////////////////////
// PACKET HEAP BLOCK SCHEMATIC
// PACKET SCHEMATIC
// NOTE: had to align unsigned long on 4 byte boundry or a read would segfault
////////////////////////////////////////////////////////////////////////////////////////////////////
//	BLOCK STRUCTURE
//	NOTE THESE NUMBERS ARE IN NET FORM !!!!
//	ONLY THE LINK IS IN HOST FORM!!!!	
//	Bytes	Meaning					Example
//	0-3	in_addr_t IP address of remote peer	uint32 , either from or to
//	4-5	in_port_t of dest 			uint16 , 
//	8-11	link to next in chain 			uint32 , 
//	12-23	Unused
//	24-599	Packet data				aaabbbcccddd
//
//
//      
//	Inside a packet there is further structure
//	NOTE THESE NUMBERS ARE IN NETWORK FORM !!!!!
//	0-3	packet header				'RUDP' 
//	4-7	Serial
//	8-11	Retry number
//	12-15	session
//	16	Data/Ctrl Flag (dc flag)
//		0x0 =>	Data 
//		0x1 =>	Ctrl
//	17	ack flag , does this packet require an ack ?
//		0x0 =>  No ack
//		0x1 =>  Ack required
//	18-19	if data, this is packet data length (size)(64 + ????) 
//			
//	if Data
//	64-575	Data
//
//	if Ctrl
//	64	Type
//		1 =>	Ping
//		2 =>	Ack
//		3 => 	Connect Request
//		4 => 	Disconnect
//		5 => 	RetransRequest
//
////////////////////////////////////////////////////////////////////////////////////////////////////
// Defines for packets and blocks based on above
//////////////////////////////////////////////////////////////////////////////////////////////////// 
#define 	MAX_CONNECTIONS	400			// The max number of connections
#define 	MAX_SES_PER_IP	10			// The max number of sessions per IP
#define 	PACKETSIZE	576l			// The largest packet we can send over UDP
#define 	PACKETHEAPHDR	24l			// Extra space for pkt management in the heap
#define 	PACKETHEAPCOUNT	1000l
#define 	HASHBLKSIZE	8l			// two unsigned longs
#define 	HASHHEAPCOUNT	1000l			// 
#define 	MAXLINE         4096


////////////////////////////////////////////////////////////////////////////////////////////////////
// Defines for RUDP Error codes
//////////////////////////////////////////////////////////////////////////////////////////////////// 
#define		RUDP_OK				0	// No errors 
#define		RUDP_ERR_NO_PKT_HEAP		1	// No more memory slots in packet heap to alloc 
#define		RUDP_ERR_BAD_HEAP_NUM		2	// Heap blk number out of range
#define		RUDP_ERR_SER_NOT_FOUND		3	// Heap blk number out of range
#define		RUDP_ERR_ACK_NOT_FOUND		4	// Couldnt find the acked packet in retry queue
#define		RUDP_ERR_NO_MORE_CONNS		5	// At max connections   
#define		RUDP_ERR_NO_HSH_HEAP		6	// No more memory slots in hash heap to alloc 
#define		RUDP_ERR_SES_NOT_FOUND		7	// Session not found
#define		RUDP_ERR_BAD_SOCK_TYPE		8	// Socket type was not either 'c' or 's'
#define		RUDP_ERR_SOCK_NOT_SERV		9	// HandleConnect was called with sockettype=='c'
#define		RUDP_ERR_SOCK_ERR		10	// Sock error
#define		RUDP_ERR_EINVAL			11	// Invalid value in system call

////////////////////////////////////////////////////////// Offset from begin of :
#define		RUDP_IP			0		// heap block to peer IP
#define		RUDP_PORT		4		// peer port 
#define		RUDP_NEXT		8		// Link to next in chain
#define		RUDP_PACKET		24		// heap block to packet data 
#define		RUDP_PKT_HDR		0		// id's packet as an RUDP packet
#define		RUDP_PKT_SER		4		// packet to serial
#define		RUDP_PKT_RET		8		// packet to retry count for this packet
#define		RUDP_PKT_SES		12		// packet to session id
#define		RUDP_PKT_FLG		16		// data/ctrl type flag
#define		RUDP_PKT_ACK		17		// ack type flag
#define		RUDP_PKT_SIZE		18		// packet to data size, 0-512 bytes
#define		RUDP_PKT_DATA		64		// packet to payload data

#define		RUDP_HSH_CONSIDX	0		// These are the offsets to the connections index
#define		RUDP_HSH_NXT		4		// and the next node in linked list.

////////////////////////////////////////////////////////////////////////////////////////////////////
// Defines for RUDP Control packet types
// Note that RUDP_CTRL_COM gets sent by server to client after a RUDP_CTRL_CON is received by server
// telling client "we are connected"
// then client sends RUDP_CTRL_COM back to server to complete and say to server "we are connected"
// For disconnect
// - either can initiate
// - first one sends RUDP_CTRL_DIS, 
// - second one notifies ext thread, tears down, responds with RUDP_CTRL_DISCOM
// - first one when receives RUDP_CTRL_DISCOM, notifies ext thread, tears down
//////////////////////////////////////////////////////////////////////////////////////////////////// 
#define		RUDP_CTRL_PNG		1		// ping packet to check keep alive
#define		RUDP_CTRL_ACK		2		// ack a sent packet
#define		RUDP_CTRL_CON		3		// request a connection
#define		RUDP_CTRL_COM		4		// complete a connection
#define		RUDP_CTRL_DIS		5		// disconnect start
#define		RUDP_CTRL_RET		6		// Retrans request
#define		RUDP_CTRL_DISCOM	7		// disconnect complete

////////////////////////////////////////////////////////////////////////////////////////////////////
// Defines for RUDP Flags
//////////////////////////////////////////////////////////////////////////////////////////////////// 
#define		RUDP_DC_DATA	0
#define		RUDP_DC_CTRL	7
#define		RUDP_AF_NOACK	0
#define		RUDP_AF_ACK	9

////////////////////////////////////////////////////////////////////////////////////////////////////
// Defines for RUDP State codes
// server goes like this
// 0->1
// 1->2 or 1->0
// 2->0 or 2->3
// 3->2 or 3->0
//
// client goes like this
// 0->4
// 4->2 or 4->0 
// 2->3 or 2->0
// 3->2 or 3->0
//////////////////////////////////////////////////////////////////////////////////////////////////// 
#define		RUDP_STA_NOCON		0l		// No connection
#define		RUDP_STA_CONRQ		1l		// set after received and processed 
							// connection request ok
#define		RUDP_STA_CNCT		2l		// Set after connection properly set up
#define		RUDP_STA_KA		3l		// Havent heard from other side in a while
							// trying a ping to see if he's still there 
#define		RUDP_STA_REQ		4l		// state client goes to when requesting a con
#define		RUDP_STA_CLOSING	5l		// state client goes to when requesting a con

 

 

