/////////////////////////////////////////////////////////////////////////////////////
// wii_structures.h
// This include will define various strucutres 
// 
// by Eric J Krejci
// Copyright (C) 1999 Eric J Krejci
// Revised : 22 Apr 2006
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation version 2.
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
// Contact:
//              email           worldii@viagloria.com
//              web             www.viagloria.com
/////////////////////////////////////////////////////////////////////////////////////

#if !defined(_STRUCTURES_H_)  //skip this stuff if it has already been seen
#define _STRUCTURES_H_        //by the compiler

/////////////////////////////////////////////////////////////////////////////////////
// Includes
/////////////////////////////////////////////////////////////////////////////////////
#include <netinet/in.h>
#include <sys/socket.h>
#include <netdb.h>
#include <curses.h>

#include "cstring.hpp"
/////////////////////////////////////////////////////////////////////////////////////
// Defines
// These are class specific defines
/////////////////////////////////////////////////////////////////////////////////////
typedef struct playerchartype {
	CString            *playerid;
	int               objsaid;
	CString            *fname;
	CString            *race;        // race
	float              xpos;
	float              ypos;
	float              zpos;
	CString            *title;
	} pctype;

typedef struct VECT {
	float	x;
	float	y;
	float	z;
	} vect;

typedef struct POSUP {		// Position states of player character avatars
	vect	pos;		// Position vector
	vect	vel;		// Velocity vectors, actual velocity right now.
	long	id;		// ObjID
	float	dir;		// direction in degrees
	float	speed;		// requested speed
	float	elev;		// direction in degrees
	CString model;		// model of the char
	} posup;

typedef struct PARMSTRUCT{
	bool		b[10];
	long int	i[10];
	unsigned long	u[10];
	double		d[10];
	CString 	s[10];
	void		*extra;
	} parmstruct;

typedef struct CMDTYPE {
	CString	cmd;
	int	parmcnt;
	char	exact;		// < less than given number of params
				// - upto or equal to given number of parms
				// = exactly given number of parms
				// + equal to or greater than given number of parms
				// > greater than the given nmber of parms
	int	msg;		// the WIIM code for this message
	} cmdtype;

typedef struct TEXPOINT {
	float	u;
	float	v;
	} tpoint;

typedef struct MODVECT {
	VECT	point;
	VECT	norm;
	float	u,v;
	} modvect;

typedef struct CURSWIN {
	WINDOW	*mainwin;
	WINDOW	*menuwin;
	WINDOW	*debugwin;
	} curswin;

///////////////////////////////////////////////////////////////////////////////////////////////////
// Note the speed is what is set by user, ie user selects 0-9 as speed levels
// that is stored here. then at selection time we find the x and z components so that we can avoid
// the sin's and cos's each time we simulate. thus all we need to do is add vx * dt etc.
///////////////////////////////////////////////////////////////////////////////////////////////////
class CModel;
class CServer;
typedef struct CHARSTATES {	// Position states of player character avatars
	bool	active;		// !active means this slot can be reused.
	bool	render;		// Is the mover to be rendered ?
	bool	moving;		// Is the mover to be rendered ?
	bool	turning;	// Is the mover to be rendered ?
	vect	pos;		// Position vector
	vect	vel;		// Velocity vector
	float	trnspd;		// how many degrees per dt we are turning
	float	speed;		// magnitude only
	long	id;		// ObjID
	float	dir;		// direction in degrees
	float	elev;		// direction in degrees
	CModel	*model;		// The pointer of the top level model that is 
				// the model for the mover
	} charstates;

typedef struct workchunktype {
	char		*packet;
	int		pcktlen;
	CServer		*server;
	workchunktype	*next;
	void		(*routine)(void*);
	} WORKCHUNK;

typedef struct connect
{
        unsigned long   session;        // session ID
        in_port_t       port;           // port of client
        in_addr_t       IP;             // IP address of client
        int             msg;            // Message
        } connecttype;

typedef struct chunktype {
	int       		size;
	chunktype *  	nextchunk;		// this is a doubly linked list 
	chunktype *  	prevchunk; 
	char *    		data;  
	} chunk;

typedef struct newmessagetype {
	int		msg;
	int		parm;
	CString 		text1;
	CString 		text2;
	void *		data0;
	void *		data1;
	} newmessage;

//typedef struct messagetype {
//	int		msg;
//	int		parm;
//	messagetype *	nextmsg;
//	messagetype *	prevmsg;
//	void *		data0;
//	void *		data1;
//	} message;

typedef struct acctreqtype {
	void * 		fname;
	void *		lname;
	void * 		email;
	} acctreq;

#ifdef WIN32

typedef struct {
	int    nThread;  // Thread number.
	HWND  hWnd;
	} THREADPACK, *PTHREADPACK;

typedef struct {
	void *		mqueue;
	void *		raw;
	char *		host;
	char *		port;
	HWND			hWnd;
	} PORTMONPARM;

typedef struct {
	HANDLE		h;
	} MHAND;

#endif

#endif //_STRUCTURES_H_

