//////////////////////////////////////////////////////////////////////////////////////////////
// display.cpp -- a test of routines for basic background life
//
//
// Life module 
// for World II
// by Eric J Krejci
// Copyright Eric J Krejci February 1998
// Revised : 01 Feb 2000
//////////////////////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////////////////////
// Defines
//////////////////////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////////////////////
// Includes
//////////////////////////////////////////////////////////////////////////////////////////////
#include <pthread.h>
#include <unistd.h>
#include <curses.h>
#include "life.h"
#include "csoil.hpp"

//////////////////////////////////////////////////////////////////////////////////////////////
// Includes
//////////////////////////////////////////////////////////////////////////////////////////////
extern		int 		movecommand;
extern		WINDOW		* mainwin;
extern		WINDOW		* debugwin;
extern		int		show;
extern		CSoil   	soil[1000][1000];
extern 		pthread_mutex_t movemutex;              // to protect the move command var

//////////////////////////////////////////////////////////////////////////////////////////////
// Display
// This is the simulation function for the Soil
//////////////////////////////////////////////////////////////////////////////////////////////
void *Display(void * param)
{
int	i,j,m;
int	x,y;
int     c,d;

     c=int(COLS/11);
     d=int(LINES/2);
     x=0; y=0;

     while (1)
     {
          sleep(3);
          // display routine
          // the main menu thread will take keyboard input in the form of arrow keys to window over the
          // entire 1000 x 1000 sim landscape. these commands will be stored in the global vars
          // for window movements. this thread will reset those global vars to unset state.
          for (i=0; i<c; i++) for (j=0; j<d; j++)
          {
               // draw the currently selected matrix every two seconds
               switch (show)
               {
               case SOIL:
                    mvwprintw(mainwin,j,i*11,"%.3d %.3d %.2d ", soil[i+x][j+y].GetNut(),
                                                                soil[i+x][j+y].GetWater(),
                                                                soil[i+x][j+y].GetSun());
                    wrefresh(mainwin);
                    break;

               case PLANT:
               default:
                    wprintw(debugwin,"NOT IMPLEMENTED\n");wrefresh(debugwin);
                    break;
                    }
               }// end for loop
          wrefresh(mainwin);
          // use movecommand to update x,y
          pthread_mutex_lock(&movemutex);
          m=movecommand;
          movecommand=NONE;
          wprintw(debugwin,"Move Reset\n");wrefresh(debugwin);
          pthread_mutex_unlock(&movemutex);
          switch (m)
          {
          case LEFT:
               x=x-1;
               if (x<0) x=0;
               break;

          case RIGHT:
               x=x+1;
               if (x>1000-COLS) x=1000-COLS;
               break;

          case UP:
               y=y-1;
               if (y<0) y=0;
               break;


          case DOWN:
               y=y+1;
               if (y>1000-LINES) y=1000-LINES;
               break;

          default:
               break;
               } //end switch
          }
     } ///////////////////////////////////////////////////////////////////////////////end Life

