//////////////////////////////////////////////////////////////////////////////////////////////
// main.cpp -- a stream socket server demo
//
//
// Life module 
// for World II
// by Eric J Krejci
// Copyright Eric J Krejci February 1998
// Revised : 31 Jan 2000
//////////////////////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////////////////////
// Defines
//////////////////////////////////////////////////////////////////////////////////////////////
#define MYPORT 3490    /* the port users will be connecting to */
#define BACKLOG 10     /* how many pending connections queue will hold */
//#define _REENTRANT

//////////////////////////////////////////////////////////////////////////////////////////////
// Includes
//////////////////////////////////////////////////////////////////////////////////////////////
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>
#include <curses.h>
#include <strings.h>
#include <sys/types.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/socket.h>
#include <sys/wait.h>
#include <pthread.h>

#include "life.h"
#include "csoil.hpp"
#include "cplant.hpp"

//////////////////////////////////////////////////////////////////////////////////////////////
// Globals
//////////////////////////////////////////////////////////////////////////////////////////////
pthread_t	threadlist[5],ttemp;  //list of threads
CSoil 		soil[1000][1000];
CPlant		plant[1000];
int 		show;			// which matrix to show
int		runstate;		// are we running or pausing or what ?

WINDOW *menuwin;        // menu=top left 1/4
WINDOW *debugwin;       // debug= top right 1/4
WINDOW *mainwin;        // main=bottom half

int 		movecommand;		// which way to scroll the window
pthread_mutex_t	movemutex;		// to protect the move command var




//////////////////////////////////////////////////////////////////////////////////////////////
// Init
//////////////////////////////////////////////////////////////////////////////////////////////
void Init()
{
int i,j;

     // init the screens
     initscr();
     cbreak();
     noecho();
     nonl();
     intrflush(stdscr, FALSE);
     keypad(stdscr, TRUE);
     pthread_mutex_init(&movemutex, NULL);
     } ///////////////////////////////////////////////////////////////////////////////end Init

//////////////////////////////////////////////////////////////////////////////////////////////
// DeInit
//////////////////////////////////////////////////////////////////////////////////////////////
void DeInit()
{
     endwin();

     } /////////////////////////////////////////////////////////////////////////////end DeInit

//////////////////////////////////////////////////////////////////////////////////////////////
// Main
//////////////////////////////////////////////////////////////////////////////////////////////
main()
{
long 	         pid;
int              t;	                 //counter for the next thread
pthread_attr_t   attr;                   // thread attr structure for setting attributes.
int		c,i,j;

     Init();
     // erase the screen
     clear();
     refresh();

     // create the windows
     menuwin =newwin(int(LINES/2),int(COLS/2), 0,0);
     debugwin=newwin(int(LINES/2),int(COLS/2),0,int(COLS/2));
     mainwin =newwin(int(LINES/2),COLS,int(LINES/2),0);
     scrollok(menuwin, TRUE);
     scrollok(debugwin,TRUE);
     scrollok(mainwin, TRUE);

     for (i=0; i<1000; i++)
     for (j=0; j<1000; j++)
     {
          soil[i][j].Init();
          }


     pthread_attr_init(&attr);    // initialize attr with default attributes 
     pthread_attr_setscope(&attr, PTHREAD_SCOPE_SYSTEM);  // system-wide contention 

     // start simulation in the Pause astate
     runstate=PAUSE;

     t=0;  // start at thread zero
     pthread_create( &ttemp, &attr, Display, NULL);  
     threadlist[t++]=ttemp;
     wprintw(debugwin,"Created new thread for display. %d\n",ttemp);wrefresh(debugwin);
     
     // create soil thread
     pthread_create( &ttemp, &attr, Soil, NULL);  
     threadlist[t++]=ttemp;
     wprintw(debugwin,"Created new thread for soil. %d\n",ttemp);wrefresh(debugwin);

     do
     {
          mvwprintw(menuwin,0,0,"Options");
          mvwprintw(menuwin,1,0,"'S' = Single Step Sim");
          mvwprintw(menuwin,2,0,"'G' = Go");
          mvwprintw(menuwin,3,0,"'P' = Pause");
          mvwprintw(menuwin,4,0,"'1' = Show Soil Matrix");
          mvwprintw(menuwin,5,0,"'2' = Show Plants");
          mvwprintw(menuwin,6,0,"'3' = Show Herbivores");
          mvwprintw(menuwin,7,0,"'4' = Show Carnivores");
          mvwprintw(menuwin,8,0,"'Z' = End Program");
          wrefresh(menuwin);
          c=getch();

//        wprintw(debugwin,"After getch \n"); wrefresh(debugwin);
//        wprintw(debugwin,"Got %d  \n",c); wrefresh(debugwin);
 
          switch (c) 
          {
          case 'S':
               wprintw(debugwin,"Set single step\n");wrefresh(debugwin);
               runstate=SINGLESTEP;	// the thread sets it to PAUSE again
               break;
          
          case 'G':
               wprintw(debugwin,"Set go\n");wrefresh(debugwin);
               runstate=GO;
               break;

          case 'P':
               wprintw(debugwin,"Set pause\n");wrefresh(debugwin);
               runstate=PAUSE;
               break;

          case '1':
               wprintw(debugwin,"Show Soil\n");wrefresh(debugwin);
               show=SOIL;
               break;

          case '2':
               wprintw(debugwin,"Show Plant \n");wrefresh(debugwin);
               show=PLANT;
               break;

          case '3':
               wprintw(debugwin,"Show Herbivore\n");wrefresh(debugwin);
               show=HERB;
               break;

          case '4':
               wprintw(debugwin,"Show Carnivore\n");wrefresh(debugwin);
               show=CARN;
               break;

          case KEY_RIGHT:
               pthread_mutex_lock(&movemutex);
               movecommand=RIGHT;
               wprintw(debugwin,"Right\n");wrefresh(debugwin);
               pthread_mutex_unlock(&movemutex);
               break;

          case KEY_LEFT:
               pthread_mutex_lock(&movemutex);
               movecommand=LEFT;
               wprintw(debugwin,"Left\n");wrefresh(debugwin);
               pthread_mutex_unlock(&movemutex);
               break;

          case KEY_UP:
               pthread_mutex_lock(&movemutex);
               movecommand=UP;
               wprintw(debugwin,"Up\n");wrefresh(debugwin);
               pthread_mutex_unlock(&movemutex);
               break;

          case KEY_DOWN:
               pthread_mutex_lock(&movemutex);
               movecommand=DOWN;
               wprintw(debugwin,"Down\n");wrefresh(debugwin);
               pthread_mutex_unlock(&movemutex);
               break;


          default :
               beep();                              //("Please try again\n");
               break;
               }

          } while (c!='Z');

     DeInit();

     } ///////////////////////////////////////////////////////////////////////////////end main

