//////////////////////////////////////////////////////////////////////////////////////////////
// cworldtest.cpp 
// 
// This will be a test bed for the new cworld 3d engine. 
//
// World II
//
// by Eric J Krejci
// Copyright (C) 1999 Eric J Krejci 
// Revised : 31 Jan 2006
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation version 2.
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
// Contact:
//              email           worldii@viagloria.com
//              web             www.viagloria.com
//////////////////////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////////////////////
// Defines
//////////////////////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////////////////////
// Includes
//////////////////////////////////////////////////////////////////////////////////////////////
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "cworld.h"
#include "cpart.hpp"
#include "cregion.hpp"
#include "cmodel.hpp"
#include "cworld.hpp"

//////////////////////////////////////////////////////////////////////////////////////////////
// Globals
//////////////////////////////////////////////////////////////////////////////////////////////
bool 	debug=FALSE;

//////////////////////////////////////////////////////////////////////////////////////////////
// Main
//////////////////////////////////////////////////////////////////////////////////////////////
main(int argc, char** argv)
{
CWorld	*world;
CRegion	*r1;
CRegion	*r2;
CRegion	*r3;
CModel	*model1;
CModel	*model2;
CModel	*model3;
CModel	*model4;
CModel	*model5;
CModel	*submodel1;
CModel	*submodel2;
CModel	*submodel3;
CModel	*submodel4;
CModel	*ssubmodel1;
CModel	*ssubmodel2;
CModel	*ssubmodel3;
CModel	*ssubmodel4;
CPart	*part1;
CPart	*part2;
CPart	*part3;
CList	*list1;
CList	*list2;
CList	*list3;
int 	i;

	// find out if we are debugging
	if ((argc==2) && (strcmp(argv[1],"debug")==0))
	{
		printf("DEBUGMODE ON\n");
		debug=TRUE;
	}

	if (debug) 
	{
		for (i=0;i<argc;i++) printf("Argument %d, s='%s' \n",i,argv[i]);

		printf("Hello worldii!\n");
		printf("Allocation CWorld\n");
	}

	world = new CWorld;

	// Make two regions
	r1 = new CRegion;
	r1->SetIndex(1,1,1);
	world->InsertRegion(r1);

	r3 = new CRegion;
	r3->SetIndex(3,3,3);
	world->InsertRegion(r3);
	
	r2 = new CRegion;
	r2->SetIndex(2,2,2);
	world->InsertRegion(r2);

	// Make four models
	model1 = new CModel;
	model1->SetFileName("Model 1");
	world->InsertModel(model1,r1);

	model2 = new CModel;
	model2->SetFileName("Model 2");
	world->InsertModel(model2,r2);

	model3 = new CModel;
	model3->SetFileName("Model 3");
	world->InsertModel(model3,r2);

	model4 = new CModel;
	model4->SetFileName("Model 4");
	world->InsertModel(model4,r2);

	model5 = new CModel;
	model5->SetFileName("Model 5");
	world->InsertModel(model5,r1);

	// Make four submodels
	submodel1 = new CModel;
	submodel1->SetFileName("SubModel 1");
	world->InsertModel(submodel1,model1);

	submodel2 = new CModel;
	submodel2->SetFileName("SubModel 2");
	world->InsertModel(submodel2,model1);

	submodel3 = new CModel;
	submodel3->SetFileName("SubModel 3");
	world->InsertModel(submodel3,model1);

	submodel4 = new CModel;
	submodel4->SetFileName("SubModel 4");
	world->InsertModel(submodel4,model1);

	// Make for more Subsub models
	ssubmodel1 = new CModel;
	ssubmodel1->SetFileName("SSubModel 1");
	world->InsertModel(ssubmodel1,submodel3);

	ssubmodel2 = new CModel;
	ssubmodel2->SetFileName("SSubModel 2");
	world->InsertModel(ssubmodel2,submodel3);

	ssubmodel3 = new CModel;
	ssubmodel3->SetFileName("SSubModel 3");
	world->InsertModel(ssubmodel3,submodel3);

	ssubmodel4 = new CModel;
	ssubmodel4->SetFileName("SSubModel 4");
	world->InsertModel(ssubmodel4,submodel3);

	// Make some parts
	part1 = new CPart;
	part2 = new CPart;
	part3 = new CPart;
	ssubmodel4->InsertPart(part1);
	ssubmodel4->InsertPart(part2);
	ssubmodel4->InsertPart(part3);

	// Make some Lists
	list1 = new CList;
	list2 = new CList;
	list3 = new CList;
	part2->InsertList(list1);
	part2->InsertList(list2);
	part2->InsertList(list3);

	
	// Print it all 
	world->Print();


} //////////////////////////////////////////////////////////////////////////////////////////end main

