///////////////////////////////////////////////////////////////////////////////////////////////////
// cxtest.cpp 
// This is the main module for the cworldtest application.
//
// World II
//
// by Eric J Krejci
// Copyright (C) 1999 Eric J Krejci 
// Revised : 27 May 2007
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation version 2.
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
// Contact:
//              email           worldii@viagloria.com
//              web             www.viagloria.com
///////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////
// Defines
///////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////
// Includes
///////////////////////////////////////////////////////////////////////////////////////////////////
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>
#include <curses.h>
#include <strings.h>
#include <sys/timeb.h>

#include "wiii_curses.h"
#include "wiii_util.h"
#include "wiii_structures.h"
#include "cstring.hpp"
#include "cxitem.hpp"
#include "cengine.hpp"

///////////////////////////////////////////////////////////////////////////////////////////////////
// Globals
///////////////////////////////////////////////////////////////////////////////////////////////////

//WINDOW		*debugwin;
//WINDOW  		*mainwin;
//WINDOW  		*menuwin;
bool 			debug=false;
int			menucontext;
FILE 			*logfile;
char			logfilename[50]="logs/cworldtest.log";
pthread_attr_t		attr;                           // thread attr structure for setting attributes.
pthread_mutexattr_t	m_attr;
int			loglevel;
struct timeb		starttime;
long			logcount;
pthread_mutex_t		screenmutex;
pthread_mutex_t		logmutex;
CXIterator		*list;
WIIILIST		*currentlist;			// points to array of current lists
int			currentlistcount;		// how many lists are there
WIIILIST		current=0;
WIIIHANDLE		currentcursor=0;
CEngine			*engine;
bool			cmdline;			// are we running from the command line, or
							// are we running interactive
bool			exe;				// do we have an execute parm
CString			exefile;

////////////////////////////////////////////////////////////////////////////////////////////////////
// Menu Def
// Menu moves are not actions
////////////////////////////////////////////////////////////////////////////////////////////////////
typedef struct menutype
{
	int	context;	// which sub menu are we in?
	char	option;		// option letter to type in 
	char	*description;	// description of that option
	bool	action;		// Is this option a sub menu or an action, 1=action, 0=submenu ?
	int	next;		// the next menu context
	} menu_t;

#define MAIN_CONTEXT	0   
#define FILE_CONTEXT	1   
#define OBJ_CONTEXT	2   
#define NAV_CONTEXT	3   
#define HELP_CONTEXT	4   
#define NEW_CONTEXT	5   
#define DEL_CONTEXT	6   
#define ASM_CONTEXT	7   
#define COUNT   41
menu_t	menu[]=
{
//	context		opt	description		action		parent/child context
	{MAIN_CONTEXT,	'f',	"File               ",	0,		FILE_CONTEXT},
	{MAIN_CONTEXT,	'o',	"Object Management  ",	0,		OBJ_CONTEXT},
	{MAIN_CONTEXT,	'n',	"Navigation         ",	0,		NAV_CONTEXT},
	{MAIN_CONTEXT,	'a',	"Assemble           ",	0,		ASM_CONTEXT},
	{MAIN_CONTEXT,	'r',	"Run Script         ",	1,		MAIN_CONTEXT},
	{MAIN_CONTEXT,	'h',	"Help               ",	0,		HELP_CONTEXT},
	{MAIN_CONTEXT,	'z',	"Exit Program       ",	1,		MAIN_CONTEXT},

	{FILE_CONTEXT, 	'o',	"Open               ",	1,		MAIN_CONTEXT},
	{FILE_CONTEXT, 	's',	"Save               ",	1,		MAIN_CONTEXT},
	{FILE_CONTEXT, 	'e',	"Exit               ",	1,		MAIN_CONTEXT},
	{FILE_CONTEXT, 	'z',	"Previous Menu      ",	0,		MAIN_CONTEXT},

	{OBJ_CONTEXT, 	'n',	"New                ",	0,		NEW_CONTEXT},
	{OBJ_CONTEXT, 	'd',	"Del                ",	0,		DEL_CONTEXT},
	{OBJ_CONTEXT, 	'm',	"Move               ",	1,		MAIN_CONTEXT},
	{OBJ_CONTEXT, 	'p',	"Print              ",	1,		MAIN_CONTEXT},
	{OBJ_CONTEXT, 	'P',	"Print All          ",	1,		MAIN_CONTEXT},
	{OBJ_CONTEXT, 	'B',	"Print All backward ",	1,		MAIN_CONTEXT},
	{OBJ_CONTEXT, 	's',	"Select Cursor      ",	1,		MAIN_CONTEXT},
	{OBJ_CONTEXT, 	'c',	"Change             ",	1,		MAIN_CONTEXT},
	{OBJ_CONTEXT, 	'z',	"Previous Menu      ",	0,		MAIN_CONTEXT},

	{ASM_CONTEXT, 	'o',	"Open  for assembly ",	1,		MAIN_CONTEXT},
	{ASM_CONTEXT, 	'f',	"Do First Pass      ",	1,		MAIN_CONTEXT},
	{ASM_CONTEXT, 	's',	"Do Second Pass     ",	1,		MAIN_CONTEXT},
	{ASM_CONTEXT, 	'w',	"Write Bin          ",	1,		MAIN_CONTEXT},
	{ASM_CONTEXT, 	'z',	"Previous Menu      ",	0,		MAIN_CONTEXT},

	{NAV_CONTEXT, 	'f',	"First              ",	1,		MAIN_CONTEXT},
	{NAV_CONTEXT, 	'p',	"Prev               ",	1,		MAIN_CONTEXT},
	{NAV_CONTEXT, 	'P',	"Print All          ",	1,		MAIN_CONTEXT},
	{NAV_CONTEXT, 	'n',	"Next               ",	1,		MAIN_CONTEXT},
	{NAV_CONTEXT, 	'l',	"Last               ",	1,		MAIN_CONTEXT},
	{NAV_CONTEXT, 	'i',	"Switch List        ",	1,		MAIN_CONTEXT},
	{NAV_CONTEXT, 	'g',	"Get List Count     ",	1,		MAIN_CONTEXT},
	{NAV_CONTEXT, 	'z',	"Previous Menu      ",	0,		MAIN_CONTEXT},

	{HELP_CONTEXT, 	'a',	"About              ",	1,		MAIN_CONTEXT},
	{HELP_CONTEXT, 	'h',	"Help               ",	1,		MAIN_CONTEXT},
	{HELP_CONTEXT, 	'z',	"Previous Menu      ", 	0,		MAIN_CONTEXT},

	{NEW_CONTEXT, 	'o',	"New Object         ", 	1,		OBJ_CONTEXT},
	{NEW_CONTEXT, 	'c',	"New Cursor         ", 	1,		OBJ_CONTEXT},
	{NEW_CONTEXT, 	'l',	"New Linked List    ", 	1,		OBJ_CONTEXT},
	{NEW_CONTEXT, 	's',	"Select Cursor      ", 	1,		OBJ_CONTEXT},
	{NEW_CONTEXT, 	'z',	"Previous Menu      ", 	0,		OBJ_CONTEXT},

	{DEL_CONTEXT, 	'o',	"Delete Object      ", 	1,		OBJ_CONTEXT},
	{DEL_CONTEXT, 	'c',	"Delete Cursor      ", 	1,		OBJ_CONTEXT},
	{DEL_CONTEXT, 	'l',	"Delete Links       ", 	1,		OBJ_CONTEXT},
	{DEL_CONTEXT, 	'z',	"Previous Menu      ", 	0,		OBJ_CONTEXT},
	};


///////////////////////////////////////////////////////////////////////////////////////////////////
// External Functions
///////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////
// Forward Decs
///////////////////////////////////////////////////////////////////////////////////////////////////
void ShowLists(curswin *p);

////////////////////////////////////////////////////////////////////////////////////////////////////
// ParseCommandLine
////////////////////////////////////////////////////////////////////////////////////////////////////
void ParseCommandLine(int argc, char** argv)
{
int     i;

        // set defaults for parms
        loglevel=0;

        //getting parms
        for (i=0;i<argc;i++)
        {

                // --loglevel how much logging to do
                if ((strncmp(argv[i],"--loglevel",10))==0)
                {
                        printf("DEBUGMODE ON\n");
                        loglevel=atoi(argv[i+1]);
                        i++;
                        }

                // -e script.bin from the command line
                if ((strncmp(argv[i],"-e",2))==0)
                {
                        if (argc>i+1)
                        {
                                exefile.SetString(argv[i+1]);
                                i++;
                                cmdline=true;
                                exe=true;
                                }
                        else
                        {
                                printf("Error not enough parameters\n");
                                exit (1);
                                }
                        }

                } //end for loop

        } //////////////////////////////////////////////////////////////////////end ParseCommandLine

///////////////////////////////////////////////////////////////////////////////////////////////////
// Init
///////////////////////////////////////////////////////////////////////////////////////////////////
void Init(int argc, char** argv)
{
int i;

	// Parse input options
	ParseCommandLine(argc,argv);

        // print out parms if requested
        if (loglevel>0)
        {
        int i;

                for (i=0;i<argc;i++) printf("Argument %d, s='%s' \n",i,argv[i]);
                sleep (2);
                }

        pthread_attr_init(&attr);    // initialize attr with default attributes
        pthread_attr_setscope(&attr, PTHREAD_SCOPE_SYSTEM);  // system-wide contention

        InitMutex(&screenmutex);
        InitMutex(&logmutex);

	} //////////////////////////////////////////////////////////////////////////////////end Init

////////////////////////////////////////////////////////////////////////////////////////////////////
// InitSock
////////////////////////////////////////////////////////////////////////////////////////////////////
void InitSock()
{

     } /////////////////////////////////////////////////////////////////////////////////end InitSock

////////////////////////////////////////////////////////////////////////////////////////////////////
// DeInit
////////////////////////////////////////////////////////////////////////////////////////////////////
void DeInit()
{
     // 

     } ///////////////////////////////////////////////////////////////////////////////////end DeInit

////////////////////////////////////////////////////////////////////////////////////////////////////
// InitScreen
////////////////////////////////////////////////////////////////////////////////////////////////////
void InitScreen(curswin **p)
{
	// init the ncurses library
	initscr();
	cbreak();
	noecho();
	nonl();
	intrflush(stdscr, false);
	keypad(stdscr, true);
	
	// get mem for curswin
	(*p) = new curswin;

	// create the windows
	(*p)->menuwin =newwin(int(LINES/2),int(COLS/2), 0,0);
	(*p)->debugwin=newwin(int(LINES/2),int(COLS/2),0,int(COLS/2));
	(*p)->mainwin =newwin(int(LINES/2),COLS,int(LINES/2),0);
	scrollok((*p)->menuwin, true);
	scrollok((*p)->debugwin,true);
	scrollok((*p)->mainwin, true);
	menucontext=MAIN_CONTEXT;
	} ////////////////////////////////////////////////////////////////////////////end InitScreen

////////////////////////////////////////////////////////////////////////////////////////////////////
// DeInitScreen
////////////////////////////////////////////////////////////////////////////////////////////////////
void DeInitScreen(curswin *p)
{
	// DeInit the ncurses library
	wrefresh(p->menuwin);
	wrefresh(p->debugwin);
	wrefresh(p->mainwin);
	endwin();
	} //////////////////////////////////////////////////////////////////////////end DeInitScreen

////////////////////////////////////////////////////////////////////////////////////////////////////
// PrintMenu
////////////////////////////////////////////////////////////////////////////////////////////////////
char PrintMenu(int *context, curswin *cw)
{
int	i,p;
char	buf[80];
char	c;


PrintMenu0:
	// print menu
	werase(cw->menuwin);
	p=0;
	for (i=0;i<COUNT;i++)
	{
		if (menu[i].context==*context) 
		{
			snprintf(buf,80,"'%c' = %s",menu[i].option,menu[i].description);
			MyLocPrint(cw->menuwin,p,0,buf);
			p++;
			}
		}

	// get choice
	c=(char)getch();

	// check for exit condition
	if ((*context==MAIN_CONTEXT)&&(c=='z')) return ('z');

	// see if we need to change menu context
	for (i=0;i<COUNT;i++)	
	if ((menu[i].context==*context)&&(c==menu[i].option)&&!menu[i].action)
	{
		*context=menu[i].next;
		goto PrintMenu0;
		}

	// return action choice
	return (c);
	
	} /////////////////////////////////////////////////////////////////////////////end Printmenu

////////////////////////////////////////////////////////////////////////////////////////////////////
// GetAnInt
////////////////////////////////////////////////////////////////////////////////////////////////////
int GetAnInt(curswin *p)
{
CString		myinput;
char		c1[2];
char		buf[80];

	c1[0]='\0';
	c1[1]='\0';
	while (c1[0]!=13)
	{
		c1[0]=(char)getch();
		c1[1]='\0';
		MyPrint(p->mainwin,c1,true);
		if (c1[0]!=13) myinput.Append(c1);
		if (debug) snprintf(buf,80,"String is '%s' \n",myinput.GetStringPtr());
		if (debug) MyPrint(p->debugwin,buf,true);
		}
	return (atoi(myinput.GetStringPtr()));
	} //////////////////////////////////////////////////////////////////////////////end GetAnInt

////////////////////////////////////////////////////////////////////////////////////////////////////
// PrintCurr
////////////////////////////////////////////////////////////////////////////////////////////////////
void PrintCurr(curswin *p)
{
char	buf[80];

	MyPrint(p->debugwin,"Print Object \n", true);
//	snprintf(buf,80,"value is %d \n",(list->GetCurrent())->GetVal());
	MyPrint(p->mainwin,buf,true);
	} /////////////////////////////////////////////////////////////////////////////end PrintCurr

////////////////////////////////////////////////////////////////////////////////////////////////////
// PrintAllBackwards
////////////////////////////////////////////////////////////////////////////////////////////////////
void PrintAllBackwards(curswin *p)
{
char	buf[80];

	list->MoveLast(currentcursor);
	while (list->MoreItems(currentcursor))
	{
		snprintf(buf,80,"value is %d \n",(list->GetCurrent(currentcursor))->GetVal());
		MyPrint(p->mainwin,buf,true);
		list->MovePrev(currentcursor);
		}
	} /////////////////////////////////////////////////////////////////////end PrintAllBackwards

////////////////////////////////////////////////////////////////////////////////////////////////////
// PrintAll
////////////////////////////////////////////////////////////////////////////////////////////////////
void PrintAll(curswin *p)
{
char	buf[80];

	list->MoveFirst(currentcursor);
	while (list->MoreItems(currentcursor))
	{
		snprintf(buf,80,"value is %d \n",(list->GetCurrent(currentcursor))->GetVal());
		MyPrint(p->mainwin,buf,true);
		list->MoveNext(currentcursor);
		}
	} //////////////////////////////////////////////////////////////////////////////end PrintAll

////////////////////////////////////////////////////////////////////////////////////////////////////
// CreateObject
////////////////////////////////////////////////////////////////////////////////////////////////////
void CreateObject(curswin *p)
{
CXItem	*item;
int	z;
bool	done=false;
char	buf[80];
char	c1;
			
	// make object
	while (!done)
	{
		MyPrint(p->mainwin,"Please enter the value \n", true);
		z=GetAnInt(p);
		snprintf(buf,80,"Is %d  correct ? (n=no)\n ",z);
		MyPrint(p->mainwin,buf,true);
		c1=(char)getch();
		if (c1!='n')
		{
			item = new CXItem;
			done=true;
			item->SetVal(z);
			if (list->InsertItem(currentcursor,item)==0)
			{
				MyPrint(p->mainwin,"FAILED to insert item.\n",true);
				}
			else MyPrint(p->mainwin,"Inserted item.\n",true);
			}
		}
	} //////////////////////////////////////////////////////////////////////////end CreateObject

////////////////////////////////////////////////////////////////////////////////////////////////////
// CreateCursor
////////////////////////////////////////////////////////////////////////////////////////////////////
void CreateCursor(curswin *p)
{
CXItem		*item;
int		z;
bool		done=false;
char		buf[80];
char		c1;
WIIIHANDLE	curs;

			
	// make object
	while (!done)
	{
		MyPrint(p->mainwin,"Links available\n", true);
		ShowLists(p);
		MyPrint(p->mainwin,"Please enter the list for the new cursor\n", true);
		z=GetAnInt(p);
		snprintf(buf,80,"Is %d  correct ? (n=no)\n",z);
		MyPrint(p->mainwin,buf,true);
		c1=(char)getch();
		if (c1!='n')
		{
			done=true;
			if (list->GetCursor((WIIILIST)z,(unsigned long)0,&curs)==ERR_NOERROR)
			{
				snprintf(buf,80,"New cursor is %d \n",curs);
				MyPrint(p->mainwin,buf,true);
				}
			else MyPrint(p->mainwin,"Create cursor failed\n",true);
			}
		}
	} //////////////////////////////////////////////////////////////////////////end CreateCursor

////////////////////////////////////////////////////////////////////////////////////////////////////
// DeleteCursor
////////////////////////////////////////////////////////////////////////////////////////////////////
void DeleteCursor(curswin *p)
{
CXItem	*item;
int	z,res;
bool	done=false;
char	buf[80];
char	c1;
			
	// make object
	while (!done)
	{
		MyPrint(p->mainwin,"Please enter the cursor to delete\n", true);
		z=GetAnInt(p);
		snprintf(buf,80,"Is %d  correct ? (n=no)\n",z);
		MyPrint(p->mainwin,buf,true);
		c1=(char)getch();
		if (c1!='n')
		{
			done=true;
			res=list->ReleaseCursor((WIIILIST)current,(unsigned long)0);
			snprintf(buf,80,"Result is %d  \n",res);
			MyPrint(p->mainwin,buf,true);
			}
		}
	} //////////////////////////////////////////////////////////////////////////end DeleteCursor

////////////////////////////////////////////////////////////////////////////////////////////////////
// GetNewLink
////////////////////////////////////////////////////////////////////////////////////////////////////
void GetNewLink(curswin *p)
{

CXItem	*item;
int	z;
bool	done=false;
char	buf[80];
char	c1;
			
	// make object
	while (!done)
	{
		// Print out existing lists
		MyPrint(p->mainwin,"Please enter the new link. \n", true);
		z=GetAnInt(p);
		snprintf(buf,80,"Is %d  correct ? (n=no)\n",z);
		MyPrint(p->mainwin,buf,true);
		c1=(char)getch();
		if (c1!='n')
		{
			item = new CXItem;
			done=true;
			item->SetVal(z);
//			list->InsertItem(item);
			MyPrint(p->mainwin,"Inserted item.\n",true);
			}
		}

	} ////////////////////////////////////////////////////////////////////////////end GetNewLink

////////////////////////////////////////////////////////////////////////////////////////////////////
// SelectCursor
////////////////////////////////////////////////////////////////////////////////////////////////////
void SelectCursor(curswin *p)
{

CXItem	*item;
int	z;
bool	done=false;
char	buf[80];
char	c1;
			
	// make object
	while (!done)
	{
		// Print out existing lists
		MyPrint(p->mainwin,"Please enter the new current cursor. \n", true);
		z=GetAnInt(p);
		snprintf(buf,80,"Is %d  correct ? (n=no)\n",z);
		MyPrint(p->mainwin,buf,true);
		c1=(char)getch();
		if (c1!='n')
		{
			done=true;
			currentcursor=z;
			}
		}

	} //////////////////////////////////////////////////////////////////////////end SelectCursor

////////////////////////////////////////////////////////////////////////////////////////////////////
// DeleteLink
////////////////////////////////////////////////////////////////////////////////////////////////////
bool DeleteLink(curswin *p)
{

CXItem	*item;
int	z;
bool	done=false;
char	buf[80];
char	c1;
			
	// make object
	while (!done)
	{
		// Print out existing lists
		MyPrint(p->mainwin,"Please enter the link to delete. \n", true);
		z=GetAnInt(p);
		snprintf(buf,80,"Is %d  correct ? (n=no)\n",z);
		MyPrint(p->mainwin,buf,true);
		c1=(char)getch();
		if (c1!='n')
		{
			done=true;
			return (list->DelLinkedList((WIIILIST)z));
			}
		}

	} ////////////////////////////////////////////////////////////////////////////end DeleteLink

////////////////////////////////////////////////////////////////////////////////////////////////////
// ChangeList
////////////////////////////////////////////////////////////////////////////////////////////////////
void ChangeList(curswin *p)
{

CXItem	*item;
int	z;
bool	done=false;
char	buf[80];
char	c1;
			
	// make object
	while (!done)
	{
		// Print out existing lists
		MyPrint(p->mainwin,"Please enter the desired list. \n", true);
		z=GetAnInt(p);
		snprintf(buf,80,"Is %d  correct ? (n=no)\n",z);
		MyPrint(p->mainwin,buf,true);
		c1=(char)getch();
		if (c1!='n')
		{
			done=true;
			current=(WIIILIST)z;
			}
		}

	} ///////////////////////////////////////////////////////////////////////////end ChangeList

////////////////////////////////////////////////////////////////////////////////////////////////////
// ShowLists
// This will first get the count of lists
// Then it will get the array containing the handles to currently used lists.
////////////////////////////////////////////////////////////////////////////////////////////////////
void ShowLists(curswin *p)
{
char	buf[80];
int	i;

	// get list count
	snprintf(buf,80,"GetCountOfLinkedLists count= %d\n",list->GetCountOfLinkedLists());
	MyPrint(p->mainwin,buf,true);
	
	// get list count and current list array
	// remember that we have to delete it when done.
	list->GetLinkedLists(&currentlist,&currentlistcount);
	snprintf(buf,80,"GetLinkedLists array count result= %d\n",currentlistcount);
	MyPrint(p->mainwin,buf,true);
	
	// now print lists in existence
	for (i=0; i<currentlistcount; i++)
	{
		snprintf(buf,80,"Array index= %d  List handle = %d\n",i,currentlist[i]);
		MyPrint(p->mainwin,buf,true);
		}
	} /////////////////////////////////////////////////////////////////////////////end ShowLists

////////////////////////////////////////////////////////////////////////////////////////////////////
// CEPrintInt
// CEngine external function
// Takes single int
// No returns
// Prints integer
////////////////////////////////////////////////////////////////////////////////////////////////////
void CEPrintInt(parmstruct *p)
{
char	buf[80];
curswin	*cw;

	// These are simple external routines for the scripting engine
	cw=(curswin*)(p->extra);
	snprintf(buf,80,"Print from script = %d\n",p->i[0]);
	MyPrint(cw->mainwin,buf,true);
	} ////////////////////////////////////////////////////////////////////////////end CEPrintInt

////////////////////////////////////////////////////////////////////////////////////////////////////
// CEPrintCString
// CEngine external function
// Takes single int
// No returns
// Prints integer
////////////////////////////////////////////////////////////////////////////////////////////////////
void CEPrintCString(parmstruct *p)
{
char	buf[80];
curswin	*cw;

	// These are simple external routines for the scripting engine
	cw=(curswin*)(p->extra);
	snprintf(buf,80,"Print from script = '%s'\n",p->s[0].GetStringPtr());
	MyPrint(cw->mainwin,buf,true);
	} ////////////////////////////////////////////////////////////////////////end CEPrintCString

////////////////////////////////////////////////////////////////////////////////////////////////////
// CEPrintf
// CEngine external function
// Takes single CString
// No returns
// Prints CString
////////////////////////////////////////////////////////////////////////////////////////////////////
void CEPrintf(parmstruct *p)
{
char    buf[80];

	// These are simple external routines for the scripting engine
	snprintf(buf,80,"%s",p->s[0].GetStringPtr());
	printf(buf);
	} //////////////////////////////////////////////////////////////////////////////end CEPrintf

////////////////////////////////////////////////////////////////////////////////////////////////////
// CEPrintfU
// CEngine external function
// Takes single uint
// No returns
// Prints CString
////////////////////////////////////////////////////////////////////////////////////////////////////
void CEPrintfU(parmstruct *p)
{
char    buf[80];

	// These are simple external routines for the scripting engine
	snprintf(buf,80,"%lu\n",p->u[0]);
	printf(buf);
	} /////////////////////////////////////////////////////////////////////////////end CEPrintfU

////////////////////////////////////////////////////////////////////////////////////////////////////
// CEPrintfI
// CEngine external function
// Takes single int
// No returns
// Prints CString
////////////////////////////////////////////////////////////////////////////////////////////////////
void CEPrintfI(parmstruct *p)
{
char    buf[80];

	// These are simple external routines for the scripting engine
	snprintf(buf,80,"%li\n",p->i[0]);
	printf(buf);
	} /////////////////////////////////////////////////////////////////////////////end CEPrintfI

////////////////////////////////////////////////////////////////////////////////////////////////////
// CECreateLinks
// CEngine external function
// Takes noinput
// Returns Single int and single bool
// If bool==true then int contains handle
////////////////////////////////////////////////////////////////////////////////////////////////////
void CECreateLinks(parmstruct *p)
{
WIIILIST newlist;

	// Create the link and pass back the value
	if ((list->AddLinkedList(&newlist))==ERR_NOERROR)
	{
		p->b[0]=true;
		p->i[0]=newlist;
		}
	else
	{
		p->b[0]=false;
		p->i[0]=0;
		}
	} /////////////////////////////////////////////////////////////////////////end CECreateLinks

////////////////////////////////////////////////////////////////////////////////////////////////////
// CEShowLinks
// CEngine external function
// Takes noinput
// Returns no value 
////////////////////////////////////////////////////////////////////////////////////////////////////
void CEShowLinks(parmstruct *p)
{
char	buf[80];
int	i;

	// get list count
	snprintf(buf,80,"GetCountOfLinkedLists count= %d\n",list->GetCountOfLinkedLists());
	printf(buf);
	
	// get list count and current list array
	// remember that we have to delete it when done.
	list->GetLinkedLists(&currentlist,&currentlistcount);
	snprintf(buf,80,"GetLinkedLists array count result= %d\n",currentlistcount);
	printf(buf);
	
	// now print lists in existence
	for (i=0; i<currentlistcount; i++)
	{
		snprintf(buf,80,"Array Index=%d List handle = %d\n",i,currentlist[i]);
		printf(buf);
		}
	} ///////////////////////////////////////////////////////////////////////////end CEShowLinks

////////////////////////////////////////////////////////////////////////////////////////////////////
// CEMovePrev
// CEngine external function
// Takes one int as input:	Which cursor
// Returns Single int:		Pointer to item
////////////////////////////////////////////////////////////////////////////////////////////////////
void CEMovePrev(parmstruct *p)
{
int		z;
unsigned long	result;
CString		Log;
char		buf[80];
	
	// get the cursor to move 
	Log="CEMovePrev: Entered \n"; Log.Log();
	z=p->i[0];

	snprintf(buf,80,"%d\n",z);
	Log="CEMovePrev: List=";Log=Log+buf;Log.Log();

	// do the move
	result=(unsigned long)(list->MovePrev((WIIIHANDLE)z));
	p->u[0]=result;

	Log="CEMovePrev: Exited \n"; Log.Log();
	} ////////////////////////////////////////////////////////////////////////////end CEMovePrev

////////////////////////////////////////////////////////////////////////////////////////////////////
// CEMoveNext
// CEngine external function
// Takes one int as input:	Which cursor
// Returns Single int:		Pointer to item
////////////////////////////////////////////////////////////////////////////////////////////////////
void CEMoveNext(parmstruct *p)
{
int		z;
unsigned long	result;
CString		Log;
char		buf[80];
	
	// get the cursor to move 
	Log="CEMoveNext: Entered \n"; Log.Log();
	z=p->i[0];

	snprintf(buf,80,"%d\n",z);
	Log="CEMoveNext: List=";Log=Log+buf;Log.Log();

	// do the move
	result=(unsigned long)(list->MoveNext((WIIIHANDLE)z));
	p->u[0]=result;

	Log="CEMoveNext: Exited \n"; Log.Log();
	} ////////////////////////////////////////////////////////////////////////////end CEMoveNext

////////////////////////////////////////////////////////////////////////////////////////////////////
// CECreateCursor
// CEngine external function
// Takes one int as input
// Returns Single int and single bool
// bool==true means create cursor, false means it failed
////////////////////////////////////////////////////////////////////////////////////////////////////
void CECreateCursor(parmstruct *p)
{
int		i,z;
WIIIHANDLE	curs;
CString		Log;
char		buf[80];

	// get the list to create cursor in
	Log="CECreateCursor: Entered \n"; Log.Log();
	z=p->i[0];

	snprintf(buf,80,"%d\n",z);
	Log="CECreateCursor: List=";Log=Log+buf;Log.Log();
	// get the cursor
	if (list->GetCursor((WIIILIST)z,(unsigned long)0,&curs)==ERR_NOERROR)
	{
		// if it worked return ok
		Log="CECreateCursor: Got OK return \n"; Log.Log();
		p->b[0]=true;
		p->i[0]=curs;
		}
	else 
	{
		// else return not ok
		Log="CECreateCursor: Got Failed return \n"; Log.Log();
		p->b[0]=false;
		p->i[0]=0;
		}

	Log="CECreateCursor: Exited \n"; Log.Log();
	} ////////////////////////////////////////////////////////////////////////end CECreateCursor

////////////////////////////////////////////////////////////////////////////////////////////////////
// CEMoveFirst
// CEngine external function
// Takes two int as input
// i[0]=which curs
////////////////////////////////////////////////////////////////////////////////////////////////////
void CEMoveFirst(parmstruct *p)
{
WIIIHANDLE	curs;
unsigned long	res;

	curs=(WIIIHANDLE)(p->i[0]);
	res=(unsigned long)(list->MoveFirst(curs));
	} ///////////////////////////////////////////////////////////////////////////end CEMoveFirst

////////////////////////////////////////////////////////////////////////////////////////////////////
// CEPrintList
// CEngine external function
// Takes two int as input
// i[0]=which curs
////////////////////////////////////////////////////////////////////////////////////////////////////
void CEPrintList(parmstruct *p)
{
WIIIHANDLE	curs;
char		buf[80];
unsigned long	res;

	curs=(WIIIHANDLE)(p->i[0]);
	res=(unsigned long)(list->MoveFirst(curs));
	snprintf(buf,80,"res=%lu curs=%d\n",res,curs);
//	printf(buf);
	while (list->MoreItems(curs))
	{
		snprintf(buf,80,"value is %d \n",(list->GetCurrent(curs))->GetVal());
		printf(buf);
		res=(unsigned long)(list->MoveNext(curs));
		snprintf(buf,80,"res= is %lu \n",res);
//		printf(buf);
		
		}
	} ///////////////////////////////////////////////////////////////////////////end CEPrintList

////////////////////////////////////////////////////////////////////////////////////////////////////
// CEDeleteObject
// CEngine external function
// Takes one int as input
// i[0]=which curs points to delete
// Returns Single bool
// bool==true means delete object worked, false means it failed
////////////////////////////////////////////////////////////////////////////////////////////////////
void CEDeleteObject(parmstruct *p)
{
WIIIHANDLE	curs;
unsigned long	res;
char		buf[80];

	curs=(WIIIHANDLE)(p->i[0]);
	snprintf(buf,80,"CEDeleteItem curs=%d\n",curs);
//	printf(buf);
	res=(unsigned long)(list->DeleteItem(curs));
	} ////////////////////////////////////////////////////////////////////////end CEDeleteObject

////////////////////////////////////////////////////////////////////////////////////////////////////
// CECreateObject
// CEngine external function
// Takes two int as input
// i[0]=which curs, i[1]=what val
// Returns Single bool
// bool==true means create object worked, false means it failed
////////////////////////////////////////////////////////////////////////////////////////////////////
void CECreateObject(parmstruct *p)
{
int		val;
CString		Log;
WIIIHANDLE	curs;
CXItem		*item;

	// get the cursor to create object with, and value to assign to object
	Log="CECreateObject: Entered \n"; Log.Log();
	curs=(WIIIHANDLE)(p->i[0]);
	val=p->i[1];

	// create object
	item = new CXItem;
	item->SetVal(val);
	if (list->InsertItem(curs,item)==0)
	{
		Log="CECreateObject: FAILED TO INSERT \n"; Log.Log();
		p->b[0]=false;
		}
	else 
	{
		Log="CECreateObject: Insert OK \n"; Log.Log();
		p->b[0]=true;
		}

	Log="CECreateObject: Exited \n"; Log.Log();
	} ////////////////////////////////////////////////////////////////////////end CECreateObject

////////////////////////////////////////////////////////////////////////////////////////////////////
// TestFile
////////////////////////////////////////////////////////////////////////////////////////////////////
void TestFile(int menucontext, char c)
{
	// ok read the file until no more input and execute the test.
	
	} //////////////////////////////////////////////////////////////////////////////end TestFile

////////////////////////////////////////////////////////////////////////////////////////////////////
// LoadFile
////////////////////////////////////////////////////////////////////////////////////////////////////
//bool LoadFile(CString *filename, curswin *p)
//bool LoadFile(CString *filename)
//{
//
//	
//	} //////////////////////////////////////////////////////////////////////////////end LoadFile

////////////////////////////////////////////////////////////////////////////////////////////////////
// ExecComm
////////////////////////////////////////////////////////////////////////////////////////////////////
void ExecComm(int menucontext, char c, curswin *p)
{
char testmsg[]="This is a message from console. Hello World.\n";
char msg[sizeof(int)];
int  l,m;
int  count,code,result;
bool timedout;
CString		myinput;

	switch (menucontext)
	{
	case MAIN_CONTEXT: 
		switch (c)
		{
		case 'r':
			// this will run the script file. 
			MyPrint(p->debugwin,"Run Script File \n", true);
			engine->Run();
			break;
			}

		break;

	case ASM_CONTEXT: 
		switch (c)
		{
		case 'o':
			{
			CString	*filename;
				// this will be a script file.	it will run in the script engine and 
				// will exercise the cxitem infrastructure.
				ThreadLog("ExecComm: Init the script engine Again.");	
				engine->Init();
				MyPrint(p->debugwin,"Open Script File \n", true);
				filename=GetAFileName(p);
//				engine->Assemble(CE_ASM_OPEN,filename);
				result=engine->Assemble(CE_ASM_OPEN,filename,cmdline);
				}
			break;

		case 'f':
			// this will run the first pass of the assembler. 
			MyPrint(p->debugwin,"Run First Pass \n", true);
//			result=engine->Assemble(CE_ASM_FIRST,0);
			result=engine->Assemble(CE_ASM_FIRST,0,cmdline);
			if (result==CE_ERR_BADFILE)
			{		
				MyPrint(p->debugwin,"Assembly Error: Couldn't open file \n", true);
				}
			if (result==CE_ERR_NOINST)
			{		
				MyPrint(p->debugwin,"Assembly Error: Expected instruction \n", true);
				}
			if (result==CE_ERR_NOERR)
			{		
				MyPrint(p->debugwin,"First Pass Assembly Complete \n", true);
				}
			break;

		case 's':
			// this will run the second pass of the assembler. 
			MyPrint(p->debugwin,"Run Second Pass \n", true);
			break;

		case 'w':
			// this will write the bin script. 
			MyPrint(p->debugwin,"Write Bin \n", true);
			break;
			}

		break;





	case FILE_CONTEXT: 
		switch (c)
		{
		case 'o':
			{
			CString	*filename;
				// this will be a script file.	it will run in the script engine and 
				// will exercise the cxitem infrastructure.
				MyPrint(p->debugwin,"Open Script ABin File \n", true);
				filename=GetAFileName(p);
//				LoadFile(filename,p);
				LoadFile(filename,engine);
				}
			break;

		case 's':
			MyPrint(p->debugwin,"Save File \n", true);
			break;

		default:
			beep();                              //("Please try again\n");
			break;
		} // end FILE_CONTEXT
		break;

	case NAV_CONTEXT: 
		switch (c)
		{
		case 'f':
			MyPrint(p->debugwin,"Move First  \n", true);
//			list->MoveFirst();
			break;

		case 'P':
			MyPrint(p->debugwin,"Print All Objects \n", true);
			PrintAll(p);
			break;
		case 'p':
			MyPrint(p->debugwin,"Move Prev \n", true);
//			list->MovePrev();
			break;

		case 'n':
			MyPrint(p->debugwin,"Move Next \n", true);
//			list->MoveNext();
			break;

		case 'l':
			MyPrint(p->debugwin,"Move Last \n", true);
//			list->MoveLast();
			break;

		case 'i':
			MyPrint(p->debugwin,"Change List \n", true);
			ChangeList(p);
			break;

		case 'g':
			MyPrint(p->debugwin,"Get Lists Count \n", true);
			ShowLists(p);
			break;

		default:
			beep();                              //("Please try again\n");
			break;
		} // end NAV_CONTEXT
		break;

	case OBJ_CONTEXT: 
		switch (c)
		{
		case 'd':
			MyPrint(p->debugwin,"Del Current Object \n", true);
//			list->DeleteItem(list->GetCurrent());
			break;

		case 'm':
			MyPrint(p->debugwin,"Move Object \n", true);
			break;

		case 'p':
			break;

		case 'B':
			MyPrint(p->debugwin,"Print All Objects backwards\n", true);
			PrintAllBackwards(p);
			break;

		case 'P':
			MyPrint(p->debugwin,"Print All Objects \n", true);
			PrintAll(p);
			break;

		case 's':
			MyPrint(p->debugwin,"Select Cursor \n", true);
			SelectCursor(p);
			break;

		case 'c':
			MyPrint(p->debugwin,"Change Object \n", true);
			break;

		default:
			beep();                              //("Please try again\n");
			break;
		} // OBJ_CONTEXT
		break;

	case HELP_CONTEXT:
		switch (c)
		{
		case 'a':
			MyPrint(p->debugwin,"About \n", true);
			break;

		case 'h':

			MyPrint(p->debugwin,"Help \n", true);
			break;

		default:
			beep();                              //("Please try again\n");
			break;
		} // HELP_CONTEXT
		break;

	case NEW_CONTEXT: 
		switch (c)
		{
		case 'o':
			MyPrint(p->debugwin,"New Object \n", true);
			CreateObject(p);
			break;

		case 's':
			MyPrint(p->debugwin,"Select Cursor \n", true);
			SelectCursor(p);
			break;

		case 'c':
			MyPrint(p->debugwin,"New Cursor \n", true);
			CreateCursor(p);
			break;

		case 'l':
			{
			char	buf[80];
			WIIILIST newlist;
				MyPrint(p->debugwin,"New Links \n", true);
				if ((list->AddLinkedList(&newlist))==ERR_NOERROR)
				{
	
					snprintf(buf,80,"New link index is %d",newlist);
					MyPrint(p->debugwin,"Add links OK \n", true);
					}
				else
				{
					MyPrint(p->debugwin,"Add links failed \n", true);
					}
				}
			break;

		default:
			beep();                              //("Please try again\n");
			break;
			} // end NEW_CONTEXT
		break;

	case DEL_CONTEXT: 
		switch (c)
		{
		case 'o':
			MyPrint(p->debugwin,"Delete Object \n", true);
			break;

		case 'c':
			MyPrint(p->debugwin,"New Cursor \n", true);
			DeleteCursor(p);
			break;

		case 'l':
			MyPrint(p->debugwin,"Delete Links \n", true);
			if (DeleteLink(p))  MyPrint(p->mainwin,"Deleted OK\n",true); 
			else MyPrint(p->mainwin,"Delete failed\n",true); 
			break;
		default:
			beep();                              //("Please try again\n");
			break;
			} // end DEL_CONTEXT

	break;

		}
	} //////////////////////////////////////////////////////////////////////////////end ExecComm

////////////////////////////////////////////////////////////////////////////////////////////////////
// CmdLine
////////////////////////////////////////////////////////////////////////////////////////////////////
int CmdLineExe()
{
bool            bresult;
int             result;
curswin         *winp;
CString         Log;

	// get an engine
	Log="CmdLineExe: Get an engine \n"; Log.Log();
	printf(Log.GetStringPtr());
	engine = new CEngine;
	engine->Init();
	engine->SetLogFunc(ThreadLog);

	// we arent going to use curses here so zero it out
	winp=0;

	// execute a script and exit
	Log="CmdLineExe: Set CEngine External Callbacks \n"; Log.Log();
	printf(Log.GetStringPtr());
	
	//				 b,i,u,d,s  b,i,u,d,s, opc,        extra
//	engine->Register(CEPrintInt,     0,1,0,0,0, 0,0,0,0,0, 1000,(void*)winp);
//	engine->Register(CEPrintCString, 0,0,0,0,1, 0,0,0,0,0, 1001,(void*)winp);
	engine->Register(CEPrintfI,      0,1,0,0,0, 0,0,0,0,0, 1002,(void*)winp);
	engine->Register(CEPrintf,       0,0,0,0,1, 0,0,0,0,0, 1003,(void*)winp);
	engine->Register(CEPrintfU,      0,0,1,0,0, 0,0,0,0,0, 1004,(void*)winp);
	engine->Register(CECreateLinks,  0,0,0,0,0, 1,1,0,0,0, 1010,(void*)winp);
	engine->Register(CEShowLinks,    0,0,0,0,0, 0,0,0,0,0, 1011,(void*)winp);
	engine->Register(CECreateCursor, 0,1,0,0,0, 1,1,0,0,0, 1012,(void*)winp);
	engine->Register(CECreateObject, 0,2,0,0,0, 1,0,0,0,0, 1021,(void*)winp);
	engine->Register(CEDeleteObject, 0,1,0,0,0, 0,0,0,0,0, 1022,(void*)winp);
	engine->Register(CEPrintList,    0,1,0,0,0, 0,0,0,0,0, 1031,(void*)winp);
	engine->Register(CEMoveFirst,    0,1,0,0,0, 0,0,0,0,0, 1041,(void*)winp);
	engine->Register(CEMoveNext,     0,1,0,0,0, 0,0,1,0,0, 1042,(void*)winp);
	engine->Register(CEMovePrev,     0,1,0,0,0, 0,0,1,0,0, 1043,(void*)winp);

	// create the iterator to use
	list = new CXIterator;

	// load the script mentioned on command line
	Log="CmdLineExe: Loadfile trying to load '"+exefile+"'\n"; Log.Log();
	printf(Log.GetStringPtr());
	bresult=LoadFile(&exefile,engine);
	if (bresult)
	{

		Log="CmdLineExe: Run Script \n"; Log.Log();
		printf(Log.GetStringPtr());
		result=engine->Run();
		if (result!=CE_ERR_NOERR)
		{
		char	buf[BUFFSIZE];
			Log="CmdLineExe: CEngine Error \n"; Log.Log();
			snprintf(buf,BUFFSIZE,"Engine error: %d\n",result);
			printf(buf);
			return(1);
			}
		}
	else
	{
		Log="CmdLineExe: Loadfile failed \n"; Log.Log();
		return(1);
		}

	Log="CmdLineExe: CEngine OK \n"; Log.Log();
	printf(Log.GetStringPtr());
	return(0);
	} ////////////////////////////////////////////////////////////////////////////end CmdLineExe

////////////////////////////////////////////////////////////////////////////////////////////////////
// CmdLine
////////////////////////////////////////////////////////////////////////////////////////////////////
int CmdLine()
{
CString         Log;

	if (exe)
	{
		Log="CmdLine: Calling CmdLineExe\n"; Log.Log();
		return(CmdLineExe());
		}

	return (0);
	} ///////////////////////////////////////////////////////////////////////////////end CmdLine

////////////////////////////////////////////////////////////////////////////////////////////////////
// Main
////////////////////////////////////////////////////////////////////////////////////////////////////
int main(int argc, char** argv)
{
char		choice;
void            *param;
int		context;
curswin		*winp;
int		ret;		// return to shell the results of the cmmdline run
CString		StaticInit;

	// init the screen & then threading and such
	// erase the screen
	Init(argc, argv);		// Init debug model

	// Set up CString Logging
        StaticInit.SetLogFunc(ThreadLog);


        // Do they want to run on commandline ?
        if (cmdline)
        {
                ret=CmdLine();
                exit(ret);
                }

	// OK interactive so init the screen
	InitScreen(&winp);	// Init the curses library & screen mutex

	// Init the screen
	clear();
	refresh();

//	world = new CWorld;
	list = new CXIterator;

	// init the script engine
	ThreadLog("Main: Init the script engine.");	
	engine = new CEngine;
	engine->Init();
	engine->SetLogFunc(ThreadLog);
	//				 b,i,u,d,s  b,i,u,d,s, opc,        extra
	engine->Register(CEPrintInt,     0,1,0,0,0, 0,0,0,0,0, 1000,(void*)winp);
	engine->Register(CEPrintCString, 0,0,0,1,0, 0,0,0,0,0, 1001,(void*)winp);
	engine->Register(CEPrintfI,      0,1,0,0,0, 0,0,0,0,0, 1002,(void*)winp);
	engine->Register(CEPrintf,       0,0,0,0,1, 0,0,0,0,0, 1003,(void*)winp);
	engine->Register(CEPrintfU,      0,0,1,0,0, 0,0,0,0,0, 1004,(void*)winp);
	engine->Register(CECreateLinks,  0,0,0,0,0, 1,1,0,0,0, 1010,(void*)winp);
	engine->Register(CEShowLinks,    0,0,0,0,0, 0,0,0,0,0, 1011,(void*)winp);

	// Start the listener
	// pack any params for the listener thread
	// tlist is a global thread tracking structure
	//c[0]=' ';
	context=MAIN_CONTEXT;
	do  
	{
		choice=PrintMenu(&context,winp);	// this will update context as required
		ExecComm(context,choice,winp);
		} while (choice!='z');
	DeInit();
	MyPrint(winp->debugwin,"Before DeInitScreen \n", true);
	DeInitScreen(winp);
	printf("Returning from main.\n");
	} //////////////////////////////////////////////////////////////////////////////////end main

