///////////////////////////////////////////////////////////////////////////////////////////////////
// engtest.cpp 
// This is the main module for the engtest application.
//
// World II
//
// by Eric J Krejci
// Copyright (C) 1999 Eric J Krejci 
// Revised : 27 May 2007
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation version 2.
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
// Contact:
//              email           worldii@viagloria.com
//              web             www.viagloria.com
///////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////
// Defines
///////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////
// Includes
///////////////////////////////////////////////////////////////////////////////////////////////////
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>
#include <curses.h>
#include <strings.h>
#include <sys/timeb.h>

//#include "wiii_types.h"
#include "wiii_curses.h"
#include "worldiii.h"
#include "wiii_util.h"
#include "wiii_structures.h"
#include "cengine.hpp"
#include "engtest.h"


///////////////////////////////////////////////////////////////////////////////////////////////////
// Globals
///////////////////////////////////////////////////////////////////////////////////////////////////

WINDOW			*debugwin;
WINDOW  		*mainwin;
WINDOW  		*menuwin;
bool 			debug=FALSE;
int			menucontext;
FILE 			*logfile;
char			logfilename[50]="logs/engtest.log";
CString			*scriptfilename;
long			logcount;
CEngine 	 	*engine;
struct timeb		starttime;
pthread_mutex_t		logmutex;
pthread_mutex_t         screenmutex;
int			loglevel;		// sets level of logging 0 lowest= no logging,
						// 9 highest insanely verbose


///////////////////////////////////////////////////////////////////////////////////////////////////
// External Functions
///////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////
// Init
///////////////////////////////////////////////////////////////////////////////////////////////////
void Init(int argc, char** argv)
{
int i;

     // init the threading model 
     logfile=fopen(logfilename,"w");
     fclose(logfile);

	// find out if we are debugging
	if ((argc==2) && (strcmp(argv[1],"debug")==0))
	{
		printf("DEBUGMODE ON\n");
		debug=TRUE;
	}

	if (debug) 
	{
		for (i=0;i<argc;i++) printf("Argument %d, s='%s' \n",i,argv[i]);

		printf("Hello worldii!\n");
		printf("Allocation CEngine\n");
	}
	
} /////////////////////////////////////////////////////////////////////////////////////////end Init

///////////////////////////////////////////////////////////////////////////////////////////////////
// InitSock
///////////////////////////////////////////////////////////////////////////////////////////////////
void InitSock()
{

     } ////////////////////////////////////////////////////////////////////////////////end InitSock

///////////////////////////////////////////////////////////////////////////////////////////////////
// DeInit
///////////////////////////////////////////////////////////////////////////////////////////////////
void DeInit()
{
     // 

     } /////////////////////////////////////////////////////////////////////////////end DeInit

///////////////////////////////////////////////////////////////////////////////////////////////////
// InitScreen
///////////////////////////////////////////////////////////////////////////////////////////////////
void InitScreen(curswin **p)
{
     // init the ncurses library
	initscr();
	cbreak();
	noecho();
	nonl();
	intrflush(stdscr, false);
	keypad(stdscr, true);

        // get mem for curswin
        (*p) = new curswin;

	// create the windows
	(*p)->menuwin =newwin(int(LINES/2),int(COLS/2), 0,0);
	(*p)->debugwin=newwin(int(LINES/2),int(COLS/2),0,int(COLS/2));
	(*p)->mainwin =newwin(int(LINES/2),COLS,int(LINES/2),0);
	scrollok(menuwin, true);
	scrollok(debugwin,true);
	scrollok(mainwin, true);
	menucontext=MAIN_CONTEXT;
} ////////////////////////////////////////////////////////////////////////////////////end InitScreen

////////////////////////////////////////////////////////////////////////////////////////////////////
// DeInitScreen
////////////////////////////////////////////////////////////////////////////////////////////////////
void DeInitScreen()
{
	// DeInit the ncurses library
	wrefresh(menuwin);
	wrefresh(debugwin);
	wrefresh(mainwin);
	endwin();
} /////////////////////////////////////////////////////////////////////////////////end DeInitScreen

///////////////////////////////////////////////////////////////////////////////////////////////////
// Menu
///////////////////////////////////////////////////////////////////////////////////////////////////
void PrintMenu()
{
	switch (menucontext)
	{
	case MAIN_CONTEXT:
		if (debug)  MyLocPrint(menuwin,0,0,"Main Debug Mode ON               ");
		if (!debug) MyLocPrint(menuwin,0,0,"Main                             ");
		MyLocPrint(menuwin,1,0,"'t' = Test Script                ");
		MyLocPrint(menuwin,2,0,"'e' = Edit script                ");
		MyLocPrint(menuwin,3,0,"'l' = Load script data file      ");
		MyLocPrint(menuwin,4,0,"                                 ");
		MyLocPrint(menuwin,5,0,"                                 ");
		MyLocPrint(menuwin,6,0,"                                 ");
		MyLocPrint(menuwin,7,0,"                                 ");
		MyLocPrint(menuwin,8,0,"'z' = Exit console.              ");
		break;

	case TEST_CONTEXT:
		MyLocPrint(menuwin,0,0,"Test Script                      ");
		MyLocPrint(menuwin,1,0,"'s' = Single step                ");
		MyLocPrint(menuwin,2,0,"'r' = Run script                 ");
		MyLocPrint(menuwin,3,0,"'d' = Display vars, flags        ");
		MyLocPrint(menuwin,4,0,"'R' = Reset                      ");
		MyLocPrint(menuwin,5,0,"'p' = Set PC                     ");
		MyLocPrint(menuwin,6,0,"'l' = List script                ");
		MyLocPrint(menuwin,7,0,"                                 ");
		MyLocPrint(menuwin,8,0,"'z' = Main Menu                  ");
		break;

	case EDIT_CONTEXT:
		MyLocPrint(menuwin,0,0,"Move                             ");
		MyLocPrint(menuwin,1,0,"'l' = List script                ");
		MyLocPrint(menuwin,2,0,"'i' = Input script               ");
		MyLocPrint(menuwin,3,0,"'a' = Auto Enter                 ");
		MyLocPrint(menuwin,4,0,"                                 ");
		MyLocPrint(menuwin,5,0,"                                 ");
		MyLocPrint(menuwin,6,0,"                                 ");
		MyLocPrint(menuwin,7,0,"                                 ");
		MyLocPrint(menuwin,8,0,"'z' = Main Menu                  ");
		break;

	case LOAD_CONTEXT:
		MyLocPrint(menuwin,0,0,"Load                             ");
		MyLocPrint(menuwin,1,0,"'l' = Load script file           ");
		MyLocPrint(menuwin,2,0,"'s' = Save script file           ");
		MyLocPrint(menuwin,3,0,"'n' = Name script file           ");
		MyLocPrint(menuwin,4,0,"'w' = What is name of script file");
		MyLocPrint(menuwin,5,0,"                                 ");
		MyLocPrint(menuwin,6,0,"                                 ");
		MyLocPrint(menuwin,7,0,"                                 ");
		MyLocPrint(menuwin,8,0,"'z' = Main Menu                  ");
		break;

	case INSERT_CONTEXT:
		MyLocPrint(menuwin,0,0,"Insert                           ");
		MyLocPrint(menuwin,1,0,"'r' = CRegion                    ");
		MyLocPrint(menuwin,2,0,"'m' = CModel                     ");
		MyLocPrint(menuwin,3,0,"'s' = Sub Model                  ");
		MyLocPrint(menuwin,4,0,"'p' = CPart                      ");
		MyLocPrint(menuwin,5,0,"'l' = CList                      ");
		MyLocPrint(menuwin,6,0,"'R' = CRoom                      ");
		MyLocPrint(menuwin,7,0,"'P' = CPortal                    ");
		MyLocPrint(menuwin,8,0,"'z' = Main Menu                  ");
		break;

	case DELETE_CONTEXT:
		MyLocPrint(menuwin,0,0,"Delete                           ");
		MyLocPrint(menuwin,1,0,"'r' = CRegion                    ");
		MyLocPrint(menuwin,2,0,"'m' = CModel                     ");
		MyLocPrint(menuwin,3,0,"'s' = Sub Model                  ");
		MyLocPrint(menuwin,4,0,"'p' = CPart                      ");
		MyLocPrint(menuwin,5,0,"'l' = CList                      ");
		MyLocPrint(menuwin,6,0,"'R' = CRoom                      ");
		MyLocPrint(menuwin,7,0,"'P' = CPortal                    ");
		MyLocPrint(menuwin,8,0,"'z' = Main Menu                  ");
		break;

	default:
		break;
	}
} ////////////////////////////////////////////////////////////////////////////////////end PrintMenu

///////////////////////////////////////////////////////////////////////////////////////////////////
// GetAnInt
///////////////////////////////////////////////////////////////////////////////////////////////////
int GetAnInt()
{
CString		myinput;
char		c1[2];
char		buf[80];

	c1[0]='\0';
	c1[1]='\0';
	while (c1[0]!=13)
	{
		c1[0]=(char)getch();
		c1[1]='\0';
		if (c1[0]!=13) myinput.Append(c1);
		if (debug) snprintf(buf,80,"String is '%s' \n",myinput.GetStringPtr());
		if (debug) MyPrint(debugwin,buf,TRUE);
	}
	return (atoi(myinput.GetStringPtr()));
} //////////////////////////////////////////////////////////////////////////////////////end GetAnInt

///////////////////////////////////////////////////////////////////////////////////////////////////
// GetAString
///////////////////////////////////////////////////////////////////////////////////////////////////
//CString *GetAString()
//{
//CString         *myinput;
//char            c1[2];
//char            buf[80];
//
//	myinput = new CString;
//        c1[0]='\0';
//        c1[1]='\0';
//        while (c1[0]!=13)
//        {
//                c1[0]=(char)getch();
//                c1[1]='\0';
//                if (c1[0]!=13) myinput->Append(c1);
//                if (debug) snprintf(buf,80,"String is '%s' \n",myinput->GetStringPtr());
//                if (debug) MyPrint(debugwin,buf,TRUE);
//		MyPrint(mainwin,c1,TRUE);
//	        }
//        return (myinput);
//	}///////////////////////////////////////////////////////////////////////////////end GetAnInt

////////////////////////////////////////////////////////////////////////////////////////////////////
// Display
////////////////////////////////////////////////////////////////////////////////////////////////////
void Display()
{
int i;
bool b[10],eq,gt,lt,cy;
unsigned long int pc,sp;
long int ints[10];
double dbls[10];
CString s[10];
char buf[BUFFSIZE];

	MyPrint(debugwin,"Display registers \n", TRUE);

	MyPrint(mainwin,"Display registers \n", TRUE);
	engine->GetState(&pc,&sp,&eq,&gt,&lt,&cy);
	snprintf(buf,BUFFSIZE,"pc=%d sp=%d eq=%d gt=%d lt=%d  \n", pc,sp,eq,gt,lt);
	MyPrint(mainwin,buf, TRUE);

	MyPrint(mainwin,"Booleans \n", TRUE);
	engine->GetBools(b);
	snprintf(buf,BUFFSIZE,"%d %d %d %d %d %d %d %d %d %d \n", 
		b[0],b[1],b[2],b[3],b[4],b[5],b[6],b[7],b[8],b[9]);
	MyPrint(mainwin,buf, TRUE);

	MyPrint(mainwin,"Integers \n", TRUE);
	engine->GetInts(ints);
	snprintf(buf,BUFFSIZE,"%li %li %li %li %li %li %li %li %li %li \n", 
		ints[0],ints[1],ints[2],ints[3],ints[4],ints[5],ints[6],ints[7],ints[8],ints[9]);
	MyPrint(mainwin,buf, TRUE);

	MyPrint(mainwin,"Doubles \n", TRUE);
	engine->GetDbls(dbls);
	snprintf(buf,BUFFSIZE,"%f %f %f %f %f %f %f %f %f %f \n", 
		dbls[0],dbls[1],dbls[2],dbls[3],dbls[4],dbls[5],dbls[6],dbls[7],dbls[8],dbls[9]);
	MyPrint(mainwin,buf, TRUE);

	MyPrint(mainwin,"CStrings \n", TRUE);
	engine->GetStrings(s);
	for (i=0;i<10;i++)
	{
		if (s[i].GetStringPtr()>0) 
		{
			snprintf(buf,BUFFSIZE,"'%s' \n", s[i].GetStringPtr());
			MyPrint(mainwin,buf, TRUE);
			}
		else
		{
			snprintf(buf,BUFFSIZE,"'(NULL)' \n");
			MyPrint(mainwin,buf, TRUE);
			}
		}

	} ///////////////////////////////////////////////////////////////////////////////end Display

////////////////////////////////////////////////////////////////////////////////////////////////////
// ListScript
////////////////////////////////////////////////////////////////////////////////////////////////////
void ListScript(unsigned long int start)
{
unsigned long int	li;
char			buf[BUFFSIZE];

	MyPrint(mainwin,"Listing script  \n", TRUE);
	for (li=start;li<start+10;li++)
	{
		snprintf(buf,BUFFSIZE,"%d     %d\n",li,engine->GetValue(li));
		MyPrint(mainwin,buf, TRUE);
		
		}
	} ////////////////////////////////////////////////////////////////////////////end ListScript

////////////////////////////////////////////////////////////////////////////////////////////////////
// ExecComm
////////////////////////////////////////////////////////////////////////////////////////////////////
void ExecComm(char *c, curswin *p)
{
char testmsg[]="This is a message from console. Hello World.\n";
char msg[sizeof(int)];
int  l,m;
int  count,code;
bool timedout;
CString		myinput;

	switch (menucontext)
	{
	case MAIN_CONTEXT:
		switch (c[0])
		{
		case 't':
			menucontext=TEST_CONTEXT;
			break;

		case 'e':
			menucontext=EDIT_CONTEXT;
			break;

		case 'l':
			menucontext=LOAD_CONTEXT;
			break;

		case 'z':
			menucontext=END_CONTEXT;
			break;

		default:
			beep();                              //("Please try again\n");
			break;
		} // end MAIN_CONTEXT
		break;

	case TEST_CONTEXT: 
		switch (c[0])
		{
		case 's':
			{
			int result;

				MyPrint(debugwin,"Single step \n", TRUE);
				result=engine->SingleStep();
				if (result!=CE_ERR_NOERR)
				{
				char	buf[BUFFSIZE];
					snprintf(buf,BUFFSIZE,"Engine error: %d\n",result);
					MyPrint(mainwin,buf, TRUE);
					}
				}
			break;

		case 'r':
			{
			int	result;

				MyPrint(debugwin,"Run script \n", TRUE);
				result=engine->Run();
				if (result!=CE_ERR_NOERR)
				{
				char	buf[BUFFSIZE];
					snprintf(buf,BUFFSIZE,"Engine error: %d\n",result);
					MyPrint(mainwin,buf, TRUE);
					}
				}
			break;

		case 'd':
			{
				Display();
				}		
			break;

		case 'R':
			MyPrint(debugwin,"Reset \n", TRUE);
			engine->Reset();
			break;

		case 'p':
			MyPrint(debugwin,"Set PC \n", TRUE);
			{
			long int li;
				MyPrint(mainwin,"Enter PC  \n", TRUE);
				li=GetAnInt();
				engine->SetPC(li);
				}
			break;

		case 'l':
			{
			long int li;
				MyPrint(mainwin,"Enter starting location  \n", TRUE);
				li=GetAnInt();
				ListScript(li);
				}
			break;

		case 'z':
			menucontext=MAIN_CONTEXT;
			break;

		default:
			beep();                              //("Please try again\n");
			break;
		} // end MAINT_CONTEXT
		break;

	case EDIT_CONTEXT: 
		switch (c[0])
		{
		case 'l':
			{
			long int li;
				MyPrint(mainwin,"Enter starting location  \n", TRUE);
				li=GetAnInt();
				ListScript(li);
				}
			break;

		case 'i':
			{
			unsigned long int li,val;
				MyPrint(debugwin,"Input one script value \n", TRUE);
				MyPrint(mainwin,"Enter location \n", TRUE);
				li=GetAnInt();
				MyPrint(mainwin,"Enter value \n", TRUE);
				val=GetAnInt();
				engine->SetValue(li,val);
				ListScript(li);
				}
			break;

		case 'a':
			{
			unsigned long int li,val;
			char	buf[BUFFSIZE];

				MyPrint(debugwin,"Automatically input many values 9999=Done\n", TRUE);
				MyPrint(mainwin,"Enter location \n", TRUE);
				li=GetAnInt();
				val=0;
				snprintf(buf, BUFFSIZE, "Enter value for %d\n",li);
				MyPrint(mainwin,buf,TRUE);
				while ((val=GetAnInt())!=9999)
				{
					engine->SetValue(li,val);
					li++;
					snprintf(buf, BUFFSIZE, "Enter value for %d\n",li);
					MyPrint(mainwin,buf,TRUE);
					}
				}
			break;

		case 'z':
			menucontext=MAIN_CONTEXT;
			break;

		default:
			beep();                              //("Please try again\n");
			break;
			} // end MOVE_CONTEXT
		break;

	case LOAD_CONTEXT: 
		switch (c[0])
		{
		case 'l':
			{
			FILE			*scriptfile;
			unsigned long int 	li,val;
			char			buf[BUFFSIZE];

				if (!(scriptfile=fopen(scriptfilename->GetStringPtr(),"r"))) 
				{
					snprintf(buf,BUFFSIZE,"FAILED to open file for reading '%s'\n",scriptfilename->GetStringPtr());
					MyPrint(debugwin,buf, TRUE);
					break;
					}
				for (li=0;li<1000;li++)	
				{
					fscanf(scriptfile,"%d %d",&li,&val);
					engine->SetValue(li,val);
					}
				fclose(scriptfile);
				snprintf(buf,BUFFSIZE,"Loaded Script File '%s' OK\n",scriptfilename->GetStringPtr());
				MyPrint(debugwin,buf, TRUE);
				}
			break;

		case 's':
			{
			FILE	*scriptfile;
			unsigned long int li;

				MyPrint(debugwin,"Save Script File \n", TRUE);
				if (!(scriptfile=fopen(scriptfilename->GetStringPtr(),"w"))) 
				{
					MyPrint(debugwin,"Couldnt open File for Writing \n", TRUE);
					}
				for (li=0;li<1000;li++)	fprintf(scriptfile,"%d %d\n",li,engine->GetValue(li));
				fclose(scriptfile);
				}
			break;

		case 'n':
			MyPrint(mainwin,"Please enter the name of the script file \n", TRUE);
//			MyPrint(mainwin,"Characters are not echoed back yet \n", TRUE);
			scriptfilename=GetAString(p);
			MyPrint(mainwin,"Script file name = '", TRUE);
			if (scriptfilename) MyPrint(mainwin,scriptfilename->GetStringPtr(), TRUE);
			MyPrint(mainwin,"'\n", TRUE);

			break;

		case 'w':
			MyPrint(debugwin,"What is name of script file ?\n", TRUE);
			MyPrint(mainwin,"Script file name = '", TRUE);
			if (scriptfilename) MyPrint(mainwin,scriptfilename->GetStringPtr(), TRUE);
			MyPrint(mainwin,"'\n", TRUE);
			break;

		case 'z':
			menucontext=MAIN_CONTEXT;
			break;
 
		default:
			beep();                              //("Please try again\n");
			break;
		} // LOAD_CONTEXT
		break;

	case INSERT_CONTEXT:
		switch (c[0])
		{
		case 'r':
			{
			int x,y,z;
			bool done=FALSE;
			char buf[80];
			char c1[2];
//			CRegion *r;
				MyPrint(debugwin,"Region \n", TRUE);
				while (!done)
				{

					MyPrint(mainwin,"Please enter the x coord \n", TRUE);
					x=GetAnInt();
					MyPrint(mainwin,"Please enter the y coord \n", TRUE);
					y=GetAnInt();
					MyPrint(mainwin,"Please enter the z coord \n", TRUE);
					z=GetAnInt();
					snprintf(buf,80,"Is %d, %d, %d correct ? (y=yes)\n ",x,y,z);
					MyPrint(mainwin,buf,TRUE);
					c1[0]=(char)getch();
					c1[1]='\0';
					if (c1[0]=='y') 
					{
						MyPrint(mainwin,"Inserted region.\n",TRUE);
						done=TRUE;
						}
					}
			

				}
			break;

		case 'm':
                        {
			int x,y,z;
			CString *s;
			bool done1=FALSE;
			bool done2=FALSE;
                        char buf[80];
                        char c1[2];

				MyPrint(debugwin,"Model \n", TRUE);
				while (!done1)
				{

					MyPrint(mainwin,"Please enter the x coord of region to insert in\n", TRUE);
					x=GetAnInt();
					MyPrint(mainwin,"Please enter the y coord of region to insert in\n", TRUE);
					y=GetAnInt();
					MyPrint(mainwin,"Please enter the z coord of region to insert in\n", TRUE);
					z=GetAnInt();
					snprintf(buf,80,"Is %d, %d, %d correct ? (y=yes)\n ",x,y,z);
					MyPrint(mainwin,buf,TRUE);
					c1[0]=(char)getch();
					c1[1]='\0';
					if (c1[0]=='y') 
					{
						}
						done1=TRUE;
					}
				}
                        break;

		case 's':
			MyPrint(debugwin,"submodel \n", TRUE);
			break;

		case 'p':
			MyPrint(debugwin,"part \n", TRUE);
			break;

		case 'l':
			MyPrint(debugwin,"list \n", TRUE);
			break;

		case 'R':
			MyPrint(debugwin,"room \n", TRUE);
			break;

		case 'P':
			MyPrint(debugwin,"portal \n", TRUE);
			break;

		case 'z':
			menucontext=TEST_CONTEXT;
			break;

		default:
			beep();                              //("Please try again\n");
			break;
		} // INSERT_CONTEXT
		break;

	case DELETE_CONTEXT:
		switch (c[0])
		{
		case 'r':
			MyPrint(debugwin,"Region \n", TRUE);
			break;

		case 'm':
			MyPrint(debugwin,"model \n", TRUE);
			break;

		case 's':
			MyPrint(debugwin,"submodel \n", TRUE);
			break;

		case 'p':
			MyPrint(debugwin,"part \n", TRUE);
			break;

		case 'l':
			MyPrint(debugwin,"list \n", TRUE);
			break;

		case 'R':
			MyPrint(debugwin,"room \n", TRUE);
			break;

		case 'P':
			MyPrint(debugwin,"portal \n", TRUE);
			break;

		case 'z':
			menucontext=TEST_CONTEXT;
			break;

		default:
			beep();                              //("Please try again\n");
			break;
		} // DELETE_CONTEXT
	break;



	}
} /////////////////////////////////////////////////////////////////////////////////////end ExecComm

///////////////////////////////////////////////////////////////////////////////////////////////////
// Main
///////////////////////////////////////////////////////////////////////////////////////////////////
int main(int argc, char** argv)
{
char		c[4];
void            *param;
curswin         *winp;

	// init the screen & then threading and such
	// erase the screen
	Init(argc, argv);		// Init debug model
	InitScreen(&winp);	// Init the curses library & screen mutex

	// Init the screen
	clear();
	refresh();

	engine = new CEngine;
//	engine->SetLogFunc2(MyLog);
	engine->SetLogFunc(ThreadLog);
	engine->Init();

	// Start the listener
	// pack any params for the listener thread
	// tlist is a global thread tracking structure
	//c[0]=' ';
	do  
	{
		PrintMenu();
		c[0]=(char)getch();
		c[1]=0;
		ExecComm(c,winp);
	} while ((menucontext!=END_CONTEXT)||(c[0]!='z'));
	DeInit();
	MyPrint(debugwin,"Before DeInitScreen \n", TRUE);
	DeInitScreen();
	printf("Returning from main.\n");
} //////////////////////////////////////////////////////////////////////////////////////////end main

