////////////////////////////////////////////////////////////////////////////////////////////////////
// keycode.cpp
// this is from the example from the Linux Games book
////////////////////////////////////////////////////////////////////////////////////////////////////

#include	<unistd.h>
#include	<fcntl.h>
#include	<stdio.h>
#include	<stdlib.h>
#include	<termios.h>
#include	<sys/ioctl.h>
#include	<linux/keyboard.h>
#include	<linux/kd.h>


////////////////////////////////////////////////////////////////////////////////////////////////////
// is_keyboard
////////////////////////////////////////////////////////////////////////////////////////////////////
int is_keyboard(int fd)
{
int	data;

	// is fd for a keyboard ?
	data=0;
	if (ioctl(fd, KDGKBTYPE, &data) !=0 ) return 0;
	if (data==KB_84)
	{
		printf("84-key keyboard found.\n");
		return 1;
		}
	else if (data==KB_101)
	{	
		printf("101-key keyboard found.\n");
		return 1;
		}
	return 0;

	} ///////////////////////////////////////////////////////////////////////////end is_keyboard

////////////////////////////////////////////////////////////////////////////////////////////////////
// main
////////////////////////////////////////////////////////////////////////////////////////////////////
int main()
{
struct termios 	old_term, new_term;
int		kb=-1;			// to hold the kb fd
char		*files_to_try[]={"/dev/tty","/dev/console",NULL};
int		old_mode=-1;
int		i;

	for (i=0;files_to_try[i]!=NULL;i++)
	{
		kb=open(files_to_try[i],O_RDONLY);
		if (kb<0) continue;
		if (is_keyboard(kb))
		{
			printf("Using keyboard on %s.\n",files_to_try[i]);
			break;
			}
		close(kb);
		}

	// others didnt work so try stdin stdout stderr
	if (files_to_try[i]==NULL)
	{
		for (kb=0;kb<3;kb++) if (is_keyboard(i)) break;
		
		printf("Unable to find fd for keyboard.\n");
		return 1;
		}

	// get kb mode for later restore
	if (ioctl(kb,KDGKBMODE, &old_mode) != 0)
	{
		printf("unable to query kb mode.\n");
		goto error;
		}

	// get oric attr to add ours
	if (tcgetattr(kb,&old_term)!=0)
	{
		printf("unable to query terminal settings.\n");
		goto error;
		}

	// adjust settings
	new_term=old_term;
	new_term.c_iflag=0;
	new_term.c_lflag&=~(ECHO|ICANON|ISIG);

	//
	if (tcsetattr(kb,TCSAFLUSH, &new_term) !=0)
	{
		printf("Unable to change settings.\n");
		}
	
	// goto medium raw mode
	if (ioctl(kb,KDSKBMODE, K_MEDIUMRAW)!=0)
	{
		printf ("Unable to set meduim raw.\n");
		goto error;
		}

	//
	printf("Reading key codes. Press ESC (keycode 1) to exit.\n");

	for (;;)
	{
	unsigned char data;
		if (read(kb,&data,1)<1)
		{
			printf ("Unable to readdata try to exit clean.\n");
			goto error;
			}
		// print keycode
		printf("%s: %2Xh (%i) \n", (data & 0x80) ? "Released" : "Pressed",
			(unsigned int)data &0x7F,
			(unsigned int)data &0x7F);
		if ((data & 0x7F) == 1)
		{
			printf ("Escape Pressed.\n");
			break;
			}
		} // end for
	printf ("Exiting normally.\n");
	ioctl(kb, KDSKBMODE, old_mode);
	tcsetattr(kb, 0, &old_term);
	if (kb > 3) close(kb);

	return 0;

error:
	printf("Cleaning up.\n");
	fflush(stdout);
	if (old_mode !=-1)
	{
		ioctl(kb,KDSKBMODE, old_mode);
		tcsetattr(kb, 0, &old_term);
		}
	if (kb>3) close(kb);
	return 1;
	
	
	} //////////////////////////////////////////////////////////////////////////////////end main


























