///////////////////////////////////////////////////////////////////////////////////////////////////
// main.cpp 
// This is the main module for the cworldtest application.
//
// World II
//
// by Eric J Krejci
// Copyright (C) 1999 Eric J Krejci 
// Revised : 31 Jan 2006
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation version 2.
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
// Contact:
//              email           worldii@viagloria.com
//              web             www.viagloria.com
///////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////
// Defines
///////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////
// Includes
///////////////////////////////////////////////////////////////////////////////////////////////////
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>
#include <curses.h>
#include <strings.h>

#include "wii_types.h"
#include "wii_curses.h"
#include "worldii.h"
#include "cworld.h"
#include "wii_util.h"
#include "cworld.hpp"
#include "cpart.hpp"
#include "cregion.hpp"
#include "cmodel.hpp"


///////////////////////////////////////////////////////////////////////////////////////////////////
// Globals
///////////////////////////////////////////////////////////////////////////////////////////////////

WINDOW		*debugwin;
WINDOW  	*mainwin;
WINDOW  	*menuwin;
bool 		debug=FALSE;
int		menucontext;
FILE 		*logfile;
char		logfilename[50]="logs/cworldtest.log";
long		logcount;
CWorld  	*world;

///////////////////////////////////////////////////////////////////////////////////////////////////
// External Functions
///////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////
// Init
///////////////////////////////////////////////////////////////////////////////////////////////////
void Init(int argc, char** argv)
{
int i;

     // init the threading model 
     logfile=fopen(logfilename,"w");
     fclose(logfile);

	// find out if we are debugging
	if ((argc==2) && (strcmp(argv[1],"debug")==0))
	{
		printf("DEBUGMODE ON\n");
		debug=TRUE;
	}

	if (debug) 
	{
		for (i=0;i<argc;i++) printf("Argument %d, s='%s' \n",i,argv[i]);

		printf("Hello worldii!\n");
		printf("Allocation CWorld\n");
	}
	
} /////////////////////////////////////////////////////////////////////////////////////////end Init

///////////////////////////////////////////////////////////////////////////////////////////////////
// InitSock
///////////////////////////////////////////////////////////////////////////////////////////////////
void InitSock()
{

     } ////////////////////////////////////////////////////////////////////////////////end InitSock

///////////////////////////////////////////////////////////////////////////////////////////////////
// DeInit
///////////////////////////////////////////////////////////////////////////////////////////////////
void DeInit()
{
     // 

     } /////////////////////////////////////////////////////////////////////////////end DeInit

///////////////////////////////////////////////////////////////////////////////////////////////////
// InitScreen
///////////////////////////////////////////////////////////////////////////////////////////////////
void InitScreen()
{
     // init the ncurses library
	initscr();
	cbreak();
	noecho();
	nonl();
	intrflush(stdscr, FALSE);
	keypad(stdscr, TRUE);

	// create the windows
	menuwin =newwin(int(LINES/2),int(COLS/2), 0,0);
	debugwin=newwin(int(LINES/2),int(COLS/2),0,int(COLS/2));
	mainwin =newwin(int(LINES/2),COLS,int(LINES/2),0);
	scrollok(menuwin, TRUE);
	scrollok(debugwin,TRUE);
	scrollok(mainwin, TRUE);
	menucontext=MAIN_CONTEXT;
} ////////////////////////////////////////////////////////////////////////////////////end InitScreen

////////////////////////////////////////////////////////////////////////////////////////////////////
// DeInitScreen
////////////////////////////////////////////////////////////////////////////////////////////////////
void DeInitScreen()
{
	// DeInit the ncurses library
	wrefresh(menuwin);
	wrefresh(debugwin);
	wrefresh(mainwin);
	endwin();
} /////////////////////////////////////////////////////////////////////////////////end DeInitScreen

///////////////////////////////////////////////////////////////////////////////////////////////////
// Menu
///////////////////////////////////////////////////////////////////////////////////////////////////
void PrintMenu()
{
	switch (menucontext)
	{
	case MAIN_CONTEXT:
		if (debug)  MyLocPrint(menuwin,0,0,"Main Debug Mode ON               ");
		if (!debug) MyLocPrint(menuwin,0,0,"Main                             ");
		MyLocPrint(menuwin,1,0,"'i' = Ins/Del/Mod/Print Objects  ");
		MyLocPrint(menuwin,2,0,"'m' = Move in CWorld             ");
		MyLocPrint(menuwin,3,0,"'l' = Load CWorld data file      ");
		MyLocPrint(menuwin,4,0,"                                 ");
		MyLocPrint(menuwin,5,0,"                                 ");
		MyLocPrint(menuwin,6,0,"                                 ");
		MyLocPrint(menuwin,7,0,"                                 ");
		MyLocPrint(menuwin,8,0,"'z' = Exit console.              ");
		break;

	case MAINT_CONTEXT:
		MyLocPrint(menuwin,0,0,"Maintain CWorld                  ");
		MyLocPrint(menuwin,1,0,"'i' = Insert object              ");
		MyLocPrint(menuwin,2,0,"'d' = Delete object              ");
		MyLocPrint(menuwin,3,0,"'m' = Modify object              ");
		MyLocPrint(menuwin,4,0,"'p' = Print tree                 ");
		MyLocPrint(menuwin,5,0,"                                 ");
		MyLocPrint(menuwin,6,0,"                                 ");
		MyLocPrint(menuwin,7,0,"                                 ");
		MyLocPrint(menuwin,8,0,"'z' = Main Menu                  ");
		break;

	case MOVE_CONTEXT:
		MyLocPrint(menuwin,0,0,"Move                             ");
		MyLocPrint(menuwin,1,0,"'h' = Turn left 1.0              ");
		MyLocPrint(menuwin,2,0,"'j' = Move backwards 1.0         ");
		MyLocPrint(menuwin,3,0,"'k' = Move forward 1.0           ");
		MyLocPrint(menuwin,4,0,"'l' = Turn right 1.0             ");
		MyLocPrint(menuwin,5,0,"                                 ");
		MyLocPrint(menuwin,6,0,"                                 ");
		MyLocPrint(menuwin,7,0,"                                 ");
		MyLocPrint(menuwin,8,0,"'z' = Main Menu                  ");
		break;

	case LOAD_CONTEXT:
		MyLocPrint(menuwin,0,0,"Load                             ");
		MyLocPrint(menuwin,1,0,"'l' = Load CWorld                ");
		MyLocPrint(menuwin,2,0,"'s' = Save CWorld                ");
		MyLocPrint(menuwin,3,0,"                                 ");
		MyLocPrint(menuwin,4,0,"                                 ");
		MyLocPrint(menuwin,5,0,"                                 ");
		MyLocPrint(menuwin,6,0,"                                 ");
		MyLocPrint(menuwin,7,0,"                                 ");
		MyLocPrint(menuwin,8,0,"'z' = Main Menu                  ");
		break;

	case INSERT_CONTEXT:
		MyLocPrint(menuwin,0,0,"Insert                           ");
		MyLocPrint(menuwin,1,0,"'r' = CRegion                    ");
		MyLocPrint(menuwin,2,0,"'m' = CModel                     ");
		MyLocPrint(menuwin,3,0,"'s' = Sub Model                  ");
		MyLocPrint(menuwin,4,0,"'p' = CPart                      ");
		MyLocPrint(menuwin,5,0,"'l' = CList                      ");
		MyLocPrint(menuwin,6,0,"'R' = CRoom                      ");
		MyLocPrint(menuwin,7,0,"'P' = CPortal                    ");
		MyLocPrint(menuwin,8,0,"'z' = Main Menu                  ");
		break;

	case DELETE_CONTEXT:
		MyLocPrint(menuwin,0,0,"Delete                           ");
		MyLocPrint(menuwin,1,0,"'r' = CRegion                    ");
		MyLocPrint(menuwin,2,0,"'m' = CModel                     ");
		MyLocPrint(menuwin,3,0,"'s' = Sub Model                  ");
		MyLocPrint(menuwin,4,0,"'p' = CPart                      ");
		MyLocPrint(menuwin,5,0,"'l' = CList                      ");
		MyLocPrint(menuwin,6,0,"'R' = CRoom                      ");
		MyLocPrint(menuwin,7,0,"'P' = CPortal                    ");
		MyLocPrint(menuwin,8,0,"'z' = Main Menu                  ");
		break;

	default:
		break;
	}
} ////////////////////////////////////////////////////////////////////////////////////end PrintMenu

///////////////////////////////////////////////////////////////////////////////////////////////////
// GetAnInt
///////////////////////////////////////////////////////////////////////////////////////////////////
int GetAnInt()
{
CString		myinput;
char		c1[2];
char		buf[80];

	c1[0]='\0';
	c1[1]='\0';
	while (c1[0]!=13)
	{
		c1[0]=(char)getch();
		c1[1]='\0';
		if (c1[0]!=13) myinput.Append(c1);
		if (debug) snprintf(buf,80,"String is '%s' \n",myinput.GetString.Ptr());
		if (debug) MyPrint(debugwin,buf,TRUE);
	}
	return (atoi(myinput.GetString.Ptr()));
} //////////////////////////////////////////////////////////////////////////////////////end GetAnInt

///////////////////////////////////////////////////////////////////////////////////////////////////
// GetAString
///////////////////////////////////////////////////////////////////////////////////////////////////
CString *GetAString()
{
CString         *myinput;
char            c1[2];
char            buf[80];

	myinput = new CString;
        c1[0]='\0';
        c1[1]='\0';
        while (c1[0]!=13)
        {
                c1[0]=(char)getch();
                c1[1]='\0';
                if (c1[0]!=13) myinput->Append(c1);
                if (debug) snprintf(buf,80,"String is '%s' \n",myinput->GetString.Ptr());
                if (debug) MyPrint(debugwin,buf,TRUE);
        }
        return (myinput);
} //////////////////////////////////////////////////////////////////////////////////////end GetAnInt

////////////////////////////////////////////////////////////////////////////////////////////////////
// ExecComm
////////////////////////////////////////////////////////////////////////////////////////////////////
void ExecComm(char *c)
{
char testmsg[]="This is a message from console. Hello World.\n";
char msg[sizeof(int)];
int  l,m;
int  count,code;
bool timedout;
CString		myinput;

	switch (menucontext)
	{
	case MAIN_CONTEXT:
		switch (c[0])
		{
		case 'i':
			menucontext=MAINT_CONTEXT;
			break;

		case 'm':
			menucontext=MOVE_CONTEXT;
			break;

		case 'l':
			menucontext=LOAD_CONTEXT;
			break;

		case 'z':
			menucontext=END_CONTEXT;
			break;

		default:
			beep();                              //("Please try again\n");
			break;
		} // end MAIN_CONTEXT
		break;

	case MAINT_CONTEXT: 
		switch (c[0])
		{
		case 'i':
			menucontext=INSERT_CONTEXT;
			break;

		case 'd':
			menucontext=DELETE_CONTEXT;
			break;

		case 'm':
			break;

		case 'p':
			world->Print();

			break;

		case 'z':
			menucontext=MAIN_CONTEXT;
			break;

		default:
			beep();                              //("Please try again\n");
			break;
		} // end MAINT_CONTEXT
		break;

	case MOVE_CONTEXT: 
		switch (c[0])
		{
		case 'h':
			MyPrint(debugwin,"Turn left 1.0 \n", TRUE);
			break;

		case 'j':
			MyPrint(debugwin,"Move forward 1.0 \n", TRUE);
			break;

		case 'k':
			MyPrint(debugwin,"Move backward 1.0 \n", TRUE);
			break;

		case 'l':
			MyPrint(debugwin,"Turn right 1.0 \n", TRUE);
			break;

		case 'z':
			menucontext=MAIN_CONTEXT;
			break;

		default:
			beep();                              //("Please try again\n");
			break;
		} // end MOVE_CONTEXT
		break;

	case LOAD_CONTEXT: 
		switch (c[0])
		{
		case 's':
			MyPrint(debugwin,"Show summary statistics \n", TRUE);
			break;

		case 'c':
			MyPrint(debugwin,"Show server statistics \n", TRUE);
			break;

		case 'd':
			MyPrint(debugwin,"Show database statistics \n", TRUE);
			break;

		case 'z':
			menucontext=MAIN_CONTEXT;
			break;
 
		default:
			beep();                              //("Please try again\n");
			break;
		} // LOAD_CONTEXT
		break;

	case INSERT_CONTEXT:
		switch (c[0])
		{
		case 'r':
			{
			int x,y,z;
			bool done=FALSE;
			char buf[80];
			char c1[2];
			CRegion *r;
				MyPrint(debugwin,"Region \n", TRUE);
				while (!done)
				{

					MyPrint(mainwin,"Please enter the x coord \n", TRUE);
					x=GetAnInt();
					MyPrint(mainwin,"Please enter the y coord \n", TRUE);
					y=GetAnInt();
					MyPrint(mainwin,"Please enter the z coord \n", TRUE);
					z=GetAnInt();
					snprintf(buf,80,"Is %d, %d, %d correct ? (y=yes)\n ",x,y,z);
					MyPrint(mainwin,buf,TRUE);
					c1[0]=(char)getch();
					c1[1]='\0';
					if (c1[0]=='y') 
					{
						r = new CRegion;
						r->SetIndex(x,y,z);
						world->InsertRegion(r);
						MyPrint(mainwin,"Inserted region.\n",TRUE);
						done=TRUE;
					}
				}
			

			}
			break;

		case 'm':
                        {
			int x,y,z;
			CString *s;
			bool done1=FALSE;
			bool done2=FALSE;
                        char buf[80];
                        char c1[2];
                        CModel *m;
			CRegion *r;

				MyPrint(debugwin,"Model \n", TRUE);
				while (!done1)
				{

					MyPrint(mainwin,"Please enter the x coord of region to insert in\n", TRUE);
					x=GetAnInt();
					MyPrint(mainwin,"Please enter the y coord of region to insert in\n", TRUE);
					y=GetAnInt();
					MyPrint(mainwin,"Please enter the z coord of region to insert in\n", TRUE);
					z=GetAnInt();
					snprintf(buf,80,"Is %d, %d, %d correct ? (y=yes)\n ",x,y,z);
					MyPrint(mainwin,buf,TRUE);
					c1[0]=(char)getch();
					c1[1]='\0';
					if (c1[0]=='y') 
					{
						r = world->FindRegion(x,y,z);
						if (r) 
						{
							MyPrint(mainwin,"Found region.\n",TRUE);
			                                while (!done2)
                        			        {
        			                                MyPrint(mainwin,"Please enter the filename \n", TRUE);
			                                        s=GetAString();	// allocates a CString with keybord responses
			                                        snprintf(buf,80,"Is '%s' correct ? (y=yes)\n ",s->GetString.Ptr());
			                                        MyPrint(mainwin,buf,TRUE);
			                                        c1[0]=(char)getch();
			                                        c1[1]='\0';
			                                        if (c1[0]=='y')
			                                        {
			                                                m = new CModel;
			                                                m->SetFileName(s->GetString.Ptr());
			                                                world->InsertModel(m,r);
			                                                MyPrint(mainwin,"Inserted model.\n",TRUE);
			                                                done2=TRUE;
			                                        }
			                                }
						}
						done1=TRUE;
					}
				}
                        }
			break;

		case 's':
			MyPrint(debugwin,"submodel \n", TRUE);
			break;

		case 'p':
			MyPrint(debugwin,"part \n", TRUE);
			break;

		case 'l':
			MyPrint(debugwin,"list \n", TRUE);
			break;

		case 'R':
			MyPrint(debugwin,"room \n", TRUE);
			break;

		case 'P':
			MyPrint(debugwin,"portal \n", TRUE);
			break;

		case 'z':
			menucontext=MAINT_CONTEXT;
			break;

		default:
			beep();                              //("Please try again\n");
			break;
		} // INSERT_CONTEXT
		break;

	case DELETE_CONTEXT:
		switch (c[0])
		{
		case 'r':
			MyPrint(debugwin,"Region \n", TRUE);
			break;

		case 'm':
			MyPrint(debugwin,"model \n", TRUE);
			break;

		case 's':
			MyPrint(debugwin,"submodel \n", TRUE);
			break;

		case 'p':
			MyPrint(debugwin,"part \n", TRUE);
			break;

		case 'l':
			MyPrint(debugwin,"list \n", TRUE);
			break;

		case 'R':
			MyPrint(debugwin,"room \n", TRUE);
			break;

		case 'P':
			MyPrint(debugwin,"portal \n", TRUE);
			break;

		case 'z':
			menucontext=MAINT_CONTEXT;
			break;

		default:
			beep();                              //("Please try again\n");
			break;
		} // DELETE_CONTEXT
	break;



	}
} /////////////////////////////////////////////////////////////////////////////////////end ExecComm

///////////////////////////////////////////////////////////////////////////////////////////////////
// Main
///////////////////////////////////////////////////////////////////////////////////////////////////
int main(int argc, char** argv)
{
char		c[4];
void            *param;


	// init the screen & then threading and such
	// erase the screen
	Init(argc, argv);		// Init debug model
	InitScreen();	// Init the curses library & screen mutex

	// Init the screen
	clear();
	refresh();

	world = new CWorld;

	// Start the listener
	// pack any params for the listener thread
	// tlist is a global thread tracking structure
	//c[0]=' ';
	do  
	{
		PrintMenu();
		c[0]=(char)getch();
		c[1]=0;
		ExecComm(c);
	} while ((menucontext!=END_CONTEXT)||(c[0]!='z'));
	DeInit();
	MyPrint(debugwin,"Before DeInitScreen \n", TRUE);
	DeInitScreen();
	printf("Returning from main.\n");
} //////////////////////////////////////////////////////////////////////////////////////////end main

