//////////////////////////////////////////////////////////////////////////////////////////////
// parse.cpp 
// 
// This is the new parser for the new portmonitor. 
//
// World II
//
// by Eric J Krejci
// Copyright (C) 1999 Eric J Krejci 
// Revised : 31 Jan 2006
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation version 2.
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
// Contact:
//              email           worldii@viagloria.com
//              web             www.viagloria.com
//////////////////////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////////////////////
// Defines
//////////////////////////////////////////////////////////////////////////////////////////////
#define		MESSAGESIZE	1500
#define		MINPRINTABLE	' '
#define		MAXPRINTABLE	'~'

//////////////////////////////////////////////////////////////////////////////////////////////
// Includes
//////////////////////////////////////////////////////////////////////////////////////////////
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "cstring.hpp"
#include "cmessage.hpp"
#include "wii_structures.h"
#include "worldii.h"

//////////////////////////////////////////////////////////////////////////////////////////////
// Globals
//////////////////////////////////////////////////////////////////////////////////////////////
int		cmdarraycnt;		// How many commands in commands array
//int		cmdstate=0;		// Tells wether we are getting cmd or parms
//int		p;			// A position pointer into rawbuf
//int		tp;			// A trailing position pointer into rawbuf
//char		delim='|';		// The delimiter
CMessage	*Message;

//////////////////////////////////////////////////////////////////////////////////////////////
// Code
//////////////////////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////////////////////
// Enqueue
// for now this will just print out the contents of the CMessage
//////////////////////////////////////////////////////////////////////////////////////////////
bool Enqueue(CMessage *m,int debug)
{
	if (debug==1) printf("Enqueue \n");
	if (m->CheckMessage(debug))
	{
		printf				("message 	= %d \n",m->GetMessage());
		if (m->GetParm0()) printf("parm0 		= %s \n",m->GetParm0());
		else printf("parm0           = NULL\n");
		if (m->GetParm1()) printf("parm1 		= %s \n",m->GetParm1());
		else printf("parm1           = NULL\n");
	}
        return (TRUE);
} /////////////////////////////////////////////////////////////////////////////////end Enqueue

//////////////////////////////////////////////////////////////////////////////////////////////
// Parse
//////////////////////////////////////////////////////////////////////////////////////////////
int Parse(char *rawbuf,int length,int debug)
{
CMessage *msg;
int		cmdstate=0;		// Tells wether we are getting cmd or parms
int		p;			// A position pointer into rawbuf
int		tp;			// A trailing position pointer into rawbuf
char		delim='|';		// The delimiter

	// This will be a state machine
	// but we will strive to read the message just once.

Init:
	if (debug>0) printf("Init \n");
	p=0;
	tp=0;	
	cmdstate=0;
	length=strlen(rawbuf);	// remember, for now all valid data is printable, and null termed. 
	msg = new CMessage;

GetChars:
	if (debug>0) printf("GetChars ");
	if (p>length) goto Done;
	
	//filter out non printable chars
	if (debug>0) printf("rawbuf[%d] = '%c'  ",p,rawbuf[p]);
	if ((rawbuf[p]<MINPRINTABLE) || (rawbuf[p]>MAXPRINTABLE)) rawbuf[p]=' '; 
	if (debug>0) printf("rawbuf[%d] = '%c'  \n",p,rawbuf[p]);
	if (rawbuf[p]!=delim)
	{
		p++;
		goto GetChars;
	}	
	// Note intentional fall through

LookAhead:
	if (debug>0) printf("LookAhead \n");
	if (p+1>length) goto Done;
	if (rawbuf[p+1]!=delim)	// is there a second delim ?
	{
		if (cmdstate==0) goto CheckProperCmd;
		if (cmdstate==1) goto GettingParm;
	}
	if ((cmdstate==0)&&(rawbuf[p+1]==delim)) goto CheckProperCmdNoParm;	// is there a second delim ?
	// Note intentional fall through

GettingLastParm:
	if (debug>0) printf("GettingLastParm \n");
	if (!msg->AddParmToMsg(p,tp,rawbuf)) return (1);
	// Note intentional fall through

CheckCmdComplete:
	if (debug>0) printf("CheckCmdComplete \n");
	if (msg->CheckMessage(debug)) goto InsertCmdIntoQueue;
	else delete msg;
	p++;
	p++;
	tp=p;
	goto GetChars;

CheckProperCmd:
	if (debug>0) printf("CheckProperCmd \n");
	if (msg->CheckProperCmd(p,tp,rawbuf,debug))
	{
		cmdstate=1;
		p++;
		tp=p;
		goto GetChars;
	}
	goto DiscardToDouble;

CheckProperCmdNoParm:
        if (debug>0) printf("CheckProperCmdNoParm \n");
        if (msg->CheckProperCmd(p,tp,rawbuf,debug))
        {
                cmdstate=0;
                goto CheckCmdComplete;
        }
        // Note intentional fall through

DiscardToDouble:
        if (debug>0) printf("DiscardToDouble \n");
	// Not a proper cmd so flush to next double delim or end of rawbuf
	while ((p<length)&&(rawbuf[p]!=delim)) p++;
	if ((p<length) && (rawbuf[p]==delim) && (rawbuf[p+1]==delim)) 
	{
		p++;
		p++;
		tp=p;		// set tp and p to the next char to consider, skipping the current delim
		cmdstate=0;
		goto GetChars;
	}
	else p++;
	if (p>=length) goto Done;
	goto DiscardToDouble;

GettingParm:
	if (debug>0) printf("GettingParm \n");
	if (!msg->AddParmToMsg(p,tp,rawbuf)) return (1);
	p++;
	tp=p;
	goto GetChars;

InsertCmdIntoQueue:
	if (debug>0) printf("InsertCmdIntoQueue \n");
	if (!Enqueue(msg,debug)) return(2); 
	else
	{
	CMessage *msgold;
		p++;
		p++;
		tp=p;
		cmdstate=0;
		msgold=msg;
		msg = new CMessage;
		delete msgold;
		goto GetChars;
	}

Done:
	return(p);	// dont really need to do this, but the compiler wanted it.

} ///////////////////////////////////////////////////////////////////////////////////end Parse


