///////////////////////////////////////////////////////////////////////////////////////////////////
// queuetest.cpp 
// This is a test prog for the queue class.
//
// World II
//
// by Eric J Krejci
// Copyright (C) 1999 Eric J Krejci 
// Revised : 27 May 2007
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation version 2.
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
// Contact:
//              email           worldii@viagloria.com
//              web             www.viagloria.com
///////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////
// Defines
///////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////
// Includes
///////////////////////////////////////////////////////////////////////////////////////////////////
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>
#include <curses.h>
#include <strings.h>
#include <sys/timeb.h>

#include "cstring.hpp"
#include "cqueue.hpp"
#include "wiii_util.h"
#include "wiii_curses.h"


///////////////////////////////////////////////////////////////////////////////////////////////////
// Globals
///////////////////////////////////////////////////////////////////////////////////////////////////

WINDOW		*debugwin;
WINDOW  	*mainwin;
WINDOW  	*menuwin;
bool 		debug=FALSE;
int		menucontext;
FILE 		*logfile;
char		logfilename[50]="logs/queuetest.log";
long		logcount;
CQueue  	*cq;
int		loglevel;
struct timeb	starttime;
pthread_mutex_t	screenmutex;
pthread_mutex_t	logmutex;

///////////////////////////////////////////////////////////////////////////////////////////////////
// External Functions
///////////////////////////////////////////////////////////////////////////////////////////////////
#define		MAIN_CONTEXT	0
#define		MAINT_CONTEXT	1
#define		INSERT_CONTEXT	2
#define		DELETE_CONTEXT	3
#define		MOVE_CONTEXT	4
#define		TEST_CONTEXT	5
#define		END_CONTEXT	6

///////////////////////////////////////////////////////////////////////////////////////////////////
// Init
///////////////////////////////////////////////////////////////////////////////////////////////////
void Init(int argc, char** argv)
{
int i;

     // init the threading model 
     logfile=fopen(logfilename,"w");
     fclose(logfile);

	// find out if we are debugging
	if ((argc==2) && (strcmp(argv[1],"debug")==0))
	{
		printf("DEBUGMODE ON\n");
		debug=TRUE;
	}

	if (debug) 
	{
		for (i=0;i<argc;i++) printf("Argument %d, s='%s' \n",i,argv[i]);

		printf("Hello cq!\n");
		printf("Allocation CQueue\n");
	}
	
} /////////////////////////////////////////////////////////////////////////////////////////end Init

///////////////////////////////////////////////////////////////////////////////////////////////////
// InitScreen
///////////////////////////////////////////////////////////////////////////////////////////////////
void InitScreen()
{
     // init the ncurses library
	initscr();
	cbreak();
	noecho();
	nonl();
	intrflush(stdscr, FALSE);
	keypad(stdscr, TRUE);

	// create the windows
	menuwin =newwin(int(LINES/2),int(COLS/2), 0,0);
	debugwin=newwin(int(LINES/2),int(COLS/2),0,int(COLS/2));
	mainwin =newwin(int(LINES/2),COLS,int(LINES/2),0);
	scrollok(menuwin, TRUE);
	scrollok(debugwin,TRUE);
	scrollok(mainwin, TRUE);
	menucontext=MAIN_CONTEXT;
} ////////////////////////////////////////////////////////////////////////////////////end InitScreen

////////////////////////////////////////////////////////////////////////////////////////////////////
// DeInitScreen
////////////////////////////////////////////////////////////////////////////////////////////////////
void DeInitScreen()
{
	// DeInit the ncurses library
	wrefresh(menuwin);
	wrefresh(debugwin);
	wrefresh(mainwin);
	endwin();
} /////////////////////////////////////////////////////////////////////////////////end DeInitScreen

///////////////////////////////////////////////////////////////////////////////////////////////////
// Menu
///////////////////////////////////////////////////////////////////////////////////////////////////
void PrintMenu()
{
	switch (menucontext)
	{
	case MAIN_CONTEXT:
		if (debug)  MyLocPrint(menuwin,0,0,"Main Debug Mode ON               ");
		if (!debug) MyLocPrint(menuwin,0,0,"Main                             ");
		MyLocPrint(menuwin,1,0,"'m' = Maintain (Ins/Mod/Del)     ");
		MyLocPrint(menuwin,2,0,"'M' = Move in CQueue             ");
		MyLocPrint(menuwin,3,0,"'t' = Test                       ");
		MyLocPrint(menuwin,4,0,"'b' = Big Random Test            ");
		MyLocPrint(menuwin,5,0,"                                 ");
		MyLocPrint(menuwin,6,0,"                                 ");
		MyLocPrint(menuwin,7,0,"                                 ");
		MyLocPrint(menuwin,8,0,"                                 ");
		MyLocPrint(menuwin,9,0,"'z' = Exit console.              ");
		break;

	case MAINT_CONTEXT:
		MyLocPrint(menuwin,0,0,"Maintain CQueue                  ");
		MyLocPrint(menuwin,1,0,"'i' = Insert object              ");
		MyLocPrint(menuwin,2,0,"'c' = Change object              ");
		MyLocPrint(menuwin,3,0,"'a' = Append object              ");
		MyLocPrint(menuwin,4,0,"'p' = Prepend object             ");
		MyLocPrint(menuwin,5,0,"'d' = Delete Current             ");
		MyLocPrint(menuwin,6,0,"'D' = Delete whole list          ");
		MyLocPrint(menuwin,7,0,"                                 ");
		MyLocPrint(menuwin,8,0,"                                 ");
		MyLocPrint(menuwin,9,0,"'z' = Main Menu                  ");
		break;

	case MOVE_CONTEXT:
		MyLocPrint(menuwin,0,0,"Move                             ");
		MyLocPrint(menuwin,1,0,"'f' = Forward one item           ");
		MyLocPrint(menuwin,2,0,"'b' = Begin of list              ");
		MyLocPrint(menuwin,3,0,"'e' = End of list                ");
		MyLocPrint(menuwin,4,0,"                                 ");
		MyLocPrint(menuwin,5,0,"                                 ");
		MyLocPrint(menuwin,6,0,"                                 ");
		MyLocPrint(menuwin,7,0,"                                 ");
		MyLocPrint(menuwin,8,0,"                                 ");
		MyLocPrint(menuwin,9,0,"'z' = Main Menu                  ");
		break;

	case TEST_CONTEXT:
		MyLocPrint(menuwin,0,0,"Test                             ");
		MyLocPrint(menuwin,1,0,"'e' = IsEmpty                    ");
		MyLocPrint(menuwin,2,0,"'f' = IsFull                     ");
		MyLocPrint(menuwin,3,0,"'B' = BegList                    ");
		MyLocPrint(menuwin,4,0,"'E' = EndList                    ");
		MyLocPrint(menuwin,5,0,"'c' = Count items                ");
		MyLocPrint(menuwin,6,0,"'C' = Getcount                   ");
		MyLocPrint(menuwin,7,0,"'p' = Print current              ");
		MyLocPrint(menuwin,8,0,"'P' = Print List                 ");
		MyLocPrint(menuwin,9,0,"'z' = Main Menu                  ");
		break;

	default:
		break;
	}
} ////////////////////////////////////////////////////////////////////////////////////end PrintMenu

///////////////////////////////////////////////////////////////////////////////////////////////////
// GetAnInt
///////////////////////////////////////////////////////////////////////////////////////////////////
int GetAnInt()
{
CString		myinput;
char		c1[2];
char		buf[80];

	c1[0]='\0';
	c1[1]='\0';
	while (c1[0]!=13)
	{
		c1[0]=(char)getch();
		c1[1]='\0';
		if (c1[0]!=13) myinput.Append(c1);
		if (debug) snprintf(buf,80,"String is '%s' \n",myinput.GetStringPtr());
		if (debug) MyPrint(debugwin,buf,TRUE);
	}
	return (atoi(myinput.GetStringPtr()));
} //////////////////////////////////////////////////////////////////////////////////////end GetAnInt

///////////////////////////////////////////////////////////////////////////////////////////////////
// GetAString
///////////////////////////////////////////////////////////////////////////////////////////////////
CString *GetAString()
{
CString         *myinput;
char            c1[2];
char            buf[80];

	myinput = new CString;
        c1[0]='\0';
        c1[1]='\0';
        while (c1[0]!=13)
        {
                c1[0]=(char)getch();
                c1[1]='\0';
                if (c1[0]!=13) myinput->Append(c1);
                if (debug) snprintf(buf,80,"String is '%s' \n",myinput->GetStringPtr());
                if (debug) MyPrint(debugwin,buf,TRUE);
		MyPrint(mainwin,c1,TRUE);
        }
        return (myinput);
} //////////////////////////////////////////////////////////////////////////////////////end GetAnInt

////////////////////////////////////////////////////////////////////////////////////////////////////
// ExecComm
////////////////////////////////////////////////////////////////////////////////////////////////////
void ExecComm(char *c)
{
char testmsg[]="This is a message from console. Hello World.\n";
char msg[sizeof(int)];
int  l,m;
int  count,code;
bool timedout;
CString		myinput;

	switch (menucontext)
	{
	case MAIN_CONTEXT:
		switch (c[0])
		{
		case 'm':
			menucontext=MAINT_CONTEXT;
			break;

		case 'M':
			menucontext=MOVE_CONTEXT;
			break;

		case 't':
			menucontext=TEST_CONTEXT;
			break;

		case 'b':
			{
			unsigned long int	i,ii;
			int			op;
			unsigned int		ap,in,pr,de,mv;
			unsigned long 		mvc;
			char			buf[80];
			CString			*s;

				ap=in=pr=de=mv=mvc=0;
				for (i=0; i<100000; i++)
				{
					op=random();
					if (op>=0 && op<RAND_MAX * .20)
					{
						s= new CString;
						snprintf(buf,80,"ap=%lu",ap);
						s->SetString(buf);
						if (cq->Append((void*)(s)))  MyPrint(debugwin,"Append OK\n", TRUE); 
						else  MyPrint(debugwin,"Append FAILED\n", TRUE); 
						ap++;
						}
					if (op>=RAND_MAX*.20 && op<RAND_MAX * .40)
					{
						s= new CString;
						snprintf(buf,80,"in=%lu",in);
						s->SetString(buf);
						if (cq->Insert((void*)(s)))  MyPrint(debugwin,"Insert OK\n", TRUE); 
						else  MyPrint(debugwin,"Insert FAILED\n", TRUE); 
						in++;
						}
					if (op>=RAND_MAX*.40 && op<RAND_MAX * .60)
					{
						s= new CString;
						snprintf(buf,80,"pr=%lu",pr);
						s->SetString(buf);
						if (cq->Prepend((void*)(s)))  MyPrint(debugwin,"Prepend OK\n", TRUE); 
						else  MyPrint(debugwin,"Prepend FAILED\n", TRUE); 
						pr++;
						}
					if (op>=RAND_MAX*.60 && op<RAND_MAX * .80)
					{
					double  ran,ranm,cnt;
						ran=random();
						ranm=RAND_MAX;
						cnt=cq->GetCount();
						mvc=(unsigned long)(ran/ranm*cnt);
						cq->MoveFirst();
						snprintf(buf,80,"MoveNext=%lu\n",mvc);
						MyPrint(debugwin,buf, true); 
						ThreadLog(buf);
						for(ii=0;ii<mvc;ii++)
						{
							cq->MoveNext();
//							MyPrint(debugwin,"M",true);
							}
						mv++;
						}
					if (op>=RAND_MAX*.80 && op<RAND_MAX )
					{
					CString *s;
			
						MyPrint(debugwin,"Delete \n", TRUE);
						if (s=(CString*)(cq->GetItem())) 
						{
							// There was a string to delete
							delete s;
							MyPrint(debugwin,"Delete string OK\n", TRUE);
							if (cq->DelCurr())  MyPrint(debugwin,"Delete inode OK\n", TRUE); 
							}
						else 
						{
							MyPrint(debugwin,"No string to delete\n", TRUE);
							if (cq->DelCurr())  MyPrint(debugwin,"Delete inode OK\n", TRUE);
							else MyPrint(debugwin,"Delete inode FAILED\n", TRUE);
							}
						de++;
						}
					}
				snprintf (buf,80,"ap=%lu in=%lu pr=%lu de=%lu mv=%lu mvc=%lu \n",ap,in,pr,de,mv,mvc);
				MyPrint(mainwin,buf,true);	
				}
			break;

		case 'z':
			menucontext=END_CONTEXT;
			break;

		default:
			beep();                              //("Please try again\n");
			break;
			} // end MAIN_CONTEXT
		break;

	case MAINT_CONTEXT: 
		switch (c[0])
		{
		case 'a':	// APPEND
			{	
			CString *s;
				
				// GetAString makes a new string
				s=GetAString();
				if (cq->Append((void*)s))  MyPrint(debugwin,"Append OK\n", TRUE); 
				else  MyPrint(debugwin,"Append FAILED\n", TRUE); 
				}
			break;

		case 'p':	// PREPEND
			{	
			CString *s;
				
				// GetAString makes a new string
				s=GetAString();
				if (cq->Prepend((void*)s))  MyPrint(debugwin,"Prepend OK\n", TRUE); 
				else  MyPrint(debugwin,"Prepend FAILED\n", TRUE); 
				}
			break;

		case 'i':	// INSERT
			{	
			CString *s;
				
				// GetAString makes a new string
				s=GetAString();
				if (cq->Insert((void*)s))  MyPrint(debugwin,"Insert OK\n", TRUE); 
				else  MyPrint(debugwin,"Insert FAILED\n", TRUE); 
				}
			break;

		case 'd':
			{	
			CString *s;
	
				MyPrint(debugwin,"Delete \n", TRUE);
				if (s=(CString*)(cq->GetItem())) 
				{
					// There was a string to delete
					delete s;
					MyPrint(debugwin,"Delete string OK\n", TRUE);
					if (cq->DelCurr())  MyPrint(debugwin,"Delete inode OK\n", TRUE); 
					}
				else 
				{
					MyPrint(debugwin,"No string to delete\n", TRUE);
					if (cq->DelCurr())  MyPrint(debugwin,"Delete inode OK\n", TRUE);
					else MyPrint(debugwin,"Delete inode FAILED\n", TRUE);
					}
				}	
			break;

		case 'D':
			MyPrint(debugwin,"Delete Whole List Start\n", TRUE);
			cq->MoveFirst();
			while (!(cq->IsEmpty()))
			{
				delete (CString *)(cq->GetItem());
				// kind of a side effect DelCurr sets cur to head when deleting from head
				// so we dont need MoveNext
				cq->DelCurr();
//				cq->MoveNext();
				}
			MyPrint(debugwin,"Delete Whole List Complete\n", TRUE);
			break;

		case 'c':
			MyPrint(debugwin,"Change \n", TRUE);
			break;

		case 'z':
			menucontext=MAIN_CONTEXT;
			break;

		default:
			beep();                              //("Please try again\n");
			break;
			} // end MAINT_CONTEXT
		break;

	case MOVE_CONTEXT: 
		switch (c[0])
		{
		case 'f':
			MyPrint(debugwin,"Move forward \n", TRUE);
			cq->MoveNext();
			break;

		case 'b':
			MyPrint(debugwin,"Begin of list \n", TRUE);
			cq->MoveFirst();
			break;

		case 'e':
			MyPrint(debugwin,"End of list \n", TRUE);
			cq->MoveLast();
			break;

		case 'z':
			menucontext=MAIN_CONTEXT;
			break;

		default:
			beep();                              //("Please try again\n");
			break;
			} // end MOVE_CONTEXT
		break;

	case TEST_CONTEXT:
		switch (c[0])
		{
		case 'e':
			MyPrint(debugwin,"IsEmpty \n", TRUE);
			if (cq->IsEmpty()) MyPrint(mainwin,"Q is empty\n",true);
			if (!(cq->IsEmpty())) MyPrint(mainwin,"Q is not empty\n",true);
			break;

		case 'f':
			MyPrint(debugwin,"IsFull \n", TRUE);
			if (cq->IsFull()) MyPrint(mainwin,"Q is Full\n",true);
			if (!(cq->IsFull())) MyPrint(mainwin,"Q is not Full\n",true);
			break;

		case 'B':
			MyPrint(debugwin,"Begin of list \n", TRUE);
			if (cq->BegList())  MyPrint(mainwin,"Begin of list\n", TRUE); 
			else  MyPrint(mainwin,"Not Begin of list\n", TRUE); 
			break;

		case 'E':
			MyPrint(debugwin,"End of list \n", TRUE);
			if (cq->EndList())  MyPrint(mainwin,"End of list\n", TRUE); 
			else  MyPrint(mainwin,"Not End of list\n", TRUE); 
			break;

		case 'c':
			MyPrint(debugwin,"Count items in list \n", TRUE);
			if (!(cq->IsEmpty())) 
			{
			int	c;
			char buf[80];
				c=0;
				cq->MoveFirst();
				while (!(cq->EndList()))
				{
					c++;
					cq->MoveNext();
					}
				snprintf (buf,80,"Counting list => %ld\n",c);
				MyPrint(mainwin,buf,true);	
				
				}
			else
			{
				beep();
				}
			
			break;

		case 'C':
			MyPrint(debugwin,"GetCount of items in list. \n", TRUE);
			{
			char buf[80];
				snprintf (buf,80,"GetCount => %ld\n",cq->GetCount());
				MyPrint(mainwin,buf,true);	
				}
			break;

		case 'p':
			{
			char buf[80];
				MyPrint(debugwin,"Print current \n", TRUE);
				if (cq->GetItem()) // Remember that there may not be a current item.
				{
					snprintf (buf,80,"List item => %s\n",((CString*)(cq->GetItem()))->GetStringPtr());
					MyPrint(mainwin,buf,true);	
					}
				else  beep(); 
				}
			break;

		case 'P':
			MyPrint(debugwin,"Print items in list \n", TRUE);
			if (!(cq->IsEmpty())) 
			{
			char buf[80];
				cq->MoveFirst();
				while (!(cq->EndList()))
				{
					snprintf (buf,80,"List item => %s\n",((CString*)(cq->GetItem()))->GetStringPtr());
					MyPrint(mainwin,buf,true);	
					cq->MoveNext();
					}
				
				}
			else  beep(); 
			break;

		case 'z':
			menucontext=MAIN_CONTEXT;
			break;

		default:
			beep();                              //("Please try again\n");
			break;
			} // INSERT_CONTEXT
		break;

	break;



	}
} /////////////////////////////////////////////////////////////////////////////////////end ExecComm

///////////////////////////////////////////////////////////////////////////////////////////////////
// Main
///////////////////////////////////////////////////////////////////////////////////////////////////
int main(int argc, char** argv)
{
char		c[4];
void            *param;


	// init the screen & then threading and such
	// erase the screen
	Init(argc, argv);		// Init debug model
	InitScreen();	// Init the curses library & screen mutex

	// Init the screen
	clear();
	refresh();

	cq = new CQueue;

	cq->SetMaxCount(100000);
	// Start the listener
	// pack any params for the listener thread
	// tlist is a global thread tracking structure
	//c[0]=' ';
	do  
	{
		PrintMenu();
		c[0]=(char)getch();
		c[1]=0;
		ExecComm(c);
	} while ((menucontext!=END_CONTEXT)||(c[0]!='z'));
	MyPrint(debugwin,"Before DeInitScreen \n", TRUE);
	DeInitScreen();
	delete cq;
	printf("Returning from main.\n");
} //////////////////////////////////////////////////////////////////////////////////////////end main

