////////////////////////////////////////////////////////////////////////////////////////////////////
// Merged
// cpool.cpp
// This class will implement the thread pool 
// the socket
// 
// by Eric J Krejci
// Copyright (C) 2003 Eric J Krejci 
// Revised : 08 Apr 2006
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation version 2.
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
// Contact:
//              email           worldii@viagloria.com
//              web             www.viagloria.com
////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////
// Note: Some code derived from O'Rielly book: Programming Pthreads examples.
////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////
// Defines
////////////////////////////////////////////////////////////////////////////////////////////////////
//#define		BU FFSIZE	120

////////////////////////////////////////////////////////////////////////////////////////////////////
// Include
////////////////////////////////////////////////////////////////////////////////////////////////////
#include <pthread.h>
#include <errno.h>

#include "cpool.hpp"       
#include "worldiii.h"
#include "wiii_util.h"
////////////////////////////////////////////////////////////////////////////////////////////////////
// Externals
////////////////////////////////////////////////////////////////////////////////////////////////////
extern char             logfilename[];
extern pthread_attr_t	attr;

extern struct timeb	starttime;
extern void 		ThreadLog(char * s);



////////////////////////////////////////////////////////////////////////////////////////////////////
// LocalLock
// This is a wrapper for pthread lock that can print tid if required
////////////////////////////////////////////////////////////////////////////////////////////////////
//int CPool::LocalLock(pthread_mutex_t *mutex, bool MyP, char * who)
//{
//char buf[100];
//int r;
//
//	r=pthread_mutex_lock(mutex);
//
//	if (MyP)
//	{
//		snprintf(buf,100,"Thread = %09lu locked mutex. Caller = '%s' Success = %d. MUTEX=%lu",
//			(unsigned long)pthread_self(),who,r,(unsigned long)mutex);
//		ThreadLog(buf);
//
//		}
//	if (r) PError(errno);
//	return (r);
//	} /////////////////////////////////////////////////////////////////////////////end LocalLock

////////////////////////////////////////////////////////////////////////////////////////////////////
// LocalUnLock
// This is a wrapper for pthread lock that can print tid if required
////////////////////////////////////////////////////////////////////////////////////////////////////
//int CPool::LocalUnLock(pthread_mutex_t *mutex, bool MyP, char * who)
//{
//char buf[100];
//int r;
//
//	r=pthread_mutex_unlock(mutex);
//
//        if (MyP)
//        {
//		snprintf(buf,100,"Thread = %09lu unlocked mutex. Caller = '%s' Success = %d. MUTEX=%lu",
//			(unsigned long)pthread_self(),who,r,(unsigned long)mutex);
//		ThreadLog(buf);
//		}
//	if (r) PError(errno);
//	return (r);
//	} ///////////////////////////////////////////////////////////////////////////end LocalUnLock

////////////////////////////////////////////////////////////////////////////////////////////////////
// Lock
////////////////////////////////////////////////////////////////////////////////////////////////////
int CPool::Lock()
{
	// lock the mutex
//	return(LocalLock(q_lock,false,"CPool:: Lock"));
	return(MyLock(q_lock,false,"CPool:: Lock"));
} //////////////////////////////////////////////////////////////////////////////////////////end Lock

////////////////////////////////////////////////////////////////////////////////////////////////////
// UnLock
////////////////////////////////////////////////////////////////////////////////////////////////////
int CPool::UnLock()
{
	// lock the mutex
//	return(LocalUnLock(q_lock,false,"CPool:: UnLock")); 
	return(MyUnLock(q_lock,false,"CPool:: UnLock")); 
} ////////////////////////////////////////////////////////////////////////////////////////end UnLock


////////////////////////////////////////////////////////////////////////////////////////////////////
// CPool Constructor
////////////////////////////////////////////////////////////////////////////////////////////////////
CPool::CPool()
{
	// set state
	ThreadLog("CPool:: Constructor: Entered.\n");
	current_q_size=0;
	workq=0;
	q_closed=false;
	shutdown=false;
	ThreadLog("CPool:: Constructor: Exited.\n");
	} ///////////////////////////////////////////////////////////////////////////end Constructor

////////////////////////////////////////////////////////////////////////////////////////////////////
// CPool Destructor
////////////////////////////////////////////////////////////////////////////////////////////////////
CPool::~CPool()
{
	// delete the memory pointed to by buf
	if (LogFunc) (*LogFunc)("CPool:: Destructor: Entered.\n", extlogfile);
//	DeInit();
	if (LogFunc) (*LogFunc)("CPool:: Destructor: Exited.\n", extlogfile);
	} ////////////////////////////////////////////////////////////////////////////end Destructor

////////////////////////////////////////////////////////////////////////////////////////////////////
// ThreadProc
////////////////////////////////////////////////////////////////////////////////////////////////////
void *ThreadProc(void *tparm)
{
WORKCHUNK	*my_workp;
CPool		*pool;
CQueue		*wq;

	ThreadLog("CPool:: ThreadProc: Thread Entry.\n");
	// Put this proc into infinite loop
	while(1) 
	{
		// unpack the pointer to the object
		pool=(CPool*)tparm;
		wq=pool->workq;

		// lock the pool
		pool->Lock();
		while ((wq->IsEmpty())&&(!(pool->shutdown)))
		{
			pthread_cond_wait((pool->q_not_empty),(pool->q_lock));
			}

		// 
		if (pool->shutdown)
		{
			pool->UnLock();
			pthread_exit(NULL);
			}
	
		// get some work, and dec counter
		if (!(wq->IsEmpty()))
		{
			my_workp=(WORKCHUNK*)(wq->MoveFirst());	// MoveFirst returns (void*)data pointer
			wq->DelCurr();				// DelCurr does NOT delete data item
			}

		// Handle waiting add_work threads 
		if (!(pool->dont_block) && !(wq->IsFull()))
		{
		int rtn;
	
			rtn = pthread_cond_broadcast(pool->q_not_full);
			if (rtn != 0) 
			{
				ThreadLog("CPool:: ThreadProc: Error in pthread broadcast q_not_full.");
				exit(1);
				}
			}
	
		// Handle waiting destroyer threads 
		if (wq->IsEmpty())
		{
		int rtn;
	
			rtn = pthread_cond_signal(pool->q_empty);
			if (rtn != 0) 
			{
				ThreadLog("CPool:: ThreadProc: Error in pthread signal q_empty.");
				exit(1);
				}
			}
		pool->UnLock();

		// Do this work item 
		(*(my_workp->routine))((void*)my_workp);
		delete my_workp;	// remember to delete things, not FREE them
		}
	} ////////////////////////////////////////////////////////////////////////////end ThreadProc

////////////////////////////////////////////////////////////////////////////////////////////////////
// CPool::Init
// Initialize the instance
// Because allocate mem here, need to release it in close
// We receive the data into a local buffer first because we call receive here
////////////////////////////////////////////////////////////////////////////////////////////////////
bool CPool::Init(int num_threads, int max_q, bool dbflag)
{
int			i;
int			rtn;	// used as return val for thread creation
//pthread_mutexattr_t 	m_attr;	// usd to help init mutecies


	ThreadLog("CPool:: Init: Entered.\n");
	q_lock = new pthread_mutex_t;
	q_not_empty = new pthread_cond_t;
	q_not_full  = new pthread_cond_t;
	q_empty     = new pthread_cond_t;
	workq       = new CQueue;

	if (!q_lock) return (false);
	if (!q_not_empty) return (false);
	if (!q_not_full) return (false);
	if (!q_empty) return (false);
	if (!workq) return(false);

//	pthread_mutexattr_init(&m_attr);
	
//#ifdef LINUX
//	pthread_mutexattr_settype(&m_attr,PTHREAD_MUTEX_ERRORCHECK_NP);
//#endif	

//	pthread_mutex_init(q_lock, &m_attr);
	InitMutex(q_lock);
	pthread_cond_init(q_not_empty,NULL);
	pthread_cond_init(q_not_full,NULL);
	pthread_cond_init(q_empty,NULL);

	// Initialize the pool
	threadcount=num_threads;
	workq->SetMaxCount(max_q);
	dont_block=dbflag;

	// allocate space to store thread handles
	thread_array= new pthread_t[threadcount];
	if (!thread_array) 
	{
		ThreadLog("CPool:: Init: Failed to allocate thread space.\n");
		exit(1);
		}

	// setup the parm for the thread
	thread_parm = this;
	
	for (i=0;i<threadcount;i++)
	{
		// create threads
//		rtn=pthread_create(&(thread_array[i]), NULL, ThreadProc, (void *)thread_parm);
		rtn=CreateThread(&(thread_array[i]), ThreadProc, (void *)thread_parm);
		if (rtn!=0)
		{
			ThreadLog("CPool:: Init: Failed to create threads.\n");
			exit(1);
			}
		}
	ThreadLog("CPool:: Init: Exited.\n");

	return (true);
	} //////////////////////////////////////////////////////////////////////////////////end Init

////////////////////////////////////////////////////////////////////////////////////////////////////
// CPool::Destroy
// Destroy and shutdown
////////////////////////////////////////////////////////////////////////////////////////////////////
int CPool::Destroy()
{
int		i,rtn;

	// Lock the object
	Lock();

	// Is a shutdown already in progress?
	if (q_closed || shutdown)
	{
		UnLock();
		return(0);
		}

	// If not set it
	q_closed=true;

	// If finish is set wait for threads to finish
	if (finish==true)
	{
		while ((workq->GetCount()) > 0)
		{
			rtn=pthread_cond_wait(q_empty,q_lock);
			if (rtn!=0)
			{
				ThreadLog("CPool:: Destroy: Failed to wait on q_empty.");
				exit(1);
				}
			
			}
		}

	// Now that all threads are finished 
	shutdown=true;
	UnLock();
	
	// Wake up any workers so they recheck their shutdown flag
	rtn=pthread_cond_broadcast(q_not_empty);
	if (rtn!=0)
	{
		ThreadLog("CPool:: Destroy: Failed to broadcast on q_not_empty.");
		exit(1);
		}
	rtn=pthread_cond_broadcast(q_not_full);
	if (rtn!=0)
	{
		ThreadLog("CPool:: Destroy: Failed to broadcast on q_not_empty.");
		exit(1);
		}

	// wait for threads to exit
	for (i=0; i<threadcount; i++)
	{
		rtn=pthread_join(thread_array[i],NULL);
		if (rtn!=0)
		{
			ThreadLog("CPool:: Destroy: Failed to join threads.");
			exit(1);
			}
		}

	delete [] thread_array;

	// walk the list and delete all
	workq->MoveFirst();
	while (!(workq->IsEmpty()))
	{
		delete (WORKCHUNK *)(workq->GetItem());	// the queue cannot delete its data item
		workq->DelCurr();			// Delete the queue entry, when delete head curr
							// points again to head
		}

	return (0);
	} ///////////////////////////////////////////////////////////////////////////////end Destroy

////////////////////////////////////////////////////////////////////////////////////////////////////
// CPool::AddWork
// AddWork adds an item to the incoming work queue
////////////////////////////////////////////////////////////////////////////////////////////////////
int CPool::AddWork(void (*routine)(void*), CServer *cs, char *pkt, int pktlen)
{
int		rtn;
WORKCHUNK	*workp;

	// Lock the pool
	Lock();

	// no space and don't want to wait 
	if ((workq->IsFull()) && dont_block) 
	{
		UnLock();
		ThreadLog("CPool:: AddWork: Exiting through queue full.");
		return(-1);
		}

	// no space and ok to wait
	while((workq->IsFull()) && (!shutdown) && (!q_closed)) 
	{
		rtn=pthread_cond_wait(q_not_full, q_lock);
		if (rtn != 0) fprintf(stderr,"pthread_cond_wait %d",rtn), exit(1);
		{
			ThreadLog("CPool:: AddWork: Failed to wait for q_not_full.");
			exit(1);
			}
		}

	// is the pool in the process of being destroyed  ?
	if (shutdown || q_closed) 
	{
		UnLock();
		ThreadLog("CPool:: AddWork: Pool being destroyed.");
		return (-1);
		}

	// allocate work structure and fill it
	workp = new WORKCHUNK;
	if (workp==0) 
	{
		ThreadLog("CPool:: AddWork: New failed creating workp.");
		exit(1);
		}
	workp->routine = routine;
	workp->server = cs;
	workp->next = 0;

	// adding work
	ThreadLog("CPool:: AddWork: Adding work to pool queue.");
	workq->Append((void*)workp);

	// Broadcast pool not empty
	rtn = pthread_cond_broadcast(q_not_empty);
	if (rtn != 0)
	{
		ThreadLog("CPool:: AddWork: Failed to brodcast q_not_empty.");
		exit(1);
		}

	UnLock();
	
	return (0);

	} ///////////////////////////////////////////////////////////////////////////////end AddWork



