////////////////////////////////////////////////////////////////////////////////////////////////////
// Merged
// cpool.hpp
// This class will implement the thread pool.
// 
// by Eric J Krejci
// Copyright (C) 1999 Eric J Krejci 
// Revised : 26 Jan 2006
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation version 2.
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
// Contact:
//              email           worldii@viagloria.com
//              web             www.viagloria.com
////////////////////////////////////////////////////////////////////////////////////////////////////
#if !defined(_CPOOL_HPP_)  //skip this stuff if it has already been seen
#define _CPOOL_HPP_        //by the compiler

////////////////////////////////////////////////////////////////////////////////////////////////////
// Includes
////////////////////////////////////////////////////////////////////////////////////////////////////
#include <netinet/in.h>
#include <sys/socket.h>
#include <netdb.h>
#include <stdlib.h>

#include "worldiii.h"
//#include "wiii_types.h"
#include "wiii_structures.h"
#include "cserver.hpp"
#include "cqueue.hpp"

////////////////////////////////////////////////////////////////////////////////////////////////////
// Defines
// These are class specific defines
////////////////////////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////////////////////////
// Structures
////////////////////////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////////////////////////
// CPool class
////////////////////////////////////////////////////////////////////////////////////////////////////
class CPool
{
friend void	*ThreadProc(void *);		// This is the routine for managing a port
public:
			CPool();
			~CPool();
	bool		Init(int, int, bool);	// to have a non fail constructor, 
	int		Destroy();		// To shutdown and destroy a CPool
	int		AddWork(
				void (*routine)(void*), 
				CServer *,	// ptr to CServer
				char *,		// ptr to packet
				int 		// packet length
				);	// To add a work item to the pool queue

private:
	int		Lock();
	int		UnLock();
	int 		LocalLock(pthread_mutex_t *mutex, 
				bool MyP, 
				char * who
				);
	int 		LocalUnLock(pthread_mutex_t *mutex, 
				bool MyP, 
				char * who
				);
        void    	SetLogFunc(
				int (*pfunc)(char*,char*),
				char *
				);                      // sets the logging function to call,

	int		threadcount;	// how many threads are currently created
	bool		dont_block;	// flag, dont block on full queue
	
	// pool state
	int		current_q_size;	// the current number of items in queue
	CQueue		*workq;		// to replace the queue code
	pthread_mutex_t	*q_lock;		// lock the queue for update
	pthread_cond_t	*q_not_empty;	// 
	pthread_cond_t	*q_not_full;	//
	pthread_cond_t	*q_empty;	//
	bool		q_closed;	// 
	bool		shutdown;	// we are closing down for server shutdown
	bool		finish;		// allow threads to finish what they are doing
	pthread_t	*thread_array;	// Array to store all the thread handles in

			
        char            locallogfile[LOGFILENAMELEN];   // This is loaded with the correct logfile for each thread
        char            extlogfile[LOGFILENAMELEN];     // This is set to the ext thread logfile

        int             (*LogFunc) (char * s, char * log);      // Pointer to the logging function
	CPool		*thread_parm;			// a pointer to the master pool object
	
	};

#endif //_CPOOL_HPP_

