////////////////////////////////////////////////////////////////////////////////////////////////////
// cserver.cpp
// This class will implement all the per player info 
// 
// by Eric J Krejci
// Copyright (C) 1999 Eric J Krejci 
// Revised : 26 Jan 2006
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation version 2.
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
// Contact:
//              email           worldii@viagloria.com
//              web             www.viagloria.com
////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////
// Defines
////////////////////////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////////////////////////
// Include
////////////////////////////////////////////////////////////////////////////////////////////////////
#include <string.h>

#include "rudp.h"
#include "cstring.hpp"       
#include "cserver.hpp"       
#include "worldiii.h"       
#include "wiii_util.h"

////////////////////////////////////////////////////////////////////////////////////////////////////
// Externals
////////////////////////////////////////////////////////////////////////////////////////////////////
extern	char		logfilename[];
extern	CUSock		*servsock;
extern	int		loglevel;

////////////////////////////////////////////////////////////////////////////////////////////////////
// CServer Constructor
////////////////////////////////////////////////////////////////////////////////////////////////////
CServer::CServer()
{
	session=0;
	port=0;
	IP=0;
	playername=0;
	charactername=0;
//	current_message=0;
	head=0;
	tail=0;
	ce=0;
	} ///////////////////////////////////////////////////////////////////////////end Constructor

////////////////////////////////////////////////////////////////////////////////////////////////////
// CServer Destructor
////////////////////////////////////////////////////////////////////////////////////////////////////
CServer::~CServer()
{
//	itemcount--;  //decrease the item count
	} ///////////////////////////////////////////////////////////////////////////end ConnectWait

////////////////////////////////////////////////////////////////////////////////////////////////////
// Init
////////////////////////////////////////////////////////////////////////////////////////////////////
bool CServer::Init()
{
	ce = new CEngine; 			if (!ce) return false;
	playername = new CString;		if (!playername) return false;
	charactername = new CString;		if (!charactername) return false;
	
	return (true);
	} //////////////////////////////////////////////////////////////////////////////////end Init

////////////////////////////////////////////////////////////////////////////////////////////////////
// NewServer
////////////////////////////////////////////////////////////////////////////////////////////////////
bool CServer::NewServer(connecttype *cparm)
{

	// set some basic parms
	session	= cparm->session;        // session ID
	port	= cparm->port;           // port of client
	IP	= cparm->IP;             // IP address of client
 
	return (true);
	} /////////////////////////////////////////////////////////////////////////////end NewServer

////////////////////////////////////////////////////////////////////////////////////////////////////
// GetServerConn
////////////////////////////////////////////////////////////////////////////////////////////////////
bool CServer::GetServerConn(connecttype *cparm)
{

	// allocate some memory for the return value
	if (!(cparm)) return (false);

	// set some basic parms
	(cparm)->session	= session;        // session ID
	(cparm)->port		= port;           // port of client
	(cparm)->IP		= IP;             // IP address of client
 
	return (true);
	} /////////////////////////////////////////////////////////////////////////end GetServerConn

////////////////////////////////////////////////////////////////////////////////////////////////////
// DelMessage
////////////////////////////////////////////////////////////////////////////////////////////////////
bool CServer::DelMessage()
{
	// point to the location of the message
	if (current_message) 
	{
		delete current_message;
		current_message=0;
		return true;
		}
	else
	{
		return false;
		}
	} ////////////////////////////////////////////////////////////////////////////end DelMessage

////////////////////////////////////////////////////////////////////////////////////////////////////
// SetPlayerName
////////////////////////////////////////////////////////////////////////////////////////////////////
void CServer::SetPlayerName()
{
char buf[80];

	// point to the location of the message
	if (loglevel>=1) ThreadLog("CServer::SetPlayerName: Entered.");
	playername->SetString(current_message->GetParm(0));
	snprintf(buf,80,"CServer::SetPlayerName: PlayerName='%s'",current_message->GetParm(0));
	if (loglevel>=2) ThreadLog(buf);
	if (loglevel>=1) ThreadLog("CServer::SetPlayerName: Exited.");
	} /////////////////////////////////////////////////////////////////////////end SetPlayerName

////////////////////////////////////////////////////////////////////////////////////////////////////
// SetCharacterName
////////////////////////////////////////////////////////////////////////////////////////////////////
void CServer::SetCharacterName()
{
char buf[80];

	// point to the location of the message
	if (loglevel>=1) ThreadLog("CServer::SetCharacterName: Entered.");
	charactername->SetString(current_message->GetParm(0));
	snprintf(buf,80,"CServer::SetCharacterName: CharacterName='%s'",current_message->GetParm(0));
	if (loglevel>=2) ThreadLog(buf);
	if (loglevel>=1) ThreadLog("CServer::SetCharacterName: Exited.");
	} //////////////////////////////////////////////////////////////////////end SetCharacterName

////////////////////////////////////////////////////////////////////////////////////////////////////
// SetMessage
////////////////////////////////////////////////////////////////////////////////////////////////////
void CServer::SetMessage(CMessage *m)
{
	// point to the location of the message
	current_message=m;
	} ////////////////////////////////////////////////////////////////////////////end SetMessage

////////////////////////////////////////////////////////////////////////////////////////////////////
// GetMessageErrorType
////////////////////////////////////////////////////////////////////////////////////////////////////
int CServer::GetMessageErrorType()
{
	// point to the location of the message
//	return (current_message);
	if (current_message) return (current_message->GetErrorType());
	else return(CMSG_NOMSG);

	} ////////////////////////////////////////////////////////////////////////////end GetMessage

////////////////////////////////////////////////////////////////////////////////////////////////////
// GetMessage
////////////////////////////////////////////////////////////////////////////////////////////////////
int CServer::GetMessage()
{
	// point to the location of the message
//	return (current_message);
	if (current_message) return (current_message->GetMessage());
	else return(WIIIM_UNHANDLED);

	} ////////////////////////////////////////////////////////////////////////////end GetMessage

////////////////////////////////////////////////////////////////////////////////////////////////////
// GetCharacterName
////////////////////////////////////////////////////////////////////////////////////////////////////
char *CServer::GetCharacterName()
{
	// return a ptr to the string
	return (charactername->GetStringPtr());

	} //////////////////////////////////////////////////////////////////////end GetCharacterName

////////////////////////////////////////////////////////////////////////////////////////////////////
// GetPlayerName
////////////////////////////////////////////////////////////////////////////////////////////////////
char *CServer::GetPlayerName()
{
	// return a ptr to the string
	return (playername->GetStringPtr());

	} /////////////////////////////////////////////////////////////////////////end GetPlayerName

////////////////////////////////////////////////////////////////////////////////////////////////////
// SendToClient
// This send a message to the client for this instance
// The message should be a properly formed string, ie char array, NULL termed. 
////////////////////////////////////////////////////////////////////////////////////////////////////
bool CServer::SendToClient(CString *msg)
{
	return(msg->GetStringPtr());
	} //////////////////////////////////////////////////////////////////////////end SendToClient

////////////////////////////////////////////////////////////////////////////////////////////////////
// SendToClient
// This send a message to the client for this instance
// The message should be a properly formed string, ie char array, NULL termed. 
////////////////////////////////////////////////////////////////////////////////////////////////////
bool CServer::SendToClient(char *msg)
{
struct sockaddr_in      cliaddr;        // peer connection info of client
socklen_t               cliaddrlen;
int			result;

	// get the connection info
//	cs->GetServerConn(&pcon);
 
	// setup to send response
	cliaddr.sin_addr.s_addr=IP;
	cliaddr.sin_port=port;
	cliaddrlen=sizeof(cliaddr);

	// send message
	result=servsock->Send(msg,strlen(msg),0,&cliaddr, cliaddrlen, session);
	if (result==RUDP_OK) return(true);
	else return (false);
	
	} //////////////////////////////////////////////////////////////////////////end SendToClient

////////////////////////////////////////////////////////////////////////////////////////////////////
// GetMessageParm
// This returns a string pointer to the message parm indicated 
////////////////////////////////////////////////////////////////////////////////////////////////////
char *CServer::GetMessageParm(int i)
{
	// point to the location of the message
	if (current_message)  return (current_message->GetParm(i)); 
	else  return (NULL); 
	
	} ////////////////////////////////////////////////////////////////////////end GetMessageParm

////////////////////////////////////////////////////////////////////////////////////////////////////
// GetMessageParm
// This returns a string pointer to the message parm indicated 
////////////////////////////////////////////////////////////////////////////////////////////////////
CString *CServer::GetMessageParmString(int i)
{
	// point to the location of the message
	if (current_message)  return (current_message->GetParmCS(i)); 
	else  return (NULL); 
	
	} ////////////////////////////////////////////////////////////////////////end GetMessageParm

