////////////////////////////////////////////////////////////////////////////////////////////////////
// cserver.hpp
// This class will implement the server that handles all per player info
// 
// by Eric J Krejci
// Copyright (C) 1999 Eric J Krejci 
// Revised : 26 Jan 2006
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation version 2.
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
// Contact:
//              email           worldii@viagloria.com
//              web             www.viagloria.com
////////////////////////////////////////////////////////////////////////////////////////////////////
#if !defined(_CSERVER_HPP_)  //skip this stuff if it has already been seen
#define _CSERVER_HPP_        //by the compiler

////////////////////////////////////////////////////////////////////////////////////////////////////
// Includes
////////////////////////////////////////////////////////////////////////////////////////////////////
//#include "wiii_types.h"
#include "wiii_structures.h"
#include "cstring.hpp"
#include "citem.hpp"
#include "cmessage.hpp"
#include "cengine.hpp"

////////////////////////////////////////////////////////////////////////////////////////////////////
// Defines
// These are class specific defines
////////////////////////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////////////////////////
// Structures
////////////////////////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////////////////////////
// CServer class
////////////////////////////////////////////////////////////////////////////////////////////////////
class CServer : public CItem
{
public:
			CServer();
			~CServer();
	bool		Init();				// to have a non fail constructor, 
	bool		NewServer(
				connecttype *
				);			// Finish initializing a server

	bool 		GetServerConn(
				connecttype *cparm
				);
	void		SetPlayerName();		// Uses the current message to set theplayer name
	void		SetCharacterName();		// Uses the current message to set theplayer name
	void		SetMessage(			// attach a message to cserver for the 
							// worker thread to process
				CMessage *		// the worker thread to process
				);
//	CMessage	*GetMessage();			// return ptr to message
	int		GetMessage();			// return message
	char		*GetMessageParm(int);		// get pointer to string 
	CString		*GetMessageParmString(int);	// get pointer to string 
	int		GetMessageErrorType();		// return message error type
	char		*GetCharacterName();		// return the name of the character
	char		*GetPlayerName();		// return the name of the player
	bool		DelMessage();			// del and return whether there was a message to del
	bool		SendToClient(char *msg);	// Send the given message to my client			
	bool		SendToClient(CString *msg);	// Send the given message to my client			

private:
	CString		*playername;			// a single player can have many characters
	CString		*charactername;			// a single player can have many characters

	CMessage	*current_message;		// the message for the current cserver thread 
	CMessage	*head;				// the message q head for the current cserver thread 
							// messages are removed from head
							// THIS is the "current_message"
	CMessage	*tail;				// the message q tail for the current cserver thread 
							// messages are added to tail

	// connection info
	unsigned long	session;			// session ID
	in_port_t	port;				// port of client
	in_addr_t	IP;				// IP address of client

	// script object
	CEngine		*ce;				// script engine
	};

#endif //_CSERVER_HPP_

