////////////////////////////////////////////////////////////////////////////////////////////////////
// dispatcher.cpp - This module implements the thread that listens for 
// connection requests and then starts the portmon, qloader, and server
// threads.
//
// dispatcher module
// 
// by Eric J Krejci
// Copyright (C) 1999 Eric J Krejci 
// Revised : 10 May 2007
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation version 2.
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
// Contact:
//              email           worldii@viagloria.com
//              web             www.viagloria.com
////////////////////////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////////////////////////
// Includes
////////////////////////////////////////////////////////////////////////////////////////////////////
#include <sys/socket.h>
#include <sys/errno.h>
#include <errno.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <pthread.h>
#include <curses.h>
#include <netinet/in.h>
#include <netdb.h>
#include <errno.h>
#include <arpa/inet.h>
#ifdef SOLARIS
#include <strings.h>
#endif

//#include "wiii_types.h"
#include "wiii_structures.h"
#include "worldiii.h"
#include "cusock.hpp"
#include "cthread.hpp"
#include "cpool.hpp"
#include "cserver.hpp"
#include "wiii_util.h"

////////////////////////////////////////////////////////////////////////////////////////////////////
// Defines
////////////////////////////////////////////////////////////////////////////////////////////////////
#define		MAXCLIENTCOUNT	10
#define         BUFFSIZE        120
#define		MAXWORKERS	10
#define		MAXQUEUELEN	100	 // 10 workers, 100 msgs max in queue, 

////////////////////////////////////////////////////////////////////////////////////////////////////
// Local Globals
////////////////////////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////////////////////////
// External Globals
////////////////////////////////////////////////////////////////////////////////////////////////////
extern	int		listenstate;
extern  char		logfilename[];
extern 	CPool           *pool;                  // this object managers the worker threads
extern	int		loglevel;

extern  CThreadList	tlist;
extern  CIterator	serverlist;
extern  CUSock		*servsock;
extern	void		ThreadLog(char * s);
extern	int		Parse(char *rawbuf,int length, CPool *pool, CServer *s);
extern  pthread_mutexattr_t	m_attr;
extern	void		ConPrint(char *);
extern	void		ConPrint(CString *);

	
////////////////////////////////////////////////////////////////////////////////////////////////////
// ShowServers
////////////////////////////////////////////////////////////////////////////////////////////////////
void ShowServers(CIterator *s)
{
connecttype	pcon;
char		buf[BUFFSIZE];
char		dst[BUFFSIZE];

	// list all the servers for debug purposes
	s->MoveFirst();
	while (!(s->EndList()))
	{
		if (!(((CServer*)(s->GetCurrent()))->GetServerConn(&pcon))) return;
		snprintf(buf,BUFFSIZE,"ShowServers: Connection IP= %s session= %lu port=%d.\n",
			inet_ntop(AF_INET,(void*)&(pcon.IP),dst,BUFFSIZE),
			pcon.session,
			ntohs(pcon.port)
			);
		if (loglevel>=1) ThreadLog(buf);
//		if (loglevel>=1) ConPrint(buf);
		ConPrint(buf);
		s->MoveNext();
		}

	} ///////////////////////////////////////////////////////////////////////////end ShowServers

////////////////////////////////////////////////////////////////////////////////////////////////////
// WorkerSend
////////////////////////////////////////////////////////////////////////////////////////////////////
void WorkerSend(void *ptr)
{
WORKCHUNK		*wc;
struct sockaddr_in 	cliaddr;	// peer connection info of client
socklen_t		cliaddrlen;
CServer			*cs;
connecttype		pcon;
unsigned long		session;
char 			*mesg;
int			n;

	// unpack our parm
	if (loglevel>=1) ThreadLog("WorkerSend: Entered.");
	wc=(WORKCHUNK *)ptr;
	cs=wc->server;
	
	// get the connection info
	cs->GetServerConn(&pcon);

	// setup to send response
	cliaddr.sin_addr.s_addr=pcon.IP;
	cliaddr.sin_port=pcon.port;
	cliaddrlen=sizeof(cliaddr);
	session=pcon.session;
	mesg=wc->packet;
	n=wc->pcktlen;

	// send response
	servsock->Send(mesg,n,0,&cliaddr, cliaddrlen, session);

	// free the mem for the packet
	delete [] mesg;  // the WorkerSend will delete for now
	
	if (loglevel>=1) ThreadLog("WorkerSend: Exited.");
	} ////////////////////////////////////////////////////////////////////////////end WorkerSend

////////////////////////////////////////////////////////////////////////////////////////////////////
// DisConnectCallBack
////////////////////////////////////////////////////////////////////////////////////////////////////
int DisConnectCallBack(connecttype *cparm)
{
connecttype	pcon;
bool		found;

        // save the function pointer to be used for logging
	if (loglevel>=1) ThreadLog("DisConnectCallBack: Removing connection before ConPrint.");
	if (loglevel>=1) ConPrint("DisConnectCallBack: Removing connection.\n");

	// find the server to delete
	serverlist.MoveFirst();
	found=false;
	while (!(serverlist.EndList())&&!found)
	{
		if (!(((CServer*)(serverlist.GetCurrent()))->GetServerConn(&pcon))) return(0);
		if ((pcon.IP==cparm->IP)&&(pcon.session==cparm->session))
		{
			serverlist.DeleteItem(serverlist.GetCurrent());
			found=true;
			}
		else serverlist.MoveNext();
		}

	// show the servers
	ShowServers(&serverlist);                                                                                                                                 
	// we have to delete the cparm memory chunk, as the protocol will not
	delete cparm;

	if (loglevel>=1) ThreadLog("DisConnectCallBack: Exiting.");
	return (0);	
        } ////////////////////////////////////////////////////////////////////end DisConnectCallBack

////////////////////////////////////////////////////////////////////////////////////////////////////
// ConnectCallBack
// Note: as this routine and the disconnect callback routine will always be called by the PortMan
// thread it is safe for them to both use the same (default) cursor in the serverlist iterator.
// Every other thread that does anything with the serverlist should get its own cursor.
// Note: each worker thread will have to get its own cursor because other wise one thread could 
// mess up anothers search.
////////////////////////////////////////////////////////////////////////////////////////////////////
int ConnectCallBack(connecttype *cparm)
{
char		buf[BUFFSIZE];
CServer		*newserver;

        // save the function pointer to be used for logging
	if (loglevel>=1) ThreadLog("ConnectCallBack: New Connection before ConPrint.");
	if (loglevel>=1) ConPrint("ConnectCallBack: Creating New CServer and Connection.\n");

	// create a new server object to manage the connection to the player/user
	newserver=new CServer;
	newserver->Init();
	if (loglevel>=2) 
	{
		snprintf(buf, BUFFSIZE, "Listener: Adding server newserver = %lu ", (unsigned long)newserver);
		ThreadLog(buf);
		}

	serverlist.InsertItem(newserver);
	
	// set the new server's connection parameters
	newserver->NewServer(cparm);

	// show the servers
	ShowServers(&serverlist);                                                                                                                                 
	// we have to delete the cparm memory chunk, as the protocol will not
	delete cparm;

	if (loglevel>=1) ThreadLog("ConnectCallBack: Exiting.");
	return (0);	
        } ///////////////////////////////////////////////////////////////////////end ConnectCallBack

////////////////////////////////////////////////////////////////////////////////////////////////////
// Dispatcher
// Remember that CServers get allocated by the Callback Functions
// That way once 
// - a connection is made 
// - cserver created via callback
// - a packet gets here, 
// - there is a cserver ready for it
////////////////////////////////////////////////////////////////////////////////////////////////////
void *Dispatcher(void *ptr)
{
struct sockaddr_in 	servaddr;	// temp var used to setup the socket
struct sockaddr_in 	cliaddr;	// peer connection info of client
socklen_t		cliaddrlen;
int			n;		// count returned from Recv() of received chars
unsigned long		session;		// count returned from Recv() of received chars
char			*mesg;
CServer			*s;
WIIHANDLE		disp_curs;	// cursor for dispatcher thread to use
bool			found;		// flag to help is searching for correct cserver 
connecttype		pcon;
int			result;

	ThreadLog("Dispatcher: Thread Entry.\n");

	// get a private cursor from serverlist
	disp_curs=serverlist.GetCursor((unsigned long)pthread_self());	// no other thread will have this id
							// so the dispatcher can use it to get his own cursor
	
	// set up main CUSock for communicating with all clients
	servsock->SetLogFunc(ThreadLog);
//	servsock->Init(&m_attr);
	servsock->Init();
	bzero(&servaddr, sizeof(servaddr));
	servsock->SetConnectFunc(ConnectCallBack);
	servsock->SetDisConnFunc(DisConnectCallBack);
	servaddr.sin_addr.s_addr = htonl(INADDR_ANY);
	servaddr.sin_port        = htons(BASEWORLDPORT);
	servaddr.sin_family      = AF_INET;
	servsock->Create(&servaddr,sizeof(servaddr));
	if (!servsock->Bind())
	{
		ThreadLog("Dispatcher: Bind Failed. Exiting.");
		return (0);
		}

	// start listening
	servsock->Listen();

	// create and init thread pool
	pool = new CPool;
	pool->Init(MAXWORKERS,MAXQUEUELEN,true);	// 10 workers, 100 msgs max in queue, dont block. IE fail
							// if the queue is full
	
	///////////////////////////////////////
	// start main dispatcher loop
	///////////////////////////////////////
	while (1)
	{	
		// receive a packet
		if (loglevel>=3) ThreadLog("Dispatcher: Go get another packet.");
		n=servsock->Recv(&mesg, 0, &cliaddr, &cliaddrlen, &session);

		// search all the servers for matching IP
		serverlist.MoveFirst(disp_curs);
		found=false;
		if (loglevel>=3) ThreadLog("Dispatcher: Search for its server.");
		while (!serverlist.EndList(disp_curs) && !found)
		{
			// get the current server's connection info
			if (!(((CServer*)(serverlist.GetCurrent(disp_curs)))->GetServerConn(&pcon))) return(0);

			// comparing pcon and cliaddr
			if ((pcon.IP==cliaddr.sin_addr.s_addr)&&(pcon.session==session))  found=true; 
			else serverlist.MoveNext(disp_curs);
			}

		// if we found a server
		if (found) 
		{
			if (loglevel>=1) ConPrint("Dispatcher: Found a Server for the message.\n");
			s=(CServer*)serverlist.GetCurrent(disp_curs);

			// now add work to pool queue
			result=Parse(mesg,n,pool,s);
			
			// we have to delete the packet, the protocol will not. 
			delete [] mesg;	
			
			if (loglevel>=1) ConPrint("Dispatcher: After Parse.\n");
			} // end if (found)
		else
		{
			ThreadLog("Dispatcher: Packet without a server.");
			ConPrint("Dispatcher: ERROR: Packet without a server.\n");
			delete [] mesg;	
			}
		pthread_yield();
		} // end while (1)

	return (0);
	} ////////////////////////////////////////////////////////////////////////////end Dispatcher


