////////////////////////////////////////////////////////////////////////////////////////////////////
// server.cpp -- This is it.
// This is the main module for the first real attempt to put together my 3d world game.
// This will spawn the listener then become the console listener thread.
//   
// World III
//
// by Eric J Krejci
// Copyright (C) 1999 Eric J Krejci 
// Revised : 26 Jan 2006
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation version 2.
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
// Contact:
//              email           worldii@viagloria.com
//              web             www.viagloria.com
////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////
// Includes
////////////////////////////////////////////////////////////////////////////////////////////////////
#include <sys/timeb.h>
#include <pthread.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <arpa/inet.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#ifdef SOLARIS
#include <strings.h>
#endif

#include "rudp.h"
#include "cusock.hpp"
#include "wiii_util.h"
#include "citem.hpp"
#include "cthread.hpp"
#include "cpool.hpp"


////////////////////////////////////////////////////////////////////////////////////////////////////
// Globals
////////////////////////////////////////////////////////////////////////////////////////////////////
CMessage		initheclass;
pthread_mutex_t		logmutex;
//pthread_mutexattr_t	m_attr;
FILE *          	logfile;
//char			logfilename[50]="logs/worldiii.log";
unsigned long		logcount;
pthread_mutex_t 	screenmutex;
bool			consoleconnected;       // tells when we are connected to console.
unsigned long		consession;		// Console session
sockaddr_in		consoleaddr;		// Console IP address
socklen_t		consoleaddrlen;		// Console addr len
CUSock			*consock;                // This is the console socket
CUSock			*servsock;		// used by listener

bool			listenstate;		// This puts server into debug mode 
//bool			debug;			// This puts server into debug mode 
pthread_attr_t		attr;			// thread attr structure for setting attributes.
bool			donelistening;
struct timeb    	starttime;
CThreadList     	tlist;			// All threads will get tracked via this structure
CPool			*pool;			// this object managers the worker threads
	
CIterator		serverlist;		// A list of all the cserver objects. These track the currently
					// logged in players.
int			loglevel;		// sets level of logging 0 lowest= no logging, 
						// 9 highest insanely verbose

////////////////////////////////////////////////////////////////////////////////////////////////////
// External Functions
////////////////////////////////////////////////////////////////////////////////////////////////////
extern  void 	*Dispatcher(void *p);
extern	bool 	ConServer(char *data, int count);
extern	void	ThreadLog(char * s);

////////////////////////////////////////////////////////////////////////////////////////////////////
// Defines
////////////////////////////////////////////////////////////////////////////////////////////////////
//#define         BU FFSIZE        120

////////////////////////////////////////////////////////////////////////////////////////////////////
// Code
////////////////////////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////////////////////////
// ConPrint
////////////////////////////////////////////////////////////////////////////////////////////////////
void ConPrint(CString *s)
{

	// send stuff out the socket only if connected.
	if (consoleconnected)
	{
		consock->Send(s->GetStringPtr(), s->GetLength(),0,&consoleaddr,consoleaddrlen, consession);
		}
	} //////////////////////////////////////////////////////////////////////////////end ConPrint

////////////////////////////////////////////////////////////////////////////////////////////////////
// ConPrint
////////////////////////////////////////////////////////////////////////////////////////////////////
void ConPrint(char *s )
{

	// send stuff out the socket only if connected.
	if (consoleconnected)
	{
		consock->Send(s, strlen(s),0,&consoleaddr,consoleaddrlen, consession);
		}
	} //////////////////////////////////////////////////////////////////////////////end ConPrint

////////////////////////////////////////////////////////////////////////////////////////////////////
// StubCallBack
////////////////////////////////////////////////////////////////////////////////////////////////////
int StubCallBack(connecttype *cparm)
{

        // save the function pointer to be used for logging
        ThreadLog("StubCallBack: New Connection.");

        return (0);
        } //////////////////////////////////////////////////////////////////////////end StubCallBack

////////////////////////////////////////////////////////////////////////////////////////////////////
// ConsoleListener
// This will become the main loop in the server
// We will always either be connected, or trying to connect
// When this procedure exits (this is the original thread)
// The main procedure will exit shortly there after.
////////////////////////////////////////////////////////////////////////////////////////////////////
void ConsoleListener()
{
int                     status;
char                    *mesg;
bool                    sockopen,done;
int                     count;                  // data length

	// init the socket
	consock->SetLogFunc(ThreadLog);
	consock->SetConnectFunc(StubCallBack);

//	if (!consock->Init(&m_attr))
	if (!consock->Init())
	{
		printf("Port Initialization failed. Probably port is in use. \n");
		return;
		}
	ThreadLog("ConsoleListener: After sock init\n");
	bzero(&consoleaddr, sizeof(consoleaddr));

	// fill out our address information
	consoleaddr.sin_family = AF_INET;			// host byte order
	consoleaddr.sin_port = htons(CONWORLDPORT);		// short, network byte order
	consoleaddr.sin_addr.s_addr = htonl(INADDR_ANY);	// automatically fill with my IP
	consock->Create(&consoleaddr,sizeof(consoleaddr));

	// create the socket, and bind the socket
	if (!consock->Bind())
	{
		printf("Port Initialization failed. Probably port is in use. \n");
		return;
		}

	// set up the listen
	status=consock->Listen();

	done=false;
	sockopen=false;

	while (!done)
	{
		consoleconnected=true;
		sockopen=true;
		while ((sockopen)&&(!done))
		{
			// protocol will return a buffer
			consoleaddrlen=sizeof(consoleaddrlen);
			count=consock->Recv(&mesg, 0, &consoleaddr, &consoleaddrlen, &consession);     
			if (count>0)
			if (ConServer(mesg,count)==true)	// did we get an exit command ?
			{
				done=true;
				ConPrint("ConsoleListener: Done=true.\n ");
				}
			} // end while (sockopen)
		consoleconnected=false;
	
		} // end while (!done)


	ThreadLog("ConsoleListener: Exiting\n");
	} ///////////////////////////////////////////////////////////////////////end ConsoleListener

////////////////////////////////////////////////////////////////////////////////////////////////////
// ParseCmdArgs
////////////////////////////////////////////////////////////////////////////////////////////////////
void ParseCmdArgs(int argc, char** argv)
{
int     i;

        // set defaults for parms
//	strncpy(serverhostname,"worldiii",80);
	loglevel=0;

	//getting parms
	for (i=0;i<argc;i++)
	{
		// --loglevel how much logging to do
		if ((strncmp(argv[i],"--loglevel",10))==0)
		{
			printf("DEBUGMODE ON\n");
			if (argc>=i+2) loglevel=atoi(argv[i+1]);
			i++;
			}
		// --h print command line options 
		if ((strncmp(argv[i],"-h",2))==0)
		{
			printf("usage: server [-h] | [--loglevel {0-9}]\n");
			exit (0);
			}
		} //end for loop

	} //////////////////////////////////////////////////////////////////////////end ParseCmdArgs

////////////////////////////////////////////////////////////////////////////////////////////////////
// Init
////////////////////////////////////////////////////////////////////////////////////////////////////
void Init(int argc, char** argv)
{
	// setup logging
	logcount=0;
	loglevel=0;
	ThreadLog("Init: Main Thread entry (Logs initialized).\n");
	ParseCmdArgs(argc,argv);

	// init the threading model
	pthread_attr_init(&attr);		// initialize attr with default attributes
//	pthread_mutexattr_init(&m_attr);
	pthread_attr_setscope(&attr, PTHREAD_SCOPE_SYSTEM);  // system-wide contention
//	pthread_mutex_init(&logmutex, NULL);


//	pthread_mutexattr_settype(&m_attr,PTHREAD_MUTEX_ERRORCHECK_NP);

//	pthread_mutex_init(&logmutex, &m_attr);
	
	InitMutex(&logmutex);	

	// display command line parms
	if (loglevel>0)
	{
	int i;

		for (i=0;i<argc;i++) printf("Argument %d, s='%s' \n",i,argv[i]);
		}

	listenstate=true;	
	consoleconnected=false;
	donelistening=false;
	initheclass.ClassStaticInit();       // yes it is an ugly hack

	ThreadLog("Init: Exited.\n");
	} //////////////////////////////////////////////////////////////////////////////////end Init

////////////////////////////////////////////////////////////////////////////////////////////////////
// DeInit
////////////////////////////////////////////////////////////////////////////////////////////////////
void DeInit()
{
	//
//	world_cache->FlushDB();
//	world_cache->CloseDB();

	} ////////////////////////////////////////////////////////////////////////////////end DeInit

////////////////////////////////////////////////////////////////////////////////////////////////////
// main
////////////////////////////////////////////////////////////////////////////////////////////////////
int main (int argc , char *argv[])
{
//struct sockaddr_in	servaddr;
void            	*param;

	Init(argc, argv);	
	ThreadLog("Main: After Init.\n");

	consock = new CUSock;
	servsock = new CUSock;

	// Start the dispatcher
	// pack any params for the dispatcher thread
	// tlist is a global thread tracking structure
	if (tlist.StartThread(Dispatcher,param)==ERR_CANTSTARTTHREAD)
	{
		ThreadLog("Main: Can't start dispatcher.\n");
		exit(1);
		}

	ConsoleListener();
	ConPrint("Main: After ConsoleListener exit.\n");

	// we are done listening, this allows us to close listensock
	donelistening=true;

	// Thread pool shutdown

	// no more messages
	consock->Close();

	DeInit();

	} //////////////////////////////////////////////////////////////////////////////////end main
