///////////////////////////////////////////////////////////////////////////////////////////////////
// Server
// cengine.cpp : This is the script engine. This is used by the server, 
// with one instance per CServer.
//
// by Eric J Krejci
// Copyright (C) 1999 Eric J Krejci
// Revised : 23 May 2007
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation version 2.
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
// Contact:
//              email           worldii@viagloria.com
//              web             www.viagloria.com
///////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////
// Includes
///////////////////////////////////////////////////////////////////////////////////////////////////
#include <stdio.h>
#include <errno.h>
#include <sys/timeb.h>
#include "cengine.hpp"
#include "worldiii.h"

///////////////////////////////////////////////////////////////////////////////////////////////////
// Defines
///////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////
// Externals
///////////////////////////////////////////////////////////////////////////////////////////////////
extern int loglevel;

////////////////////////////////////////////////////////////////////////////////////////////////////
// Constructor
////////////////////////////////////////////////////////////////////////////////////////////////////
CEngine::CEngine()
{
int	i;
	
	for (i=0;i<10;i++)
	{
		bools[i]=false;
		uints[i]=0;
		ints[i]=0;
		dbls[i]=0.0;
		}
	cy=false;
	eq=false;
	gt=false;
	lt=false;
	ss=false;
	pc=0;
	engerr=0;
	sp=MAXSCRIPT-1;		// set the starting val for stack pointer
	for (i=0;i<MAXSCRIPT;i++) scripts[i]=0;
	for (i=0;i<256;i++) OpCodes[i]=0;
	breakon=false;
	breakaddr=0;
	outputfile="script.bin";
	makelistfile=false;
	lstfile="script.lst";
	} ///////////////////////////////////////////////////////////////////////////end constructor

////////////////////////////////////////////////////////////////////////////////////////////////////
// Name: ~CEngine()
// Desc: Font class destructor
////////////////////////////////////////////////////////////////////////////////////////////////////
CEngine::~CEngine()
{
	} ////////////////////////////////////////////////////////////////////////////end destructor

////////////////////////////////////////////////////////////////////////////////////////////////////
// SetLogFunc
////////////////////////////////////////////////////////////////////////////////////////////////////
void CEngine::SetLogFunc(void (*pfunc)(char*))
{
	LogFunc=pfunc;
	} ////////////////////////////////////////////////////////////////////////////end SetLogFunc

////////////////////////////////////////////////////////////////////////////////////////////////////
// SetCStringLogFunc
////////////////////////////////////////////////////////////////////////////////////////////////////
void CEngine::SetCStringLogFunc(void (*pfunc)(CString*))
{
	CStringLogFunc=pfunc;
	} /////////////////////////////////////////////////////////////////////end SetCStringLogFunc

////////////////////////////////////////////////////////////////////////////////////////////////////
// ProtLog
////////////////////////////////////////////////////////////////////////////////////////////////////
void CEngine::ProtLog(CString *s)
{
	if (CStringLogFunc) (*CStringLogFunc)(s);
	} ///////////////////////////////////////////////////////////////////////////////end ProtLog

////////////////////////////////////////////////////////////////////////////////////////////////////
// ProtLog
////////////////////////////////////////////////////////////////////////////////////////////////////
void CEngine::ProtLog(char *s)
{
	if (LogFunc) (*LogFunc)(s);
	} ///////////////////////////////////////////////////////////////////////////////end ProtLog

////////////////////////////////////////////////////////////////////////////////////////////////////
// Init
// To add an instruction, add to following places
// - here in the Init()
// - add instruction procedure
// - add to Convert() procedure
// - add to AddToSymTab procedure
// - add to Check() procedure
// - add to cengine.hpp defines section
// - add to cengine.hpp private procedure section
////////////////////////////////////////////////////////////////////////////////////////////////////
bool CEngine::Init()
{
int	i;

	if (loglevel>=1) ProtLog("CEngine:: Init: Entered. \n ");
	OpCodes[CNT]=&CEngine::OpCode_cnt;	Mnem[CNT].symbol="cnt";		Mnem[CNT].parmcount=1;
	OpCodes[ADD]=&CEngine::OpCode_add;	Mnem[ADD].symbol="add";		Mnem[ADD].parmcount=6;
	OpCodes[SUB]=&CEngine::OpCode_sub;	Mnem[SUB].symbol="sub";		Mnem[SUB].parmcount=6;
	OpCodes[MUL]=&CEngine::OpCode_mul;	Mnem[MUL].symbol="mul";		Mnem[MUL].parmcount=6;
	OpCodes[DIVD]=&CEngine::OpCode_divd;	Mnem[DIVD].symbol="divd";	Mnem[DIVD].parmcount=6;
	OpCodes[DIVI]=&CEngine::OpCode_divi;	Mnem[DIVI].symbol="divi";	Mnem[DIVI].parmcount=6;
	OpCodes[MOD]=&CEngine::OpCode_mod;	Mnem[MOD].symbol="mod";		Mnem[MOD].parmcount=6;
	OpCodes[EXP]=&CEngine::OpCode_exp;	Mnem[EXP].symbol="exp";		Mnem[EXP].parmcount=6;
	OpCodes[NEG]=&CEngine::OpCode_neg;	Mnem[NEG].symbol="neg";		Mnem[NEG].parmcount=2;
	OpCodes[PUSH]=&CEngine::OpCode_push;	Mnem[PUSH].symbol="push";	Mnem[PUSH].parmcount=2;
	OpCodes[POP]=&CEngine::OpCode_pop;	Mnem[POP].symbol="pop";		Mnem[POP].parmcount=2;

	OpCodes[JMP]=&CEngine::OpCode_jmp;	Mnem[JMP].symbol="jmp";		Mnem[JMP].parmcount=1;
	OpCodes[JEQ]=&CEngine::OpCode_jeq;	Mnem[JEQ].symbol="jeq";		Mnem[JEQ].parmcount=1;
	OpCodes[JLT]=&CEngine::OpCode_jlt;	Mnem[JLT].symbol="jlt";		Mnem[JLT].parmcount=1;
	OpCodes[JGT]=&CEngine::OpCode_jgt;	Mnem[JGT].symbol="jgt";		Mnem[JGT].parmcount=1;
	OpCodes[JV]=&CEngine::OpCode_jv;	Mnem[JV].symbol="jv";		Mnem[JV].parmcount=3;
	OpCodes[JNZ]=&CEngine::OpCode_jnz;	Mnem[JNZ].symbol="jnz"; 	Mnem[JNZ].parmcount=1;

	OpCodes[CALL]=&CEngine::OpCode_call;	Mnem[CALL].symbol="call";	Mnem[CALL].parmcount=1;
	OpCodes[CEQ]=&CEngine::OpCode_ceq;	Mnem[CEQ].symbol="ceq";		Mnem[CEQ].parmcount=1;
	OpCodes[CLT]=&CEngine::OpCode_clt;	Mnem[CLT].symbol="clt";		Mnem[CLT].parmcount=1;
	OpCodes[CGT]=&CEngine::OpCode_cgt;	Mnem[CGT].symbol="cgt";		Mnem[CGT].parmcount=1;
	OpCodes[CV]=&CEngine::OpCode_cv;	Mnem[CV].symbol="cv";		Mnem[CV].parmcount=3;
	OpCodes[CNZ]=&CEngine::OpCode_cnz;	Mnem[CNZ].symbol="cnz"; 	Mnem[CNZ].parmcount=1;

	OpCodes[RET]=&CEngine::OpCode_ret;	Mnem[RET].symbol="ret";		Mnem[RET].parmcount=0;
	OpCodes[REQ]=&CEngine::OpCode_req;	Mnem[REQ].symbol="req";		Mnem[REQ].parmcount=0;
	OpCodes[RLT]=&CEngine::OpCode_rlt;	Mnem[RLT].symbol="rlt";		Mnem[RLT].parmcount=0;
	OpCodes[RGT]=&CEngine::OpCode_rgt;	Mnem[RGT].symbol="rgt";		Mnem[RGT].parmcount=0;
	OpCodes[RV]=&CEngine::OpCode_rv;	Mnem[RV].symbol="rv";		Mnem[RV].parmcount=2;
	OpCodes[RNZ]=&CEngine::OpCode_rnz;	Mnem[RNZ].symbol="rnz"; 	Mnem[RNZ].parmcount=0;
	
	OpCodes[CMP]=&CEngine::OpCode_cmp;	Mnem[CMP].symbol="cmp";		Mnem[CMP].parmcount=4;
	OpCodes[CPI]=&CEngine::OpCode_cpi;	Mnem[CPI].symbol="cpi";		Mnem[CPI].parmcount=4;
	OpCodes[LD]=&CEngine::OpCode_ld;	Mnem[LD].symbol="ld";		Mnem[LD].parmcount=4;
	OpCodes[LDR]=&CEngine::OpCode_ldr;	Mnem[LDR].symbol="ldr";		Mnem[LDR].parmcount=4;
	OpCodes[LDM]=&CEngine::OpCode_ldm;	Mnem[LDM].symbol="ldm";		Mnem[LDM].parmcount=4;
	OpCodes[LDI]=&CEngine::OpCode_ldi;	Mnem[LDI].symbol="ldi";		Mnem[LDI].parmcount=4;
	OpCodes[LDSP]=&CEngine::OpCode_ldsp;	Mnem[LDSP].symbol="ldsp";	Mnem[LDSP].parmcount=2;
	OpCodes[LDTI]=&CEngine::OpCode_ldti;	Mnem[LDTI].symbol="ldti";	Mnem[LDTI].parmcount=2;

	OpCodes[AND]=&CEngine::OpCode_and;	Mnem[AND].symbol="and";		Mnem[AND].parmcount=6;
	OpCodes[OR]=&CEngine::OpCode_or;	Mnem[OR].symbol="or";		Mnem[OR].parmcount=6;
	OpCodes[XOR]=&CEngine::OpCode_xor;	Mnem[OR].symbol="xor";		Mnem[OR].parmcount=6;
	OpCodes[NOT]=&CEngine::OpCode_not;	Mnem[NOT].symbol="not";		Mnem[NOT].parmcount=2;
	OpCodes[CNV]=&CEngine::OpCode_cnv;	Mnem[CNV].symbol="cnv";		Mnem[CNV].parmcount=4;
	OpCodes[INC]=&CEngine::OpCode_inc;	Mnem[INC].symbol="inc";		Mnem[INC].parmcount=2;
	OpCodes[DEC]=&CEngine::OpCode_dec;	Mnem[DEC].symbol="dec";		Mnem[DEC].parmcount=2;
	OpCodes[INCM]=&CEngine::OpCode_incm;	Mnem[INCM].symbol="incm";	Mnem[INCM].parmcount=2;
	OpCodes[DECM]=&CEngine::OpCode_decm;	Mnem[DECM].symbol="decm";	Mnem[DECM].parmcount=2;
	OpCodes[STM]=&CEngine::OpCode_stm;	Mnem[STM].symbol="stm";		Mnem[STM].parmcount=3;
	OpCodes[STR]=&CEngine::OpCode_str;	Mnem[STR].symbol="str";		Mnem[STR].parmcount=3;
	OpCodes[HLT]=0;				Mnem[HLT].symbol="hlt";		Mnem[HLT].parmcount=0;

	Direct[0].symbol=EQU;	Direct[0].parmcount=1;
	Direct[1].symbol=ORG;	Direct[1].parmcount=1;
	Direct[2].symbol=DEFB;	Direct[2].parmcount=1;
	Direct[3].symbol=DEFS;	Direct[3].parmcount=1;
	Direct[4].symbol=DEFD;	Direct[4].parmcount=1;


	for (i=0; i<MAXEXTOPCODE;i++)
	{
	int	j;

		ExtOpCodes[i].ptr=0;
		ExtOpCodes[i].bin=0;
		ExtOpCodes[i].iin=0;
		ExtOpCodes[i].uin=0;
		ExtOpCodes[i].din=0;
		ExtOpCodes[i].sin=0;
		ExtOpCodes[i].bout=0;
		ExtOpCodes[i].iout=0;
		ExtOpCodes[i].uout=0;
		ExtOpCodes[i].dout=0;
		ExtOpCodes[i].sout=0;
		for (j=0;j<10;j++)
		{
			ExtOpCodes[i].p.b[j]=false;
			ExtOpCodes[i].p.i[j]=0;
			ExtOpCodes[i].p.u[j]=0;
			ExtOpCodes[i].p.d[j]=0.0;
			ExtOpCodes[i].p.extra=0;
			// CStrings initialize with empty strings
			}
		}
	if (loglevel>=1) ProtLog("CEngine:: Init: Exited. \n ");
        return( true );
	} //////////////////////////////////////////////////////////////////////////////////end Init

////////////////////////////////////////////////////////////////////////////////////////////////////
// GetValue
////////////////////////////////////////////////////////////////////////////////////////////////////
unsigned long int CEngine::GetValue(unsigned long int ind)
{
	return (scripts[ind]);
	} //////////////////////////////////////////////////////////////////////////////end GetValue

////////////////////////////////////////////////////////////////////////////////////////////////////
// GetBools
////////////////////////////////////////////////////////////////////////////////////////////////////
void CEngine::GetBools(bool b[])
{
int	i;

	for (i=0;i<10;i++) b[i]=bools[i];
	} //////////////////////////////////////////////////////////////////////////////end GetBools

////////////////////////////////////////////////////////////////////////////////////////////////////
// GetUInts
////////////////////////////////////////////////////////////////////////////////////////////////////
void CEngine::GetUInts(unsigned long int is[])
{
int	i;

	for (i=0;i<10;i++) is[i]=uints[i];
	} //////////////////////////////////////////////////////////////////////////////end GetUInts

////////////////////////////////////////////////////////////////////////////////////////////////////
// GetInts
////////////////////////////////////////////////////////////////////////////////////////////////////
void CEngine::GetInts(long int is[])
{
int	i;

	for (i=0;i<10;i++) is[i]=ints[i];
	} ///////////////////////////////////////////////////////////////////////////////end GetInts

////////////////////////////////////////////////////////////////////////////////////////////////////
// GetDbls
////////////////////////////////////////////////////////////////////////////////////////////////////
void CEngine::GetDbls(double ds[])
{
int	i;

	for (i=0;i<10;i++) ds[i]=dbls[i];
	} ///////////////////////////////////////////////////////////////////////////////end GetDbls

////////////////////////////////////////////////////////////////////////////////////////////////////
// GetStrings
////////////////////////////////////////////////////////////////////////////////////////////////////
void CEngine::GetStrings(CString ss[])
{
int	i;

	for (i=0;i<10;i++) ss[i]=strs[i];
	} ////////////////////////////////////////////////////////////////////////////end GetStrings

////////////////////////////////////////////////////////////////////////////////////////////////////
// GetState
////////////////////////////////////////////////////////////////////////////////////////////////////
void CEngine::GetState(unsigned long int *p, unsigned long int *s, bool *e, bool *g, bool *l, bool *c)
{
	*c=cy;
	*p=pc;
	*s=sp;
	*e=eq;
	*g=gt;
	*l=lt;
	} //////////////////////////////////////////////////////////////////////////////end GetState

////////////////////////////////////////////////////////////////////////////////////////////////////
// SetPC
////////////////////////////////////////////////////////////////////////////////////////////////////
void CEngine::SetPC(unsigned long int ind)
{
	if (ind>=MAXSCRIPT) pc=MAXSCRIPT-1;
	else pc=ind;
	} /////////////////////////////////////////////////////////////////////////////////end SetPC

////////////////////////////////////////////////////////////////////////////////////////////////////
// SetValue
////////////////////////////////////////////////////////////////////////////////////////////////////
void CEngine::SetValue(unsigned long int ind,unsigned long int val)
{
	scripts[ind]=val;
	} //////////////////////////////////////////////////////////////////////////////end SetValue

////////////////////////////////////////////////////////////////////////////////////////////////////
// Reset
////////////////////////////////////////////////////////////////////////////////////////////////////
void CEngine::Reset()
{
int	i;
	
	for (i=0;i<10;i++)
	{
		bools[i]=false;
		uints[i]=0;
		ints[i]=0;
		dbls[i]=0.0;
		strs[i].Reset();
		}
	eq=false;
	gt=false;
	lt=false;
	ss=false;
	cy=false;
	pc=0;
	sp=MAXSCRIPT-1;
	
	} /////////////////////////////////////////////////////////////////////////////////end Reset

////////////////////////////////////////////////////////////////////////////////////////////////////
// SingleStep
////////////////////////////////////////////////////////////////////////////////////////////////////
int CEngine::SingleStep()
{
	if (loglevel>=1) ProtLog("CEngine:: SingleStep: Entered. \n ");
	ss=true;
	return (Run());
	} ////////////////////////////////////////////////////////////////////////////end SingleStep

////////////////////////////////////////////////////////////////////////////////////////////////////
// Run
// The key thing here is that we are using an array of function pointers, that is indexed by the 
// opcodes. Thus calling will be very fast.
////////////////////////////////////////////////////////////////////////////////////////////////////
int CEngine::Run()
{
char	buf[80];
	
	if (loglevel>=1) ProtLog("CEngine:: Run: Entered.\n");
	
	// set error flag to no error
	engerr=CE_ERR_NOERR;

	// start executing script
	while ((scripts[pc]!=HLT)&&(engerr==CE_ERR_NOERR))
	{
		if (loglevel>=3) ProtLog("CEngine:: Run: Entered run while.\n");
		// rudimentary breakpoints
		if (breakon) if (pc==breakaddr) return(CE_ERR_BRK);
		if (breakon) if (pc==breakaddr) return(CE_ERR_BRK);
		
		snprintf(buf,80,"CEngine:: Run: pc=%u, scripts[pc]=%u \n",pc,scripts[pc]);
		if (loglevel>=3) ProtLog(buf);

		// Execute next inst, but check for pc wrap first
		// This could put us into an endless loop, but the whole semantics of the cpu style
		// engine require this for consistency's sake.
		if (pc>=MAXSCRIPT) pc=0;

		// using two arrays of function pointers here
		if (scripts[pc]<=255)	// these are the internal engine defined operands
		{
			if (loglevel>=2) ProtLog("CEngine:: Run: Internal function.\n");

			// Execute Normal instruction
			if (OpCodes[scripts[pc]]) (this->*OpCodes[scripts[pc]])();
			else 
			{
				if (loglevel>=2) ProtLog("CEngine:: Run: OpCode Function not defined.\n");
				return (CE_ERR_OPPRC);
				}
			if (ss==true)
			{
				ss=false;
				return(CE_ERR_NOERR);
				}
			}
		else			// these are any enternally defined operands
		{
			if (loglevel>=2) ProtLog("CEngine:: Run: External function.\n");

			// Execute externally defined opecode
			if (ExtOpCodes[scripts[pc]-1000].ptr) 
			{
				CallExt();
				}
			else 
			{
				if (loglevel>=2) ProtLog("CEngine:: Run: Return Failed.\n");
				return (CE_ERR_OPPRC);
				}
			if (ss==true)
			{
				ss=false;
				return(CE_ERR_NOERR);
				}
			}
		}
	
	if (loglevel>=1) ProtLog("CEngine:: Run: Exiting.\n");
	return (engerr);
	} //////////////////////////////////////////////////////////////////////////////end SetValue

////////////////////////////////////////////////////////////////////////////////////////////////////
// GetParms
// THIS IS OVERLOADED
// This version gets a single parms
// Note side effect pc is updated!!!
////////////////////////////////////////////////////////////////////////////////////////////////////
void CEngine::GetParms(char *vt1, int  *idx1)
{
	// get the parms or the v1,v2,result case
	////// get vartype1
	pc++;
	*vt1=(char)scripts[pc];

	// get the idx to test, and keep it in range
	pc++;
	*idx1=(int)(scripts[pc]);
	if (*idx1>=10) *idx1=9;
	if (*idx1<0) *idx1=0;
	
	} //////////////////////////////////////////////////////////////////////////////end GetParms

////////////////////////////////////////////////////////////////////////////////////////////////////
// GetParms
// THIS IS OVERLOADED
// This version gets a single parms
// Note side effect pc is updated!!!
////////////////////////////////////////////////////////////////////////////////////////////////////
void CEngine::GetParms(unsigned long int *addr, char  *vt1, int  *idx1)
{
	// get the addr, and keep it in range
	pc++;
	*addr=scripts[pc];
	if (*addr>=MAXSCRIPT) *addr=MAXSCRIPT-1;
	
	////// get vartype1
	pc++;
	*vt1=(char)scripts[pc];

	// get the idx to test, and keep it in range
	pc++;
	*idx1=(int)(scripts[pc]);
	if (*idx1>=10) *idx1=9;
	if (*idx1<0) *idx1=0;
	
	} //////////////////////////////////////////////////////////////////////////////end GetParms

////////////////////////////////////////////////////////////////////////////////////////////////////
// GetParms
// THIS IS OVERLOADED
// This version gets a single parms
// Note side effect pc is updated!!!
////////////////////////////////////////////////////////////////////////////////////////////////////
void CEngine::GetParms(char *vt1, int  *idx1, int  *idx2)
{
	// get the parms or the v1,v2,result case
	////// get vartype1
	pc++;
	*vt1=(char)scripts[pc];

	// get the idx to test, and keep it in range
	pc++;
	*idx1=(int)(scripts[pc]);
	if (*idx1>=10) *idx1=9;
	if (*idx1<0) *idx1=0;
	
	// get the idx to test, and keep it in range
	pc++;
	*idx2=(int)(scripts[pc]);
	if (*idx2>=10) *idx2=9;
	if (*idx2<0) *idx2=0;
	
	} //////////////////////////////////////////////////////////////////////////////end GetParms

////////////////////////////////////////////////////////////////////////////////////////////////////
// GetParms
// THIS IS OVERLOADED
// This version gets a single parms
// Note side effect pc is updated!!!
////////////////////////////////////////////////////////////////////////////////////////////////////
void CEngine::GetParms(char *vt1, int  *idx1, int  *idx2, int *idx3)
{
	// get the parms or the v1,v2,result case
	////// get vartype1
	pc++;
	*vt1=(char)scripts[pc];

	// get the idx to test, and keep it in range
	pc++;
	*idx1=(int)(scripts[pc]);
	if (*idx1>=10) *idx1=9;
	if (*idx1<0) *idx1=0;
	
	// get the idx to test, and keep it in range
	pc++;
	*idx2=(int)(scripts[pc]);
	if (*idx2>=10) *idx2=9;
	if (*idx2<0) *idx2=0;
	
	// get the idx to test, and keep it in range
	pc++;
	*idx3=(int)(scripts[pc]);
	if (*idx3>=10) *idx3=9;
	if (*idx3<0) *idx3=0;
	
	} //////////////////////////////////////////////////////////////////////////////end GetParms

////////////////////////////////////////////////////////////////////////////////////////////////////
// GetParms
// THIS IS OVERLOADED
// This version gets a single parms
// Note side effect pc is updated!!!
////////////////////////////////////////////////////////////////////////////////////////////////////
void CEngine::GetParms(char *vt1, int  *idx1, unsigned long  *idx2, unsigned long *idx3)
{
	// get the parms or the v1,v2,result case
	////// get vartype1
	pc++;
	*vt1=(char)scripts[pc];

	// get the idx to test, and keep it in range
	pc++;
	*idx1=(int)(scripts[pc]);
	if (*idx1>=10) *idx1=9;
	if (*idx1<0) *idx1=0;
	
	// get the data 
	pc++;
	*idx2=(unsigned long)(scripts[pc]);
	
	// get the data
	pc++;
	*idx3=(unsigned long)(scripts[pc]);
	
	} //////////////////////////////////////////////////////////////////////////////end GetParms

////////////////////////////////////////////////////////////////////////////////////////////////////
// GetParms
// THIS IS OVERLOADED
// This version gets a single parms
// Note side effect pc is updated!!!
////////////////////////////////////////////////////////////////////////////////////////////////////
void CEngine::GetParms(char *vt1, int  *idx1, char *vt2, int  *idx2)
{
	// get the parms or the v1,v2,result case
	////// get vartype1
	pc++;
	*vt1=(char)scripts[pc];

	// get the idx to test, and keep it in range
	pc++;
	*idx1=(int)(scripts[pc]);
	if (*idx1>=10) *idx1=9;
	if (*idx1<0) *idx1=0;
	
	////// get vartype2
	pc++;
	*vt2=(char)scripts[pc];

	// get the idx to test, and keep it in range
	pc++;
	*idx2=(int)(scripts[pc]);
	if (*idx2>=10) *idx2=9;
	if (*idx2<0) *idx2=0;
	
	} //////////////////////////////////////////////////////////////////////////////end GetParms

////////////////////////////////////////////////////////////////////////////////////////////////////
// GetParms
// THIS IS OVERLOADED
// This version gets a two parms and results 
// Note side effect pc is updated!!!
////////////////////////////////////////////////////////////////////////////////////////////////////
void CEngine::GetParms(char *vt1, int  *idx1, char *vt2, int  *idx2, char *rt, int  *ri)
{
	// get the parms or the v1,v2,result case
	////// get vartype1
	pc++;
	*vt1=(char)scripts[pc];

	// get the idx to test, and keep it in range
	pc++;
	*idx1=(int)(scripts[pc]);
	if (*idx1>=10) *idx1=9;
	if (*idx1<0) *idx1=0;
	
	////// get vartype2
	pc++;
	*vt2=(char)scripts[pc];

	// get the idx to test, and keep it in range
	pc++;
	*idx2=(int)(scripts[pc]);
	if (*idx2>=10) *idx2=9;
	if (*idx2<0) *idx2=0;
	
	////// get result type
	pc++;
	*rt=(char)scripts[pc];

	// get the idx for result, and keep it in range
	pc++;
	*ri=(int)(scripts[pc]);
	if (*ri>=10) *ri=9;
	if (*ri<0) *ri=0;
	} //////////////////////////////////////////////////////////////////////////////end GetParms

////////////////////////////////////////////////////////////////////////////////////////////////////
// Register
// This allows external code to create and define an opcode 
// The registering conventions are when registering an external function need to pass
// - 5 parms for parm counts incoming
// - 5 parms for parm counts for return values
// - the actual function pointer
// - the opcode to assign
// Limitations
// - parms must come from registers
// - registers are used in order starting at 0 and couting up. EX if there are 4 integer registers 
//   to pass as parms then registers i[0] i[1] i[2] i[3] will be used in contiguous order.
////////////////////////////////////////////////////////////////////////////////////////////////////
int CEngine::Register(ptrtoextfunc ptr, int bin,  int iin,  int uin, int din,  int sin, 
					int bout, int iout, int uout, int dout, int sout,	
					unsigned long opc,
					void * p)
{
	if (opc<1000) return (CE_ERR_OPCINVAL);
	ExtOpCodes[opc-1000].ptr=ptr;	// how many bool regs does the external functions need ?
	ExtOpCodes[opc-1000].bin=bin;
	ExtOpCodes[opc-1000].iin=iin;
	ExtOpCodes[opc-1000].uin=uin;
	ExtOpCodes[opc-1000].din=din;
	ExtOpCodes[opc-1000].sin=sin;

	ExtOpCodes[opc-1000].bout=bout;	// how many bools does it return ?
	ExtOpCodes[opc-1000].iout=iout;
	ExtOpCodes[opc-1000].uout=uout;
	ExtOpCodes[opc-1000].dout=dout;
	ExtOpCodes[opc-1000].sout=sout;
	ExtOpCodes[opc-1000].extra=p;	// a place to store some value the external function may need
	return (CE_ERR_NOERR);
	} //////////////////////////////////////////////////////////////////////////////end Register

////////////////////////////////////////////////////////////////////////////////////////////////////
// CallExt
////////////////////////////////////////////////////////////////////////////////////////////////////
int CEngine::CallExt()
{
parmstruct	*p;
int		i;
char		buf[BUFFSIZE];

	if (loglevel>=1) ProtLog("CEngine:: CallExt: Entered.\n");
	
	// get address of passing struct
	p=&(ExtOpCodes[scripts[pc]-1000].p);

	// here we need to package up the call to a previously registered external function 
	for (i=0;i<ExtOpCodes[scripts[pc]-1000].bin;i++) p->b[i]=bools[i]; 
	for (i=0;i<ExtOpCodes[scripts[pc]-1000].iin;i++) p->i[i]=ints[i]; 
	for (i=0;i<ExtOpCodes[scripts[pc]-1000].uin;i++) p->u[i]=uints[i]; 
	for (i=0;i<ExtOpCodes[scripts[pc]-1000].din;i++) p->d[i]=dbls[i]; 
	for (i=0;i<ExtOpCodes[scripts[pc]-1000].sin;i++) p->s[i]=strs[i]; 
	p->extra=ExtOpCodes[scripts[pc]-1000].extra;

	// call the external opcoded function
	(*(ExtOpCodes[scripts[pc]-1000].ptr))(p);

	// and then unpackage the return values
	for (i=0;i<ExtOpCodes[scripts[pc]-1000].bout;i++) bools[i]=p->b[i]; 
	for (i=0;i<ExtOpCodes[scripts[pc]-1000].iout;i++) ints[i]=p->i[i]; 
	for (i=0;i<ExtOpCodes[scripts[pc]-1000].uout;i++) uints[i]=p->u[i]; 
	for (i=0;i<ExtOpCodes[scripts[pc]-1000].dout;i++) dbls[i]=p->d[i]; 
	for (i=0;i<ExtOpCodes[scripts[pc]-1000].sout;i++) strs[i]=p->s[i]; 
	
	// point to next instruction
	pc++;
	if (loglevel>=1) ProtLog("CEngine:: CallExt: Exited.\n");
	} ///////////////////////////////////////////////////////////////////////////////end CallExt

////////////////////////////////////////////////////////////////////////////////////////////////////
// SetInputFile
////////////////////////////////////////////////////////////////////////////////////////////////////
void CEngine::SetInputFile(CString *filename)
{
	inputfile=*filename;
	} //////////////////////////////////////////////////////////////////////////end SetInputFile

////////////////////////////////////////////////////////////////////////////////////////////////////
// SetOutputFile
////////////////////////////////////////////////////////////////////////////////////////////////////
void CEngine::SetOutputFile(CString *filename)
{
	outputfile=*filename;
	} /////////////////////////////////////////////////////////////////////////end SetOutputFile

////////////////////////////////////////////////////////////////////////////////////////////////////
// IsExtOpCode
////////////////////////////////////////////////////////////////////////////////////////////////////
bool CEngine::IsExtOpCode(CString *tok)
{
int	i;
char	buf[BUFFSIZE];

//	snprintf(buf,BUFFSIZE,"CEngine:: tok='%s'",tok->GetStringPtr());
//	ProtLog(buf);
	tok->TokenInit(" ");
	if (!(tok->GetInt(&i))) return false;
//	snprintf(buf,BUFFSIZE,"CEngine:: IsOpCode: i=%d",i);
//	ProtLog(buf);
	if ((i>=1000)&&(i<1000+MAXEXTOPCODE)) return true;
	return false;
	} ///////////////////////////////////////////////////////////////////////////end IsExtOpCode

////////////////////////////////////////////////////////////////////////////////////////////////////
// IsOpcode
// returns 
// - bool for found
// - int for the opcode it is
////////////////////////////////////////////////////////////////////////////////////////////////////
bool CEngine::IsOpCode(CString *tok, int *code)
{
int	i;

	*code=MAXOPCODE;
	for (i=0; i<MAXOPCODE; i++) 
	{ 
		if (*tok==Mnem[i].symbol)
		{
			*code=i;
			return true; 
			}
		}
	return false;
	} //////////////////////////////////////////////////////////////////////////////end IsOpcode

////////////////////////////////////////////////////////////////////////////////////////////////////
// IsLabel
////////////////////////////////////////////////////////////////////////////////////////////////////
bool CEngine::IsLabel(CString *tok)
{
	if (((tok->GetChar(0) >= 'A') && (tok->GetChar(0) <='Z')) 
	|| ((tok->GetChar(0) >= 'a') && (tok->GetChar(0) <= 'z')))
	{
		if (tok->GetChar(tok->GetLength()-1)==':')
		{
			return true;
			}
		}
	return false;
	} ///////////////////////////////////////////////////////////////////////////////end IsLabel

////////////////////////////////////////////////////////////////////////////////////////////////////
// IsDir
////////////////////////////////////////////////////////////////////////////////////////////////////
bool CEngine::IsDir(CString *tok)
{
int	i;
char	buf[BUFFSIZE];

	for (i=0; i<MAXDIRECTIVE; i++) 
	{ 
		if (*tok==Direct[i].symbol) return true; 
		}

	return false;
	} /////////////////////////////////////////////////////////////////////////////////end IsDir

////////////////////////////////////////////////////////////////////////////////////////////////////
// GetParmCountDir
////////////////////////////////////////////////////////////////////////////////////////////////////
int CEngine::GetParmCountDir(CString *tok)
{
int	i;
//char	buf[BUFFSIZE];

	for (i=0; i<MAXDIRECTIVE; i++) 
	{ 
		if (*tok==Direct[i].symbol) 
		{
			return Direct[i].parmcount;
			} 
		}

	return -1;
	} ///////////////////////////////////////////////////////////////////////end GetParmCountDir

////////////////////////////////////////////////////////////////////////////////////////////////////
// GetParmCountOpCode
////////////////////////////////////////////////////////////////////////////////////////////////////
int CEngine::GetParmCountOpCode(CString *tok)
{
int	i;
//char	buf[BUFFSIZE];

	for (i=0; i<MAXOPCODE; i++) 
	{ 
		if (*tok==Mnem[i].symbol) 
		{
			return Mnem[i].parmcount;
			} 
		}

	return -1;
	} ////////////////////////////////////////////////////////////////////end GetParmCountOpCode

////////////////////////////////////////////////////////////////////////////////////////////////////
// InsertBool
// We'll insert 1 for true 0 for false
// for test purposes, 0 (zero) is always false, anything other than zero is true
// for bool 0 = false , 1 = true
// for int  0 = false , anything else is true
// for double 0.00000000 = false anything else is true
// for string "f" or "0" (zero) is false , anything else is true
////////////////////////////////////////////////////////////////////////////////////////////////////
bool CEngine::InsertBool(CString *s, FILE * lstf)
{
int	l,i;

	s->TokenInit(" ");
	l=s->GetLength();
	if ((*s=="0") || (*s=="f"))
	{
		scripts[stmtcntr]=0;	// zero or false	
		if (makelistfile) { fprintf(lstf,"                %09lu     %s     %09lu \n",stmtcntr,s->GetStringPtr(),scripts[stmtcntr]); }
		stmtcntr++;
		return true;
		}
	else 
	{
		scripts[stmtcntr]=1;	// one or true	
		if (makelistfile) { fprintf(lstf,"                %09lu     %s     %09lu \n",stmtcntr,s->GetStringPtr(),scripts[stmtcntr]); }
		stmtcntr++;
		return true;
		}
	return false;
	} ////////////////////////////////////////////////////////////////////////////end InsertBool

////////////////////////////////////////////////////////////////////////////////////////////////////
// InsertString
////////////////////////////////////////////////////////////////////////////////////////////////////
bool CEngine::InsertString(CString *s,FILE * lstf)
{
int	l,i;

	s->TokenInit(" ");
	l=s->GetLength();
	for (i=0;i<l;i++)
	{
		scripts[stmtcntr]=(unsigned long)(s->GetChar(i));
		if (makelistfile) 
		{ 
			if (s->GetChar(i)=='\n')
			fprintf(lstf,"                %09lu     %s    %09lu \n",stmtcntr,"\\n",scripts[stmtcntr]); 
			else fprintf(lstf,"                %09lu     %c     %09lu \n",stmtcntr,s->GetChar(i),scripts[stmtcntr]); 
			}
		stmtcntr++;
		}
	return true;
	} //////////////////////////////////////////////////////////////////////////end InsertString

////////////////////////////////////////////////////////////////////////////////////////////////////
// InsertInt
////////////////////////////////////////////////////////////////////////////////////////////////////
bool CEngine::InsertInt(CString *s,FILE* lstf)
{
	s->TokenInit(" ");
	if (s->GetULongInt(&(scripts[stmtcntr])))
	{
		if (makelistfile) { fprintf(lstf,"                %09lu     %s     %09lu \n",stmtcntr,s->GetStringPtr(),scripts[stmtcntr]); }
		stmtcntr++;
		ProtLog("CEngine:: InsertInt: OK.\n"); 
		return true;
		}
	else 
	{
		ProtLog("CEngine:: InsertInt: FAIL.\n"); 
		return false;
		}
	} /////////////////////////////////////////////////////////////////////////////end InsertInt

////////////////////////////////////////////////////////////////////////////////////////////////////
// InsertAddr
// Given the address parm insert it. Perform lookup if necessary.
////////////////////////////////////////////////////////////////////////////////////////////////////
bool CEngine::InsertAddr(CString *s, FILE *lstf)
{
unsigned long addr;

	s->TokenInit(" ");
	if (s->GetULongInt(&addr)) 
	{
		InsertInt(s,lstf);
		return true;
		}
	else
	{
		// we gotta look up the labels value
		if (Lookup(s,&addr))
		{
			// insert value
			scripts[stmtcntr]=addr;
			if (makelistfile) { fprintf(lstf,"                %09lu     %s     %09lu \n",stmtcntr,s->GetStringPtr(),scripts[stmtcntr]); }
			stmtcntr++;
			return true;
			}
		else 
		{
			return false;
			}
		}
	
	} ////////////////////////////////////////////////////////////////////////////end InsertAddr

////////////////////////////////////////////////////////////////////////////////////////////////////
// InsertType
////////////////////////////////////////////////////////////////////////////////////////////////////
bool CEngine::InsertType(CString *s, FILE *lstf)
{
	scripts[stmtcntr]=(unsigned long)(s->GetChar(0));
	if (makelistfile) { fprintf(lstf,"                %09lu     %s     %06d \n",stmtcntr,s->GetStringPtr(),scripts[stmtcntr]); }
	stmtcntr++;
	return true;
	} /////////////////////////////////////////////////////////////////////////////end InserType

////////////////////////////////////////////////////////////////////////////////////////////////////
// InsertOpCode
////////////////////////////////////////////////////////////////////////////////////////////////////
bool CEngine::InsertOpCode(CString *l, CString *c, int code, FILE *lstf)
{
char	buf[BUFFSIZE];

	snprintf(buf,BUFFSIZE,"CEngine:: InsertOpCode: stmtcntr=%lu code=%d\n",stmtcntr,code);
	ProtLog(buf); 
	scripts[stmtcntr]=code;
	if (makelistfile)
	{
		if (l)  fprintf(lstf,"%s   %09lu   %s     %06d \n",l->GetStringPtr(),stmtcntr,c->GetStringPtr(),code); 
		else   fprintf(lstf,"                %09lu   %s     %06d \n",stmtcntr,c->GetStringPtr(),code); 
		}

	stmtcntr++;
	return true;
	} //////////////////////////////////////////////////////////////////////////end InsertOpCode

////////////////////////////////////////////////////////////////////////////////////////////////////
// Convert
// returns 0 = OK
// returns not zero = error
// This converts the token, and then for opcodes actually inserts the opcode into the binary
////////////////////////////////////////////////////////////////////////////////////////////////////
int CEngine::Convert(CString *l, CString *c, 
			CString *p1, CString *p2, CString *p3, 
			CString *p4, CString *p5, CString *p6,
			int code, FILE *lstf)
{
int	i,op;
char	buf[BUFFSIZE];
CString Log;

	ProtLog("CEngine:: Convert: Entered.\n"); 
	// convert the ORG assembler directive
	if (*c==ORG)
	{
		p1->TokenInit(" ");
		p1->GetULongInt(&stmtcntr);
		snprintf(buf,BUFFSIZE,"CEngine:: Convert: Setting org symbol '%s' with value '%lu'.\n",l->GetStringPtr(),stmtcntr);
		ProtLog(buf); 
		return CE_ERR_NOERR;
		}

	// convert the EQU assembler directive
	if (*c==EQU)
	{
		p1->TokenInit(" ");
		p1->GetULongInt(&stmtcntr);
		snprintf(buf,BUFFSIZE,"CEngine:: Convert: Setting an equ symbol '%s' with value '%lu'.\n",l->GetStringPtr(),stmtcntr);
		ProtLog(buf); 
		return CE_ERR_NOERR;
		}

	// convert define string "ds"
	if (*c==DEFS)
	{
		// insert the string
		InsertString(p1,lstf);
		Log="CEngine:: Convert: Inserted '"+*p1+"'\n";
		Log.Log(); 
		return CE_ERR_NOERR;
		}

	// convert external operands
	c->TokenInit(" ");
	if (c->GetInt(&op))
	{
		ProtLog("CEngine:: Convert: extop\n"); 
		if ((op>=1000)&&(op<=1199))
		{
			InsertInt(c,lstf);
			snprintf(buf,BUFFSIZE,"CEngine:: Convert: AFTER EXT stmtcntr %d \n",stmtcntr);
			ProtLog(buf); 
			return CE_ERR_NOERR;
			}
		return CE_ERR_OPCINVAL;
		}

	// convert normal opcodes
	switch (code)
	{
	case CNT:
		break;

	case SUB:
	case ADD:
	case AND:
		ProtLog("CEngine:: Convert: ADD/SUB/AND\n"); 
		if (!InsertOpCode(l,c,code,lstf)) {return CE_ERR_CONV;}
		if (!InsertType(p1,lstf)) 	 {return CE_ERR_CONV;}
		if (!InsertInt(p2,lstf))	 {return CE_ERR_CONV;}			// in this case an index
		if (!InsertType(p3,lstf))	 {return CE_ERR_CONV;}			// in this case a count
		if (!InsertInt(p4,lstf))	 {return CE_ERR_CONV;}			// in this case an index
		if (!InsertType(p5,lstf))	 {return CE_ERR_CONV;}			// in this case a count
		if (!InsertInt(p6,lstf))	 {return CE_ERR_CONV;}			// in this case an index
		Log="CEngine:: Convert: Inserted '"+*p1
			+"' '"+*p2
			+"' '"+*p3
			+"' '"+*p4
			+"' '"+*p5
			+"' '"+*p6
			+"'\n"; Log.Log(); 
		return CE_ERR_NOERR;
		break;

	case CMP:
		ProtLog("CEngine:: Convert: CMP\n"); 
		if (!InsertOpCode(l,c,code,lstf)) {return CE_ERR_CONV;}
		if (!InsertType(p1,lstf)) 	 {return CE_ERR_CONV;}
		if (!InsertInt(p2,lstf))	 {return CE_ERR_CONV;}			// in this case an index
		if (!InsertType(p3,lstf))	 {return CE_ERR_CONV;}			// in this case a count
		if (!InsertInt(p4,lstf))	 {return CE_ERR_CONV;}			// in this case an index
		Log="CEngine:: Convert: Inserted '"+*p1
			+"' '"+*p2
			+"' '"+*p3
			+"' '"+*p4
			+"'\n"; Log.Log(); 
		return CE_ERR_NOERR;
		break;

	case MUL:
	case DIVD:
	case DIVI:
	case MOD:
	case EXP:
	case OR:
	case XOR:
		break;

	case STR:
		ProtLog("CEngine:: Convert: STR\n"); 
		if (!InsertOpCode(l,c,code,lstf)) {return CE_ERR_CONV;}
		if (!InsertType(p1,lstf)) 	 {return CE_ERR_CONV;}
		if (!InsertInt(p2,lstf))	 {return CE_ERR_CONV;}			// in this case an index
		if (!InsertInt(p3,lstf))	 {return CE_ERR_CONV;}			// in this case an addr
		snprintf(buf,BUFFSIZE,"CEngine:: Convert: AFTER STR stmtcntr %d \n",stmtcntr);
		ProtLog(buf); 
		return CE_ERR_NOERR;
		break;

	case LDR:
		ProtLog("CEngine:: Convert: LDR\n"); 
		if (!InsertOpCode(l,c,code,lstf)) {return CE_ERR_CONV;}
		if (!InsertType(p1,lstf)) 	 {return CE_ERR_CONV;}
		if (!InsertInt(p2,lstf))	 {return CE_ERR_CONV;}			// in this case an index
		if (!InsertInt(p3,lstf))	 {return CE_ERR_CONV;}			// in this case an addr
		if (!InsertInt(p4,lstf))	 {return CE_ERR_CONV;}			// in this case a len
		snprintf(buf,BUFFSIZE,"CEngine:: Convert: AFTER LDR stmtcntr %d \n",stmtcntr);
		ProtLog(buf); 
		return CE_ERR_NOERR;
		break;

	case LDM:
		ProtLog("CEngine:: Convert: LDM\n"); 
		if (!InsertOpCode(l,c,code,lstf)) {return CE_ERR_CONV;}
		if (!InsertType(p1,lstf)) 	  {return CE_ERR_CONV;}
		if (!InsertInt(p2,lstf))	  {return CE_ERR_CONV;}			// in this case an index
		if (!InsertInt(p3,lstf))	  {return CE_ERR_CONV;}			// in this case a count
		if (!InsertAddr(p4,lstf))	  {return CE_ERR_CONV;}			// in this case a count
		snprintf(buf,BUFFSIZE,"CEngine:: Convert: AFTER LDM stmtcntr %d \n",stmtcntr);
		ProtLog(buf); 
		return CE_ERR_NOERR;
		break;

	case LDI:
		ProtLog("CEngine:: Convert: LDI\n"); 
		if (!InsertOpCode(l,c,code,lstf)) {return CE_ERR_CONV;}
		if (!InsertType(p1,lstf)) 	 {return CE_ERR_CONV;}
		if (!InsertInt(p2,lstf))	 {return CE_ERR_CONV;}			// in this case an index
		if (!InsertInt(p3,lstf))	 {return CE_ERR_CONV;}			// in this case a count
		if      (*p1=="s") InsertString(p4,lstf);	
//		else if (*p1=="d") InsertDouble(p4);
		else if (*p1=="i") InsertInt(p4,lstf);
		else if (*p1=="u") InsertInt(p4,lstf);
		else if (*p1=="b") InsertBool(p4,lstf);
		else return CE_ERR_INVTYPE;
		snprintf(buf,BUFFSIZE,"CEngine:: Convert: AFTER LDI stmtcntr %d \n",stmtcntr);
		ProtLog(buf); 
		return CE_ERR_NOERR;
		break;

	case STM:

		Log="CEngine:: Convert: STM\n"; Log.Log(); 
		if (!InsertOpCode(l,c,code,lstf))	{return CE_ERR_CONV;}
		if (!InsertType(p1,lstf))		{return CE_ERR_CONV;}
		if (!InsertInt(p2,lstf))		{return CE_ERR_CONV;}			// in this case an index
		if (!InsertAddr(p3,lstf))		{return CE_ERR_CONV;}
		snprintf(buf,BUFFSIZE,"CEngine:: Convert: AFTER STM stmtcntr %d \n",stmtcntr);
		ProtLog(buf); 
		return CE_ERR_NOERR;
		break;

	case LDTI:
	case LDSP:
	case INC:
	case DEC:
	case PUSH:	
		ProtLog("CEngine:: Convert: INC/DEC/PUSH\n"); 
		if (!InsertOpCode(l,c,code,lstf)) {return CE_ERR_CONV;}
		if (!InsertType(p1,lstf)) 	 {return CE_ERR_CONV;}
		if (!InsertInt(p2,lstf))	 {return CE_ERR_CONV;}			// in this case an index
		Log="CEngine:: Convert: Inserted '"+*p1+"'\n"; Log.Log(); 
		Log="CEngine:: Convert: Inserted '"+*p2+"'\n"; Log.Log(); 
		return CE_ERR_NOERR;
		break;

	case CALL:	
	case JMP:	
	case JEQ:	
	case JGT:	
	case JNZ:	
		if (!InsertOpCode(l,c,code,lstf)) {return CE_ERR_CONV;}
		if (!InsertAddr(p1,lstf))	  {return CE_ERR_CONV;}
		return CE_ERR_NOERR;
		break;

	case RET:
	case HLT:	
		ProtLog("CEngine:: Convert: HLT\n"); 
		InsertOpCode(l,c,code,lstf);
		snprintf(buf,BUFFSIZE,"CEngine:: Convert: AFTER RET/HLT stmtcntr %d \n",stmtcntr);
		ProtLog(buf); 
		return CE_ERR_NOERR;
		break;

	case LD:
		Log="CEngine:: Convert: LD\n"; 
		Log.Log(); 
		if (!InsertOpCode(l,c,code,lstf)) {return CE_ERR_CONV;}
		if (!InsertType(p1,lstf)) 	 {return CE_ERR_CONV;}
		if (!InsertInt(p2,lstf))	 {return CE_ERR_CONV;}			// in this case an index
		if (!InsertType(p3,lstf)) 	 {return CE_ERR_CONV;}
		if (!InsertInt(p4,lstf))	 {return CE_ERR_CONV;}			// in this case an index
		return CE_ERR_NOERR;
		break;

	case JV:

		Log="CEngine:: Convert: JV\n";
		Log.Log();
		if (!InsertOpCode(l,c,code,lstf)) {return CE_ERR_CONV;}
		if (!InsertType(p1,lstf)) 	 {return CE_ERR_CONV;}
		if (!InsertInt(p2,lstf))	 {return CE_ERR_CONV;}			// in this case an index
		if (!InsertAddr(p3,lstf))	 {return CE_ERR_CONV;}
		return CE_ERR_NOERR;
		break;

	default:
		return CE_ERR_CONV;
		}

	} ///////////////////////////////////////////////////////////////////////////////end Convert

////////////////////////////////////////////////////////////////////////////////////////////////////
// Lookup
// Return true and value if defined
////////////////////////////////////////////////////////////////////////////////////////////////////
bool CEngine::Lookup(CString *l, unsigned long *u)
{
int	i;
char	buf[BUFFSIZE];

	// if the label is not null
	if (l)
	{
		// first check for unique label
		for (i=0; i<MAXSYMTAB; i++)
		{
			snprintf(buf,BUFFSIZE,"CEngine:: Lookup: Is symbol used ? symtab[%d] %d\n",i, 
				symtab[i].used
				);
			ProtLog(buf); 
			// is the symbol in the table now ?
//			if (symtab[i].used==true)
			if (symtab[i].used)
			{
				snprintf(buf,BUFFSIZE,"CEngine:: Lookup: comparing symtab[%d].symbol='%s' with label.'%s' \n", 
					i,
					symtab[i].symbol.GetStringPtr(),
					l->GetStringPtr());
				ProtLog(buf); 
				if (symtab[i].symbol==*l)
				{
					snprintf(buf,BUFFSIZE,"CEngine:: Lookup: Found symbol.'%s' \n", l->GetStringPtr());
					ProtLog(buf); 
					*u=symtab[i].value;
					return (true);
					}
				}
			}
		
		return false;
		}
	} ////////////////////////////////////////////////////////////////////////////////end Lookup

////////////////////////////////////////////////////////////////////////////////////////////////////
// AddToSymTab
// Should Rename to AdvanceStmtCntr or something
// returns 0 = OK
// returns not zero = error
////////////////////////////////////////////////////////////////////////////////////////////////////
int CEngine::AddToSymTab(CString *l, CString *c, 
			CString *p1, CString *p2, CString *p3, 
			CString *p4, CString *p5, CString *p6,
			int code)
{
int		i,op;
char		buf[BUFFSIZE];
unsigned long	u;
CString		Log;

//	snprintf(buf,BUFFSIZE,"CEngine:: FirstPass: command='%s' checks good",command->GetStringPtr());
	ProtLog("CEngine:: AddToSymtab: Entered.\n"); 

	// if there is a label enter it into the symtab
	if (l)
	{
		l->Chop(":");			// get rid of ':' char
		if (!Lookup(l,&u))
		{
			// now add the symbol
			if (*c==ORG)
			{
				p1->TokenInit(" ");
				p1->GetULongInt(&stmtcntr);
				snprintf(buf,BUFFSIZE,"CEngine:: AddToSymtab: Adding org Symbol '%s' with value '%lu'.\n",
					l->GetStringPtr(),stmtcntr);
				ProtLog(buf); 
				// add the label
				symtab[symcount].used=true;
				symtab[symcount].symbol=*l;
				symtab[symcount].value=stmtcntr;
				symcount++;
				org=stmtcntr;
				return CE_ERR_NOERR;
				}
			else
			{
				snprintf(buf,BUFFSIZE,"CEngine:: AddToSymtab: Adding Symbol '%s' with value '%lu'.\n",
					l->GetStringPtr(),stmtcntr);
				ProtLog(buf); 
				// add the label
				symtab[symcount].used=true;
				symtab[symcount].symbol=*l;
				symtab[symcount].value=stmtcntr;
				symcount++;
				}
			}
		else
		{
			snprintf(buf,BUFFSIZE,"CEngine:: AddToSymtab: Err redefined symbol.'%s'\n ", 
				l->GetStringPtr());
			ProtLog(buf); 
			return (CE_ERR_REDEF);
			}
		}

	// check for external operands
	c->TokenInit(" ");
	if (c->GetInt(&op))
	{	
		if (op>=1000 && op<=1199)
		{
			stmtcntr+=1;
			ProtLog("CEngine:: AddToSymtab: Adding 1\n"); 
			return CE_ERR_NOERR;
			}
		}

	// check to see if it is an assenbly directive
	if (*c==DEFS)
	{
		Log="CEngine:: AddToSymtab: parm1='"+(*p1)+"'\n";
		Log.Log();
		stmtcntr+=p1->Length();
		snprintf(buf,BUFFSIZE,"CEngine:: AddToSymtab: Adding %d.\n", p1->Length());
		ProtLog(buf);
		}
	else
	{
		// check the label, command, parm combinations
		switch (code)
		{
		case CNT:
			break;
		case ADD:
		case SUB:
		case MUL:
		case DIVD:
		case DIVI:
		case MOD:
		case EXP:
		case AND:
		case OR:
		case XOR:
			// these are all six operand opcodes
			ProtLog("CEngine:: AddToSymtab: Adding 7\n"); 
			stmtcntr+=7;
			break;

		case CMP:
		case LDR:
		case LD:
		case LDM:
			// these are all four operand opcodes
			ProtLog("CEngine:: AddToSymtab: Adding 5\n"); 
			stmtcntr+=5;
			break;

		case LDI:
			if (*p1=="b" || *p1=="i" || *p1=="d" || *p1=="u") 
			{
				ProtLog("CEngine:: AddToSymtab: Adding 5\n"); 
				stmtcntr+=5;
				}
			else
			{
				ProtLog("CEngine:: AddToSymtab: Adding 4+\n"); 
				stmtcntr+=4+p4->GetLength();
				}
			break;

		case STM:
		case STR:
		case JV:
			// these are all three operand opcodes
			ProtLog("CEngine:: AddToSymtab: Adding 4\n"); 
			stmtcntr+=4;
			break;

		case LDTI:
		case LDSP:
		case PUSH:
		case INC:
		case DEC:
			ProtLog("CEngine:: AddToSymtab: Adding 3\n"); 
			stmtcntr+=3;
			break;

		case CALL:
		case JMP:	
		case JEQ:	
		case JGT:	
		case JNZ:	
			ProtLog("CEngine:: AddToSymtab: Adding 2\n"); 
			stmtcntr+=2;
			break;
	
		case RET:
		case HLT:	
			ProtLog("CEngine:: AddToSymtab: Adding 1\n"); 
			stmtcntr+=1;
			break;
	
		default:
			return CE_ERR_SYMTABADD;
			}
		}
	snprintf(buf,BUFFSIZE,"CEngine:: AddToSymtab: stmtcntr='%d'\n",stmtcntr);
	ProtLog(buf); 
	ProtLog("CEngine:: AddToSymtab: Exited.\n"); 
	return CE_ERR_NOERR;

	} ///////////////////////////////////////////////////////////////////////////end AddToSymTab

////////////////////////////////////////////////////////////////////////////////////////////////////
// Check_Reg
// Checks to ensure the index is in range
////////////////////////////////////////////////////////////////////////////////////////////////////
bool CEngine::Check_Reg(CString *r)
{
int	v;

	r->TokenInit(" "); 
	if (r->GetInt(&v))
	{
		if (v<0 && v>9) return (false); 
		else return (true);
		}
	else return (false);
	} /////////////////////////////////////////////////////////////////////////////end Check_Reg

////////////////////////////////////////////////////////////////////////////////////////////////////
// Check_CMP
// returns 0 = OK
// returns not zero = error
////////////////////////////////////////////////////////////////////////////////////////////////////
int CEngine::Check_CMP(CString *l, CString *c, CString *p1, CString *p2, CString *p3, CString *p4)
{
char buf[BUFFSIZE];
int	r1,r2;
	snprintf(buf,BUFFSIZE,"CEngine:: Check: CMP Entry parm1='%s', parm2='%s', parm3='%s', parm4='%s'\n",
		p1->GetStringPtr(), p2->GetStringPtr(), p3->GetStringPtr(), p4->GetStringPtr());
	ProtLog(buf); 

	// check that types work
	if (
		((*p1=="b")&&(*p3=="b"))
	      ||((*p1=="i")&&(*p3=="i"))
	      ||((*p1=="u")&&(*p3=="u"))
	      ||((*p1=="d")&&(*p3=="d"))
	      ||((*p1=="s")&&(*p3=="s"))
		)
	{

		// check that registers are in range
		p2->TokenInit(" "); p4->TokenInit(" "); 
		p2->GetInt(&r1);    p4->GetInt(&r2);
		if (r1>=0 && r1<=9 && r2>=0 && r2<=9 )
		{
			snprintf(buf,BUFFSIZE,"CEngine:: Check: returning check register ranges good\n");
			ProtLog(buf); 
			return (CE_ERR_NOERR);
			}
		else 
		{
			snprintf(buf,BUFFSIZE,"CEngine:: Check: returning check register ranges bad\n");
			ProtLog(buf); 
			return CE_ERR_INVTYPE;
			}
		}
	else return (CE_ERR_INVTYPE);
	} /////////////////////////////////////////////////////////////////////////////end Check_CMP

////////////////////////////////////////////////////////////////////////////////////////////////////
// Check_AND
// returns 0 = OK
// returns not zero = error
////////////////////////////////////////////////////////////////////////////////////////////////////
int CEngine::Check_AND(CString *l, CString *c, CString *p1, CString *p2, 
			CString *p3, CString *p4, CString *p5, CString *p6)
{
CString Log;
int	r1,r2,rd;
	
	Log="CEngine:: Check: AND Entry parm1='"; 
	Log=Log+*p1+"', '"+*p2+"', '"+*p3+"', '"+*p4+"', '"+*p5+"', '"+*p6+"'\n";
	Log.Log();

	// Check to see if we are trying to AND strings
	if ((*p1=="s")||(*p3=="s")||(*p5=="s")) return CE_ERR_INVTYPE;

	// check that types work
	if (
		((*p1=="b")&&(*p3=="b")&&(*p5=="b"))
	      ||((*p1=="i")&&(*p3=="i")&&(*p5=="i"))
	      ||((*p1=="u")&&(*p3=="u")&&(*p5=="u"))
		)
	{
		// check that registers are in range
		p2->TokenInit(" "); p4->TokenInit(" "); p6->TokenInit(" ");
		p2->GetInt(&r1);
		p4->GetInt(&r2);
		p6->GetInt(&rd);
		if (r1>=0 && r1<=9 && r2>=0 && r2<=9 && rd>=0 && rd<=9)
		{
			ProtLog("CEngine:: Check: registers indexes check good\n");

			// check operation allowed, ie not anding doubles or strings.
			if (
				(p1->StrNCmp("d",1)==0) 
				|| (p3->StrNCmp("d",1)==0) 
				|| (p5->StrNCmp("d",1)==0)
				|| (p1->StrNCmp("s",1)==0)
				|| (p3->StrNCmp("s",1)==0)
				|| (p5->StrNCmp("s",1)==0)
							)
			{
				ProtLog("CEngine:: Check: returning check register types bad\n");
				return (CE_ERR_INVTYPE);
				}
			else 
			{
				ProtLog("CEngine:: Check: returning check register types good\n");
				return CE_ERR_NOERR;
				}
			}
		}

	} /////////////////////////////////////////////////////////////////////////////end Check_AND

////////////////////////////////////////////////////////////////////////////////////////////////////
// Check_ADDSUB
// returns 0 = OK
// returns not zero = error
////////////////////////////////////////////////////////////////////////////////////////////////////
int CEngine::Check_ADDSUB(CString *l, CString *c, 
			CString *p1, CString *p2, CString *p3, CString *p4, CString *p5, CString *p6)
{
char buf[BUFFSIZE];
int	r1,r2,rd;

	snprintf(buf,BUFFSIZE,"CEngine:: Check: ADDSUB Entry parm1='%s', parm2='%s', parm3='%s', parm4='%s', parm5='%s', parm6='%s'\n",
		p1->GetStringPtr(), p2->GetStringPtr(), p3->GetStringPtr(),
		p4->GetStringPtr(), p5->GetStringPtr(), p6->GetStringPtr());
	ProtLog(buf); 

	// Check to see if we are trying to sub strings
	if (((*p1=="s")||(*p3=="s")||(*p5=="s"))&&(*c=="SUB")) return CE_ERR_INVTYPE;

	// check that types work
	if (
		((*p1=="i")&&(*p3=="i")&&(*p5=="i"))
	      ||((*p1=="u")&&(*p3=="u")&&(*p5=="u"))
	      ||((*p1=="d")&&(*p3=="d")&&(*p5=="d"))
	      ||((*p1=="s")&&(*p3=="s")&&(*p5=="s"))
		)
	{

		// check that registers are in range
		p2->TokenInit(" "); p4->TokenInit(" "); p6->TokenInit(" ");
		p2->GetInt(&r1);
		p4->GetInt(&r2);
		p6->GetInt(&rd);
		if (r1>=0 && r1<=9 && r2>=0 && r2<=9 && rd>=0 && rd<=9)
		{
			ProtLog("CEngine:: Check: registers indexes check good\n");

			// check operation allowed, ie not adding bools.
			if ((p1->StrNCmp("b",1)==0) || (p3->StrNCmp("b",1)==0) || (p5->StrNCmp("b",1)==0))
			{
				ProtLog("CEngine:: Check: returning check register types bad\n");
				return (CE_ERR_INVTYPE);
				}
			else 
			{
				ProtLog("CEngine:: Check: returning check register types good\n");
				return CE_ERR_NOERR;
				}
			}
		}
	else return (CE_ERR_INVTYPE);
	} //////////////////////////////////////////////////////////////////////////end Check_ADDSUB

////////////////////////////////////////////////////////////////////////////////////////////////////
// Check_LDI
// returns 0 = OK
// returns not zero = error
////////////////////////////////////////////////////////////////////////////////////////////////////
int CEngine::Check_LDI(CString *l, CString *c, CString *p1, CString *p2, CString *p3, CString *p4)
{
char buf[BUFFSIZE];
int	r1,r2,rd;
CString	Log;

	//ldi     b,0,1,10
	Log="CEngine:: Check_LDI: LDI Entry parm1='"+*p1+"' parm2='"+*p2+"' parm3='"+*p3+"' parm4='"+*p4+"' \n";
	Log.Log();

	// check that reg index is good
	if (!Check_Reg(p2)) return (CE_ERR_BADREG);

	// check the diffrent types
	if (*p1=="b")
	{
		// check boolean case data length
		if (*p3!="1") return (CE_ERR_LENGTH);
		}
	else if (*p1=="u")
	{
		// check integer case data length
		if (*p3!="1") return (CE_ERR_LENGTH);
		}
	else if (*p1=="i")
	{
		// check integer case data length
		if (*p3!="1") return (CE_ERR_LENGTH);
		}
	else if (*p1=="d")
	{
		// check double case data length
		if (*p3!="4") return (CE_ERR_LENGTH);
		}
	else if (*p1=="s")
	{
	int l;
		// check string case data length
		p3->TokenInit(" ");
		p3->GetInt(&l);
		p4->ConvertBackSlash();
		if (p4->GetLength()!=l) 
		{
			ProtLog("CEngine:: Check_LDI: Failed string length check.\n"); 
			snprintf(buf,BUFFSIZE,"CEngine:: Check_LDI: parm4 length='%d'\n", p4->GetLength() );
			ProtLog(buf); 
			return (CE_ERR_LENGTH);
			}
		}
	else return (CE_ERR_INVTYPE);
	return (CE_ERR_NOERR);
	} /////////////////////////////////////////////////////////////////////////////end Check_LDI

////////////////////////////////////////////////////////////////////////////////////////////////////
// Check_STR
// returns 0 = OK
// returns not zero = error
// parms are Vartype,varindex,indexof u containing offset, index of u containinglen
////////////////////////////////////////////////////////////////////////////////////////////////////
int CEngine::Check_STR(CString *l, CString *c, CString *p1, CString *p2, CString *p3)
{
char buf[BUFFSIZE];
int	r1,r2,rd;
CString	Log;

	//ldm     b,0,addr index,lengthindex
	Log="CEngine:: Check_STR: Entry parm1='"+*p1+"' parm2='"+*p2+"' parm3='"+*p3+"' \n";
	Log.Log();

	// check that reg index is good
	if (!Check_Reg(p2)) return (CE_ERR_BADREG);
	if (!Check_Reg(p3)) return (CE_ERR_BADREG);

	// check the diffrent types
	if ((*p1!="b") && (*p1!="u") && (*p1!="i") && (*p1!="d") && (*p1!="s")) return (CE_ERR_INVTYPE);
	return (CE_ERR_NOERR);
	} /////////////////////////////////////////////////////////////////////////////end Check_STR

////////////////////////////////////////////////////////////////////////////////////////////////////
// Check_LDR
// returns 0 = OK
// returns not zero = error
// parms are Vartype,varindex,indexof u containing offset, index of u containinglen
////////////////////////////////////////////////////////////////////////////////////////////////////
int CEngine::Check_LDR(CString *l, CString *c, CString *p1, CString *p2, CString *p3, CString *p4)
{
char buf[BUFFSIZE];
int	r1,r2,rd;
CString	Log;

	//ldm     b,0,addr index,lengthindex
	Log="CEngine:: Check_LDR: Entry parm1='"+*p1+"' parm2='"+*p2+"' parm3='"+*p3+"' parm4='"+*p4+"' \n";
	Log.Log();

	// check that reg index is good
	if (!Check_Reg(p2)) return (CE_ERR_BADREG);
	if (!Check_Reg(p3)) return (CE_ERR_BADREG);
	if (!Check_Reg(p4)) return (CE_ERR_BADREG);

	// check the diffrent types
	if ((*p1!="b") && (*p1!="u") && (*p1!="i") && (*p1!="d") && (*p1!="s")) return (CE_ERR_INVTYPE);
	return (CE_ERR_NOERR);
	} /////////////////////////////////////////////////////////////////////////////end Check_LDR

////////////////////////////////////////////////////////////////////////////////////////////////////
// Check_LDM
// returns 0 = OK
// returns not zero = error
// parms are Vartype,varindex,len, addr
////////////////////////////////////////////////////////////////////////////////////////////////////
int CEngine::Check_LDM(CString *l, CString *c, CString *p1, CString *p2, CString *p3, CString *p4)
{
char buf[BUFFSIZE];
int	r1,r2,rd;
CString	Log;

	//ldm     b,0,offsetindex,lengthindex
	Log="CEngine:: Check_LDM: LDM Entry parm1='"+*p1+"' parm2='"+*p2+"' parm3='"+*p3+"' parm4='"+*p4+"' \n";
	Log.Log();

	// check that reg index is good
	if (!Check_Reg(p2)) return (CE_ERR_BADREG);

	// check the diffrent types
	if ((*p1!="b") && (*p1!="u") && (*p1!="i") && (*p1!="d") && (*p1!="s")) return (CE_ERR_INVTYPE);

	// TODO
	// check that len is either symbol or int
	// check that addr is either symbol or int

	return (CE_ERR_NOERR);
	} /////////////////////////////////////////////////////////////////////////////end Check_LDM

////////////////////////////////////////////////////////////////////////////////////////////////////
// Check
// returns 0 = OK
// returns not zero = error
////////////////////////////////////////////////////////////////////////////////////////////////////
int CEngine::Check(CString *l, CString *c, 
			CString *p1, CString *p2, CString *p3, 
			CString *p4, CString *p5, CString *p6,
			int code)
{
int	i;
char	buf[BUFFSIZE];
int	r1,r2,rd;
int	op;
CString	Log;

	Log="CEngine:: Check: Entered\n";
	Log.Log();
	// check for assembler directives
	if (*c==ORG)
	{
		snprintf(buf,BUFFSIZE,"CEngine:: Check: ORG Entry parm1='%s'\n", p1->GetStringPtr() );
		ProtLog(buf); 
	
		// check that registers are in range
		p1->TokenInit(" "); 
		p1->GetInt(&r1);
		if (r1>=0 && r1<MAXSCRIPT)
		{
			ProtLog("CEngine:: Check: org value checks good\n");
			return CE_ERR_NOERR;
			}
		else 
		{
			snprintf(buf,BUFFSIZE,"CEngine:: Check: returning check register types bad\n");
			ProtLog(buf); 
			return (CE_ERR_INVTYPE);
			}
		}
	
	Log="CEngine:: Check: Before assembler directives\n";
	Log.Log();
	// check for assembler directives
	if (*c==DEFS)
	{
		p1->ConvertBackSlash();
		Log="CEngine:: Check: 'ds' Entry label = '"+(*l)+"', parm1='"+(*p1)+"'\n";
		Log.Log();
		return CE_ERR_NOERR;
		}

	// check for external operands
	c->TokenInit(" ");
	if (c->GetInt(&op))
	{	
		if (op>=1000 && op<=1199)
		{
			return CE_ERR_NOERR;
			}
		}

	Log="CEngine:: Check: Before switch\n";
	Log.Log();
	// check the label, command, parm combinations
	switch (code)
	{
	case CNT:
		break;

	case SUB:
	case ADD:
		{
		int	result;

			result=Check_ADDSUB(l,c,p1,p2,p3,p4,p5,p6);
			if (result==CE_ERR_NOERR)
			ProtLog("CEngine:: Check: Check_ADDSUB returned ok.\n");
			else ProtLog("CEngine:: Check: Check_ADDSUB returned falure.\n");
			return (result);
			}
		break;

	case AND:
		{
		int	result;

			result=Check_AND(l,c,p1,p2,p3,p4,p5,p6);
			if (result==CE_ERR_NOERR)
			ProtLog("CEngine:: Check: Check_AND returned ok.\n");
			else ProtLog("CEngine:: Check: Check_AND returned falure.\n");
			return (result);
			}
		break;

	case CMP:
		{
		int	result;

			result=Check_CMP(l,c,p1,p2,p3,p4);
			if (result==CE_ERR_NOERR)
			ProtLog("CEngine:: Check: Check_CMP returned ok.\n");
			else ProtLog("CEngine:: Check: Check_CMP returned falure.\n");
			return (result);
			}
		break;

	case JMP:
	case JEQ:
	case JGT:
	case JNZ:
		snprintf(buf,BUFFSIZE,"CEngine:: Check: J?? Entry parm1='%s'\n", p1->GetStringPtr());
		ProtLog(buf); 
		return CE_ERR_NOERR;
		break;

	case JV:
		snprintf(buf,BUFFSIZE,"CEngine:: Check: JV Entry parm1='%s'\n", p1->GetStringPtr());
		ProtLog(buf); 
		if (!Check_Reg(p2)) return (CE_ERR_BADREG);
		else return (CE_ERR_NOERR);
		break;

	case MUL:
	case DIVD:
	case DIVI:
	case MOD:
	case EXP:
	case OR:
	case XOR:
		// these are all six operand opcodes
		
		break;

	case RET:
		// if no parms (this is correct for ret)
		if ((!p1) && (!p2) && (!p3) && (!p4) && (!p5) && (!p6))
		return (CE_ERR_NOERR);
		else return(CE_ERR_PARM);
		break;
	
	case HLT:
		return (CE_ERR_NOERR);
		break;

	case LDI:
		{
		int	result;

			result=Check_LDI(l,c,p1,p2,p3,p4);
			if (result==CE_ERR_NOERR)
			ProtLog("CEngine:: Check: Check_LDI returned ok.\n");
			else ProtLog("CEngine:: Check: Check_LDI returned falure.\n");
			return (result);
			}
		break;

	case STR:
		{
		int	result;

			result=Check_STR(l,c,p1,p2,p3);
			if (result==CE_ERR_NOERR)
			ProtLog("CEngine:: Check: Check_STR returned ok.\n");
			else ProtLog("CEngine:: Check: Check_STR returned falure.\n");
			return (result);
			}
		break;

	case LDR:
		{
		int	result;

			result=Check_LDR(l,c,p1,p2,p3,p4);
			if (result==CE_ERR_NOERR)
			ProtLog("CEngine:: Check: Check_LDR returned ok.\n");
			else ProtLog("CEngine:: Check: Check_LDR returned falure.\n");
			return (result);
			}
		break;

	case LDM:
		{
		int	result;

			result=Check_LDM(l,c,p1,p2,p3,p4);
			if (result==CE_ERR_NOERR)
			ProtLog("CEngine:: Check: Check_LDM returned ok.\n");
			else ProtLog("CEngine:: Check: Check_LDM returned falure.\n");
			return (result);
			}
		break;

	case STM:
		Log="CEngine:: Check: STM Entry parm3='"+*p3+"'\n"; Log.Log();
		// check that types work
		if ( (*p1=="u") || (*p1=="i") || (*p1=="b") )
		{
	
			// check that registers are in range
			if (!Check_Reg(p2)) return (CE_ERR_BADREG);
			return(CE_ERR_NOERR);

			}
		else return (CE_ERR_INVTYPE);
		
		break;

	case CALL:
//		snprintf(buf,BUFFSIZE,"CEngine:: Check: CALL Entry parm1='%s'\n", p1->GetStringPtr());
//		ProtLog(buf); 
		Log="CEngine:: Check: CALL Entry parm1='"+*p1+"'\n";
		Log.Log();
		return CE_ERR_NOERR;
		break;

	case INC:
	case DEC:
		snprintf(buf,BUFFSIZE,"CEngine:: Check: INC/DEC Entry parm1='%s', parm2='%s'\n",
			p1->GetStringPtr(), p2->GetStringPtr() );
		ProtLog(buf); 

		// check that types work
		if ( (*p1=="u") || (*p1=="i") || (*p1=="d") )
		{
	
			// check that registers are in range
			p2->TokenInit(" "); 
			p2->GetInt(&r1);
			if (r1>=0 && r1<=9)
			{
				ProtLog("CEngine:: Check: registers indexes check good\n");

				// check operation allowed, ie not adding bools.
				if ((p1->StrNCmp("b",1)==0) || (p1->StrNCmp("s",1)==0))
				{
					snprintf(buf,BUFFSIZE,"CEngine:: Check: returning check register types bad\n");
					ProtLog(buf); 
					return (CE_ERR_INVTYPE);
					}
				else 
				{
					snprintf(buf,BUFFSIZE,"CEngine:: Check: returning check register types good\n");
					ProtLog(buf); 
					return CE_ERR_NOERR;
					}
				}
			}
		else return (CE_ERR_INVTYPE);
		break;

	case LDTI:
	case LDSP:
	case PUSH:
		Log="CEngine:: Check: LDTI/LDSP/PUSH Entry parm1='"+*p1+"'\n";
		Log.Log();
		
		// check type
		if (*p1=="i") 
		{
			// check that registers are in range
			p2->TokenInit(" "); 
			p2->GetInt(&r1);
			if (r1>=0 && r1<=9)
			{
				// returning good
				
				return CE_ERR_NOERR;
				}
			else return (CE_ERR_BADREG);
			}
		else 
		{
			Log="CEngine:: Check: Returning ERR_INVTYPE\n";
			Log.Log();
			return (CE_ERR_INVTYPE);
			}
		break;

	case LD:
		// check that registers are in range
		p2->TokenInit(" "); 
		p4->TokenInit(" "); 
		p2->GetInt(&r1);
		p4->GetInt(&r2);
		if (r1<0 || r1>9) return CE_ERR_BADREG; 
		if (r2<0 || r2>9) return CE_ERR_BADREG; 

		// test register move combinations
		if ((*p1=="b")  && (*p3=="b")) return CE_ERR_NOERR;
		if ((*p1=="i")  && (*p3=="i")) return CE_ERR_NOERR;
		if ((*p1=="u")  && (*p3=="u")) return CE_ERR_NOERR;
		if ((*p1=="d")  && (*p3=="d")) return CE_ERR_NOERR;
		if ((*p1=="s")  && (*p3=="s")) return CE_ERR_NOERR;
		
		// implicit conversions to dbls
		if ((*p1=="i")  && (*p3=="d")) return CE_ERR_NOERR;
		if ((*p1=="u")  && (*p3=="d")) return CE_ERR_NOERR;
//		if ((*p1=="s")  && (*p3=="d")) return CE_ERR_NOERR;

		// implicit conversions to strs
//		if ((*p1=="b")  && (*p3=="s")) return CE_ERR_NOERR;
//		if ((*p1=="i")  && (*p3=="s")) return CE_ERR_NOERR;
//		if ((*p1=="u")  && (*p3=="s")) return CE_ERR_NOERR;
//		if ((*p1=="d")  && (*p3=="s")) return CE_ERR_NOERR;
		
		// implicit conversions to ints
		if ((*p1=="i")  && (*p3=="u")) return CE_ERR_NOERR;
		if ((*p1=="u")  && (*p3=="i")) return CE_ERR_NOERR;
//		if ((*p1=="s")  && (*p3=="i")) return CE_ERR_NOERR;
	
		// implicit conversions to uints
//		if ((*p1=="s")  && (*p3=="u")) return CE_ERR_NOERR;

		return (CE_ERR_INVTYPE);
		break;

	default:
		return CE_ERR_BADOPC;
		}
	return CE_ERR_INTERR;
	} /////////////////////////////////////////////////////////////////////////////////end Check

////////////////////////////////////////////////////////////////////////////////////////////////////
// ResetSymtab
////////////////////////////////////////////////////////////////////////////////////////////////////
void CEngine::ResetSymtab()
{
int	i;

	symcount=0;
	symtab = new symtabtype[MAXSYMTAB];
	for (i=0;i<MAXSYMTAB;i++)
	{
		symtab[i].used=false;
		symtab[i].symbol.Reset();
		symtab[i].value=0;
		}
	org=MAXSCRIPT;	// set org to INVALID value, remember 0->MAXSCRIPT-1 
	} ///////////////////////////////////////////////////////////////////////////end ResetSymtab

////////////////////////////////////////////////////////////////////////////////////////////////////
// Pass 
// This will be a VERY simple two pass assembler.
// When Assembling we WONT be running a script.
// Here we will read through the source file and generate the symbol table
////////////////////////////////////////////////////////////////////////////////////////////////////
int CEngine::Pass(int pass, bool cmdline)
		// check operatin allowed, ie not adding bools.
{
FILE		*f;
FILE		*lstf;
char		buf[BUFFSIZE];
int		state;
int		i;
//char            delims[8]={',',' ','\t','/',';','"','\0'};  		// <<===String contains a tab and a space
char            delims[6]={',',' ','\t','"','\0'};  			// <<===String contains a tab and a space
char            delims1[3]={',','"','\0'};  				// <<===String for matching strings
CString		line;
int		code;		// this is the opcode to go along with the command
CString		*label;		// this is what will hold the label
CString		*command;	// this is what will hold the command
CString		*parm1;		// this is what will hold the parm
CString		*parm2;		// this is what will hold the parm
CString		*parm3;		// this is what will hold the parm
CString		*parm4;		// this is what will hold the parm
CString		*parm5;		// this is what will hold the parm
CString		*parm6;		// this is what will hold the parm
CString		*comment;	// this is what will hold the parm
char		ps[10];
CString		Log;
int result;

	ProtLog("CEngine:: Pass: Entered.\n");

	if (pass==CE_ASM_FIRST)
	{
		strncpy(ps,"First",10);
		}
	else
	{
		strncpy(ps,"Second",10);
		}

	// open the file
	if (!(f=fopen(inputfile.GetStringPtr(),"r"))) return (CE_ERR_BADFILE);

	// open the lstfile
	if (makelistfile) 
	{
		if (!(lstf=fopen(lstfile.GetStringPtr(),"w"))) return (CE_ERR_BADFILE);
		fprintf(lstf,"LIST FILE FOR %s \n",inputfile.GetStringPtr());
		}

	// start statemachine
	state=ST_RESET;
	while (state!=ST_END)
	switch (state)
	{
	case ST_RESET:
		snprintf(buf,BUFFSIZE,"CEngine:: %s Pass: RESET\n",ps);
		ProtLog(buf); 
		// reset the engine, and prep for build symtab
		Reset();
		stmtcntr=0;
		linenum=0;
		if (pass==CE_ASM_FIRST) ResetSymtab();
		state=ST_BEGIN;
		break;

	case ST_BEGIN:
		snprintf(buf,BUFFSIZE,"CEngine:: %s Pass: =========================== ST_BEGIN\n",ps);
		ProtLog(buf); 
		label=0;command=0;parm1=0;parm2=0;parm3=0;parm4=0;parm5=0;parm6=0;comment=0;
		if (line.GetLine(f))
		{
		CString	*tok1;

			linenum++;
			line.Chomp();
			snprintf(buf,BUFFSIZE,"CEngine:: %s Pass: Working on '%s' linenum='%d'\n",
				ps,line.GetStringPtr(),linenum);
			ProtLog(buf); 
			if ((line.StrNCmp("//",2)==0)||(line.StrNCmp(";",1)==0)||line.StrNCmp("\n",1)==0)  
			{
				state=ST_COMMENT;
				}
			else
			{
				// prep tokenizer
				line.TokenInit(delims);	// <<===String contains a tab and a space
				
				// get first token and find out what it is
				tok1=line.GetToken();
				if      (IsOpCode(tok1,&code)) {state=ST_OPCODE;    command=tok1;}
				else if (IsExtOpCode(tok1))    {state=ST_EXTOP;     command=tok1;}
				else if (IsDir(tok1))          {state=ST_DIRECTIVE; command=tok1;}
				else if (IsLabel(tok1))        {state=ST_LABEL;     label=tok1;}
				else state=ST_ERR4;
				}	
			}
		else
		{
			state=ST_END;
			}

		break;

	case ST_COMMENT:
		if (loglevel>0) snprintf(buf,BUFFSIZE,"CEngine:: %s Pass: ST_COMMENT\n",ps);
		if (loglevel>0) ProtLog(buf); 
		line.Chomp();
		if (loglevel>0) snprintf(buf,BUFFSIZE,"CEngine:: %s Pass: '%s'\n",ps,line.GetStringPtr());
		if (loglevel>0) ProtLog(buf); 
		state=ST_BEGIN;
		break;

	case ST_LABEL:
		snprintf(buf,BUFFSIZE,"CEngine:: %s Pass: ST_LABEL\n",ps);
		ProtLog(buf); 
//		ProtLog("CEngine:: FirstPass: ST_LABEL"); 

		// get next token and find out what it is
		command=line.GetToken();
		if (!command)                        state=ST_ERR6;
		else if (IsOpCode(command,&code))    state=ST_OPCODE;
		else if (IsDir(command))             state=ST_DIRECTIVE;
		else if (IsExtOpCode(command))       state=ST_EXTOP;
		else    state=ST_ERR6;
		break;

	case ST_OPCODE:
		{
		int cnt;
			snprintf(buf,BUFFSIZE,"CEngine:: %s Pass: ST_OPCODE\n",ps);
			ProtLog(buf); 
//			ProtLog("CEngine:: FirstPass: ST_OPCODE"); 
			cnt=GetParmCountOpCode(command);
			if      ((cnt<0)||(cnt>6)) state=ST_ERR25;
			else if (cnt==0) state=ST_CHECK;
			else if (cnt==1) state=ST_ONE_PARM;
			else if (cnt==2) state=ST_TWO_PARM;
			else if (cnt==3) state=ST_THREE_PARM;
			else if (cnt==4) state=ST_FOUR_PARM;
			else if (cnt==5) state=ST_FIVE_PARM;
			else if (cnt==6) state=ST_SIX_PARM;
			else state=ST_ERR25;
			}

//		state=ST_BEGIN;
		break;

	case ST_EXTOP:
		snprintf(buf,BUFFSIZE,"CEngine:: %s Pass: ST_EXTOP\n",ps);
		ProtLog(buf); 
//		ProtLog("CEngine:: FirstPass: ST_EXTOP"); 
		parm1=line.GetToken();
		if (parm1)
		{	
			Log="CEngine:: "; Log=Log+ps+" Pass: ST_EXTOP parm1='"+*parm1+"'\n";
			Log.Log();
			if (parm1->GetChar(0)==';') state=ST_CHECK;
			else if (parm1->GetChar(0)=='/' && parm1->GetChar(1)=='/') state=ST_CHECK;
			else 
			{
				result=CE_ERR_PARM;
				state=ST_ERR26;
				}
			}
		else state=ST_CHECK;
		break;

	case ST_DIRECTIVE:
		{
		int cnt;
			snprintf(buf,BUFFSIZE,"CEngine:: %s Pass: ST_DIRECTIVE\n",ps);
			ProtLog(buf); 
//			ProtLog("CEngine:: FirstPass: ST_DIRECTIVE"); 
			cnt=GetParmCountDir(command);
			if      (cnt<0) state=ST_ERR25;
			else if (cnt==0) state=ST_CHECK;
			else if (cnt==1) state=ST_ONE_PARM;
			else state=ST_ERR25;
			}
		break;

	case ST_CHECK:
		snprintf(buf,BUFFSIZE,"CEngine:: %s Pass: ST_CHECK\n",ps);
		ProtLog(buf); 
		{
//		int result;
		
			result=Check(label, command, parm1, parm2, parm3, parm4, parm5, parm6, code);
			if ((result==CE_ERR_NOERR)&&(pass==CE_ASM_SECOND)) 
			{
				state=ST_ASSEM;
				}
			else if ((result==CE_ERR_NOERR)&& (pass==CE_ASM_FIRST))
			{
				state=ST_SYMTAB;
				}
			else
			{
				state=ST_CHECK_ERR;
				}
			}
		break;

	case ST_CHECK_ERR:
		Log="CEngine:: ";Log=Log+ps; Log=Log+" Pass: ST_CHECK_ERR\n";
		Log.Log();
		if (cmdline) 
		{
			printf("==============================\n");
			printf("Error: Line = '%d' errornum='%d'\n", linenum,result);
			line.Print(); 
			printf("\n");
			Error(result,CE_BOTH);
//			printf("Error: Line = '%d'\n", linenum);
			}
		else
		{
			snprintf(buf,BUFFSIZE,"Error: Line = '%d'\n",linenum);
			ProtLog(buf); 
			}
		state=ST_BEGIN;
		break;

	case ST_SYMTAB:
		{
//		int	result;

			snprintf(buf,BUFFSIZE,"CEngine:: %s Pass: ST_SYMTAB\n",ps);
			ProtLog(buf); 
			snprintf(buf,BUFFSIZE,"CEngine:: %s Pass: command='%s' checks good\n",ps,command->GetStringPtr());
			ProtLog(buf); 
			result=AddToSymTab(label, command, parm1, parm2, parm3, parm4, parm5, parm6, code);
			if (result!=CE_ERR_NOERR)
			{
				if (result==CE_ERR_REDEF)
				{
					snprintf(buf,BUFFSIZE,"CEngine:: %s Pass: Line = %d redfined symbol '%s'\n",
						ps,linenum,label->GetStringPtr());
					}

				if (cmdline) { printf(buf); }
				else { ProtLog(buf); }

				state=ST_SYMTABERR;
				}
			else state=ST_BEGIN;
			}
		break;

	case ST_SYMTABERR:
		Log="CEngine:: "; Log=Log+ps; Log=Log+" Pass: ST_SYMTABERR\n";
		Log.Log();

		state=ST_BEGIN;
		break;

	case ST_ONE_PARM:

		snprintf(buf,BUFFSIZE,"CEngine:: %s Pass: ST_ONE_PARM\n",ps);
		ProtLog(buf); 
//		ProtLog("CEngine:: FirstPass: ST_ONE_PARM"); 
		if (*command==DEFS)
		{
			snprintf(buf,BUFFSIZE,"CEngine:: %s Pass: Getting 'ds' parms\n",ps);
			ProtLog(buf); 
//			ProtLog("CEngine:: FirstPass: Getting 'ds' parms"); 
			line.SetDelim(delims1);
			parm1=line.GetToken();
			parm1=line.GetToken();
			}
		else parm1=line.GetToken();
		
		// now just check to see if they skipped the one parm they needed
		if (parm1)
		{
			snprintf(buf,BUFFSIZE,"CEngine:: %s Pass: parm1='%s'\n",ps,parm1->GetStringPtr());
			ProtLog(buf); 
			if      (parm1->StrNCmp("//",2)==0) {result=CE_ERR_PARM;state=ST_ERR26;}
			else if (parm1->StrNCmp(";",1)==0)  {result=CE_ERR_PARM;state=ST_ERR26;}
			else state=ST_CHECK;
			}
		else {result=CE_ERR_PARM;state=ST_ERR26;}
//		if (parm2) state=ST_ERR26; if they have more than one parm
		break;

	case ST_TWO_PARM:
		snprintf(buf,BUFFSIZE,"CEngine:: %s Pass: ST_TWO_PARM\n",ps);
		ProtLog(buf); 
		parm1=line.GetToken();
		parm2=line.GetToken();
		
		// now just check to see if they skipped the two parms they needed
		if ((parm1)&&(parm2))
		{
			snprintf(buf,BUFFSIZE,"CEngine:: %s Pass: parm1='%s', parm2='%s'\n",
				ps,
				parm1->GetStringPtr(),
				parm2->GetStringPtr());
			ProtLog(buf); 
			if      (parm1->StrNCmp("//",2)==0)  state=ST_ERR26;
			else if (parm1->StrNCmp(";",1)==0)   state=ST_ERR26;
			else if (parm2->StrNCmp("//;",2)==0) state=ST_ERR26;
			else if (parm2->StrNCmp(";",1)==0)   state=ST_ERR26;
			else state=ST_CHECK;
			}
		else state=ST_ERR26;
		break;

	case ST_THREE_PARM:
		snprintf(buf,BUFFSIZE,"CEngine:: %s Pass: ST_THREE_PARM\n",ps);
		ProtLog(buf); 
		parm1=line.GetToken();
		parm2=line.GetToken();
		parm3=line.GetToken();
		
		// now just check to see if they skipped the two parms they needed
		if ((parm1)&&(parm2)&&(parm3))
		{
			snprintf(buf,BUFFSIZE,"CEngine:: %s Pass: parm1='%s', parm2='%s', parm3='%s'\n",
				ps,
				parm1->GetStringPtr(),
				parm2->GetStringPtr(),
				parm3->GetStringPtr());
			ProtLog(buf); 
			if      (parm1->StrNCmp("//",2)==0)  state=ST_ERR26;
			else if (parm1->StrNCmp(";",1)==0)   state=ST_ERR26;
			else if (parm2->StrNCmp("//;",2)==0) state=ST_ERR26;
			else if (parm2->StrNCmp(";",1)==0)   state=ST_ERR26;
			else if (parm3->StrNCmp("//;",2)==0) state=ST_ERR26;
			else if (parm3->StrNCmp(";",1)==0)   state=ST_ERR26;
			else state=ST_CHECK;
			}
		else state=ST_ERR26;
		break;

	case ST_FOUR_PARM:
		snprintf(buf,BUFFSIZE,"CEngine:: %s Pass: ST_FOUR_PARM\n",ps);
		ProtLog(buf); 
		parm1=line.GetToken();
		parm2=line.GetToken();
		parm3=line.GetToken();
		if (parm1)
		if (parm1->StrNCmp("s",1)==0)
		{
			line.SetDelim(delims1);
			parm4=line.GetToken();
			}
		else parm4=line.GetToken();
		
		// now just check to see if they skipped any of the four parms they needed
		if ((parm1)&&(parm2)&&(parm3)&&(parm4))
		{
			snprintf(buf,BUFFSIZE,"CEngine:: %s Pass: parm1='%s', parm2='%s', parm3='%s', parm4='%s'\n",
				ps,
				parm1->GetStringPtr(),
				parm2->GetStringPtr(),
				parm3->GetStringPtr(),
				parm4->GetStringPtr());
			ProtLog(buf); 
			if      (parm1->StrNCmp("//",2)==0)  {result=CE_ERR_PARM;state=ST_ERR26;}
			else if (parm1->StrNCmp(";",1)==0)   {result=CE_ERR_PARM;state=ST_ERR26;}
			else if (parm2->StrNCmp("//;",2)==0) {result=CE_ERR_PARM;state=ST_ERR26;}
			else if (parm2->StrNCmp(";",1)==0)   {result=CE_ERR_PARM;state=ST_ERR26;}
			else if (parm3->StrNCmp("//;",2)==0) {result=CE_ERR_PARM;state=ST_ERR26;}
			else if (parm3->StrNCmp(";",1)==0)   {result=CE_ERR_PARM;state=ST_ERR26;}
			else if (parm4->StrNCmp("//;",2)==0) {result=CE_ERR_PARM;state=ST_ERR26;}
			else if (parm4->StrNCmp(";",1)==0)   {result=CE_ERR_PARM;state=ST_ERR26;}
			else state=ST_CHECK;
			}
		else {result=CE_ERR_PARM;state=ST_ERR26;}
		break;

	case ST_FIVE_PARM:
		snprintf(buf,BUFFSIZE,"CEngine:: %s Pass: ST_FIVE_PARM\n",ps);
		ProtLog(buf); 
		parm1=line.GetToken();
		parm2=line.GetToken();
		parm3=line.GetToken();
		parm4=line.GetToken();
		parm5=line.GetToken();
		
		// now just check to see if they skipped the two parms they needed
		if ((parm1)&&(parm2)&&(parm3)&&(parm4)&&(parm5))
		{
			snprintf(buf,BUFFSIZE,"CEngine:: %s Pass: parm1='%s', parm2='%s', parm3='%s', parm4='%s', parm5='%s'\n",
				ps,
				parm1->GetStringPtr(),
				parm2->GetStringPtr(),
				parm3->GetStringPtr(),
				parm4->GetStringPtr(),
				parm5->GetStringPtr());
			ProtLog(buf); 
			if      (parm1->StrNCmp("//",2)==0)  state=ST_ERR26;
			else if (parm1->StrNCmp(";",1)==0)   state=ST_ERR26;
			else if (parm2->StrNCmp("//;",2)==0) state=ST_ERR26;
			else if (parm2->StrNCmp(";",1)==0)   state=ST_ERR26;
			else if (parm3->StrNCmp("//;",2)==0) state=ST_ERR26;
			else if (parm3->StrNCmp(";",1)==0)   state=ST_ERR26;
			else if (parm4->StrNCmp("//;",2)==0) state=ST_ERR26;
			else if (parm4->StrNCmp(";",1)==0)   state=ST_ERR26;
			else if (parm5->StrNCmp("//;",2)==0) state=ST_ERR26;
			else if (parm5->StrNCmp(";",1)==0)   state=ST_ERR26;
			else state=ST_CHECK;
			}
		else state=ST_ERR26;
		break;

	case ST_SIX_PARM:
		snprintf(buf,BUFFSIZE,"CEngine:: %s Pass: ST_SIX_PARM\n",ps);
		ProtLog(buf); 
		parm1=line.GetToken();
		parm2=line.GetToken();
		parm3=line.GetToken();
		parm4=line.GetToken();
		parm5=line.GetToken();
		parm6=line.GetToken();
		
		// now just check to see if they skipped the two parms they needed
		if ((parm1)&&(parm2)&&(parm3)&&(parm4)&&(parm5)&&(parm6))
		{
			snprintf(buf,BUFFSIZE,"CEngine:: %s Pass: parm1='%s', parm2='%s', parm3='%s', parm4='%s', parm5='%s', parm6='%s'\n",
				ps,
				parm1->GetStringPtr(),
				parm2->GetStringPtr(),
				parm3->GetStringPtr(),
				parm4->GetStringPtr(),
				parm5->GetStringPtr(),
				parm6->GetStringPtr());
			ProtLog(buf); 
			if      (parm1->StrNCmp("//",2)==0)  state=ST_ERR26;
			else if (parm1->StrNCmp(";",1)==0)   state=ST_ERR26;
			else if (parm2->StrNCmp("//;",2)==0) state=ST_ERR26;
			else if (parm2->StrNCmp(";",1)==0)   state=ST_ERR26;
			else if (parm3->StrNCmp("//;",2)==0) state=ST_ERR26;
			else if (parm3->StrNCmp(";",1)==0)   state=ST_ERR26;
			else if (parm4->StrNCmp("//;",2)==0) state=ST_ERR26;
			else if (parm4->StrNCmp(";",1)==0)   state=ST_ERR26;
			else if (parm5->StrNCmp("//;",2)==0) state=ST_ERR26;
			else if (parm5->StrNCmp(";",1)==0)   state=ST_ERR26;
			else if (parm6->StrNCmp("//;",2)==0) state=ST_ERR26;
			else if (parm6->StrNCmp(";",1)==0)   state=ST_ERR26;
			else state=ST_CHECK;
			}
		else state=ST_ERR26;
		break;

	case ST_ASSEM:
		{
//		int	result;
		char	errormsg[256];

			result=Convert(label, command, parm1, parm2, parm3, parm4, parm5, parm6, code, lstf);
			if (result==CE_ERR_NOERR)
			{
				state=ST_BEGIN;
				}
			else
			{
				if (cmdline) 
				{
					printf("Second pass error: Invalid token linenumber = '%d' \n",linenum); 
					printf("%s \n",line.GetStringPtr());
					printf("Error = '%d' \n\n",result); 

					}
				else
				{
					snprintf(buf,BUFFSIZE,"Second pass error: Invalid token linenumber = '%d'\n",linenum);
					ProtLog(buf);
					}

				state=ST_BEGIN;
				}
			}
		break;

	case ST_ERR4:
		if (cmdline) 
		{
			printf("Error: Invalid token linenumber = '%d' \n",linenum); 
			snprintf(buf,BUFFSIZE,"'%s'\n",line.GetStringPtr());
			printf(buf);
			}
		else
		{
			snprintf(buf,BUFFSIZE,"Error: Invalid token linenumber = '%d' \n",linenum); 
			ProtLog(buf);
			snprintf(buf,BUFFSIZE,"'%s'\n",line.GetStringPtr());
			ProtLog(buf);
			}
		if (label) delete label;
		if (command) delete command;
		state=ST_BEGIN;
		break;

	case ST_ERR6:
		snprintf(buf,BUFFSIZE,"CEngine:: %s Pass: ST_ERR6\n",ps);
		ProtLog(buf); 

		snprintf(buf,BUFFSIZE,"Error: Invalid opcode. linenumber ='%d' \n",linenum);
		printf(buf);
		if (label) delete label;
		if (command) delete command;
		state=ST_BEGIN;
		break;

	case ST_ERR25:
		snprintf(buf,BUFFSIZE,"CEngine:: %s Pass: ST_ERR25\n",ps);
		ProtLog(buf); 
		state=ST_BEGIN;
		break;

	case ST_ERR26:
		snprintf(buf,BUFFSIZE,"CEngine:: %s Pass: ST_ERR26\n",ps);
		ProtLog(buf); 
		if (cmdline)
		{
//			printf("Error: Wrong parameter count linenumber = '%d' \n",linenum); 
//			snprintf(buf,BUFFSIZE,"'%s'\n",line.GetStringPtr());
//			printf(buf);
			printf("==============================\n");
			printf("Error: Line = '%d' errornum='%d'\n", linenum,result);
			line.Print(); 
			printf("\n");
			Error(result,CE_BOTH);
			}
		else
		{
//			snprintf(buf,BUFFSIZE,"Error: Wrong parameter count linenumber = '%d' \n",linenum); 
//			ProtLog(buf);
//			snprintf(buf,BUFFSIZE,"'%s'\n",line.GetStringPtr());
//			ProtLog(buf);
			line.Log(); 
			Error(result,CE_LOG);
			}
		state=ST_BEGIN;
		break;

	default:
		ProtLog("CEngine:: FirstPass: State machine error default state.\n");
		}

	// close file
	fclose(f);
	if (makelistfile) fclose(lstf);
	ProtLog("CEngine:: FirstPass: Exit.\n");
	return (CE_ERR_NOERR);

	} //////////////////////////////////////////////////////////////////////////////////end Pass

////////////////////////////////////////////////////////////////////////////////////////////////////
// ListBin 
////////////////////////////////////////////////////////////////////////////////////////////////////
int CEngine::ListBin()
{
unsigned long	i;
char		buf[BUFFSIZE];

	ProtLog("================================================================================\n");
	ProtLog("=======================List dump================================================\n");
	ProtLog("================================================================================\n");
	for (i=0; i<MAXSCRIPT; i++)
	{
		snprintf(buf,BUFFSIZE,"CEngine:: Listbin:  %07lu   %04lu\n",i,scripts[i]);
		ProtLog(buf);
		}
	return (CE_ERR_NOERR);
	} ///////////////////////////////////////////////////////////////////////////////end ListBin

////////////////////////////////////////////////////////////////////////////////////////////////////
// DumpSymtab 
////////////////////////////////////////////////////////////////////////////////////////////////////
void CEngine::DumpSymtab()
{
int		i;
char		buf[BUFFSIZE];

	ProtLog("CEngine:: DumpSymtab: Entered.\n"); 
	
	for (i=0; i<symcount; i++)
	{
		if (symtab[i].used) 
		{
			snprintf(buf,BUFFSIZE,"CEngine:: DumpSymtab: Symbol = '%s'  Value = %lu\n",
			symtab[i].symbol.GetStringPtr(),
			symtab[i].value);
			ProtLog(buf); 
			}
		}	

	snprintf(buf,BUFFSIZE,"CEngine:: DumpSymtab: Lines read = '%d'\n", linenum);
	ProtLog(buf); 
	ProtLog("CEngine:: DumpSymtab: Exited.\n"); 
	} ////////////////////////////////////////////////////////////////////////////end DumpSymTab

////////////////////////////////////////////////////////////////////////////////////////////////////
// WriteBin 
// Here we will write the output "binary"
////////////////////////////////////////////////////////////////////////////////////////////////////
int CEngine::WriteBin()
{
char            filename[255];
char            *s;
FILE            *file;
unsigned long 	i;
char		buf[BUFFSIZE];

//	snprintf(filename,200,"logs/%lu.log",threadnum);
	outputfile.Log("Opening '","' for writing. \n");
        file=fopen(outputfile.GetStringPtr(),"w");
        if (file==0)
        {
                // capture errno
                printf("errno=%d \n",errno);
                printf("Couldn't open binary file '%s' for write. \n",outputfile.GetStringPtr());
                exit (7);
                }

	snprintf(buf,BUFFSIZE,"CEngine:: WriteBin: org=%llu\n",org);
	ProtLog(buf);

	for (i=org;i<stmtcntr; i++)
	{
	        fprintf(file,"%09lu %lu \n",i,scripts[i]);
		}
        fclose(file);

	} //////////////////////////////////////////////////////////////////////////////end WriteBin

////////////////////////////////////////////////////////////////////////////////////////////////////
// Assemble 
// This will be a VERY simple two pass assembler.
// When Assembling we WONT be running a script.
// We will pass assemble a parm saying what to do
// 
////////////////////////////////////////////////////////////////////////////////////////////////////
int CEngine::Assemble(int action, CString *f, bool cmdline)
{
int result;

//	if (loglevel>=1) ProtLog("CEngine:: Assemble: Entered.");
	ProtLog("CEngine:: Assemble: Entered.\n");
	switch (action)
	{
	case CE_ASM_OPEN:
		ProtLog("CEngine:: Assemble: Open Entered.\n");
		// this doesn't really open but just set the file name
		if (f==0) return CE_ERR_NOFILE;
		inputfile=*f;
		return (CE_ERR_NOERR);
		break;

	case CE_ASM_OUTPUT:
		ProtLog("CEngine:: Assemble: Output Entered.\n");

		// this doesn't really open but just set the file name
		if (f==0) return CE_ERR_NOFILE;
		outputfile=*f;
		return (CE_ERR_NOERR);
		break;

	case CE_ASM_FIRST:
		
		ProtLog("CEngine:: Assemble: First Pass.\n");
		result=Pass(CE_ASM_FIRST,cmdline);
		DumpSymtab();
		return(result);
		break;

	case CE_ASM_SECOND:
		ProtLog("CEngine:: Assemble: Second Pass.\n"); 
		DumpSymtab();
		return(Pass(CE_ASM_SECOND,cmdline));
		break;

	case CE_ASM_WRITE:
		ProtLog("CEngine:: Assemble: Write binary.\n"); 
		return(WriteBin());
		break;

	case CE_ASM_LIST:
		ProtLog("CEngine:: Assemble: List dump.\n"); 
		return(ListBin());
		break;

	case CE_ASM_LSTFILE:
		ProtLog("CEngine:: Assemble: List file write.\n"); 
		makelistfile=true;

		// this doesn't really open but just set the file name
		if (f==0) return CE_ERR_NOFILE;
		lstfile=*f;
		break;

	default:
		return (CE_ERR_UNHANDLED);
		}
	return (CE_ERR_NOERR);
	} //////////////////////////////////////////////////////////////////////////////end Assemble

////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////
// OPCODES
////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////////////////////////
// OpCode_cnt  
////////////////////////////////////////////////////////////////////////////////////////////////////
void CEngine::OpCode_cnt()
{
	if (loglevel>=1) ProtLog("CEngine:: OpCode_cnt: Entered.\n");
	pc++;
	pc++;

	} ////////////////////////////////////////////////////////////////////////////end OpCode_cnt

////////////////////////////////////////////////////////////////////////////////////////////////////
// OpCode_add  
// cant mix type types
// cant add bools
// if a string exists the greater than flag will be set
// if it has len zero the e flag will be set
////////////////////////////////////////////////////////////////////////////////////////////////////
void CEngine::OpCode_add()
{
char	vt1,vt2,rt;
int 	idx1,idx2,ri;
bool	value;
CString	s;

	// the two types must be the same or compatible
	// get the parms
	// GetParms UPDATES pc
	GetParms(&vt1,&idx1,&vt2,&idx2,&rt,&ri);

	// now do it
	if (vt1=='i' && vt2=='i' && rt=='i') 
	{
		ints[ri]=ints[idx1]+ints[idx2];
		if (ints[ri]==0) { eq=true; lt=false; gt=false; }
		if (ints[ri]>0) { eq=false; lt=false; gt=true; }
		if (ints[ri]<0) { eq=false; lt=true; gt=false; }
		}

	if (vt1=='d' && vt2=='d' && rt=='d') 
	{
		dbls[ri]=dbls[idx1]+dbls[idx2];
		if (dbls[ri]==0) { eq=true; lt=false; gt=false; }
		if (dbls[ri]>0) { eq=false; lt=false; gt=true; }
		if (dbls[ri]<0) { eq=false; lt=true; gt=false; }
		}

	if (vt1=='s' && vt2=='s' && rt=='s') 
	{
		s=strs[idx1];
		s.Append(strs[idx2].GetStringPtr());
		strs[ri]=s;
		if ((s.GetLength())==0) { eq=true; lt=false; gt=false; }
		if ((s.GetLength())>0) { eq=false; lt=false; gt=true; }
		}

	// now point to next inst
	pc++;
	} ////////////////////////////////////////////////////////////////////////////end OpCode_add

////////////////////////////////////////////////////////////////////////////////////////////////////
// OpCode_sub  
// not valid for bools or strings
////////////////////////////////////////////////////////////////////////////////////////////////////
void CEngine::OpCode_sub()
{
char	vt1,vt2,rt;
int 	idx1,idx2,ri;
bool	value;

	// the two types must be the same or compatible
	// get the parms
	// GetParms UPDATES pc
	GetParms(&vt1,&idx1,&vt2,&idx2,&rt,&ri);

	// now do it
	if (vt1=='i' && vt2=='i' && rt=='i') 
	{
		ints[ri]=ints[idx1]-ints[idx2];
		if (ints[ri]==0) { eq=true; lt=false; gt=false; }
		if (ints[ri]>0) { eq=false; lt=false; gt=true; }
		if (ints[ri]<0) { eq=false; lt=true; gt=false; }
		}

	if (vt1=='d' && vt2=='d' && rt=='d') 
	{
		dbls[ri]=dbls[idx1]-dbls[idx2];
		if (dbls[ri]==0) { eq=true; lt=false; gt=false; }
		if (dbls[ri]>0) { eq=false; lt=false; gt=true; }
		if (dbls[ri]<0) { eq=false; lt=true; gt=false; }
		}

	// now point to next inst
	pc++;
	} ////////////////////////////////////////////////////////////////////////////end OpCode_sub

////////////////////////////////////////////////////////////////////////////////////////////////////
// OpCode_mul  
////////////////////////////////////////////////////////////////////////////////////////////////////
void CEngine::OpCode_mul()
{
char	vt1,vt2,rt;
int 	idx1,idx2,ri;
bool	value;

	// the two types must be the same or compatible
	// get the parms
	// GetParms UPDATES pc
	GetParms(&vt1,&idx1,&vt2,&idx2,&rt,&ri);

	// now do it
	if (vt1=='i' && vt2=='i' && rt=='i') 
	{
		ints[ri]=ints[idx1]*ints[idx2];
		if (ints[ri]==0) { eq=true; lt=false; gt=false; }
		if (ints[ri]>0) { eq=false; lt=false; gt=true; }
		if (ints[ri]<0) { eq=false; lt=true; gt=false; }
		}

	if (vt1=='d' && vt2=='d' && rt=='d') 
	{
		dbls[ri]=dbls[idx1]*dbls[idx2];
		if (dbls[ri]==0) { eq=true; lt=false; gt=false; }
		if (dbls[ri]>0) { eq=false; lt=false; gt=true; }
		if (dbls[ri]<0) { eq=false; lt=true; gt=false; }
		}

	// now point to next inst
	pc++;
	} ////////////////////////////////////////////////////////////////////////////end OpCode_mul

////////////////////////////////////////////////////////////////////////////////////////////////////
// OpCode_divd 
// this only valid for both parms float
////////////////////////////////////////////////////////////////////////////////////////////////////
void CEngine::OpCode_divd()
{
char	vt1,vt2,rt;
int 	idx1,idx2,ri;
bool	value;

	// the two types must be the same or compatible
	// get the parms
	// GetParms UPDATES pc
	GetParms(&vt1,&idx1,&vt2,&idx2,&rt,&ri);

	// now do it
	if (vt1=='d' && vt2=='d' && rt=='d') 
	{
		dbls[ri]=dbls[idx1]/dbls[idx2];
		if (dbls[ri]==0) { eq=true; lt=false; gt=false; }
		if (dbls[ri]>0) { eq=false; lt=false; gt=true; }
		if (dbls[ri]<0) { eq=false; lt=true; gt=false; }
		}

	// now point to next inst
	pc++;
	} ///////////////////////////////////////////////////////////////////////////end OpCode_divd

////////////////////////////////////////////////////////////////////////////////////////////////////
// OpCode_divi  
// this only valid for both parms integer
////////////////////////////////////////////////////////////////////////////////////////////////////
void CEngine::OpCode_divi()
{
char	vt1,vt2,rt;
int 	idx1,idx2,ri;
bool	value;

	// the two types must be the same or compatible
	// get the parms
	// GetParms UPDATES pc
	GetParms(&vt1,&idx1,&vt2,&idx2,&rt,&ri);

	// now do it
	if (vt1=='i' && vt2=='i' && rt=='i') 
	{
		ints[ri]=ints[idx1]/ints[idx2];
		if (ints[ri]==0) { eq=true; lt=false; gt=false; }
		if (ints[ri]>0) { eq=false; lt=false; gt=true; }
		if (ints[ri]<0) { eq=false; lt=true; gt=false; }
		}

	// now point to next inst
	pc++;
	} ///////////////////////////////////////////////////////////////////////////end OpCode_divi

////////////////////////////////////////////////////////////////////////////////////////////////////
// OpCode_mod  
////////////////////////////////////////////////////////////////////////////////////////////////////
void CEngine::OpCode_mod()
{
	} ////////////////////////////////////////////////////////////////////////////end OpCode_mod

////////////////////////////////////////////////////////////////////////////////////////////////////
// OpCode_exp  
////////////////////////////////////////////////////////////////////////////////////////////////////
void CEngine::OpCode_exp()
{
	} ////////////////////////////////////////////////////////////////////////////end OpCode_exp

////////////////////////////////////////////////////////////////////////////////////////////////////
// OpCode_neg 
////////////////////////////////////////////////////////////////////////////////////////////////////
void CEngine::OpCode_neg()
{
char	vt1;
int 	idx1;
bool	value;

	// the two types must be the same or compatible
	// get the parms
	// GetParms UPDATES pc
	GetParms(&vt1,&idx1);

	// now do it
	if (vt1=='i') 
	{
		ints[idx1]=-ints[idx1];
		if (ints[idx1]==0) { eq=true; lt=false; gt=false; }
		if (ints[idx1]>0) { eq=false; lt=false; gt=true; }
		if (ints[idx1]<0) { eq=false; lt=true; gt=false; }
		}

	if (vt1=='d') 
	{
		dbls[idx1]=-dbls[idx1];
		if (dbls[idx1]==0) { eq=true; lt=false; gt=false; }
		if (dbls[idx1]>0) { eq=false; lt=false; gt=true; }
		if (dbls[idx1]<0) { eq=false; lt=true; gt=false; }
		}

	// now point to next inst
	pc++;
	} ////////////////////////////////////////////////////////////////////////////end OpCode_neg

////////////////////////////////////////////////////////////////////////////////////////////////////
// OpCode_push 
// Push decrements then stores
// if SP<=1 then this op will fail and cause the script to end.
////////////////////////////////////////////////////////////////////////////////////////////////////
void CEngine::OpCode_push()
{
char	vt;	// vartype
int 	idx;	// index
bool	value;

	// get the parms
	if (sp<=1) 
	{
		engerr=CE_ERR_SPUF;
		return;
		}

	// GetParms UPDATES pc
	GetParms(&vt,&idx);

	if (vt!='i') 
	{
		engerr=CE_ERR_BADREG;
		return;
		}

	// now do it
	sp=sp-1;
	
//	if (vt=='i') scripts[sp]=ints[idx];
	scripts[sp]=ints[idx];

	// now point to next inst
	pc++;
	} ///////////////////////////////////////////////////////////////////////////end OpCode_push

////////////////////////////////////////////////////////////////////////////////////////////////////
// OpCode_pop 
////////////////////////////////////////////////////////////////////////////////////////////////////
void CEngine::OpCode_pop()
{
char	vt;
int 	idx;
bool	value;

	// the two types must be the same or compatible
	// get the parms
	if (sp>=MAXSCRIPT-1) engerr=CE_ERR_SPOF;

	// GetParms UPDATES pc
	GetParms(&vt,&idx);

	// now do it
	if (vt=='i') ints[idx]=scripts[sp];
	sp=sp+1;

	// now point to next inst
	pc++;
	} ////////////////////////////////////////////////////////////////////////////end OpCode_pop

////////////////////////////////////////////////////////////////////////////////////////////////////
// OpCode_jmp 
////////////////////////////////////////////////////////////////////////////////////////////////////
void CEngine::OpCode_jmp()
{
	pc++;
	if (scripts[pc]<MAXSCRIPT) pc=scripts[pc];
	} ////////////////////////////////////////////////////////////////////////////end OpCode_jmp

////////////////////////////////////////////////////////////////////////////////////////////////////
// OpCode_jeq 
////////////////////////////////////////////////////////////////////////////////////////////////////
void CEngine::OpCode_jeq()
{
	pc++;
	if ((eq)&&(scripts[pc]<MAXSCRIPT)) pc=scripts[pc];
	else pc++;
	} ////////////////////////////////////////////////////////////////////////////end OpCode_jeq

////////////////////////////////////////////////////////////////////////////////////////////////////
// OpCode_jlt 
////////////////////////////////////////////////////////////////////////////////////////////////////
void CEngine::OpCode_jlt()
{
	pc++;
	if ((lt)&&(scripts[pc]<MAXSCRIPT)) pc=scripts[pc];
	else pc++;
	} ////////////////////////////////////////////////////////////////////////////end OpCode_jlt

////////////////////////////////////////////////////////////////////////////////////////////////////
// OpCode_jgt 
////////////////////////////////////////////////////////////////////////////////////////////////////
void CEngine::OpCode_jgt()
{
	pc++;
	if ((gt)&&(scripts[pc]<MAXSCRIPT)) pc=scripts[pc];
	else pc++;
	} ////////////////////////////////////////////////////////////////////////////end OpCode_jgt

////////////////////////////////////////////////////////////////////////////////////////////////////
// OpCode_jv 
// zero = false
// not zero = true
// works on all types
// strings the first char would be '\0' for false
////////////////////////////////////////////////////////////////////////////////////////////////////
void CEngine::OpCode_jv()
{
char	vartype;
int 	idx;
bool	value;

	// get the var type to test
	pc++;
	vartype=(char)scripts[pc];

	// get the idx to test, and keep it in range
	pc++;
	idx=(int)(scripts[pc]);
	if (idx>=10) idx=9;
	if (idx<0) idx=0;
	
	switch (vartype)
	{
	case 'b':
		if (bools[idx]==0) value=false; else value=true;
//		value=(bool)bools[idx];
		break;

	case 'd':
		if (dbls[idx]==0.0) value=false; else value=true;
		break;

	case 's':
		value=strs[idx].GetChar(0);
		break;

	case 'i':	// if the script is mal formed treat var type as an int
	default:
		value=(bool)ints[idx];
		break;
		}

	// now do the jump
	pc++;
	if (value) pc=scripts[pc]; else pc++;
	} /////////////////////////////////////////////////////////////////////////////end OpCode_jv

////////////////////////////////////////////////////////////////////////////////////////////////////
// OpCode_jnz 
////////////////////////////////////////////////////////////////////////////////////////////////////
void CEngine::OpCode_jnz()
{
	pc++;
	if ((!eq)&&(scripts[pc]<MAXSCRIPT)) pc=scripts[pc];
	else pc++;
	} ////////////////////////////////////////////////////////////////////////////end OpCode_jnz

////////////////////////////////////////////////////////////////////////////////////////////////////
// OpCode_call 
////////////////////////////////////////////////////////////////////////////////////////////////////
void CEngine::OpCode_call()
{
	// check for stack pointer underflow
	if (sp<=1) 
	{
		engerr=CE_ERR_SPUF;
		return;
		}

	// do the push of return address 
	sp=sp-1;
	scripts[sp]=pc+2;

	// do the jump to call address
	if (scripts[pc+1]<MAXSCRIPT) pc=scripts[pc+1];

	} ///////////////////////////////////////////////////////////////////////////end OpCode_call

////////////////////////////////////////////////////////////////////////////////////////////////////
// OpCode_ceq 
////////////////////////////////////////////////////////////////////////////////////////////////////
void CEngine::OpCode_ceq()
{
	} ////////////////////////////////////////////////////////////////////////////end OpCode_ceq

////////////////////////////////////////////////////////////////////////////////////////////////////
// OpCode_clt 
////////////////////////////////////////////////////////////////////////////////////////////////////
void CEngine::OpCode_clt()
{
	} ////////////////////////////////////////////////////////////////////////////end OpCode_clt

////////////////////////////////////////////////////////////////////////////////////////////////////
// OpCode_cgt 
////////////////////////////////////////////////////////////////////////////////////////////////////
void CEngine::OpCode_cgt()
{
	} ////////////////////////////////////////////////////////////////////////////end OpCode_cgt

////////////////////////////////////////////////////////////////////////////////////////////////////
// OpCode_cv 
////////////////////////////////////////////////////////////////////////////////////////////////////
void CEngine::OpCode_cv()
{
	} /////////////////////////////////////////////////////////////////////////////end OpCode_cv

////////////////////////////////////////////////////////////////////////////////////////////////////
// OpCode_cnz 
////////////////////////////////////////////////////////////////////////////////////////////////////
void CEngine::OpCode_cnz()
{
	} ////////////////////////////////////////////////////////////////////////////end OpCode_cnz

////////////////////////////////////////////////////////////////////////////////////////////////////
// OpCode_ret
////////////////////////////////////////////////////////////////////////////////////////////////////
void CEngine::OpCode_ret()
{
	// check for stack pointer overflow
	if (sp>=MAXSCRIPT-1) 
	{
		engerr=CE_ERR_SPOF;
		return;
		}

	// do the pop of return address 
	pc=scripts[sp];
	sp=sp+1;

	} ////////////////////////////////////////////////////////////////////////////end OpCode_ret

////////////////////////////////////////////////////////////////////////////////////////////////////
// OpCode_req 
////////////////////////////////////////////////////////////////////////////////////////////////////
void CEngine::OpCode_req()
{
	} ////////////////////////////////////////////////////////////////////////////end OpCode_req

////////////////////////////////////////////////////////////////////////////////////////////////////
// OpCode_rlt 
////////////////////////////////////////////////////////////////////////////////////////////////////
void CEngine::OpCode_rlt()
{
	} ////////////////////////////////////////////////////////////////////////////end OpCode_rlt

////////////////////////////////////////////////////////////////////////////////////////////////////
// OpCode_rgt 
////////////////////////////////////////////////////////////////////////////////////////////////////
void CEngine::OpCode_rgt()
{
	} ////////////////////////////////////////////////////////////////////////////end OpCode_rgt

////////////////////////////////////////////////////////////////////////////////////////////////////
// OpCode_rv 
////////////////////////////////////////////////////////////////////////////////////////////////////
void CEngine::OpCode_rv()
{
	} /////////////////////////////////////////////////////////////////////////////end OpCode_rv

////////////////////////////////////////////////////////////////////////////////////////////////////
// OpCode_rnz 
////////////////////////////////////////////////////////////////////////////////////////////////////
void CEngine::OpCode_rnz()
{
	} ////////////////////////////////////////////////////////////////////////////end OpCode_rnz

////////////////////////////////////////////////////////////////////////////////////////////////////
// OpCode_cmp 
// cmp strings restype=int (compare is based on length of string one)
// 	string1> string2 = +1
// 	string1< string2 = -1
// 	string1==string2 = 0
// cmp int : restype = int
//	res=i1-i2
// cmp bool : restype = bool
//	res=(b1==b2)
// cmp dbls : restype = dbl
// 	res=dbl1-dbl2
// if types dont match
// restype = bool , res=false
////////////////////////////////////////////////////////////////////////////////////////////////////
void CEngine::OpCode_cmp()
{
char	vt1,vt2;
int 	idx1,idx2;
int	value;

	// GetParms UPDATES pc
	GetParms(&vt1,&idx1,&vt2,&idx2);
	
	// first the variable type must be the same or answer is false
	if (vt1!=vt2)
	{
		engerr=CE_ERR_TYPE;
		// point to next inst
		pc++;
		return;
		}

	// now do it
	switch (vt1)
	{
	case 'b':
		eq=false; lt=false; gt=false;
		if (bools[idx1]==bools[idx2])  eq=true; 
		else if (bools[idx1]<bools[idx2])  lt=true; 
		else gt=true; 
		break;

	case 'd':
		eq=false; lt=false; gt=false;
		if (dbls[idx1]==dbls[idx2])  eq=true; 
		else if (dbls[idx1]<dbls[idx2])  lt=true; 
		else gt=true; 
		break;

	case 's':
		{
			eq=false; lt=false; gt=false;
			if (strs[idx1] > strs[idx2]) gt=true; 
			if (strs[idx1] < strs[idx2]) lt=true; 
			if (strs[idx1] == strs[idx2]) eq=true; 
			break;
			}

	case 'u':	
		eq=false; lt=false; gt=false;
		if (uints[idx1]==uints[idx2])  eq=true; 
		else if (uints[idx1]<uints[idx2])  lt=true; 
		else gt=true; 
		break;

	case 'i':
		eq=false; lt=false; gt=false;
		if (ints[idx1]==ints[idx2])  eq=true; 
		else if (ints[idx1]<ints[idx2])  lt=true; 
		else gt=true; 
		break;

	default:
		engerr=CE_ERR_INVTYPE;
		break;
		}

	// now point to next inst
	pc++;

	} ////////////////////////////////////////////////////////////////////////////end OpCode_cmp

////////////////////////////////////////////////////////////////////////////////////////////////////
// OpCode_cpi 
////////////////////////////////////////////////////////////////////////////////////////////////////
void CEngine::OpCode_cpi()
{
	} ////////////////////////////////////////////////////////////////////////////end OpCode_cpi

////////////////////////////////////////////////////////////////////////////////////////////////////
// OpCode_ldti 
// Load register from system time
// This uses TWO registers
// The specified i[] register and the next higher
// Thus valid values to specify are 
// idx=0 to idx=8
////////////////////////////////////////////////////////////////////////////////////////////////////
void CEngine::OpCode_ldti()
{
char		dst;
int		idx;
struct timeb    t;

	// 
	if (loglevel>=1) ProtLog("CEngine:: OpCode_ld: Entered. \n ");

	////// get parms
	pc++;
	dst=(char)scripts[pc];		// get which type of var to store data in

	pc++;				// get index
	idx=(int)(scripts[pc]);
	
	if ((idx<0) || (idx>8))			
	{
		engerr=CE_ERR_BADREG;
		return;
		}

	if (dst!='i') 
	{
		engerr=CE_ERR_BADREG;
		return;
		}
	
	// doit
        ftime(&t);
	ints[idx]=(unsigned long)t.time;
        ints[idx+1]=(unsigned long)t.millitm;

	pc++;				// final pc++


	} ///////////////////////////////////////////////////////////////////////////end OpCode_ldti

////////////////////////////////////////////////////////////////////////////////////////////////////
// OpCode_ldsp 
// Load register from sp
////////////////////////////////////////////////////////////////////////////////////////////////////
void CEngine::OpCode_ldsp()
{
char	dst;
int	idx;

	// 
	if (loglevel>=1) ProtLog("CEngine:: OpCode_ld: Entered. \n ");

	////// get parms
	pc++;
	dst=(char)scripts[pc];		// get which type of var to store data in

	pc++;				// get index
	idx=(int)(scripts[pc]);
	
	if ((idx<0) || (idx>9))			
	{
		engerr=CE_ERR_BADREG;
		return;
		}

	if (dst!='i') 
	{
		engerr=CE_ERR_BADREG;
		return;
		}
	
	// doit
	ints[idx]=sp;
	pc++;				// get index


	} ///////////////////////////////////////////////////////////////////////////end OpCode_ldsp

////////////////////////////////////////////////////////////////////////////////////////////////////
// OpCode_ld 
// Load from register 
////////////////////////////////////////////////////////////////////////////////////////////////////
void CEngine::OpCode_ld()
{
char	vt1,vt2;
int 	idx1,idx2;
unsigned long int offset,len;
char buf[BUFFSIZE];
char c;
	
	
	////// get parms
	if (loglevel>=1) ProtLog("CEngine:: OpCode_ld: Entered. \n ");
	GetParms(&vt1, &idx1, &vt2, &idx2);
			
	// test register move combinations
	if (loglevel>=1) ProtLog("CEngine:: OpCode_ld: After GetParms. \n ");
	if ((vt1=='b')  && (vt2=='b')) bools[idx2] =bools[idx1]; 
	if ((vt1=='i')  && (vt2=='i')) ints[idx2]  =ints[idx1]; 
	if ((vt1=='u')  && (vt2=='u')) uints[idx2] =uints[idx1]; 
	if ((vt1=='d')  && (vt2=='d')) dbls[idx2]  =dbls[idx1];  
	if ((vt1=='s')  && (vt2=='s')) strs[idx2]  =strs[idx1];  
	
	// implicit conversions to dbls
	if (loglevel>=1) ProtLog("CEngine:: OpCode_ld: After basic lds. \n ");
	if ((vt1=='i')  && (vt2=='d')) dbls[idx2]  =ints[idx1];  
	if ((vt1=='u')  && (vt2=='d')) dbls[idx2]  =uints[idx1]; 
//	if ((vt1=='s')  && (vt2=='d')) strs[idx2]  =bools[idx1]; 

	// implicit conversions to strs
//	if ((vt1=='b')  && (vt2=='s')) strs[idx2]  =bools[idx1];  
//	if ((vt1=='i')  && (vt2=='s')) strs[idx2]  =ints[idx1];  
//	if ((vt1=='u')  && (vt2=='s')) strs[idx2]  =uints[idx1];  
//	if ((vt1=='d')  && (vt2=='s')) strs[idx2]  =dbls[idx1];  
	
	// implicit conversions to ints
	if (loglevel>=1) ProtLog("CEngine:: OpCode_ld: After Dbls conversion. \n ");
	if ((vt1=='i')  && (vt2=='u')) uints[idx2] =ints[idx1];  
	if ((vt1=='u')  && (vt2=='i')) ints[idx2]  =uints[idx1];  
//	if ((vt1=='s')  && (vt2=='i')) ints[idx2]  =strs[idx1];  

	// implicit conversions to uints
//	if ((vt1=='s')  && (vt2=='u')) strs[idx2]  =bools[idx1];  


	// point to next inst
	if (loglevel>=1) ProtLog("CEngine:: OpCode_ld: Exiting. \n ");
	pc++;
	} /////////////////////////////////////////////////////////////////////////////end OpCode_ld

////////////////////////////////////////////////////////////////////////////////////////////////////
// OpCode_ldm 
// Load from memory offset
////////////////////////////////////////////////////////////////////////////////////////////////////
void CEngine::OpCode_ldm()
{
char	vt;
int	idx,i;
unsigned long int addr,len;
char buf[BUFFSIZE];
char c;
	
	if (loglevel>=1) ProtLog("CEngine:: OpCode_ldm: Entered. \n ");

	////// get parms
	GetParms(&vt, &idx, &len, &addr);

	switch (vt)
	{
	case 'b':
		// if the mem location is >0 then set the bool[idx] to 1
		bools[idx]=(scripts[addr]>0);
		pc++;				// Point to next opcode
		break;

	case 'i':
		ints[idx]=scripts[addr];
		pc++;				// Point to next opcode
		break;

	case 'd':
		dbls[idx]=scripts[addr];
		pc++;				// Point to next opcode
		break;

	case 's':
		for (i=addr;i<addr+len;i++) 
		{

			c=(char)scripts[i];
			if (!strs[idx].Append(c)) 
			{
				engerr=CE_ERR_APND;
				return;
				}
			snprintf(buf, BUFFSIZE,"CEngine:: OpCode_ASSIGN: Apending %c",c);
			if (loglevel>=2) ProtLog(buf);
			}
		pc++;				// Point to next opcode
		break;

	default:
		break;
		}

	} ////////////////////////////////////////////////////////////////////////////end OpCode_ldm

////////////////////////////////////////////////////////////////////////////////////////////////////
// OpCode_ldr 
// Load from register pointer
////////////////////////////////////////////////////////////////////////////////////////////////////
void CEngine::OpCode_ldr()
{
char	vt1;
int 	idx1,idx2,idx3;
unsigned long int addr,len;
char	buf[BUFFSIZE];
char	c;
int	i;
	
	// get parms
	if (loglevel>=1) ProtLog("CEngine:: OpCode_ldr: Entered. \n ");
	GetParms(&vt1, &idx1, &idx2, &idx3);
	
	// check that uints[idx2] is in range
	if (uints[idx2]>MAXSCRIPT)  uints[idx2]=uints[idx2] % MAXSCRIPT; 
	
	// Do it
	// remember id3 is only used for strings
	switch (vt1)
	{
	case 'b':	
		bools[idx1]=scripts[uints[idx2]];
		pc++;				// Point to next opcode
		break;

	case 'i':
		ints[idx1]=scripts[uints[idx2]];
		pc++;				// Point to next opcode
		break;

	case 'd':
		pc++;				// Point to next opcode
		break;

	case 'u':
		uints[idx1]=scripts[uints[idx2]];
		pc++;				// Point to next opcode
		break;

	case 's':
		for (i=uints[idx2];i<uints[idx2]+idx3;i++) strs[idx1].Append((char)scripts[i]);
		break;

	default:
		break;
		}
			
	} ////////////////////////////////////////////////////////////////////////////end OpCode_ldr

////////////////////////////////////////////////////////////////////////////////////////////////////
// OpCode_str 
// Store to register pointer
////////////////////////////////////////////////////////////////////////////////////////////////////
void CEngine::OpCode_str()
{
char	vt1;
int 	idx1,idx2,idx3;
unsigned long int addr,len;
char	buf[BUFFSIZE];
char	c;
int	i;
	
	// get parms
	if (loglevel>=1) ProtLog("CEngine:: OpCode_str: Entered. \n ");
	GetParms(&vt1, &idx1, &idx2);
	
	// check that uints[idx2] is in range
	if (uints[idx2]>MAXSCRIPT)  uints[idx2]=uints[idx2] % MAXSCRIPT; 
	
	// Do it
	// remember id3 is only used for strings
	switch (vt1)
	{
	case 'b':	
		scripts[uints[idx2]]=bools[idx1];
		pc++;				// Point to next opcode
		break;

	case 'i':
		scripts[uints[idx2]]=ints[idx1];
		pc++;				// Point to next opcode
		break;

	case 'd':
		pc++;				// Point to next opcode
		break;

	case 'u':
		scripts[uints[idx2]]=uints[idx1];
		pc++;				// Point to next opcode
		break;

	case 's':
		for (i=0;i<strs[idx1].Length();i++) scripts[i]=(unsigned long)strs[idx1].GetChar(i);
		break;

	default:
		break;
		}

	} ////////////////////////////////////////////////////////////////////////////end OpCode_str

////////////////////////////////////////////////////////////////////////////////////////////////////
// OpCode_ldi 
// Load register with immediate data
////////////////////////////////////////////////////////////////////////////////////////////////////
void CEngine::OpCode_ldi()
{
char	dst;
int	idx,i;
unsigned long int offset,len;

	if (loglevel>=1) ProtLog("CEngine:: OpCode_ldi: Entered.\n");

	////// get parms
	pc++;
	dst=(char)scripts[pc];		// get which type of var to store data in

	pc++;				// get index
	idx=(int)(scripts[pc]);
			
	pc++;				// Get Length of data
	len=scripts[pc];

	pc++;				// Point to data
	

	switch (dst)
	{
	case 'b':
		bools[idx]=(scripts[pc]>0);
		pc=pc+len;			// Point to next opcode
//		pc++;				// Point to next opcode
		break;

	case 'i':
		ints[idx]=scripts[pc];
		pc=pc+len;			// Point to next opcode
		break;

	case 'u':
		uints[idx]=scripts[pc];
		pc=pc+len;			// Point to next opcode
		break;

	case 'd':
		dbls[idx]=scripts[pc];
		pc=pc+len;			// Point to next opcode
//		pc++;				// Point to next opcode
		break;

	case 's':
		strs[idx]="";
		for (i=pc;i<pc+len;i++) strs[idx].Append((char)scripts[i]);
		pc=pc+len;			// Point to next opcode
		break;

	default:
		break;
		}
	if (loglevel>=1) ProtLog("CEngine:: OpCode_ldi: Exited.\n");

	} ////////////////////////////////////////////////////////////////////////////end OpCode_ldi

////////////////////////////////////////////////////////////////////////////////////////////////////
// OpCode_and 
////////////////////////////////////////////////////////////////////////////////////////////////////
void CEngine::OpCode_and()
{
char	vt1,vt2,rt;
int 	idx1,idx2,ri;
bool	value;
CString	s;

	if (loglevel>=1) ProtLog("CEngine:: OpCode_and: Entered. \n ");
	// the two types must be the same or compatible
	// get the parms
	// GetParms UPDATES pc
	GetParms(&vt1,&idx1,&vt2,&idx2,&rt,&ri);
	
	// Do it
	if (vt1=='b' && vt2=='b' && rt=='b') 
	{
		bools[ri]=bools[idx1] & bools[idx2];
		if (bools[ri]==0) { eq=true; lt=false; gt=false; }
		if (bools[ri]>0) { eq=false; lt=false; gt=true; }
		if (bools[ri]<0) { eq=false; lt=true; gt=false; }
		}

	if (vt1=='i' && vt2=='i' && rt=='i') 
	{
		ints[ri]=ints[idx1] & ints[idx2];
		if (ints[ri]==0) { eq=true; lt=false; gt=false; }
		if (ints[ri]>0) { eq=false; lt=false; gt=true; }
		if (ints[ri]<0) { eq=false; lt=true; gt=false; }
		}

	if (vt1=='u' && vt2=='u' && rt=='i') 
	{
		uints[ri]=uints[idx1] & uints[idx2];
		if (uints[ri]==0) { eq=true; lt=false; gt=false; }
		if (uints[ri]>0) { eq=false; lt=false; gt=true; }
		if (uints[ri]<0) { eq=false; lt=true; gt=false; }
		}

//	switch (vt1)
//	{
//	case 'b':	
//		scripts[uints[idx2]]=bools[idx1];
//		pc++;				// Point to next opcode
//		break;
//
//	case 'i':
//		scripts[uints[idx2]]=ints[idx1];
//		pc++;				// Point to next opcode
//		break;
//
//	case 'd':
//		pc++;				// Point to next opcode
//		break;
//
//	case 'u':
//		scripts[uints[idx2]]=uints[idx1];
//		pc++;				// Point to next opcode
//		break;
//
//	case 's':
//		for (i=0;i<strs[idx1].Length();i++) scripts[i]=(unsigned long)strs[idx1].GetChar(i);
//		break;
//
//	default:
//		break;
//		}

	// now point to next inst
	pc++;

	} ////////////////////////////////////////////////////////////////////////////end OpCode_and

////////////////////////////////////////////////////////////////////////////////////////////////////
// OpCode_or 
////////////////////////////////////////////////////////////////////////////////////////////////////
void CEngine::OpCode_or()
{
	} /////////////////////////////////////////////////////////////////////////////end OpCode_or

////////////////////////////////////////////////////////////////////////////////////////////////////
// OpCode_xor 
////////////////////////////////////////////////////////////////////////////////////////////////////
void CEngine::OpCode_xor()
{
	} ////////////////////////////////////////////////////////////////////////////end OpCode_xor

////////////////////////////////////////////////////////////////////////////////////////////////////
// OpCode_not 
////////////////////////////////////////////////////////////////////////////////////////////////////
void CEngine::OpCode_not()
{
	} ////////////////////////////////////////////////////////////////////////////end OpCode_not

////////////////////////////////////////////////////////////////////////////////////////////////////
// OpCode_cnv 
////////////////////////////////////////////////////////////////////////////////////////////////////
void CEngine::OpCode_cnv()
{
	} ////////////////////////////////////////////////////////////////////////////end OpCode_cnv

////////////////////////////////////////////////////////////////////////////////////////////////////
// OpCode_inc 
////////////////////////////////////////////////////////////////////////////////////////////////////
void CEngine::OpCode_inc()
{
char	vt1;
int 	idx1;
bool	value;

	// get the parms
	// GetParms UPDATES pc
	GetParms(&vt1,&idx1);

	// now do it
	if (vt1=='i') 
	{
		ints[idx1]++;
		if (ints[idx1]==0) { eq=true; lt=false; gt=false; }
		if (ints[idx1]>0) { eq=false; lt=false; gt=true; }
		if (ints[idx1]<0) { eq=false; lt=true; gt=false; }
		}

	if (vt1=='d') 
	{
		dbls[idx1]=dbls[idx1]+1.0;
		if (dbls[idx1]==0) { eq=true; lt=false; gt=false; }
		if (dbls[idx1]>0) { eq=false; lt=false; gt=true; }
		if (dbls[idx1]<0) { eq=false; lt=true; gt=false; }
		}

	// now point to next inst
	pc++;
	} ////////////////////////////////////////////////////////////////////////////end OpCode_inc

////////////////////////////////////////////////////////////////////////////////////////////////////
// OpCode_dec 
////////////////////////////////////////////////////////////////////////////////////////////////////
void CEngine::OpCode_dec()
{
char	vt1;
int 	idx1;
bool	value;

	// get the parms
	// GetParms UPDATES pc
	GetParms(&vt1,&idx1);

	// now do it
	if (vt1=='i') 
	{
		ints[idx1]--;
		if (ints[idx1]==0) { eq=true; lt=false; gt=false; }
		if (ints[idx1]>0) { eq=false; lt=false; gt=true; }
		if (ints[idx1]<0) { eq=false; lt=true; gt=false; }
		}

	if (vt1=='d') 
	{
		dbls[idx1]=dbls[idx1]-1.0;
		if (dbls[idx1]==0.0) { eq=true; lt=false; gt=false; }
		if (dbls[idx1]>0.0) { eq=false; lt=false; gt=true; }
		if (dbls[idx1]<0.0) { eq=false; lt=true; gt=false; }
		}

	// now point to next inst
	pc++;
	} ////////////////////////////////////////////////////////////////////////////end OpCode_dec

////////////////////////////////////////////////////////////////////////////////////////////////////
// OpCode_incm 
////////////////////////////////////////////////////////////////////////////////////////////////////
void CEngine::OpCode_incm()
{
	} ///////////////////////////////////////////////////////////////////////////end OpCode_incm

////////////////////////////////////////////////////////////////////////////////////////////////////
// OpCode_decm 
////////////////////////////////////////////////////////////////////////////////////////////////////
void CEngine::OpCode_decm()
{
	} ///////////////////////////////////////////////////////////////////////////end OpCode_decm

////////////////////////////////////////////////////////////////////////////////////////////////////
// OpCode_stm 
// store register in memory
////////////////////////////////////////////////////////////////////////////////////////////////////
void CEngine::OpCode_stm()
{
char		vt1;
int 		idx1;
unsigned long	offset;

	// get the parms
	// GetParms UPDATES pc
	GetParms(&vt1,&idx1);

	pc++;
	offset=scripts[pc];
	
	// now do it
	if (vt1=='b') { scripts[offset]=bools[idx1]; pc++;}

	// now do it
	if (vt1=='i') { scripts[offset]=ints[idx1]; pc++;}

	// now do it
	if (vt1=='u') { scripts[offset]=uints[idx1]; pc++;}

	} ////////////////////////////////////////////////////////////////////////////end OpCode_stm

////////////////////////////////////////////////////////////////////////////////////////////////////
// OpCode_hlt 
// This is implemented in the Run procedure
////////////////////////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////////////////////////
// Error
// Prints out error text given an error code
////////////////////////////////////////////////////////////////////////////////////////////////////
void CEngine::Error(int e, int which)
{
CString Log;

	switch (e)
	{
	case CE_ERR_NOERR:	Log="No error reported.\n"; break;
	case CE_ERR_BADOPC:	Log="Bad opcode Mnemonic found.\n"; break;
	case CE_ERR_PARM:	Log="Wrong parameter count.\n"; break;
	case CE_ERR_BADREG:	Log="Bad register number.\n"; break;
	case CE_ERR_LENGTH: 	Log="Bad immediate parameter length.\n"; break;

	default:
		Log="Unhandled error code\n";
		break;
		}

	if (which==CE_PRINT)  Log.Print();
	if (which==CE_LOG)  Log.Log();
	if (which==CE_BOTH)  
	{
		Log.Print();
		Log.Log();
		}
	} /////////////////////////////////////////////////////////////////////////////////end Error

