///////////////////////////////////////////////////////////////////////////////////////////////////
// Client
// cfont.cpp : Defines the entry point for the application.
// This is the first real attempt at the worldii client-v3-dx8
// to go with the worldii server-v3
//
// by Eric J Krejci
// Copyright (C) 1999 Eric J Krejci
// Revised : 30 Aug 2006
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation version 2.
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
// Contact:
//              email           worldii@viagloria.com
//              web             www.viagloria.com
///////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////
//  This draws from example code by Jeff Molofee '99
//  (ported to Linux/SDL by Ti Leggett '01)
//
//  Jeff Molofee: Yes i did find it useful :) Thanks !
//
//  Visit Jeff at http://nehe.gamedev.net/
//
//  or for port-specific comments, questions, bugreports etc.
//  email to leggett@eecs.tulane.edu
////////////////////////////////////
// This also draws on example code by Sven Olsen 2003
// Also from the NeHe Site
// Thanks !!!
////////////////////////////////////////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////////////////////////////////////
// Includes
///////////////////////////////////////////////////////////////////////////////////////////////////
#include <stdio.h>
#include "cfont.hpp"
#include "SDL_image.h"

///////////////////////////////////////////////////////////////////////////////////////////////////
// Externals
///////////////////////////////////////////////////////////////////////////////////////////////////
extern void CheckForGLErrors(void (*logfunc)(char*));

///////////////////////////////////////////////////////////////////////////////////////////////////
// Constructor
///////////////////////////////////////////////////////////////////////////////////////////////////
CFont::CFont()
{
	fontname.SetString(" ");
	base = 0;
	list_base = 0;
	fonttype=CF_TYPE_INIT;
	} //////////////////////////////////////////////////////////////////////////end constructor

///////////////////////////////////////////////////////////////////////////////////////////////////
// Name: ~CFont()
// Desc: Font class destructor
///////////////////////////////////////////////////////////////////////////////////////////////////
CFont::~CFont()
{
	if (fonttype==CF_TYPE_INIT) return;

	if (fonttype==CF_TYPE_OLD)
	{
		glDeleteLists( base, 96 );
		}
///////////////////////////////
	if (fonttype==CF_TYPE_TT)
	{
		glDeleteLists(list_base,128);
		glDeleteTextures(128,textures);
		delete [] textures;
		}

	} ////////////////////////////////////////////////////////////////////////////end destructor

////////////////////////////////////////////////////////////////////////////////////////////////////
// SetLogFunc
////////////////////////////////////////////////////////////////////////////////////////////////////
void CFont::SetLogFunc(void (*pfunc)(char*))
{
	LogFunc=pfunc;
	} ////////////////////////////////////////////////////////////////////////////end SetLogFunc

////////////////////////////////////////////////////////////////////////////////////////////////////
// ProtLog
////////////////////////////////////////////////////////////////////////////////////////////////////
void CFont::ProtLog(char *s)
{
	if (LogFunc) (*LogFunc)(s);
	} ////////////////////////////////////////////////////////////////////////////////end ProtLog

////////////////////////////////////////////////////////////////////////////////////////////////////
// DrawText
// Draws 2D text
////////////////////////////////////////////////////////////////////////////////////////////////////
bool CFont::DrawTTText(GLint x, GLint y, GLdouble width, GLdouble height, CString *text)
{
	// If there's no text, do nothing
	if ( text->GetLength()<1 ) return(false);
	if ( height==0 ) height=1;
	return(DrawTTTextInt(x,y,width,height,(text->GetStringPtr())));
	}

////////////////////////////////////////////////////////////////////////////////////////////////////
// DrawTextInt
// Draws 2D text
////////////////////////////////////////////////////////////////////////////////////////////////////
bool CFont::DrawTTText(GLint x, GLint y, GLdouble width, GLdouble height, char *text)
{
	if ( height==0 ) height=1;
	return(DrawTTTextInt(x,y,width,height,text));
	}

////////////////////////////////////////////////////////////////////////////////////////////////////
// DrawTTText
// Draws 2D True Type text
////////////////////////////////////////////////////////////////////////////////////////////////////
bool CFont::DrawTTTextInt(GLint x, GLint y, GLdouble width, GLdouble height, char *text)
{
GLuint	font;
float	modelview_matrix[16];
float	fh;			//We make the height about 1.5* that of
int	i;
GLfloat	ratio;

	// We want a coordinate system where things coresponding to window pixels.
	font=list_base;
	fh=((float)h)/0.63f;			//We make the height about 1.5* that of

        // Preserve matrix
        glPushMatrix();

        // setup for 2D rendering for the text
        glMatrixMode(GL_PROJECTION);
        glLoadIdentity();
        gluOrtho2D(0.0, (GLdouble)width, 0.0, (GLdouble)height);
        glMatrixMode(GL_MODELVIEW);
        glLoadIdentity();
        glShadeModel( GL_FLAT );

        // Position The Text On The Screen
	glTranslatef(x,y,0);

        glPushAttrib(GL_LIST_BIT);                       // Pushes the Display List Bits
        glBindTexture( GL_TEXTURE_2D, texture );
	glListBase(list_base);

        glCallLists(strlen(text),GL_UNSIGNED_BYTE,text);

        glPopAttrib();                                   // Pops the Display List Bits

        // Restore saved matrix
        glPopMatrix();

        // reset for 3D rendering
        glMatrixMode( GL_PROJECTION );
        glLoadIdentity( );
        ratio = ( GLfloat )width / ( GLfloat )height;
        gluPerspective( 45.0f, ratio, 0.1f, 100.0f );
        glMatrixMode( GL_MODELVIEW );
        glShadeModel( GL_SMOOTH );
        glLoadIdentity( );

	} ////////////////////////////////////////////////////////////////////////////end DrawTTText

////////////////////////////////////////////////////////////////////////////////////////////////////
// DrawTextInt
// Draws 2D text
////////////////////////////////////////////////////////////////////////////////////////////////////
bool CFont::DrawTextInt(GLint x, GLint y, GLdouble width, GLdouble height, char *text)
{
GLfloat	ratio;

	// Preserve matrix
	glPushMatrix();

	// setup for 2D rendering for the text
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	gluOrtho2D(0.0, (GLdouble)width, 0.0, (GLdouble)height);
	glMatrixMode(GL_MODELVIEW);
	glLoadIdentity();
	glShadeModel( GL_FLAT );

        // Position The Text On The Screen
        glRasterPos2i(x ,y);

        glPushAttrib(GL_LIST_BIT);				// Pushes the Display List Bits
	glBindTexture( GL_TEXTURE_2D, texture );
        glListBase(base-32);					// Sets base character to 32

	glCallLists(strlen(text),GL_UNSIGNED_BYTE,text);

        glPopAttrib();						// Pops the Display List Bits

	// Restore saved matrix
	glPopMatrix();

	// reset for 3D rendering
	glMatrixMode( GL_PROJECTION );
	glLoadIdentity( );
	ratio = ( GLfloat )width / ( GLfloat )height;
	gluPerspective( 45.0f, ratio, 0.1f, 100.0f );
	glMatrixMode( GL_MODELVIEW );
	glShadeModel( GL_SMOOTH );
	glLoadIdentity( );

        return( true );

	} ///////////////////////////////////////////////////////////////////////////end DrawTextInt

////////////////////////////////////////////////////////////////////////////////////////////////////
// BuildTTFont
// creates the font from given X font name
////////////////////////////////////////////////////////////////////////////////////////////////////
int CFont::BuildTTFont(CString *fn, int requested_height)
{
FT_Library	library;
FT_Face		face; //The object in which Freetype holds information on a given font is called a "face".
unsigned char	i;
int		result;

	// keep track of which type of font we are using
	fonttype=CF_TYPE_TT;

	//Allocate some memory to store the texture ids.
	textures = new GLuint[128];
	h=requested_height;

	//Create and initilize a freetype font library.
	if (FT_Init_FreeType( &library )) return CF_ERR_INITFREETYPEFAILED;


	//This is where we load in the font information from the file.
	//Of all the places where the code might die, this is the most likely,
	//as FT_New_Face will die if the font file does not exist or is somehow broken.
	if (FT_New_Face( library, fn->GetStringPtr(), 0, &face )) return CF_ERR_NEWFACEFAILED;

	//For some twisted reason, Freetype measures font size
	//in terms of 1/64ths of pixels.  Thus, to make a font
	//h pixels high, we need to request a size of h*64.
	//(h << 6 is just a prettier way of writting h*64)
	FT_Set_Char_Size( face, h * 64, h * 64, 96, 96);

	//Here we ask opengl to allocate resources for
	//all the textures and displays lists which we
	//are about to create.
	list_base=glGenLists(128);
	glGenTextures( 128, textures );

	//This is where we actually create each of the fonts display lists.
	for(i=0;i<128;i++) 
	{
		result=MakeDisplayList(face,i,list_base,textures);
		if (result!=CF_OK) return result;
		}

	//We don't need the face information now that the display
	//lists have been created, so we free the assosiated resources.
	FT_Done_Face(face);

	//Ditto for the library.
	FT_Done_FreeType(library);

	return CF_OK;

	} ///////////////////////////////////////////////////////////////////////////end BuildTTFont

////////////////////////////////////////////////////////////////////////////////////////////////////
// BuildFont
// creates the font from given X font name
////////////////////////////////////////////////////////////////////////////////////////////////////
bool CFont::BuildFont(CString *fn)
{
Display		*dpy;           // Our current X display
XFontStruct	*fontInfo;  // Our font info
//char		*dum;

	fonttype=CF_TYPE_OLD;
	fontname=*fn;

	// Storage for 96 characters
	base = glGenLists( 96 );

	// Get our current display long enough to get the fonts */
	dpy = XOpenDisplay( NULL );

	// Get the font information
	fontInfo = XLoadQueryFont( dpy, fontname.GetStringPtr());

	// If the above font didn't exist try one that should
	if ( fontInfo == NULL )
	{
		fontInfo = XLoadQueryFont( dpy, "fixed" );
		// If that font doesn't exist, something is wrong
		if ( fontInfo == NULL )
		{
			return false;
			}
		}

	// generate the list
	glXUseXFont( fontInfo->fid, 32, 96, base );

	// Recover some memory
	XFreeFont( dpy, fontInfo );

	// close the display now that we're done with it
	XCloseDisplay( dpy );
	return true;

	} /////////////////////////////////////////////////////////////////////////////end BuildFont

////////////////////////////////////////////////////////////////////////////////////////////////////
// SetTexure
// Sets the texture for the font 
////////////////////////////////////////////////////////////////////////////////////////////////////
bool CFont::SetTexture(char *texturename)
{
	// get texture filename, get texture "handle", make it a 2D texture, bind image to texture,
	CheckForGLErrors(LogFunc);
	ProtLog("CFont::SetTexture: Entered.");
	if ((TextureImage=IMG_Load(texturename))==NULL) return (false);
	ProtLog("CFont::SetTexture: Called IMG_Load.");
	glGenTextures(1, &texture);
	ProtLog("CFont::SetTexture: After GenTextures.");
	glBindTexture(GL_TEXTURE_2D, texture);
	glTexImage2D( GL_TEXTURE_2D, 0, 3, TextureImage->w, TextureImage->h,
			0, GL_RGB, GL_UNSIGNED_BYTE, TextureImage->pixels );
	ProtLog("CFont::SetTexture: After TexImage2D.");

	// set filters
	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST );
	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST );

	// free up memory used to create the texture
	if (TextureImage) SDL_FreeSurface(TextureImage);
	ProtLog("CFont::SetTexture: Exited.");
	return (true);

	} //////////////////////////////////////////////////////////////////////////////end SetTexure

////////////////////////////////////////////////////////////////////////////////////////////////////
// MakeDisplayList
////////////////////////////////////////////////////////////////////////////////////////////////////
int CFont::MakeDisplayList(FT_Face face, char ch, GLuint list_base, GLuint * tex_base )
{
FT_Glyph 	glyph;
FT_BitmapGlyph	bitmap_glyph;
//FT_Bitmap	&bitmap=0;
int		width;
int		height;
GLubyte		*expanded_data;
float   	x, y;
	
	//The first thing we do is get FreeType to render our character
	//into a bitmap.  This actually requires a couple of FreeType commands:

	//Load the Glyph for our character.
	if (FT_Load_Glyph( face, FT_Get_Char_Index( face, ch ), FT_LOAD_DEFAULT )) return CF_ERR_LOADGLYPHFAILED;

	//Move the face's glyph into a Glyph object.
	if(FT_Get_Glyph( face->glyph, &glyph )) return CF_ERR_GETGLYPHFAILED;

	//Convert the glyph to a bitmap.
	FT_Glyph_To_Bitmap( &glyph, ft_render_mode_normal, 0, 1 );
	bitmap_glyph = (FT_BitmapGlyph)glyph;

	//This reference will make accessing the bitmap easier
//	bitmap=bitmap_glyph->bitmap;

	//Use our helper function to get the widths of
	//the bitmap data that we will need in order to create
	//our texture.
//	width = next_p2( bitmap.width );
//	height = next_p2( bitmap.rows );
	width = NextPowerOfTwo(bitmap_glyph->bitmap.width);
	height = NextPowerOfTwo(bitmap_glyph->bitmap.rows);

	//Allocate memory for the texture data.
	expanded_data = new GLubyte[ 2 * width * height];

	//Here we fill in the data for the expanded bitmap.
	//Notice that we are using two channel bitmap (one for
	//luminocity and one for alpha), but we assign
	//both luminocity and alpha to the value that we
	//find in the FreeType bitmap.
	//We use the ?: operator so that value which we use
	//will be 0 if we are in the padding zone, and whatever
	//is the the Freetype bitmap otherwise.
	for(int j=0; j <height;j++) 
	{
		for(int i=0; i < width; i++)
		{
			expanded_data[2*(i+j*width)]= expanded_data[2*(i+j*width)+1] =
				(i>=bitmap_glyph->bitmap.width || j>=bitmap_glyph->bitmap.rows) ?
				0 : bitmap_glyph->bitmap.buffer[i + bitmap_glyph->bitmap.width*j];
			}
		}


        //Now we just setup some texture paramaters.
	glBindTexture( GL_TEXTURE_2D, tex_base[ch]);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);

	//Here we actually create the texture itself, notice
	//that we are using GL_LUMINANCE_ALPHA to indicate that
	//we are using 2 channel data.
	glTexImage2D( GL_TEXTURE_2D, 0, GL_RGBA, width, height,
		0, GL_LUMINANCE_ALPHA, GL_UNSIGNED_BYTE, expanded_data );

	//With the texture created, we don't need to expanded data anymore
	delete [] expanded_data;

	//So now we can create the display list
	glNewList(list_base+ch,GL_COMPILE);
	glBindTexture(GL_TEXTURE_2D,tex_base[ch]);

	//first we need to move over a little so that
	//the character has the right amount of space
	//between it and the one before it.
	glTranslatef(bitmap_glyph->left,0,0);

	//Now we move down a little in the case that the
	//bitmap extends past the bottom of the line
	//(this is only true for characters like 'g' or 'y'.
	glPushMatrix();
	glTranslatef(0,bitmap_glyph->top-bitmap_glyph->bitmap.rows,0);

	//Now we need to account for the fact that many of
	//our textures are filled with empty padding space.
	//We figure what portion of the texture is used by
	//the actual character and store that information in
	//the x and y variables, then when we draw the
	//quad, we will only reference the parts of the texture
	//that we contain the character itself.
	x=(float)bitmap_glyph->bitmap.width / (float)width; 
	y=(float)bitmap_glyph->bitmap.rows / (float)height;

        //Here we draw the texturemaped quads.
        //The bitmap that we got from FreeType was not
        //oriented quite like we would like it to be,
        //so we need to link the texture to the quad
        //so that the result will be properly aligned.
        glBegin(GL_QUADS);
        glTexCoord2d(0,0); glVertex2f(0,bitmap_glyph->bitmap.rows);
        glTexCoord2d(0,y); glVertex2f(0,0);
        glTexCoord2d(x,y); glVertex2f(bitmap_glyph->bitmap.width,0);
        glTexCoord2d(x,0); glVertex2f(bitmap_glyph->bitmap.width,bitmap_glyph->bitmap.rows);
        glEnd();
        glPopMatrix();
        glTranslatef(face->glyph->advance.x >> 6 ,0,0);


        //increment the raster position as if we were a bitmap font.
        //(only needed if you want to calculate text length)
        //glBitmap(0,0,0,0,face->glyph->advance.x >> 6,0,NULL);

        //Finnish the display list
        glEndList();
	return (CF_OK);

	} ///////////////////////////////////////////////////////////////////////end MakeDisplayList

////////////////////////////////////////////////////////////////////////////////////////////////////
// PushScreenCoordinateMatrix
////////////////////////////////////////////////////////////////////////////////////////////////////
inline void CFont::PushScreenCoordinateMatrix() 
{
GLint   viewport[4];

	glPushAttrib(GL_TRANSFORM_BIT);
	glGetIntegerv(GL_VIEWPORT, viewport);
	glMatrixMode(GL_PROJECTION);
	glPushMatrix();
	glLoadIdentity();
	gluOrtho2D(viewport[0],viewport[2],viewport[1],viewport[3]);
	glPopAttrib();
	} ////////////////////////////////////////////////////////////end PushScreenCoordinateMatrix

////////////////////////////////////////////////////////////////////////////////////////////////////
// PopProjectionMatrix
////////////////////////////////////////////////////////////////////////////////////////////////////
inline void CFont::PopProjectionMatrix() 
{
        glPushAttrib(GL_TRANSFORM_BIT);
        glMatrixMode(GL_PROJECTION);
        glPopMatrix();
        glPopAttrib();
	} ///////////////////////////////////////////////////////////////////end PopProjectionMatrix


