/* ************************************************************************* 
 *  Simple http daemon                                                     *
 *                                                                         *
 *  Author   :  Charlie Lee                                                *
 *  Date     :  May 26, 2003                                               *
 *  Filename :  httpd.c                                                    *
 *  Email    :  charlie.lee@anemoi.com                                     *
 ************************************************************************* */

#include <stdio.h>
#include <netdb.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#include <pwd.h>
#include <grp.h>
#include <syslog.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>

/* Required configuration */
#define HTML_DIRECTORY "/export/home/httpd/www"
#define PORT "80"
#define DEFAULT_FILE "index.html"
#define SERVER_NAME "www.anemoi.com"
#define VERSION "Simple HTTPD 1.0"
#define DEFAULT_USER "nobody"
#define DEFAULT_GROUP "nobody"
/* End of Require configuration */

#define BUFFER_SIZE 8192

////////////////////////////////////////////////////////////////////////////////////////////////////
// drop_privileges
////////////////////////////////////////////////////////////////////////////////////////////////////
static void drop_privileges()
{
struct passwd *pwd;
struct group *grp;

	if ((pwd = getpwnam(DEFAULT_USER)) == 0)
	{
		fprintf(stderr, "User not found in /etc/passwd\n");
		exit(1);
		}

	if ((grp = getgrnam(DEFAULT_GROUP)) == 0)
	{
		fprintf(stderr, "Group not found in /etc/group\n");
		exit(1);
		}

	if (chdir(HTML_DIRECTORY) != 0)
	{
		fprintf(stderr, "chdir(...) failed\n");
		exit(1);
		}
	
	if (chroot(HTML_DIRECTORY) != 0)
	{
		fprintf(stderr, "chroot(...) failed\n");
		exit(1);
		}

	if (setgid(grp->gr_gid) != 0)
	{
		fprintf(stderr, "setgid(...) failed\n");
		exit(1);
		}

	if (setuid(pwd->pw_uid) != 0)
	{
		fprintf(stderr, "setuid(...) failed\n");
		exit(1);
		}

	} ///////////////////////////////////////////////////////////////////////end drop_priveleges

////////////////////////////////////////////////////////////////////////////////////////////////////
// doit
////////////////////////////////////////////////////////////////////////////////////////////////////
static int doit(int connection, char *from_addr)
{
char request_orig[BUFFER_SIZE];
char request[BUFFER_SIZE];
FILE *fp;
struct stat file_stat;
char *result;
char *parse;
int n;
int i;

	memset(request_orig, '\0', sizeof(request));

	if (read(connection, request_orig, BUFFER_SIZE) > 0)
	{
		request_orig[strlen(request_orig) - 2] = 0;

		parse = strtok(request_orig, " \t\n");
		parse = strtok(NULL, " \t\n");

		snprintf(request, BUFFER_SIZE, "%s%s", parse, 
			(parse[strlen(parse) - 1] == '/') ? DEFAULT_FILE : "");

		syslog(LOG_WARNING, "Requesting %s from %s\n", request, from_addr);

		if ( (fp = fopen(request, "r")) != NULL)
		{
			if (lstat(request, &file_stat) != 0) return 1;

			result = malloc(file_stat.st_size);

			fread(result, file_stat.st_size, 1, fp);

			for (i = 0; i < file_stat.st_size; i += n)
	  		  	n = write(connection, result, file_stat.st_size);

			free(result);
			fclose(fp);
			}
		else
		{
			result = malloc(BUFFER_SIZE);

			snprintf(result, BUFFER_SIZE, "%s%s%s%s%s%s%s%s%s%s%s%s",
					"<HTML><HEAD><TITLE>404 Not Found</TITLE>",
					"</HEAD><BODY><H1>Not Found</H1>",
					"<P>The requested URL ",
					request,
					" was not found on this server.</P><ADDRESS>",
					VERSION,
					" (Unix) Server at ",
					SERVER_NAME,
					request,
					" Port ",
					PORT,
					"</ADDRESS></BODY></HTML>\r\n\r\n");

			for (i = 0; i < strlen(result); i += n)
  	   			n = write(connection, result, strlen(result));

			free(result);
			}
		}

	return 0;
	} //////////////////////////////////////////////////////////////////////////////////end doit

////////////////////////////////////////////////////////////////////////////////////////////////////
// main
////////////////////////////////////////////////////////////////////////////////////////////////////
int main(int argc, char *argv[])
{
char myhostname[BUFFER_SIZE];
struct hostent *hp;
int sd;
int connect_d;
int fromlen;
struct sockaddr_in sin;
struct sockaddr_in fsin;

	// open a connection to syslogd
	openlog(argv[0], LOG_PID | LOG_NDELAY, LOG_DAEMON);

	// Get my hostname.
	if (gethostname(myhostname, BUFFER_SIZE) != 0)
	{
		fprintf(stderr, "failed on gethostname(...) %s\n");
		exit(1);
		}

	// * lookup up our network number.  */
	if ( (hp = gethostbyname(myhostname)) == NULL)
	{
		fprintf(stderr, "Can't get hostname %s\n", myhostname);
		exit(1);
		}

	// Try to get a network socket.  
	if ( (sd = socket(AF_INET, SOCK_STREAM, 0)) < 0)
	{
		fprintf(stderr, "Can't get socket\n");
		exit(1);
		}

	// Create address we will bind to.  
	sin.sin_family = AF_INET;
	sin.sin_port = htons(atoi(PORT));
	bcopy(hp->h_addr, &sin.sin_addr, hp->h_length);

	// Bind to the port.
	if (bind(sd, (const struct sockaddr *) &sin, sizeof(sin)) < 0)
	{
		fprintf(stderr, "Can't bind\n");
		close(sd);
		exit(1);
		}
 
	// Now drop previleges.
	drop_privileges();

	// Listen for the connection.
	if (listen(sd, 5) < 0)
	{
		fprintf(stderr, "Can't listen to port\n");
		close(sd);
		exit(1);
		}

	// Accept connection.    
	for (;;)
	{
		fromlen = sizeof(struct sockaddr);
		if ( (connect_d = accept(sd, (struct sockaddr *) &fsin, &fromlen)) < 0)
		{
			fprintf(stderr, "Can't accept\n");
			close(sd);
			exit(1);
			}

		if (fork() == 0)
		{
			doit(connect_d, (char *) inet_ntoa(fsin.sin_addr));
			closelog();
			close(sd);
			close(connect_d);
			exit(0);
			}
		else close(connect_d);
		}

	return 0;
	} //////////////////////////////////////////////////////////////////////////////////end main

