//////////////////////////////////////////////////////////////////////////////////////////////////////
// citem.cpp
//
// the purpose is to build a doubly linked list using pointers to objects
// then to be able to move a current pointer back and forth along the list
// we will also have the ability to implement a tree structure by having 
// parent and child pointers
//
// by Eric J Krejci
// Copyright (C) 1999 Eric J Krejci 
// Revised : 26 Jan 2006
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation version 2.
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
// Contact:
//              email           worldii@viagloria.com
//              web             www.viagloria.com
//////////////////////////////////////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////////////////////////////////////
// Includes
//////////////////////////////////////////////////////////////////////////////////////////////////////
#include <curses.h>

#include "citem.hpp"	   
#include "worldiii.h"
#include "wiii_util.h"

////////////////////////////////////////////////////////////////////////////////////////////////////
// Externals
////////////////////////////////////////////////////////////////////////////////////////////////////
//extern bool	debug;
extern	int		loglevel;
extern	void		ThreadLog(char *s);
//extern	void		ConPrint(char *);
//extern	void		ConPrint(CString *);

////////////////////////////////////////////////////////////////////////////////////////////////////
// Globals
////////////////////////////////////////////////////////////////////////////////////////////////////
//int CItem::itemcount=0;            //init the static member
//int CItem::itemid=0;            //init the static member

////////////////////////////////////////////////////////////////////////////////////////////////////
//CNode
////////////////////////////////////////////////////////////////////////////////////////////////////
CNode::CNode()
{
     child=0;
     nextnode=0;
     }

////////////////////////////////////////////////////////////////////////////////////////////////////
//Node Destructor
////////////////////////////////////////////////////////////////////////////////////////////////////
CNode::~CNode()
{
     }

////////////////////////////////////////////////////////////////////////////////////////////////////
//Default Constructor
////////////////////////////////////////////////////////////////////////////////////////////////////
CItem::CItem()
{
//	pthread_mutex_init(&mutex, NULL);
	InitMutex(&mutex);

//     id=itemid++;             //set the id upon creation
//     itemcount++;
	nextitem=previtem=0;    //these are pointers
	childlist=0;            //points to the child list
	parent=0;               //points to child 
	errorstate=ERR_NOERROR;

     } //////////////////////////////////////////////////////////////////////////end ItemConstructor

////////////////////////////////////////////////////////////////////////////////////////////////////
//Default Destructor
////////////////////////////////////////////////////////////////////////////////////////////////////
CItem::~CItem()
{
//     itemcount--;  //decrease the item count
     } ///////////////////////////////////////////////////////////////////////////end ItemDestructor


////////////////////////////////////////////////////////////////////////////////////////////////////
//Default Constructor
////////////////////////////////////////////////////////////////////////////////////////////////////
CIterator::CIterator()
{
int		i;
//curstype	*c;

//	pthread_mutex_init(&mutex, NULL);
	InitMutex(&mutex);
     root=curr=last=0;     //zero is the nil value for ptr's in C++
     cptr=0;
     errorstate=ERR_NOERROR;
     curscount=MAXCURSORS;

	// Init cursors
	for (i=0;i<curscount;i++)
	{
		cursarray[i].curr=0;			// Not pointing to a CItem right now
		cursarray[i].inuse=false;		// Not in use right now
		cursarray[i].serverid=INVALIDSERVER;	// No real server has this guy right now
							// depend on "inuse" as INVALIDSERVER may be a valid thread
							// we're really just saying that the curs is not is use
		}
	} ///////////////////////////////////////////////////////////////////end IteratorConstructor

////////////////////////////////////////////////////////////////////////////////////////////////////
//Default Destructor
//- start at one end, if it is a child then orphan it
//- then delete it
//- a child can only have one parent so, when it is 
//  orphaned then we can delete it
////////////////////////////////////////////////////////////////////////////////////////////////////
CIterator::~CIterator()
{
     MoveFirst();
     while (!EndList())
     {
          if (curr->parent) 
          {
               OrphanChild(curr);
               }
          DeleteItem(curr);
          MoveFirst();
          }
     DeleteItem(curr); 

     } ////////////////////////////////////////////////////////////////////////////////end IteratorDestructor

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Lock
//  
/////////////////////////////////////////////////////////////////////////////////////////////////////////////
int CIterator::Lock()
{
     // lock the mutex
     return(MyLock(&mutex,true,"CIterator::Lock"));
     } //////////////////////////////////////////////////////////////////////////////////////////////end Lock

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// UnLock
//  
/////////////////////////////////////////////////////////////////////////////////////////////////////////////
int CIterator::UnLock()
{
     // unlock the mutex
     return(MyUnLock(&mutex,true,"CIterator::UnLock"));
     } ////////////////////////////////////////////////////////////////////////////////////////////end UnLock

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
//adds something to the list at the last end
/////////////////////////////////////////////////////////////////////////////////////////////////////////////
CItem *CIterator::AppendItem(CItem *additem)
{    
//static int i=0;

     Lock(); 

     if (last==0) // this should be the empty list case
     {
          curr=root=last=additem;
          cptr=curr->childlist;
          }
     else  //non empty list case
     {
          last->nextitem=additem;
          additem->previtem=last;
          curr=last=additem;
          cptr=curr->childlist;
          }
     UnLock();
     return (curr);
     }/////////////////////////////////////////////////////////////////////////////////////////end AppendItem

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
//displays the whole list
/////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CIterator::DisplayList()
{
CItem *temp;
int i;

     i=0;
     if ((root==0) || (last==0)) return;
     temp=GetCurrent();
     MoveFirst();                           //this shows how to use member
     while (!EndList())                     //functions inside other member
     {                                      //functions to accomplish work
          MoveNext();                       //using member functions is                                            
          }                                 //probably better because i can
     SetCurrent(temp);
     }

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
//displays the whole list backwards 
//starting from the last item
/////////////////////////////////////////////////////////////////////////////////////////////////////////////
void CIterator::BackDisplayList()
{
//CItem *temp;
int i;

     i=0;
     if ((root==0) || (last==0)) return;
     //this is another way to do the while loop to print out the list
     //by actually accessing the member variables directly

     } ///////////////////////////////////////////////////////////////////////////////////end BackDisplayList
          
/////////////////////////////////////////////////////////////////////////////////////////////////////////////
//Move next returns a pointer to the next item
//and moves the internal pointer to the next item
/////////////////////////////////////////////////////////////////////////////////////////////////////////////
CItem *CIterator::MoveNext()
{
     Lock();
     if ((root==0) && (last==0)) 
     {
		curr=0;
          UnLock();
          return(0);  // handle the empty list case
          }
     if (curr==0)
     {
          curr=root;
          cptr=curr->childlist;
          UnLock();
          return (curr);	// note: if empty list then we are returning nil anyway 20000227
          }
     if (curr==last) 
     {
          curr=0;
          UnLock();
          return (0);             //we should stick to convention, and return the nill pointer,
          }                       //this will make the lists work the way we unconsciously expect them to
     curr=curr->nextitem;
     cptr=curr->childlist;
     UnLock();
     return (curr);           //return what it was
     } //////////////////////////////////////////////////////////////////////////////////////////end MoveNext

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
//Move prev returns a pointer to the prev item
//and moves the internal pointer to the prev item
/////////////////////////////////////////////////////////////////////////////////////////////////////////////
CItem *CIterator::MovePrev()
{
     Lock();
     if ((root==0) && (last==0)) 
     {
          UnLock();
          return(0);  // handle the empty list case
          }
     if (curr==0)
     {
          curr=last;
          cptr=curr->childlist;
          UnLock();
          return (curr);
          }
     if (curr==root) 
     {
          cptr=curr->childlist;
          UnLock();
          return (0);
          }
     curr=curr->previtem;
     cptr=curr->childlist;
     UnLock();
     return (curr);           //return what it was
     } /////////////////////////////////////////////////////////////////////////////////end MovePrev

////////////////////////////////////////////////////////////////////////////////////////////////////
//Insert item
//a general insert statement that inserts 
//before the item where next is pointing, 
//returns a pointer to the item inserted
//next is always set to something valid
////////////////////////////////////////////////////////////////////////////////////////////////////
CItem *CIterator::InsertItem(CItem *additem)
{

//     ThreadLog("CIterator:: InsertItem");
     Lock(); 
     if ((root==0) && (last==0)) //nothing in list
     {
          ThreadLog("CIterator:: InsertItem nothing in list");
          root=last=additem;
          curr=root;
          cptr=curr->childlist;
          UnLock();
          return (curr);
          }

     if (root==last)     // single item list 
     {
          ThreadLog("CIterator:: InsertItem Single");
          additem->nextitem=root;
          root->previtem=additem;
          root=additem;
          curr=additem;
          cptr=curr->childlist;
          UnLock();
          return (curr);   //push on a stack like behavior
          }
 
     if (root==curr)	// if at first in list
     {
          ThreadLog("CIterator:: InsertItem pointing to first in list");
          additem->nextitem=root;
          root->previtem=additem;
          root=additem;
          curr=additem;
          cptr=curr->childlist;
          UnLock();
          return (curr);   //push on a stack like behavior
          }
     else    // this is the insert into multiple item list not at first position
     {
          if (curr==0) // 20000227: at least two items in list. 
          {
               // we probably just walked the list
               // so make curr=last
               ThreadLog("CIterator:: InsertItem setting curr=last");
               curr=last;
               }

          additem->nextitem=curr;
          curr->previtem->nextitem=additem;
          additem->previtem=additem->nextitem->previtem;
          curr->previtem=additem;
          curr=additem;
          cptr=curr->childlist;
          UnLock();
          return (curr);   //push on a stack like behavior
          }
     } ///////////////////////////////////////////////////////////////////////////////end InsertItem

////////////////////////////////////////////////////////////////////////////////////////////////////
// DeleteItem
// This is an overload that will ccall the main one with the proper stuff set
////////////////////////////////////////////////////////////////////////////////////////////////////
CItem *CIterator::DeleteItem(WIIHANDLE curs) 
{
//CItem *p;

	// We are given the curs handle that points to the item to delete.
	// Find the item to delete then pass the pointer to the standard DeleteItem
 	
	return(DeleteItem(cursarray[curs].curr));
} ////////////////////////////////////////////////////////////////////////////////////end DeleteItem

////////////////////////////////////////////////////////////////////////////////////////////////////
//Delete item
//- a general delete statement that deletes 
//  the item where delitem is pointing,
//  if delitem == 0 then just return zero 
//- returns a pointer to the item next 
//  towards last
//- curr is always set to something valid
//- returns curr if successful, 0 if failed
//  or 0 if list is now empty
//assert:if object is not a child then 
//       parent==0
//       else parent!=0
//use this to check for child parent link 
//when deleting an object from the double list
//- if it is a child then destroy parent child      
//  link before destroy object, 
//  use OrphanChild()
//- if it is a parent then do not destroy     
////////////////////////////////////////////////////////////////////////////////////////////////////
CItem *CIterator::DeleteItem(CItem *delitem ) 
{
	ThreadLog("CIterator:: DeleteItem: Entered.");
	Lock();

	// error checking
	if (delitem==0) { UnLock(); return (0); }			// prevent delete of null pointer
	if (delitem->childlist) { UnLock(); return (0); }		// parent? cant delete parent
	if (delitem->parent) OrphanChild(delitem);			// child? unlink from parent first
//	ThreadLog("CIterator:: DeleteItem: Before root==0.");
	if (root==0 && last==0) 
	{ 
		root=curr=last=0;
		UnLock(); 
		return (0); 
	}// no items in list


	if (root==last && curr==root)                             //exactly one item in list
	{
		delete curr;
		cptr=0;                             //nothing in child list anymore
		root=curr=last=0;
		ThreadLog("CIterator:: DeleteItem: Deleted single item, returning.");
		UnLock();
		return (root);
	}

	if (curr==root)                           //first item in list
	{
		root=root->nextitem;
		root->previtem=0;
		delete curr;
		curr=root;
		cptr=curr->childlist;                //update current child
		UnLock();
		return(curr);
	}

	if (curr==last)                           //last item in list
	{
		last=last->previtem;
		last->nextitem=0;
		delete curr;
		curr=last;
		cptr=curr->childlist;                //update current child
		UnLock();
		return (curr);
	}

	if (curr!=0)                              //item in middle
	{
	CItem *n;
		n=curr->previtem->nextitem=curr->previtem->nextitem->nextitem;
		curr->nextitem->previtem=curr->nextitem->previtem->previtem;
		delete curr;
		curr=n;
		cptr=curr->childlist;                //update current child
		UnLock();
		return (curr);
	}
	else
	{
//		ConPrint("CIterator:: DeleteItem: Unhandled delete condition in CItem delete.");
		curr=root;
		UnLock();
		return (curr);
	}

} ////////////////////////////////////////////////////////////////////////////////////end DeleteItem

////////////////////////////////////////////////////////////////////////////////////////////////////
//returns a pointer if the item is in 
//the list, curr unaffected     
////////////////////////////////////////////////////////////////////////////////////////////////////
CItem *CIterator::FindItem(CItem *r)
{
CItem *p;

     Lock();
     if (root==0) { UnLock(); return(0);}
     p=root;
     while (p!=0)  //////////////////////////////////check this!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
     {
          if (p==r) { UnLock(); return (r);} //found the record
          p=p->nextitem;
          }
     UnLock();
     return (0); //did not find the record we were looking for
     } //////////////////////////////////////////////////////////////////////////////////////////end FindItem

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
//set next
//this uses finditem to ensure that 
//the item is in the list
//then sets the pointer
/////////////////////////////////////////////////////////////////////////////////////////////////////////////
CItem *CIterator::SetCurrent(CItem *r)
{
CItem *p;

     Lock();
     if ((p=FindItem(r))) 
     {
          curr=p;
          cptr=curr->childlist;
          UnLock();
          return (curr);
          }
     else {UnLock(); return(0);}        
     } ///////////////////////////////////////////////////////////////////////////////end SetCurrent

////////////////////////////////////////////////////////////////////////////////////////////////////
//AdoptChild
//takes an existing object already in the list
//and makes it a child of a parent
//assume: both parent and child are 
//already in the double list
//- ensure a child is not it's own parent
//- ensure only one parent pointing at it
////////////////////////////////////////////////////////////////////////////////////////////////////
CItem *CIterator::AdoptChild(CItem *parent,CItem *child)
{
CNode *q;

     Lock();
     if (parent==child) {UnLock();return (0);}
     if (child->parent!=0) {UnLock();return (0);}

     q = new CNode;
     q->nextnode=parent->childlist; //insert if there is already a list
     parent->childlist=q;

     q->child=child;         //node points to child
     child->parent=parent;       //child points to parent
     UnLock();
     return (parent);
} ////////////////////////////////////////////////////////////////////////////////////end AdoptChild

////////////////////////////////////////////////////////////////////////////////////////////////////
//OrphanChild
//destroys parent child link
//assert:if object is not a child then 
//       up==0
//       else up!=0
//use this to check for child parent link 
//when deleting an object from the double list
////////////////////////////////////////////////////////////////////////////////////////////////////
void CIterator::OrphanChild(CItem *child)
{
CNode *t;

     Lock();
     if (child->parent->childlist->child==child) //if this is the first in the child list
     {
          t=child->parent->childlist;
          child->parent->childlist=child->parent->childlist->nextnode;
          child->parent=0;                    //not a child anymore
                                      //dn unaffected, it still has it's
          delete t;                   //children      
          }
     else
     {
     CNode *t1;

          //find the node that makes the parent child link, use trail ptr
          t=child->parent->childlist;
          while ((t!=0)&&(t->child!=child))
          {
               t1=t;          //standard trail pointer
               t=t->nextnode;
               }

          if (t!=0)
          {
               t1->nextnode=t->nextnode;  //cut the node out
               delete t;
               child->parent=0;
               }
          }//end if (child->parent->childlist->child==child)
     UnLock();
} ///////////////////////////////////////////////////////////////////////////////////end OrphanChild

////////////////////////////////////////////////////////////////////////////////////////////////////
//MoveNextChild
//moves to the next child of the curr item
//if the current item pointer changes
//then reset the cptr
////////////////////////////////////////////////////////////////////////////////////////////////////
CNode *CIterator::MoveNextChild()
{
     
     Lock();
     if (cptr) cptr=cptr->nextnode;
     UnLock();
     return (cptr);
} /////////////////////////////////////////////////////////////////////////////////end MoveNextChild

////////////////////////////////////////////////////////////////////////////////////////////////////
// GetCurrent
////////////////////////////////////////////////////////////////////////////////////////////////////
CItem *CIterator::GetCurrent() 
{
	return (curr);
} ////////////////////////////////////////////////////////////////////////////////////end GetCurrent

////////////////////////////////////////////////////////////////////////////////////////////////////
// GetCurrent
////////////////////////////////////////////////////////////////////////////////////////////////////
CItem *CIterator::GetCurrent(WIIHANDLE curs) 
{
	return (cursarray[curs].curr);
} ////////////////////////////////////////////////////////////////////////////////////end GetCurrent

////////////////////////////////////////////////////////////////////////////////////////////////////
//GetCurrentChild
//moves to the next child of the curr item
////////////////////////////////////////////////////////////////////////////////////////////////////
CItem *CIterator::GetCurrentChild()
{
	Lock();
	if (cptr) 
	{
		UnLock(); 
		return (cptr->child);
	}
	else 
	{
		UnLock(); 
		return(0);
	}

} ///////////////////////////////////////////////////////////////////////////////end GetCurrentChild

////////////////////////////////////////////////////////////////////////////////////////////////////
// GetCursor
// This is going to be called by many threads so we must protect the object from concurrent access
// Watch for deadlocks
// These cursors allow for many threads to have their own "current" position in the list maintained
// by this iterator
////////////////////////////////////////////////////////////////////////////////////////////////////
WIIHANDLE CIterator::GetCursor(unsigned long s)
{
int	i;
//char	buf[80];

	if (loglevel>=4) ThreadLog("CIterator:: GetCursor: Entered ");
				
	Lock();
	for (i=0;i<curscount;i++)
	{
		if (cursarray[i].inuse==false) goto GetCursor0;
		}
	UnLock();
	if (loglevel>=4) ThreadLog("CIterator:: GetCursor: Exit no cursors left");
	return (ERR_INVALID_CURSOR);

GetCursor0:
	cursarray[i].inuse=true;
	cursarray[i].serverid=s;
	cursarray[i].curr=root;
	UnLock();			// once i set the flag true its mine
	if (loglevel>=4) ThreadLog("CIterator:: GetCursor: Exit ok");
	return((WIIHANDLE)i);
} /////////////////////////////////////////////////////////////////////////////////////end GetCursor

////////////////////////////////////////////////////////////////////////////////////////////////////
// ReleaseCursor
////////////////////////////////////////////////////////////////////////////////////////////////////
bool CIterator::ReleaseCursor(WIIHANDLE curs, unsigned long serverid)
{
	// ok now release it
	if (cursarray[curs].serverid!=serverid) return false;
	Lock();
	if (cursarray[curs].inuse==true)
	{
		// release it
		cursarray[curs].inuse=false;
		cursarray[curs].serverid=INVALIDSERVER;
		cursarray[curs].curr=0;
		UnLock();
		return (true);
		}
	else
	{
		UnLock();
		return(false);
		}
	UnLock();
	} /////////////////////////////////////////////////////////////////////////end ReleaseCursor

////////////////////////////////////////////////////////////////////////////////////////////////////
// MoveFirst
// 
////////////////////////////////////////////////////////////////////////////////////////////////////
CItem *CIterator::MoveFirst(WIIHANDLE curs)
{

	Lock();
	if (loglevel>=4) ThreadLog("CIterator:: MoveFirst: Enter ");
	cursarray[curs].curr=root;
	UnLock();
	if (loglevel>=4) ThreadLog("CIterator:: MoveFirst: Exit ");
	return (cursarray[curs].curr);
} /////////////////////////////////////////////////////////////////////////////////////end MoveFirst

////////////////////////////////////////////////////////////////////////////////////////////////////
//Move next returns a pointer to the next item
//and moves the internal pointer to the next item
////////////////////////////////////////////////////////////////////////////////////////////////////
CItem *CIterator::MoveNext(WIIHANDLE curs)
{
CItem *retval;

	if ((curs>=curscount)||(curs<0)) return (0);
	Lock();
	if ((root==0) && (last==0))
	{
		UnLock();
		return(0);  // handle the empty list case
	}

	if (cursarray[curs].curr==0)
	{
		cursarray[curs].curr=root;
		cptr=(cursarray[curs].curr)->childlist;
		retval=cursarray[curs].curr;
		UnLock();
		return (retval);   // note: if empty list then we are returning nil anyway 20000227
	}

	if (cursarray[curs].curr==last)
	{
		cursarray[curs].curr=0;
		UnLock();
		return (0);             //we should stick to convention, and return the nill pointer,
	}                       //this will make the lists work the way we unconsciously expect them to

	cursarray[curs].curr=(cursarray[curs].curr)->nextitem;
	cptr=(cursarray[curs].curr)->childlist;
	retval=cursarray[curs].curr;
	UnLock();
	return (retval);           //return what it was
} //////////////////////////////////////////////////////////////////////////////////////end MoveNext                               

////////////////////////////////////////////////////////////////////////////////////////////////////
// EndList
////////////////////////////////////////////////////////////////////////////////////////////////////
bool CIterator::EndList(WIIHANDLE curs) 
{	
bool f;

//	ConPrint("CIterator::EndList: Checking Value.\n ");
	Lock();	
	if (loglevel>=4) ThreadLog("CIterator:: EndList: Entered ");
	if ((cursarray[curs].curr)==0) f=true;
	else f=false;
	UnLock();
	if (loglevel>=4) ThreadLog("CIterator:: EndList: Exit ");
	return (f);
} ///////////////////////////////////////////////////////////////////////////////////////end EndList






////////////////////////////////////////////////////////////////////////////////////////////////////
// Virtual Functions
////////////////////////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////////////////////////
// GetServerId
// Virtual
////////////////////////////////////////////////////////////////////////////////////////////////////
int CItem::GetObjectId()
{ return (0);}

////////////////////////////////////////////////////////////////////////////////////////////////////
// GetServerId
// Virtual
////////////////////////////////////////////////////////////////////////////////////////////////////
int CItem::GetServerId()
{ return (0);}

////////////////////////////////////////////////////////////////////////////////////////////////////
// GetRawRootPtr
// Virtual
////////////////////////////////////////////////////////////////////////////////////////////////////
//CRawBuf *CItem::GetRawRootPtr()
//{ return (0);}

////////////////////////////////////////////////////////////////////////////////////////////////////
// GetSockPtr
// Virtual
////////////////////////////////////////////////////////////////////////////////////////////////////
CUSock *CItem::GetSockPtr()
{ return (0);}

////////////////////////////////////////////////////////////////////////////////////////////////////
// GetCharacterPtr
// Virtual
////////////////////////////////////////////////////////////////////////////////////////////////////
CChar *CItem::GetCharacterPtr()
{ return (0);}

////////////////////////////////////////////////////////////////////////////////////////////////////
// HarvestPortmon
// Virtual
////////////////////////////////////////////////////////////////////////////////////////////////////
bool CItem::HarvestPortmon()
{ return (0);}

////////////////////////////////////////////////////////////////////////////////////////////////////
// HarvestQLoader
// Virtual
////////////////////////////////////////////////////////////////////////////////////////////////////
//bool CItem::HarvestQLoader()
//{ return (0);}

////////////////////////////////////////////////////////////////////////////////////////////////////
// HarvestServer
// Virtual
////////////////////////////////////////////////////////////////////////////////////////////////////
bool CItem::HarvestServer()
{ return (0);}

////////////////////////////////////////////////////////////////////////////////////////////////////
// HarvestServer
// Virtual
////////////////////////////////////////////////////////////////////////////////////////////////////
OBJIDTYPE CItem::GetCharacterID()
{ return (0);}

