////////////////////////////////////////////////////////////////////////////////////////////////////
// clist.cpp (client and modeler)
// This class will implement the base level of the model object 
// 
// by Eric J Krejci
// Copyright (C) 1999 Eric J Krejci 
// Revised : 19 Apr 2006 
//  
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation version 2.
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
// Contact:
//              email           worldii@viagloria.com
//              web             www.viagloria.com
////////////////////////////////////////////////////////////////////////////////////////////////////
#include <stdlib.h>
#include <stdio.h> 
#include <string.h>

#include "cworld.hpp"
#include "cstring.hpp"

////////////////////////////////////////////////////////////////////////////////////////////////////
// Defines
////////////////////////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////////////////////////
// Externals
////////////////////////////////////////////////////////////////////////////////////////////////////

extern char logfilename[];

////////////////////////////////////////////////////////////////////////////////////////////////////
// CList Constructor
////////////////////////////////////////////////////////////////////////////////////////////////////
CList::CList()
{
	pos=0;
	rot=0;
	list=0;
	tricount=0;
	vertcount=0;
	dontknow=0;
	listtype=0;
	} ///////////////////////////////////////////////////////////////////////////end Constructor

////////////////////////////////////////////////////////////////////////////////////////////////////
// CList Destructor
////////////////////////////////////////////////////////////////////////////////////////////////////
CList::~CList()
{
	// BUILD DESTRUCTOR !!!!!!!!
	// 977K memory leak !!!!!!!!
	} ////////////////////////////////////////////////////////////////////////////end Destructor

////////////////////////////////////////////////////////////////////////////////////////////////////
// CList Destructor
////////////////////////////////////////////////////////////////////////////////////////////////////
bool CList::Init()
{
	return true;
	} //////////////////////////////////////////////////////////////////////////////////end Init

////////////////////////////////////////////////////////////////////////////////////////////////////
// CList LoadHead
// Loads the small header and filters comments
////////////////////////////////////////////////////////////////////////////////////////////////////
int CList::LoadHead(FILE *f, int level, CString **tok, CString **tex)
{
CString		line;
char            delimss[3]={' ','\t','\0'};  // <<===String contains a tab and a space

LoadHead1:
	// Get a Line
	if (line.GetLine(f))
	{
		// check if first chars in the string are comment chars
		if (line.StrNCmp("//",2)==0)  
		{
			printf("FOUND COMMENT =>"); line.Display();// printf("\n");
			goto LoadHead1; 
			}

		// check tokens to ensure format correct (FIXME)
		line.TokenInit(delimss);	// <<===String contains a tab and a space
		*tok=line.GetToken();
		*tex=line.GetToken();
		(*tex)->Chomp();		
		return (CW_OK);
		}
	else return (CW_ERR_CONVERSION);

	} //////////////////////////////////////////////////////////////////////////////end LoadHead

////////////////////////////////////////////////////////////////////////////////////////////////////
// CList LoadLine
// Loads the small header and filters comments
////////////////////////////////////////////////////////////////////////////////////////////////////
int CList::LoadLine(FILE *f, int level, MODVECT *list)
{
CString		line;
CString		*tok;
int		ind;
char            delimss[3]={' ','\t','\0'};  // <<===String contains a tab and a space
float		tf;

CListLoadLine1:

	// Get a Line
	if (line.GetLine(f))
	{
		// check if first chars are comment chars
		if (line.StrNCmp("//",2)==0)  
		{
			printf("FOUND COMMENT =>"); line.Display(); //printf("\n");
			goto CListLoadLine1; 
			}

		// check tokens to ensure format correct (FIXME)
		line.Chomp();
		line.TokenInit(delimss);	// <<===String contains a tab and a space

		// extract tokens
		tok=line.GetToken();	delete tok;	// get first token saving type
		if (!line.GetInt(&ind))  goto CListLoadLine2;			// index

		if (!line.GetFloat(&(list[ind].point.x))) goto CListLoadLine2;	// point x
		if (!line.GetFloat(&(list[ind].point.y))) goto CListLoadLine2;	// point y
		if (!line.GetFloat(&(list[ind].point.z))) goto CListLoadLine2;	// point z

		if (!line.GetFloat(&(list[ind].norm.x))) goto CListLoadLine2;	// point x
		if (!line.GetFloat(&(list[ind].norm.y))) goto CListLoadLine2;	// point y
		if (!line.GetFloat(&(list[ind].norm.z))) goto CListLoadLine2;	// point z

		if (!line.GetFloat(&(list[ind].u))) 	goto CListLoadLine2;	// point u
		if (!line.GetFloat(&(list[ind].v))) 	goto CListLoadLine2;	// point v

		return (CW_OK);
		}
CListLoadLine2:

	printf("CList:: LoadLine: line='%s' length=%d\n",line.GetStringPtr(),line.GetLength());
	printf("CList:: LoadLine: Conversion error in loadline.\n");
	
	return (CW_ERR_CONVERSION);

	} //////////////////////////////////////////////////////////////////////////////end LoadLine

////////////////////////////////////////////////////////////////////////////////////////////////////
// CList LoadTriList
////////////////////////////////////////////////////////////////////////////////////////////////////
int CList::LoadTriList(FILE *f, int level)
{
int		i,res;
char		token[80];	// this is where tokens are returned to
int		ind;
CString         *tok, *tex;

	listtype=GL_TRIANGLES;

	if ((res=LoadHead(f,level, &tok, &tex))!=CW_OK) return (res);

	// get texture filename, get texture "handle", make it a 2D texture, bind image to texture,
	if ((TextureImage=IMG_Load(tex->GetStringPtr()))==NULL) return (CW_ERR_TEXLOAD);
	glGenTextures(1, &texture);
	glBindTexture(GL_TEXTURE_2D, texture);
	glTexImage2D( GL_TEXTURE_2D, 0, 3, TextureImage->w, TextureImage->h, 
			0, GL_RGB, GL_UNSIGNED_BYTE, TextureImage->pixels );

	// set filters
	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST );
	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST );

	// free up memory used to create the texture
	if (TextureImage) SDL_FreeSurface(TextureImage);

	// allocate memory for data
	list = new MODVECT [vertcount];
	
	// now read data
	for (i=0;i<vertcount;i++)  if (LoadLine(f, level, list)!=CW_OK) return (CW_ERR_CONVERSION); 

	return (CW_OK);
	} ///////////////////////////////////////////////////////////////////////////end LoadTriList

////////////////////////////////////////////////////////////////////////////////////////////////////
// CList LoadTriFan
////////////////////////////////////////////////////////////////////////////////////////////////////
int CList::LoadTriFan(FILE *f, int level)
{
int		i,res;
char		token[80];	// this is where tokens are returned to
int		ind;
CString         *tok, *tex;

	listtype=GL_TRIANGLE_FAN;

	if ((res=LoadHead(f,level, &tok, &tex))!=CW_OK) return (res);

	// get texture filename, get texture "handle", make it a 2D texture, bind image to texture,
	if ((TextureImage=IMG_Load(tex->GetStringPtr()))==NULL) return (CW_ERR_TEXLOAD);
	glGenTextures(1, &texture);
	glBindTexture(GL_TEXTURE_2D, texture);
	glTexImage2D( GL_TEXTURE_2D, 0, 3, TextureImage->w, TextureImage->h, 
			0, GL_RGB, GL_UNSIGNED_BYTE, TextureImage->pixels );

	// set filters
	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST );
	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST );

	// free up memory used to create the texture
	if (TextureImage) SDL_FreeSurface(TextureImage);

	// allocate memory for data
	list = new MODVECT [vertcount];
	
	// now read data
	for (i=0;i<vertcount;i++)  if (LoadLine(f, level, list)!=CW_OK) return (CW_ERR_CONVERSION); 

	return (CW_OK);
	} ////////////////////////////////////////////////////////////////////////////end LoadTriFan

////////////////////////////////////////////////////////////////////////////////////////////////////
// CList LoadTriStrip
////////////////////////////////////////////////////////////////////////////////////////////////////
int CList::LoadTriStrip(FILE *f, int level)
{
int		i,res;
char		token[80];//,texname[80];	// this is where tokens are returned to
int		ind;
CString         *tok, *tex;

	listtype=GL_TRIANGLE_STRIP;

	if ((res=LoadHead(f,level, &tok, &tex))!=CW_OK) return (res);

	// get texture filename, get texture "handle", make it a 2D texture, bind image to texture,
	if ((TextureImage=IMG_Load(tex->GetStringPtr()))==NULL) return (CW_ERR_TEXLOAD);
	glGenTextures(1, &texture);
	glBindTexture(GL_TEXTURE_2D, texture);
	glTexImage2D( GL_TEXTURE_2D, 0, 3, TextureImage->w, TextureImage->h, 
			0, GL_RGB, GL_UNSIGNED_BYTE, TextureImage->pixels );

	// set filters
	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST );
	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST );

	// free up memory used to create the texture
	if (TextureImage) SDL_FreeSurface(TextureImage);

	// allocate memory for data
	list = new MODVECT [vertcount];
	
	// now read data
	for (i=0;i<vertcount;i++)  if (LoadLine(f, level, list)!=CW_OK) return (CW_ERR_CONVERSION);

	return (CW_OK);
	} //////////////////////////////////////////////////////////////////////////end LoadTriStrip

////////////////////////////////////////////////////////////////////////////////////////////////////
// CList Load
////////////////////////////////////////////////////////////////////////////////////////////////////
int CList::Load(FILE *f, int level)
{
CString		*tok;
CString		line;
char            delimss[3]={' ','\t','\0'};  // <<===String contains a tab and a space

	// is it there
CListLoad1:
	// Get a Line
	if (line.GetLine(f))
	{
		// check if first chars are comment chars
		if (line.StrNCmp("//",2)==0)  
		{
			printf("FOUND COMMENT =>"); line.Display();// printf("\n");
			goto CListLoad1; 
			}

		// If we read a line that is too short go back and get another line
		if (line.GetLength()<=2) goto CListLoad1; 

		// check tokens to ensure format correct (FIXME)
		line.TokenInit(delimss);
		tok=line.GetToken();
		if (!line.GetInt(&tricount)) return (CW_ERR_CONVERSION);	// 
		if (!line.GetInt(&vertcount)) return (CW_ERR_CONVERSION);	// 
		if (!line.GetInt(&dontknow)) return (CW_ERR_CONVERSION);	// 
		}

	// call the correct loader for the list type we have
//	if (strncmp(tok->GetStringPtr(),TOKEN_TRISTRIP,80)==0) return(LoadTriStrip(f,level));
//	if (strncmp(tok->GetStringPtr(),TOKEN_TRILIST,80)==0) return(LoadTriList(f,level));
//	if (strncmp(tok->GetStringPtr(),TOKEN_TRIFAN,80)==0) return(LoadTriFan(f,level));
	if (*tok==TOKEN_TRISTRIP) return(LoadTriStrip(f,level));
	if (*tok==TOKEN_TRILIST) return(LoadTriList(f,level));
	if (*tok==TOKEN_TRIFAN) return(LoadTriFan(f,level));

	// shouldn't get here
	return (CW_ERR_UKNOWNLIST);
	} //////////////////////////////////////////////////////////////////////////////////end Load

////////////////////////////////////////////////////////////////////////////////////////////////////
// RenderList
////////////////////////////////////////////////////////////////////////////////////////////////////
int CList::RenderList()
{
int	i;

	// Bind the texture
	glBindTexture( GL_TEXTURE_2D, texture );

	// Start Drawing Triangles
	glBegin(GL_TRIANGLES);

	// Render the triangles
	for (i=0; i<vertcount; i++)
	{
		// Normal vector 
		glNormal3f(list[i].norm.x, list[i].norm.y, list[i].norm.z);

		// Set The TexCoord And Vertice
		glTexCoord2f(list[i].u, list[i].v);
		glVertex3f(list[i].point.x, list[i].point.y, list[i].point.z);
                }

	// End Drawing Triangles
	glEnd( );
	return CW_OK;

	} ////////////////////////////////////////////////////////////////////////////end RenderList

////////////////////////////////////////////////////////////////////////////////////////////////////
// RenderFan
////////////////////////////////////////////////////////////////////////////////////////////////////
int CList::RenderFan()
{
int	i;

	// Bind the texture
	glBindTexture( GL_TEXTURE_2D, texture );

	// Start Drawing Triangles
	glBegin(GL_TRIANGLE_FAN);

	// Render the triangles
	for (i=0; i<vertcount; i++)
	{
		// Normal vector 
		glNormal3f(list[i].norm.x, list[i].norm.y, list[i].norm.z);

		// Set The TexCoord And Vertice
		glTexCoord2f(list[i].u, list[i].v);
		glVertex3f(list[i].point.x, list[i].point.y, list[i].point.z);
                }

	// End Drawing Triangles
	glEnd( );
	return CW_OK;

	} ////////////////////////////////////////////////////////////////////////////end RenderFan

////////////////////////////////////////////////////////////////////////////////////////////////////
// RenderStrip
////////////////////////////////////////////////////////////////////////////////////////////////////
int CList::RenderStrip()
{
int	i;

	// Bind the texture
	glBindTexture( GL_TEXTURE_2D, texture );

	// Start Drawing Triangles
	glBegin(GL_TRIANGLE_STRIP);

	// Render the triangles
	for (i=0; i<vertcount; i++)
	{
		// Normal vector 
		glNormal3f(list[i].norm.x, list[i].norm.y, list[i].norm.z);

		// Set The TexCoord And Vertice
		glTexCoord2f(list[i].u, list[i].v);
		glVertex3f(list[i].point.x, list[i].point.y, list[i].point.z);
                }

	// End Drawing Triangles
	glEnd( );
	return CW_OK;

	} ////////////////////////////////////////////////////////////////////////////end RenderStrip

////////////////////////////////////////////////////////////////////////////////////////////////////
// Render
////////////////////////////////////////////////////////////////////////////////////////////////////
int CList::Render()
{
int	i;

	switch (listtype)
	{
	case GL_TRIANGLE_STRIP:
		return (RenderStrip());
		break;

	case GL_TRIANGLES:
		return (RenderList());
		break;

	case GL_TRIANGLE_FAN:
		return (RenderFan());
		break;

	default:
		return CW_ERR_UKNOWNLIST;
		}

	} ////////////////////////////////////////////////////////////////////////////////end Render

////////////////////////////////////////////////////////////////////////////////////////////////////
// CList Destroy
////////////////////////////////////////////////////////////////////////////////////////////////////
int CList::Destroy()
{

	// Destroy this CList 
	return 0;
	} ///////////////////////////////////////////////////////////////////////////////end Destroy

////////////////////////////////////////////////////////////////////////////////////////////////////
// CList Save
////////////////////////////////////////////////////////////////////////////////////////////////////
bool CList::Save(FILE *f)
{
//FILE		*f;
int		count;//,partcount;
//CInode		*p;
//CPart		*part;


	// is it there
//	if (!(f=fopen(myfilename.GetString(),"w"))) return (false);

	// count submodels
	count=0;
//	p=sublist;
//	while (p)
//	{
//		if (p->model) count++;
//		p=p->nextnode;
//		}

	fprintf(f,"%s %d\n", "[subcount]",count);

	// save submodels
//	p=sublist;
//	while (p)
//	{
//		// Now save each model name and offset and such
//		fprintf(f,"%s %s %.2f %.2f %.2f %.2f %.2f %.2f\n",
//			"[mod]",p->model->myfilename.GetString(),
//			p->model->xpos,p->model->ypos,p->model->zpos,
//			p->model->roll,p->model->pitch,p->model->yaw);
//		
//		if (!((p->model)->Save())) 
//		{
//			return false;
//			}
//		p=p->nextnode;
//		}

	// count parts
	count=0;
//	part=partlist;
//	while (part)
//	{
//		if (part) count++;
//		part=part->nextpart;
//		}

	fprintf(f,"%s %d\n", "[partcount]",count);

	// save parts
//	part=partlist;
//	while (part)
//	{
//		// Now save each model name and offset and such
//		if (!(part->Save(f))) 
//		{
//			return false;
//			}
//		part=part->nextpart;
//		}

	fclose(f);	
	return true;
} /////////////////////////////////////////////////////////////////////////////////////////end Save



