////////////////////////////////////////////////////////////////////////////////////////////////////
// cmessage.cpp
// This class will implement the message management 
// 
// by Eric J Krejci
// Copyright (C) 1999 Eric J Krejci 
// Revised : 26 Jan 2006
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation version 2.
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
// Contact:
//              email           worldii@viagloria.com
//              web             www.viagloria.com
////////////////////////////////////////////////////////////////////////////////////////////////////
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "cmessage.hpp"	   
#include "cstring.hpp"	   
#include "wiii_util.h"	   
#include "wiii_structures.h"	   

////////////////////////////////////////////////////////////////////////////////////////////////////
// Externals
////////////////////////////////////////////////////////////////////////////////////////////////////
//extern bool debug;
extern	int	loglevel;
extern	char	*logfilename;

extern	void	ThreadLog(char *s);

////////////////////////////////////////////////////////////////////////////////////////////////////
// Class globals
////////////////////////////////////////////////////////////////////////////////////////////////////
int		CMessage::count=CMDARRAYSIZE;
cmdtype		CMessage::cmdarray[CMDARRAYSIZE];         // An Array containing all legal commands
CString		CMessage::locallogfile;
CString		*CMessage::errorarray;


////////////////////////////////////////////////////////////////////////////////////////////////////
// CMessage
////////////////////////////////////////////////////////////////////////////////////////////////////
CMessage::CMessage()
{
int	i;

     // Initialize the instance
	msg=0;
	nextparm=0;
	nextmsg=0;
	prevmsg=0;
	errortype=0;

	for (i=0;i<MAXMESSAGEPARMS;i++) parm[i]=0;


} //////////////////////////////////////////////////////////////////////////////////////////end Init

////////////////////////////////////////////////////////////////////////////////////////////////////
// CMessage
// Destructor
////////////////////////////////////////////////////////////////////////////////////////////////////
CMessage::~CMessage()
{
int	i;
CString *s;

	// De-Initialize the instance
	if (loglevel>=7) ThreadLog("CMessage:: Destructor: Entered.");
	for (i=0;i<nextparm;i++) 
	{
		s=parm[i];
		delete s;
		}
	if (loglevel>=7) ThreadLog("CMessage:: Destructor: Exited.");
} //////////////////////////////////////////////////////////////////////////////////////////end Init

////////////////////////////////////////////////////////////////////////////////////////////////////
// CMessage ClassStaticInit
////////////////////////////////////////////////////////////////////////////////////////////////////
//void CMessage::ClassStaticInit(CString filename)
void CMessage::ClassStaticInit()
{

	if (loglevel>=1) ThreadLog("CMessage:: ClassStaticInit: Entered.");
        cmdarray[WIIIM_UNHANDLED].cmd.SetString(TOKEN_UNHANDLED);
        cmdarray[WIIIM_UNHANDLED].parmcnt=0;
        cmdarray[WIIIM_UNHANDLED].exact='+';
        cmdarray[WIIIM_UNHANDLED].msg=WIIIM_UNHANDLED;

        cmdarray[WIIIM_SETDIR].cmd.SetString(TOKEN_SETDIR);
        cmdarray[WIIIM_SETDIR].parmcnt=1;
        cmdarray[WIIIM_SETDIR].exact='=';
        cmdarray[WIIIM_SETDIR].msg=WIIIM_SETDIR;

        cmdarray[WIIIM_SETPITCH].cmd.SetString(TOKEN_SETPITCH);
        cmdarray[WIIIM_SETPITCH].parmcnt=1;
        cmdarray[WIIIM_SETPITCH].exact='=';
        cmdarray[WIIIM_SETPITCH].msg=WIIIM_SETPITCH;

        cmdarray[WIIIM_SETTRNSPD].cmd.SetString(TOKEN_SETTRNSPD);
        cmdarray[WIIIM_SETTRNSPD].parmcnt=1;
        cmdarray[WIIIM_SETTRNSPD].exact='=';
        cmdarray[WIIIM_SETTRNSPD].msg=WIIIM_SETTRNSPD;

        cmdarray[WIIIM_SETSPEED].cmd.SetString(TOKEN_SETSPEED);
        cmdarray[WIIIM_SETSPEED].parmcnt=1;
        cmdarray[WIIIM_SETSPEED].exact='=';
        cmdarray[WIIIM_SETSPEED].msg=WIIIM_SETSPEED;

        cmdarray[WIIIM_NEWPLAYER].cmd.SetString(TOKEN_NEWPLAYER);
        cmdarray[WIIIM_NEWPLAYER].parmcnt=1;
        cmdarray[WIIIM_NEWPLAYER].exact='=';
        cmdarray[WIIIM_NEWPLAYER].msg=WIIIM_NEWPLAYER;

        cmdarray[WIIIM_SETMODEL].cmd.SetString(TOKEN_SETMODEL);
        cmdarray[WIIIM_SETMODEL].parmcnt=1;
        cmdarray[WIIIM_SETMODEL].exact='=';
        cmdarray[WIIIM_SETMODEL].msg=WIIIM_SETMODEL;

        cmdarray[WIIIM_LOGOUT].cmd.SetString(TOKEN_LOGOUT);
        cmdarray[WIIIM_LOGOUT].parmcnt=1;
        cmdarray[WIIIM_LOGOUT].exact='=';
        cmdarray[WIIIM_LOGOUT].msg=WIIIM_LOGOUT;

        cmdarray[WIIIM_ACCTREQ].cmd.SetString(TOKEN_ACCTREQ);
        cmdarray[WIIIM_ACCTREQ].parmcnt=1;
        cmdarray[WIIIM_ACCTREQ].exact='=';
        cmdarray[WIIIM_ACCTREQ].msg=WIIIM_ACCTREQ;

        cmdarray[WIIIM_PING].cmd.SetString(TOKEN_PING);
        cmdarray[WIIIM_PING].parmcnt=0;
        cmdarray[WIIIM_PING].exact='=';
        cmdarray[WIIIM_PING].msg=WIIIM_PING;

        cmdarray[WIIIM_CLPRINT].cmd.SetString(TOKEN_CLPRINT);
        cmdarray[WIIIM_CLPRINT].parmcnt=1;
        cmdarray[WIIIM_CLPRINT].exact='=';
        cmdarray[WIIIM_CLPRINT].msg=WIIIM_CLPRINT;

        cmdarray[WIIIM_PRI].cmd.SetString(TOKEN_PRI);	
        cmdarray[WIIIM_PRI].parmcnt=2;	
        cmdarray[WIIIM_PRI].exact='=';
        cmdarray[WIIIM_PRI].msg=WIIIM_PRI;

        cmdarray[WIIIM_LOCATION].cmd.SetString(TOKEN_LOCATION);
        cmdarray[WIIIM_LOCATION].parmcnt=0;
        cmdarray[WIIIM_LOCATION].exact='=';
        cmdarray[WIIIM_LOCATION].msg=WIIIM_LOCATION;

        cmdarray[WIIIM_PORTCLOSED].cmd.SetString(TOKEN_PORTCLOSED);
        cmdarray[WIIIM_PORTCLOSED].parmcnt=0;
        cmdarray[WIIIM_PORTCLOSED].exact='=';
        cmdarray[WIIIM_PORTCLOSED].msg=WIIIM_PORTCLOSED;

        cmdarray[WIIIM_PLAYERID].cmd.SetString(TOKEN_PLAYERID);
        cmdarray[WIIIM_PLAYERID].parmcnt=2;
        cmdarray[WIIIM_PLAYERID].exact='=';
        cmdarray[WIIIM_PLAYERID].msg=WIIIM_PLAYERID;

        cmdarray[WIIIM_ACK].cmd.SetString(TOKEN_ACK);
        cmdarray[WIIIM_ACK].parmcnt=0;
        cmdarray[WIIIM_ACK].exact='=';
        cmdarray[WIIIM_ACK].msg=WIIIM_ACK;

        cmdarray[WIIIM_FORCEDBREAK].cmd.SetString(TOKEN_FORCEDBREAK);
        cmdarray[WIIIM_FORCEDBREAK].parmcnt=0;
        cmdarray[WIIIM_FORCEDBREAK].exact='=';
        cmdarray[WIIIM_FORCEDBREAK].msg=WIIIM_FORCEDBREAK;

        cmdarray[WIIIM_SETMODE].cmd.SetString(TOKEN_SETMODE);
        cmdarray[WIIIM_SETMODE].parmcnt=1;
        cmdarray[WIIIM_SETMODE].exact='=';
        cmdarray[WIIIM_SETMODE].msg=WIIIM_SETMODE;

        cmdarray[WIIIM_PLAYERCHAR].cmd.SetString(TOKEN_PLAYERCHAR);
        cmdarray[WIIIM_PLAYERCHAR].parmcnt=8;
        cmdarray[WIIIM_PLAYERCHAR].exact='=';
        cmdarray[WIIIM_PLAYERCHAR].msg=WIIIM_PLAYERCHAR;

        cmdarray[WIIIM_CHARSELECT].cmd.SetString(TOKEN_CHARSELECT);
        cmdarray[WIIIM_CHARSELECT].parmcnt=0;
        cmdarray[WIIIM_CHARSELECT].exact='=';
        cmdarray[WIIIM_CHARSELECT].msg=WIIIM_CHARSELECT;

        cmdarray[WIIIM_PLAY].cmd.SetString(TOKEN_PLAY);
        cmdarray[WIIIM_PLAY].parmcnt=1;
        cmdarray[WIIIM_PLAY].exact='=';
        cmdarray[WIIIM_PLAY].msg=WIIIM_PLAY;

        cmdarray[WIIIM_TURNLEFT].cmd.SetString(TOKEN_LEFT);
        cmdarray[WIIIM_TURNLEFT].parmcnt=0;
        cmdarray[WIIIM_TURNLEFT].exact='=';
        cmdarray[WIIIM_TURNLEFT].msg=WIIIM_TURNLEFT;

        cmdarray[WIIIM_TURNRIGHT].cmd.SetString(TOKEN_RIGHT);
        cmdarray[WIIIM_TURNRIGHT].parmcnt=0;
        cmdarray[WIIIM_TURNRIGHT].exact='=';
        cmdarray[WIIIM_TURNRIGHT].msg=WIIIM_TURNRIGHT;

        cmdarray[WIIIM_LISTACTIVEPLAYERS].cmd.SetString(TOKEN_LISTACTIVEPLAYERS);
        cmdarray[WIIIM_LISTACTIVEPLAYERS].parmcnt=0;
        cmdarray[WIIIM_LISTACTIVEPLAYERS].exact='=';
        cmdarray[WIIIM_LISTACTIVEPLAYERS].msg=WIIIM_LISTACTIVEPLAYERS;

        cmdarray[WIIIM_SETPOS].cmd.SetString(TOKEN_SETPOS);
        cmdarray[WIIIM_SETPOS].parmcnt=4;
        cmdarray[WIIIM_SETPOS].exact='=';
        cmdarray[WIIIM_SETPOS].msg=WIIIM_SETPOS;

        cmdarray[WIIIM_FORWARD].cmd.SetString(TOKEN_FORWARD);
        cmdarray[WIIIM_FORWARD].parmcnt=0;
        cmdarray[WIIIM_FORWARD].exact='=';
        cmdarray[WIIIM_FORWARD].msg=WIIIM_FORWARD;

        cmdarray[WIIIM_STOPMOVE].cmd.SetString(TOKEN_STOPMOVE);
        cmdarray[WIIIM_STOPMOVE].parmcnt=0;
        cmdarray[WIIIM_STOPMOVE].exact='=';
        cmdarray[WIIIM_STOPMOVE].msg=WIIIM_STOPMOVE;

        cmdarray[WIIIM_BACKWARD].cmd.SetString(TOKEN_BACKWARD);
        cmdarray[WIIIM_BACKWARD].parmcnt=0;
        cmdarray[WIIIM_BACKWARD].exact='=';
        cmdarray[WIIIM_BACKWARD].msg=WIIIM_BACKWARD;

        cmdarray[WIIIM_STOPTURN].cmd.SetString(TOKEN_STOPTURN);
        cmdarray[WIIIM_STOPTURN].parmcnt=0;
        cmdarray[WIIIM_STOPTURN].exact='=';
        cmdarray[WIIIM_STOPTURN].msg=WIIIM_STOPTURN;

        cmdarray[WIIIM_TELL].cmd.SetString(TOKEN_TELL);
        cmdarray[WIIIM_TELL].parmcnt=2;
        cmdarray[WIIIM_TELL].exact='=';
        cmdarray[WIIIM_TELL].msg=WIIIM_TELL;


	errorarray = new CString[ERRORITEMCOUNT];
	errorarray[CMSG_NOERROR].SetString(CMSG_NOERROR_T);
	errorarray[CMSG_CMDUNRQ].SetString(CMSG_CMDUNRQ_T);
	errorarray[CMSG_PARMCNT].SetString(CMSG_PARMCNT_T);
	errorarray[CMSG_NOMSG].SetString(CMSG_NOMSG_T);
	errorarray[CMSG_UNKERR].SetString(CMSG_UNKERR_T);



	count=CMDARRAYSIZE;
	if (loglevel>=1) ThreadLog("CMessage:: ClassStaticInit: Exited.");

	} ///////////////////////////////////////////////////////////////////////end ClassStaticInit

////////////////////////////////////////////////////////////////////////////////////////////////////
// CMessage GetErrorType
////////////////////////////////////////////////////////////////////////////////////////////////////
int CMessage::GetErrorType()
{
	return (errortype);
	} //////////////////////////////////////////////////////////////////////////end GetErrorType

////////////////////////////////////////////////////////////////////////////////////////////////////
// CMessage SetErrorType
////////////////////////////////////////////////////////////////////////////////////////////////////
void CMessage::SetErrorType(int e)
{
	if (e<ERRORITEMCOUNT) errortype=e;
	else errortype=CMSG_UNKERR;
	} //////////////////////////////////////////////////////////////////////////end SetErrorType

////////////////////////////////////////////////////////////////////////////////////////////////////
// CMessage SetMessage
////////////////////////////////////////////////////////////////////////////////////////////////////
void CMessage::SetMessage(int m)
{
        msg=m;
} ////////////////////////////////////////////////////////////////////////////////////end SetMessage

////////////////////////////////////////////////////////////////////////////////////////////////////
// CMessage SetUnhandled
////////////////////////////////////////////////////////////////////////////////////////////////////
void CMessage::SetUnhandled()
{
        msg=WIIIM_UNHANDLED;
} //////////////////////////////////////////////////////////////////////////////////end SetUnhandled

////////////////////////////////////////////////////////////////////////////////////////////////////
// CMessage GetMessage
////////////////////////////////////////////////////////////////////////////////////////////////////
int CMessage::GetMessage()
{
        return(msg);
} ////////////////////////////////////////////////////////////////////////////////////end GetMessage

////////////////////////////////////////////////////////////////////////////////////////////////////
// CMessage AddParmToMsg
////////////////////////////////////////////////////////////////////////////////////////////////////
bool CMessage::AddParmToMsg(int p, int tp, char *c)
{
char    *b;

        // get the parm and put in cstring
	ThreadLog("CMessage:: AddParmToMsg. Entered.");

	if ((nextparm<MAXMESSAGEPARMS) && (parm[nextparm]==0))
	{
        	b=c+tp;
		parm[nextparm] = new CString;
		parm[nextparm]->SetString(b,p-tp);
		nextparm++;
		ThreadLog("CMessage:: AddParmToMsg. Added and exited.");
		return (true);
	}
	ThreadLog("CMessage:: AddParmToMsg. DID NOT add and exited.");
        return (false);

} //////////////////////////////////////////////////////////////////////////////////end AddParmToMsg

////////////////////////////////////////////////////////////////////////////////////////////////////
// CMessage CheckMessage
// Check to ensure there are enough parameters for the message type
////////////////////////////////////////////////////////////////////////////////////////////////////
bool CMessage::CheckMessage()
{
	if (loglevel>=1) ThreadLog("CMessage:: CheckMessage. Entered.");

	switch (cmdarray[msg].exact)
	{
	case '<':
		if (loglevel>=1) ThreadLog("CMessage:: Check Message <. ");
		if (cmdarray[msg].parmcnt>nextparm) return (true);
		else return (false);
		break;

	case '-':
		if (loglevel>=1) ThreadLog("CMessage:: Check Message -. ");
		if (cmdarray[msg].parmcnt>=nextparm) return (true);
		else return (false);
		break;

	case '=':
		if (loglevel>=1) ThreadLog("CMessage:: Check Message =. ");
		if (cmdarray[msg].parmcnt==nextparm) return (true);
		else return (false);
		break;

	case '+':
		// greater than or equal
		if (cmdarray[msg].parmcnt<=nextparm) return (true);
		else return (false);
		if (loglevel>=1) ThreadLog("CMessage:: Check Message +. ");
		break;

	case '>':
		if (loglevel>=1) ThreadLog("CMessage:: Check Message >. ");
		if (cmdarray[msg].parmcnt<nextparm) return (true);
		else return (false);
		break;

	default:
		return(false);
		break;
	}
	return (false);
} //////////////////////////////////////////////////////////////////////////////////end CheckMessage

////////////////////////////////////////////////////////////////////////////////////////////////////
// CMessage::CheckProperCmd
////////////////////////////////////////////////////////////////////////////////////////////////////
bool CMessage::CheckProperCmd(int p, int tp, char *c)
{
char            *b;
int             i;
char	buf[90];
//CString	logitem;
	
	if (loglevel>=1) ThreadLog("CMessage:: CheckProperCmd: Entering.");
        b=c+tp;
	if (loglevel>=2) 
	{
		snprintf(buf,90,"CMessage:: CheckProperCmd. Checking token='%s' ",b);
		ThreadLog(buf);
		}
        for (i=1;i<count;i++) // start with 0 because the 0 command is the unrecognized string
        {
		snprintf(buf,90,"CMessage:: CheckProperCmd. Checking cmdarray='%s' ",cmdarray[i].cmd.GetStringPtr());
//		logitem.SetString("CMessage:: CheckProperCmd. Checking cmdarray='");
//		logitem=logitem+cmdarray[i].cmd+"'\n";
		ThreadLog(buf);

                // compare to find if message matches command
                if ((cmdarray[i].cmd.CompareN(b,p-tp))==0)
                {
                        // now we know what kind of command it is, so assign it.
			if (loglevel>=1) 
			{
				snprintf(buf,90,"CMessage:: CheckProperCmd. Found cmd='%s' ", cmdarray[i].cmd.GetStringPtr());
//				logitem.SetString("CMessage:: CheckProperCmd. Found cmd='");
//				logitem=logitem+cmdarray[i].cmd+"'\n";
				ThreadLog(buf);
//				ThreadLog(&logitem);
				}
                        SetMessage(cmdarray[i].msg);
			
			if (loglevel>=1) ThreadLog("CMessage:: CheckProperCmd: Returning true. ");
                        return (true);
                }
        }
	if (loglevel>=1) ThreadLog("CMessage:: CheckProperCmd: Returning false. ");
        return (false);
} ////////////////////////////////////////////////////////////////////////////end CheckProperCommand
	
////////////////////////////////////////////////////////////////////////////////////////////////////
// CMessage GetParm
////////////////////////////////////////////////////////////////////////////////////////////////////
bool CMessage::GetPlayerChar(pctype *p,int index)                    // get message
{

	// check for valid pointer and parms
	if (!p) return (false);
	if (index >= nextparm) return (false);
	// extract data
	p->playerid=parm[0];
	parm[1]->GetInt(&(p->objsaid));
	p->fname   =parm[2];
	p->race    =parm[3];
	parm[4]->GetFloat(&(p->xpos));
	parm[5]->GetFloat(&(p->ypos));
	parm[6]->GetFloat(&(p->zpos));
	p->title   =parm[7];
	return (true);

} ///////////////////////////////////////////////////////////////////////////////////////end GetParm

////////////////////////////////////////////////////////////////////////////////////////////////////
// CMessage GetParm
////////////////////////////////////////////////////////////////////////////////////////////////////
char *CMessage::GetParm(int index)                    // get message
{
//char *s;
	if (parm[index]) return(parm[index]->GetStringPtr());
	else return (0);
} ///////////////////////////////////////////////////////////////////////////////////////end GetParm

////////////////////////////////////////////////////////////////////////////////////////////////////
// CMessage GetParmCS
////////////////////////////////////////////////////////////////////////////////////////////////////
CString *CMessage::GetParmCS(int index)                    // get message
{
	if (parm[index]) return(parm[index]);
	else return (0);
} /////////////////////////////////////////////////////////////////////////////////////end GetParmCS

////////////////////////////////////////////////////////////////////////////////////////////////////
// CMessage GetParm
////////////////////////////////////////////////////////////////////////////////////////////////////
int CMessage::GetParmCount()
{
        return (nextparm);
} //////////////////////////////////////////////////////////////////////////////////////end GetData1

////////////////////////////////////////////////////////////////////////////////////////////////////
// CMessage GetPosup
////////////////////////////////////////////////////////////////////////////////////////////////////
posup *CMessage::GetPosup()
{
posup *p;

	// fill a posup struct to send to who ever
	if (nextparm<11 )return (0);
	if (	   (msg!=WIIIM_SETPOS) 
		&& (msg!=WIIIM_FORWARD)
		&& (msg!=WIIIM_BACKWARD)
		&& (msg!=WIIIM_TURNLEFT)
		&& (msg!=WIIIM_TURNRIGHT)
		&& (msg!=WIIIM_STOPMOVE)
		&& (msg!=WIIIM_STOPTURN)
		) return (0);

	// all checks ok now build the posup to send back
	p= new posup;
	if (!p) return (0);
	if (parm[0]) parm[0]->GetFloat(&(p->pos.x)); else p->pos.x = 0.0f; 
	if (parm[1]) parm[1]->GetFloat(&(p->pos.y)); else p->pos.y = 0.0f; 
	if (parm[2]) parm[2]->GetFloat(&(p->pos.z)); else p->pos.z = 0.0f; 
	if (parm[3]) parm[3]->GetFloat(&(p->vel.x)); else p->vel.x = 0.0f; 
	if (parm[4]) parm[4]->GetFloat(&(p->vel.y)); else p->vel.y = 0.0f; 
	if (parm[5]) parm[5]->GetFloat(&(p->vel.z)); else p->vel.z = 0.0f; 
	if (parm[6]) parm[6]->GetLongInt(&(p->id));  else p->id	   = 0;
	if (parm[7]) parm[7]->GetFloat(&(p->dir));   else p->dir   = 0.0f; 
	if (parm[8]) parm[8]->GetFloat(&(p->speed)); else p->speed = 0.0f; 
	if (parm[9]) parm[9]->GetFloat(&(p->elev));  else p->elev  = 0.0f;
	if (parm[10])p->model = *parm[10]; else p->model  = 0;
 
	return (p);
} //////////////////////////////////////////////////////////////////////////////////////end GetPosup


