////////////////////////////////////////////////////////////////////////////////////////////////////
// cmodel.cpp (client and modeler)
// This class will implement the model object 
// 
// by Eric J Krejci
// Copyright (C) 1999 Eric J Krejci 
// Revised : 02 May 2006 
//  
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation version 2.
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
// Contact:
//              email           worldii@viagloria.com
//              web             www.viagloria.com
////////////////////////////////////////////////////////////////////////////////////////////////////
#include <stdlib.h>
#include <stdio.h> 
#include <string.h>

#include "cworld.hpp"
#include "cstring.hpp"

////////////////////////////////////////////////////////////////////////////////////////////////////
// Defines
////////////////////////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////////////////////////
// Externals
////////////////////////////////////////////////////////////////////////////////////////////////////

extern char logfilename[];
extern GLfloat	angle;
////////////////////////////////////////////////////////////////////////////////////////////////////
// CModel Constructor
////////////////////////////////////////////////////////////////////////////////////////////////////
CModel::CModel()
{
	sublist=0;
	partlist=0;
	pos=0;
	rot=0;
	} ///////////////////////////////////////////////////////////////////////////end Constructor

////////////////////////////////////////////////////////////////////////////////////////////////////
// CModel Destructor
////////////////////////////////////////////////////////////////////////////////////////////////////
CModel::~CModel()
{
     // BUILD DESTRUCTOR !!!!!!!!
	// 977K memory leak !!!!!!!!
	} ////////////////////////////////////////////////////////////////////////////end Destructor

////////////////////////////////////////////////////////////////////////////////////////////////////
// CModel Destructor
////////////////////////////////////////////////////////////////////////////////////////////////////
bool CModel::Init()
{
	return true;
	} //////////////////////////////////////////////////////////////////////////////////end Init

////////////////////////////////////////////////////////////////////////////////////////////////////
// CModel SetFileName
////////////////////////////////////////////////////////////////////////////////////////////////////
//void CModel::SetFileName(const char  *f)
//{
//	myfilename.SetString(f);
//	} ///////////////////////////////////////////////////////////////////////////end SetFileName
	
////////////////////////////////////////////////////////////////////////////////////////////////////
// CModel InsertPart
////////////////////////////////////////////////////////////////////////////////////////////////////
bool CModel::InsertPart(CPart *p)
{

	// This should look A LOT like InsertRegion 
	// use existing allocated object and insert it.
	p->nextpart=partlist;
	partlist=p;
	return true;
	} ///////////////////////////////////////////////////////////////////////////end SetFileName

////////////////////////////////////////////////////////////////////////////////////////////////////
// CModel LoadHead
// Loads the small header and filters comments
////////////////////////////////////////////////////////////////////////////////////////////////////
int CModel::LoadHead(FILE *f, int level, CString **tok, int *val)
{
CString         line;
char            delimss[3]={' ','\t','\0'};  // <<===String contains a tab and a space

LoadHead1:
        // Get a Line
        if (line.GetLine(f))
        {
                // check if first chars in string are comment chars
                if (line.StrNCmp("//",2)==0)
                {
                        printf("FOUND COMMENT =>"); line.Display();// printf("\n");
                        goto LoadHead1;
                        }

                // check tokens to ensure format correct (FIXME)
                line.TokenInit(delimss);        // <<===String contains a tab and a space
                *tok=line.GetToken();
                line.GetInt(val);
                return (CW_OK);
                }
        else return (CW_ERR_CONVERSION);

        } //////////////////////////////////////////////////////////////////////////////end LoadHead

////////////////////////////////////////////////////////////////////////////////////////////////////
// CModel LoadLine
// Loads the small header and filters comments
////////////////////////////////////////////////////////////////////////////////////////////////////
int CModel::LoadLine(FILE *f, char **token, char **modelname,
			float *x, float *y, float *z,
			float *roll,  float *pitch, float *yaw)
{
CString         line;
CString         *tok;
int             ind;
char            delimss[3]={' ','\t','\0'};  // <<===String contains a tab and a space

CModelLoadLine1:

	// Get a Line
	if (line.GetLine(f))
	{
		// check if first chars in string are comment chars
		if (line.StrNCmp("//",2)==0)
		{
			printf("FOUND COMMENT =>"); line.Display(); //printf("\n");
			goto CModelLoadLine1;
			}

		// check tokens to ensure format correct (FIXME)
		line.Chomp();
		line.TokenInit(delimss);        // <<===String contains a tab and a space

		// extract tokens
		tok=line.GetToken();    tok->GetString(token); delete tok;// get first token saving type
		tok=line.GetToken();    tok->GetString(modelname); delete tok;// get first token saving type

		if (!line.GetFloat(x)) return (CW_ERR_CONVERSION);	// point x
		if (!line.GetFloat(y)) return (CW_ERR_CONVERSION);	// point y
		if (!line.GetFloat(z)) return (CW_ERR_CONVERSION);	// point z

		if (!line.GetFloat(roll)) return (CW_ERR_CONVERSION);	// roll
		if (!line.GetFloat(pitch)) return (CW_ERR_CONVERSION);	// pitch
		if (!line.GetFloat(yaw)) return (CW_ERR_CONVERSION);	// yaw

		return (CW_OK);
		}
	else return (CW_ERR_CONVERSION);
	}///////////////////////////////////////////////////////////////////////////////end LoadLine

////////////////////////////////////////////////////////////////////////////////////////////////////
// CModel Load
////////////////////////////////////////////////////////////////////////////////////////////////////
int CModel::Load(char *filename, int level)
{
FILE		*f;
float		subxpos,subypos,subzpos;
float		subroll,subpitch,subyaw;
int		i,subcount,partcount;
int		ind,res;
//char		token[80],modelname[80],texname[80];	// this is where tokens are returned to
char		*token,*modelname;	// this is where tokens are returned to
CModel		*submodel;
CPart		*part;
CString		*tok;

	// is it there
	printf("===================================================== \n");
	printf("Cmodel:: Load: Trying to open file: '%s' \n",filename);
	if (!(f=fopen(filename,"r"))) return (CW_ERR_FILE_OPEN);
	printf("Cmodel:: Load: Current File: '%s' level=%d\n",filename,level);

	// get mod count
	if ((res=LoadHead(f,level, &tok, &subcount))!=CW_OK) return (res);
	printf("Cmodel:: Load: token='%s' subcount=%d \n",tok->GetStringPtr(), subcount);

	// get submod info so it can load
	for (i=0;i<subcount;i++)
	{
		if (LoadLine(f, &token, &modelname, 
				&subxpos,&subypos,&subzpos,
				&subroll,&subpitch,&subyaw)!=CW_OK) return (CW_ERR_CONVERSION);
 
		submodel=new CModel;
		if (!submodel) return CW_ERR_MEMALLOCATE;
		else
		{
		CModInode *iptr;
     
			submodel->Init();
 
			// load submodel
			if ((submodel->Load(modelname,level+1))!=CW_OK) return CW_ERR_SUBMODLOAD;
 
			// insert submodel
			iptr = new CModInode;
			if (!iptr) return CW_ERR_MEMALLOCATE;
			iptr->SetTrans(subxpos,subypos,subzpos);
			iptr->SetRot(subroll,subpitch,subyaw);
			iptr->model=submodel;
			iptr->next=sublist;
			sublist=iptr;
			}
		}

	// get part count
	if ((res=LoadHead(f,level, &tok, &partcount))!=CW_OK) return (res);
	printf("Cmodel:: Load: token='%s' partcount=%d \n",tok->GetStringPtr(), partcount);
	for (i=0;i<partcount;i++)
	{
		part=new CPart;
		if ((part->Load(f,level+1))!=CW_OK) 
		{
			delete part;
			printf("CModel:: Load: Error on CPart load.\n");
			return CW_ERR_SUBPARTLOAD;
			}
		printf("Cmodel:: Load: part number %d \n",i);
                                                                                                                                     
		// insert part at head of list
		part->nextpart=partlist;
		partlist=part;
		}

	fclose(f);	
	return (CW_OK);
	} //////////////////////////////////////////////////////////////////////////////////end Load

////////////////////////////////////////////////////////////////////////////////////////////////////
// Render
// Dont worry about where i might be,
// just inherit the model matrix and the projection matrix, and render away.
// Although _this_ instance will set up the matrix for the sub instances
////////////////////////////////////////////////////////////////////////////////////////////////////
int CModel::Render()
{
int		i;
CModInode	*iptr;
CPart		*partptr;
GLfloat		matrix[16];


//	printf("CModel:: Render: Entered.\n");
	glGetFloatv(GL_MODELVIEW_MATRIX,matrix);	// Save the matrix so i can render me

	// Render each of _this_ models submodels
//	printf("CModel:: Render: Before iptr=sublist;.\n");
//	printf("CModel:: Render: Before iptr=sublist;.\n");
//	printf("CModel:: Render: Before iptr=sublist;.\n");
	iptr=sublist;
//	printf("CModel:: Render: After iptr=sublist;.\n");
	while (iptr)
	{
		if (iptr->model)
		{
			glTranslatef(iptr->xpos, iptr->ypos, iptr->zpos);	// Translate 
//			glRotatef(iptr->pitch+angle, 1.0f, 0.0f , 0.0f);		// Rotate pitch 
//			glRotatef(360.0f-(iptr->yaw)+angle, 0.0f, 1.0f , 0.0f);	// Rotate yaw 
//			glRotatef(iptr->roll+angle, 0.0f, 0.0f , 1.0f);		// Rotate roll 
			glRotatef(iptr->pitch, 1.0f, 0.0f , 0.0f);		// Rotate pitch 
			glRotatef(360.0f-(iptr->yaw), 0.0f, 1.0f , 0.0f);	// Rotate yaw 
			glRotatef(iptr->roll, 0.0f, 0.0f , 1.0f);		// Rotate roll 

			if ((iptr->model)->Render()==CW_ERR_RENDER) return CW_ERR_RENDER;
			glLoadMatrixf(matrix);					// Restore matrix
			}
//		printf("CModel:: Render: before pointing to next.\n");
		iptr=iptr->next;
//		printf("CModel:: Render: after pointing to next.\n");
		}
                                                                                                                                        
	// Render _this_ models own parts and lists
	partptr=partlist;
	while (partptr)
	{
		if ((partptr->Render())==CW_ERR_RENDER) return CW_ERR_RENDER;
		partptr=partptr->nextpart;
		}

	// Return  
//	printf("CModel:: Render: Exited.\n");
	return CW_OK;

	} ////////////////////////////////////////////////////////////////////////////////end Render

////////////////////////////////////////////////////////////////////////////////////////////////////
// CModel Destroy
////////////////////////////////////////////////////////////////////////////////////////////////////
int CModel::Destroy()
{

	// Destroy each of _this_ models submodels
	return 0;
	} ///////////////////////////////////////////////////////////////////////////////end Destroy

////////////////////////////////////////////////////////////////////////////////////////////////////
// CModel Save
////////////////////////////////////////////////////////////////////////////////////////////////////
bool CModel::Save(char *filename)
{
FILE		*f;
int		count;//,partcount;
//CModInode		*p;
//CPart		*part;


	// is it there
	if (!(f=fopen(filename,"w"))) return (false);

	// count submodels
	count=0;
//	p=sublist;
//	while (p)
//	{
//		if (p->model) count++;
//		p=p->nextnode;
//		}

	fprintf(f,"%s %d\n", "[subcount]",count);

	// save submodels
//	p=sublist;
//	while (p)
//	{
//		// Now save each model name and offset and such
//		fprintf(f,"%s %s %.2f %.2f %.2f %.2f %.2f %.2f\n",
//			"[mod]",p->model->myfilename.GetStringPtr(),
//			p->model->xpos,p->model->ypos,p->model->zpos,
//			p->model->roll,p->model->pitch,p->model->yaw);
//		
//		if (!((p->model)->Save())) 
//		{
//			return false;
//			}
//		p=p->nextnode;
//		}

	// count parts
	count=0;
//	part=partlist;
//	while (part)
//	{
//		if (part) count++;
//		part=part->nextpart;
//		}

	fprintf(f,"%s %d\n", "[partcount]",count);

	// save parts
//	part=partlist;
//	while (part)
//	{
//		// Now save each model name and offset and such
//		if (!(part->Save(f))) 
//		{
//			return false;
//			}
//		part=part->nextpart;
//		}

	fclose(f);	
	return true;
} /////////////////////////////////////////////////////////////////////////////////////////end Save



