////////////////////////////////////////////////////////////////////////////////////////////////////
// cqueue.cpp 
// This is a first in first out queue. It will be implemented as a singly linked list with
// void pointers to the data being listed. The caling functions will be required to know what type
// data in kept.
//
// by Eric J Krejci
// Copyright (C) 1999 Eric J Krejci
// Revised : 26 Jan 2006
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation version 2.
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
// Contact:
//              email           worldii@viagloria.com
//              web             www.viagloria.com
////////////////////////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////////////////////////
// Includes
////////////////////////////////////////////////////////////////////////////////////////////////////
#include "cqueue.hpp"

////////////////////////////////////////////////////////////////////////////////////////////////////
// Externals
////////////////////////////////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////////////////////////////////
// Constructor
////////////////////////////////////////////////////////////////////////////////////////////////////
CQueue::CQueue()
{
	head=0;
	tail=0;
	curr=0;
	tp=0;
	count=0;
	max=0;
	} ///////////////////////////////////////////////////////////////////////////end constructor

////////////////////////////////////////////////////////////////////////////////////////////////////
// Name: ~CFont()
// Desc: Font class destructor
////////////////////////////////////////////////////////////////////////////////////////////////////
CQueue::~CQueue()
{

	} ////////////////////////////////////////////////////////////////////////////end destructor

////////////////////////////////////////////////////////////////////////////////////////////////////
// Print
////////////////////////////////////////////////////////////////////////////////////////////////////
void CQueue::Print()
{
	} /////////////////////////////////////////////////////////////////////////////////end Print

////////////////////////////////////////////////////////////////////////////////////////////////////
// GetCount
////////////////////////////////////////////////////////////////////////////////////////////////////
unsigned long CQueue::GetCount()
{
	return (count);
	} //////////////////////////////////////////////////////////////////////////////end GetCount

////////////////////////////////////////////////////////////////////////////////////////////////////
// SetCount
////////////////////////////////////////////////////////////////////////////////////////////////////
void CQueue::SetMaxCount(unsigned long c)
{
	max=c;
	} ///////////////////////////////////////////////////////////////////////////end SetMaxCount

////////////////////////////////////////////////////////////////////////////////////////////////////
// IsEmpty
////////////////////////////////////////////////////////////////////////////////////////////////////
bool CQueue::IsEmpty()
{
	if ((head==0)&&(tail==0)&&(curr==0)&&(count==0)) return true;
	if ((head!=0)&&(tail!=0)&&(count>=0)&&(count<=max)) return false;
	exit(1); // it should not get here
	} ///////////////////////////////////////////////////////////////////////////////end IsEmpty

////////////////////////////////////////////////////////////////////////////////////////////////////
// IsFull
////////////////////////////////////////////////////////////////////////////////////////////////////
bool CQueue::IsFull()
{
	if (IsEmpty()) return false;
	if ((head!=0)&&(tail!=0)&&(count==max)) return true;
	if ((head!=0)&&(tail!=0)&&(count>=0)&&(count<max)) return false;

	} ////////////////////////////////////////////////////////////////////////////////end IsFull

////////////////////////////////////////////////////////////////////////////////////////////////////
// BegList
////////////////////////////////////////////////////////////////////////////////////////////////////
bool CQueue::BegList()
{
	// IsEmpty does a lot of cool error checking for us
	if (IsEmpty()) return false; 
	if (curr==head) return true; else return false;
	} ///////////////////////////////////////////////////////////////////////////////end BegList

////////////////////////////////////////////////////////////////////////////////////////////////////
// GetItem
////////////////////////////////////////////////////////////////////////////////////////////////////
void *CQueue::GetItem()
{
	// IsEmpty does a lot of cool error checking for us
	if (IsEmpty()) return 0; 
	if (curr) return (curr->data); else return 0;
	} ///////////////////////////////////////////////////////////////////////////////end GetItem

////////////////////////////////////////////////////////////////////////////////////////////////////
// EndList
////////////////////////////////////////////////////////////////////////////////////////////////////
bool CQueue::EndList()
{
	if (IsEmpty()) return false;
	if (curr==0) return true; else return false;
	} ///////////////////////////////////////////////////////////////////////////////end EndList

////////////////////////////////////////////////////////////////////////////////////////////////////
// MoveNext
////////////////////////////////////////////////////////////////////////////////////////////////////
void *CQueue::MoveNext()
{
	// if empty do nothing and return nothing
	if (IsEmpty()) return 0;

	// check this first, and leave curr unchanged
	if (EndList()) return (0);

	// if curr is valid
	tp=curr;
	curr=curr->next;
	if (curr) return (curr->data);
	else return (0);

	} //////////////////////////////////////////////////////////////////////////////end MoveNext

////////////////////////////////////////////////////////////////////////////////////////////////////
// MoveFirst
////////////////////////////////////////////////////////////////////////////////////////////////////
void *CQueue::MoveFirst()
{
	// if empty do nothing and return nothing
	if (IsEmpty()) return 0;

	curr=head;
	tp=head;
	return (curr->data);
	} /////////////////////////////////////////////////////////////////////////////end MoveFirst

////////////////////////////////////////////////////////////////////////////////////////////////////
// MoveLast
////////////////////////////////////////////////////////////////////////////////////////////////////
void *CQueue::MoveLast()
{
	// if empty do nothing and return nothing
	if (IsEmpty()) return 0;

	curr=tail;
	FixTP();
	return (curr->data);
	} //////////////////////////////////////////////////////////////////////////////end MoveLast

////////////////////////////////////////////////////////////////////////////////////////////////////
// AddFirst
////////////////////////////////////////////////////////////////////////////////////////////////////
bool CQueue::AddFirst(void *p)
{
nodetype	*inode;

	// allocate chunk of mem for node
	inode=new nodetype;
	if (inode)
	{
		// add to list
		head=inode;
		tail=inode;
		curr=inode;
		tp=inode;

		// since it is a single item, set next to null
		inode->next=0;

		// set data pointer
		inode->data=p;

		// keep track of count
		count++;
		return true;
		}
	else return false;

	} //////////////////////////////////////////////////////////////////////////////end AddFirst

////////////////////////////////////////////////////////////////////////////////////////////////////
// Append
// Append sets curr only if list was empty.
////////////////////////////////////////////////////////////////////////////////////////////////////
bool CQueue::Append(void *p)
{
nodetype	*inode;

	if (IsFull()) return false;
	if (IsEmpty()) return(AddFirst(p)); 
	else //list is not empty
	{
		// allocate chunk of mem for node
		inode=new nodetype;
		if (inode)
		{
			// add to list
			tail->next=inode;
			tail=inode;

			// since it is a single item, set next to null
			inode->next=0;

			// set data pointer
			inode->data=p;

			// keep track of count
			count++;
			return true;
			}
		else return false;
		}
	
	} ////////////////////////////////////////////////////////////////////////////////end Append

////////////////////////////////////////////////////////////////////////////////////////////////////
// Prepend 
// Sets curr only if list was empty.
////////////////////////////////////////////////////////////////////////////////////////////////////
bool CQueue::Prepend(void *p)
{
nodetype	*inode;

	if (IsFull()) return false;
	if (IsEmpty()) return(AddFirst(p)); 
	else //list is not empty
	{
		// allocate chunk of mem for node
		inode=new nodetype;
		if (inode)
		{
			// add to list
			inode->next=head;
			head=inode;

			// set data pointer
			inode->data=p;

			// keep track of count
			count++;
			return true;
			}
		else return false;
		}
	
	} ////////////////////////////////////////////////////////////////////////////////end Prepend

////////////////////////////////////////////////////////////////////////////////////////////////////
// FixTp
// This ensures that tp is set correctly
////////////////////////////////////////////////////////////////////////////////////////////////////
bool CQueue::FixTP()
{
nodetype	*lp; // lead pointer

	tp=head;
	lp=head;

	// so lets loop through till lp is curr, then tp is properly set
	while (lp!=curr)
	{
		if (lp->next)
		{
			tp=lp;
			lp=lp->next;
			}

		// in here lp should never nil out.
		else return(false);
		}
	return (true);
					
	} /////////////////////////////////////////////////////////////////////////////////end FixTP
 
////////////////////////////////////////////////////////////////////////////////////////////////////
// Insert 
// Sets curr only if list was empty.
// INSERT IS POTENTIALLY UNSAFE IF TP==CURR
////////////////////////////////////////////////////////////////////////////////////////////////////
bool CQueue::Insert(void *p)
{
nodetype	*inode;

	if (IsFull()) return false;
	if (IsEmpty()) return(AddFirst(p)); 
	if (EndList()) return(Append(p));
	else //list is not empty
	{
		// allocate chunk of mem for node
		inode=new nodetype;
		if (inode)
		{
			// is curr==head
			if (curr==head)
			{
				// insert before curr and leave tp pointing before curr
				inode->next=curr;
				head=inode;
				tp=inode;
				inode->data=p;
				}
			else
			{
				if (curr!=tp)
				{
					// curr!=head
					inode->next=curr;
					tp->next=inode;
					tp=inode;
					inode->data=p;
					}
				else
				{
//				nodetype	*lp; // lead pointer
//					// curr==tp. we can get here through a carfeully 
//					// selected order of operations. so we deal with 
//					// the case by finding out what tp should be, then 
//					// doing the normal operation
//					tp=head;
//					lp=head;
//					
//					// so lets loop through till lp is curr, then tp is properly set
//					while (lp!=curr)
//					{
//						if (lp->next)
//						{
//							tp=lp;
//							lp=lp->next;
//							}
//					
//						// in here lp should never nil out.
//						else exit(1);
//						}
					if (!FixTP()) exit(1);

					// ok now do the normal insert
					inode->next=curr;
					tp->next=inode;
					tp=inode;
					inode->data=p;
					}
				}
			// keep track of count
			count++;
			return true;
			}
		else return false;
		}
	} ////////////////////////////////////////////////////////////////////////////////end Insert

////////////////////////////////////////////////////////////////////////////////////////////////////
// DelCurr
// Deleteing from curr==tail is the most inefficient way to delete list
////////////////////////////////////////////////////////////////////////////////////////////////////
bool CQueue::DelCurr()
{
nodetype *p;

	// check for empty list
	if (EndList()) return(false); 
	
	// check for empty list
	if (IsEmpty()) return(false); 
	
	// check for single item in list
	if (head==tail)
	{
		// delete single inode and set head and tail
		if (head) 
		{
			delete head;
			head=0;
			tail=0;
			curr=0;
			tp=0;
			count=0;
			return (true);
			}
		
		}

	// if removing from head
	if (curr==head)
	{
		// save access to node to delete
		p=head;

		head=head->next;
		if (p)
		{
			delete p;
			curr=head;
			tp=head;
			count--;
			return (true);
			}
		}

	// if removing from tail
	else if (curr==tail)
	{
		// if tp is not correct
		if ((tp->next)!=curr)
		{
			// fix tp	
			if (!FixTP()) exit(1);
			}

		// now do normal delete
		p=curr;
		if(p)
		{
			delete p;
			tail=tp;
			curr=tp;
			tp->next=0;
			count--;
			return (true);
			}
		
		}

	// if removing from middle
	else 
	{
		// if tp is not correct
		if ((tp->next)!=curr)
		{
			// fix tp	
			if (!FixTP()) exit(1);
			}

		// now do normal delete
		p=curr;
		if(p)
		{
			tp->next=curr->next;
			curr=curr->next;
			count--;
			delete p;
			return (true);
			}
		}
	} ///////////////////////////////////////////////////////////////////////////////////DelCurr



