////////////////////////////////////////////////////////////////////////////////////////////////////
// Client
// ctextwin.cpp : This is a class to support the text window for player communications
//
// by Eric J Krejci
// Copyright (C) 1999 Eric J Krejci
// Revised : 11 Apr 2006
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation version 2.
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
// Contact:
//              email           worldii@viagloria.com
//              web             www.viagloria.com
////////////////////////////////////////////////////////////////////////////////////////////////////
#include <pthread.h>
#include <errno.h>

#include "ctextwin.hpp"	   

////////////////////////////////////////////////////////////////////////////////////////////////////
// Externals
////////////////////////////////////////////////////////////////////////////////////////////////////
extern char		logfilename[];
extern bool		debug;
extern int      	loglevel;

////////////////////////////////////////////////////////////////////////////////////////////////////
// Local Globals
////////////////////////////////////////////////////////////////////////////////////////////////////



////////////////////////////////////////////////////////////////////////////////////////////////////
// CTextWin Constructor
////////////////////////////////////////////////////////////////////////////////////////////////////
CTextWin::CTextWin()
{

	linecount=MESSAGE_BOX_SIZE;
	linesdisplayed=MESSAGE_BOX_DISP_SIZE;
	nextline=linesdisplayed-1;
	nextchar=0;
	width=height=0;
	mutex=0;
	pFont=0;
	left=top=0;
	bottomline=0;
	} ///////////////////////////////////////////////////////////////////////////end Constructor

////////////////////////////////////////////////////////////////////////////////////////////////////
// SetLogFunc
////////////////////////////////////////////////////////////////////////////////////////////////////
void CTextWin::SetLogFunc(void (*pfunc)(char*)) 
{
	LogFunc=pfunc;
	} ///////////////////////////////////////////////////////////////////////////end SetLongFunc

////////////////////////////////////////////////////////////////////////////////////////////////////
// ProtLog
////////////////////////////////////////////////////////////////////////////////////////////////////
void CTextWin::ProtLog(char *s)
{
	if (LogFunc) (*LogFunc)(s);
	} ///////////////////////////////////////////////////////////////////////////////end ProtLog

////////////////////////////////////////////////////////////////////////////////////////////////////
// Init
// The threading setup for the mutexes is supplied by the caller. this way the threading can be 
// setup consistently across the whole program.
////////////////////////////////////////////////////////////////////////////////////////////////////
//bool CTextWin::Init(pthread_mutexattr_t *m_attr)
bool CTextWin::Init()
{
int	i;
CString	fontname;
int	result;

//	pFont= new CText( strFont, dwFontSize, 0 );
	ProtLog("CTextWin::Init: Entered.");

	// create font 
	pFont= new CFont();
	pFont->SetLogFunc(LogFunc);
//	fontname.SetString("-adobe-helvetica-medium-r-normal--18-*-*-*-p-*-iso8859-1");
	fontname.SetString("Test.ttf");
	ProtLog("CTextWin::Init: Created pFont.");
//	pFont->BuildFont(&fontname);

	// create font and check for errors
	result=pFont->BuildTTFont(&fontname,12);
	if (result!=CF_OK) return (false);
	ProtLog("CTextWin::Init: Built pFont.");

	pFont->SetTexture("white.png");
	ProtLog("CTextWin::Init: SetTexture pFont.");
	for (i=0;i<MESSAGE_BOX_SIZE;i++) buffer[i].Reset();
	ProtLog("CTextWin::Init: Exited.");

	// init the mutex
	mutex = new pthread_mutex_t;
	if (!mutex) return (false);
//	pthread_mutex_init(mutex, m_attr);
	InitMutex(mutex);

	return (true);

	} ///////////////////////////////////////////////////////////////////////////end Constructor

////////////////////////////////////////////////////////////////////////////////////////////////////
// CServer Destructor
// MEM LEAK !!!!    Need to write destructor for array of cstrings
////////////////////////////////////////////////////////////////////////////////////////////////////
CTextWin::~CTextWin()
{
//	MyLog("CTextWin:: Destructor: Entered.",logfilename);

	// delete the allocated objects here
//	MyLog("CTextWin:: Destructor: Exited.", logfilename);
	} ////////////////////////////////////////////////////////////////////////////end Destructor

////////////////////////////////////////////////////////////////////////////////////////////////////
// LocalLock
// This is a wrapper for pthread lock that can print tid if required
////////////////////////////////////////////////////////////////////////////////////////////////////
int CTextWin::LocalLock(pthread_mutex_t *mutex, char *who)
{

	return (MyLock(mutex,true,who));
//	return (r);
	} /////////////////////////////////////////////////////////////////////////////end LocalLock

////////////////////////////////////////////////////////////////////////////////////////////////////
// LocalUnLock
// This is a wrapper for pthread lock that can print tid if required
////////////////////////////////////////////////////////////////////////////////////////////////////
int CTextWin::LocalUnLock(pthread_mutex_t *mutex, char * who)
{

	return (MyUnLock(mutex,true,who));
//	return (r);
	} ///////////////////////////////////////////////////////////////////////////end LocalUnLock

////////////////////////////////////////////////////////////////////////////////////////////////////
// SetTextWinSize
// Send in pointers to requested window position
// on exit variables contain actual window position
////////////////////////////////////////////////////////////////////////////////////////////////////
void	CTextWin::SetTextWinSize(int w,int h)
{
	LocalLock(mutex,"CTextWin");
	width=w;
	height=h;
	LocalUnLock(mutex,"CTextWin");
	} ////////////////////////////////////////////////////////////////////////end SetTextWinSize

////////////////////////////////////////////////////////////////////////////////////////////////////
// SetTextWinPos
// Send in pointers to requested window position
// on exit variables contain actual window position
////////////////////////////////////////////////////////////////////////////////////////////////////
void	CTextWin::SetTextWinPos(int x,int y)
{
	LocalLock(mutex,"CTextWin");
	top=y;
	left=x;
	LocalUnLock(mutex,"CTextWin");
	} /////////////////////////////////////////////////////////////////////////end SetTextWinPos

////////////////////////////////////////////////////////////////////////////////////////////////////
// InsertChar
////////////////////////////////////////////////////////////////////////////////////////////////////
bool	CTextWin::InsertChar(char c)
{
char buf[80];

//	if (debug) MyLog("CTextWin:: InsertChar: Entered.",logfilename);
//	printf("CTextWin:: InsertChar: Entered. c='%c'\n",c);
	LocalLock(mutex,"CTextWin");
	// if not carriage return
	if (c!='\n') 
	{
		// if the lines not full just appand
		if (buffer[nextline].GetLength()<MESSAGE_BOX_LINE_LEN) buffer[nextline].Append(c);
		else 
		{
			snprintf(buf,80,"Char to append='%c'\n",c);
			printf(buf);
			// else append then 
			buffer[nextline].Append(c);

			// point to next line and clear the line
			IncLine(&nextline);
			buffer[nextline].Reset();
			}
		}
	else
	{
		IncLine(&nextline);
		buffer[nextline].Reset();
		}
	LocalUnLock(mutex,"CTextWin");
	if (loglevel>=1) ProtLog("CTextWin:: InsertChar: Exit.");
//	printf("CTextWin:: InsertChar: Exit.\n");
	return(true);
	} ////////////////////////////////////////////////////////////////////////////end InsertChar

////////////////////////////////////////////////////////////////////////////////////////////////////
//
////////////////////////////////////////////////////////////////////////////////////////////////////
void	CTextWin::Return()
{
	LocalLock(mutex,"CTextWin");
	IncLine(&nextline);
	buffer[nextline].Reset();
	LocalUnLock(mutex,"CTextWin");
	} ////////////////////////////////////////////////////////////////////////////////end Return

////////////////////////////////////////////////////////////////////////////////////////////////////
// InsertLine
////////////////////////////////////////////////////////////////////////////////////////////////////
void	CTextWin::InsertLine(CString *cstr)
{
//	if (debug) MyLog("CTextWin:: InsertLine: Entered.",logfilename);

	LocalLock(mutex,"CTextWin");
//	IncLine(&nextline);
//	buffer[nextline]=(*cstr);
	buffer[nextline]=cstr->GetStringPtr();
	IncLine(&nextline);
	LocalUnLock(mutex,"CTextWin");
//	if (debug) MyLog("CTextWin:: InsertLine: Exit.",logfilename);

	} ////////////////////////////////////////////////////////////////////////////end InsertLine

////////////////////////////////////////////////////////////////////////////////////////////////////
// InsertLine
////////////////////////////////////////////////////////////////////////////////////////////////////
void	CTextWin::InsertLine(char *str)
{
//	if (debug) MyLog("CTextWin:: InsertLine: Entered.",logfilename);

	LocalLock(mutex,"CTextWin");
//	IncLine(&nextline);
//	buffer[nextline]=(*cstr);
	buffer[nextline].SetString(str);
	IncLine(&nextline);
	LocalUnLock(mutex,"CTextWin");
//	if (debug) MyLog("CTextWin:: InsertLine: Exit.",logfilename);

	} ////////////////////////////////////////////////////////////////////////////end InsertLine

////////////////////////////////////////////////////////////////////////////////////////////////////
// DrawTextWindow
////////////////////////////////////////////////////////////////////////////////////////////////////
void	CTextWin::DrawTextWindow(int mainwidth, int mainheight) // these are about the player window
								// not the text window
{
char	buf[200];
int i,index;
int startline, stopline;

	// call the text drawing from here
	LocalLock(mutex,"CTextWin");
	stopline  = nextline; 
	index = nextline; 
	for (i=0;i<MESSAGE_BOX_DISP_SIZE;i++)
	{
//		snprintf(buf,200,"CTextWin:: DrawTextWindow: length = %d\n",buffer[index].GetLength());
//		printf(buf);
		// i is the counter 
		// index is the line we are displaying
		if (buffer[index].GetLength()>0) 
		{
//			pFont->DrawText( left, (int)(top+(float)i*18), mainwidth, mainheight, &(buffer[index]) );
//			printf("CTextWin:: DrawTextWindow\n");
			pFont->DrawTTText( left, (int)(top+(float)i*(12+1)), mainwidth, mainheight, &(buffer[index]) );
//			pFont->DrawTTText( left, (int)(top+(float)i*18), &(buffer[index]) );
			}
		DecLine(&index);
		}
	LocalUnLock(mutex,"CTextWin");
	} ////////////////////////////////////////////////////////////////////////end DrawTextWindow

////////////////////////////////////////////////////////////////////////////////////////////////////
// BackSpace
////////////////////////////////////////////////////////////////////////////////////////////////////
void	CTextWin::BackSpace()				// Does the backspace function
{
	LocalLock(mutex,"CTextWin");
	buffer[nextline].Chop();
	LocalUnLock(mutex,"CTextWin");
	} /////////////////////////////////////////////////////////////////////////////end BackSpace

////////////////////////////////////////////////////////////////////////////////////////////////////
// IncLine
// no external call so doesnt need a lock
////////////////////////////////////////////////////////////////////////////////////////////////////
int	CTextWin::IncLine(int *line)		
{
	(*line)++;
	if ((*line)>=linecount) (*line)=0;
	return (*line);
	} ///////////////////////////////////////////////////////////////////////////////end IncLine

////////////////////////////////////////////////////////////////////////////////////////////////////
// DecLine
// no external call so doesnt need a lock
////////////////////////////////////////////////////////////////////////////////////////////////////
int	CTextWin::DecLine(int *line)		
{
	(*line)--;
	if ((*line)<=0) (*line)=linecount-1;
	return (*line);
	} ///////////////////////////////////////////////////////////////////////////////end DecLine

////////////////////////////////////////////////////////////////////////////////////////////////////
// Chop
////////////////////////////////////////////////////////////////////////////////////////////////////
int CTextWin::Chop()
{
	buffer[nextline].Chop();
	} //////////////////////////////////////////////////////////////////////////////////end Chop

////////////////////////////////////////////////////////////////////////////////////////////////////
// Append
////////////////////////////////////////////////////////////////////////////////////////////////////
void CTextWin::Append(const char addition)
{
	} ////////////////////////////////////////////////////////////////////////////////end Append

////////////////////////////////////////////////////////////////////////////////////////////////////
// Append
////////////////////////////////////////////////////////////////////////////////////////////////////
void CTextWin::Append(const char *addition)
{
	} ////////////////////////////////////////////////////////////////////////////////end Append




