////////////////////////////////////////////////////////////////////////////////////////////////////
// Merged
// ctimer.cpp
//
// This class will implement a timer that can read system time the provide elapsed
// times to calling procedures

// by Eric J Krejci
// Copyright (C) 1999 Eric J Krejci 
// Revised : 26 Jan 2006
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation version 2.
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
// Contact:
//              email           worldii@viagloria.com
//              web             www.viagloria.com
////////////////////////////////////////////////////////////////////////////////////////////////////
#include <sys/time.h>
#include "ctimer.hpp"       

////////////////////////////////////////////////////////////////////////////////////////////////////
// Externals
////////////////////////////////////////////////////////////////////////////////////////////////////
extern char             logfilename[];

////////////////////////////////////////////////////////////////////////////////////////////////////
// CTimer Constructor
////////////////////////////////////////////////////////////////////////////////////////////////////
CTimer::CTimer()
{
	// set timer to not run
	run=false;
	initsecs=0;                       // the time this timer was initted
	initusecs=0;                      //

	prevsecs=0;                       // the previous seconds count
	prevusecs=0;                      // the previous useconds count

	runsecs=0;                       // the elapsed seconds count
	runusecs=0;                      // the elapsed usecs count

	}////////////////////////////////////////////////////////////////////////////end Constructor

////////////////////////////////////////////////////////////////////////////////////////////////////
// CTimer Destructor
////////////////////////////////////////////////////////////////////////////////////////////////////
CTimer::~CTimer()
{
	// delete the memory pointed to by buf
	} ////////////////////////////////////////////////////////////////////////////end Destructor

////////////////////////////////////////////////////////////////////////////////////////////////////
// CTimer Init
////////////////////////////////////////////////////////////////////////////////////////////////////
void CTimer::Init()
{
struct timeval tv;
struct timezone tz;

	// get the start time and set run to false
	run=false;
	gettimeofday(&tv,&tz);
	initsecs  = tv.tv_sec;
	initusecs = tv.tv_usec;
	prevsecs  = 0;
	prevusecs = 0;
	runsecs  = 0;
	runusecs = 0;

	} //////////////////////////////////////////////////////////////////////////////////end Init

////////////////////////////////////////////////////////////////////////////////////////////////////
// CTimer GetInit
// Get time since timer Init
////////////////////////////////////////////////////////////////////////////////////////////////////
void CTimer::GetInit(long *secs, long *usecs)
{
struct timeval tvnow;
struct timezone tz;

	// get the start time and set run to false
	gettimeofday(&tvnow,&tz);
	*secs  = tvnow.tv_sec-initsecs;
	*usecs = tvnow.tv_usec-initusecs;
	if (*usecs<0)	
	{
		*secs-=1;
		*usecs+=1000000;
		}
	} ///////////////////////////////////////////////////////////////////////////////end GetInit

////////////////////////////////////////////////////////////////////////////////////////////////////
// CTimer GetDelta
// Get Elapsed time since timer started
////////////////////////////////////////////////////////////////////////////////////////////////////
void CTimer::GetDelta(long *secs, long *usecs)
{
struct timeval tvnow;
struct timezone tz;

	// get the start time and set run to false
	if (run)
	{
		gettimeofday(&tvnow,&tz);
		*secs  = tvnow.tv_sec-prevsecs;
		*usecs = tvnow.tv_usec-prevusecs;
		prevsecs=tvnow.tv_sec;
		prevusecs=tvnow.tv_usec;
		if (*usecs<0)	
		{
			*secs-=1;
			*usecs+=1000000;
			}
		}
	else
	{
		// we're not running so elapsed is zero
		*secs=0;
		*usecs=0;
		}

	} //////////////////////////////////////////////////////////////////////////////end GetDelta

////////////////////////////////////////////////////////////////////////////////////////////////////
// CTimer GetElapsed
// Get Elapsed time since timer started
////////////////////////////////////////////////////////////////////////////////////////////////////
void CTimer::GetElapsed(long *secs, long *usecs)
{
struct timeval tvnow;
struct timezone tz;

	// get the start time and set run to false

	if (run)
	{
		gettimeofday(&tvnow,&tz);
		*secs  = tvnow.tv_sec-runsecs;
		*usecs = tvnow.tv_usec-runusecs;
		if (*usecs<0)	
		{
			*secs-=1;
			*usecs+=1000000;
			}
		}
	else
	{
		// we're not running so elapsed is zero
		*secs=0;
		*usecs=0;
		}

	} ////////////////////////////////////////////////////////////////////////////end GetElapsed

////////////////////////////////////////////////////////////////////////////////////////////////////
// CTimer Run
////////////////////////////////////////////////////////////////////////////////////////////////////
void CTimer::Run()
{
struct timeval tvnow;
struct timezone tz;

	// set the flag
	run=true;

	// capture the time
	gettimeofday(&tvnow,&tz);
	runsecs  = tvnow.tv_sec;
	runusecs = tvnow.tv_usec;
	prevsecs=runsecs;
	prevusecs=runusecs;

	} ///////////////////////////////////////////////////////////////////////////////////end Run

////////////////////////////////////////////////////////////////////////////////////////////////////
// CTimer Stop
////////////////////////////////////////////////////////////////////////////////////////////////////
void CTimer::Stop()
{
	run=false;
	} //////////////////////////////////////////////////////////////////////////////////end Stop

