////////////////////////////////////////////////////////////////////////////////////////////////////
// Merged
// cusock.cpp
// This class will implement a class for sending & receiving messages & managing 
// the socket
// 
// by Eric J Krejci
// Copyright (C) 1999 Eric J Krejci 
// Revised : 01 May 2006
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation version 2.
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
// Contact:
//              email           worldii@viagloria.com
//              web             www.viagloria.com
////////////////////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////////////////////
// Defines
////////////////////////////////////////////////////////////////////////////////////////////////////
//#define		BU FFSIZE	120
//#define		__USE_UNIX98

////////////////////////////////////////////////////////////////////////////////////////////////////
// Include
////////////////////////////////////////////////////////////////////////////////////////////////////
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <strings.h>
#include <curses.h>
#include <pthread.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/time.h>
#include <sys/timeb.h>
#include <netinet/in.h>
#include <netdb.h>
#include <errno.h>
#include <arpa/inet.h>


#include "cusock.hpp"       
#include "worldiii.h"
#include "wiii_util.h"
////////////////////////////////////////////////////////////////////////////////////////////////////
// Externals
////////////////////////////////////////////////////////////////////////////////////////////////////
extern char             logfilename[];
extern pthread_attr_t	attr;

char			bigbuf[700];
extern struct timeb	starttime;
extern int		loglevel;               // sets level of logging 0 lowest= no logging,
						// 9 highest insanely verbose


////////////////////////////////////////////////////////////////////////////////////////////////////
// Set_Block_PORT_InHeap
// a wrapper to setting the packet serial to the packet in the heap
// this is to help keep all the magic numbers in one place. 
// Here p<PACKETHEAPCOUNT because we are trying to set values in existing packet
////////////////////////////////////////////////////////////////////////////////////////////////////
void CUSock::Set_Block_PORT_InHeap(unsigned long p, in_port_t port)
{
	if (p>=PACKETHEAPCOUNT) 
	{
		ProtLog("CUSock:: Set_Block_PORT_InHeap: ERROR requested packet number too high.");
		exit(1);
		}
	*(in_port_t *)(packetheap+p*(PACKETSIZE+PACKETHEAPHDR)+RUDP_PORT)=port;
	} /////////////////////////////////////////////////////////////////end Set_Block_PORT_InHeap

////////////////////////////////////////////////////////////////////////////////////////////////////
// Set_Packet_ACK_InHeap
// a wrapper to setting the packet flag to the packet in the heap
// this is to help keep all the magic numbers in one place. 
// Here p<PACKETHEAPCOUNT because we are trying to set values in existing packet
////////////////////////////////////////////////////////////////////////////////////////////////////
void CUSock::Set_Packet_ACK_InHeap(unsigned long p)
{
	if (p>=PACKETHEAPCOUNT) 
	{
		ProtLog("CUSock:: Set_Packet_FLG_InHeap: ERROR requested packet number too high.");
		exit(1);
		}
	*(unsigned char*)(packetheap+p*(PACKETSIZE+PACKETHEAPHDR)+RUDP_PACKET+RUDP_PKT_ACK)=RUDP_AF_ACK;
	} /////////////////////////////////////////////////////////////////end Set_Packet_FLG_InHeap

////////////////////////////////////////////////////////////////////////////////////////////////////
// Set_Packet_Header
// this is an overloaded function
// a wrapper to set the packet header to the packet in the heap
// this is to help keep all the magic numbers in one place. 
// Here p<PACKETHEAPCOUNT because we are trying to set values in existing packet
////////////////////////////////////////////////////////////////////////////////////////////////////
void CUSock::Set_Packet_Header(char *c)
{

	if (c==0) 
	{
		ProtLog("CUSock:: Set_Packet_Header: ERROR NULL pointer passed.");
		exit(1);
		}

	c[0]='R';
	c[1]='U';
	c[2]='D';
	c[3]='P';
	
	} /////////////////////////////////////////////////////////////////////end Set_Packet_Header

////////////////////////////////////////////////////////////////////////////////////////////////////
// Set_Packet_Header
// this is an overloaded function
// a wrapper to set the packet header to the packet in the heap
// this is to help keep all the magic numbers in one place. 
// Here p<PACKETHEAPCOUNT because we are trying to set values in existing packet
////////////////////////////////////////////////////////////////////////////////////////////////////
void CUSock::Set_Packet_Header(unsigned long p)
{
char *pointer;

	if (p>=PACKETHEAPCOUNT) 
	{
		ProtLog("CUSock:: Set_Packet_Header: ERROR requested packet number too high.");
		exit(1);
		}

	pointer=packetheap+p*(PACKETSIZE+PACKETHEAPHDR)+RUDP_PACKET+RUDP_PKT_HDR;
	pointer[0]='R';
	pointer[1]='U';
	pointer[2]='D';
	pointer[3]='P';
	
	} /////////////////////////////////////////////////////////////////////end Set_Packet_Header

////////////////////////////////////////////////////////////////////////////////////////////////////
// Set_Packet_NOACK_InHeap
// a wrapper to setting the packet flag to the packet in the heap
// this is to help keep all the magic numbers in one place. 
// Here p<PACKETHEAPCOUNT because we are trying to set values in existing packet
////////////////////////////////////////////////////////////////////////////////////////////////////
void CUSock::Set_Packet_NOACK_InHeap(unsigned long p)
{
	if (p>=PACKETHEAPCOUNT) 
	{
		ProtLog("CUSock:: Set_Packet_FLG_InHeap: ERROR requested packet number too high.");
		exit(1);
		}
	*(unsigned char*)(packetheap+p*(PACKETSIZE+PACKETHEAPHDR)+RUDP_PACKET+RUDP_PKT_FLG)=RUDP_AF_NOACK;
	} /////////////////////////////////////////////////////////////////end Set_Packet_FLG_InHeap

////////////////////////////////////////////////////////////////////////////////////////////////////
// Get_Packet_DATA_Ptr_InHeap
// a wrapper to copy the sending data into the heap
// Here p<PACKETHEAPCOUNT because we are trying to get values in existing packet
////////////////////////////////////////////////////////////////////////////////////////////////////
void *CUSock::Get_Packet_DATA_Ptr_InHeap(unsigned long p)
{

	if (p>=PACKETHEAPCOUNT) 
	{
		ProtLog("CUSock:: Get_Packet_DATA_Ptr_InHeap: ERROR requested packet number too high.");
		exit(1);
		}
	return((void*)(packetheap+p*(PACKETSIZE+PACKETHEAPHDR)+RUDP_PACKET)); 
	} ////////////////////////////////////////////////////////////end Get_Packet_DATA_Ptr_InHeap

////////////////////////////////////////////////////////////////////////////////////////////////////
// Get_Packet_DATA_InHeap
// a wrapper to copy the sending data into the heap
// Here p<PACKETHEAPCOUNT because we are trying to set values in existing packet
////////////////////////////////////////////////////////////////////////////////////////////////////
void CUSock::Get_Packet_DATA_InHeap(unsigned long p , char *s, int size)
{
//char		*packet;

	if (p>=PACKETHEAPCOUNT) 
	{
		ProtLog("CUSock:: Get_Packet_DATA_InHeap: ERROR requested packet number too high.");
		exit(1);
		}
	memcpy((void*)s,(void*)(packetheap+p*(PACKETSIZE+PACKETHEAPHDR)+RUDP_PACKET+RUDP_PKT_DATA),size);
	} ////////////////////////////////////////////////////////////////end Get_Packet_DATA_InHeap

////////////////////////////////////////////////////////////////////////////////////////////////////
// Set_Packet_InHeap
// a wrapper to copy the sending packet into the heap
// this is the WHOLE PACKET
// Here p<PACKETHEAPCOUNT because we are trying to set values in existing packet
////////////////////////////////////////////////////////////////////////////////////////////////////
void CUSock::Set_Packet_InHeap(unsigned long p , char *s, int size)
{
//char		*packet;

	if (p>=PACKETHEAPCOUNT) 
	{
		ProtLog("CUSock:: Set_Packet_InHeap: ERROR requested packet number too high.");
		exit(1);
		}
	memcpy((void*)(packetheap+p*(PACKETSIZE+PACKETHEAPHDR)+RUDP_PACKET),s,size);
	} /////////////////////////////////////////////////////////////////////end Set_Packet_InHeap

////////////////////////////////////////////////////////////////////////////////////////////////////
// Set_Packet_DATA_InHeap
// a wrapper to copy the sending data into the heap
// this is just the DATA
// Here p<PACKETHEAPCOUNT because we are trying to set values in existing packet
////////////////////////////////////////////////////////////////////////////////////////////////////
void CUSock::Set_Packet_DATA_InHeap(unsigned long p , char *s, int size)
{
//char		*packet;

	if (p>=PACKETHEAPCOUNT) 
	{
		ProtLog("CUSock:: Set_Packet_DATA_InHeap: ERROR requested packet number too high.");
		exit(1);
		}
	memcpy((void*)(packetheap+p*(PACKETSIZE+PACKETHEAPHDR)+RUDP_PACKET+RUDP_PKT_DATA),s,size);
	} ////////////////////////////////////////////////////////////////end Set_Packet_DATA_InHeap

////////////////////////////////////////////////////////////////////////////////////////////////////
// Set_Packet_FLG_InHeap
// a wrapper to setting the packet flag to the packet in the heap
// this is to help keep all the magic numbers in one place. 
// Here p<PACKETHEAPCOUNT because we are trying to set values in existing packet
////////////////////////////////////////////////////////////////////////////////////////////////////
void CUSock::Set_Packet_FLG_InHeap(unsigned long p, unsigned char flag)
{
	if (p>=PACKETHEAPCOUNT) 
	{
		ProtLog("CUSock:: Set_Packet_FLG_InHeap: ERROR requested packet number too high.");
		exit(1);
		}
	*(unsigned char*)(packetheap+p*(PACKETSIZE+PACKETHEAPHDR)+RUDP_PACKET+RUDP_PKT_FLG)=flag;
	} /////////////////////////////////////////////////////////////////end Set_Packet_FLG_InHeap

////////////////////////////////////////////////////////////////////////////////////////////////////
// Get_Packet_SES_InBuff
// a wrapper to setting the packet session of the packet in the buff supplied
// this is to help keep all the magic numbers in one place. 
////////////////////////////////////////////////////////////////////////////////////////////////////
unsigned long CUSock::Get_Packet_SES_InBuff(char *c)
{
	if (c==0) 
	{
		ProtLog("CUSock:: Get_Packet_SES_InBuff: ERROR NULL ptr passed for buffer.");
		exit(1);
		}
	return((*(unsigned long*)(c+RUDP_PKT_SES)));
	} /////////////////////////////////////////////////////////////////end Get_Packet_SES_InBuff

////////////////////////////////////////////////////////////////////////////////////////////////////
// Get_Packet_SER_InBuff
// a wrapper to setting the packet serial of the packet in the buff supplied
// this is to help keep all the magic numbers in one place. 
////////////////////////////////////////////////////////////////////////////////////////////////////
unsigned long CUSock::Get_Packet_SER_InBuff(char *c)
{
	if (c==0) 
	{
		ProtLog("CUSock:: Get_Packet_SER_InBuff: ERROR NULL ptr passed for buffer.");
		exit(1);
		}
	return((*(unsigned long*)(c+RUDP_PKT_SER)));
	} /////////////////////////////////////////////////////////////////end Get_Packet_SER_InBuff

////////////////////////////////////////////////////////////////////////////////////////////////////
// Get_Packet_SES_InHeap
// a wrapper to setting the packet serial to the packet in the heap
// this is to help keep all the magic numbers in one place. 
// Here p<PACKETHEAPCOUNT because we are trying to set values in existing packet
////////////////////////////////////////////////////////////////////////////////////////////////////
unsigned long CUSock::Get_Packet_SES_InHeap(unsigned long p)
{
	if (p>=PACKETHEAPCOUNT) 
	{
		ProtLog("CUSock:: Get_Packet_SES_InHeap: ERROR requested packet number too high.");
		exit(1);
		}
	return(*(unsigned long*)(packetheap+p*(PACKETSIZE+PACKETHEAPHDR)+RUDP_PACKET+RUDP_PKT_SES));
	} /////////////////////////////////////////////////////////////////end Get_Packet_SES_InHeap

////////////////////////////////////////////////////////////////////////////////////////////////////
// Set_Packet_SES_InHeap
// a wrapper to setting the packet serial to the packet in the heap
// this is to help keep all the magic numbers in one place. 
// Here p<PACKETHEAPCOUNT because we are trying to set values in existing packet
////////////////////////////////////////////////////////////////////////////////////////////////////
void CUSock::Set_Packet_SES_InHeap(unsigned long p, unsigned long ses)
{
	if (p>=PACKETHEAPCOUNT) 
	{
		ProtLog("CUSock:: Set_Packet_SES_InHeap: ERROR requested packet number too high.");
		exit(1);
		}
	*(unsigned long*)(packetheap+p*(PACKETSIZE+PACKETHEAPHDR)+RUDP_PACKET+RUDP_PKT_SES)=ses;
	} /////////////////////////////////////////////////////////////////end Set_Packet_SES_InHeap

////////////////////////////////////////////////////////////////////////////////////////////////////
// Set_Packet_RET_InHeap
// a wrapper to setting the packet serial to the packet in the heap
// this is to help keep all the magic numbers in one place. 
// Here p<PACKETHEAPCOUNT because we are trying to set values in existing packet
////////////////////////////////////////////////////////////////////////////////////////////////////
void CUSock::Set_Packet_RET_InHeap(unsigned long p, unsigned long retry)
{
	if (p>=PACKETHEAPCOUNT) 
	{
		ProtLog("CUSock:: Set_Packet_RET_InHeap: ERROR requested packet number too high.");
		exit(1);
		}
	*(unsigned long*)(packetheap+p*(PACKETSIZE+PACKETHEAPHDR)+RUDP_PACKET+RUDP_PKT_RET)=retry;
	} /////////////////////////////////////////////////////////////////end Set_Packet_RET_InHeap

////////////////////////////////////////////////////////////////////////////////////////////////////
// Get_Packet_SER_InHeap
// a wrapper to setting the packet size for a packet in the heap
// this is to help keep all the magic numbers in one place. 
// Here p<PACKETHEAPCOUNT because we are trying to set values in existing packet
////////////////////////////////////////////////////////////////////////////////////////////////////
unsigned long CUSock::Get_Packet_SER_InHeap(unsigned long p)
{
	if (p>=PACKETHEAPCOUNT) 
	{
		ProtLog("CUSock:: Get_Packet_SER_InHeap: ERROR requested packet number too high.");
		exit(1);
		}
	return(*(unsigned long*)(packetheap+p*(PACKETSIZE+PACKETHEAPHDR)+RUDP_PACKET+RUDP_PKT_SER));
	} ///////////////////////////////////////////////////////////////end Get_Packet_SER_FromHeap

////////////////////////////////////////////////////////////////////////////////////////////////////
// Set_Packet_SER_InHeap
// a wrapper to setting the packet size for a packet in the heap
// this is to help keep all the magic numbers in one place. 
// Here p<PACKETHEAPCOUNT because we are trying to set values in existing packet
////////////////////////////////////////////////////////////////////////////////////////////////////
void CUSock::Set_Packet_SER_InHeap(unsigned long p, unsigned long ser)
{
	if (p>=PACKETHEAPCOUNT) 
	{
		ProtLog("CUSock:: Set_PKT_SER_InHeap: ERROR requested packet number too high.");
		exit(1);
		}
	*(unsigned long*)(packetheap+p*(PACKETSIZE+PACKETHEAPHDR)+RUDP_PACKET+RUDP_PKT_SER)=ser;
	} ///////////////////////////////////////////////////////////////end Set_Packet_SER_FromHeap

////////////////////////////////////////////////////////////////////////////////////////////////////
// Set_Packet_SIZE_InHeap
// a wrapper to setting the packet size for a packet in the heap
// this is to help keep all the magic numbers in one place. 
// Here p<PACKETHEAPCOUNT because we are trying to set values in existing packet
////////////////////////////////////////////////////////////////////////////////////////////////////
void CUSock::Set_Packet_SIZE_InHeap(unsigned long p, unsigned short size)
{
	if (p>=PACKETHEAPCOUNT) 
	{
		ProtLog("CUSock:: Set_Packet_SIZE_InHeap: ERROR requested packet number too high.");
		exit(1);
		}
	*(unsigned short*)(packetheap+p*(PACKETSIZE+PACKETHEAPHDR)+RUDP_PACKET+RUDP_PKT_SIZE)=htons(size);
	} ////////////////////////////////////////////////////////////////end Set_Packet_SIZE_InHeap

////////////////////////////////////////////////////////////////////////////////////////////////////
// Get_Packet_SIZE_InHeap
// a wrapper to getting the packet size from a packet in the heap
// this is to help keep all the magic numbers in one place. 
// This returns the net form of the number
// Here p<PACKETHEAPCOUNT because we are trying to set values in existing packet
////////////////////////////////////////////////////////////////////////////////////////////////////
unsigned short CUSock::Get_Packet_SIZE_InHeap(unsigned long p)
{
	if (p>=PACKETHEAPCOUNT) 
	{
		ProtLog("CUSock:: Get_Packet_SIZE_InHeap: ERROR requested packet number too high.");
		exit(1);
		}
	return(ntohs(*(unsigned short*)(packetheap+p*(PACKETSIZE+PACKETHEAPHDR)+RUDP_PACKET+RUDP_PKT_SIZE)));
	} ////////////////////////////////////////////////////////////////end Get_Packet_SIZE_InHeap

////////////////////////////////////////////////////////////////////////////////////////////////////
// Zero_Block_InHeap
// a wrapper to setting the packet contents to zero for a packet in the heap
// this is to help keep all the magic numbers in one place. 
// Here p<PACKETHEAPCOUNT because we are trying to set values in existing packet
////////////////////////////////////////////////////////////////////////////////////////////////////
void CUSock::Zero_Block_InHeap(unsigned long p)
{
	if (p>=PACKETHEAPCOUNT) 
	{
		ProtLog("CUSock:: Set_Block_NEXT_InHeap: ERROR requested packet number too high.");
		exit(1);
		}
	memset((void*)(packetheap+p*(PACKETHEAPHDR+PACKETSIZE)),0,(PACKETHEAPHDR+PACKETSIZE));
	} /////////////////////////////////////////////////////////////////end Set_Block_NEXT_InHeap

////////////////////////////////////////////////////////////////////////////////////////////////////
// Set_Block_NEXT_InHeap
// a wrapper to setting the packet next for a packet in the heap
// this is to help keep all the magic numbers in one place. 
// Here p<PACKETHEAPCOUNT because we are trying to set values in existing packet
// Here v<=PACKETHEAPCOUNT because we can set the link the end value
////////////////////////////////////////////////////////////////////////////////////////////////////
void CUSock::Set_Block_NEXT_InHeap(unsigned long p, unsigned long v)
{
	if (p>=PACKETHEAPCOUNT) 
	{
		ProtLog("CUSock:: Set_Block_NEXT_InHeap: ERROR requested packet number too high.");
		exit(1);
		}
	if (v>PACKETHEAPCOUNT) 
	{
		ProtLog("CUSock:: Set_Block_NEXT_InHeap: ERROR setting next packet number too high.");
		exit(1);
		}
	*((unsigned long *)(packetheap+p*(PACKETHEAPHDR+PACKETSIZE)+RUDP_NEXT))=v;
	} /////////////////////////////////////////////////////////////////end Set_Block_NEXT_InHeap

////////////////////////////////////////////////////////////////////////////////////////////////////
// Set_Hash_NEXT_InHeap
// a wrapper to setting the hashnode next for a hashnode in the heap
// this is to help keep all the magic numbers in one place. 
// Here v can equal HASHHEAPCOUNT because we want to allow setting the end of list flag
// but p must always be less than HASHEAPCOUNT
////////////////////////////////////////////////////////////////////////////////////////////////////
void CUSock::Set_Hash_NEXT_InHeap(unsigned long p, unsigned long v)
{
	if (p>=HASHHEAPCOUNT) 
	{
		ProtLog("CUSock:: Set_Hash_NEXT_InHeap: ERROR requested hashnode number too high.");
		exit(1);
		}
	if (v>HASHHEAPCOUNT) 
	{
		ProtLog("CUSock:: Set_Hash_NEXT_InHeap: ERROR setting hashnode number too high.");
		exit(1);
		}
	*((unsigned long *)(hashheap+p*(HASHBLKSIZE)+RUDP_HSH_NXT))=v;
	} //////////////////////////////////////////////////////////////////end Set_Hash_NEXT_InHeap

////////////////////////////////////////////////////////////////////////////////////////////////////
// Get_Hash_NEXT_InHeap
// a wrapper to setting the hashnode next for a hashnode in the heap
// this is to help keep all the magic numbers in one place. 
// Here p<HASHHEAPCOUNT because we are trying to set values in existing hashnode
////////////////////////////////////////////////////////////////////////////////////////////////////
unsigned long CUSock::Get_Hash_NEXT_InHeap(unsigned long p)
{
	if (p>=HASHHEAPCOUNT) 
	{
		ProtLog("CUSock:: Get_Hash_NEXT_InHeap: ERROR requested hashnode number too high.");
		exit(1);
		}
	return(*((unsigned long *)(hashheap+p*(HASHBLKSIZE)+RUDP_HSH_NXT)));
	} //////////////////////////////////////////////////////////////////end Get_Hash_NEXT_InHeap

////////////////////////////////////////////////////////////////////////////////////////////////////
// Get_Hash_CONSINDEX_InHeap
// a wrapper to getting the hashnode session index for a hashnode in the heap
// this is to help keep all the magic numbers in one place. 
// Here p<HASHHEAPCOUNT because we are trying to set values in existing hashnode
////////////////////////////////////////////////////////////////////////////////////////////////////
unsigned long CUSock::Get_Hash_CONSINDEX_InHeap(unsigned long p)
{
	if (p>=HASHHEAPCOUNT) 
	{
		ProtLog("CUSock:: Get_Hash_CONSINDEX_InHeap: ERROR requested hashnode number too high.");
		exit(1);
		}
	return(*((unsigned long *)(hashheap+p*(HASHBLKSIZE)+RUDP_HSH_CONSIDX)));
	} /////////////////////////////////////////////////////////////end Get_Hash_CONSINDEX_InHeap

////////////////////////////////////////////////////////////////////////////////////////////////////
// Set_Hash_CONSINDEX_InHeap
// a wrapper to getting the hashnode session index for a hashnode in the heap
// this is to help keep all the magic numbers in one place. 
// Here p<HASHHEAPCOUNT because we are trying to set values in existing hashnode
////////////////////////////////////////////////////////////////////////////////////////////////////
void CUSock::Set_Hash_CONSINDEX_InHeap(unsigned long p, unsigned long v)
{
	if (p>=HASHHEAPCOUNT) 
	{
		ProtLog("CUSock:: Set_Hash_CONSINDEX_InHeap: ERROR requested hashnode number too high.");
		exit(1);
		}
	*((unsigned long *)(hashheap+p*(HASHBLKSIZE)+RUDP_HSH_CONSIDX))=v;
	} /////////////////////////////////////////////////////////////end Set_Hash_CONSINDEX_InHeap

////////////////////////////////////////////////////////////////////////////////////////////////////
// Get_Block_NEXT_InHeap
// a wrapper to getting the packet next for a packet in the heap
// this is to help keep all the magic numbers in one place. 
////////////////////////////////////////////////////////////////////////////////////////////////////
unsigned long CUSock::Get_Block_NEXT_InHeap(unsigned long p)
{
	if (p>=PACKETHEAPCOUNT) 
	{
		ProtLog("CUSock:: Get_Block_NEXT_InHeap: ERROR requested packet number too high.");
		exit(1);
		}
	return(*((unsigned long *)(packetheap+p*(PACKETHEAPHDR+PACKETSIZE)+RUDP_NEXT)));
	} /////////////////////////////////////////////////////////////////end Get_Block_NEXT_InHeap

////////////////////////////////////////////////////////////////////////////////////////////////////
// Get_Block_IP_InHeap
// a wrapper to getting the packet IP for a packet in the heap
// this is to help keep all the magic numbers in one place. 
////////////////////////////////////////////////////////////////////////////////////////////////////
in_addr_t CUSock::Get_Block_IP_InHeap(unsigned long p)
{
	if (p>=PACKETHEAPCOUNT) 
	{
		ProtLog("CUSock:: Get_Block_IP_InHeap: ERROR requested packet number too high.");
		exit(1);
		}
	return(*((in_addr_t *)(packetheap+p*(PACKETHEAPHDR+PACKETSIZE)+RUDP_IP)));
	} ///////////////////////////////////////////////////////////////////end Get_Block_IP_InHeap

////////////////////////////////////////////////////////////////////////////////////////////////////
// Set_Block_IP_InHeap
// a wrapper to setting the packet next for a packet in the heap
// this is to help keep all the magic numbers in one place. 
////////////////////////////////////////////////////////////////////////////////////////////////////
void CUSock::Set_Block_IP_InHeap(unsigned long p, in_addr_t addr)
{
	if (p>=PACKETHEAPCOUNT) 
	{
		ProtLog("CUSock:: Get_Block_IP_InHeap: ERROR requested packet number too high.");
		exit(1);
		}
	*((in_addr_t *)(packetheap+p*(PACKETHEAPHDR+PACKETSIZE)+RUDP_IP))=addr;
	} ///////////////////////////////////////////////////////////////////end Set_Block_IP_InHeap

////////////////////////////////////////////////////////////////////////////////////////////////////
// Get_Block_PORT_InHeap
// a wrapper to setting the packet next for a packet in the heap
// this is to help keep all the magic numbers in one place. 
////////////////////////////////////////////////////////////////////////////////////////////////////
in_port_t CUSock::Get_Block_PORT_InHeap(unsigned long p)
{
	if (p>=PACKETHEAPCOUNT) 
	{
		ProtLog("CUSock:: Get_Block_PORT_InHeap: ERROR requested packet number too high.");
		exit(1);
		}
	return(*((in_port_t *)(packetheap+p*(PACKETHEAPHDR+PACKETSIZE)+RUDP_PORT)));
	} /////////////////////////////////////////////////////////////////end Get_Block_PORT_InHeap

////////////////////////////////////////////////////////////////////////////////////////////////////
// Set_Packet_Values
// Set the header info for a packet
////////////////////////////////////////////////////////////////////////////////////////////////////
void CUSock::Set_Packet_Values( char *packet, unsigned long ser, unsigned long retry, unsigned long ses,
				unsigned char dc, unsigned char ack, unsigned short len)
{
	// set serial 
	*(unsigned long*)(packet+RUDP_PKT_SER)=ser;

	// retry 
	*(unsigned long*)(packet+RUDP_PKT_RET)=retry;

	// session
	*(unsigned long*)(packet+RUDP_PKT_SES)=ses;

	// data control flag
	*(unsigned char*)(packet+RUDP_PKT_FLG)=dc;

	// ack
	*(unsigned char*)(packet+RUDP_PKT_ACK)=ack;

	// length
	*(unsigned short*)(packet+RUDP_PKT_SIZE)=len;

	} /////////////////////////////////////////////////////////////////////end Set_Packet_Values

////////////////////////////////////////////////////////////////////////////////////////////////////
// IsCTRL
////////////////////////////////////////////////////////////////////////////////////////////////////
bool CUSock::IsCTRL(char *c)
{
	if (c[RUDP_PKT_FLG]==RUDP_DC_CTRL) return (true); 
	else return (false);
	} ////////////////////////////////////////////////////////////////////////////////end IsCTRL

////////////////////////////////////////////////////////////////////////////////////////////////////
// IsDATA
////////////////////////////////////////////////////////////////////////////////////////////////////
bool CUSock::IsDATA(char *c)
{
	if (c[RUDP_PKT_FLG]==RUDP_DC_DATA) return (true); 
	else return (false);
	} ////////////////////////////////////////////////////////////////////////////////end IsDATA

////////////////////////////////////////////////////////////////////////////////////////////////////
// IsRUDP
// This function is overloaded
// This form checks starting at the beginning of the packet
////////////////////////////////////////////////////////////////////////////////////////////////////
bool CUSock::IsRUDP(char *c)
{
	if ((c[0+RUDP_PKT_HDR]=='R') 
		&&(c[1+RUDP_PKT_HDR]=='U') 
		&&(c[2+RUDP_PKT_HDR]=='D') 
		&&(c[3+RUDP_PKT_HDR]=='P'))  
		return (true); 
	else  return (false); 
	} ////////////////////////////////////////////////////////////////////////////////end IsRUDP

////////////////////////////////////////////////////////////////////////////////////////////////////
// IsPing
////////////////////////////////////////////////////////////////////////////////////////////////////
bool CUSock::IsPing(char *r, int len)
{
	if (*(unsigned char*)(r+RUDP_PKT_DATA)==RUDP_CTRL_PNG) return (true);
	else return (false);
	} ////////////////////////////////////////////////////////////////////////////////end IsPing

////////////////////////////////////////////////////////////////////////////////////////////////////
// IsComplete
////////////////////////////////////////////////////////////////////////////////////////////////////
bool CUSock::IsComplete(char *r, int len)
{
	if (*(unsigned char*)(r+RUDP_PKT_DATA)==RUDP_CTRL_COM) return (true);
	else return (false);
	} ////////////////////////////////////////////////////////////////////////////end IsComplete

////////////////////////////////////////////////////////////////////////////////////////////////////
// IsConnect
////////////////////////////////////////////////////////////////////////////////////////////////////
bool CUSock::IsConnect(char *r, int len)
{
	if (*(unsigned char*)(r+RUDP_PKT_DATA)==RUDP_CTRL_CON) return (true);
	else return (false);
	} /////////////////////////////////////////////////////////////////////////////end IsConnect

////////////////////////////////////////////////////////////////////////////////////////////////////
// IsDisconnect
////////////////////////////////////////////////////////////////////////////////////////////////////
bool CUSock::IsDisconnect(char *r, int len)
{
	if (*(unsigned char*)(r+RUDP_PKT_DATA)==RUDP_CTRL_DIS) return (true);
	else return (false);
	} //////////////////////////////////////////////////////////////////////////end IsDisconnect

////////////////////////////////////////////////////////////////////////////////////////////////////
// IsDisconnectComplete
////////////////////////////////////////////////////////////////////////////////////////////////////
bool CUSock::IsDisconnectComplete(char *r, int len)
{
	if (*(unsigned char*)(r+RUDP_PKT_DATA)==RUDP_CTRL_DISCOM) return (true);
	else return (false);
	} //////////////////////////////////////////////////////////////////end IsDisconnectComplete

////////////////////////////////////////////////////////////////////////////////////////////////////
// IsRetrans
////////////////////////////////////////////////////////////////////////////////////////////////////
bool CUSock::IsRetrans(char *r, int len)
{
	if (*(unsigned char*)(r+RUDP_PKT_DATA)==RUDP_CTRL_RET) return (true);
	else return (false);
	} /////////////////////////////////////////////////////////////////////////////end IsRetrans

////////////////////////////////////////////////////////////////////////////////////////////////////
// IsAck
// overloaded
////////////////////////////////////////////////////////////////////////////////////////////////////
bool CUSock::IsAck(unsigned long p)
{

	if (*(unsigned char*)(packetheap+p*(PACKETSIZE+PACKETHEAPHDR)+RUDP_PACKET+RUDP_PKT_ACK)) return (true);
	else return (false);
	} /////////////////////////////////////////////////////////////////////////////////end IsAck

////////////////////////////////////////////////////////////////////////////////////////////////////
// IsAck
// overloaded
////////////////////////////////////////////////////////////////////////////////////////////////////
bool CUSock::IsAck(char *r, int len)
{

	if (*(unsigned char*)(r+RUDP_PKT_DATA)==RUDP_CTRL_ACK) return (true);
	else return (false);
	} /////////////////////////////////////////////////////////////////////////////////end IsAck

////////////////////////////////////////////////////////////////////////////////////////////////////
// IsQEmpty
// the reason we are doing this is to standardize how we check for queue empty condition
////////////////////////////////////////////////////////////////////////////////////////////////////
bool CUSock::IsRecvQEmpty()
{
	if (recvqueue==PACKETHEAPCOUNT) return (true);
	else return (false);
	} /////////////////////////////////////////////////////////////////////////////////end IsAck

////////////////////////////////////////////////////////////////////////////////////////////////////
// ProtLog  
// Since some internal functions run by both threads,
// we need a way to determine which thread is running when logging. 
////////////////////////////////////////////////////////////////////////////////////////////////////
void CUSock::ProtLog(char * s)
{
	if (LogFunc) (*LogFunc)(s);
	} ///////////////////////////////////////////////////////////////////////////////end ProtLog

////////////////////////////////////////////////////////////////////////////////////////////////////
// LocalLock
// This is a wrapper for pthread lock that can print tid if required
////////////////////////////////////////////////////////////////////////////////////////////////////
int CUSock::LocalLock(pthread_mutex_t *mutex, char *who)
{
	return (MyLock(mutex,true,who));
        } /////////////////////////////////////////////////////////////////////////////end LocalLock

////////////////////////////////////////////////////////////////////////////////////////////////////
// LocalUnLock
// This is a wrapper for pthread lock that can print tid if required
////////////////////////////////////////////////////////////////////////////////////////////////////
int CUSock::LocalUnLock(pthread_mutex_t *mutex, char * who)
{
	return (MyUnLock(mutex,true,who));
        } ///////////////////////////////////////////////////////////////////////////end LocalUnLock

////////////////////////////////////////////////////////////////////////////////////////////////////
// CUSock Constructor
////////////////////////////////////////////////////////////////////////////////////////////////////
CUSock::CUSock()
{
	// set state
	logcount=0;
	state=SOCK_ST_NOTINIT;
	he=0;
//	sockfd=0;			// the socket file descriptor
	sockfd=-1;			// the socket file descriptor
        port=0;				// the port to connect on
	pktcurr=0;			// remember that we start searching from 0 for free blocks	
	hshcurr=0;
	sendqueue=PACKETHEAPCOUNT;
	retryqueue=PACKETHEAPCOUNT;
	recvqueue=PACKETHEAPCOUNT;	
	recvqueuetail=PACKETHEAPCOUNT;
	packetheap=0;
	hashheap=0;
	waitqueue=PACKETHEAPCOUNT;
	waitqueuetail=PACKETHEAPCOUNT;
	allochead=HASHHEAPCOUNT;
	alloctail=HASHHEAPCOUNT;
	freehead=HASHHEAPCOUNT;
	freetail=HASHHEAPCOUNT;
	portmanthread=0;
	LogFunc=0;
	} ///////////////////////////////////////////////////////////////////////////end Constructor

////////////////////////////////////////////////////////////////////////////////////////////////////
// CUSock Destructor
////////////////////////////////////////////////////////////////////////////////////////////////////
CUSock::~CUSock()
{
	// delete the memory pointed to by buf
	if (LogFunc) (*LogFunc)("CUSock:: Destructor: Entered. ");
//	DeInit();
	if (LogFunc) (*LogFunc)("CUSock:: Destructor: After DeInit call. ");
	if (rawbuf!=NULL) delete [] rawbuf;
	if (LogFunc) (*LogFunc)("CUSock:: Destructor: After Delete rawbuf. ");


	// maybe they dont want us to free it ?
	if (LogFunc) (*LogFunc)("CUSock:: Destructor: After Delete he. ");
	} ////////////////////////////////////////////////////////////////////////////end Destructor

////////////////////////////////////////////////////////////////////////////////////////////////////
// CUSock::Init
// Initialize the instance
// Because allocate mem here, need to release it in close
// We receive the data into a local buffer first because we call receive here
////////////////////////////////////////////////////////////////////////////////////////////////////
//bool CUSock::Init(pthread_mutexattr_t *pm_attr) // just for now. we will put this back after thread problem debugged
bool CUSock::Init()
{

	ProtLog("CUSock:: Init: Entered.\n");
	skmutex       = new pthread_mutex_t;
	recv_condvar  = new pthread_cond_t;

	if (!skmutex) return (false);
	if (!recv_condvar) return (false);

//#ifdef SOLARIS
//	pthread_mutexattr_settype(pm_attr,PTHREAD_MUTEX_ERRORCHECK);
//pthread_condattr_t c_attr;
//	pthread_condattr_init(&c_attr);
//#endif

	// linux always returns 0, check these for solaris
//	if (pthread_mutex_init(skmutex, pm_attr))       return (false);	// for solaris
	InitMutex(skmutex);

//#ifdef LINUX
	if (pthread_cond_init(recv_condvar,NULL))       return (false);	// for linux
//#endif	

//#ifdef SOLARIS
//	pthread_condattr_setpshared(&c_attr, PTHREAD_PROCESS_SHARED);	// should take this out, dont think i need it
//	if (pthread_cond_init(recv_condvar,NULL))       return (false);	// for solaris
//#endif

	// Initialize the socket
//	sockfd=0;
	sockfd=-1;
	state=SOCK_ST_INIT;

	tv.tv_sec=0;                       // wait up to 1 secs <<<<======TUNE THIS
	tv.tv_usec=10000;			// right now start waiting at .01 seconds

	ProtLog("CUSock:: Init: Exited.\n");

	initthread=pthread_self();

	return (true);
	} //////////////////////////////////////////////////////////////////////////////////end Init

////////////////////////////////////////////////////////////////////////////////////////////////////
// RecvWait
// Puts my thread asleep to wait on a signal from recv_condvar
// FIXME FLAGS FUNCTIONALITY TO BE IMPLEMENTED
////////////////////////////////////////////////////////////////////////////////////////////////////
void CUSock::RecvWait(unsigned int flags)
{
	// check cond var
//	if (recvqueue==PACKETHEAPCOUNT) pthread_cond_wait(recv_condvar,skmutex);
	if (IsRecvQEmpty()) pthread_cond_wait(recv_condvar,skmutex);

	} //////////////////////////////////////////////////////////////////////////////end RecvWait

////////////////////////////////////////////////////////////////////////////////////////////////////
// RecvSignal
// Tells all threads who are waiting on recv_condvar to wake up
////////////////////////////////////////////////////////////////////////////////////////////////////
void CUSock::RecvSignal()
{
	// check cond var
//	if (recvqueue!=PACKETHEAPCOUNT) 
	if (!IsRecvQEmpty()) 
	{
		if (loglevel>=6) ProtLog("CUSock:: RecvSignal: Before Signal.");
		if (pthread_cond_broadcast(recv_condvar)==EINVAL)
		{
			ProtLog("CUSock:: RecvSignal: ERROR UNINIT CONDVAR .");
			}
		}

	} ////////////////////////////////////////////////////////////////////////////end RecvSignal

////////////////////////////////////////////////////////////////////////////////////////////////////
// Create
////////////////////////////////////////////////////////////////////////////////////////////////////
bool CUSock::Create(struct sockaddr_in *s, socklen_t len)
{

char    buf[BUFFSIZE];
char	dst[50];

        if (state!=SOCK_ST_INIT) return(false);        
	if ((sockfd = socket(AF_INET, SOCK_DGRAM, 0)) == -1)
        {
		ProtLog("CUSock:: Create: Create failed.");
                return (false);
                }

	addr.sin_port=s->sin_port;
	addr.sin_family=AF_INET;
	addr.sin_addr.s_addr=s->sin_addr.s_addr;
	addrlen=sizeof(struct sockaddr_in);

	state=SOCK_ST_CREATED;
	if (loglevel>=2)
	{
		snprintf(buf,BUFFSIZE,"CUSock:: Create: port=%d family=%d ip=%s sockfd=%d addrlen=%d Exited.",
			ntohs(addr.sin_port),
			addr.sin_family,
			inet_ntop(AF_INET,(void*)&(addr.sin_addr.s_addr),dst,50),
			sockfd,
			addrlen);
		ProtLog(buf);
		}
	return (true);
	} ////////////////////////////////////////////////////////////////////////////////end Create

////////////////////////////////////////////////////////////////////////////////////////////////////
// AllocHsh
// INTERNAL THREAD
// allocation maps are being redone
// 0long => unallocated  
// 1long => allocated
////////////////////////////////////////////////////////////////////////////////////////////////////
unsigned long CUSock::AllocHsh()
{	
unsigned long 	i;

	if (loglevel>=1) ProtLog("CUSock:: AllocHsh: Entered.");
	// here we find a hashnode that is unused in the heap
	// note we start at 0, end of chain is HASHHEAPCOUNT 
	// now check the alloc map for free packets
	// entries of 		 0 => unallocated
	// 		        1l => allocated
	for (i=hshcurr; i<HASHHEAPCOUNT; i++) if (hshheapmap[i]==0l) goto AllocHshDone;
	for (i=0; i<hshcurr; i++) 		if (hshheapmap[i]==0l) goto AllocHshDone;

	// if we get here then no slot was found, return failure
	ProtLog("CUSock:: AllocHsh: Exited failure. ");
	return (HASHHEAPCOUNT);

AllocHshDone:
	// Ok found a packet, make it the end of the chain for now to at least alloc it
	hshheapmap[i]=1l; 	// One long
	hshcurr=i;		// remember where we left off last time to start looking there again

	// mark heap block with HASHHEAPCOUNT in the 'next' field for being end of chain
	Set_Hash_NEXT_InHeap(hshcurr, HASHHEAPCOUNT);
	if (loglevel>=1) ProtLog("CUSock:: AllocHsh: Exited ok.");
	return (i);
	
	} //////////////////////////////////////////////////////////////////////////////end AllocHsh

////////////////////////////////////////////////////////////////////////////////////////////////////
// AllocPkt
// This is called by both threads internal, and external so it needs to be carefull which logging
// it uses.
////////////////////////////////////////////////////////////////////////////////////////////////////
int CUSock::AllocPkt()
{	
unsigned long 	i;
char		buf[BUFFSIZE];

	if (loglevel>=1) ProtLog("CUSock:: AllocPkt: Entered.");

	// here we find a packet that is unused in the heap
	// it means end on chain already
	// now check the alloc map for free packets
	// entries of 		 0 => unallocated
	// 		         1 => allocated
	for (i=pktcurr; i<PACKETHEAPCOUNT; i++) if (pktheapmap[i]==0l) goto AllocPktDone;
	for (i=0; i<pktcurr; i++) 		if (pktheapmap[i]==0l) goto AllocPktDone;

	// if we get here then no slot was found, return failure
	ProtLog("CUSock:: AllocPkt: Exit RUDP_ERR_NO_PKT_HEAP.");
	return (PACKETHEAPCOUNT);

AllocPktDone:
	if (loglevel>=2) 
	{
		snprintf(buf,BUFFSIZE,"CUSock:: AllocPkt: Found a pkt. pktcurr='%lu'",i);
		ProtLog(buf);
		}

	// Ok found a packet alloc it
	pktheapmap[i]=1l; 	// One long
	pktcurr=i;		// remember where we left off last time to start looking there again

	// mark heap block with PACKETHEAPCOUNT in the 'next' field for being end of chain
	Set_Block_NEXT_InHeap(pktcurr, PACKETHEAPCOUNT);
	if (loglevel>=1) ProtLog("CUSock:: AllocPkt: Exited.");
	return (i);
	
	} //////////////////////////////////////////////////////////////////////////////end AllocPkt

////////////////////////////////////////////////////////////////////////////////////////////////////
// DeAllocPkt
// This is called by both threads internal, and external so it needs to be carefull which logging
// it uses.
////////////////////////////////////////////////////////////////////////////////////////////////////
int CUSock::DeAllocPkt(unsigned long p)
{	
	// Here we deallocate a packet block in heap
	// remember we start at 0
	if (p>=PACKETHEAPCOUNT) return (RUDP_ERR_BAD_HEAP_NUM);
	pktheapmap[p]=0l;

	// mark heap block with PACKETHEAPCOUNT in the 'next' field for being end of chain
	Set_Block_NEXT_InHeap(p, PACKETHEAPCOUNT);
	return (RUDP_OK);	
	
	} ////////////////////////////////////////////////////////////////////////////end DeAllocPkt

////////////////////////////////////////////////////////////////////////////////////////////////////
// DeAllocHsh
////////////////////////////////////////////////////////////////////////////////////////////////////
int CUSock::DeAllocHsh(unsigned long p)
{	
	// Here we deallocate a hash block in heap
	// remember we start at 0
	if (p>=HASHHEAPCOUNT) return (RUDP_ERR_BAD_HEAP_NUM);
	hshheapmap[p]=0l;

	// mark hashnode with HASHHEAPCOUNT in the 'next' field for being end of chain
	Set_Hash_NEXT_InHeap(p, HASHHEAPCOUNT);
	return (RUDP_OK);	
	
	} ////////////////////////////////////////////////////////////////////////////end DeAllocHsh

////////////////////////////////////////////////////////////////////////////////////////////////////
// AddToSendQueue
// pkt IS AN INDEX NOT A POINTER
// remember that the 0 block is OK
// the NULL is PACKETHEAPCOUNT
////////////////////////////////////////////////////////////////////////////////////////////////////
int CUSock::AddToSendQueue(unsigned long pkt)
{
unsigned long 	p,tp;

	// Ok just append packet to linked list for sendq
	// first check that packet index is in range
	if (pkt>=PACKETHEAPCOUNT) return (RUDP_ERR_BAD_HEAP_NUM);	
	p=sendqueue;
	tp=sendqueue;

	// handle empty list
	if (sendqueue==PACKETHEAPCOUNT)
	{
		// insert first item
		sendqueue=pkt;
		Set_Block_NEXT_InHeap(pkt, PACKETHEAPCOUNT);
		return (RUDP_OK);
		} 

	// find end of list
	while (p!=PACKETHEAPCOUNT)
	{
		// go get next pointer
		tp=p; 	
		p=Get_Block_NEXT_InHeap(p);
		}
	if (tp>=PACKETHEAPCOUNT) return (RUDP_ERR_BAD_HEAP_NUM);	

	// append packet
	Set_Block_NEXT_InHeap(tp, pkt);

	// terminate the linked list
	Set_Block_NEXT_InHeap(pkt, PACKETHEAPCOUNT);
	return (RUDP_OK);

	} ////////////////////////////////////////////////////////////////////////end AddToSendQueue

////////////////////////////////////////////////////////////////////////////////////////////////////
// GetSerial 
// this performs a hash lookup based on high order byte (least sig byte(network byte order)) of
// ip address. also uses session, to return serial for packet. also returns port.
// it then increments session by amount in incr
// IT LOOKS LIKE PORT AND IP ADDDRESS INFO IS STORED IN NETWORK FORM IN CONNECTIONS ARRAY
// NOTE When running as client there will not BE a hash, so we just look in connection record
////////////////////////////////////////////////////////////////////////////////////////////////////
unsigned long CUSock::GetSerial(unsigned long *serial, in_port_t *port, in_addr_t ip, unsigned long ses, int incr)
{
unsigned char	hashkey;
unsigned long	hsh;

	if (loglevel>=1) ProtLog("CUSock:: GetSerial: Entered.");
	// handle client socket case
	if (sockettype=='c')
	{
		*serial=connections[0].serial;
		connections[0].serial+=incr;
		*port=connections[0].port;
		if (loglevel>=1) ProtLog("CUSock:: GetSerial: CLIENT Serial found, exiting OK.");
		return (RUDP_OK);
		}
	
	// get the index to the correct bucket 
	hashkey= *((unsigned char *)(&ip)+3);

	// get the buckets pointer to the linked list to search
	hsh=sessionhash[hashkey];		

	// now that we have the correct ip bucket search for the session that matches
	while (hsh!=HASHHEAPCOUNT)
	{
	unsigned long index;

		if (loglevel>=3) ProtLog("CUSock:: GetSerial: while loop.");
		// now see if this is the session we are looking for
		// DEBUG also make sure we have the correct IP, cause we could have two IP with same session
		index=Get_Hash_CONSINDEX_InHeap(hsh);
		if ((connections[index].session==ses) && (connections[index].IP==ip))
		{
			// this is the right session so get the serial
			*serial=connections[index].serial;
			*port=connections[index].port;
			connections[index].serial+=incr;
			if (loglevel>=1) ProtLog("CUSock:: GetSerial: SERVER Serial found, exiting OK.");
			return (RUDP_OK);
			}
		hsh=Get_Hash_NEXT_InHeap(hsh);
		if (loglevel>=3) ProtLog("CUSock:: GetSerial: after point to next.");
		}
	if (loglevel>=1) ProtLog("CUSock:: GetSerial: Serial NOT found, exiting.");
	return (RUDP_ERR_SER_NOT_FOUND);
	} /////////////////////////////////////////////////////////////////////////////end GetSerial	

////////////////////////////////////////////////////////////////////////////////////////////////////
// Send
// This is an over load to handle the case for the CLIENT.
// The protocol will assign session ID to outgoing packets for the client.
// External calling threads will assign session IDs for packets on the server.
// The end of list value for sessions is MAX_SES_PER_IP, ie valid sessions for each IP
// run from 0 to MAX_SES_PER_IP-1
////////////////////////////////////////////////////////////////////////////////////////////////////
int CUSock::Send(char * sendbuf,int size,int flags)
{
	return (Send(sendbuf,size,flags,NULL,0,MAX_SES_PER_IP));
	} //////////////////////////////////////////////////////////////////////////////////end Send

////////////////////////////////////////////////////////////////////////////////////////////////////
// Send   SERVER
// assume external thread supplies following
// 	- session ID	(if server)
// 	- IP address
// 	- packet size
// 	- packet data
// protocol supplies	 
// 	- session ID	(if client)
// 	- serial
// 	- port
// 	- retry num
// 	- data/ctrl
// 
// For the server the session (and IP)helps us get the serial and the port
// For the client the session just gets us the serial, the port is fixed 
// We do all the figuring here, as the SendData will just dumbly grab a block from the queue and send it
////////////////////////////////////////////////////////////////////////////////////////////////////
int CUSock::Send(char * sendbuf,int size,int flags,struct sockaddr_in *dest, socklen_t destlen, unsigned long session)
{
unsigned long	p,serial;
int		result;
int		result1;
in_port_t	port;
char		buf[BUFFSIZE];

	// OK Here all we do is lock the mutex and insert the packet into the send queue.
	// Lock mutex
	if (loglevel>=1) ProtLog("CUSock:: Send: Entered.");
	if (loglevel>=2) 
	{
		snprintf(buf,BUFFSIZE,"CUSock:: Send: Requesting lock. MUTEX=%lu", (unsigned long)skmutex);
		ProtLog(buf);
		}
	
	LocalLock(skmutex,"CUSock:: Send");

	// Allocate a packet slot from heap for send queue
	p=AllocPkt();

	if (p==PACKETHEAPCOUNT) 	// remember that the 0 packet is a valid packet to use
	{
		LocalUnLock(skmutex,"CUSock:: Send");
		if (loglevel>=1) ProtLog("CUSock:: Send: Unlock, exited no pkt available.");
		return (RUDP_ERR_NO_PKT_HEAP);
		}

	// ok got a packet, now zero it.
	Zero_Block_InHeap(p);
	
	if ((result1=AddToSendQueue(p))!=RUDP_OK) 
	{
		LocalUnLock(skmutex,"CUSock:: Send");
		ProtLog("CUSock:: Send: Unlock, exited failed adding to sendqueue.");
		return (result1);
		}

	// write RUDP packet header
	Set_Packet_Header(p);

	// Copy the IP addr to the packet heap block
	// IP given to us in net form, same as port
	if (sockettype=='s') Set_Block_IP_InHeap(p, dest->sin_addr.s_addr);
	else Set_Block_IP_InHeap(p, connections[0].IP);
	
	// Copy send buf to the queue 
	Set_Packet_DATA_InHeap(p, sendbuf, size);
	if (loglevel>=1) ProtLog("CUSock:: Send: After memcpy. ");

 	//////////////////////////////////////////
	//NOTE that when packet is in the HEAP 	all "numbers" are in NETWORK form
	//And when we add or subtract numbers HERE on this machine
	//the numbers are in HOST form, but are restored in heap in network form
	//////////////////////////////////////////
	
	// now we must fill in the rest of the data
	// get serial, GetSerial increments by amount in third parm 
	if (sockettype=='s') result=GetSerial(&serial, &port, dest->sin_addr.s_addr, session, 1);
	else result=GetSerial(&serial, &port, (in_addr_t)0, session, 1);
	if (loglevel>=1) ProtLog("CUSock:: Send: After GetSerial.");
	if (result==RUDP_OK) 
	{
		Set_Packet_SER_InHeap(p, htonl(serial));
		}
	else 
	{
		LocalUnLock(skmutex,"CUSock:: Send");
		ProtLog("CUSock:: Send: Unlock, exited RUDP_ERR_SER_NOT_FOUND.");
		return RUDP_ERR_SER_NOT_FOUND;
		}
	if (sockettype=='s') Set_Block_PORT_InHeap(p, port);		//if server, use the port stored in 
									//connection array 
	else Set_Block_PORT_InHeap(p, connections[0].port);		// if client send it to the right port
	
	// need to add length	
	Set_Packet_SIZE_InHeap(p, size+RUDP_PKT_DATA);		// conversion to net form done inside func
	
	// add retry
	Set_Packet_RET_InHeap(p,htonl(0));

	// The session var already filled in by external thread if server, 
	// because the server knows who it wants to 
	// talk to. But set it if client cause there is just one session
	if (sockettype=='c')
	{
		if (session!=MAX_SES_PER_IP)
		ProtLog("CUSock:: Send: Internal ERROR. NOT MAX_SES_PER_IP with client.");
		Set_Packet_SES_InHeap(p,htonl(connections[0].session));
		}
	else
	{
		if (session==MAX_SES_PER_IP)
		ProtLog("CUSock:: Send: Internal ERROR. MAX_SES_PER_IP with server.");
		Set_Packet_SES_InHeap(p,htonl(session));
		}

	
	// set data flag
	Set_Packet_FLG_InHeap(p, RUDP_DC_DATA);

	// ack flag set by external thread(for now require ack)
	Set_Packet_ACK_InHeap(p);
	 
	// Unlock before return
	LocalUnLock(skmutex,"CUSock:: Send");
	if (loglevel>=2) 
	{
		snprintf(buf,BUFFSIZE,"CUSock:: Send: Released lock. MUTEX=%lu", (unsigned long)skmutex);
		ProtLog(buf);
		}

	if (loglevel>=1) ProtLog("CUSock:: Send: Exited.");
	return RUDP_OK;
	} //////////////////////////////////////////////////////////////////////////////////end Send

////////////////////////////////////////////////////////////////////////////////////////////////////
// Recv
// This is an over loaded function
// This form is used by clients. The session is maintained in the protocol
////////////////////////////////////////////////////////////////////////////////////////////////////
int CUSock::Recv(char **packet, unsigned int flags)// struct sockaddr_in *dest, socklen_t *destlen)
{
	return(Recv(packet, flags, NULL, NULL, NULL));
	} //////////////////////////////////////////////////////////////////////////////////end Recv

////////////////////////////////////////////////////////////////////////////////////////////////////
// Recv
// This is an over loaded function
// This form is used by servers. The session is returned per received packet
// in the session var.
// Remember that the session we return will be in HOST form
// the session that is in the heap is in NET form
////////////////////////////////////////////////////////////////////////////////////////////////////
int CUSock::Recv(char **packet, unsigned int flags, struct sockaddr_in *dest, socklen_t *destlen, unsigned long *session)
{
unsigned short	size;
unsigned long	tp;
char		buf[BUFFSIZE];

	// OK Here all we do is check the recv queue for packets and return in order.
	// The port monitor will be responsible for checking to ensure that packets are queued
	// in order in that no serialized packets are missing.
	ProtLog("CUSock:: Recv: Entering, wait on recvqueue.\n");
	LocalLock(skmutex,"CUSock:: Recv Lock");

	RecvWait(flags);
	if (IsRecvQEmpty()) // this is the end of list special value
	{
		// nothing in queue
		if (packet) *packet=0;		// return no packet 
		if (destlen) *destlen=0;
		LocalUnLock(skmutex,"CUSock:: Recv UnLock");
		ProtLog("CUSock:: Recv: Exited.  NO PACKETS RECEIVED.\n");
		return (0);	// return zero bytes in packet
		}	
	
	// ok we have a packet on queue
	// alloc some mem and copy the packet to new buffer, pass back the buffer
	// dealloc the block from the recv queue
	// return the size
	*packet = new char [PACKETSIZE];    
	if (loglevel>=2) {snprintf(buf,BUFFSIZE,"CUSock:: Recv: *packet = %08lu ",(unsigned long)(packet)); ProtLog(buf);}
	size=Get_Packet_SIZE_InHeap(recvqueue)-RUDP_PKT_DATA;	// gives host form.
	Get_Packet_DATA_InHeap(recvqueue,*packet,size);
	if (loglevel>=1) ProtLog("CUSock:: Recv: After Get_Packet_DATA_InHeap().");
	if (sockettype=='s') {
		dest->sin_addr.s_addr=Get_Block_IP_InHeap(recvqueue);
		*destlen=sizeof(struct sockaddr_in);
		*session=ntohl(Get_Packet_SES_InHeap(recvqueue)); 
		if (loglevel>=2) {snprintf(buf,BUFFSIZE,"CUSock:: Recv: Recvqueue = %lu session=%lu",recvqueue,*session); 	ProtLog(buf);}
		}
	else 
	{
		if (loglevel>=2)
		{
			snprintf(buf,BUFFSIZE,"CUSock:: Recv: Recvqueue = %lu ",recvqueue); 
			ProtLog(buf);
			}
		}

	// now pull out of queue
	tp=recvqueue;
	recvqueue=Get_Block_NEXT_InHeap(tp);
	if (loglevel>=2) {snprintf(buf,BUFFSIZE,"CUSock:: Recv: Next in recvqueue = %lu ",recvqueue); ProtLog(buf);}

	// if there was only one item in list
	if (tp==recvqueuetail) 
	{
		recvqueuetail=PACKETHEAPCOUNT;
		recvqueue=PACKETHEAPCOUNT;
		}

	// give back heap block	
	DeAllocPkt(tp);
	LocalUnLock(skmutex,"CUSock:: Recv UnLock");
	if (loglevel>=1) ProtLog("CUSock:: Recv: Exited. GOT PACKETS !!!");
	return ((int)size);
	} //////////////////////////////////////////////////////////////////////////////////end Recv

////////////////////////////////////////////////////////////////////////////////////////////////////
// CUSock::Bind
// Bind the socket to the ip to listen on 
// do i need to lock and unlock here too ?
////////////////////////////////////////////////////////////////////////////////////////////////////
bool CUSock::Bind()
{
char	buf[BUFFSIZE];

	if (state!=SOCK_ST_CREATED) return(false);

	snprintf(buf, BUFFSIZE, "CUSock:: Bind: Binding port = '%d' sockfd=%d addrlen=%d \n",
		ntohs(addr.sin_port),
		sockfd,
		addrlen);
	ProtLog(buf);
	if (bind(sockfd, (const sockaddr*)&addr, addrlen) == -1)
	{
		ProtLog("CUSock:: Bind: Bind failed.\n");
		return (false);
		}

	state=SOCK_ST_BOUND;
	ProtLog("CUSock:: Bind: Exited.\n");
	return (true);
	} //////////////////////////////////////////////////////////////////////////////////end Bind

////////////////////////////////////////////////////////////////////////////////////////////////////
// SetLogFunc
////////////////////////////////////////////////////////////////////////////////////////////////////
void CUSock::SetLogFunc(void (*pfunc)(char*))
{

	// save the function pointer to be used for logging
	LogFunc=pfunc;
	} ////////////////////////////////////////////////////////////////////////////end SetLogFunc

////////////////////////////////////////////////////////////////////////////////////////////////////
// SetDisConnectFunc
////////////////////////////////////////////////////////////////////////////////////////////////////
void CUSock::SetDisConnFunc(int (*pfunc)(connecttype*))
{

	// Register the call be for notifying when we have a new connection
	DisConnectFunc=pfunc;
	} /////////////////////////////////////////////////////////////////////end SetDisConnectFunc

////////////////////////////////////////////////////////////////////////////////////////////////////
// SetConnectFunc
////////////////////////////////////////////////////////////////////////////////////////////////////
void CUSock::SetConnectFunc(int (*pfunc)(connecttype*))
{

	// Register the call be for notifying when we have a new connection
	ConnectFunc=pfunc;
	} ////////////////////////////////////////////////////////////////////////end SetConnectFunc

////////////////////////////////////////////////////////////////////////////////////////////////////
// SendData
// sends all packets on sendqueue
// then adds each to the retry list if ack is set.
////////////////////////////////////////////////////////////////////////////////////////////////////
int CUSock::SendData()
{
unsigned long		p,tp,tmp;
in_addr_t		ip;
in_port_t		port;
void 			*mesg;
int 			n;
struct sockaddr_in	cliaddr;
char			buf[BUFFSIZE];
//int			result;

	// send all the packets on the sendqueue
	// first better make sure no one else is messing with my data
	LocalLock(skmutex,"CUSock:: SendData");
	p=sendqueue;

	while (p!=PACKETHEAPCOUNT)
	{
		snprintf(buf,BUFFSIZE,"CUSock:: SendData: sendqueue=%lu\n", sendqueue);
		ProtLog(buf);
		// get IP address stored in block
		ip=Get_Block_IP_InHeap(p);
		port=Get_Block_PORT_InHeap(p);

		// send data there
		mesg=Get_Packet_DATA_Ptr_InHeap(p); 
		n=Get_Packet_SIZE_InHeap(p);		// conversion from net form done in func

		// remember that we are sending a lot more info than just the external thread's payload 
		// data, we are sending our protocol stuff
		cliaddr.sin_family=AF_INET;
		cliaddr.sin_addr.s_addr=ip;
		cliaddr.sin_port=port;
		snprintf(buf,BUFFSIZE,"CUSock:: SendData: CALLING SENDTO IP=%s Port=%d, length=%d str='%s'\n",
				inet_ntoa(cliaddr.sin_addr),
				ntohs(cliaddr.sin_port),
				n,
				(char *)(mesg)+RUDP_PKT_DATA);
		ProtLog(buf);
		SendTo(mesg,n,&cliaddr,sizeof(struct sockaddr_in));

		// take it off sendqueue
		tp=p;			// keep track of packet to move

		// point to next packet to consider 
		p=Get_Block_NEXT_InHeap(p);

		// make trail packet a list by himself
		ProtLog("CUSock:: SendData: Make pkt a list by itself.\n");
		Set_Block_NEXT_InHeap(tp, PACKETHEAPCOUNT);
		sendqueue=p;		// have sendqueue point to next in list 

		// add packet to retryqueue if ack is set then return
		ProtLog("CUSock:: SendData: Add to retryqueue.\n");
		if (IsAck(tp))
		{
			// ok, an ack is requested, so add it to retryqueue
			ProtLog("CUSock:: SendData: ACK requested.\n");
			tmp=retryqueue;
			retryqueue=tp;
			Set_Block_NEXT_InHeap(tp, tmp);
			// now do next loop
			} 
		else
		{
			// ok, no ack needed, free the tp block.
			DeAllocPkt(tp);
			}	
		}	
	LocalUnLock(skmutex,"CUSock:: SendData");

	return (RUDP_OK);
	} //////////////////////////////////////////////////////////////////////////////end SendData

////////////////////////////////////////////////////////////////////////////////////////////////////
// RecvData
// if port is in listen state it is the server so check for incoming connection requests
// if receives them then add to session array. 
////////////////////////////////////////////////////////////////////////////////////////////////////
int CUSock::RecvData()
{
int			pktlen;		// the amount of raw data received
struct sockaddr_in	cliaddr;	// the place to put the address of sender
socklen_t		destlen;
//char 			*c;
unsigned long		connectionindex;
char			buf[BUFFSIZE];
fd_set  		rfds;
int     		retval;
struct  timeval  	temptv;
//int			result;

	// first better make sure no one else is messing with my data
	LocalLock(skmutex,"CUSock:: RecvData");

	// check for any data on port
	// here what we need to do is do a select so we can have a time out.
	// we will adjust the timeout so that we get optimal reponse and minimal usage

	// This is to try to ensure that socket is ready for reading
	temptv.tv_sec =tv.tv_sec; 
	temptv.tv_usec=tv.tv_usec;
	FD_ZERO(&rfds);

	FD_SET(sockfd,&rfds);	// note this is the only descriptor being watched
	retval=select(sockfd+1, &rfds, NULL, NULL, &temptv);
        if (retval==0)
	{
		timedout=true;
//		ProtLog("CUSock:: RecvData: TIMEOUT.");
		goto RecvDataExit;
		}

        if (retval<0)
	{
		timedout=false;
		errorcode=errno;
		snprintf(buf,BUFFSIZE,"CUSock:: RecvData: ERROR on select sockfd=%d sec=%lu usec=%lu",
			sockfd,
			(unsigned long)temptv.tv_sec,
			(unsigned long)temptv.tv_usec);
		ProtLog(buf);
		PError(errno);

		goto RecvDataExit;
		}

	// if got something
	if ( retval>=1) 
	{
		timedout=false;
		memset(&cliaddr,0,sizeof(struct sockaddr_in));
		destlen=sizeof(struct sockaddr_in);
		pktlen=recvfrom(sockfd,(void*)rawbuf,PACKETSIZE,0,(struct sockaddr *)&cliaddr,&destlen);
//		snprintf(buf,BUFFSIZE,"CUSock:: RecvData: After recvfrom sockfd=%d IP= %s destlen= %d.",
//			sockfd,
//			inet_ntoa(cliaddr.sin_addr),
//			destlen);
//		ProtLog(buf);
		snprintf(bigbuf,100,"CUSock:: RecvData: Rawbuf===>>'%s'\n",(char*)(rawbuf+RUDP_PKT_DATA));
		ProtLog(bigbuf);
	
		if (pktlen==-1)   // this is an error on the socket
		{
			errorcode=-errno;
			timedout=false;
			ProtLog("CUSock:: RecvData: ERROR on SOCKET after receive.\n");
			goto RecvDataExit;
			}

		if (pktlen >= 1)
		{
			errorcode=0;
			timedout=false;
			}
		}

	if (pktlen<=0) goto RecvDataExit;
	if (loglevel>=1) ProtLog("CUSock:: RecvData: ===================>>>>>Got data from packet.\n");

	// is it an RUDP packet ?
	if (!IsRUDP(rawbuf))
	{
		ProtLog("CUSock:: RecvData: Not an RUDP Packet.");
		goto RecvDataExit;
		}
		
	// is it data ?
	if (IsDATA(rawbuf))
	{
		// it is data
		// Now check if the client socket is connected
		if  ((sockettype=='c') && (state!=SOCK_ST_CONNECTED)) 	
		{	
			ProtLog("CUSock:: RecvData: Data pkt w/o connection.");
			goto RecvDataExit;
			}
		
		// now check if valid IP and session, if not discard
		ProtLog("CUSock:: RecvData: It is DATA.\n");
		if (ValidIPSes(cliaddr.sin_addr.s_addr, rawbuf, pktlen, &connectionindex))
		{
			// ok now ack the packet if needed
			if (NeedAck(rawbuf, pktlen)) SendAck(rawbuf, pktlen, &cliaddr, destlen); 

			// now check to see if we have all previous packets to this serial
			if (!IsMissingPrev(rawbuf, pktlen, connectionindex)) 
			{
				// put in recvq, see if it frees other packets from waitq 
				if (PutInRecvQueue(rawbuf, pktlen, &cliaddr, connectionindex)==RUDP_ERR_NO_PKT_HEAP)
				{
					ProtLog("CUSock:: RecvData: PutInRecvQueue FAILED!!!.");
					}

				// move packets off waitq if can
			       	CheckWaitQFreeUp(rawbuf, pktlen, &cliaddr);
				}
			else
			{
				// Ok it is missing a previous packet
				AddToWaitQueue(rawbuf, pktlen, &cliaddr); 
				RequestRetrans(rawbuf, pktlen, &cliaddr, destlen); 
				goto RecvDataExit;
				}	
			}
		
		// Record receipt time for Keep alive here
		

		ProtLog("CUSock:: RecvData: Before RecvSignal.\n");
		// Recv NOW NEEDS TO BE INSIDE THE LOCKS 2004.12.08
		// due to the fact the fact that we are using the socket mutex to protect 
		// the condvar
		RecvSignal();	
		LocalUnLock(skmutex,"CUSock:: RecvData");
		return(RUDP_OK);
		}
	else
	{
		// Need to validate Valid IP Ses for here too
		// FIX ME !!!!
		// it is ctrl
		// is it an ack ?
		ProtLog("CUSock:: RecvData: It is CTRL.\n");
		if (IsAck(rawbuf, pktlen)) 
		{
			ProtLog("CUSock:: RecvData: It is ACK.\n");
			HandleAck(rawbuf, pktlen, &cliaddr);
			}

		else 
		{
			if (IsRetrans(rawbuf, pktlen)) 
			{
				ProtLog("CUSock:: RecvData: It is RETRAN.\n");
				HandleRetrans(rawbuf, pktlen, &cliaddr, destlen);
				}
			else 
			{
				if (IsConnect(rawbuf, pktlen)) 
				{
					ProtLog("CUSock:: RecvData: It is CON.\n");
					HandleConnect(rawbuf, pktlen, &cliaddr);
					}
				else 
				{
					if (IsComplete(rawbuf, pktlen)) 
					{
						ProtLog("CUSock:: RecvData: It is COMP.\n");
						HandleComplete(rawbuf, pktlen, &cliaddr);
						}
					else 
					{
						if (IsDisconnect(rawbuf, pktlen)) 
						{
							ProtLog("CUSock:: RecvData: It is DISC.\n");
							HandleDisconnect(rawbuf, pktlen, &cliaddr);
							}
						else 
						{
							if (IsPing(rawbuf, pktlen)) 
							{
								ProtLog("CUSock:: RecvData: It is PING.\n");
								HandlePing(rawbuf, pktlen);
								}
							else
							{
								if (IsDisconnectComplete(rawbuf, pktlen))
								{
									ProtLog("CUSock:: RecvData: It is DISCOM.\n");
									HandleDisconnectComplete(rawbuf, pktlen, &cliaddr);
									}	
								}
							}
						}
					}
				}
			}
	
		goto RecvDataExit;
		}	
	
RecvDataExit:
	LocalUnLock(skmutex,"CUSock:: RecvData");
	return (RUDP_OK);
	} //////////////////////////////////////////////////////////////////////////////end RecvData

////////////////////////////////////////////////////////////////////////////////////////////////////
// HandlePing
////////////////////////////////////////////////////////////////////////////////////////////////////
int CUSock::HandlePing(char *r, int len)
{
	return (RUDP_OK);
	} ////////////////////////////////////////////////////////////////////////////end HandlePing

////////////////////////////////////////////////////////////////////////////////////////////////////
// HandleRetrans
// this is a request to retransmit a packet
// for retransmit do i need to put another copy of packet in retry queue ? 
// (no, there is already one there)
////////////////////////////////////////////////////////////////////////////////////////////////////
int CUSock::HandleRetrans(char *r, int len,  struct sockaddr_in *cliaddr, socklen_t destlen)
{
unsigned long 	p;
void		*v;
int		pktlen;

	// given the request packet (r)
	// find the correct packet on the retryqueue and resend it
	// we need to match IP, session and serial
	p=retryqueue;
       	while (p!=PACKETHEAPCOUNT)
	{
		// check IP match
		if ( (cliaddr->sin_addr.s_addr)==(Get_Block_IP_InHeap(p)))

		// check Session match and check Serial match
		if (Get_Packet_SES_InBuff(r)==Get_Packet_SES_InHeap(p))
		if (Get_Packet_SER_InBuff(r)==(Get_Packet_SER_InHeap(p)))
		{
			// this is the packet to resend
			v=Get_Packet_DATA_Ptr_InHeap(p); 
			pktlen=Get_Packet_SIZE_InHeap(p);
			ProtLog("CUSock:: HandleRetrans: CALLING SENDTO.");
			SendTo(v,pktlen,cliaddr,sizeof(struct sockaddr_in));
			return (RUDP_OK); 
			} 
		p=Get_Block_NEXT_InHeap(p);
		}
	
	return (RUDP_OK);
	} /////////////////////////////////////////////////////////////////////////end HandleRetrans

////////////////////////////////////////////////////////////////////////////////////////////////////
// HandleComplete
// this is where the client acks back to us,
// and the connection is completed 
////////////////////////////////////////////////////////////////////////////////////////////////////
int CUSock::HandleComplete(char *r, int len, struct sockaddr_in *cliaddr)
{
unsigned long	index;
char		packet[65];
//int		result;
char		buf[BUFFSIZE];

	ProtLog("CUSock:: HandleComplete: Entered.\n");
	{int ii; for (ii=0;ii<65;ii++) packet[ii]='\0';}

	if (sockettype=='c') 
	{
		// client index=0
		ProtLog("CUSock:: HandleComplete: Setting connection index to 0 (CLIENT)\n");
		index=0;
		// DEBUG
		connections[index].IP		= cliaddr->sin_addr.s_addr;
		connections[index].port		= cliaddr->sin_port;
		goto HandleComplete1;
		}
	if (sockettype=='s')
	{
		// find index into connection array for this connection
		ProtLog("CUSock:: HandleComplete: Getting connection index (SERVER)\n");
		index=GetConnectionIndex(cliaddr->sin_addr.s_addr,r,len);
		if (index==MAX_CONNECTIONS)
		{
			ProtLog("CUSock:: HandleComplete: Couldn't find right session.\n");
			return(RUDP_ERR_SES_NOT_FOUND);
			}
		goto HandleComplete1;
		}
	ProtLog("CUSock:: HandleComplete: Error Bad Socket type.\n");
	return(RUDP_ERR_BAD_SOCK_TYPE);

HandleComplete1:

	// Ok now we have to correct index to complete the connection for, for both the server
	// and client cases. Now just use that index and fill in the data.
	// remember that HandleConnect() did the allochead and freehead stuff for us
	connections[index].serial	= 0l;
	connections[index].rser		= 0l;
	connections[index].retryttw	= 2000000l;		// this is the time to wait in uSecs
	connections[index].retryto	= 0l;			// this is the amount of time waited
	connections[index].retranttw	= 2000000l;
	connections[index].retranto	= 0l;
	connections[index].connected	= true;
	connections[index].kattw	= 2000000l;
	connections[index].ka_last	= 0l;
	connections[index].state	= RUDP_STA_CNCT;


	if (sockettype=='c') 	// if we are a client socket
	{
		state=SOCK_ST_CONNECTED;
		connections[index].session	= ntohl(Get_Packet_SES_InBuff(r));

		// now send ack back to server telling how many packets i intend to keep
		// build up a connection request
		ProtLog("CUSock:: HandleComplete: (CLIENT) Building packet.\n");

		// write RUDP packet header
		Set_Packet_Header(packet);
		Set_Packet_Values(packet,htonl(0l),htonl(0l),htonl(connections[index].session), RUDP_DC_CTRL, RUDP_AF_NOACK, htons(1));
		// set packet data
		*(unsigned char*)(packet+RUDP_PKT_DATA)=RUDP_CTRL_COM; 

		// send the data
		snprintf(buf,BUFFSIZE,"CUSock:: HandleComplete: (CLIENT) CALLING SENDTO sockfd=%d. IP='%s' Port=%d ses=%lu \n",
			sockfd,
			inet_ntoa(cliaddr->sin_addr),
			ntohs(cliaddr->sin_port),
			connections[index].session
			);
		ProtLog(buf);
		SendTo(packet,65,cliaddr,sizeof(struct sockaddr_in));

		}
	else
	{
		snprintf(buf,BUFFSIZE,"CUSock:: HandleComplete: (SERVER) sockfd=%d. IP='%s' Port=%d ses=%lu index=%lu\n",
			sockfd,
			inet_ntoa(cliaddr->sin_addr),
			ntohs(cliaddr->sin_port),
			connections[index].session,
			index
			);
		ProtLog(buf);
		}

	if (sockettype=='s') 
	{
	connecttype *ct;

		ct = new connecttype;
		ct->IP=cliaddr->sin_addr.s_addr;
		ct->session=connections[index].session;  // this is host form i think
		ct->port=connections[index].port; // this is network form i think
		ct->msg=CALL_BACK_MSG_NEW;
		ProtLog("CUSock:: HandleComplete: (SERVER) Before calling connect callback.\n");
		if (ConnectFunc) 
		{
			(*ConnectFunc)(ct);
			}
		else
		{
			ProtLog("CUSock:: HandleComplete: (SERVER) ERROR Null Function!! NOT calling connect callback.\n");
			}

		ProtLog("CUSock:: HandleComplete: (SERVER) After calling connect callback.\n");
		// they will delete the ct variable's memory
		}
	ProtLog("CUSock:: HandleComplete: Exited.\n");
	return (RUDP_OK);
	} ////////////////////////////////////////////////////////////////////////end HandleComplete

////////////////////////////////////////////////////////////////////////////////////////////////////
// AllocConSlot
// This will allocate a connections array slot and return the slot number allocated or
// will return MAXCONNECTIONS if no more slots
////////////////////////////////////////////////////////////////////////////////////////////////////
int CUSock::AllocConSlot()
{
int	slot;

	if (freehead==MAX_CONNECTIONS) return (MAX_CONNECTIONS);

	// give it a free slot
	slot=freehead;
		
	// now take slot off of free and put on alloctail
	if (freehead==freetail) 
	{
		// just emptied out the freelist
		freehead=MAX_CONNECTIONS;
	       	freetail=MAX_CONNECTIONS;

		// add to alloc list
		connections[alloctail].next=slot;
		connections[slot].prev=alloctail;
		connections[slot].next=MAX_CONNECTIONS;
		connections[slot].connected=true;
		connections[slot].state=RUDP_STA_CONRQ;
		alloctail=slot;
		}
	else // there are still slots in free list
	{
		// take out of the freelist
		freehead=connections[freehead].next;
		connections[freehead].prev=MAX_CONNECTIONS;

		// add to alloc list
		if (alloctail==MAX_CONNECTIONS) 	// if empty list
		{
			alloctail=slot;
			allochead=slot;
			connections[slot].prev=MAX_CONNECTIONS;
			connections[slot].next=MAX_CONNECTIONS;
			}
		else
		{
			connections[alloctail].next=slot;
			connections[slot].prev=alloctail;
			connections[slot].next=MAX_CONNECTIONS;
			connections[slot].connected=true;
			alloctail=slot;
			}
		}	

	return (slot);
	} //////////////////////////////////////////////////////////////////////////end AllocConSlot

////////////////////////////////////////////////////////////////////////////////////////////////////
// HandleConnect
// ok here we build a connection.
// - allocate a slot in the connection matrix
// - find next session number and assign it to this session
// - add an entry to the hash
// - respond with how many back packets i intend to keep
//
////////////////////////////////////////////////////////////////////////////////////////////////////
int CUSock::HandleConnect(char *r, int len, struct sockaddr_in *cliaddr)
{
//int		result;
int		slot;
unsigned long	p;
unsigned long	ses;
unsigned char	hashkey;
in_addr_t	ip;
unsigned long	hn;
char		packet[65];
char		buf[BUFFSIZE],dst[50];

//	ProtLog("CUSock:: HandleConnect: Entered.\n");

	// for debug purposes clear the packet array
	{int i; for (i=0;i<65;i++) packet[i]='\0';}

	// Check to see if we are the right kind of socket, ie not client
	if (sockettype=='c')	
	{	
		ProtLog("CUSock:: HandleConnect: Error. Not right kind of socket.\n");
		return (RUDP_ERR_SOCK_NOT_SERV);
		}

	// ok allocate a slot in connection array
	slot=AllocConSlot();
	if (slot!=MAX_CONNECTIONS) // do we have a slot available ?
	{
		// now search for the highest used session number on that IP,  and get next for this new session
		p=allochead;
		ses=0l;
		ip=cliaddr->sin_addr.s_addr;	// this is where the connection request came from
	       	while (p!=MAX_CONNECTIONS)
		{
			if (ip==connections[p].IP)
			{
				if (connections[p].session>=ses) ses=connections[p].session;
				}
			p=connections[p].next;
			}
		
		// now pick next session and assign it
		ses=ses+1;
		connections[slot].session=ses;
		//DEBUG
		connections[slot].IP=ip;
		connections[slot].port=cliaddr->sin_port;
		
		// now add entry to hash pointing to this slot.
		hashkey=   *((unsigned char *)(&ip)+3);
		snprintf(buf,BUFFSIZE,"CUSock:: HandleConnect: CREATE SES IP=%s IP[3]=%d port=%d ses=%lu slot=%i\n",
			inet_ntop(AF_INET, (void*)&(connections[slot].IP), dst,50),
			hashkey,
			ntohs(connections[slot].port),
			ses,
			slot);
		ProtLog(buf);

		// get the buckets pointer to the linked list to search
		p=sessionhash[hashkey];	
		
		// get a hashnode
		hn=AllocHsh();
		if (hn==HASHHEAPCOUNT)
		{
			return (RUDP_ERR_NO_HSH_HEAP);	//couldnt find a hash node 
			}
		snprintf(buf,BUFFSIZE,"CUSock:: HandleConnect: hash node=%lu\n",hn);
		ProtLog(buf);

		// now just insert the new session link  to the connection array
		sessionhash[hashkey]=hn;
		Set_Hash_CONSINDEX_InHeap(hn,slot);
		Set_Hash_NEXT_InHeap(hn,p);

		// now send ack back to client telling how many packets i intend to keep
		// build up a connection request
		ProtLog("CUSock:: HandleConnect:  Building packet.\n");
		Set_Packet_Header(packet);
		Set_Packet_Values(packet, htonl(0l), htonl(0l), htonl(ses), RUDP_DC_CTRL, RUDP_AF_ACK, htons(1));

		// set packet data
		*(unsigned char*)(packet+RUDP_PKT_DATA)=RUDP_CTRL_COM; 

		// send the data
		snprintf(buf,BUFFSIZE,"CUSock:: HandleConnect: CALLING SENDTO sockfd=%d. IP='%s' Port=%d\n",
			sockfd,
			inet_ntoa(cliaddr->sin_addr),
			ntohs(cliaddr->sin_port)
			);
		ProtLog(buf);
		SendTo(packet,65,cliaddr,sizeof(struct sockaddr_in));
		}
	else return (RUDP_ERR_NO_MORE_CONNS);
	ProtLog("CUSock:: HandleConnect: Exiting.\n");
	
	return (RUDP_OK);
	} /////////////////////////////////////////////////////////////////////////end HandleConnect

////////////////////////////////////////////////////////////////////////////////////////////////////
// HandleDisconnectComplete
// here we
// - check to ensure that discomp is for a connection that is already closing
// - notify external thread
// - tear down connection
////////////////////////////////////////////////////////////////////////////////////////////////////
int CUSock::HandleDisconnectComplete(char *r, int len, struct sockaddr_in *cliaddr)
{
//char		packet[65];
unsigned long	ses;
unsigned long	tp;
unsigned long	p;
connecttype 	*ct;
int		index;
in_addr_t	ip;

	// check to ensure that discomp is for connection that is closing
	ip=cliaddr->sin_addr.s_addr;
	ses=Get_Packet_SES_InBuff(r);
	p=closehead;
	
	// now compare to find which connection is the one we are closing
	while (p<MAX_CONNECTIONS)
	{
		if ((connections[p].IP==ip)&&(connections[p].session==ses))
		{
			goto HandleDisconnectComplete1;
			}
		} 

	ProtLog("CUSock:: HandleDisconnectComplete: Discarding RUDP_CTRL_DISCOM because couldnt find a closing connection.\n");
	return (1);

HandleDisconnectComplete1:

	// notify
	ct = new connecttype;
	ct->IP=cliaddr->sin_addr.s_addr;
	ct->session=ses;  // this is host form i think
	ct->port=cliaddr->sin_port; // this is network form i think
	ct->msg=CALL_BACK_MSG_NEW;
	ProtLog("CUSock:: HandleDisconnectComplete: Before calling disconnect callback.\n");
	if (DisConnectFunc) 
	{
		(*DisConnectFunc)(ct);
		}
	else
	{
		ProtLog("CUSock:: HandleDisconnectComplete: ERROR Null Function!! NOT calling disconnect callback.\n");
		}
	
	index=GetConnectionIndex(cliaddr->sin_addr.s_addr, cliaddr->sin_port, ses);
	
	// mark slot in conarray as available again,
	connections[index].session	= 0l;	// zero (long)
	connections[index].IP		= 0l;	// zero (long)
	connections[index].port		= 0;	
	connections[index].serial	= 0l;	// zero (long)

	connections[index].rser		= 0l;	// zero (long)
	connections[index].IP		= 0l;
	connections[index].retryttw	= 0l;
	connections[index].retryto	= 0l;
	connections[index].retranttw	= 0l;
	connections[index].retranto	= 0l;
	connections[index].connected	= false;
	connections[index].kattw	= 0l;
	connections[index].ka_last	= 0l;
	connections[index].state	= RUDP_STA_NOCON;

	// return it to free list
	DeAllocateConSlot(index);

	// search all queues to remove these IP ses pairs
	tp=PACKETHEAPCOUNT;
	DeleteBlockFromQueue(&sendqueue, &tp, ses, cliaddr->sin_addr.s_addr); 
	DeleteBlockFromQueue(&retryqueue, &tp, ses, cliaddr->sin_addr.s_addr); 
	DeleteBlockFromQueue(&recvqueue, &recvqueuetail, ses, cliaddr->sin_addr.s_addr); 
	DeleteBlockFromQueue(&waitqueue, &waitqueuetail, ses, cliaddr->sin_addr.s_addr); 

	// do we close socket here ?
	if (sockettype=='c') 
	{
		close(sockfd);
		sockfd=-1;
		}

	return (RUDP_OK);
	} //////////////////////////////////////////////////////////////end HandleDisconnectComplete

////////////////////////////////////////////////////////////////////////////////////////////////////
// HandleDisconnect
// here we
// - send disconnect complete packet
// - notify external thread
// - tear down connection
////////////////////////////////////////////////////////////////////////////////////////////////////
int CUSock::HandleDisconnect(char *r, int len, struct sockaddr_in *cliaddr)
{
char		packet[65];
unsigned long	ses;
unsigned long	tp;
connecttype 	*ct;
int		index;

	// send disconnect complete packet
	ProtLog("CUSock:: HandleDisconnect:  Entered, building packet DISCONNECT COMPLETE.\n");
	Set_Packet_Header(packet);

	ses=Get_Packet_SES_InBuff(r);
	
	// ser, retry, ses, dc, ack, len
	Set_Packet_Values(packet, htonl(0l), htonl(0l), htonl(ses), RUDP_DC_CTRL, RUDP_AF_ACK, htons(1));

	// set packet data
	*(unsigned char*)(packet+RUDP_PKT_DATA)=RUDP_CTRL_DISCOM; 

	SendTo(packet,65,cliaddr,sizeof(struct sockaddr_in));
	
	// notify
	ct = new connecttype;
	ct->IP=cliaddr->sin_addr.s_addr;
	ct->session=ses;  // this is host form i think
	ct->port=cliaddr->sin_port; // this is network form i think
	ct->msg=CALL_BACK_MSG_DISC;
	ProtLog("CUSock:: HandleDisconnect: Before calling disconnect callback.\n");
	if (DisConnectFunc) 
	{
		(*DisConnectFunc)(ct);
		}
	else
	{
		ProtLog("CUSock:: HandleDisconnect: ERROR Null Function!! NOT calling disconnect callback.\n");
		}
	
	index=GetConnectionIndex(cliaddr->sin_addr.s_addr, cliaddr->sin_port, ses);
	// assert that if sockettype is client index ==0
	if ((sockettype=='c')&&(index!=0)) 
	{
		ProtLog("CUSock:: HandleDisconnect: ERROR client socket and index of connection != 0.\n");
		exit (1);
		}

	// mark slot in conarray as available again,
	connections[index].session	= 0l;	// zero (long)
	connections[index].IP		= 0l;	// zero (long)
	connections[index].port		= 0;	
	connections[index].serial	= 0l;	// zero (long)

	connections[index].rser		= 0l;	// zero (long)
	connections[index].IP		= 0l;
	connections[index].retryttw	= 0l;
	connections[index].retryto	= 0l;
	connections[index].retranttw	= 0l;
	connections[index].retranto	= 0l;
	connections[index].connected	= false;
	connections[index].kattw	= 0l;
	connections[index].ka_last	= 0l;
	connections[index].state	= RUDP_STA_NOCON;

	// return in to free list
	DeAllocateConSlot(index);

	// search all queues to remove these IP ses pairs
	tp=PACKETHEAPCOUNT;
	ProtLog("CUSock:: HandleDisconnect: Deleting blocks from queues.\n");
	DeleteBlockFromQueue(&sendqueue, &tp, ses, cliaddr->sin_addr.s_addr); 
	DeleteBlockFromQueue(&retryqueue, &tp, ses, cliaddr->sin_addr.s_addr); 
	DeleteBlockFromQueue(&recvqueue, &recvqueuetail, ses, cliaddr->sin_addr.s_addr); 
	DeleteBlockFromQueue(&waitqueue, &waitqueuetail, ses, cliaddr->sin_addr.s_addr); 

	// do we close socket here ?
	if (sockettype=='c') 
	{
		close(sockfd);
		sockfd=-1;
		}

	return (RUDP_OK);
	} //////////////////////////////////////////////////////////////////////end HandleDisconnect

////////////////////////////////////////////////////////////////////////////////////////////////////
// HandleAck
// ok here we simply remove the acked packet from the retry queue
// and free the heap block
// remember that SendAck copies the header part of the packet, so all info we need is right there.
// also we are going to allow a different retry # to satisfy the requirement for an ack
////////////////////////////////////////////////////////////////////////////////////////////////////
int CUSock::HandleAck(char *r, int len, struct sockaddr_in *cliaddr)
{
unsigned long	p,tp;

	// search retry queue to find correct block
	p=retryqueue;
       	
	while (p!=PACKETHEAPCOUNT)	// remember that p is an index to an array
					// and that 0 is a valid entry
					// the invalid entry is PACKETHEAPCOUNT 			
	{
		// check IP match
		if ((cliaddr->sin_addr.s_addr) == (Get_Block_IP_InHeap(p)))

		// check Session match
		// r is the rawbuf that was received that RecvData is passing us
		// and check Serial match
		if (Get_Packet_SES_InBuff(r) == Get_Packet_SES_InHeap(p))
		if (Get_Packet_SER_InBuff(r) == Get_Packet_SER_InHeap(p))
		{
			// remove packet from queue
			if (p==retryqueue) 
			{
				// first item in list, possibly only item in list
				// retryqueue will either be zero (from single item case)
				// or point to next in list
				retryqueue=Get_Block_NEXT_InHeap(p);
				DeAllocPkt(p);

				// ok we're done
				return (RUDP_OK); 
				}
			else
			{
				Set_Block_NEXT_InHeap(tp, Get_Block_NEXT_InHeap(p));
				DeAllocPkt(p);

				// ok we're done
				return (RUDP_OK); 
				}	
			}

		// point to next in retry queue
		tp=p; 
		p=Get_Block_NEXT_InHeap(p);
		}	
	return (RUDP_ERR_ACK_NOT_FOUND);
	} /////////////////////////////////////////////////////////////////////////////end HandleAck

////////////////////////////////////////////////////////////////////////////////////////////////////
// RequestRestrans
// this is called when the immediate previous packet to the present packet (r) has not been received
// we have to be careful here:
// if packet n was dropped
// and n+1 is in waitqueue
// and r is n+2
// we don't want to request retrans of n+1, just of n
// so we request a restrans of n(IP,ses,serial(rser+1))
//
// retrans format is
// - packet header as normal
// - serial is serial of requested restrans (rser+1)
// - retry ???(figure this out)
// - session is session we are working on
// - datactrl is RUDP_DC_CTRL
// - ack is RUDP_AF_NOACK
// - length is 1
// - data is RUDP_CTRL_RET (retrans request)
////////////////////////////////////////////////////////////////////////////////////////////////////
int CUSock::RequestRetrans(char *r, int len, struct sockaddr_in *cliaddr, socklen_t destlen)
{
char		p[65];
unsigned long	ses;
unsigned long 	serial;

	// find out which serial to request restrans on
	ses=GetConnectionIndex(cliaddr->sin_addr.s_addr, r, len);
	if ((ses<0)||(ses>=MAX_CONNECTIONS)) return (RUDP_ERR_SES_NOT_FOUND);
	serial=connections[ses].rser+1;

	// build up a retrans request
	Set_Packet_Header(p);
	Set_Packet_Values(p, serial, htonl(0l), htonl(ses), RUDP_DC_CTRL, RUDP_AF_NOACK, htons(1));

	// set packet data
	*(unsigned char*)(p+RUDP_PKT_DATA)=RUDP_CTRL_RET; 

	// send the data
	ProtLog("CUSock:: RequestRetrans: CALLING SENDTO.\n");
	SendTo(p,65,cliaddr,sizeof(struct sockaddr_in));

	return (RUDP_OK);
	} ////////////////////////////////////////////////////////////////////////end RequestRetrans

////////////////////////////////////////////////////////////////////////////////////////////////////
// AddToWaitQueue
// here all we do is append to end of list
// because it is likely that most packets will come in in-order, so that if we insert
// always at begining of list, the packets will be in reverse order and will be freed
// in the slowest possible way once the late packet arrives.
// (at least thats the theory :)
////////////////////////////////////////////////////////////////////////////////////////////////////
int CUSock::AddToWaitQueue(char *r, int len, struct sockaddr_in *cliaddr)
{
unsigned long	p;	
//char		*c,*blk_base;	

	ProtLog("CUSock:: AddToWaitQueue: Entered.\n");
	// but first we must allocate a heap block and store it

	// put raw packet into waitqueue, at end of chain
	// store packet in heap 
	// modify heap map to reflect this all
	p=AllocPkt();
	Zero_Block_InHeap(p);
	
	if (p==PACKETHEAPCOUNT) 
	{
		return (RUDP_ERR_NO_PKT_HEAP);	// couldnt find a packet
		}
	
	// copy packet
	Set_Packet_InHeap(p,r,len);	

	// now write the rest of the block
	// write ip
	Set_Block_IP_InHeap(p,cliaddr->sin_addr.s_addr);
	Set_Block_PORT_InHeap(p,cliaddr->sin_port);

	// if empty list
	if (waitqueuetail==PACKETHEAPCOUNT)
	{	
		waitqueuetail=p; waitqueue=p;
		}	
	else
	{
		// now do the linked list stuff add it to the end of the chain.
		Set_Block_NEXT_InHeap(waitqueuetail, p);
		waitqueuetail=p;			// point to new end of chain 
		Set_Block_NEXT_InHeap(p, PACKETHEAPCOUNT);
		}	
	ProtLog("CUSock:: AddToWaitQueue: Exited.\n");
	return (RUDP_OK);
	} ////////////////////////////////////////////////////////////////////////end AddToWaitQueue

////////////////////////////////////////////////////////////////////////////////////////////////////
// CheckWaitQFreeUp
// the packet that got us here has gotten all its previous, and is in recvqueue already
//
// ok here we take given packet's IP and session, and see if the next serial is on list
// if so then do again with the NEXT serial and so on.
////////////////////////////////////////////////////////////////////////////////////////////////////
int CUSock::CheckWaitQFreeUp(char *r, int len, struct sockaddr_in *cliaddr)
{
in_addr_t	ip_in,ip_wt;
unsigned long	ses_in,ses_wt;
unsigned long 	ser_in,ser_wt;
unsigned long	p,tp;

	// here we check if this packet frees up anybody from being on the waitq
	ip_in	=cliaddr->sin_addr.s_addr;
	ses_in	= Get_Packet_SES_InBuff(r);
	ser_in	= Get_Packet_SER_InBuff(r);

CheckWaitQFreeUpStartList:
	
	p=waitqueue;
	tp=p;
	if (p==PACKETHEAPCOUNT) return (RUDP_OK);	// if p== the NULL index

	while (p!=PACKETHEAPCOUNT)
	{
		// is this packet the one following the incoming packet ?
		if ( ((ip_wt =Get_Block_IP_InHeap(p))==ip_in )
			&&((ses_wt=Get_Packet_SES_InHeap(p))==ses_in)
			&&((ser_wt=Get_Packet_SER_InHeap(p))==ser_in+1)
			)
		{
			// yes, take it out of list
			if (tp==waitqueue)
			{
				waitqueue=Get_Block_NEXT_InHeap(waitqueue);
				}	
			else
			{
				Set_Block_NEXT_InHeap(tp, Get_Block_NEXT_InHeap(p));
				}	
			// add it to the end of the chain recvqueue chain.
			Set_Block_NEXT_InHeap(recvqueuetail, p);
			recvqueuetail=p;                        // point to new end of chain 
			Set_Block_NEXT_InHeap(p, PACKETHEAPCOUNT);
			
			// then make ip_in, ses_in, ser_in equal to NEXT packet and start over
			ip_in=ip_wt; ses_in=ses_wt; ser_in=ser_wt;
			goto CheckWaitQFreeUpStartList;
			}
		// check next packet
		tp=p; 
		p=Get_Block_NEXT_InHeap(p);
		}
	return (RUDP_OK);

	} //////////////////////////////////////////////////////////////////////end CheckWaitQFreeUp

////////////////////////////////////////////////////////////////////////////////////////////////////
// PutInRecvQueue
// put packet in recvqueue
// also update rser, ie latest received packet (at this point we have deceieded that we have received
// all previouspackets, so update the rser)
////////////////////////////////////////////////////////////////////////////////////////////////////
int CUSock::PutInRecvQueue(char *r, int len, struct sockaddr_in *cliaddr, unsigned long ses)
{
unsigned long	p;	

	// put raw packet into recvqueue, at end of chain
	// store packet in heap 
	// modify heap map to reflect this all
	ProtLog("CUSock:: PutInRecvQueue: Entered.\n");

	// update latest received serial
	connections[ses].rser++;

	// Allocate a place for it to wait
	p=AllocPkt();

	if (p==PACKETHEAPCOUNT)
	{
		ProtLog("CUSock:: PutInRecvQueue: Exit FAIL NO ALLOC.\n");
		return (RUDP_ERR_NO_PKT_HEAP);	// couldnt find a packet
		}

	Zero_Block_InHeap(p);

	// copy packet
	Set_Packet_InHeap(p,r,len);	

	// now write the rest of the block
	// write ip
	Set_Block_IP_InHeap(p,cliaddr->sin_addr.s_addr);
	Set_Block_PORT_InHeap(p,cliaddr->sin_port);

	// OK we need to insert, if empty list
	if (IsRecvQEmpty())
	{
		recvqueuetail=p; 
		recvqueue=p;
		}
	else
	{
		// now do the linked list stuff add it to the end of the chain.
		Set_Block_NEXT_InHeap(recvqueuetail, p);
		recvqueuetail=p;			// point to new end of chain 
		Set_Block_NEXT_InHeap(p, PACKETHEAPCOUNT);
		}

	return (RUDP_OK);
	} ////////////////////////////////////////////////////////////////////////end PutInRecvQueue

////////////////////////////////////////////////////////////////////////////////////////////////////
// IsMissingPrev
////////////////////////////////////////////////////////////////////////////////////////////////////
bool CUSock::IsMissingPrev(char *r, int len, unsigned long ses)
{
unsigned long	pktser;		// the serial of the incoming packet

	// check to see if this packet is next in serial order
	ProtLog("CUSock:: IsMissingPrev: Entered.\n");
	pktser=ntohl(Get_Packet_SER_InBuff(r));
	if (pktser==connections[ses].rser+1) return (true);
	else return (false);
	} /////////////////////////////////////////////////////////////////////////end IsMissingPrev

////////////////////////////////////////////////////////////////////////////////////////////////////
// SendAck
////////////////////////////////////////////////////////////////////////////////////////////////////
int CUSock::SendAck(char *r, int len, struct sockaddr_in *dest, socklen_t destlen)
{
char	response[65];

	// send an ack packet back to given address for session and serial in r
	// we'll send back the same serial retry and session
	// and set the flag to ctrl, the ackflg to no ack, and length to 1  
	{int ii; for (ii=0;ii<65;ii++) response[ii]='\0';}
	memcpy (response,r,65);
	(*((unsigned char*)response+RUDP_PKT_FLG)) =RUDP_DC_CTRL;
	(*((unsigned char*)response+RUDP_PKT_ACK)) =RUDP_AF_NOACK;
	(*((unsigned char*)response+RUDP_PKT_DATA))=RUDP_CTRL_ACK;
	ProtLog("CUSock:: SendAck: CALLING SENDTO.");
	SendTo(response,65,dest,sizeof(struct sockaddr_in));
	return (RUDP_OK);
	} ///////////////////////////////////////////////////////////////////////////////end SendAck

////////////////////////////////////////////////////////////////////////////////////////////////////
// NeedAck
////////////////////////////////////////////////////////////////////////////////////////////////////
bool CUSock::NeedAck(char *r, int len)
{
bool	ackflg;

	// just check the flag byte
	ackflg=*(unsigned char*)(r+RUDP_PACKET+RUDP_PKT_ACK); 
	return (ackflg);
	} ///////////////////////////////////////////////////////////////////////////////end NeedAck

////////////////////////////////////////////////////////////////////////////////////////////////////
// GetConnectionIndex
// This is an overloaded function
// This one takes IP, raw packet, len
// returns session index
// 0->MAX_CONNECTIONS-1
// returns 
// MAX_CONNECTIONS if notfound
////////////////////////////////////////////////////////////////////////////////////////////////////
unsigned long CUSock::GetConnectionIndex(in_addr_t ip, char *r, int len)
{
unsigned long	p,ses;
unsigned char	hashkey;
char		buf[BUFFSIZE];
char		dst[50];

//	ProtLog("CUSock:: GetConnectionIndex: Entering.");

	// Create hash key out of fourth octet (0,1,2,3) of the IP address
	hashkey=   *((unsigned char *)(&ip)+3);

	// get the buckets pointer to the linked list to search
	p=sessionhash[hashkey];		
	ses=ntohl(Get_Packet_SES_InBuff(r));	// we have just the packet here, not the whole heap block


	snprintf(buf,BUFFSIZE,"CUSock:: GetConnectionIndex: Searching for ses=%lu hashkey=%d \n",ses,hashkey);
	ProtLog(buf);

	// if we are a client socket then just look at connections[0]
	if (sockettype=='c')
	{
		if ((connections[0].session==ses)&&(connections[0].IP)) return(0);
		else return (MAX_CONNECTIONS);
		}

	// now that we have the correct ip bucket, search for the session that matches
	while (p!=HASHHEAPCOUNT)
	{
	unsigned long index;

		index=Get_Hash_CONSINDEX_InHeap(p);
		// now see if this is the session we are looking for
		snprintf(buf,BUFFSIZE,"CUSock:: GetConnectionIndex: COMPARE ses=%lu IP=%s port=%d p=%lu index=%lu \n",
			connections[index].session,
			inet_ntop(AF_INET, (void*)&(connections[index].IP),dst,50),
			ntohs(connections[index].port),
			p,
			index);
		ProtLog(buf);
		 
		// DEBUG We should check the IP and the session
		if ((connections[index].session==ses) && (connections[index].IP==ip))
		{
			// this is the right session so return the index
			ProtLog("CUSock:: GetConnectionIndex: Exiting found session. \n");
			return (index);
			}
		p=Get_Hash_NEXT_InHeap(p);
		}

	// didnt find the right one, so return the out of bounds
	// here the out of bounds is really MAX_CONNECTIONS, because MAX_CONNECTIONS is the size of the
	// connections array
	ProtLog("CUSock:: GetConnectionIndex: Exiting DID NOT find session. \n");
	return (MAX_CONNECTIONS);
	} ////////////////////////////////////////////////////////////////////end GetConnectionIndex

////////////////////////////////////////////////////////////////////////////////////////////////////
// GetConnectionIndex
// This is an overloaded function
// This one takes IP, port, session
// returns session index
// 0->MAX_CONNECTIONS-1
// returns 
// MAX_CONNECTIONS if notfound
////////////////////////////////////////////////////////////////////////////////////////////////////
unsigned long CUSock::GetConnectionIndex(in_addr_t ip, in_port_t port, unsigned long   session)
{
unsigned long	p;
unsigned char	hashkey;
char		buf[BUFFSIZE];
char		dst[50];

//	ProtLog("CUSock:: GetConnectionIndex: Entering.");

	// Create hash key out of fourth octet (0,1,2,3) of the IP address
	hashkey=   *((unsigned char *)(&ip)+3);

	// get the buckets pointer to the linked list to search
	p=sessionhash[hashkey];		

	snprintf(buf,BUFFSIZE,"CUSock:: GetConnectionIndex: Searching for ses=%lu hashkey=%d",session,hashkey);
	ProtLog(buf);

	// if we are a client socket then just look at connections[0]
	if (sockettype=='c')
	{
		if ((connections[0].session==session)&&(connections[0].IP)) return(0);
		else return (MAX_CONNECTIONS);
		}

	// now that we have the correct ip bucket, search for the session that matches
	while (p!=HASHHEAPCOUNT)
	{
	unsigned long index;

		index=Get_Hash_CONSINDEX_InHeap(p);
		// now see if this is the session we are looking for
		snprintf(buf,BUFFSIZE,"CUSock:: GetConnectionIndex: COMPARING ses=%lu IP=%s port=%d p=%lu index=%lu",
			connections[index].session,
			inet_ntop(AF_INET, (void*)&(connections[index].IP),dst,50),
			ntohs(connections[index].port),
			p,
			index);
		ProtLog(buf);
		 
		// Check the IP and the session
		if ((connections[index].session==session) && (connections[index].IP==ip))
		{
			// this is the right session so return the index
			ProtLog("CUSock:: GetConnectionIndex: Exiting found session.");
			return (index);
			}
		p=Get_Hash_NEXT_InHeap(p);
		}

	// didnt find the right one, so return the out of bounds
	// here the out of bounds is really MAX_CONNECTIONS, because MAX_CONNECTIONS is the size of the
	// connections array
	ProtLog("CUSock:: GetConnectionIndex: Exiting DID NOT find session.");
	return (MAX_CONNECTIONS);
	} ////////////////////////////////////////////////////////////////////end GetConnectionIndex

////////////////////////////////////////////////////////////////////////////////////////////////////
// ValidIPSes
////////////////////////////////////////////////////////////////////////////////////////////////////
bool CUSock::ValidIPSes(in_addr_t ip, char *r, int len, unsigned long *ses)
{
	// given the packet make sure it is from a valid IP and session
	// Get sessionindex to session given IP
	// get the index to the correct bucket 
	*ses=GetConnectionIndex(ip, r, len);
       	
	if ((*ses>=0) && (*ses<MAX_CONNECTIONS)) return (true);
	else return (false);
	} ////////////////////////////////////////////////////////////////////////////end ValidIPSes

////////////////////////////////////////////////////////////////////////////////////////////////////
// CheckCloseList
// Here we check for any connections that may need closing, and close them, 
// we close one each time through 
////////////////////////////////////////////////////////////////////////////////////////////////////
int CUSock::CheckCloseList()
{
unsigned long		p;
char			buf[BUFFSIZE];
struct sockaddr_in	cliaddr;
char			packet[65];
//int			result;

	// check to see if there are any closing connections at all
	LocalLock(skmutex,"CUSock:: CheckCloseList");
	if (closehead==MAX_CONNECTIONS) 
	{
		LocalUnLock(skmutex,"CUSock:: CheckCloseList");
		return (0); 
		}
	if (loglevel>=1) ProtLog("CUSock:: CheckCloseList: Continuing past 'if (closehead==MAX_CONNECTIONS)' .");

	// if we are a client then there is only one connection 
	if (sockettype=='c') p=0;
	else 
	{	
		// search for a connection that needs to have a discon packet sent
		p=closehead;
		while ((p<MAX_CONNECTIONS)&&(connections[p].state==RUDP_STA_CLOSING))  
		{
			p=connections[p].next; 
			if (loglevel>=1) ProtLog("CUSock:: CheckCloseList: Checking next in close list.");
			}

		if (p==MAX_CONNECTIONS) 
		{
			if (loglevel>=1) ProtLog("CUSock:: CheckCloseList: Couldnt find any packets to close.");
			LocalUnLock(skmutex,"CUSock:: CheckCloseList");
			return (0);
			}	
		}

	// hopefully p is index of connection we want to close
	// set state of connection to closing so that we dont do him again next time
	connections[p].state=RUDP_STA_CLOSING;

	// ok, now we should send out the disconnect packet
	// write RUDP packet header
	Set_Packet_Header(packet);

	// build a disconnect packet
	// ser, retry, ses, dc, ack, len
	Set_Packet_Values(packet,htonl(0l),htonl(0l),htonl(connections[p].session), RUDP_DC_CTRL, RUDP_AF_ACK, htons(1));

	// set packet data
	*(unsigned char*)(packet+RUDP_PKT_DATA)=RUDP_CTRL_DIS; 

	// send the disconnect
	cliaddr.sin_family=AF_INET;
	cliaddr.sin_addr.s_addr=connections[p].IP;
	cliaddr.sin_port=connections[p].port;
 
	if (loglevel>=1) ProtLog("CUSock:: CheckCloseList: Sending discon packet.");
	snprintf(buf,BUFFSIZE,"CUSock:: CheckCloseList: CALLING SENDTO sockfd=%d. IP='%s' Port=%d",
		sockfd,
		inet_ntoa(cliaddr.sin_addr),
		ntohs(cliaddr.sin_port)
		);
	ProtLog(buf);
	SendTo(packet,65,&cliaddr,sizeof(struct sockaddr_in));

	if (loglevel>=1) ProtLog("CUSock:: CheckCloseList: Exiting from bottom.");
	LocalUnLock(skmutex,"CUSock:: CheckCloseList");
	return (0);
	} ////////////////////////////////////////////////////////////////////////end CheckCloseList

////////////////////////////////////////////////////////////////////////////////////////////////////
// CheckRetries
////////////////////////////////////////////////////////////////////////////////////////////////////
int CUSock::CheckRetries()
{
	LocalLock(skmutex,"CUSock:: CheckRetries");

	LocalUnLock(skmutex,"CUSock:: CheckRetries");
	return (0);
	} //////////////////////////////////////////////////////////////////////////end CheckRetries

////////////////////////////////////////////////////////////////////////////////////////////////////
// CheckRetrans
////////////////////////////////////////////////////////////////////////////////////////////////////
int CUSock::CheckRetrans()
{
	LocalLock(skmutex,"CUSock:: CheckRetrans\n");

	LocalUnLock(skmutex,"CUSock:: CheckRetrans\n");
	return (0);
	} //////////////////////////////////////////////////////////////////////////end CheckRetrans

////////////////////////////////////////////////////////////////////////////////////////////////////
// CheckKeepAlive
////////////////////////////////////////////////////////////////////////////////////////////////////
int CUSock::CheckKeepAlive()
{
	LocalLock(skmutex,"CUSock:: CheckKeepAlive\n");

	LocalUnLock(skmutex,"CUSock:: CheckKeepAlive\n");
	return (0);
	} ////////////////////////////////////////////////////////////////////////end CheckKeepAlive

////////////////////////////////////////////////////////////////////////////////////////////////////
// PortMan
// - sends data on UDP port
// - receives data on UDP port
// - queues packet(n) on recvqueue in serialized order ONCE all packets(n-) have been queued
// 	- ie sends retransmit if required.
// - does retries if no ack is received in timeout time
// - does keep alives in no packet in timeout time
////////////////////////////////////////////////////////////////////////////////////////////////////
void *PortMan(void *ptr)
{
CUSock *s;

	s=(CUSock *)ptr;

	s->ProtLog("CUSock:: PortMan: Thread entry.\n");

	while (true)
	{

		// Send any data on send queue
		s->SendData();

		// Recv any data put on recv queue (and send ack if requested)
		s->RecvData();
		pthread_yield();

		// Check for any retransmits needed
		s->CheckRetrans();
	
		// Check retries
		s->CheckRetries();
	
		// Check to see if need ping for keep alive
		s->CheckKeepAlive();
		
		// Check to see if any connections are on the close list 
		s->CheckCloseList();

		// check to see if any packets are waiting and signal the reader threads
//		s->recvSignal();
		
		}
	s->ProtLog("CUSock:: PortMan: Exited.\n");

	} ///////////////////////////////////////////////////////////////////////////////end PortMan

////////////////////////////////////////////////////////////////////////////////////////////////////
// CUSock::Listen
// Listen on the port and accept a connection when it appears 
// this will start portman for the server
////////////////////////////////////////////////////////////////////////////////////////////////////
bool CUSock::Listen()
{
int 	i;

	ProtLog("CUSock:: Listen: Entered.\n");
	if (state!=SOCK_ST_BOUND) return(false);

	// Allocate the heaps
	packetheap = new char [(PACKETSIZE+PACKETHEAPHDR)*PACKETHEAPCOUNT]; 
	if (!packetheap) return false;
	hashheap = new char [HASHBLKSIZE*HASHHEAPCOUNT]; 
	if (!hashheap) return false;

	// Init the heaps by setting alloc maps to not allocated
	for (i=0;i<PACKETHEAPCOUNT;i++)	pktheapmap[i]=0l;		
	for (i=0;i<HASHHEAPCOUNT;i++)	hshheapmap[i]=0l;		

	// For the indexes into these arrays, in order to use 0 we need to use PACKETHEAPCOUNT
	// as the end pointer
	sendqueue=PACKETHEAPCOUNT;
	retryqueue=PACKETHEAPCOUNT;
	recvqueue=PACKETHEAPCOUNT;
	recvqueuetail=PACKETHEAPCOUNT;
	waitqueue=PACKETHEAPCOUNT;
	waitqueuetail=PACKETHEAPCOUNT;
	sockettype='s';

	// allocate connection array
	connections = new conarraytype[MAX_CONNECTIONS]; 
	for (i=0;i<MAX_CONNECTIONS;i++)
	{
		// init the array	
		connections[i].serial	=0l;	// zero (long)
		connections[i].rser	=0l;	// zero (long)
		connections[i].session	=0l;	// zero (long)
		connections[i].IP	=0l;
		connections[i].retryttw	=0l;
		connections[i].retryto	=0l;
		connections[i].retranttw=0l;
		connections[i].retranto	=0l;
		connections[i].connected=false;
		connections[i].kattw	=0l;
		connections[i].ka_last	=0l;
		connections[i].prev	=MAX_CONNECTIONS;
		connections[i].next	=MAX_CONNECTIONS;
		}	

	// Init the sessionhash
	for (i=0;i<256;i++)
	{
		sessionhash[i]=HASHHEAPCOUNT;	// the value stored is an index to an array in another local
						// heap. The linked list contains indexes to the connectionarray.  
		}
	
	state=SOCK_ST_LISTEN;

	// Init the allochead and tail and free head and tail
	// ok remember that there wont be anything allocated so we can take a short cut on 
	// initting the freelist
	for (i=0;i<MAX_CONNECTIONS;i++)  
	{
		connections[i].next=i+1;
		connections[i].prev=i-1;
		}
	connections[0].prev=MAX_CONNECTIONS;			// since zero is a valid entry, we will use
	connections[MAX_CONNECTIONS-1].next=MAX_CONNECTIONS;	// MAXCONNECTIONS as the end of list flag
	allochead=MAX_CONNECTIONS;
	alloctail=MAX_CONNECTIONS;
	closehead=MAX_CONNECTIONS;
	closetail=MAX_CONNECTIONS;
	freehead=0;
	freetail=MAX_CONNECTIONS-1;
	
	// start PortMan thread, set parm  first
	ProtLog("CUSock:: Listen: Starting PortMan.\n");
//	pthread_create(&portmanthread, &attr, PortMan, (void *)this);
	CreateThread(&portmanthread, PortMan, (void *)this);

	ProtLog("CUSock:: Listen: Exited.\n");
	return (true);
	} ////////////////////////////////////////////////////////////////////////////////end Listen

////////////////////////////////////////////////////////////////////////////////////////////////////
// CUSock::Connect
// send special connect request packet after setting up data structs 
// this will start portman for the client
// This call will then block with timeout, so that the return code will tell if the connect worked.
// The PortMan thread will be the one to receive the connect complete,
// and will then signal to wait this (external) thread up.
// Remember that this proc is called by an external thread. 
////////////////////////////////////////////////////////////////////////////////////////////////////
bool CUSock::Connect()
{
int 		i;
char		packet[65];
char		buf[BUFFSIZE];
char		dst[50];

	ProtLog("CUSock:: Connect: Entered.\n");
	if (state!=SOCK_ST_CREATED) return(false);
	{int ii; for (ii=0;ii<65;ii++) packet[ii]='\0';}

	// Allocate the heaps
	packetheap = new char [(PACKETSIZE+PACKETHEAPHDR)*PACKETHEAPCOUNT]; 
	if (!packetheap) return false;
	hashheap = new char [HASHBLKSIZE*HASHHEAPCOUNT]; 
	if (!hashheap) return false;

	// Init the heaps by setting alloc maps to not allocated
	for (i=0;i<PACKETHEAPCOUNT;i++)	pktheapmap[i]=0l;		
	for (i=0;i<HASHHEAPCOUNT;i++)	hshheapmap[i]=0l;		
	sendqueue=PACKETHEAPCOUNT;
	retryqueue=PACKETHEAPCOUNT;
	recvqueue=PACKETHEAPCOUNT;
	recvqueuetail=PACKETHEAPCOUNT;
	waitqueue=PACKETHEAPCOUNT;
	waitqueuetail=PACKETHEAPCOUNT;
	sockettype='c';

	// allocate connection array
	connections = new conarraytype; 
	connections[0].serial	= 0l;	// zero (long)
	connections[0].rser	= 0l;	// zero (long)
	connections[0].session	= 0l;	// zero (long)
	connections[0].IP	= addr.sin_addr.s_addr;
	connections[0].port	= addr.sin_port;
	connections[0].retryttw	= 0l;
	connections[0].retryto	= 0l;
	connections[0].retranttw= 0l;
	connections[0].retranto	= 0l;
	connections[0].connected= false;
	connections[0].kattw	= 0l;
	connections[0].ka_last	= 0l;
	connections[0].prev	= MAX_CONNECTIONS;
	connections[0].next	= MAX_CONNECTIONS;
	closehead=MAX_CONNECTIONS;
	closetail=MAX_CONNECTIONS;
	freehead=MAX_CONNECTIONS;
	freetail=MAX_CONNECTIONS;

	// now send connect packet
	// build up a connection request
	Set_Packet_Header(packet);
	Set_Packet_Values(packet, htonl(0l), htonl(0l), htonl(0l), RUDP_DC_CTRL, RUDP_AF_ACK, htons(1));

	// set packet data
	*(unsigned char*)(packet+RUDP_PKT_DATA)=RUDP_CTRL_CON; 

	// send the data
	SendTo(packet,65,&addr,sizeof(struct sockaddr_in));
	snprintf(buf,BUFFSIZE,"CUSock:: Connect: Sent packet to sockfd=%d. IP=%s \n",
		sockfd,
		inet_ntop(AF_INET,(void*)&(addr.sin_addr.s_addr),dst,50)
		);
	ProtLog(buf);
	
	// Wait for the connect to complete
	ProtLog("CUSock:: Connect: Before ConnectWait.\n");
	if (RUDP_OK!=ConnectWait()) 
	{
		return(false);
		ProtLog("CUSock:: Connect: ERROR ConnectWait Failed.\n");
		}
	else
	{
//		pthread_create( &portmanthread, &attr, PortMan, (void *)this);
		CreateThread( &portmanthread, PortMan, (void *)this);
		ProtLog("CUSock:: Connect: Started PortMan thread, exiting.\n");
		return(true);
		}
	} ///////////////////////////////////////////////////////////////////////////////end Connect

////////////////////////////////////////////////////////////////////////////////////////////////////
// ConnectWait 
// This is called after connect request is sent, and it is called once.
// We simply wait here for the response from the server, and then check to see if it is a correctly
// formed response packet.
// We have an intentional infinite loop waiting for a connect response. 
////////////////////////////////////////////////////////////////////////////////////////////////////
int CUSock::ConnectWait()
{
int			pktlen;		// the amount of raw data received
struct sockaddr_in	cliaddr;	// the place to put the address of sender
socklen_t		destlen;
char			buf[BUFFSIZE];
fd_set  		rfds;
int     		retval;
struct  timeval  	temptv;
bool			gotpacket;

ConnectWaitAgain:
	gotpacket=false;
   
	while (!gotpacket)
	{
		ProtLog("CUSock:: ConnectWait: While Loop.\n");
		temptv.tv_sec =tv.tv_sec; 
		temptv.tv_usec=tv.tv_usec;
		FD_ZERO(&rfds);

		FD_SET(sockfd,&rfds);	// note this is the only descriptor being watched
		retval=select(sockfd+1, &rfds, NULL, NULL, &temptv);
		if (retval==0)
		{
			timedout=true;
//			ProtLog("CUSock:: ConnectWait: TIMEOUT.");
			continue;
			}

		if (retval<0)
		{
			timedout=false;
			errorcode=errno;
			snprintf(buf,BUFFSIZE,"CUSock:: ConnectWait: ERROR on select sockfd=%d sec=%lu usec=%lu\n",
				sockfd,
				(unsigned long)temptv.tv_sec,
				(unsigned long)temptv.tv_usec);
			ProtLog(buf);
			PError(errno);
			continue;
			}
	
		// if got something
		if ( retval>=1) 
		{
			timedout=false;
			memset(&cliaddr,0,sizeof(struct sockaddr_in));
			destlen=sizeof(struct sockaddr_in);
			pktlen=recvfrom(sockfd,(void*)rawbuf,PACKETSIZE,0,(struct sockaddr *)&cliaddr,&destlen);
			snprintf(buf,BUFFSIZE,"CUSock:: ConnectWait: After recvfrom sockfd=%d IP= %s destlen= %d.\n",
				sockfd,
				inet_ntoa(cliaddr.sin_addr),
				destlen);
			ProtLog(buf);
		
			if (pktlen==-1)   // this is an error on the socket
			{
				errorcode=-errno;
				timedout=false;
				ProtLog("CUSock:: ConnectWait: ERROR on SOCKET after receive.\n");
				return (RUDP_ERR_SOCK_ERR);
				}

			if (pktlen >= 1)
			{
				errorcode=0;
				timedout=false;
				gotpacket=true;
				}
			}
		} // end while (1)

	// now check to see if packet is good
	// is it an RUDP packet ?
	ProtLog("CUSock:: ConnectWait: Is it an RUDP Packet?\n");
	if (!IsRUDP(rawbuf))
	{
		ProtLog("CUSock:: ConnectWait: Not an RUDP Packet.\n");
		goto ConnectWaitAgain;
		}
		
	ProtLog("CUSock:: ConnectWait: Is it an COMP Packet?\n");
	// is it CTRL ?
	if (IsCTRL(rawbuf))
	{
		ProtLog("CUSock:: ConnectWait: It is CTRL.\n");

		if (IsComplete(rawbuf, pktlen)) 
		{
			ProtLog("CUSock:: ConnectWait: It is COM.\n");
			HandleComplete(rawbuf, pktlen, &cliaddr);
			return(RUDP_OK);
			}
		else 
		{
			ProtLog("CUSock:: ConnectWait: Wrong Control Packet.\n");
			goto ConnectWaitAgain;
			}
		}
	goto ConnectWaitAgain;
	} ///////////////////////////////////////////////////////////////////////////end ConnectWait

////////////////////////////////////////////////////////////////////////////////////////////////////
// ConnectResp
////////////////////////////////////////////////////////////////////////////////////////////////////
int CUSock::ConnectResp()
{
	return (0);
	} ///////////////////////////////////////////////////////////////////////////end ConnectResp

////////////////////////////////////////////////////////////////////////////////////////////////////
// SendTo
// this is the low level sending routine for internal use only
// this routine will do the error checking and retrying 
////////////////////////////////////////////////////////////////////////////////////////////////////
int CUSock::SendTo(void *p, int length, struct sockaddr_in *a, socklen_t alen)
{
fd_set		sendset;
int		result;
int		nready;
char		buf[BUFFSIZE];
int 		count;

	ProtLog("CUSock:: SendTo: Entering.\n");
	count=0;
	FD_ZERO(&sendset);

SendToDoAgain:

	count++;
	snprintf(buf,BUFFSIZE,"CUSock:: SendTo: Times through loop count='%d'.\n",count);
	ProtLog(buf);
	
	FD_SET(sockfd,&sendset);
	if ( (nready = select(sockfd+1, NULL, &sendset, NULL, NULL)) < 0)
	{
		if (errno==EINTR) goto SendToDoAgain;
		else 
		{
			PError(errno);
			ProtLog("CUSock:: SendTo: Exiting select error.\n");
			return(RUDP_ERR_SOCK_ERR); 
			}
		}

	if (FD_ISSET(sockfd,&sendset))
	{
		result=sendto(sockfd, p, length, 0, (struct sockaddr *)a, alen);
		}

	if (result==-1)
	{
		snprintf(buf,BUFFSIZE,"CUSock:: SendTo: Error='%d'.\n",errno);
		ProtLog(buf);
		PError(errno);
		ProtLog("CUSock:: SendTo: Exiting sockerror.\n");
		return(RUDP_ERR_SOCK_ERR);
		}

	ProtLog("CUSock:: SendTo: Exiting OK.\n");
	return(RUDP_OK);
	} ////////////////////////////////////////////////////////////////////////////////end SendTo

////////////////////////////////////////////////////////////////////////////////////////////////////
// DeleteBlockFromQueue 
// Given session and IP and a pointer (which implies a queue) and maybe an end pointer
// Remove ay matching packets from the queue
////////////////////////////////////////////////////////////////////////////////////////////////////
void CUSock::DeleteBlockFromQueue(unsigned long *qroot, unsigned long *endp, unsigned long session, in_addr_t IP)
{
unsigned long	p;	// pointer
unsigned long	tp;	// trail pointer
unsigned long	ses;
in_addr_t	blockIP;

	// setup
	p=*qroot;
	tp=*qroot;

	// find the packets to delete
	while (p!=PACKETHEAPCOUNT)
	{
		// check IP and ses pair to see if we remove it
		ses=Get_Packet_SES_InHeap(p);
		blockIP=Get_Block_IP_InHeap(p);
		if ((IP==blockIP)&&(session==ses))
		{
			ProtLog("CUSock:: ServerClose: Removing packet from queue.");

			// remove block from queue
			// find out where in the queueu we are
			if (p==*qroot)
			{
				// delete first block in list
				// Take block out of list
				*qroot=Get_Block_NEXT_InHeap(p);
				
				if (*endp<PACKETHEAPCOUNT) // does the endqueue pointer point to something real ?
				{
					// if there is only one item in chain, set end pointer to NULL (PACKETHEAPCOUNT)
					if (*endp==p) *endp=PACKETHEAPCOUNT;
					// else there is more than one item so end pointer does not change
					}
	
				// there is no *endp for above so ignore 

				}

			else if (p==*endp)
			{	
				// This is the last block in chain
				// *qroot doesnt change, but take last block out
				Set_Block_NEXT_InHeap(tp,PACKETHEAPCOUNT);
				if (*endp<PACKETHEAPCOUNT)
				{
					*endp=tp;
					}
				}
			else 
			{
				// This is a mid chain block
				// *qroot doesnt change
				// *endp doesnt change
				Set_Block_NEXT_InHeap(tp,Get_Block_NEXT_InHeap(p));
				}
			// now deallocate block
			tp=p;
			p=Get_Block_NEXT_InHeap(p);
			DeAllocPkt(p);
			
			}
		else
		{
			tp=p;
			p=Get_Block_NEXT_InHeap(p);
			}
		}
	} //////////////////////////////////////////////////////////////////end DeleteBlockFromQueue

////////////////////////////////////////////////////////////////////////////////////////////////////
// DeAllocateConSlot
// this moves the given slot to the freelist
// so that Portman can reuse it later on.
// free->alloc->close->free
////////////////////////////////////////////////////////////////////////////////////////////////////
void CUSock::DeAllocateConSlot(int slot)
{

	if (sockettype=='c') return;
	if (closehead==MAX_CONNECTIONS) 
	{
		ProtLog("CUSock:: DeAllocateConSlot: Error. No connection in list when freeing connection.");
		exit (1);
		}

	// now take slot off of close and put on free if there is one item only in closelist
	if (closehead==closetail) 
	{
		if (slot!=closehead) 
		{
			ProtLog("CUSock:: DeAllocateConSlot: Error. ASSERT failed. slot!=closehead");
			exit(1);
			}
		// just emptied out the closelist
		closehead=MAX_CONNECTIONS;
	       	closetail=MAX_CONNECTIONS;
		}
	else // there are multiple slots in close list
	{
		// take out of the closelist, head side
		if (slot==closehead)
		{
			closehead=connections[slot].next;		// skip it on forward side 
			connections[closehead].prev=MAX_CONNECTIONS;	// skip it on reverse side
			}
		// tail side
		else if (slot==closetail)
		{
			closetail=connections[slot].prev;		// skip it on forward side 
			connections[closetail].next=MAX_CONNECTIONS;	// skip it on reverse side
			}
		else // (slot in middle)
		{
			connections[connections[slot].prev].next=connections[slot].next;
			connections[connections[slot].next].prev=connections[slot].prev;
			}
		}

	// add to free list
	if (freetail==MAX_CONNECTIONS) 	// if empty list
	{
		if (freehead!=freetail) 
		{
			ProtLog("CUSock:: DeAllocateConSlot: Error. ASSERT failed. freehead!=freetail");
			exit(1);
			}
		freetail=slot;
		freehead=slot;
		connections[slot].prev=MAX_CONNECTIONS;
		connections[slot].next=MAX_CONNECTIONS;
		}
	else
	{
		connections[freetail].next=slot;
		connections[slot].next=MAX_CONNECTIONS;
		connections[slot].prev=freetail;
		freetail=slot;
		}
	connections[slot].connected=false;

	} /////////////////////////////////////////////////////////////////////end DeAllocateConSlot

////////////////////////////////////////////////////////////////////////////////////////////////////
// SetForCloseConSlot
// this moves the given slot to the close list
// so that Portman can close it later on.
////////////////////////////////////////////////////////////////////////////////////////////////////
void CUSock::SetForCloseConSlot(int slot)
{

	if (allochead==MAX_CONNECTIONS) 
	{
		ProtLog("CUSock:: SetForCloseConSlot: Error. No connection in list when closing connection.");
		exit (1);
		}

	// take slot off of alloc and put on close 
	// if there is one item only in alloclist
	if (allochead==alloctail) 
	{
		if (slot!=allochead) 
		{
			ProtLog("CUSock:: SetForCloseConSlot: Error. ASSERT failed. slot!=allochead");
			exit(1);
			}
		// just emptied out the alloclist
		allochead=MAX_CONNECTIONS;
	       	alloctail=MAX_CONNECTIONS;
		}
	else // there are multiple slots in alloc list
	{
		// take out of the freelist, head side
		if (slot==allochead)
		{
			allochead=connections[slot].next;		// skip it on forward side 
			connections[allochead].prev=MAX_CONNECTIONS;	// skip it on reverse side
			}
		// tail side
		else if (slot==alloctail)
		{
			alloctail=connections[slot].prev;		// skip it on forward side 
			connections[alloctail].next=MAX_CONNECTIONS;	// skip it on reverse side
			}
		else // (slot in middle)
		{
			connections[connections[slot].prev].next=connections[slot].next;
			connections[connections[slot].next].prev=connections[slot].prev;
			}
		}

	// add to close list
	if (closetail==MAX_CONNECTIONS) 	// if empty list
	{
		if (closehead!=closetail) 
		{
			ProtLog("CUSock:: SetForCloseConSlot: Error. ASSERT failed. closehead!=closetail");
			exit(1);
			}
		closetail=slot;
		closehead=slot;
		connections[slot].prev=MAX_CONNECTIONS;
		connections[slot].next=MAX_CONNECTIONS;
		}
	else
	{
		connections[closetail].next=slot;
		connections[slot].next=MAX_CONNECTIONS;
		connections[slot].prev=closetail;
		closetail=slot;
		}
	connections[slot].connected=false;
	//the above will be done after discon packet sent. It will be done by CheckCloseList

	} ////////////////////////////////////////////////////////////////////end SetForCloseConSlot

////////////////////////////////////////////////////////////////////////////////////////////////////
// Close SERVER
// OK here we 
// - simply mark the connection for closing
// - later on Portman will handle actually closing the connection
// - we will use GetConnectionIndex to find the correct index, then move him to a "closinglist"
//   and portman will close one guy on this list each time through (to avoid starvation of other jobs
//   that portman has to do)
////////////////////////////////////////////////////////////////////////////////////////////////////
void CUSock::Close(connecttype *cparm)
{
unsigned long	index;

	// lock the protocol
	if (LocalLock(skmutex,"CUSock:: Close()")!=0) ProtLog("CUSock:: Close: Failed locking.");

	// ok find the index for the connection so we can move it from allocated list 
	// to the close list
	index=GetConnectionIndex(cparm->IP, cparm->port, cparm->session);
	SetForCloseConSlot(index);

	if (LocalUnLock(skmutex,"CUSock:: Close()")!=0) ProtLog("CUSock:: Close: Failed unlocking.");

	} /////////////////////////////////////////////////////////////////////////////////end Close

////////////////////////////////////////////////////////////////////////////////////////////////////
// Close CLIENT
// OK here we
// - mark slot in conarray open again, and put on free list and take off alloc list
//	- this is easy as there is only one slot
// - search all queues to remove these IP, ses pairs
//	- here we only set queues to PACKETHEAPCOUNT and deallocate all blocks
// - send disconnect to peer
// -
////////////////////////////////////////////////////////////////////////////////////////////////////
int CUSock::Close()
{
	if (LocalLock(skmutex,"CUSock:: Close()")!=0) ProtLog("CUSock:: Close: Failed locking.");
	SetForCloseConSlot(0);
	if (LocalUnLock(skmutex,"CUSock:: Close()")!=0) ProtLog("CUSock:: Close: Failed unlocking.");
        return (0);
	} ///////////////////////////////////////////////////////////////////////////end ConnectResp

