////////////////////////////////////////////////////////////////////////////////////////////////////
// cworld.cpp (new version)  
// This class will implement the model object 
// 
// by Eric J Krejci 
// Copyright (C) 1999 Eric J Krejci 
// Revised : 30 Aug 2006
//   
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation version 2.
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
// Contact:
//              email           worldii@viagloria.com
//              web             www.viagloria.com
////////////////////////////////////////////////////////////////////////////////////////////////////
#include <pthread.h>
#include <math.h>


#include "cworld.hpp"
//#ifdef MODELER
//#include "..\modeler\mod-util.h"
//#endif

//#ifdef CLIENT
//#include "wii_curses.h"
/////////////////#include "util.h"
//#include "csock.hpp"
//#endif

////////////////////////////////////////////////////////////////////////////////////////////////////
// Defines
////////////////////////////////////////////////////////////////////////////////////////////////////
// constant used for converting to radians 
const float pideg = 0.0174532925f;

////////////////////////////////////////////////////////////////////////////////////////////////////
// Externals
////////////////////////////////////////////////////////////////////////////////////////////////////
extern char	logfilename[];
extern bool	debug;
extern int	loglevel;
//extern float	pideg;

//#ifdef CLIENT
//extern CSock	connectsock; 
//#endif

////////////////////////////////////////////////////////////////////////////////////////////////////
// CWorld Init
////////////////////////////////////////////////////////////////////////////////////////////////////
bool CWorld::Init()
{
CString fontname;
int	result=0;
bool	bresult;

	ProtLog("CWorld::Init: Entered.");
	mutex = new pthread_mutex_t;
	if (!mutex) return (false);
	InitMutex(mutex);

	// setup textwindow
	ProtLog("CWorld::Init: Setup TextWindow.");
	textwindow = new CTextWin;
	if (!textwindow) return (false);
	textwindow->SetLogFunc(LogFunc);
	bresult=textwindow->Init();
	if (!bresult) return (false);
	textwindow->SetTextWinPos(300,0);

	commandstring = new CString;
	if (!commandstring) return (false);

	// setup font
	ProtLog("CWorld::Init: Setup Font.");
	font = new CFont;
	if (!font) return (false);
	font->SetLogFunc(LogFunc);
//	fontname.SetString("-adobe-helvetica-medium-r-normal--18-*-*-*-p-*-iso8859-1");
	fontname.SetString("Test.ttf");
	ProtLog("CWorld::Init: Before Build Font.");

	result=(font->BuildTTFont(&fontname,10)); 
	if (!(result==CF_OK)) 
	{
		printf("Build TTFont Error code %d \n ",result);
		ProtLog("CWorld::Init: BuildTTFont failed.");
		return (false);
		}
	font->SetTexture("white.png");
	ProtLog("CWorld::Init: Exitting.");

	// setup timer
	simtime = new CTimer;
	if (!simtime) return false;

	return true;
	} //////////////////////////////////////////////////////////////////////////////////end Init

////////////////////////////////////////////////////////////////////////////////////////////////////
// SetLogFunc
////////////////////////////////////////////////////////////////////////////////////////////////////
void CWorld::SetLogFunc(void (*pfunc)(char*))
{
	// set the log function to use
	LogFunc=pfunc;
	} ////////////////////////////////////////////////////////////////////////////end SetLogFunc

////////////////////////////////////////////////////////////////////////////////////////////////////
// CWorld Init
////////////////////////////////////////////////////////////////////////////////////////////////////
//bool CWorld::Init(bool (*pfunc)(char *,HWND tv,int n,LPARAM p),HWND tv)
//{
//
//	// set the function pointer
//	Init();
//	treeadd=pfunc;
//	hwndtv=tv;
//
//	return true;
//} /////////////////////////////////////////////////////////////////////////////////////////end Init

////////////////////////////////////////////////////////////////////////////////////////////////////
// ProtLog
////////////////////////////////////////////////////////////////////////////////////////////////////
void CWorld::ProtLog(char *s)
{
	if (LogFunc) (*LogFunc)(s);
	} ///////////////////////////////////////////////////////////////////////////////end ProtLog

////////////////////////////////////////////////////////////////////////////////////////////////////
// CModel Constructor
////////////////////////////////////////////////////////////////////////////////////////////////////
CWorld::CWorld()
{
int	i;

	// init the member variables
	pgup=0;			
	pgdn=0;			
	ins=0;			
	del=0;			
	LogFunc=0;

	// navigation
	uparrow=0;		
	dnarrow=0;		
	leftarrow=0;		
	rightarrow=0;		

	// keyboard modifiers
	shift=false;		
	alt=false;		
	ctrl=false;		
	meta=false;		

	inside=false;
	movers=0;
	rooms=0;
	xindex=0;
	t0 = 0;
	frames = 0;
	framestring[0]='\0';
	textwindow=0;
	windowwidth=0;
	windowheight=0;
	turnaccel=0.0;

	for (i=0;i<NUMREGIONS;i++) regions[i] = 0;		// The regions. 
	for (i=0;i<NUMFASTREGIONS;i++) fastregions[i] = 0;	// The close by regions. 
								// This list gets updated on a move.
	for (i=0;i<MAXNUMOFVISIBLECHARS;i++) 
	{
		movingchars[i].active=false;
		movingchars[i].model=0;
		movingchars[i].pos.x=0.0;
		movingchars[i].pos.y=1.8f;
		movingchars[i].pos.z=0.0;
		movingchars[i].vel.x=0.0;
		movingchars[i].vel.y=0.0;
		movingchars[i].vel.z=0.0;
		movingchars[i].speed=0.013;
		movingchars[i].dir=0.0;
		movingchars[i].id=0;
		movingchars[i].elev=0.0;
		movingchars[i].render=false;
		movingchars[i].moving=false;
		movingchars[i].turning=false;
		}
	movingchars[0].active=true;				// [0] is this player's char
	movingchars[0].render=false;				// dont render our own char

	// init the region list indexes
//	xregh=0;
//	xregt=0;
//	yregh=0;
//	yregt=0;
//	zregh=0;
//	zregt=0;


	// This is for debugging and for inititalization 
//	treeadd=0;
	} ///////////////////////////////////////////////////////////////////////////end Constructor

////////////////////////////////////////////////////////////////////////////////////////////////////
// CModel Destructor
////////////////////////////////////////////////////////////////////////////////////////////////////
CWorld::~CWorld()
{
	// BUILD DESTRUCTOR !!!!!!!!
	// 977K memory leak !!!!!!!!
	} ////////////////////////////////////////////////////////////////////////////end Destructor

////////////////////////////////////////////////////////////////////////////////////////////////////
// CWorld InsertRegionIntoIndex
////////////////////////////////////////////////////////////////////////////////////////////////////
//bool CWorld::InsertRegionIntoIndex(CRegion *,int whichindex)
//{
//	// which=1  => insert into xindex
//	// which=2  => insert into yindex
//	// which=3  => insert into zindex
//	switch (whichindex)
//	{
//	case 1:
//		break;
//	case 2:
//		break;
//	case 3:
//		break;
//		}
//	} /////////////////////////////////////////////////////////////////end InsertRegionIntoIndex

////////////////////////////////////////////////////////////////////////////////////////////////////
// CWorld InsertRegion
////////////////////////////////////////////////////////////////////////////////////////////////////
bool CWorld::InsertRegion(CRegion *r)
{
int	i;
	// use existing allocated struct and insert it.
	// find first unused (==0) slot in array and use it
	for (i=0;i<NUMREGIONS;i++)
	{
		if (!regions[i]) 
		{
			regions[i]=r;
			return true;
			}
		}
	return false;
	} //////////////////////////////////////////////////////////////////////////end InsertRegion

////////////////////////////////////////////////////////////////////////////////////////////////////
// CWorld InsertModel
// This is when we are adding a top level model to a region
////////////////////////////////////////////////////////////////////////////////////////////////////
//bool CWorld::InsertModel(CModel *m, CInode *i)
//{
//CInode *iptr;
//
//	// given existing allocated object insert it.
//	iptr = new CInode;
//	iptr->next  = i;
//	iptr->model = m;
//	i=iptr;
//	return true;
//	} ///////////////////////////////////////////////////////////////////////////end InsertModel

////////////////////////////////////////////////////////////////////////////////////////////////////
// CWorld InsertModel
// This is when we are adding a top level model to a region
////////////////////////////////////////////////////////////////////////////////////////////////////
bool CWorld::InsertModel(CModel *m, CRegion *r, GLfloat x, GLfloat y, GLfloat z)
{
CInode 		*iptr;
CInode		*px, *py, *pz;
bool 		foundx,foundy,foundz;

	// first check for empty list
	printf("CWorld:: InsertModel.\n");
	if ((r->xmodh)==0)
	{
		iptr = new CInode;
		iptr->xpos=x;iptr->ypos=y;iptr->zpos=z;
		r->xmodh=iptr;
		r->xmodt=iptr;
		iptr->model=m;

		iptr = new CInode;
		iptr->xpos=x;iptr->ypos=y;iptr->zpos=z;
		r->ymodh=iptr;
		r->ymodt=iptr;
		iptr->model=m;

		iptr = new CInode;
		iptr->xpos=x;iptr->ypos=y;iptr->zpos=z;
		r->zmodh=iptr;
		r->zmodt=iptr;
		iptr->model=m;

		return (true);
		}


//	printf("CWorld:: InsertModel: Finding.\n");
	// given existing allocated object find the place to insert it on the xlist.
	foundx=false;
	px=r->xmodh;
	// is this the right place to insert on the x ?
	while ((px!=0)&&(!foundx)) { if ((x)<(px->xpos)) foundx=true; else px=px->next; }
	
	foundy=false;
	py=r->ymodh;
	// is this the right place to insert on the y ?
	while ((py!=0)&&(!foundy)) { if ((y)<(py->ypos)) foundy=true; else py=py->next; } 

	foundz=false;
	pz=r->zmodh;
	// is this the right place to insert on the z ?
	while ((pz!=0)&&(!foundz)) { if ((z)<(pz->zpos)) foundz=true; else pz=pz->next; }
	
//	if ((px==0)||(py==0)||(pz==0)) return false;

//	printf("CWorld:: InsertModel: Inserting on x.\n");
	// now we just need to check where in list are we inserting ?
	// at begining ? at end ? or in middle ? we need to check for each
	iptr = new CInode;
	iptr->xpos=x;iptr->ypos=y;iptr->zpos=z;
	if (iptr)
	{
		if (px==r->xmodh)
		{
			iptr->next  = px;
			iptr->prev=0;
			iptr->next->prev=iptr;
			r->xmodh=iptr;
			}
		else if (px==0)
		{
			iptr->prev = r->xmodt;
			iptr->next=0;
			iptr->prev->next=iptr;
			r->xmodt=iptr;
			}
		else 
		{
			// do x 
			iptr->next  = px;
			iptr->next->prev->next=iptr;
			iptr->prev=iptr->next->prev;
			iptr->next->prev=iptr;
			}
		}
	else return false;
	iptr->model = m;
	

	// next do y
//	printf("CWorld:: InsertModel: Inserting on y.\n");
	iptr = new CInode;
	iptr->xpos=x;iptr->ypos=y;iptr->zpos=z;
	if (iptr)
	{
		if (py==r->ymodh)
		{
			iptr->next  = py;
			iptr->prev=0;
			iptr->next->prev=iptr;
			r->ymodh=iptr;
			}
		else if (py==0)
		{
			iptr->prev = r->ymodt;
			iptr->next=0;
			iptr->prev->next=iptr;
			r->ymodt=iptr;
			}
		else 
		{
			// do y next
			iptr->next  = py;
			iptr->next->prev->next=iptr;
			iptr->prev=iptr->next->prev;
			iptr->next->prev=iptr;
			}
		}
	else return false;
	iptr->model = m;
	

	// next do z
//	printf("CWorld:: InsertModel: Inserting on z.\n");
	iptr = new CInode;
	iptr->xpos=x;iptr->ypos=y;iptr->zpos=z;
	if (iptr)
	{
		if (pz==r->zmodh)
		{
//			printf("CWorld:: InsertModel: Inserting on z first if.\n");
			iptr->next  = pz;
			iptr->prev=0;
			iptr->next->prev=iptr;
			r->zmodh=iptr;
			}
		else if (pz==0)
		{
//			printf("CWorld:: InsertModel: Inserting on z second  if.\n");
			iptr->prev = r->zmodt;
			iptr->next=0;
			iptr->prev->next=iptr;
			r->zmodt=iptr;
			}
		else 
		{
			// do z last
//			printf("CWorld:: InsertModel: Inserting on z last  if.\n");
			iptr->next  = pz;
//			printf("CWorld:: InsertModel: got here 1.\n");
			iptr->next->prev->next=iptr;
//			printf("CWorld:: InsertModel: got here 2.\n");
			iptr->prev=iptr->next->prev;
//			printf("CWorld:: InsertModel: got here 3.\n");
			iptr->next->prev=iptr;
//			printf("CWorld:: InsertModel: got here 4.\n");
			}
		}
	else return false;
	iptr->model = m;

	// include model
//	printf("CWorld:: InsertModel: Exiting.\n");
	return true;
	} ///////////////////////////////////////////////////////////////////////////end InsertModel

////////////////////////////////////////////////////////////////////////////////////////////////////
// CWorld InsertModel
// This is when we are adding a sub model to a model
////////////////////////////////////////////////////////////////////////////////////////////////////
//bool CWorld::InsertModel(CModel *m, CModel *pm)
//{
//CInode *iptr;
//
//	// given existing allocated object insert it.
//	iptr = new CInode;
//	iptr->nextnode  = pm->sublist;
//	iptr->model = m;
//	pm->sublist=iptr;
//	return true;
//	} ///////////////////////////////////////////////////////////////////////////////////end InsertModel

////////////////////////////////////////////////////////////////////////////////////////////////////
// CWorld AddToMovers
// This is when we are adding a sub model to a model
////////////////////////////////////////////////////////////////////////////////////////////////////
//bool CWorld::AddToMovers(CModel *m)
//{
//CInode *iptr;
//
//	// given existing allocated object insert it.
//	iptr = new CInode;
//	iptr->nextnode  = movers;
//	iptr->model = m;
//	movers=iptr;
//	return true;
//	} ///////////////////////////////////////////////////////////////////////////////////end AddToMovers

////////////////////////////////////////////////////////////////////////////////////////////////////
// CModel Print
////////////////////////////////////////////////////////////////////////////////////////////////////
//void CWorld::Print()
//{
//
//	// We'll use several versions of Print (overloaded) to traverse this very strange tree
//	// we've created.
//	if (regions) 
//	{
//		if (debug) MyLog("CWorld:: Print: Calling Print(regions) \n",logfilename);
//		Print(regions);
//		}
//	else MyLog("CWorld:: Print: Empty world.\n",logfilename);
//	} /////////////////////////////////////////////////////////////////////////////////////////end Print

////////////////////////////////////////////////////////////////////////////////////////////////////
// CModel Print
////////////////////////////////////////////////////////////////////////////////////////////////////
//void CWorld::Print(CRegion *p)
//{
//char buf[120];
//
//	// We'll use several versions of Print (overloaded) to traverse this very strange tree
//	// we've created.
//	if (p) 
//	{
//		if (debug) MyLog("CWorld:: Print: Calling Print(p->nextregion) \n",logfilename);
//		Print(p->nextregion);
//		}
//	else return;
//
//	// print the region
//	MyLog("======\n",logfilename);
//	_snprintf(buf,120,"Region x=%d, y=%d, z=%d. \n",p->x, p->y,p->z);
//	MyLog(buf,logfilename); 
//	
//	// Now search down the inode list and print models
//	if (debug) MyLog("Calling Print(p->nextnode) \n",logfilename);
//	Print(p->nextmodelnode,0);
//
//	} /////////////////////////////////////////////////////////////////////////////////////////end Print

////////////////////////////////////////////////////////////////////////////////////////////////////
// CModel Print
////////////////////////////////////////////////////////////////////////////////////////////////////
//void CWorld::Print(CInode *i,CModel *pm)
//{
//
//	// We'll use several versions of Print (overloaded) to traverse this very strange tree
//	// we've created.
//	if (i) 
//	{
//		if (debug) MyLog("Calling Print(i->nextnode) \n",logfilename);
//		Print(i->nextnode,pm);
//		}
//	else return;
//
//	// print the CModel
//	Print(i->model,pm);
//
//	} /////////////////////////////////////////////////////////////////////////////////////////end Print

////////////////////////////////////////////////////////////////////////////////////////////////////
// CModel Print
// This is basically a strange inorder traversal i think
////////////////////////////////////////////////////////////////////////////////////////////////////
//void CWorld::Print(CModel *m, CModel *pm)
//{
//char buf[80];
//
//	// print the model
//	if (!pm)
//	{
//		_snprintf(buf,80,"Model filename=%s. \n",m->myfilename.GetString());
//		MyLog(buf,logfilename);
//		}
//	else 
//	{
//		_snprintf(buf,80,"SubModel filename=%s. \n",m->myfilename.GetString());
//		MyLog(buf,logfilename);
//		}
//
//	// We'll use several versions of Print (overloaded) to traverse this very strange tree
//	// we've created.
//	// This should follow the submodel chain
//	if (m) if (m->sublist) 
//	{
//		if (debug) MyLog("Calling Print(m->submod) \n",logfilename);
//		Print(m->sublist,m);
//		}
//
//	// Now print down the partlist
//	if (m) if(m->partlist) 
//	{
//		if (debug) MyLog("Calling Print(m->partlist) \n",logfilename);
//		Print(m->partlist); 
//		}
//
//	} /////////////////////////////////////////////////////////////////////////////////end Print

////////////////////////////////////////////////////////////////////////////////////////////////////
// CModel Print
// This is basically a strange inorder traversal i think
////////////////////////////////////////////////////////////////////////////////////////////////////
//void CWorld::Print(CPart *p)
//{
//
//	// We'll use several versions of Print (overloaded) to traverse this very strange tree
//	// we've created.
//	if (p) 
//	{
//		if (debug) MyLog("===========================Calling Print(p->nextpart) \n",logfilename);
//		Print(p->nextpart);
//		}
//	else return;
//
//	// print the region
//	printf("Frame count = %d. \n",p->framecount);
//
//	} /////////////////////////////////////////////////////////////////////////////////end Print

////////////////////////////////////////////////////////////////////////////////////////////////////
// CRegion *FindRegion 
// Only use during Loading
// Assume that pointer ==0 if no region loaded
////////////////////////////////////////////////////////////////////////////////////////////////////
CRegion *CWorld::FindRegionUnsorted(int x, int y, int z)
{
int	i;

	for (i=0;i<NUMREGIONS;i++)
	{
		// see if the region exists, if it does, does it match ?
		if (regions[i])
		{
			if ( (((regions[i])->x)==x) && (((regions[i])->y)==y) && (((regions[i])->z)==z) )
			{
				return (regions[i]);
				}
			}
			// here we got to end of allocated regions and did not find the right one so return
			return (0);
		}
	// shouldnt get here except if all regions are allocated
	return (0);
	} ////////////////////////////////////////////////////////////////////////////////////end FindRegion

////////////////////////////////////////////////////////////////////////////////////////////////////
// Run
////////////////////////////////////////////////////////////////////////////////////////////////////
void CWorld::Run()
{
	simtime->Run();
//	updatetime.Run();
	} ///////////////////////////////////////////////////////////////////////////////////end Run

///////////////////////////////////////////////////////////////////////////////////////////////////
// SetSpeed
///////////////////////////////////////////////////////////////////////////////////////////////////
//void CWorld::SetSpeed(float s)
//{
//	movingchars[0].speed=s;
//	movingchars[0].vel.x=(float)(s*cos(PI/180*movingchars[0].dir));
//	movingchars[0].vel.z=(float)(s*sin(PI/180*movingchars[0].dir));
//
//	} /////////////////////////////////////////////////////////////////////////////////////end SetSpeed

///////////////////////////////////////////////////////////////////////////////////////////////////
// GetPos
///////////////////////////////////////////////////////////////////////////////////////////////////
//void CWorld::GetPos(float *x,float *y, float *z)
//{
//
//	*x=movingchars[0].pos.x;
//	*y=movingchars[0].pos.y;
//	*z=movingchars[0].pos.z;
//
//	} ///////////////////////////////////////////////////////////////////////////////////////end GetPos

///////////////////////////////////////////////////////////////////////////////////////////////////
// GetPosup
///////////////////////////////////////////////////////////////////////////////////////////////////
//void CWorld::GetPosup(posup *p)
//{
//
//	p->pos.x=movingchars[0].pos.x;
//	p->pos.y=movingchars[0].pos.y;
//	p->pos.z=movingchars[0].pos.z;
//	p->vel.x=movingchars[0].vel.x;
//	p->vel.y=movingchars[0].vel.y;
//	p->vel.z=movingchars[0].vel.z;
//	p->speed=movingchars[0].speed;
//	p->dir  =movingchars[0].dir;
//	p->elev =movingchars[0].elev;
//	p->id   =movingchars[0].id;
//
//	} ////////////////////////////////////////////////////////////////////////////////////end GetPosup

////////////////////////////////////////////////////////////////////////////////////////////////////
// PgDn
////////////////////////////////////////////////////////////////////////////////////////////////////
void CWorld::PgDn(bool t)
{
	pgdn=t;
	if (pgdn) pgup=false;
	} //////////////////////////////////////////////////////////////////////////////////end PgDn

////////////////////////////////////////////////////////////////////////////////////////////////////
// PgUp
////////////////////////////////////////////////////////////////////////////////////////////////////
void CWorld::PgUp(bool t)
{
	pgup=t;
	if (pgup) pgdn=false;
	} //////////////////////////////////////////////////////////////////////////////////end PgUp

////////////////////////////////////////////////////////////////////////////////////////////////////
// LevelLook
////////////////////////////////////////////////////////////////////////////////////////////////////
void CWorld::LevelLook()
{
	movingchars[0].elev=0.0;
	} /////////////////////////////////////////////////////////////////////////////end LevelLook

////////////////////////////////////////////////////////////////////////////////////////////////////
// RightArrow
////////////////////////////////////////////////////////////////////////////////////////////////////
void CWorld::RightArrow(bool t)
{
	rightarrow=t;
	if (rightarrow) 
	{
		leftarrow=false;
		}
	} ////////////////////////////////////////////////////////////////////////////end RightArrow

////////////////////////////////////////////////////////////////////////////////////////////////////
// LeftArrow
////////////////////////////////////////////////////////////////////////////////////////////////////
void CWorld::LeftArrow(bool t)
{
	leftarrow=t;
	if (leftarrow) 
	{
		rightarrow=false;
		}
	} /////////////////////////////////////////////////////////////////////////////end LeftArrow

////////////////////////////////////////////////////////////////////////////////////////////////////
// Del
////////////////////////////////////////////////////////////////////////////////////////////////////
void CWorld::Del(bool t)
{
	del=t;
	if (del) ins=false;
	} ///////////////////////////////////////////////////////////////////////////////////end Del

////////////////////////////////////////////////////////////////////////////////////////////////////
// Ins
////////////////////////////////////////////////////////////////////////////////////////////////////
void CWorld::Ins(bool t)
{
	ins=t;
	if (ins) del=false;
	} ///////////////////////////////////////////////////////////////////////////////////end Ins

////////////////////////////////////////////////////////////////////////////////////////////////////
// DnArrow
////////////////////////////////////////////////////////////////////////////////////////////////////
void CWorld::DnArrow(bool t)
{
	dnarrow=t;
	if (dnarrow) uparrow=false;
	movingchars[0].moving=dnarrow|uparrow;
	} ///////////////////////////////////////////////////////////////////////////////end DnArrow

////////////////////////////////////////////////////////////////////////////////////////////////////
// UpArrow
////////////////////////////////////////////////////////////////////////////////////////////////////
void CWorld::UpArrow(bool t)
{
	uparrow=t;
	if (uparrow) dnarrow=false;
	movingchars[0].moving=dnarrow|uparrow;
	} ///////////////////////////////////////////////////////////////////////////////end UpArrow

////////////////////////////////////////////////////////////////////////////////////////////////////
// BuildLookAt
////////////////////////////////////////////////////////////////////////////////////////////////////
//void CWorld::BuildLookAt(D3DXMATRIX	*pmatView)
//{
//	D3DXMatrixLookAtLH( pmatView, &vEyePt, &vLookAtPt, &vUp );
//	} //////////////////////////////////////////////////////////////////////////////////end BuildLookAt

////////////////////////////////////////////////////////////////////////////////////////////////////
// CWorld Render
// ok now walk down the fast region list and tell all models in fast region list to render 
// themselves 
//movingchars[0].pos.x,movingchars[0].pos.y,movingchars[0].pos.z,
////////////////////////////////////////////////////////////////////////////////////////////////////
int CWorld::Render()
{
int		i;
CInode		*p;
//CPortal		*po;
//D3DXMATRIX	mattemp,submatworld;
//
//	// get mutex
//	WaitForSingleObject(mutex,INFINITE);
//
	// setup for the render
//	printf("CWorld:: Render: Entering\n");
	CalcFrameRate();
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glLoadIdentity( );
	glRotatef(movingchars[0].elev, 1.0f, 0.0f , 0.0f); // Rotate Up And Down
	glRotatef(360.0f-movingchars[0].dir, 0.0f, 1.0f , 0.0f); // Rotate Direction
	glTranslatef(-movingchars[0].pos.x, -movingchars[0].pos.y, -movingchars[0].pos.z); // Translate
	glGetFloatv(GL_MODELVIEW_MATRIX,matrix);

	if (!inside)
	{
		for (i=0;i<fastregionscount;i++)
		{
			if(fastregions[i])
			{
//				printf("CWorld:: Render: Inside fastregions if\n");
				// walk down list of inodes and render all the models in region
				p=fastregions[i]->xmodh;	//DEBUG!!!!! CHANGEME
				while (p)
				{
					// Now render each model
					glLoadMatrixf(matrix);
					glRotatef(p->pitch, 1.0f, 0.0f , 0.0f);
					glTranslatef(p->xpos, p->ypos, p->zpos); // Translate
//					printf("CWorld:: Render: before render submodels\n");
					if (((p->model)->Render())!=CW_OK) 
					{
//						ReleaseMutex(mutex);
						return CW_ERR_RENDER;
						}
//					printf("CWorld:: Render: after render submodels\n");
					p=p->next;
					}

//				// walk down list of portals and render all in region
//				po=fastregions[i]->nextportalnode;
//				while (po)
//				{
//					// Now render each model
//					if (FAILED((po->model)->Render(pd3d,pmatview,pmatproj,&matworld))) 
//					{
//						ReleaseMutex(mutex);
//						return E_FAIL;
//						}
//					po=po->nextportal;
//					}
				} // end if fastregions
			} //end for each fastregion
		} //end if !inside
//	printf("CWorld:: Render: before SWAP BUFFERS\n");
//	else // we are inside
//	{
//	//	for (
//		}
//
//	// do the movers
//	for (i=1;i<MAXNUMOFVISIBLECHARS;i++) 
//	if (movingchars[i].model) 
//	{
//		if ((movingchars[i].active)&&(movingchars[i].render)) 
//		{
//			// Accumulate translations from higher up models
//			movingchars[i].model->xpos = movingchars[i].pos.x;
//			movingchars[i].model->ypos = movingchars[i].pos.y;
//			movingchars[i].model->zpos = movingchars[i].pos.z;
//			movingchars[i].model->yaw  = movingchars[i].dir  * pideg;
//
//			(movingchars[i].model)->Render(pd3d,pmatview,pmatproj,&matworld);
//			}
//		}
//
	textwindow->DrawTextWindow(windowwidth, windowheight);	// here we have to give the main windows values
								// because the ortho call needs the actual full screen width
								// and height, not that of the text window
	font->DrawTTText( 0, windowheight-10, windowwidth, windowheight, framestring);
	SDL_GL_SwapBuffers();
	return CW_OK;
	} ////////////////////////////////////////////////////////////////////////////////end Render

////////////////////////////////////////////////////////////////////////////////////////////////////
// CWorld Destroy
////////////////////////////////////////////////////////////////////////////////////////////////////
//HRESULT CWorld::Destroy()
//{
//int i;
//CInode *p;
//CPortal *po;
//	 
//	if (!inside)
//	{
//		for (i=0;i<NUMFASTREGIONS;i++)
//		{
//			if (fastregions[i])
//			{
//				// walk down list of inodes and print all the models in region
//				p=fastregions[i]->nextmodelnode;
//				while (p)
//				{
//					// Now render each model
//					if (FAILED((p->model)->Destroy())) return E_FAIL;
//					p=p->nextnode;
//					}
//
//				// walk down list of inodes and print all the portals in region
//				po=fastregions[i]->nextportalnode;
//				while (po)
//				{
//					// Now render each model
//					if (FAILED((po->model)->Destroy())) return E_FAIL;
//					po=po->nextportal;
//					}
//				}
//			}
//		}
//	else // we are inside
//	{
//	//	for (
//		}
//
//	// do the movers
//	p=movers;
//	while (p)
//	{
//		// do the movers models
//		if (FAILED((p->model)->Destroy())) return E_FAIL;
//		p=p->nextnode;
//		}
//
//	return S_OK;
//
//	} //////////////////////////////////////////////////////////////////////////////////////end Destroy

///////////////////////////////////////////////////////////////////////////////////////////////////
// CWorld MoversUpdate
///////////////////////////////////////////////////////////////////////////////////////////////////
//void CWorld::MoversUpdate(posup *p)
//{
//int	index;
//bool	found;
//char	buf[100];
//
//	// try to find the movers to update
//	WaitForSingleObject(mutex,INFINITE);
//	found=false;
//	index=0;
//
//	while ((index<MAXNUMOFVISIBLECHARS)&&(!found))
//	{
//		if (movingchars[index].id==p->id) found=true;	
//		else index++;
//		}
//
//	if (!found) 
//	{
//		_snprintf(buf,100,"CWorld:: MoversUpdate: NOTFOUND exiting. id=%d",p->id);
//		MyLog(buf,logfilename);
//		ReleaseMutex(mutex);
//		return;
//		}
//
//	if (movingchars[index].render)
//	{
//		if (debug) _snprintf(buf,100,"CWorld:: MoversUpdate: FOUND Udating. id=%d",p->id);
//		if (debug) MyLog(buf,logfilename);
//		// update the position and movement state 
//
//		movingchars[index].speed=p->speed;
//		movingchars[index].dir	=p->dir;
//		movingchars[index].pos.x=p->pos.x;
//		movingchars[index].pos.y=p->pos.y;
//		movingchars[index].pos.z=p->pos.z;
//		movingchars[index].active=true;
//		movingchars[index].render=true;
//
//		}
//	ReleaseMutex(mutex);
//	return;
//
//	} /////////////////////////////////////////////////////////////////////////////////end MoversUpdate

#ifdef CLIENT
///////////////////////////////////////////////////////////////////////////////////////////////////
// SendUpdateToServer
///////////////////////////////////////////////////////////////////////////////////////////////////
//void CWorld::SendUpdatesToServer()
//{
//char buf[80];
//int     code,count;
//bool    timedout;
//static bool moving=false;
//static bool turning=false;
//long	hitime, lotime;
//
//	// send only once a second
//	updatetime.GetElapsedNoReset(&hitime, &lotime);
//	if ((hitime>0)||(lotime>10000000))// 1.0 secs
//	{
//		
//		// send the updates
//		// are we moving forward ?
//		if (uparrow)
//		{
//			_snprintf(buf,80,"%s|%.1f|%.1f|%.1f|%.1f|%.1f|%.1f||",
//				TOKEN_FORWARD,
//				movingchars[0].pos.x,movingchars[0].pos.y,movingchars[0].pos.z,
//				movingchars[0].speed,movingchars[0].dir,movingchars[0].elev);
//			connectsock.Send(buf,strlen(buf),&code,&count,&timedout);
//			moving=true;
//		}
//
//		// are we backing down ?
//		if (dnarrow)
//		{
//			_snprintf(buf,80,"%s|%.1f|%.1f|%.1f|%.1f|%.1f|%.1f||",
//				TOKEN_BACKWARD,
//				movingchars[0].pos.x,movingchars[0].pos.y,movingchars[0].pos.z,
//				-movingchars[0].speed,movingchars[0].dir,movingchars[0].elev);
//			connectsock.Send(buf,strlen(buf),&code,&count,&timedout);
//			moving=true;
//		}
//
//		// have we stopped moving ?
//		if (!(uparrow||dnarrow)&&(moving))
//		{
//			_snprintf(buf,80,"%s|%.1f|%.1f|%.1f|%.1f|%.1f|%.1f|%.1f|%.1f|%.1f||",
//				TOKEN_STOPMOVE,
//				movingchars[0].pos.x,movingchars[0].pos.y,movingchars[0].pos.z,
//				0.0f,0.0f,0.0f,	
//				0.0f,movingchars[0].dir,movingchars[0].elev);	
//				connectsock.Send(buf,strlen(buf),&code,&count,&timedout);
//			moving=false;
//		}
//
//		// are we turning also ?
//		if (leftarrow)
//		{
//			_snprintf(buf,80,"%s|%.1f|%.1f|%.1f|%.1f|%.1f|%.1f||",
//				TOKEN_LEFT,
//				movingchars[0].pos.x,movingchars[0].pos.y,movingchars[0].pos.z,
//				movingchars[0].speed,movingchars[0].dir,movingchars[0].elev);
//			connectsock.Send(buf,strlen(buf),&code,&count,&timedout);
//			turning=true;
//		}
//
//		// are we turning the other way ?
//		if (rightarrow)
//		{
//			_snprintf(buf,80,"%s|%.1f|%.1f|%.1f|%.1f|%.1f|%.1f||",
//				TOKEN_RIGHT,
//				movingchars[0].pos.x,movingchars[0].pos.y,movingchars[0].pos.z,
//				movingchars[0].speed,movingchars[0].dir,movingchars[0].elev);
//			connectsock.Send(buf,strlen(buf),&code,&count,&timedout);
//			turning=true;
//		}
////
//		// have we stopped turning ?
//		if (!(leftarrow||rightarrow)&&(turning))
//		{
//			_snprintf(buf,80,"%s|%.1f|%.1f|%.1f|%.1f|%.1f|%.1f|%.1f|%.1f|%.1f||",
//				TOKEN_STOPTURN,
//				movingchars[0].pos.x,movingchars[0].pos.y,movingchars[0].pos.z,
//				0.0f,0.0f,0.0f,	
//				movingchars[0].speed,movingchars[0].dir,movingchars[0].elev);	
//				connectsock.Send(buf,strlen(buf),&code,&count,&timedout);
//			turning=false;
//		}
//		updatetime.GetElapsed(&hitime, &lotime);
//
//	}
//
//
//} //////////////////////////////////////////////////////////////////////////end SendUpdatesToServer
#endif


///////////////////////////////////////////////////////////////////////////////////////////////////
// Sim
// This is simulation thread for the display models
// remember that the actual game state is set by the server
// for now all we do is to update the fastregions list
///////////////////////////////////////////////////////////////////////////////////////////////////
void CWorld::Sim()
{
int	i;
int	x,y,z;

//
	DoMe();
//#ifdef CLIENT
//	SendUpdatesToServer();
//#endif
//	for (i=1;i<MAXNUMOFVISIBLECHARS;i++)
//	{
//		if (movingchars[i].active) SimMover(i);
//		}
	for (i=0;i<NUMREGIONS;i++)
	{
		// just find the current region
		
		}
	fastregions[0]=regions[0];
	fastregionscount=1;
	} ///////////////////////////////////////////////////////////////////////////////////end Sim

///////////////////////////////////////////////////////////////////////////////////////////////////
// SimMover
///////////////////////////////////////////////////////////////////////////////////////////////////
//void CWorld::SimMover(int i)
//{
//long hitime, lotime;
//float l;
//
//	simtime.GetElapsed(&hitime, &lotime);
//	l=	((float)lotime)/1000000.0f;
//
//	// ok remember that the speed is the speed IF we were moving,
//	// the dir is where we are facing
//	// the vx and vz are our ACTUAL velocity components RIGHT NOW 
//	// SO
//
//	movingchars[i].vel.x = (float)( movingchars[i].speed * cos(pideg*movingchars[i].dir));
//	movingchars[i].vel.z = (float)(-movingchars[i].speed * sin(pideg*movingchars[i].dir));
//
//	movingchars[i].pos.x=(float)(movingchars[i].pos.x+movingchars[i].vel.x*l);
////	movingchars[i].pos.y=(float)(movingchars[i].pos.y+movingchars[i].vel.y*l);
//	movingchars[i].pos.z=(float)(movingchars[i].pos.z+movingchars[i].vel.z*l);
//
//} /////////////////////////////////////////////////////////////////////////////////////end SimMover

///////////////////////////////////////////////////////////////////////////////////////////////////
// DoMe
///////////////////////////////////////////////////////////////////////////////////////////////////
void CWorld::DoMe()
{
long hitime, lotime;
//float l;
//float l5;

	// get elapsed times
	simtime->GetDelta(&hitime, &lotime);

	// Do turn
	if ((rightarrow==true)&&(leftarrow==false))
	{
		turnaccel=turnaccel+0.03;
		if (turnaccel>9.0) turnaccel=9.0;
		movingchars[0].dir -= (1.13f+turnaccel)*lotime/100000.0;
		}
	else if ((rightarrow==false)&&(leftarrow==true))	
	{
		turnaccel=turnaccel+0.03;
		if (turnaccel>9.0) turnaccel=9.0;
		movingchars[0].dir += (1.13f+turnaccel)*lotime/100000.0;
		}
	else if ((rightarrow==true)&&(leftarrow==true))
	{
		rightarrow=false;
		leftarrow=false;
                }
	else if ((rightarrow==false)&&(leftarrow==false)) turnaccel=0.0;

//	// calc new vx and vz
        // Do movement
	if (movingchars[0].moving)
	{
		movingchars[0].vel.x = (float)(-movingchars[0].speed * sin(pideg*movingchars[0].dir) * lotime/1000.0);
		movingchars[0].vel.z = (float)(-movingchars[0].speed * cos(pideg*movingchars[0].dir) * lotime/1000.0);
	}	
	else
	{
		movingchars[0].vel.x = 0.0f;
		movingchars[0].vel.z = 0.0f;
	}

	if (uparrow)
	{
		movingchars[0].pos.x=(movingchars[0].pos.x+movingchars[0].vel.x);
		movingchars[0].pos.z=(movingchars[0].pos.z+movingchars[0].vel.z);
		}

	if (dnarrow)
	{
		movingchars[0].pos.x=(movingchars[0].pos.x-movingchars[0].vel.x);
		movingchars[0].pos.z=(movingchars[0].pos.z-movingchars[0].vel.z);
		}

	if (ins) movingchars[0].pos.y=(movingchars[0].pos.y+0.01f);

	if (del) movingchars[0].pos.y=(movingchars[0].pos.y-0.01f);

	if (pgup) movingchars[0].elev =(float)(movingchars[0].elev-0.130); 
	if (movingchars[0].elev<-90.0) movingchars[0].elev=-90.0;

	if (pgdn) movingchars[0].elev=(float)(movingchars[0].elev+0.130); 
	if (movingchars[0].elev>90.0) movingchars[0].elev=90.0;

//	// set fastregions
//	// here we have to decide which regions we are in and which are close by
//	// Render uses this to render only those regions close to us.
//	// DEBUG for now we will cheat. And have Load hard code the fast regions for us.
//	
//
	return;
	} ///////////////////////////////////////////////////////////////////////////////////end Sim

////////////////////////////////////////////////////////////////////////////////////////////////////
// CWorld LoadHead
// Loads the small header and filters comments
////////////////////////////////////////////////////////////////////////////////////////////////////
int CWorld::LoadHead(FILE *f, CString **tok, int *val)
{
CString         line;
char            delimss[3]={' ','\t','\0'};  // <<===String contains a tab and a space

CWorldLoadHead1:
        // Get a Line
        if (line.GetLine(f))
        {
                // check if first chars in string are comment chars
                if (line.StrNCmp("//",2)==0)
                {
                        printf("FOUND COMMENT =>"); line.Display();// printf("\n");
                        goto CWorldLoadHead1;
                        }

                // check tokens to ensure format correct (FIXME)
                line.TokenInit(delimss);        // <<===String contains a tab and a space
                *tok=line.GetToken();
                line.GetInt(val);
                return (CW_OK);
                }
        else return (CW_ERR_CONVERSION);

        } //////////////////////////////////////////////////////////////////////////////end LoadHead

////////////////////////////////////////////////////////////////////////////////////////////////////
// CWorld LoadLine
////////////////////////////////////////////////////////////////////////////////////////////////////
int CWorld::LoadLine(FILE *f, char **token, char **modelname,
                        float *x, float *y, float *z,
                        float *roll,  float *pitch, float *yaw)
{
CString         line;
CString         *tok;
int             ind;
char            delimss[3]={' ','\t','\0'};  // <<===String contains a tab and a space

CWorldLoadLine1:

        // Get a Line
        if (line.GetLine(f))
        {
                // check if first chars in string are comment chars
                if (line.StrNCmp("//",2)==0)
                {
                        printf("FOUND COMMENT =>"); line.Display(); //printf("\n");
                        goto CWorldLoadLine1;
                        }

                // check tokens to ensure format correct (FIXME)
                line.Chomp();
                line.TokenInit(delimss);        // <<===String contains a tab and a space

                // extract tokens
                tok=line.GetToken();    tok->GetString(token); delete tok;// get first token saving type
                tok=line.GetToken();    tok->GetString(modelname); delete tok;// get first token saving type

                if (!line.GetFloat(x)) return (CW_ERR_CONVERSION);      // point x
                if (!line.GetFloat(y)) return (CW_ERR_CONVERSION);      // point y
                if (!line.GetFloat(z)) return (CW_ERR_CONVERSION);      // point z

                if (!line.GetFloat(roll)) return (CW_ERR_CONVERSION);   // roll
                if (!line.GetFloat(pitch)) return (CW_ERR_CONVERSION);  // pitch
                if (!line.GetFloat(yaw)) return (CW_ERR_CONVERSION);    // yaw

                return (CW_OK);
                }
        else return (CW_ERR_CONVERSION);
	} /////////////////////////////////////////////////////////////////////////////end LoadLine

///////////////////////////////////////////////////////////////////////////////////////////////////
// CWorld Load
// REGIONSIZE
// for each model in world.dat
// - read it in
// - put it in its region
// 
///////////////////////////////////////////////////////////////////////////////////////////////////
GLint CWorld::Load(char *filename)
{
FILE		*f;
char		buf[80];			// this is where tokens are returned to
//char		token[80],modelname[80];	// this is where tokens are returned to
unsigned long	startp=0,stopp=0;		// current pointer into bufptr
int		k,res;
int		x,y,z;				// indecies to determine the region
float		xpos,ypos,zpos;			// models position in world
float		roll,pitch,yaw;
CModel		*model;
CRegion		*region;
int		modelcount;
CString		*tok;
char            *token,*modelname;
 
	// this line is for debug
//	GetCurrentDirectory(80,tempbuf);
	printf("CWorld:: Load: =============LOADING WORLD================================================================ \n");
	// open file for reading
	if (!(f=fopen(filename,"r"))) return (CW_ERR_FILE_OPEN);

	// get the model count 
	if ((res=LoadHead(f, &tok, &modelcount))!=CW_OK) return (res);
//	if (treeadd) (*treeadd)(filename,hwndtv,1,(LPARAM)this);

	// the count simply tells us how many models to read in
	for (k=0;k<modelcount;k++)
	{
		snprintf(buf,80,"CWorld:: Load: =====>>>>>>Loading model # %d .\n",k);
		printf(buf);
		
		model	=new CModel;

		// get model def
                if (LoadLine(f, &token, &modelname,
                                &xpos,&ypos,&zpos,
                                &roll,&pitch,&yaw)!=CW_OK) return (CW_ERR_CONVERSION);

		model->Init();
		if ((model->Load(modelname,2))!=CW_OK) return (CW_ERR_FILE_OPEN);
//		printf("CWorld:: Load: After Loading model \n");

		// which region does he go in ?
		x=(int)(xpos/REGIONSIZE);
		y=(int)(ypos/REGIONSIZE);
		z=(int)(zpos/REGIONSIZE);

		// find that region
		if (region=FindRegionUnsorted(x,y,z))
		{
			// found his region, so add him
//			printf("CWorld:: Load: Inserting into existing region \n");
			InsertModel(model,region,xpos,ypos,zpos);
//			printf("CWorld:: Load: After Inserting into existing region \n");
			}

		else
		{
		int	i;

			// didnt find his region so create a new region
			// we're gonna cheat for DEBUG purposes and hard code the fast regions so we can tell
			// if the thing even renders
			// add a new region
			printf("CWorld:: Load: Making new region \n");
			region = new CRegion;
			region->SetIndex(x,y,z);
			printf("CWorld:: Load: After SetIndex \n");
			InsertRegion(region);
			printf("CWorld:: Load: After InsertRegion \n");
			InsertModel(model,region,xpos,ypos,zpos);
			printf("CWorld:: Load: After InsertModel \n");

//			InsertModel(model,region);
//			for (i=0;i<NUMFASTREGIONS;i++)
//			{
//				// just find a fast region that is unused and add it.
//				if (fastregions[i]==0) 
//				{
//					fastregions[i]=region;
//					goto Done;
//					}
//				}
//Done:
//			i++;	// DUMMY statement so compiler wont complain
			} // else
		} // for k=0 to model count
	printf("CWorld:: Load: After Model loop \n");
	
//
//	// now load the movers models
//	f/scanf(f,"%s %d", token,&modelcount);
//
//	// the count simply tells us how many models to read in
//	for (k=0;k<modelcount;k++)
//	{
//		model	=new CModel;
//		// get model def
//		f/scanf(f,"%s %s",token,modelname);
//////////		if (treeadd) (*treeadd)(modelname,hwndtv,1,(LPARAM)model);
//		model->Init(treeadd,hwndtv);
//		if (!(model->Load(modelname, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0,2))) return false;
//		AddToMovers(model);
//		}
//
//
	fclose (f);
//
	printf("Exiting CWorld::Load  \n");
	return CW_OK;
	} //////////////////////////////////////////////////////////////////////////////////end Load

////////////////////////////////////////////////////////////////////////////////////////////////////
// SetObjID
////////////////////////////////////////////////////////////////////////////////////////////////////
//void CWorld::SetObjID(CString *o)
//{
//
//	// movingchars[0] is for the char that this instance of the client is playing
//	// we get our present position et al from this charstate
//	movingchars[0].id=atoi(o->GetString());
//	movingchars[0].render=true;
//	movingchars[0].elev=90.0;
//} /////////////////////////////////////////////////////////////////////////////////////end SetObjid

///////////////////////////////////////////////////////////////////////////////////////////////////
// SetModel
///////////////////////////////////////////////////////////////////////////////////////////////////
//void CWorld::SetModel(int objid, CString *s)
//{
//CInode	*p;
//bool	found;
//int	i;
//char	buf[100];
//
//	// Start at beginning of list, find the model matching s or add it
////////	WaitForSingleObject(mutex,INFINITE);
//	found=false;
//	p=movers;
//	while ((p)&&(!found))
//	{
//		_snprintf(buf,100,"CWorld:: SetModel: Comparing myfilename='%s' and *s='%s'",
//							p->model->myfilename.GetString(),s->GetString());
////		MyLog(buf,logfilename);
//		if (((p->model->myfilename)==*s)==0) 
//		{
//			found=true;
//			_snprintf(buf,100,"CWorld:: SetModel: FOUND  model='%s'",s->GetString());
//			MyLog(buf,logfilename);
//		}
//		else p=p->nextnode;
//	}
//	if ((!p)&&(!found))
//	{
//		// now we have to load the model
//			_snprintf(buf,100,"CWorld:: SetModel: NOTFOUND  model='%s'",s->GetString());
//			MyLog(buf,logfilename);
//	}
//
//	if (!found) 
//	{
////		ReleaseMutex(mutex);
//		return;
//	}
//
//	if (p && found)
//	{
//		// now look for objid to assign it to
//		i=0;
//		found=false; // Reusing found
//		while ((i<MAXNUMOFVISIBLECHARS)&&(!found))
//		{
//			if (movingchars[i].id==objid) found=true;
//			else i++;
//		}
//		if ((i<MAXNUMOFVISIBLECHARS)&&(found))
//		{
//			movingchars[i].model=p->model;
//			movingchars[i].active=true;
//			movingchars[i].render=true;
//		}
//	}
//
////////	ReleaseMutex(mutex);
//
//} /////////////////////////////////////////////////////////////////////////////////////end SetModel

///////////////////////////////////////////////////////////////////////////////////////////////////
// NewMover
///////////////////////////////////////////////////////////////////////////////////////////////////
//void CWorld::NewMover(posup *p)				// Add a new mover for simulation & display
//{
//int	i;
//
//	// new mover
//////	WaitForSingleObject(mutex,INFINITE);
//	for (i=0;i<MAXNUMOFVISIBLECHARS;i++)
//	{
//		// find an empty slot (!active)
//		if (movingchars[i].id==p->id)
//		{
//////////			ReleaseMutex(mutex);
//			return;
//		}
//
//		if (!movingchars[i].active)
//		{
//			movingchars[i].id=p->id;
//			movingchars[i].active=true;
//			movingchars[i].render=true;
//			movingchars[i].pos.x=p->pos.x;
//			movingchars[i].pos.y=p->pos.y;
//			movingchars[i].pos.z=p->pos.z;
//			movingchars[i].vel.x=p->vel.x;
//			movingchars[i].vel.y=p->vel.y;
//			movingchars[i].vel.z=p->vel.z;
//			movingchars[i].dir=p->dir;
//			movingchars[i].elev=p->elev;
//			movingchars[i].model=0;
//////////			ReleaseMutex(mutex);
//			return;
//		}
//	}
//	ReleaseMutex(mutex);
//
//	} /////////////////////////////////////////////////////////////////////////////////////end NewMover

///////////////////////////////////////////////////////////////////////////////////////////////////
// Name: CWorld Save
// Description : Saves world to file parm, and causes all models and submods and parts and lists
// to save as well
///////////////////////////////////////////////////////////////////////////////////////////////////
//bool CWorld::Save(char *filename)
//{
//FILE		*f;
//CRegion		*region;
//CInode		*p;
//CPortal		*po;
//int		count,i;
// 
//	// open file for reading
//	if (!(f=fopen(filename,"w"))) return (false);
//
//	// we arent going to do any treeview stuff during save. Any changes made to treeview
//	// control (ordering, filenames, etc, will be assumed to be already put into world structure
//	// before Save is called
////////	if (treeadd) (*treeadd)(filename,hwndtv,1,(LPARAM)this);
//
//	///////////////////////////////////////////////////
//	// count the top level models to save in the world.dat file
//	///////////////////////////////////////////////////
//	count=0;
//	region=regions;
//	while (region)
//	{
//		// walk down list of inodes and render all the models in region
//		p=region->nextmodelnode;
//		while (p)
//		{
//			if (p->model) count++;
//			p=p->nextnode;
//		}
//		region=region->nextregion;
//	}
//
//	// save the toplevel model count 
//	fprintf(f,"%s   %d\n", "[model]",count);
//
//	// now save all the toplevel models (and their submodels
//	region=regions;
//	while (region)
//	{
//		// walk down list of inodes and save all the models in this region
//		p=region->nextmodelnode;
//		while (p)
//		{
//			// Now save each model name and offset and such
//			fprintf(f,"%s %s %.2f %.2f %.2f %.2f %.2f %.2f\n",
//				"[mod]",p->model->myfilename.GetString(),
//				p->model->xpos,p->model->ypos,p->model->zpos,
//				p->model->roll,p->model->pitch,p->model->yaw);
//			
//			if (!((p->model)->Save())) 
//			{
//				return false;
//			}
//			p=p->nextnode;
//		}
//		region=region->nextregion;
//	}
//
//	///////////////////////////////////////////////////
//	// count the mover models to save
//	///////////////////////////////////////////////////
//	count=0;
//	p=movers;
//	while (p)
//	{
//		if (p->model) count++;
//		p=p->nextnode;
//	}
//
//	// save the mover model count 
//	fprintf(f,"%s   %d\n", "[moverscount]",count);
//
//	// save movers
//	p=movers;
//	while (p)
//	{
//			// Now save each model name and offset and such
//			fprintf(f,"%s %s %.2f %.2f %.2f %.2f %.2f %.2f\n",
//				"[mod]",p->model->myfilename.GetString(),
//				p->model->xpos,p->model->ypos,p->model->zpos,
//				p->model->roll,p->model->pitch,p->model->yaw);
//			
//			if (!((p->model)->Save())) 
//			{
//				return false;
//			}
//			p=p->nextnode;
//	}
//
//	///////////////////////////////////////////////////
//	// count the room models to save
//	///////////////////////////////////////////////////
//	// save the room model count 
//	if (xindex)
//	{
//		fprintf(f,"%s   %d\n", "[roomcount]",xindex->roomcount);
//
//		// save rooms
//	//	p=nextmodel;
//		for (i=0;i<xindex->roomcount;i++)
//		{
//				// Now save each model name and offset and such
//				fprintf(f,"%s %s %.2f %.2f %.2f %.2f %.2f %.2f\n",
//					"[room]",
//					xindex->rooms[i]->roomname.GetString(),
//					xindex->rooms[i]->roomfilename.GetString(),
//					xindex->rooms[i]->xpos,
//					xindex->rooms[i]->ypos,
//					xindex->rooms[i]->zpos,
//					xindex->rooms[i]->roll,
//					xindex->rooms[i]->pitch,
//					xindex->rooms[i]->yaw);
//				if (!((p->model)->Save())) 
//				{
//					return false;
//				}
//				p=p->nextnode;
//		}
//	}
//	else
//	{
//		fprintf(f,"%s   %d\n", "[roomcount]",0);
//	}
//
//	///////////////////////////////////////////////////
//	// count the portal models to save
//	///////////////////////////////////////////////////
//	count=0;
//	region=regions;
//	while (region)
//	{
//		// walk down list of inodes and render all the models in region
//		po=region->nextportalnode;
//		while (po)
//		{
//			if (po) count++;
//			po=po->nextportal;
//		}
//		region=region->nextregion;
//	}
//
//	// save the mover model count 
//	fprintf(f,"%s   %d\n", "[moverscount]",count);
//
//	// save movers
//	region=regions;
//	while (region)
//	{
//		po=region->nextportalnode;
//		while (po)
//		{
//			// Now save each model name and offset and such
//			fprintf(f,"%s %s %s %.2f %.2f %.2f %.2f %.2f %.2f\n",
//				"[portal]",
//				po->portalname.GetString(), 
//				po->portalfilename.GetString(),
//				po->model->xpos,po->model->ypos, po->model->zpos,
//				po->model->roll,po->model->pitch,po->model->yaw);
//			
//			if (!((po->model)->Save())) 
//			{
//				return false;
//			}
//			po=po->nextportal;
//		}
//		region=region->nextregion;
//	}
//
//	// save room connect
//
//
//	fclose (f);
//
//	return true;
//	} /////////////////////////////////////////////////////////////////////////////////////////end Save

////////////////////////////////////////////////////////////////////////////////////////////////////
// Here goes our drawing code
////////////////////////////////////////////////////////////////////////////////////////////////////
int CWorld::CalcFrameRate()
{
int t;
                                                                                                                                                  
	// Gather our frames per second
	frames++;
	{
	GLint t = SDL_GetTicks();
		if (t - t0 >= 5000)
		{
		GLfloat seconds = (t - t0) / 1000.0;
		GLfloat fps = frames / seconds;
			snprintf(framestring,100,"%d frames in %.2f seconds = %.1f FPS", frames, seconds, fps);
			t0 = t;
			frames = 0;
			}
		}

	return( true );
	} /////////////////////////////////////////////////////////////////////////////end DrawWorld

////////////////////////////////////////////////////////////////////////////////////////////////////
// Shift
////////////////////////////////////////////////////////////////////////////////////////////////////
void CWorld::Shift(bool t)
{
	// just chain to cstring call
	shift=t;
	} /////////////////////////////////////////////////////////////////////////////////end Shift
 
////////////////////////////////////////////////////////////////////////////////////////////////////
// Return
////////////////////////////////////////////////////////////////////////////////////////////////////
void CWorld::Return()
{
	// just chain to cstring call
//	commandstring->Chop();
	textwindow->Return();
	} ////////////////////////////////////////////////////////////////////////////////end Return
 
////////////////////////////////////////////////////////////////////////////////////////////////////
// Chop
////////////////////////////////////////////////////////////////////////////////////////////////////
void CWorld::Chop()
{
	// just chain to cstring call
//	commandstring->Chop();
	textwindow->Chop();
	} //////////////////////////////////////////////////////////////////////////////////end Chop
 
////////////////////////////////////////////////////////////////////////////////////////////////////
// Append
////////////////////////////////////////////////////////////////////////////////////////////////////
void CWorld::Append(char addition)
{
//char	c;
char	buf[80];

	// just chain to cstring call
	if ((shift)&&(addition>='a')&&(addition<='z') )
	{
		textwindow->InsertChar(addition-32);
		}
	else 
	{
		textwindow->InsertChar(addition);
		}
	} ////////////////////////////////////////////////////////////////////////////////end Append
 
////////////////////////////////////////////////////////////////////////////////////////////////////
// SetWindow
////////////////////////////////////////////////////////////////////////////////////////////////////
void CWorld::SetWindow(int w, int h)
{
	windowwidth=w;
	windowheight=h;
	} /////////////////////////////////////////////////////////////////////////////end SetWindow

////////////////////////////////////////////////////////////////////////////////////////////////////
// SetTextWinSize
////////////////////////////////////////////////////////////////////////////////////////////////////
void CWorld::SetTextWinSize(int w, int h)
{
	textwindow->SetTextWinSize(w,h);
	} /////////////////////////////////////////////////////////////////////////////end SetWindow

////////////////////////////////////////////////////////////////////////////////////////////////////
// GetSock
////////////////////////////////////////////////////////////////////////////////////////////////////
CUSock *CWorld::GetSock()
{
	return(sock);
	} ///////////////////////////////////////////////////////////////////////////////end GetSock

////////////////////////////////////////////////////////////////////////////////////////////////////
// SetSock
////////////////////////////////////////////////////////////////////////////////////////////////////
void CWorld::SetSock(CUSock *s)
{
	sock=s;
	} ///////////////////////////////////////////////////////////////////////////////end SetSock

////////////////////////////////////////////////////////////////////////////////////////////////////
// InsertLine
////////////////////////////////////////////////////////////////////////////////////////////////////
void CWorld::InsertLine(CString *s)
{
	textwindow->InsertLine(s);

	} ////////////////////////////////////////////////////////////////////////////end InsertLine

////////////////////////////////////////////////////////////////////////////////////////////////////
// InsertLine
////////////////////////////////////////////////////////////////////////////////////////////////////
void CWorld::InsertLine(char *s)
{
	textwindow->InsertLine(s);

	} ////////////////////////////////////////////////////////////////////////////end InsertLine




