/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Merged
// parse.cpp
//
// These routines parse the incoming bytes stream from the clients and prepare 
// it for easy reading by the worker threads
//
// by Eric J Krejci
// Copyright (C) 1999 Eric J Krejci 
// Revised : 08 May 2006
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation version 2.
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
// Contact:
//              email           worldii@viagloria.com
//              web             www.viagloria.com
/////////////////////////////////////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Includes
/////////////////////////////////////////////////////////////////////////////////////////////////////////////
#include <pthread.h>
#include <errno.h>
#include <string.h>
#include "wiii_util.h"
#include "cmessage.hpp"
#include "cpool.hpp"

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Globals
/////////////////////////////////////////////////////////////////////////////////////////////////////////////
extern	void	ThreadLog(char *s);
extern	void	WorkerSend(void *ptr);
extern	void	Handler(void *ptr);
extern  int             loglevel;

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Local Globals
/////////////////////////////////////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Defines
/////////////////////////////////////////////////////////////////////////////////////////////////////////////
#define		MINPRINTABLE    ' '
#define		MAXPRINTABLE    '~'

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Enqueue 
// for now this will just print out the contents of the CMessage
/////////////////////////////////////////////////////////////////////////////////////////////////////////////
bool Enqueue(CMessage *m, CPool *pool, CServer *s)
{
	if (loglevel>=1) ThreadLog("Enqueue: entered.");
	if (m->CheckMessage())
	{
		s->SetMessage(m);
		pool->AddWork(Handler,s,0,0);		// for now dont use last two parms
		}
	if (loglevel>=1) ThreadLog("Enqueue: exit.");
	return (true);
	} ////////////////////////////////////////////////////////////////////////////////////////end Enqueue


/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Parse
// FIXME when getting a parm (password in this case under playerid) and the message doesn't end with 
//       the delimiter '||' the message should be discarded. it is not.
//
/////////////////////////////////////////////////////////////////////////////////////////////////////////////
int Parse(char *rawbuf,int length, CPool *pool, CServer *s)
{
CMessage        *msg;
int             cmdstate=0;             // Tells wether we are getting cmd or parms
int             p;                      // A position pointer into rawbuf
int             tp;                     // A trailing position pointer into rawbuf
char            delim;              // The delimiter
char            buf[100];
					// once we have finished examining a rawbuf

	// This will be a state machine
	// but we will strive to read the message just once.
	delim=CHAR_DELIM;
	if (loglevel>=1) ThreadLog("Parse: Entering Parse.");
	goto Init;	// stupid compiler warning, mutter, mutter . . .

Init:
	if (length==0) goto Done;	// handle the degenerate case
	if (loglevel>=2) 
	{
		snprintf(buf,100,"Parse: Init. rawbuf='%s'",rawbuf); 	
		ThreadLog(buf);
		}

	p=0;
	tp=0;
	cmdstate=0;
	length=strlen(rawbuf);  // remember, for now all valid data is printable, and null termed.
	msg = new CMessage;

GetChars:
	if (p>length) goto Done;

	//filter out non printable chars
	if ((rawbuf[p]<MINPRINTABLE) || (rawbuf[p]>MAXPRINTABLE)) rawbuf[p]=' ';
	if (rawbuf[p]!=delim)
	{
		p++;
		goto GetChars;
		}
	// Note intentional fall through
	goto LookAhead;	// stupid compiler warning, mutter, mutter . . .

LookAhead:
	if (loglevel>=1) ThreadLog("Parse: LookAhead.");
        if (p+1>length) goto Done;
        if (rawbuf[p+1]!=delim) // is there a second delim ?
        {
                if (cmdstate==0) goto CheckProperCmd;
                if (cmdstate==1) goto GettingParm;
        }
        if ((cmdstate==0)&&(rawbuf[p+1]==delim)) goto CheckProperCmdNoParm;     // is there a second delim ?
        // Note intentional fall through
	goto GettingLastParm;	// stupid compiler warning, mutter, mutter . . .

GettingLastParm:
	if (loglevel>=1) ThreadLog("Parse: GettingLastParm.");
        if (!msg->AddParmToMsg(p,tp,rawbuf)) 
	{
		if ((msg)) delete msg;
		return (1);
		}
        // Note intentional fall through
            
CheckCmdComplete:
	if (loglevel>=1) ThreadLog("Parse: CheckCmdComplete.");
        if (msg->CheckMessage()) goto InsertCmdIntoQueue;
	else 
	{
		msg->SetUnhandled();
		msg->SetErrorType(CMSG_PARMCNT);
		goto InsertCmdIntoQueue;
		}
	// There is no fall through here
         
CheckProperCmd:
	if (loglevel>=1) ThreadLog("Parse: CheckProperCmd.");
        if (msg->CheckProperCmd(p,tp,rawbuf))
        {
                cmdstate=1;	// ok now we are getting parms 
                p++;
                tp=p;
                goto GetChars;
		}
        goto DiscardToDouble;

CheckProperCmdNoParm:
	if (loglevel>=1) ThreadLog("Parse: CheckProperCmdNoParm.");
	if (msg->CheckProperCmd(p,tp,rawbuf))
        {
                cmdstate=0;
                goto CheckCmdComplete;
        	}
        // Note intentional fall through

DiscardToDouble:
	if (loglevel>=1) ThreadLog("Parse: DiscardToDouble.");
        // Not a proper cmd so flush to next double delim or end of rawbuf
	// and send Unhandled command
        while ((p<length)&&(rawbuf[p]!=delim)) p++;
        if ((p<length-2) && (rawbuf[p]==delim) && (rawbuf[p+1]==delim))
        {
                p++;
                p++;
                tp=p;           // set tp and p to the next char to consider, skipping the current delim
                cmdstate=0;	// now we are getting commands
		//new stuff
		msg->SetUnhandled();
		msg->SetErrorType(CMSG_CMDUNRQ);
		goto InsertCmdIntoQueue;
        }
        else p++;
        if (p>=length) goto Done;
        goto DiscardToDouble;

GettingParm:
	if (loglevel>=1) ThreadLog("Parse: GettingParm.");
        if (!msg->AddParmToMsg(p,tp,rawbuf)) 
	{
		if ((msg)) delete msg;
		return (1);
		}
        p++;
        tp=p;
        goto GetChars;

InsertCmdIntoQueue:
	if (loglevel>=1) ThreadLog("Parse: InsertCmdIntoQueue.");
//	if (!Enqueue(msg,loglevel,pool,s)) return(2);
	if (!Enqueue(msg,pool,s)) return(2);
        else
        {
                p++;
                p++;
                tp=p;
                cmdstate=0;
                msg = new CMessage;
                goto GetChars;
        }
Done:
	if ((msg)) delete msg;
        if (loglevel>=1) ThreadLog("Parse: Exiting.");
        return(p);      // dont really need to do this, but the compiler wanted it.
                                                                                                                                            
} ///////////////////////////////////////////////////////////////////////////////////end Parse
                                                                                                                                            






