/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Merged
// util.cpp
//
// this is to have some useful debugging functions
//
// by Eric J Krejci
// Copyright (C) 1999 Eric J Krejci 
// Revised : 22 May 2007
// 
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation version 2.
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
// Contact:
//              email           worldii@viagloria.com
//              web             www.viagloria.com
/////////////////////////////////////////////////////////////////////////////////////////////////////////////

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Includes
/////////////////////////////////////////////////////////////////////////////////////////////////////////////
#include <pthread.h>
#include <curses.h>
#include <errno.h>
#include <string.h>
#include <sys/timeb.h>
#include "wiii_structures.h"
#include "wiii_util.h"
#include "cusock.hpp"
#include "cstring.hpp"
#include "cengine.hpp"

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Globals
/////////////////////////////////////////////////////////////////////////////////////////////////////////////
extern  FILE		*logfile;
extern  char		logfilename[];
extern  unsigned long	logcount;
extern  CUSock		*newconsock;
extern  bool		consoleconnected;
extern struct timeb	starttime;
extern  pthread_mutex_t	screenmutex;
extern  pthread_mutex_t	logmutex;
extern	int		loglevel;               // sets level of logging 0 lowest= no logging,
                                                // 9 highest insanely verbose


/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// ThreadLog
/////////////////////////////////////////////////////////////////////////////////////////////////////////////
void ThreadLog(CString *cs)
{
#ifdef	LINUX
pthread_t	threadnum;
#endif
#ifdef SOLARIS
thread_t	threadnum;
#endif
char		logfilename[200];
char		*s;
struct timeb	t;
FILE		*logfile;

	pthread_mutex_lock(&logmutex);
        threadnum=pthread_self();
	ftime(&t);

#ifdef	LINUX
	snprintf(logfilename,200,"logs/%lu.log",threadnum);
#endif
#ifdef SOLARIS
	snprintf(logfilename,200,"logs/%u.log",threadnum);
#endif
	logfile=fopen(logfilename,"a");
	s=(cs->GetStringPtr());
	if (logfile==0)
	{
		// capture errno
		printf("errno=%d",errno);
		exit (7);
		}

	fprintf(logfile,"%09lu:%lu.%03lu:%09lu %s",
			logcount,
			(unsigned long)(t.time-starttime.time),
			(unsigned long)t.millitm,
			(unsigned long)pthread_self(),s);
//	delete [] s;
	logcount++;
	fclose(logfile);
	pthread_mutex_unlock(&logmutex);
	} //////////////////////////////////////////////////////////////////////////////////////end ThreadLog

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// ThreadLog
/////////////////////////////////////////////////////////////////////////////////////////////////////////////
void ThreadLog(char *s)
{
#ifdef	LINUX
pthread_t	threadnum;
#endif
#ifdef SOLARIS
thread_t	threadnum;
#endif
char		logfilename[200];
struct timeb	t;
FILE		*logfile;

	pthread_mutex_lock(&logmutex);
	threadnum=pthread_self();
	ftime(&t);

#ifdef	LINUX
	snprintf(logfilename,200,"logs/%lu.log",threadnum);
#endif
#ifdef SOLARIS
	snprintf(logfilename,200,"logs/%u.log",threadnum);
#endif

	logfile=fopen(logfilename,"a");
	if (logfile==0)
	{
		// capture errno
		printf("errno=%d",errno);
		exit (7);
		}

//	fprintf(logfile,"%09lu:%lu.%03lu:%09lu %s\n",
	fprintf(logfile,"%09lu:%lu.%03lu:%09lu %s",
			logcount,
			(unsigned long)(t.time-starttime.time),
			(unsigned long)t.millitm,
			(unsigned long)pthread_self(),s);
	logcount++;
	fclose(logfile);
	pthread_mutex_unlock(&logmutex);
	} //////////////////////////////////////////////////////////////////////////////////////end ThreadLog

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// MyPrintInt
/////////////////////////////////////////////////////////////////////////////////////////////////////////////
int MyPrintInt(WINDOW *w, char *s, bool wait)
{

	if (wait)
	{
		pthread_mutex_lock(&screenmutex);
		wprintw(w,"%s",s);wrefresh(w);
		pthread_mutex_unlock(&screenmutex);
		return(0);
		}
	else
	{
		if (pthread_mutex_trylock(&screenmutex)==0)
		{
			wprintw(w,"%s",s);wrefresh(w);
			pthread_mutex_unlock(&screenmutex);
			return(0);
			}
		else return(1); // this says that it was busy
		}
	} //////////////////////////////////////////////////////////////////////////////////////end MyPrint

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// MyPrint
// Overloaded function
/////////////////////////////////////////////////////////////////////////////////////////////////////////////
int MyPrint(WINDOW * w, char *s, bool wait)
{
	return(MyPrintInt(w,s,wait));
	} ////////////////////////////////////////////////////////////////////////////////////////end MyPrint

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// MyPrint
// Overloaded function
/////////////////////////////////////////////////////////////////////////////////////////////////////////////
int MyPrint(WINDOW * w, CString *s, bool wait)
{
	return(MyPrintInt(w,s->GetStringPtr(),wait));
	} ////////////////////////////////////////////////////////////////////////////////////////end MyPrint

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// MyLocPrint
/////////////////////////////////////////////////////////////////////////////////////////////////////////////
void MyLocPrint(WINDOW * w, int x, int y, char * s )
{
     pthread_mutex_lock(&screenmutex);
     mvwprintw(w,x,y,s);
     wrefresh(w);
     pthread_mutex_unlock(&screenmutex);
     } ////////////////////////////////////////////////////////////////////////////////////////end MyLocPrint

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// MyReadLock
// This is a wrapper for pthread lock that can print tid if required
/////////////////////////////////////////////////////////////////////////////////////////////////////////////
int MyReadLock(pthread_rwlock_t *rwlock, bool MyP, char * who)
{
char buf[100];
int r;

	r=pthread_rwlock_rdlock(rwlock);
	// print tid then return the call
	if (loglevel>=9)  
	{
		snprintf(buf,100,"Thread:: %09lu READ locked rwlock.    Caller = '%s'   Success = %d. RWLOCK=%lu",
			(unsigned long)pthread_self(),who,r,(unsigned long)rwlock);
		ThreadLog(buf);
		}
	if (r) PError(errno);
	return (r);
	} /////////////////////////////////////////////////////////////////////////////////////end MyReadLock

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// MyWriteLock
// This is a wrapper for pthread lock that can print tid if required
/////////////////////////////////////////////////////////////////////////////////////////////////////////////
int MyWriteLock(pthread_rwlock_t *rwlock, bool MyP, char * who)
{
char buf[100];
int r;

	r=pthread_rwlock_wrlock(rwlock);
	// print tid then return the call
	if (loglevel>=9)  
	{
		snprintf(buf,100,"Thread:: %09lu WRITE locked rwlock.   Caller = '%s'   Success = %d. RWLOCK=%lu",
			(unsigned long)pthread_self(),who,r,(unsigned long)rwlock);
		ThreadLog(buf);
		}
	if (r) PError(errno);
	return (r);
	} ////////////////////////////////////////////////////////////////////////////////////end MyWriteLock

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// MyTryWriteLock
// This is a wrapper for pthread lock that can print tid if required
/////////////////////////////////////////////////////////////////////////////////////////////////////////////
int MyTryWriteLock(pthread_rwlock_t *rwlock, bool MyP, char * who)
{
char buf[100];
int r;

	r=pthread_rwlock_trywrlock(rwlock);

	// if we get EBUSY it means that someone else already holds the writelock
	// print tid then return the call
	if (r==EBUSY) 
	{
		if (loglevel>=9)  
		{
			snprintf(buf,100,"Thread:: %09lu TRYWRITE FAILED on rwlock.   Caller = '%s'   Code = %d. RWLOCK=%lu",
				(unsigned long)pthread_self(),who,r,(unsigned long)rwlock);
			ThreadLog(buf);
			}
		return (EBUSY);
		}

	// print tid then return the call
	if (loglevel>=9)  
	{
		snprintf(buf,100,"Thread:: %09lu WRITE locked rwlock.   Caller = '%s'   Success = %d. RWLOCK=%lu",
			(unsigned long)pthread_self(),who,r,(unsigned long)rwlock);
		ThreadLog(buf);
		}
	if (r) PError(errno);
	return (r);
	} /////////////////////////////////////////////////////////////////////////////////end MyTryWriteLock

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// MyRWUnLock
// This is a wrapper for pthread lock that can print tid if required
/////////////////////////////////////////////////////////////////////////////////////////////////////////////
int MyRWUnLock(pthread_rwlock_t *rwlock, bool MyP, char * who)
{
char buf[100];
int r;

	r=pthread_rwlock_unlock(rwlock);
	// print tid then return the call
	if (loglevel>=9)
	{
		snprintf(buf,100,"Thread:: %09lu UNLOCKING rwlock. Caller = '%s' Success = %d. RWLOCK=%lu",
			(unsigned long)pthread_self(),who,r,(unsigned long)rwlock);
		ThreadLog(buf);
		}
	if (r) PError(errno);
	return (r);
	} /////////////////////////////////////////////////////////////////////////////////////end MyRWUnLock

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// MyLock
// This is a wrapper for pthread lock that can print tid if required
/////////////////////////////////////////////////////////////////////////////////////////////////////////////
int MyLock(pthread_mutex_t *mutex, bool MyP, char * who)
{
char buf[100];
int r;

	r=pthread_mutex_lock(mutex);
	// print tid then return the call
	if (loglevel>=9)  
	{
		snprintf(buf,100,"Thread:: %09lu locked mutex.    Caller = '%s'   Success = %d. MUTEX=%lu",
			(unsigned long)pthread_self(),who,r,(unsigned long)mutex);
		ThreadLog(buf);
		}
	if (r) PError(errno);
	return (r);
	} /////////////////////////////////////////////////////////////////////////////////////////end MyLock

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// MyUnLock
// This is a wrapper for pthread lock that can print tid if required
/////////////////////////////////////////////////////////////////////////////////////////////////////////////
int MyUnLock(pthread_mutex_t *mutex, bool MyP, char * who)
{
char buf[100];
int r;

	r=pthread_mutex_unlock(mutex);
	// print tid then return the call
	if (loglevel>=9)
	{
		snprintf(buf,100,"Thread:: %09lu unlocking mutex. Caller = '%s' Success = %d. MUTEX=%lu",
			(unsigned long)pthread_self(),who,r,(unsigned long)mutex);
		ThreadLog(buf);
		}
	if (r) PError(errno);
	return (r);
	} ///////////////////////////////////////////////////////////////////////////////////////end MyUnLock

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// CopyStrip
// Copies until delim is reached, or until n chars copied.
// Both Buffers must exist. Caller should terminate with '\0'.
/////////////////////////////////////////////////////////////////////////////////////////////////////////////
int CopyStrip(char *dest, const char *src, const char *delim, size_t n)
{
unsigned int i;
bool done;

     done=false;
     // copy but check for first occurence of first char of src
     i=0;
     while ((i<n) && (!done)) 
     {
          if ((src[i]!=delim[0])&&(src[i]!='\0')) 
          {
               dest[i]=src[i];
               i++;
               }
          else done = true;
          }
     return (i);
     } /////////////////////////////////////////////////////////////////////////////////////////end CopyStrip

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// CreateThread
// Initializes thread attr and creates a thread the same way across all applications. 
// Either it works and returns or it fails and programs stop dead.
// They wont work without threads
/////////////////////////////////////////////////////////////////////////////////////////////////////////////
int CreateThread(pthread_t  *thread, void *(*start_routine)(void *), void *arg)
//void CreateThread()
{
pthread_attr_t          attr;

	// setup threading
	pthread_attr_init(&attr);    // initialize attr with default attributes
	pthread_attr_setscope(&attr, PTHREAD_SCOPE_SYSTEM);  // system-wide contention

	return(pthread_create(thread, &attr, start_routine, arg));
	} ///////////////////////////////////////////////////////////////////////////////////end CreateThread

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// InitRWLock
// Initializes rw mutecies the same way across all applications. 
// Either it works and returns or it fails and programs stop dead.
// They wont work without mutecies
/////////////////////////////////////////////////////////////////////////////////////////////////////////////
void InitRWLock(pthread_rwlock_t *rwlock)
{
int			res;                   // thread attr structure for setting attributes.
pthread_rwlockattr_t	m_rwattr;

	// set up attr object
	if ((res=pthread_rwlockattr_init(&m_rwattr))!=0) 
	{
		PError(res);
		exit(-1);
		}
//#ifdef LINUX
//	if ((res=pthread_mutexattr_settype(&m_rwattr,PTHREAD_MUTEX_ERRORCHECK_NP))!=0)
//#endif
//#ifdef SOLARIS
//	if ((res=pthread_mutexattr_settype(&m_rwattr,PTHREAD_MUTEX_ERRORCHECK))!=0)
//#endif
//	{
//		PError(res);
//		exit(-1);
//		}
	if ((res=pthread_rwlock_init(rwlock, &m_rwattr))!=0)
	{
		PError(res);
		exit(-1);
		}

	if ((res=pthread_rwlockattr_destroy(&m_rwattr))!=0)
	{
		PError(res);
		exit(-1);
		}

	} /////////////////////////////////////////////////////////////////////////////////////end InitRWLock

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// InitMutex
// Initializes mutecies the same way across all applications. 
// Either it works and returns or it fails and programs stop dead.
// They wont work without mutecies
/////////////////////////////////////////////////////////////////////////////////////////////////////////////
void InitMutex(pthread_mutex_t *mutex)
{
int			res;                   // thread attr structure for setting attributes.
pthread_mutexattr_t     m_attr;

	// set up attr object
	if ((res=pthread_mutexattr_init(&m_attr))!=0) 
	{
		PError(res);
		exit(-1);
		}
#ifdef LINUX
	if ((res=pthread_mutexattr_settype(&m_attr,PTHREAD_MUTEX_ERRORCHECK_NP))!=0)
#endif
#ifdef SOLARIS
	if ((res=pthread_mutexattr_settype(&m_attr,PTHREAD_MUTEX_ERRORCHECK))!=0)
#endif
	{
		PError(res);
		exit(-1);
		}
	if ((res=pthread_mutex_init(mutex, &m_attr))!=0)
	{
		PError(res);
		exit(-1);
		}

	if ((res=pthread_mutexattr_destroy(&m_attr))!=0)
	{
		PError(res);
		exit(-1);
		}
	
	} //////////////////////////////////////////////////////////////////////////////////////end InitMutex

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// NextPowerOfTwo
// Returns the next higher power of two.
// Thanks to Sven Olsen
/////////////////////////////////////////////////////////////////////////////////////////////////////////////
int NextPowerOfTwo(int  a)
{
int	rval=1;
	while (rval<a) rval<<=1;
	return rval;
	} /////////////////////////////////////////////////////////////////////////////////end NextPowerOfTwo

/////////////////////////////////////////////////////////////////////////////////////////////////////////////
// PError
// Prints the socket error, and its string.
/////////////////////////////////////////////////////////////////////////////////////////////////////////////
void PError(int  e)
{

	// find out which error it is the use ThreadLog
	switch (e)
	{
	case EBADF:
		ThreadLog("Error: EBADF An invalid descriptor was specified.");
		break;

	case ENOTSOCK:
		ThreadLog("Error: ENOTSOCK The argument s is not a socket.");
		break;

	case EFAULT:
		ThreadLog("Error: EFAULT An invalid user space address was specified for a parameter.");
		break;

	case EMSGSIZE:
		ThreadLog("Error: EMSGSIZE Msg too big for atomic sending.");
		break;

	case EAFNOSUPPORT:
		ThreadLog("Error: Address family not supported by protocol family.");
		break;

//	case EAGAIN:
	case EWOULDBLOCK:
		ThreadLog("Error: EWOULDBLOCK The socket is marked non-blocking and the requested operation would block.");
		break;

	case ENOBUFS:
		ThreadLog("Error: ENOBUFS The output queue for a network interface was full.");
		break;

	case EINTR:
		ThreadLog("Error: EINTR A signal occurred.");
		break;

	case ENOMEM:
		ThreadLog("Error: ENOMEM No memory available.");
		break;

	case EINVAL:
		ThreadLog("Error: EINVAL Invalid argument passed.");
		break;

	case EPIPE:
		ThreadLog("Error: EPIPE The local end has been shut down on a connection oriented socket.");
		break;

	case EBUSY:
		ThreadLog("Error: EBUSY Mutex already locked.");
		break;

	case EDEADLK:
		ThreadLog("Error: EDEAD Current thread already owns mutex.");
		break;

	case EPERM:
		ThreadLog("Error: EPERM Current thread does not own mutex.");
		break;



	default:	
		ThreadLog("Error: Unhandled error number.");
		break;
		}

	} ////////////////////////////////////////////////////////////////////////////end PSockError

////////////////////////////////////////////////////////////////////////////////////////////////////
// GetAString
////////////////////////////////////////////////////////////////////////////////////////////////////
CString *GetAString(curswin *p)
{
CString         *myinput;
char            c1[2];
char            buf[80];

	myinput = new CString;
	c1[0]='\0';
	c1[1]='\0';
	while (c1[0]!=13)
	{
		c1[0]=(char)getch();
		c1[1]='\0';
		if (c1[0]!=13) myinput->Append(c1);
		if (loglevel>=4) snprintf(buf,80,"String is '%s' \n",myinput->GetStringPtr());
		if (loglevel>=4) ThreadLog(buf);
		MyPrint(p->mainwin,c1,TRUE);
		}
	return (myinput);
	}/////////////////////////////////////////////////////////////////////////////end GetAString

////////////////////////////////////////////////////////////////////////////////////////////////////
// GetAFileName
////////////////////////////////////////////////////////////////////////////////////////////////////
CString *GetAFileName(curswin *p)
{
char	c;
CString	*name=0;

	do 
	{
		MyPrint(p->mainwin,"Please enter filename \n", TRUE);
		if (name) delete name;
		name=GetAString(p);
		MyPrint(p->mainwin,"Is this OK ? Script file name = '", TRUE);
		if (name) MyPrint(p->mainwin,name->GetStringPtr(), TRUE);
		MyPrint(p->mainwin,"'\n", TRUE);
		MyPrint(p->mainwin,"('Y'|'y') \n", TRUE);
		c=(char)getch();
		} while (c!='Y' && c!='y');
	return(name);
	} //////////////////////////////////////////////////////////////////////////end GetAFileName

////////////////////////////////////////////////////////////////////////////////////////////////////
// LoadFile
////////////////////////////////////////////////////////////////////////////////////////////////////
bool LoadFile(CString *filename, CEngine *e)
{
FILE                    *scriptfile;
unsigned long int       li,val;
int                     result;
bool                    done;

	// Prep script engine
        e->Reset();

	// open binary file
        if (!(scriptfile=fopen(filename->GetStringPtr(),"r"))) { return(false); }

	// read in file
        done=false;
        li=0;
        while ((li<1000) && (!done))
        {
                result=fscanf(scriptfile,"%lu %lu",&li,&val);
                if (result==EOF) done=true;
                else
                {
                        e->SetValue(li,val);
                        li++;
                        }
                }
        fclose(scriptfile);
        return (true);
        } //////////////////////////////////////////////////////////////////////////////end LoadFile

#ifdef SOLARIS

////////////////////////////////////////////////////////////////////////////////////////////////////
// wrapper for Solaris case
////////////////////////////////////////////////////////////////////////////////////////////////////
void pthread_yield()
{
        thr_yield();
        }

#endif

