\! echo '/////////////////////////////////////////////////////////////'
\! echo '// WorldBuild.sql'
\! echo '// This creates the worldii data base '
\! echo '// '
\! echo '// by Eric J Krejci'
\! echo '// Copyright Eric J Krejci December 1999'
\! echo '// Revised : 01 May 2001'
\! echo '/////////////////////////////////////////////////////////////'

\! echo players
CREATE TABLE players 
(
     playerid		char (20)	NOT NULL        ,
     lname		char (20)	                ,
     fname		char (20)	                ,
     password		char (20)	                ,
     loggedin		int                             ,

     PRIMARY KEY   
     (
          playerid
          )
     ) 
;

\! echo wiiiobjecttypes
CREATE TABLE wiiiobjecttypes
(
     code 		char (15) 	NOT NULL ,
     description 	varchar (255) 	         ,
     
     PRIMARY KEY   
     (
          code
          )
     )
;

\! echo regions
CREATE TABLE regions
(
     x               int       NOT NULL ,
     y               int       NOT NULL ,
     z               int       NOT NULL ,
     

     PRIMARY KEY
     (
          x,y,z
          )
     )
;

\! echo wiiiobjects
CREATE TABLE wiiiobjects
(
     said 		int8		NOT NULL ,
     objecttypefkey 	char (15) 	NOT NULL ,
     xpos		float			,
     ypos		float			,
     zpos		float			,
     yaw		float			,
     pitch		float			,
     roll		float			,
     container		bool			,
     carrier		bool			,
     contained		bool			,
     carried		bool			,
     weight		int			,
     length		int			,
     width		int			,
     height		int			,

     CONSTRAINT wiiiobjects_pk PRIMARY KEY   
     (
          said
          ),
     CONSTRAINT object_to_objecttype FOREIGN KEY
     (
          objecttypefkey
          )
     REFERENCES wiiiobjecttypes
     (
          code
          )
     )
;

\! echo Mineral
CREATE TABLE Mineral
(
     Said               int8            NOT NULL ,
     physhp             int                     ,
     physdam            int                     ,
     LastUpdate		TIMESTAMP	         ,

     CONSTRAINT Mineral_To_Object FOREIGN KEY
     (
          Said
          )
     REFERENCES wiiiobjects
     (
          Said
          )
     )
;

\! echo Live
CREATE TABLE Live
(
     Said               int8            NOT NULL ,
     biohp              int                     ,
     biodmg             int                     ,
     LastUpdate         TIMESTAMP                 ,

     CONSTRAINT Live_To_Object FOREIGN KEY
     (
          Said
          )
     REFERENCES wiiiobjects
     (
          Said
          )
     )
;

\! echo PChar
CREATE TABLE PChar
(
     Said               int8            NOT NULL ,
     FName              char (40)                ,
     LName              char (40)                ,
     PTitle             char (40)                ,
     Race               char (40)                ,
     LastUpdate         TIMESTAMP                 ,

     CONSTRAINT PChar_To_Object FOREIGN KEY
     (
          Said
          )
     REFERENCES wiiiobjects
     (
          Said
          )
     )
;

\! echo VarType
CREATE TABLE VarType 
(
     Code 		char (15) 	NOT NULL ,
     Description 	varchar (255) 	         ,
     
     PRIMARY KEY  
     (
          Code
          )
     )
;

\! echo wiiiattributes
CREATE TABLE wiiiattributes 
(
     ObjectFkey 	int8		NOT NULL ,
     VarTypeFkey 	char (15) 	NOT NULL ,
     Name  		char (20)	NOT NULL ,
     Value 		char (20)	         ,
     LastUpdate		TIMESTAMP	         ,


     CONSTRAINT wiiiattr_pk PRIMARY KEY   
     (
          ObjectFkey, VarTypeFkey, Name
          ),
     CONSTRAINT wiiiattr_to_wiiiobjs FOREIGN KEY   
     (
          objectfkey
          )
     REFERENCES wiiiobjects 
     (
          said
          )
     )
;

\! echo Method
CREATE TABLE Method
(
     MethodName 	char (30)	NOT NULL ,
     DLL 		char (255) 	NOT NULL ,
     HasReturn		char (1)	NOT NULL , /* Functions return, Procedures do not*/
     CONSTRAINT PK___9__11 PRIMARY KEY   
     (
          MethodName
          )

     )
;

\! echo Parms
CREATE TABLE Parms
(
     VarTypeFkey	char (15)	NOT NULL ,
     MethodNameFkey 	char (30)	NOT NULL ,
     Name		char (20)	NOT NULL ,
     Serial		int		NOT NULL ,

     CONSTRAINT PK___10__11 PRIMARY KEY   
     (
          VarTypeFkey, MethodNameFkey, Name
          ),

     CONSTRAINT FK___11__11 FOREIGN KEY   
     (
          VarTypeFkey
          )
     REFERENCES VarType 
     (
          Code
          ),
     
     CONSTRAINT FK___12__11 FOREIGN KEY 
     (
          MethodNameFkey
          ) 
     REFERENCES Method 
     (
          MethodName
          )

     )
;

\! echo Actions
CREATE TABLE Actions
(
     ObjectFkey 	int8		NOT NULL ,
     MethodNameFkey 	char (30)	NOT NULL ,
     SkillLevel		int		NOT NULL , /* The skill level a particular */ 
						   /*object can perform this action at.*/
     CONSTRAINT PK___13__11 PRIMARY KEY   
     (
          ObjectFkey, MethodNameFkey
          ),

     CONSTRAINT FK___14__11 FOREIGN KEY   
     (
          ObjectFkey
          )
     REFERENCES wiiiobjects 
     (
          said
          ),
     
     CONSTRAINT FK___15__11 FOREIGN KEY 
     (
          MethodNameFkey
          ) 
     REFERENCES Method 
     (
          MethodName
          )

     )
;

\! echo Servers
CREATE TABLE Servers
(
     Said		int8		NOT NULL ,
     IP 		char (15) 	NOT NULL ,
     PlayerObjectFkey	int		NOT NULL , /* The object associated with the player*/

     CONSTRAINT PK___16__11 PRIMARY KEY   
     (
          Said
          ),

     CONSTRAINT FK___17__11 FOREIGN KEY   
     (
          PlayerObjectFkey
          )
     REFERENCES wiiiobjects 
     (
          said
          )
     
     )
;

\! echo ViewList
CREATE TABLE ViewList
(
     ServerFkey 	int8		NOT NULL ,
     ObjectFkey		int8		NOT NULL ,
     WhenSent		TIMESTAMP	         ,

     CONSTRAINT FK___18__11 FOREIGN KEY   
     (
          ObjectFkey
          )
     REFERENCES wiiiobjects 
     (
          said
          ),
     
     CONSTRAINT FK___19__11 FOREIGN KEY 
     (
          ServerFkey
          ) 
     REFERENCES Servers 
     (
          Said
          )

     )
;

\! echo characters
CREATE TABLE playerchar
(
     playeridfkey       char (20)       NOT NULL        ,
     objectFkey         int8            NOT NULL        ,

     CONSTRAINT CH_to_P FOREIGN KEY
     (
          playeridfkey
          )
     REFERENCES players
     (
          playerid
          ),

     CONSTRAINT CH_to_O FOREIGN KEY
     (
          objectFkey
          )
     REFERENCES wiiiobjects
     (
          Said
          )

     )
;

